unit BitmapLib;

interface

uses Forms,Windows,ExtCtrls,Dialogs,Classes,SysUtils,Graphics,Math,ClipBrd,FilmStp,
     Vector,IntVector,DSupport,GraphicsLib,HugeArry,ImageEx;


type TJPEGResUnits = (juNoUnits=0, juDotsPerInch=1, juDotsPerCentimeter);

const MaxSize = 1024 * 1024 * 128;	{Arrays cannot be more than 2gig, so use this for max}

{24-bit color arrays}

type TRGBTripleArray = array [0..MaxSize] of TRGBTriple;
type PRGBTripleArray = ^TRGBTripleArray;

{32-bit color arrays}

type TRGBQuadArray = array [0..MaxSize] of TRGBQuad;
type PRGBQuadArray = ^TRGBQuadArray;


{Hue, Staturation and Brightness structure}

type THSBRecord=record
 Hue,Sat,Bright: extended;
 end;


{Hue, Staturation and Luminesance structure}

type THSLRecord=record
 Hue,Sat,Lume: extended;
 end;



{----------- Gaussian Blur variables ------------------------------------------}


type TRow = array[0..MaxSize] of TRGBTriple;
type PRow = ^TRow;

type TPRows = array[0..MaxSize] of PRow;
type PPRows = ^TPRows;

{the idea is that when using a TKernel you ignore the Weights except for Weights in the range -Size..Size.}

const MaxKernelSize = 100;

type TKernelSize = 1..MaxKernelSize;
type TKernel = record
 Size: TKernelSize;
 Weights: array[-MaxKernelSize..MaxKernelSize] of single;
 end;

procedure GaussianBlur(theBitmap: TBitmap; radius: double);


{Framing options for rotation: }
{foSource = clipped to Source retangle size}
{foDestination = clipped to Destination retangle size}
{foDestCenter - Puts rotational center in center of destination}
{foFit = Rectangle expanded to fit rotated image}
{foWindow = Rectangle always expanded to max rotation size}

type TFrameOptions = (foSource,foDestination,foDestCenter,foFit,foWindow);

type PBitmap = ^TBitmap;

{============= Bitmap list object =============================================}


type TBitmapProgress = procedure(Percent: integer);


type TBitmapList = class(TList)
 private
  FAuxInfo: THugeInteger;
  function GetBitmap(Inx: integer): TBitmap;
  procedure PutBitmap(Inx: integer; Item: TBitmap);
  function GetAuxInfo(Index: integer): integer;
  procedure PutAuxInfo(Index: integer; const Value: integer);
  function GetCount: integer;
  procedure SetCount(V: integer);
 public
  constructor Create;
  destructor Destroy; override;
  function AddBitmap(BM: TBitmap): integer; virtual;
  procedure Clear; override;
  procedure Delete(Index: Integer); virtual;
  procedure Load(Filename: string; Progress: TBitmapProgress);
  procedure Save(Filename: string; Progress: TBitmapProgress);
  procedure PasteFromClipboard;
  procedure CopyToClipboard(Inx: integer);
  property Bitmaps[Inx: integer]: TBitmap read GetBitmap write PutBitmap;
  property AuxInfo[Inx: integer]: integer read GetAuxInfo write PutAuxInfo;
  property Count: integer read GetCount write SetCount;
 end;


function MakeRBGTriple(R,G,B: byte): TRGBTriple;

function ColorToQuad(C: TColor): TRGBQuad;
function ColorToTriple(C: TColor): TRGBTriple;
function EqualRGBTriple(C1,C2: TRGBTriple): boolean;
function TripleDistance(C1,C2: TRGBTriple): double;
function ColorDistance(C1,C2: TColor): double;

procedure ColorToRGBBytes(Color: TColor; var R,G,B: byte);
function QuadToColor(T: TRGBQuad): TColor;
function TripleToColor(T: TRGBTriple): TColor;
function ColorToVector(Color: TColor): T3DVector;
function VectorToColor(V: T3DVector): TColor;
function RGBTripleToVector(Color: TRGBTriple): T3DVector;
function Color15ToColor(C: integer): TColor;
function Color16ToColor(C: integer): TColor;
function Color24ToColor(C: integer): TColor;
function ContrastColor(C: TColor): TColor;

procedure FlipBitmapHoriz(Source,Destination: TBitmap);
procedure FlipBitmapVertical(Source,Destination: TBitmap);
procedure FlipBitmap90(Source,Destination: TBitmap);
procedure FlipBitmap180(Source,Destination: TBitmap);
procedure FlipBitmap270(Source,Destination: TBitmap);


procedure RotateBitmap(Source,Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
procedure RotateBitmap32(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
procedure BorderBitmap(BM: TBitmap; Color: TColor; Width: integer);

function RGBtoHSB(RGB: TRGBTriple): THSBRecord;
function ColorToHSB(Color: TColor): THSBRecord;
function HSBtoRGB(HSB: THSBRecord): TRGBTriple;
function MakeHSB(Hue,Sat,Bright: extended): THSBRecord;

function HSLToRGB(HSL: THSLRecord): TRGBTriple;
function RGBToHSL(RGB: TRGBTriple): THSLRecord;
function MakeHSL(Hue,Sat,Lume: single): THSLRecord;

function GetRGBIntensity(C: TColor): double;

procedure MakeBlackAndWhite(Source,Destination: TBitmap; Invert: boolean; Threshold: double);
procedure AdjustHue(Source,Destination: TBitmap; Offset: double);
procedure AdjustSaturation(Source,Destination: TBitmap; Offset: double);
procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double);

procedure DoWinResize(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Mode: integer);
procedure ResizeBitmap(Source,Destination: TBitmap; Percent: double; OverSample: boolean); overload;
procedure ResizeBitmap(Source,Destination: TBitmap; Size: TPoint; OverSample: boolean); overload;
procedure OversampleBitmap(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap);

procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte); overload;
procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte; PerPixel: boolean); overload;
procedure AlphaBlendCanvas(DestCanvas: TCanvas; DestRect: TRect; SourceCanvas: TCanvas; SourceRect: TRect; Alpha: byte; PerPixel: boolean);
procedure SetAlphaByLightness(BM: TBitmap; DoLightness: boolean; Lightness: byte);
procedure SetAlphaByColor(BM: TBitmap; C: TColor; Alpha: byte; Reset: boolean); overload;
procedure SetAlphaByColor(BM: TBitmap; C: TColor); overload;
procedure MakeMask(var Mask: TBitmap; Color: TColor);

procedure WrapImageAngle(BM: TBitmap; Angle,Len: double);
procedure WrapImage(BM: TBitmap; X,Y: integer);
procedure WrapImageVertical(BM: TBitmap; Delta: integer);
procedure WrapImageHorizontal(BM: TBitmap; Delta: integer);
function ClipBitmap(BM: TBitmap; R: TRect): boolean;

function CreateDisabledBitmap(Glyph: TBitmap): TBitmap;


procedure SaveRectToClipboard(Bitmap: TBitmap; R: TRect);

procedure ForceBitmapHandle(BM: TBitmap);
function InterpolateColors(C1,C2: TColor; Width, Pos: double): TColor;
procedure DrawGradientLine(Canvas: TCanvas; P1,P2: TPoint; C1,C2: TColor);
procedure VertGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor);
procedure HorizGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor) ;

function IconToBitmap(Icon: TIcon): TBitmap;
function LoadCursorToBitmap(Filename: string): TBitmap;
function LoadCursorToIcon(Filename: string): TIcon;

function ColorToRGBString(C: TColor): string; overload;
function ColorToRGBString(C: TColor; Hex: boolean): string; overload;

function RGBTripleToString(C: TRGBTriple; Hex: boolean): string; overload;
function RGBTripleToString(C: TRGBTriple): string; overload;
function RGBQuadToString(C: TRGBQuad; Hex: boolean): string; overload;
function RGBQuadToString(C: TRGBQuad): string; overload;

procedure NormalizeImageSize(Image: TImage);
procedure ClearImage(Image: TImage; Color: TColor);
procedure ClearCanvas(Canvas: TCanvas; Rect: TRect; Color: TColor);

procedure DrawPixels(Image: TImage; X,Y, Size: integer; C: TColor);
procedure ContinuousDraw(Image: TImage; Start: boolean; X,Y, Size: integer; C: TColor);
procedure ColorFill(BM: TBitmap; X,Y: integer; Threshold: double; NewColor: TColor);


type TFloodProgress = procedure(Percent: integer);

procedure ThresholdFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
procedure SeedFloodFill(BM: TBitmap; X, Y: integer; Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
procedure AntiAliasFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);

procedure ClearBitmap(BM: TBitmap; Block: TRect; Color: TColor);  overload;
procedure ClearBitmap(BM: TBitmap; Color: TColor);  overload;
function IsBitmapBlank(BM: TBitmap): boolean;

procedure SetBMPFilePPM(FileName: String; PPMX,PPMY: integer);
procedure SetBMPPPM(BM: TBitmap; PPMX,PPMY: integer);
procedure GetBmpPPM(Bitmap: TBitmap; var PPMX,PPMY: integer);
procedure GetBMPFilePPM(FileName: string; var PPMX,PPMY: integer);
procedure CopyBMPPPM(Source,Destination: TBitmap);

procedure GetJPEGResolution(Filename: string; var Units: TJPEGResUnits; var XRes,YRes: integer);
procedure SetJPEGResolution(Filename: string; DPIX, DPIY: Integer; Units: TJPEGResUnits=juDotsPerInch);

procedure SetBMPPixelFormat(BM: TBitmap; PF: TPixelFormat);
function GetPixelFormatStr(BM: TBitmap): string;

procedure FadeInBitmap(Image: TImage; BM: TBitmap; Millisec: integer);
procedure FadeToColor(Image: TImage; C: TColor; Millisec: integer);

procedure BitmapToGrayscaleBuffer(BM: TBitmap; AllocMem: boolean; var Buffer: PByteArray);


implementation



function MakeRBGTriple(R,G,B: byte): TRGBTriple;
begin
Result.rgbtRed:=R;
Result.rgbtGreen:=G;
Result.rgbtBlue:=B;
end;


procedure ForceBitmapHandle(BM: TBitmap);
{Necessary when assigning bitmaps - Forces bitmaps to be independent}
var H: HBITMAP;
{$HINTS OFF}
begin
{Even though the handle is never used, it must be assigned or
else the Handle will not be created making the BM independent}
H:=BM.Handle;
end;
{$HINTS ON}




function ColorToTriple(C: TColor): TRGBTriple;
{Convert Delphi TColor to RGBTriple}
begin
Result.rgbtRed:=C and $FF;
Result.rgbtGreen:=(C shr 8) and $FF;
Result.rgbtBlue:=(C shr 16) and $FF;
end;



function ColorToQuad(C: TColor): TRGBQuad;
{Convert Delphi TColor to RGBQuad}
begin
Result.rgbRed:=C and $FF;
Result.rgbGreen:=(C shr 8) and $FF;
Result.rgbBlue:=(C shr 16) and $FF;
Result.rgbReserved:=$FF;
end;



function EqualRGBTriple(C1,C2: TRGBTriple): boolean;
begin
Result:=(C1.rgbtBlue = C2.rgbtBlue) and
	(C1.rgbtGreen = C2.rgbtGreen) and
	(C1.rgbtRed = C2.rgbtRed);
end;



function TripleDistance(C1,C2: TRGBTriple): double;
{Calculate distance between two colors - normalized to 0 to 1 range}
const MaxDist = 441.67295593;	{sqrt(sqr(256)+sqr(256)+sqr(256))}
begin
Result:=sqrt(sqr(C1.rgbtBlue - C2.rgbtBlue) +
	     sqr(C1.rgbtGreen - C2.rgbtGreen) +
	     sqr(C1.rgbtRed - C2.rgbtRed));
Result:=EnsureRange(Result/MaxDist,0,1);
end;



function ColorDistance(C1,C2: TColor): double;
{Calculate distance between two colors - normalized to 0 to 1 range}
begin
Result:=TripleDistance(ColorToTriple(C1),ColorToTriple(C2));
end;


function TripleToColor(T: TRGBTriple): TColor;
{Convert RGBTriple to Delphi TColor}
begin
Result:=T.rgbtRed or (T.rgbtGreen shl 8) or (T.rgbtBlue shl 16);
end;



function QuadToColor(T: TRGBQuad): TColor;
{Convert RGBTriple to Delphi TColor}
begin
Result:=T.rgbRed or (T.rgbGreen shl 8) or (T.rgbBlue shl 16);
end;



procedure ColorToRGBBytes(Color: TColor; var R,G,B: byte);
{Extract individual RGB values for TColor variable}
var C: integer;
begin
C:=ColorToRGB(Color);
R:=GetRValue(C);
G:=GetGValue(C);
B:=GetBValue(C);
end;


function ColorToRGBString(C: TColor; Hex: boolean): string;
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
if Hex then Result:=IntToHex(R,2)+','+IntToHex(G,2)+','+IntToHex(B,2)
else Result:=IntToStr(R)+','+IntToStr(G)+','+IntToStr(B);
end;



function ColorToRGBString(C: TColor): string;
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
Result:=IntToStr(R)+','+IntToStr(G)+','+IntToStr(B);
end;


function RGBTripleToString(C: TRGBTriple; Hex: boolean): string;
begin
if Hex then Result:=IntToHex(C.rgbtRed,2)+','+IntToHex(C.rgbtGreen,2)+','+IntToHex(C.rgbtBlue,2)
else Result:=IntToStr(C.rgbtRed)+','+IntToStr(C.rgbtGreen)+','+IntToStr(C.rgbtBlue);
end;


function RGBTripleToString(C: TRGBTriple): string;
begin
Result:=RGBTripleToString(C,False);
end;


function RGBQuadToString(C: TRGBQuad; Hex: boolean): string;
begin
if Hex then Result:=IntToHex(C.rgbRed,2)+','+IntToHex(C.rgbGreen,2)+','+IntToHex(C.rgbBlue,2)+','+IntToHex(C.rgbReserved,2)
else Result:=IntToStr(C.rgbRed)+','+IntToStr(C.rgbGreen)+','+IntToStr(C.rgbBlue)+','+IntToStr(C.rgbReserved);
end;


function RGBQuadToString(C: TRGBQuad): string;
begin
Result:=RGBQuadToString(C,False);
end;


function ColorToVector(Color: TColor): T3DVector;
{Convert Delphi color to color vector}
var R,G,B: byte;
begin
ColorToRGBBytes(Color,R,G,B);
Result:=MakeVector3D(R,G,B);
Result:=ScalarDivide3D(Result,255);
end;



function RGBTripleToVector(Color: TRGBTriple): T3DVector;
{Convert RGB Triple color to color vector}
begin
Result:=MakeVector3D(Color.rgbtRed,Color.rgbtGreen,Color.rgbtBlue);
Result:=ScalarDivide3D(Result,255);
end;


function ByteRange(V: double): byte;
begin
Result:=Trunc(EnsureRange(V,0,255));
end;


function VectorToColor(V: T3DVector): TColor;
{Convert 3D Vector to Delphi Color}
begin
V:=ScalarProduct3D(V,255);
Result:=RGB(ByteRange(V.X),ByteRange(V.Y),ByteRange(V.Z));
end;



function Color15ToColor(C: integer): TColor;
{Convert a 5:5:5 RGB Color to Delphi Color}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $1F;
G:=(C shr 5) and $1F;
R:=(C shr 10) and $1F;
Result:=RGB(MulDiv(R,255,31),MulDiv(G,255,31),MulDiv(B,255,31));
end;



function Color16ToColor(C: integer): TColor;
{Convert a 5:6:5 Color to Delphi Color}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $1F;
G:=(C shr 5) and $3F;
R:=(C shr 11) and $1F;
Result:=RGB(MulDiv(R,255,31),MulDiv(G,255,63),MulDiv(B,255,31));
end;



function Color24ToColor(C: integer): TColor;
{Convert a 8:8:8 Color to Delphi Color}
{Also works with 32-bit color although alphi is not handled}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $FF;
G:=(C shr 8) and $FF;
R:=(C shr 16) and $FF;
Result:=RGB(R,G,B);
end;




function GetRGBIntensity(C: TColor): double;
{Return intensity 0-1 value}
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
Result:=(R*299 + G*587 + B*114)/255000;
end;


function ContrastColor(C: TColor): TColor;
{Return most constrating color to C}
var I: double;
begin
I:=GetRGBIntensity(C);
if I>0.5 then Result:=clBlack else Result:=clWhite;
end;




{--------------------- Bitmap List object -------------------------------------}


function TBitmapList.AddBitmap(BM: TBitmap): integer;
var TBM: TBitmap;
var H: HBITMAP;
{$HINTS OFF}
begin
TBM:=TBitmap.Create;
TBM.Assign(BM);
{Forces bitmap to be independent}
{Ignore warnings Assignment to H required}
H:=TBM.Handle;
Result:=Add(TBM);
FAuxInfo.Add(0);
end;
{$HINTS ON}



function TBitmapList.GetBitmap(Inx: integer): TBitmap;
begin
Result:=nil;
if Inx>=Count then exit;
Result:=Items[Inx];
end;



procedure TBitmapList.PutBitmap(Inx: integer; Item: TBitmap);
begin
if Inx>=Count then exit;
Items[Inx]:=Item;
end;



function TBitmapList.GetAuxInfo(Index: integer): integer;
begin
Result:=FAuxInfo[Index];
end;



procedure TBitmapList.PutAuxInfo(Index: integer; const Value: integer);
begin
FAuxInfo[Index]:=Value;
end;



procedure TBitmapList.Clear;
var I: integer;
begin
for I:=0 to Count-1 do TBitmap(Items[I]).Free;
FAuxInfo.Clear;
inherited Clear;
end;


procedure TBitmapList.Delete(Index: Integer);
begin
TBitmap(Items[Index]).Free;
FAuxInfo.Delete(Index);
inherited Delete(Index);
end;



function TBitmapList.GetCount: integer;
begin
Result:=TList(Self).Count;
end;


procedure TBitmapList.SetCount(V: integer);
var I: integer;
begin
for I:=Count-1 downto V do
	begin
	Delete(I);
	FAuxInfo.Delete(I);
	end;
end;




constructor TBitmapList.Create;
begin
inherited Create;
FAuxInfo:=THugeInteger.Create(5);
end;




destructor TBitmapList.Destroy;
begin
inherited Destroy;
FAuxInfo.Free;
end;



procedure TBitmapList.Load(Filename: string; Progress: TBitmapProgress);
var InFilm: TInFilm;
var I: integer;
var BM: TBitmap;
begin
InFilm:=TInFilm.Create;
try
BM:=TBitmap.Create;
try
InFilm.OpenFile(FileName);
InFilm.Rewind;
Self.Clear;
for I:=0 to InFilm.Count-1 do
	begin
	InFilm.ReadBitmap(BM);
	Self.AddBitmap(BM);
	if Assigned(Progress) then Progress(MulDiv(100,I,InFilm.Count));
	Application.ProcessMessages;
	end;
finally BM.Free; end;
finally InFilm.Free; end;
end;





procedure TBitmapList.Save(Filename: string; Progress: TBitmapProgress);
var OutFilm: TOutFilm;
var I: integer;
begin
OutFilm:=TOutFilm.Create;
try
OutFilm.OpenFile(Filename);
try
for I:=0 to Self.Count-1 do
	begin
	OutFilm.WriteBitmap(Self.Bitmaps[I]);
	if Assigned(Progress) then Progress(MulDiv(100,I,Self.Count));
	Application.ProcessMessages;
	end;
except MessageDlg('Out Of Disk Space',mtWarning,[mbOK], 0); end;
 finally
 OutFilm.CloseFile;
 OutFilm.Free;
 end;
end;


procedure TBitmapList.PasteFromClipboard;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.LoadFromClipboardFormat(cf_BitMap,ClipBoard.GetAsHandle(cf_Bitmap),0);
AddBitmap(BM);
finally BM.Free; end;
end;



procedure TBitmapList.CopyToClipboard(Inx: integer);
var MyFormat: Word;
var AData: THandle;
var APalette: HPALETTE;
begin
if Inx>=Count then raise Exception.Create('Index out of range');
Self.Bitmaps[Inx].SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
end;



{------------------ Gaussian Blur routines ------------------------------------}


procedure MakeGaussianKernel(var K: TKernel; radius: double; MaxData, DataGranularity: double);
{makes K into a gaussian kernel with standard deviation = radius. For the 
current application you set MaxData = 255 and DataGranularity = 1. Now 
the procedure sets the value of K.Size so that when we use K we will 
ignore the Weights that are so small they can't possibly matter. (Small Size 
is good because the execution time is going to be propertional to K.Size.)} 
var j: integer; 
var temp, delta: double; 
var KernelSize: TKernelSize; 
begin 
for j:= Low(K.Weights) to High(K.Weights) do 
	begin 
	temp := j / radius;
	K.Weights[j] := exp(-temp * temp / 2); 
	end; 

{now divide by constant so sum(Weights) = 1:} 
temp := 0; 
for j := Low(K.Weights) to High(K.Weights) do  temp := temp + K.Weights[j]; 
for j := Low(K.Weights) to High(K.Weights) do  K.Weights[j] := K.Weights[j] / temp; 

{now discard (or rather mark as ignorable by setting Size) the entries that 
are too small to matter - this is important, otherwise a blur with a small radius 
will take as long as with a large radius...} 

KernelSize := MaxKernelSize; 
delta := DataGranularity / (2 * MaxData); 
temp := 0; 
while (temp < delta) and (KernelSize > 1) do 
	begin 
	temp := temp + 2 * K.Weights[KernelSize]; 
	dec(KernelSize); 
	end; 
K.Size := KernelSize; 

{now just to be correct go back and jiggle again so the sum of the entries we'll be using is exactly 1} 
temp := 0; 
for j := -K.Size to K.Size do temp := temp + K.Weights[j]; 
for j := -K.Size to K.Size do K.Weights[j] := K.Weights[j] / temp; 
end;
   


function TrimInt(Lower, Upper, theInteger: integer): integer; 
begin 
if (theInteger <= Upper) and (theInteger >= Lower) then result := theInteger 
else if theInteger > Upper then result := Upper 
else  result := Lower; 
end; 


function TrimReal(Lower, Upper: integer; x: double): integer;
begin
if (x <=upper) and (x >= lower) then result := trunc(x)
else if x > Upper then result := Upper
else result := Lower;
end;



procedure BlurRow(var theRow: array of TRGBTriple; K: TKernel; P: PRow);
var  j, n: integer;
var tr, tg, tb: double; {tempRed, etc}
var w: double;
begin
for j := 0 to High(theRow) do
	begin
	tb := 0;
	tg := 0;
	tr := 0;
	for n := -K.Size to K.Size do
		begin
		w := K.Weights[n];
		{the TrimInt keeps us from running off the edge of the row...}
		with theRow[TrimInt(0, High(theRow), j - n)] do
		 begin 
		 tb := tb + w * rgbtBlue;
		 tg := tg + w * rgbtGreen;
		 tr := tr + w * rgbtRed;
		 end; 
		end; 
	with P[j] do 
	 begin 
	 rgbtBlue:= TrimReal(0, 255, tb);
	 rgbtGreen := TrimReal(0, 255, tg);
	 rgbtRed := TrimReal(0, 255, tr);
	 end; 
	end; 
Move(P[0], theRow[0], (High(theRow) + 1) * Sizeof(TRGBTriple)); 
end; 
   


procedure GaussianBlur(theBitmap: TBitmap; radius: double); 
var  Row, Col: integer; 
var theRows: PPRows; 
var K: TKernel; 
var ACol: PRow; 
var P: PRow; 
begin 
if (theBitmap.HandleType <> bmDIB) or (theBitmap.PixelFormat <> pf24Bit) then raise exception.Create('GBlur only works for 24-bit bitmaps'); 
MakeGaussianKernel(K, radius, 255, 1); 
GetMem(theRows, theBitmap.Height * SizeOf(PRow)); 
GetMem(ACol, theBitmap.Height * SizeOf(TRGBTriple)); 

{record the location of the bitmap data:} 
for Row := 0 to theBitmap.Height - 1 do theRows[Row] := theBitmap.Scanline[Row]; 

{blur each row:} 
P := AllocMem(theBitmap.Width * SizeOf(TRGBTriple));
for Row := 0 to theBitmap.Height - 1 do BlurRow(Slice(theRows[Row]^, theBitmap.Width), K, P);

{now blur each column} 
ReAllocMem(P, theBitmap.Height * SizeOf(TRGBTriple)); 
for Col := 0 to theBitmap.Width - 1 do 
	begin 
	{first read the column into a TRow:} 
	for Row := 0 to theBitmap.Height - 1 do ACol[Row] := theRows[Row][Col]; 
	BlurRow(Slice(ACol^, theBitmap.Height), K, P); 

	{now put that row, um, column back into the data:} 
	for Row := 0 to theBitmap.Height - 1 do theRows[Row][Col] := ACol[Row]; 
	end;
FreeMem(theRows);
FreeMem(ACol);
ReAllocMem(P, 0);
end;


{------------- Drawing and Filling routines -----------------------------------}


procedure ColorFill(BM: TBitmap; X,Y: integer; Threshold: double; NewColor: TColor);
{Do fill all pixels that match the pixel found at X,Y}
{Threshold controls how close the match must be}
var X1,Y1: integer;
var TargetColor: TColor;
var Scan: pRGBTripleArray;
var N,T: TRGBTriple;
var D: double;
begin
BM.PixelFormat:=pf24Bit;
N:=ColorToTriple(NewColor);
TargetColor:=BM.Canvas.Pixels[X,Y];
T:=ColorToTriple(TargetColor);
for Y1:=0 to BM.Height-1 do
	begin
	Scan:=BM.ScanLine[Y1];
	for X1:=0 to BM.Width-1 do
		begin
                D:=TripleDistance(Scan[X1],T);
		if D<=Threshold then Scan[X1]:=N;
		end;
	end;
end;



procedure DrawPixels(Image: TImage; X,Y, Size: integer; C: TColor);
{Draw pixels at position specified at XY with pen of Size Width and Height}
var OX,OY,X1,Y1: integer;
begin
Size:=Size-1;
for OY:=-Size to Size do
 for OX:=-Size to Size do
	begin
	X1:=X+OX; Y1:=Y+OY;
	if (X1>=0) and (X1<Image.Width) or
	   (Y1>=0) and (Y1<Image.Height) then Image.Canvas.Pixels[X1,Y1]:=C;
	end;
end;


var OldX,OldY: integer;


procedure ContinuousDraw(Image: TImage; Start: boolean; X,Y, Size: integer; C: TColor);
{Draw pixels at position specified at XY with pen of Size Width and Height}
{Guaranteed to be connected with previously drawing pixels}
begin
if Start then
	begin
	DrawPixels(Image,X,Y,Size div 2,C);
	end
else
	begin
	Image.Canvas.MoveTo(OldX,OldY);
	Image.Canvas.Pen.Width:=Size;
	Image.Canvas.Pen.Color:=C;
	Image.Canvas.LineTo(X,Y);
	end;
OldX:=X; OldY:=Y;
end;

{------------------ Threshold Controllable Fill Routines  --------------------------}

function GetAliasColor(OldColor,FillColor: TRGBTriple): TRGBTriple;
var H1,H2: THSBRecord;
begin
H1:=RGBToHSB(OldColor);
H2:=RGBToHSB(FillColor);
H2.Bright:=H1.Bright * H2.Bright;
Result:=HSBToRGB(H2);
end;


procedure AntiAliasFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{ Fill An Area Of Target Color with Fill Color }
{ Threshold controls which colors match the target color}
{ Threshold = 0 = only an exact match. Threshold = 1 = any color matches}
var Raw: pointer;
var Row: pRGBTRipleArray;
var Que: THugePoint;
var E,BytesPerRow: Integer;
var FC,TC: TRGBTriple;
var MaxInt,I: integer;
const TrimCount=1000;
begin
MaxInt:=round(BM.Width*BM.Height*4);
Que:=THugePoint.Create(5);
try
Que.Clear;
BM.PixelFormat:=pf24Bit;
TC:=ColorToTriple(Target);
FC:=ColorToTriple(Fill);
Raw:=BM.Scanline[0];
BytesPerRow:=(integer(BM.Scanline[1]) - integer(BM.Scanline[0]));
E:= 0;
I:=0;
Que.AddPointer(Point(X,Y),Que.Count);
while E < Que.Count do
	begin
	if ((E mod 1000)=0) and Assigned(ProgressProc) then ProgressProc(MulDiv(100,I,MaxInt));
	X:= Que[E].X; Y:= Que[E].Y;
	Inc(E); Inc(I);
	Row:=pRGBTRipleArray(Integer(Raw) + (BytesPerRow * Y));
	if not EqualRGBTriple(Row[X],FC) and
	   (TripleDistance(Row[X],TC)<=Threshold) then
		begin
		if EqualRGBTriple(Row[X],TC) then
			begin
			Row[X]:=FC;
			if X<BM.Width-1 then Que.AddPointer(Point(X+1, Y),Que.Count);
			if X>0 then Que.AddPointer(Point(X-1, Y),Que.Count);
			if Y<BM.Height-1 then Que.AddPointer(Point(X, Y+1),Que.Count);
			if Y>0 then Que.AddPointer(Point(X, Y-1),Que.Count);
			end
		else Row[X]:=GetAliasColor(Row[X],FC);
		end;

	{Trim off part of array so we don't run out of memory}
	if E>TrimCount then
		begin
		Que.DeleteRange(0,TrimCount-1);
		Dec(E,TrimCount);
		end;
	end;
if Assigned(ProgressProc) then ProgressProc(100);
finally Que.Free; end;
end;



procedure ThresholdFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{ Fill An Area Of Target Color with Fill Color }
{ Threshold controls which colors match the target color}
{ Threshold = 0 = only an exact match. Threshold = 1 = any color matches}
var Raw: pointer;
var Row: pRGBTRipleArray;
var Que: THugePoint;
var E,BytesPerRow: Integer;
var FC,TC: TRGBTriple;
var MaxInt,I: integer;
const TrimCount=1000;
begin
MaxInt:=round(BM.Width*BM.Height*4);
Que:=THugePoint.Create(5);
try
Que.Clear;
BM.PixelFormat:=pf24Bit;
TC:=ColorToTriple(Target);
FC:=ColorToTriple(Fill);
Raw:=BM.Scanline[0];
BytesPerRow:=(integer(BM.Scanline[1]) - integer(BM.Scanline[0]));
E:= 0;
I:=0;
Que.AddPointer(Point(X,Y),Que.Count);
while E < Que.Count do
	begin
	X:= Que[E].X; Y:= Que[E].Y; Inc(E);
	Inc(I);
	Row:=pRGBTRipleArray(Integer(Raw) + (BytesPerRow * Y));
	if not EqualRGBTriple(Row[X],FC) and
	   (TripleDistance(Row[X],TC)<=Threshold) then
		begin
		Row[X]:=FC;
		if X<BM.Width-1 then Que.AddPointer(Point(X+1, Y),Que.Count);
		if X>0 then Que.AddPointer(Point(X-1, Y),Que.Count);
		if Y<BM.Height-1 then Que.AddPointer(Point(X, Y+1),Que.Count);
		if Y>0 then Que.AddPointer(Point(X, Y-1),Que.Count);
		end;
	if ((E mod 1000)=0) and Assigned(ProgressProc) then
		begin
		ProgressProc(MulDiv(100,I,MaxInt));
		end;
	{Trim off part of array so we don't run out of memory}
	if E>TrimCount then
		begin
		Que.DeleteRange(0,TrimCount-1);
		Dec(E,TrimCount);
		end;
	end;
if Assigned(ProgressProc) then ProgressProc(100);
finally Que.Free; end;
end;



procedure SeedFloodFill(BM: TBitmap; X, Y: integer; Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{Flood fills based on seed at X,Y}
var Target: TColor;
begin
Target:=BM.Canvas.Pixels[X,Y];
ThresholdFill(BM,X,Y,Target,Fill,Threshold,ProgressProc);
end;



{------------------Clipboard routines -----------------------------------------}


procedure SaveRectToClipboard(Bitmap: TBitmap; R: TRect);
{Copy specified retangle from bitmap to clipboard}
var MyFormat: Word;
var AData: THandle;
var APalette: HPALETTE;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.Width:=R.Right - R.Left;
BM.Height:=R.Bottom - R.Top;
BM.Canvas.CopyRect(Rect(0,0,BM.Width,BM.Height),Bitmap.Canvas,R);
BM.SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
finally BM.Free; end;
end;



{-------------- Routines to set Bitmap Pixels Per Meter  ----------------------}


const PelPerMeterPosX = 38;
const PelPerMeterPosY = 42;


procedure GetBmpStreamPPM(Stream: TStream; var PPMX,PPMY: integer);
begin
Stream.Position := PelPerMeterPosX;
if Stream.Read(PPMX,4) <> 4 then raise Exception.Create('Error reading to Bitmap.');
Stream.Position := PelPerMeterPosY;
if Stream.Read(PPMY,4) <> 4 then raise Exception.Create('Error reading to Bitmap.');
end;





procedure GetBmpPPM(Bitmap: TBitmap; var PPMX,PPMY: integer);
var Stream: TMemoryStream;
begin
Stream := TMemoryStream.Create;
try
Bitmap.SaveToStream(Stream);
GetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;





procedure GetBMPFilePPM(FileName: string; var PPMX,PPMY: integer);
var Stream: TFileStream;
begin
Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
try
GetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;

{-----------------------------------------------------------------------------}



procedure SetBMPStreamPPM(Stream: TStream; PPMX,PPMY: integer);
begin
Stream.Position := PelPerMeterPosX;
if Stream.Write(PPMX,4) <> 4 then raise Exception.Create('Error writing to Bitmap.');
Stream.Position := PelPerMeterPosY;
if Stream.Write(PPMY,4) <> 4 then raise Exception.Create('Error writing to Bitmap.');
Stream.Position := 0;
end;




procedure SetBMPPPM(BM: TBitmap; PPMX,PPMY: integer);
var Stream: TMemoryStream;
begin
Stream := TMemoryStream.Create;
try
BM.SaveToStream(Stream);
SetBMPStreamPPM(Stream,PPMX,PPMY);
Stream.Position := 0;
BM.LoadFromStream(Stream);
finally Stream.Free; end;
end;



procedure SetBMPFilePPM(FileName: String; PPMX,PPMY: integer);
var Stream: TFileStream;
begin
Stream := TFileStream.Create(FileName, fmOpenWrite or fmShareExclusive);
try
SetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;


procedure CopyBMPPPM(Source,Destination: TBitmap);
{Copy the Bits Per Meter from source to destination}
var X,Y: integer;
begin
GetBmpPPM(Source,X,Y);
SetBmpPPM(Destination,X,Y);
end;


procedure SetBMPPixelFormat(BM: TBitmap; PF: TPixelFormat);
{Set pixel format while maintaining other attributes}
var X,Y: integer;
begin
if not BM.Empty then GetBmpPPM(BM,X,Y);
BM.PixelFormat:=PF;
if not BM.Empty then SetBmpPPM(BM,X,Y);
end;

{------------------------------------------------------------------------------}



procedure GetJPEGResolution(Filename: string; var Units: TJPEGResUnits; var XRes,YRes: integer);
const BufferSize = 50;
var Buffer: string;
var Index: integer;
var FileStream: TFileStream;
var DP: Byte;
var HorzRes, VertRes: word;
begin
FileStream := TFileStream.Create(Filename, fmOpenReadWrite);
try
SetLength(Buffer, BufferSize);
FileStream.Read(buffer[1], BufferSize);
Index := Pos('JFIF' + #$00, buffer);
if Index > 0 then
	begin
	FileStream.Seek(Index + 6, soFromBeginning);
	FileStream.Read(DP, 1);
	case DP of
	 0: Units:=juNoUnits;
	 1: Units:=juDotsPerInch;
	 2: Units:=juDotsPerCentimeter;
	 end;
	FileStream.Read(HorzRes, 2); // x axis
	XRes := Swap(HorzRes);
	FileStream.Read(VertRes, 2); // y axis
	YRes := Swap(VertRes);
	end
finally FileStream.Free; end;
end;



procedure SetJPEGResolution(Filename: string; DPIX, DPIY: Integer; Units: TJPEGResUnits=juDotsPerInch);
const BufferSize = 50;
var Buffer: string;
var index: integeR;
var FileStream: TFileStream;
var XResolution: word;
var YResolution: word;
var ResUints: byte;
begin
FileStream := TFileStream.Create(Filename,fmOpenReadWrite);
try
SetLength(Buffer, BufferSize);
FileStream.Read(buffer[1], BufferSize);
index := POS('JFIF' + #$00, buffer);
if index > 0 then
	begin
	FileStream.Seek(index + 6, soFromBeginning);
	ResUints := byte(Units);
	FileStream.write(ResUints, 1);
	XResolution := swap(DPIX);
	FileStream.write(XResolution, 2);
	YResolution := swap(DPIY);
	FileStream.write(YResolution, 2);
	end
finally FileStream.Free; end;
end;



{-------------- Routines to do Flips and rotations mod 90 degrees -------------}


procedure FlipBitmapHoriz(Source,Destination: TBitmap);
var X,Y,W: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;
W:=Source.Width-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2[W-X]:=Scan1[X];
		end;
	end;
end;




procedure FlipBitmapVertical(Source,Destination: TBitmap);
var X,Y,H: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;
H:=Source.Height-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[H-Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2[X]:=Scan1[X];
		end;
	end;
end;





procedure FlipBitmap90(Source,Destination: TBitmap);
var X,Y,H: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Height;
Destination.Height:=Source.Width;
H:=Source.Height-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2:=Destination.ScanLine[X];
		Scan2[H-Y]:=Scan1[X];
		end;
	end;
end;


procedure FlipBitmap270(Source,Destination: TBitmap);
var BM,BM2: TBitmap;
begin
BM:=TBitmap.Create;
BM2:=TBitmap.Create;
try
FlipBitmap90(Source,BM);
FlipBitmapHoriz(BM,BM2);
FlipBitmapVertical(BM2,Destination);
finally
 BM.Free;
 BM2.Free;
 end;
end;




procedure FlipBitmap180(Source,Destination: TBitmap);
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
FlipBitmapHoriz(Source,BM);
FlipBitmapVertical(BM,Destination);
finally
 BM.Free;
 end;
end;


{------------------------------------------------------------------------------}



procedure DoRotateBitmap(Source, Destination: TBitmap; Center,Offset: TPoint; Angle,Scale: double; BackGround: TColor);
{Do low level bitmap rotation - highly optimized for speed }
{Copies and rotates bitmap from "Source" to "Destination" (both must be 24-bit)}
{Rotation is around "Center" and image is shifted by "Offset" vector }
{"Angle" controls rotation - Positive angles rotate clockwise}
{"Scale" scales the image up or down in size i.e. }
{"Background" is the color of the background when rotated}
var X,Y: integer;
var BytesPerRowSrc,BytesPerRowDest: integer;
var CosTheta,SinTheta: extended;
var ScaledSinTheta,ScaledCosTheta: integer;
var SourceX,SourceY: integer;
var DY: integer;
var RotatedRow: pRGBTRipleArray;
var BackTriple: TRGBTriple;
var RawSource,RawDestination: Pointer;
var SourceWidth,SourceHeight,DestinationWidth,DestinationHeight: integer;
var XOffFact,YOffFact: integer;
var SinFact1,CosFact1: integer;
var SinFact2,CosFact2: array of integer;
const IntegerScaler = 512;
begin
GDIFlush;
{Convert background color to RGB triple for speed}
BackTriple:=ColorToTriple(BackGround);

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale by power of two so we can use integers for speed}
{Also scale to control size}
ScaledSinTheta:=Round(SinTheta * IntegerScaler * (1/Scale));
ScaledCosTheta:=Round(CosTheta * IntegerScaler * (1/Scale));

{Calculate byte per row}
BytesPerRowSrc:=(integer(Source.Scanline[1]) - integer(Source.Scanline[0]));
BytesPerRowDest:=(integer(Destination.Scanline[1]) - integer(Destination.Scanline[0]));

{Get pointer to raw pixels for speed}
RawSource:=Source.Scanline[0];
RawDestination:= Destination.Scanline[0];

{Use predecremented constants for speed}
SourceWidth:=Source.Width-1;
SourceHeight:=Source.Height-1;
DestinationWidth:=Destination.Width-1;
DestinationHeight:=Destination.Height-1;

{Precalculate Offset factors for speed}
XOffFact:=Center.X + Offset.X;
YOffFact:=Center.Y + Offset.Y;

{Precalculate X-based Sin/Cos factors for speed}
{ First, Allocate memory for arrays}
SetLength(SinFact2,Destination.Width);
SetLength(CosFact2,Destination.Width);
{ Then fill arrays with factors}
for X:=0 to Destination.Width -1 do
	begin
	SinFact2[X]:=(X - XOffFact) * ScaledSinTheta;
	CosFact2[X]:=(X - XOffFact) * ScaledCosTheta;
	end;


{Step through all rows (Decrementing a for-loop is faster)}
for Y:= DestinationHeight downto 0 do
	begin
	{Caculate all "Y" based parameters}
	RotatedRow:= pRGBTRipleArray(Integer(RawDestination) + (BytesPerRowDest * Y));
	DY:=Y - YOffFact;
	SinFact1:=DY * ScaledSinTheta;
	CosFact1:=DY * ScaledCosTheta;
	{Step through all pixels in row}
	for X:= DestinationWidth downto 0 do
		begin
		{calculate the rotated X,Y indexes that point into source bitmap}
		SourceX:= Center.X + (CosFact2[X] - SinFact1) div IntegerScaler;
		SourceY:= Center.Y + (SinFact2[X] + CosFact1) div IntegerScaler;
		{Is Rotated X,Y still inside Source? If not write background color}
		if (SourceX >= 0) and (SourceX <= SourceWidth) and
		   (SourceY >= 0) and (SourceY <= SourceHeight) then
			RotatedRow[X]:= pRGBTRipleArray(Integer(RawSource)+(SourceY * BytesPerRowSrc))^[SourceX]
		else RotatedRow[X]:=BackTriple;
		end;
	end;
end;



procedure RotateBitmap(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
{Rotates 'Source' and returns result in 'Destination'. Positive angles rotate clock wise}
{Options explained above. Center specifies center of rotation only for clipped modes}
{Background is the color of the background when rotated}
var Offset: TPoint;
var CosTheta,SinTheta: extended;
var SS: TPoint;
begin
Destination.Canvas.Lock;
Source.Canvas.Lock;
try
{Force pixel format and make DIBs (otherwise there will be memory limitations)}
Source.PixelFormat:=pf24bit;
Destination.PixelFormat:= pf24bit;

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale Source Size for scaling operations}
SS:=IntScalarProduct(Point(Source.Width,Source.Height),Scale);
{Handle framing options}
case Options of
 foSource:
	begin
	{Use un-rotated source for bitmap size}
	Destination.Width:= Source.Width;
	Destination.Height:= Source.Height;
	Offset.X:=0; Offset.Y:=0;
	end;
 foDestination:
	begin
	{Use un-rotated destination for bitmap size}
       	Offset.X:=0; Offset.Y:=0;
	end;
 foDestCenter:
	begin
	{Put rotational center in center of destination}
	Offset.X:=-(Center.X - (Destination.Width div 2));
	Offset.Y:=-(Center.Y - (Destination.Height div 2));
	end;
 foFit:
	begin
	{Use rotated image as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:= abs(round(SS.Y * sinTheta)) + abs(round(SS.X * cosTheta));
	Destination.Height:= abs(round(SS.X * sinTheta)) + abs(round(SS.Y * cosTheta));
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 foWindow:
	begin
	{Use largest possible size as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:=round(sqrt(sqr(SS.Y) + sqr(SS.X)));
	Destination.Height:=Destination.Width;
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 else begin Offset.X:=0; Offset.Y:=0; end;
 end;

DoRotateBitmap(Source, Destination,Center,Offset,Angle,Scale,BackGround);

finally
 Destination.Canvas.Unlock;
 Source.Canvas.Unlock;
 end;
end;



{------------------------------------------------------------------------------}


procedure DoRotateBitmap32(Source, Destination: TBitmap; Center,Offset: TPoint; Angle,Scale: double; BackGround: TColor);
{Do low level bitmap rotation - highly optimized for speed }
{Copies and rotates bitmap from "Source" to "Destination" (both must be 32-bit)}
{Rotation is around "Center" and image is shifted by "Offset" vector }
{"Angle" controls rotation - Positive angles rotate clockwise}
{"Scale" scales the image up or down in size i.e. }
{"Background" is the color of the background when rotated}
var X,Y: integer;
var BytesPerRowSrc,BytesPerRowDest: integer;
var CosTheta,SinTheta: extended;
var ScaledSinTheta,ScaledCosTheta: integer;
var SourceX,SourceY: integer;
var DY: integer;
var RotatedRow: pRGBQuadArray;
var BackQuad: TRGBQuad;
var RawSource,RawDestination: Pointer;
var SourceWidth,SourceHeight,DestinationWidth,DestinationHeight: integer;
var XOffFact,YOffFact: integer;
var SinFact1,CosFact1: integer;
var SinFact2,CosFact2: array of integer;
const IntegerScaler = 512;
begin
GDIFlush;
{Convert background color to RGB Quad for speed}
BackQuad:=ColorToQuad(BackGround);
BackQuad.rgbReserved:=0;
{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale by power of two so we can use integers for speed}
{Also scale to control size}
ScaledSinTheta:=Round(SinTheta * IntegerScaler * (1/Scale));
ScaledCosTheta:=Round(CosTheta * IntegerScaler * (1/Scale));

{Calculate byte per row}
BytesPerRowSrc:=(integer(Source.Scanline[1]) - integer(Source.Scanline[0]));
BytesPerRowDest:=(integer(Destination.Scanline[1]) - integer(Destination.Scanline[0]));

{Get pointer to raw pixels for speed}
RawSource:=Source.Scanline[0];
RawDestination:= Destination.Scanline[0];

{Use predecremented constants for speed}
SourceWidth:=Source.Width-1;
SourceHeight:=Source.Height-1;
DestinationWidth:=Destination.Width-1;
DestinationHeight:=Destination.Height-1;

{Precalculate Offset factors for speed}
XOffFact:=Center.X + Offset.X;
YOffFact:=Center.Y + Offset.Y;

{Precalculate X-based Sin/Cos factors for speed}
{ First, Allocate memory for arrays}
SetLength(SinFact2,Destination.Width);
SetLength(CosFact2,Destination.Width);
{ Then fill arrays with factors}
for X:=0 to Destination.Width -1 do
	begin
	SinFact2[X]:=(X - XOffFact) * ScaledSinTheta;
	CosFact2[X]:=(X - XOffFact) * ScaledCosTheta;
	end;


{Step through all rows (Decrementing a for-loop is faster)}
for Y:= DestinationHeight downto 0 do
	begin
	{Caculate all "Y" based parameters}
	RotatedRow:= pRGBQuadArray(Integer(RawDestination) + (BytesPerRowDest * Y));
	DY:=Y - YOffFact;
	SinFact1:=DY * ScaledSinTheta;
	CosFact1:=DY * ScaledCosTheta;
	{Step through all pixels in row}
	for X:= DestinationWidth downto 0 do
		begin
		{calculate the rotated X,Y indexes that point into source bitmap}
		SourceX:= Center.X + (CosFact2[X] - SinFact1) div IntegerScaler;
		SourceY:= Center.Y + (SinFact2[X] + CosFact1) div IntegerScaler;
		{Is Rotated X,Y still inside Source? If not write background color}
		if (SourceX >= 0) and (SourceX <= SourceWidth) and
		   (SourceY >= 0) and (SourceY <= SourceHeight) then
			RotatedRow[X]:= pRGBQuadArray(Integer(RawSource)+(SourceY * BytesPerRowSrc))^[SourceX]
		else RotatedRow[X]:=BackQuad;
		end;
	end;
end;



procedure RotateBitmap32(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
{Rotates 'Source' and returns result in 'Destination'. Positive angles rotate clock wise}
{Options explained above. Center specifies center of rotation only for clipped modes}
{Background is the color of the background when rotated}
var Offset: TPoint;
var CosTheta,SinTheta: extended;
var SS: TPoint;
begin
Destination.Canvas.Lock;
Source.Canvas.Lock;
try
{Force pixel format and make DIBs (otherwise there will be memory limitations)}
Source.PixelFormat:=pf32bit;
Destination.PixelFormat:=pf32bit;

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale Source Size for scaling operations}
SS:=IntScalarProduct(Point(Source.Width,Source.Height),Scale);
{Handle framing options}
case Options of
 foSource:
	begin
	{Use un-rotated source for bitmap size}
	Destination.Width:= Source.Width;
	Destination.Height:= Source.Height;
	Offset.X:=0; Offset.Y:=0;
	end;
 foDestination:
	begin
	{Use un-rotated destination for bitmap size}
       	Offset.X:=0; Offset.Y:=0;
	end;
 foDestCenter:
	begin
	{Put rotational center in center of destination}
	Offset.X:=-(Center.X - (Destination.Width div 2));
	Offset.Y:=-(Center.Y - (Destination.Height div 2));
	end;
 foFit:
	begin
	{Use rotated image as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:= abs(round(SS.Y * sinTheta)) + abs(round(SS.X * cosTheta));
	Destination.Height:= abs(round(SS.X * sinTheta)) + abs(round(SS.Y * cosTheta));
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 foWindow:
	begin
	{Use largest possible size as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:=round(sqrt(sqr(SS.Y) + sqr(SS.X)));
	Destination.Height:=Destination.Width;
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 else begin Offset.X:=0; Offset.Y:=0; end;
 end;

DoRotateBitmap32(Source, Destination,Center,Offset,Angle,Scale,BackGround);

finally
 Destination.Canvas.Unlock;
 Source.Canvas.Unlock;
 end;
end;


{------------------------------------------------------------------------------}

procedure BorderBitmap(BM: TBitmap; Color: TColor; Width: integer);
{Add equal border around bitmap of specified width and color}
var BM2: TBitmap;
begin
BM2:=TBitmap.Create;
try
BM2.Assign(BM);
BM.Width:=BM.Width + Width * 2;
BM.Height:=BM.Height + Width * 2;
BM.Canvas.Brush.Color:=Color;
BM.Canvas.Pen.Color:=Color;
BM.Canvas.Rectangle(Rect(0,0,BM.Width,BM.Height));
BM.Canvas.Draw(Width,Width,BM2);
finally BM2.Free; end;
end;

{---------------- Routines to Manipulate RGB and HSV --------------------------}




procedure MakeBlackAndWhite(Source,Destination: TBitmap; Invert: boolean; Threshold: double);
{Make image black and white based on threshold setting (0-1)}
var X,Y: integer;
var Scan1,Scan2: pRGBTripleArray;
var A: double;
const MaxAbs = 1.73205080757;
var Black,White,T: TRGBTriple;
begin
Threshold:=Threshold * MaxAbs;
Black:=MakeRBGTriple(0,0,0);
White:=MakeRBGTriple(255,255,255);
if Invert then begin T:=Black; Black:=White; White:=T; end;

Source.PixelFormat:=pf24Bit;
Destination.PixelFormat:=pf24Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;

for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		A:=VectorABS3D(RGBTripleToVector(Scan1[X]));
		if A>Threshold then Scan2[X]:=White
		else Scan2[X]:=Black;
		end;
	end;
end;



function RGBtoHSB(RGB: TRGBTriple): THSBRecord;
{Convert RGB to  Hue,Saturation and Brightness}
{Hue = 0-360, Sat=0-1, Bright = 0-1}
var MinValue,MaxValue,Range: extended;
var RedPercent,GreenPercent,BluePercent: extended;
begin
Result.Sat:=0;
Result.Hue:=-1;
{Convert RGB to percentage}
RedPercent:=RGB.rgbtRed/255; GreenPercent:=RGB.rgbtGreen/255; BluePercent:=RGB.rgbtBlue/255;
MinValue:=min(RedPercent,min(GreenPercent,BluePercent));
MaxValue:=max(RedPercent,max(GreenPercent,BluePercent));
{Calculate Value (Brightness)}
Result.Bright:=MaxValue;
{Calculate Saturation}
Range:=MaxValue-MinValue;
if MaxValue<>0 then Result.Sat:=Range/MaxValue else  exit;
{Calculate Hue}
if Range=0 then Result.Hue:=0
else if RedPercent=MaxValue then Result.Hue:= (GreenPercent-BluePercent)/Range	// between yellow & magenta
else if GreenPercent=MaxValue then Result.Hue:=2+(BluePercent-RedPercent)/Range	// between cyan & yellow
else Result.Hue:=4+(RedPercent-GreenPercent)/Range;				// between magenta & cyan
{Convert to degrees}
Result.Hue:=Result.Hue * 60;
while Result.Hue<0 do Result.Hue:=Result.Hue + 360;
end;



function ColorToHSB(Color: TColor): THSBRecord;
var RGB: TRGBTriple;
begin
RGB:=ColorToTriple(Color);
Result:=RGBToHSB(RGB);
end;



function MakeHSB(Hue,Sat,Bright: extended): THSBRecord;
begin
Result.Hue:=Hue;
Result.Sat:=Sat;
Result.Bright:=Bright;
end;



function HSBtoRGB(HSB: THSBRecord): TRGBTriple;
{Convert Hue,Saturation and Brightness to RGB}
{Hue = 0-360, Sat=0-1, Bright = 0-1}
var HexIndex: integer;
var Fraction,X,Y,Z: double;
var Hextant: double;
begin
{if saturation = 0 then the color is gray}
if HSB.Sat=0 then
	begin
	{Set gray value according to brightness}
	Result.rgbtRed:=trunc(HSB.Bright*255);
	Result.rgbtGreen:=trunc(HSB.Bright*255);
	Result.rgbtBlue:=trunc(HSB.Bright*255);
	exit;
	end;
{Calculate Hue Hextant}
Hextant:=HSB.Hue/60;
HexIndex:=floor(Hextant);
Fraction:=Hextant-HexIndex;

{Calculate coor vector}
X:=HSB.Bright * (1-HSB.Sat);
Y:=HSB.Bright * (1-HSB.Sat*Fraction);
Z:=HSB.Bright * (1-HSB.Sat*(1-Fraction));
case HexIndex of
 0:
	begin
	{Handle Red - Yellow Hextant}
	Result.rgbtRed := trunc(HSB.Bright*255);
	Result.rgbtGreen := trunc(Z*255);
	Result.rgbtBlue := trunc(X*255);
	end;
 1:
	begin
	{Handle Yellow - Green Hextant}
	Result.rgbtRed := trunc(Y*255);
	Result.rgbtGreen := trunc(HSB.Bright*255);
	Result.rgbtBlue := trunc(X*255);
	end;
 2:
	begin
	{Handle Green - Cyan Hextant}
	Result.rgbtRed:= trunc(X*255);
	Result.rgbtGreen := trunc(HSB.Bright*255);
	Result.rgbtBlue := trunc(Z*255);
	end;
 3:
	begin
	{Handle Cyan - Blue Hextant}
	Result.rgbtRed := trunc(X*255);
	Result.rgbtGreen := trunc(Y*255);
	Result.rgbtBlue := trunc(HSB.Bright*255);
	end;
 4:
	begin
	{Handle Blue - Magenta Hextant}
	Result.rgbtRed := trunc(Z*255);
	Result.rgbtGreen := trunc(X*255);
	Result.rgbtBlue := trunc(HSB.Bright*255);
	end;
 else
	begin
	{Handle Magenta - Red Hextant}
	Result.rgbtRed := trunc(HSB.Bright*255);
	Result.rgbtGreen := trunc(X*255);
	Result.rgbtBlue := trunc(Y*255);
	end;
 end;
end;


{------------------ Routines to manipulate HSL Colors models ------------------}


function HSLToRGB(HSL: THSLRecord): TRGBTriple;
{Convert Hue,Saturation and Luminesance to RGB}
{Hue = 0-360, Sat=0-1, Lume = 0-1}
var M1, M2: Single;

	function HueToColourValue(Hue: Single): Byte;
	var V: Double;
	begin
	Hue := Hue - Floor(Hue);

	if 6 * Hue < 1 then V := M1 + (M2 - M1) * Hue * 6
	else if 2 * Hue < 1 then V := M2
	else if 3 * Hue < 2 then V := M1 + (M2 - M1) * (2 / 3 - Hue) * 6
	else V := M1;
	Result := Round(255 * V);
	end;


begin
{Convert 0-360 to 0-1}
HSL.Hue:=HSL.Hue/360;
if HSL.Sat = 0 then
	begin
	Result.rgbtRed := Round(255 * HSL.Lume);
	Result.rgbtGreen := Result.rgbtRed;
	Result.rgbtBlue := Result.rgbtRed;
	end
else
	begin
	if HSL.Lume <= 0.5 then M2 := HSL.Lume * (1 + HSL.Sat)
	else M2 := HSL.Lume + HSL.Sat - HSL.Lume * HSL.Sat;
	M1 := 2 * HSL.Lume - M2;
	Result.rgbtRed := HueToColourValue(HSL.Hue + 1 / 3);
	Result.rgbtGreen := HueToColourValue(HSL.Hue);
	Result.rgbtBlue := HueToColourValue(HSL.Hue - 1 / 3)
	end;
end;




function RGBToHSL(RGB: TRGBTriple): THSLRecord;
{Convert RGB to Hue,Saturation and Luminesance }
{Hue = 0-360, Sat=0-1, Lume = 0-1}
var R, G, B, D, Cmax, Cmin: Single;
begin
R := RGB.rgbtRed / 255;
G := RGB.rgbtGreen / 255;
B := RGB.rgbtBlue / 255;
Cmax := Max(R, Max(G, B));
Cmin := Min(R, Min(G, B));
Result.Lume := (Cmax + Cmin) / 2;

if Cmax = Cmin then
	begin
	Result.Hue := 0;
	Result.Sat := 0
	end
else
	begin
	D := Cmax - Cmin;
	if Result.Lume < 0.5 then Result.Sat := D / (Cmax + Cmin)
	else Result.Sat := D / (2 - Cmax - Cmin);
	if R = Cmax then Result.Hue := (G - B) / D
	else if G = Cmax then Result.Hue := 2 + (B - R) /D
	else Result.Hue := 4 + (R - G) / D;
	Result.Hue := Result.Hue / 6;
	if Result.Hue < 0 then Result.Hue := Result.Hue + 1;
	Result.Hue := Result.Hue * 360;
	end;
end;



function MakeHSL(Hue,Sat,Lume: single): THSLRecord;
begin
Result.Hue:=Hue;
Result.Sat:=Sat;
Result.Lume:=Lume;
end;


{------------------------------------------------------------------------------}


procedure AdjustHue(Source,Destination: TBitmap; Offset: double);
{Adjusts hue values by Offset }
{Ofset should be in the range of -180 to +180}
var X, Y: Integer;
var Scan1,Scan2: pRGBTripleArray;
var HSB: THSBRecord;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		HSB:=RGBToHSB(Scan1[X]);
		{Offset Hue and normalize to 0-360}
		HSB.Hue:=HSB.Hue + Offset;
		while HSB.Hue>360 do HSB.Hue:=HSB.Hue -360;
		while HSB.Hue<0 do HSB.Hue:=HSB.Hue +360;
		Scan2[X]:=HSBtoRGB(HSB);
		end;
	end;
end;




procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double);
{Brightness, Constrast and Gamma should be between 0 and 1 with 0.5 representing no change}
var X, Y: Integer;
var Scale: double;
var Scan1,Scan2: pRGBTripleArray;
var RGB: TRGBTriple;
var AdjustArray: array [0..255] of integer;
var I: integer;
var Tmp: double;
const Gray = 128;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;

{Change Brightness/Contrast ranges from 0 - 1 to 0.1 to 10 }
Brightness:=Power(10,(Brightness - 0.5)*2);
Contrast:=Power(10,(Contrast - 0.5)*2);

{Change Gamma range from 0 - 1 to 0.1 to 10 }
Gamma:=Power(10,(Gamma - 0.50) * 2);
if Gamma<=0 then Gamma:=100 else Gamma:=1/Gamma;

{Used to scale gamma values back to 0 to 255 range}
Scale:=256/power(256,Gamma);

{Precalculate all 256 possible combined Gamma, Constrast and Brightness}
{Gamma function f(x) = x ^ (1 / gamma)}
for I:=0 to 255 do
	begin
	Tmp:=Power(I,Gamma) * Scale;
	Tmp:=((Tmp - Gray) * Contrast) + Gray;
	AdjustArray[I]:=trunc(EnsureRange(Tmp * Brightness,0,255));
	end;

for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		RGB:=Scan1[X];
		{Adjust Gamma, Contrast and Brightness}
		RGB.rgbtRed := AdjustArray[RGB.rgbtRed];
		RGB.rgbtGreen := AdjustArray[RGB.rgbtGreen];
		RGB.rgbtBlue := AdjustArray[RGB.rgbtBlue];
		Scan2[X]:=RGB;
		end;
	end;
end;



procedure AdjustSaturation(Source,Destination: TBitmap; Offset: double);
{Adjusts saturation values by Offset }
{Ofset should be in the range of -1 to +1}
var X, Y: Integer;
var Scan1,Scan2: pRGBTripleArray;
var HSB: THSBRecord;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		HSB:=RGBToHSB(Scan1[X]);
		{Offset saturation and normalize 0 - 1}
		HSB.Sat:=HSB.Sat + Offset;
		if HSB.Sat>1 then HSB.Sat:=1;
		if HSB.Sat<0 then HSB.Sat:=0;
		Scan2[X]:=HSBtoRGB(HSB);
		end;
	end;
end;



procedure DoWinResize(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Mode: integer);
{Use windows StretchBlt to resize image}
var Width,Height: integer;
begin
SetStretchBltMode(Canvas.Handle, Mode);
Width:= DestRect.Right  - DestRect.Left;
Height:= DestRect.Bottom - DestRect.Top;
StretchBlt(Canvas.Handle,DestRect.Left,DestRect.Top,Width,Height, Bitmap.Canvas.Handle,0,0, Bitmap.Width, Bitmap.Height, Canvas.CopyMode);
end;


procedure OversampleBitmap(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap);
{Use oversampling to smooth image}
begin
DoWinResize(Canvas,DestRect,Bitmap,HALFTONE);
end;



procedure ResizeBitmap(Source,Destination: TBitmap; Percent: double; OverSample: boolean);
{Resize image using Window API for faster oversampling}
var R: TRect;
begin
R.Left:=0;
R.Top:=0;
R.Right:=Trunc(Source.Width * Percent);
R.Bottom:=Trunc(Source.Height * Percent);
Destination.Width:=R.Right;
Destination.Height:=R.Bottom;
if Oversample then OverSampleBitmap(Destination.Canvas,R,Source)
else Destination.Canvas.StretchDraw(R,Source);
end;



procedure ResizeBitmap(Source,Destination: TBitmap; Size: TPoint; OverSample: boolean);
{Resize image using Window API for faster oversampling}
var R: TRect;
begin
R.Left:=0;
R.Top:=0;
R.Right:=Size.X;
R.Bottom:=Size.Y;
Destination.Width:=R.Right;
Destination.Height:=R.Bottom;
if Oversample then OverSampleBitmap(Destination.Canvas,R,Source)
else Destination.Canvas.StretchDraw(R,Source);
end;

{---------------- Alpha Blending Routines -------------------------------------}




procedure SetAlphaByColor(BM: TBitmap; C: TColor; Alpha: byte; Reset: boolean);
{Set each pixel to opaque or transparent based on color}
{Any color that is the same as C will be set tranparent according to the Alpha value}
{If Reset is true, the non matching colors are set to opaque}
{If Reset is false, the non matching colors are ignored }
{This allows different colors to be set to different levels of transparency}
{Used with the AlphaBlendBitmaps routine and per pixel mode}
var X,Y: integer;
var SL: PRGBQuadArray;
var A: integer;
var CT,T: TRGBTriple;
begin
BM.PixelFormat:=pf32Bit;
CT:=ColorToTriple(C);
for Y:=0 to BM.Height-1 do
	begin
	SL:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
		begin
		T:=PRGBTriple(@SL[X])^;
		if EqualRGBTriple(T,CT) then A:=Alpha
		else if Reset then A:=255 else continue;
		SL[X].rgbReserved:=A;
		SL[X].rgbRed:=MulDiv(SL[X].rgbRed,A,255);
		SL[X].rgbGreen:=MulDiv(SL[X].rgbGreen,A,255);
		SL[X].rgbBlue:=MulDiv(SL[X].rgbBlue,A,255);
		end;
	end;
end;


procedure SetAlphaByColor(BM: TBitmap; C: TColor);
begin
SetAlphaByColor(BM,C,0,True);
end;


function GetPixelBrightness(R,G,B: byte): integer;
{Get perceived brightness of a pixel}
var V: double;
begin
V:=sqrt((0.241*R*R) + (0.691*G*G) + (0.068*B*B));
Result:=trunc(V);
end;



procedure SetAlphaByLightness(BM: TBitmap; DoLightness: boolean; Lightness: byte);
{Set transparency based on Lightness of pixel}
{Pixels that are as light or lighter will be set transparent}
{Used with the AlphaBlendBitmaps routine}
var X,Y: integer;
var SL: PRGBQuadArray;
var Alpha: integer;
var L: byte;
begin
BM.PixelFormat:=pf32Bit;
for Y:=0 to BM.Height-1 do
	begin
	SL:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
		begin
                L:=GetPixelBrightness(SL[X].rgbRed,SL[X].rgbGreen,SL[X].rgbBlue);
		if DoLightness and (L>=Lightness) then Alpha:=0 else Alpha:=255;
		SL[X].rgbReserved:=Alpha;
		SL[X].rgbRed:=MulDiv(SL[X].rgbRed,Alpha,255);
		SL[X].rgbGreen:=MulDiv(SL[X].rgbGreen,Alpha,255);
		SL[X].rgbBlue:=MulDiv(SL[X].rgbBlue,Alpha,255);
		end;
	end;
end;


procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte; PerPixel: boolean);
{Do alpha blending of Bitmap with specified canvas}
var Width,Height: integer;
var Blend: TBlendFunction;
begin
Width:= DestRect.Right  - DestRect.Left;
Height:= DestRect.Bottom - DestRect.Top;
{Note: AC_SRC_OVER only used with 32bit, per pixel alpha}
if PerPixel then Blend.AlphaFormat:=AC_SRC_ALPHA
else Blend.AlphaFormat:=0;
Blend.BlendOp:=AC_SRC_OVER;
Blend.BlendFlags:=0;
Blend.SourceConstantAlpha := Alpha;

AlphaBlend(Canvas.Handle,DestRect.Left,DestRect.Top,Width,Height,Bitmap.Canvas.Handle,0,0, Bitmap.Width, Bitmap.Height, Blend);
end;



procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte);
begin
AlphaBlendBitmaps(Canvas, DestRect,  Bitmap, Alpha, False);
end;




procedure AlphaBlendCanvas(DestCanvas: TCanvas; DestRect: TRect; SourceCanvas: TCanvas; SourceRect: TRect; Alpha: byte; PerPixel: boolean);
{Do alpha blending of specified canvases}
var DestSize, SourceSize: TPoint;
var Blend: TBlendFunction;
begin
DestSize:=IntVectorSubtract(DestRect.BottomRight,DestRect.TopLeft);
SourceSize:=IntVectorSubtract(SourceRect.BottomRight,SourceRect.TopLeft);
{Note: AC_SRC_OVER only used with 32bit, per pixel alpha}
if PerPixel then Blend.AlphaFormat:=AC_SRC_ALPHA
else Blend.AlphaFormat:=0;
Blend.BlendOp:=AC_SRC_OVER;
Blend.BlendFlags:=0;
Blend.SourceConstantAlpha := Alpha;

Windows.AlphaBlend(DestCanvas.Handle,DestRect.Left,DestRect.Top,DestSize.X,DestSize.Y,
		   SourceCanvas.Handle,SourceRect.Left,SourceRect.Top,SourceSize.X,SourceSize.Y, Blend);
end;


{------------------ Routines to Handle And Masks ------------------------------}




procedure MakeMask(var Mask: TBitmap; Color: TColor);
{Generate a monochrome mask }
{ All colors match that match "Color" are white eveything else black}
{Solves the problem of the Brush Color interferings with white color}
var BM: TBitmap;
begin
Mask.Mask(Color);
BM:=TBitmap.Create;
BM.Width:=Mask.Width;
BM.Height:=Mask.Height;
BM.Canvas.Brush.Color:=clWhite;
BM.Canvas.Draw(0,0,Mask);
Mask.Assign(BM);
BM.Free;
end;



{-------------------Do Image Clipping -----------------------------------------}



function IsBitmapBlank(BM: TBitmap): boolean;
{Gheck if bitmap is all white or blank}
var X,Y: integer;
var Scan: pRGBTripleArray;
begin
Result:=False;
BM.PixelFormat:=pf24Bit;
for Y:=0 to BM.Height-1 do
	begin
	Scan:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
	 if (Scan[X].rgbtRed<>$FF) or
	    (Scan[X].rgbtGreen<>$FF) or
	    (Scan[X].rgbtBlue<>$FF) then exit;
	end;
Result:=True;
end;

{-------------------Do Image Clipping -----------------------------------------}


function ClipBitmap(BM: TBitmap; R: TRect): boolean;
{Clip bitmap to rectangle}
var R2: TRect;
var T: integer;
begin
BM.Dormant;
BM.FreeImage;
Result:=False;
if R.Left=R.Right then exit;
if R.Top=R.Bottom then exit;

if R.Left>R.Right then begin T:=R.Left; R.Left:=R.Right; R.Right:=T; end;
if R.Top>R.Bottom then begin T:=R.Top; R.Top:=R.Bottom; R.Bottom:=T; end;

if R.Right>BM.Width then R.Right:=BM.Width;
if R.Left<0 then R.Left:=0;
if R.Bottom>BM.Height then R.Bottom:=BM.Height;
if R.Top<0 then R.Top:=0;

R2:=R;
OffsetRect(R2,-R.Left,-R.Top);
try
BM.Canvas.CopyRect(R2,BM.Canvas,R);
BM.Width:=R.Right-R.Left;
BM.Height:=R.Bottom-R.Top;
Result:=True;
except end;
end;



{----------- Routines to Offset bitmap wrapping around the image --------------}



procedure WrapImageHorizontal(BM: TBitmap; Delta: integer);
{Wrap bitmap in the horizontal axis according to the Delta}
var TBM: TBitmap;
var BandFrom,BandTo,BlockFrom,BlockTo: TRect;
begin
TBM:=TBitmap.Create;
try
TBM.Height:=BM.Height;
TBM.Width:=abs(Delta);

BandFrom:=Rect(0,0,abs(Delta),TBM.Height);
BandTo:=BandFrom;
OffsetRect(BandTo,BM.Width-abs(Delta),0);
BlockFrom:=Rect(abs(Delta),0,BM.Width,BM.Height);
BlockTo:=BlockFrom;
OffsetRect(BlockTo,-abs(Delta),0);
if Delta>=0 then
	begin
	SwapRectangles(BandFrom,BandTo);
	SwapRectangles(BlockFrom,BlockTo);
	end;
TBM.Canvas.CopyRect(Rect(0,0,TBM.Width,TBM.Height),BM.Canvas,BandFrom);
BM.Canvas.CopyRect(BlockTo,BM.Canvas,BlockFrom);
BM.Canvas.CopyRect(BandTo,TBM.Canvas,Rect(0,0,TBM.Width,TBM.Height));
finally TBM.Free; end;
end;




procedure WrapImageVertical(BM: TBitmap; Delta: integer);
{Wrap bitmap in the Vertical axis according to the Delta}
var TBM: TBitmap;
var BandFrom,BandTo,BlockFrom,BlockTo: TRect;
begin
TBM:=TBitmap.Create;
try
TBM.Height:=abs(Delta);
TBM.Width:=BM.Width;

BandFrom:=Rect(0,0,TBM.Width,abs(Delta));
BandTo:=BandFrom;
OffsetRect(BandTo,0,BM.Height-abs(Delta));

BlockFrom:=Rect(0,abs(Delta),BM.Width,BM.Height);
BlockTo:=BlockFrom;
OffsetRect(BlockTo,0,-abs(Delta));
if Delta>=0 then
	begin
	SwapRectangles(BandFrom,BandTo);
	SwapRectangles(BlockFrom,BlockTo);
	end;
TBM.Canvas.CopyRect(Rect(0,0,TBM.Width,TBM.Height),BM.Canvas,BandFrom);
BM.Canvas.CopyRect(BlockTo,BM.Canvas,BlockFrom);
BM.Canvas.CopyRect(BandTo,TBM.Canvas,Rect(0,0,TBM.Width,TBM.Height));
finally TBM.Free; end;
end;


procedure WrapImage(BM: TBitmap; X,Y: integer);
{Wrap bitmap in the X,Y axis according to the Delta}
begin
WrapImageHorizontal(BM,X);
WrapImageVertical(BM,Y);
end;


var OldAngle,RemX,RemY: double;	{Remainder of angled wrap}

procedure WrapImageAngle(BM: TBitmap; Angle,Len: double);
{Wrap bitmap along specified angle}
var SinTheta,CosTheta: extended;
var X,Y: double;
begin
Angle:=((360 -Angle)+180) * Degree2Radian;
if Angle<>OldAngle then
	begin
	RemX:=0; RemY:=0;
	OldAngle:=Angle;
	end;
SinCos(Angle,SinTheta,CosTheta);
X:=SinTheta * Len + RemX;
Y:=CosTheta * Len + RemY;
WrapImage(BM, trunc(X),Trunc(Y));
RemX:=Frac(X); RemY:=Frac(Y);
end;



function InterpolateColors(C1,C2: TColor; Width, Pos: double): TColor;
{Find the interpolated color between C1 and C2 at Pos}
var CV1,CV2,Step,R: T3DVector;
begin
Result:=C1;
if (Pos<0) or (Pos>Width) then exit;
CV1:=ColorToVector(C1);
CV2:=ColorToVector(C2);
Step:=ScalarDivide3D(VectorSubtract3D(CV2,CV1),Width);
R:=VectorAdd3D(ScalarProduct3D(Step,Pos),CV1);
Result:=VectorToColor(R);
end;



procedure DrawGradientLine(Canvas: TCanvas; P1,P2: TPoint; C1,C2: TColor);
{Draw gradient line between P1 and P2 with colors between C1 and C2}
var Delta: TPoint;			{ Delta X, Y }
var Increment: TPoint;			{ Plotting Increments }
var DoDiagonal: integer;		{ Decision Variable (move Straight Or Diagonally) }
var StraightInc, DiagonalInc: integer;	{ East, Northeast (straight Or Diagonal) }
var StepAlongY: boolean;		{ Flag: Step Along Y Direction }
var I: Integer;
begin
{Calculate deltas}
Delta:=IntVectorSubtract(P2,P1);

{ If Positive Then Increment Else Absolute Value And Decrement }
if Delta.X >= 0 then Increment.X:=1 else begin Delta.X:= -Delta.X; Increment.X:=-1; end;
if Delta.Y >= 0 then Increment.Y:=1 else begin Delta.Y:= -Delta.Y; Increment.Y:=-1; end;

{Work out step direction}
StepAlongY:=Delta.X < Delta.Y;
if StepAlongY then IntSwapComponents(Delta);

{ Set Up Decision Variable Stuff }
DoDiagonal:=2 * Delta.Y - Delta.X;
StraightInc:=2 * Delta.Y;
DiagonalInc:=2 * (Delta.Y-Delta.X);

{ Plot Initial Point }
Canvas.Pixels[P1.X, P1.Y]:=C1;
for I:= 1 to  Delta.X do
	begin
	if DoDiagonal < 0 then
		begin
		{ Move Straight }
		DoDiagonal:= DoDiagonal + StraightInc;
		if StepAlongY then P1.Y:= P1.Y + Increment.Y else P1.X:= P1.X + Increment.X;
		end
	 else
		begin
		{ Move Diagonally }
		DoDiagonal:= DoDiagonal + DiagonalInc;
		P1:=IntVectorAdd(P1,Increment);
		end;
	Canvas.Pixels[P1.X, P1.Y]:=InterpolateColors(C1,C2,Delta.X,I);
	end;
end;	{ Drawline }



procedure HorizGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor) ;
var X: integer;
var DR,DG,DB,DX:  extended;
var R1,R2,G1,G2,B1,B2: byte;
var R,G,B: Byte;
var Cnt: integer;
begin
ColorToRGBBytes(FromColor,R1,G1,B1);
ColorToRGBBytes(ToColor,R2,G2,B2);
DX:=Rect.Right-Rect.Left;
DR:=(R2-R1) / DX;
DG:=(G2-G1) / DX;
DB:=(B2-B1) / DX;

cnt:=0;
for X:=Rect.Left to Rect.Right-1 do
	begin
	R:=R1+Ceil(DR*Cnt);
	G:=G1+Ceil(DG*Cnt);
	B:=B1+Ceil(DB*cnt);

	Canvas.Pen.Color:=RGB(R,G,B);
	Canvas.MoveTo(X,Rect.Top);
	Canvas.LineTo(X,Rect.Bottom);
	inc(Cnt);
	end;
end;


procedure VertGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor);
var Y: integer;
var DR,DG,DB,DY: extended;
var R1,R2,G1,G2,B1,B2: byte;
var R,G,B: Byte;
var Cnt: Integer;
begin
ColorToRGBBytes(FromColor,R1,G1,B1);
ColorToRGBBytes(ToColor,R2,G2,B2);
DY:=Rect.Bottom-Rect.Top;
DR:=(R2-R1) / DY;
DG:=(G2-G1) / DY;
DB:=(B2-B1) / DY;

Cnt:=0;
for Y:=Rect.Top to Rect.Bottom-1 do
	begin
	R:=R1+Ceil(DR*Cnt);
	G:=G1+Ceil(DG*Cnt);
	B:=B1+Ceil(DB*Cnt);

	Canvas.Pen.Color:=RGB(R,G,B);
	Canvas.MoveTo(Rect.Left,Y);
	Canvas.LineTo(Rect.Right,Y);
	Inc(Cnt);
	end;
end;



{-------------------- Routines to handle cursor loading ----------------------}




function IconToBitmap(Icon: TIcon): TBitmap;
begin
Result:=TBitmap.Create;
Result.Width:=Icon.Width;
Result.Height:=ICon.Height;
Result.Canvas.Draw(0,0,Icon);
end;



function LoadCursorToIcon(Filename: string): TIcon;
{Load and convert ".CUR" file to TIcon}
begin
Result:=TIcon.Create;
Result.Handle:=LoadImage(0,PChar(FileName),IMAGE_CURSOR,0,0,LR_DEFAULTSIZE or LR_LOADFROMFILE);
end;




function LoadCursorToBitmap(Filename: string): TBitmap;
{Load and convert ".CUR" file to TBitmap}
{Note: transparent portion of Cursor become opaque}
var Icon: TIcon;
begin
Result:=TBitmap.Create;
Icon:=LoadCursorToIcon(Filename);
try
Icon.Transparent;
Result.Width:=Icon.Width;
Result.Height:=Icon.Height;
Result.Canvas.Draw(0,0,Icon);
finally Icon.Free; end;
end;



procedure ClearBitmap(BM: TBitmap; Block: TRect; Color: TColor);
begin
BM.Canvas.Brush.Color:=Color;
BM.Canvas.Brush.Style:=bsSolid;
BM.Canvas.Pen.Mode:=pmCopy;
BM.Canvas.Pen.Style:=psSolid;
BM.Canvas.Pen.Color:=Color;
BM.Canvas.Rectangle(Block);
end;


procedure ClearBitmap(BM: TBitmap; Color: TColor);
begin
ClearBitmap(BM,Rect(0,0,BM.Width,BM.Height),Color);
end;



{------------------ TImage Support routines -----------------------------------}


procedure NormalizeImageSize(Image: TImage);
{Force Images display size to be the same as the underlying bitmap}
var OldAuto: boolean;
begin
OldAuto:=Image.AutoSize;
Image.Stretch:=False;
Image.AutoSize:=True;
Image.Width:=Image.Picture.Bitmap.Width;
Image.Height:=Image.Picture.Bitmap.Height;
Image.Invalidate;
Image.AutoSize:=OldAuto;
end;


procedure ClearCanvas(Canvas: TCanvas; Rect: TRect; Color: TColor);
begin
Canvas.Brush.Color:=Color;
Canvas.Brush.Style:=bsSolid;
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=Color;
Canvas.Rectangle(Rect);
end;



procedure ClearImage(Image: TImage; Color: TColor);
begin
ClearCanvas(Image.Canvas,Image.ClientRect,Color);
end;



function GetPixelFormatStr(BM: TBitmap): string;
begin
case BM.PixelFormat of
 pfDevice: Result:= 'Device';
 pf1bit: Result:= '1-bit';
 pf4bit: Result:= '4-bit';
 pf8bit: Result:= '8-bit';
 pf15bit: Result:= '15-bit';
 pf16bit: Result:= '16-bit';
 pf24bit: Result:= '24-bit';
 pf32bit: Result:= '32-bit';
 pfCustom: Result:= 'Custom';
 else Result:= 'Unknown';
 end;
end;








procedure FadeInBitmap(Image: TImage; BM: TBitmap; Millisec: integer);
{Fade in specified Bitmap using Alpha blending}
var I: integer;
var Step: integer;
var Alpha: double;
var A: double;

	function GetCurve(X: double): double;
	begin
	Result:=X/(1+A*(1-X));
	end;

begin
Step:=Millisec div 255;
A:=50;
for I:=0 to 255 do
	begin
	Alpha:=GetCurve(I/255);
	AlphaBlendBitmaps(Image.Canvas,Image.ClientRect,BM,round(Alpha*255));
	Image.Invalidate;
	DelayProcessMessages(Step);
	end;
end;



procedure FadeToColor(Image: TImage; C: TColor; Millisec: integer);
{Fade to specified Color using Alpha blending}
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.Width:=Image.Width;
BM.Height:=Image.Height;
ClearBitmap(BM,C);
FadeInBitmap(Image,BM,Millisec);
finally BM.Free; end;
end;



function CreateDisabledBitmap(Glyph: TBitmap): TBitmap;
{Created grayed out disabled bitmap from ordinary image}
const ROP_DSPDxax = $00E20746;
var MonoBmp: TBitmap;
var Col: TColor;
var tr, tg, tb: Byte;
var P, P1, P2, P3: PRGBTripleArray;
var R: PRGBTriple;
var C: array[0..3] of PRGBTriple;
var x, y, i: Integer;
var IsEdge: Boolean;
begin
Result:=TBitmap.Create;
if Assigned(Glyph) then
	begin
	Glyph.PixelFormat := pf24bit;
	Result.Assign(Glyph);
	Result.PixelFormat := pf24bit;
	Result.Canvas.Brush.Color := Glyph.TransparentColor;
	Result.Canvas.FillRect(Rect(0, 0, Result.Width, Result.Height));

	Col := ColorToRGB(Glyph.TransparentColor);
	tr := GetRValue(Col);
	tg := GetGValue(Col);
	tb := GetBValue(Col);
	for y := 0 to Result.Height - 1 do
		begin
		P := Result.ScanLine[y];
		P1 := Glyph.ScanLine[Max(y - 1, 0)];
		P2 := Glyph.ScanLine[y];
		P3 := Glyph.ScanLine[Min(y + 1, Result.Height - 1)];
		for x := 0 to Result.Width - 1 do
			begin
			R := @P2[x];
			if (R.rgbtRed <> tr) or (R.rgbtGreen <> tg) or (R.rgbtBlue <> tb) then
				begin
				IsEdge := (R.rgbtRed + R.rgbtGreen + R.rgbtBlue) < 3 * $80;
				if not IsEdge then
					begin
					C[0] := @P1[x];
					C[1] := @P3[x];
					C[2] := @P2[Max(x - 1, 0)];
					C[3] := @P2[Min(x + 1, Result.Width - 1)];
					for i := 0 to 3 do
					 if (C[i].rgbtRed = tr) and (C[i].rgbtGreen = tg) and (C[i].rgbtBlue = tb) then
					 	begin
					 	IsEdge := True;
					 	Break;
					 	end;
					 end;
				if IsEdge then
					begin
					R := @P[x];
					R.rgbtBlue := 0;
					R.rgbtGreen := 0;
					R.rgbtRed := 0;
					end
				end;
			end;
		end;
	MonoBmp := TBitmap.Create;
	try
	with MonoBmp do
	 begin
	 Assign(Result);
	 HandleType := bmDDB;
	 Canvas.Brush.Color := clBlack;
	 Width := Result.Width;
	 if Monochrome then
	 	begin
	 	Canvas.Font.Color := clWhite;
	 	Monochrome := False;
	 	Canvas.Brush.Color := clWhite;
	 	end;
	 Monochrome := True;
	 end;
with Result.Canvas do
	begin
	Brush.Color := clBtnFace;
	FillRect(Rect(0, 0, Result.Width, Result.Height));
	Brush.Color := clBtnHighlight;
	SetTextColor(Handle, clBlack);
	SetBkColor(Handle, clWhite);
	BitBlt(Handle, 1, 1, Result.Width, Result.Height,
	MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
	Brush.Color := clBtnShadow;
	SetTextColor(Handle, clBlack);
	SetBkColor(Handle, clWhite);
	BitBlt(Handle, 0, 0, Result.Width, Result.Height, MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
	end;
 finally MonoBmp.Free; end;
 end;
end;





procedure BitmapToGrayscaleBuffer(BM: TBitmap; AllocMem: boolean; var Buffer: PByteArray);
{Convert Bitmap to grayscale and put in byte buffer}
var X: Integer;
var Y: Integer;
var Gray: Byte;
var Scan: ^TRGBTriple;
begin
if AllocMem then GetMem(Buffer,BM.Width*BM.Height);
BM.PixelFormat:=pf24Bit;
for Y := 0 to BM.Height - 1 do
	begin
	Scan:= BM.ScanLine[Y];
	for X := 0 to BM.Width - 1 do
		begin
		Buffer^[Y*BM.Width+X]:=Round((0.299 * Scan.rgbtRed) + (0.587 * Scan.rgbtGreen) + (0.114 * Scan.rgbtBlue));
		Inc(Scan);
		end;
	end;
end;



end.
