unit WINTRUST;
{**************************************************************************}
{                                                                          }
{  Microsoft Windows                                                       }
{                                                                          }
{  Copyright (C) Microsoft Corporation,                                    }
{                                                                          }
{  Translation of File:       wintrust.h                                   }
{                                                                          }
{  Contents:   Microsoft Internet Security Trust Provider Model            }
{                                                                          }
{                                                                          }
{**************************************************************************}

interface

uses Windows,WinCrypt;


type PVOID = Pointer;

const ANYSIZE_ARRAY = 1;

{ Trust subject data type }
type WIN_TRUST_SUBJECT = PVOID;

{ Trust registry keys }

const WINTRUST_CONFIG_REGPATH = 'Software\Microsoft\Cryptography\Wintrust\Config';
const WINTRUST_MAX_HEADER_BYTES_TO_MAP_DEFAULT = $00A00000;
const WINTRUST_MAX_HASH_BYTES_TO_MAP_VALUE_NAME = 'MaxHashBytesToMap';
const WINTRUST_MAX_HASH_BYTES_TO_MAP_DEFAULT = $00100000;

{ Trust data UI choices }
const WTD_UI_ALL = 1;
const WTD_UI_NONE = 2;
const WTD_UI_NOBAD = 3;
const WTD_UI_NOGOOD = 4;

{ Trust data revocation check flags }
const WTD_REVOKE_NONE = $00000000;
const WTD_REVOKE_WHOLECHAIN = $00000001;

{ Trust data union choices }
const WTD_CHOICE_FILE = 1;
const WTD_CHOICE_CATALOG = 2;
const WTD_CHOICE_BLOB = 3;
const WTD_CHOICE_SIGNER = 4;
const WTD_CHOICE_CERT = 5;

{ Trust data state actions }
const WTD_STATEACTION_IGNORE = $00000000;
const WTD_STATEACTION_VERIFY = $00000001;
const WTD_STATEACTION_CLOSE = $00000002;
const WTD_STATEACTION_AUTO_CACHE = $00000003;
const WTD_STATEACTION_AUTO_CACHE_FLUSH = $00000004;

{ Trust data provider flags }
const WTD_PROV_FLAGS_MASK = $0000FFFF;
const WTD_USE_IE4_TRUST_FLAG = $00000001;
const WTD_NO_IE4_CHAIN_FLAG = $00000002;
const WTD_NO_POLICY_USAGE_FLAG = $00000004;
const WTD_REVOCATION_CHECK_NONE = $00000010;
const WTD_REVOCATION_CHECK_END_CERT = $00000020;
const WTD_REVOCATION_CHECK_CHAIN = $00000040;
const WTD_REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT = $00000080;
const WTD_SAFER_FLAG = $00000100;
const WTD_HASH_ONLY_FLAG = $00000200;
const WTD_USE_DEFAULT_OSVER_CHECK = $00000400;
const WTD_LIFETIME_SIGNING_FLAG = $00000800;
const WTD_CACHE_ONLY_URL_RETRIEVAL = $00001000;

{ Trust data UI contexts }
const WTD_UICONTEXT_EXECUTE = 0;
const WTD_UICONTEXT_INSTALL = 1;

{ Trust certificate information flags }
const WTCI_DONT_OPEN_STORES = $00000001;
const WTCI_OPEN_ONLY_ROOT = $00000002;
const WTCI_USE_LOCAL_MACHINE = $00000004;

{ Trust policy flags }
const WTPF_TRUSTTEST = $00000020;
const WTPF_TESTCANBEVALID = $00000080;
const WTPF_IGNOREEXPIRATION = $00000100;
const WTPF_IGNOREREVOKATION = $00000200;
const WTPF_OFFLINEOK_IND = $00000400;
const WTPF_OFFLINEOK_COM = $00000800;
const WTPF_OFFLINEOKNBU_IND = $00001000;
const WTPF_OFFLINEOKNBU_COM = $00002000;
const WTPF_VERIFY_V1_OFF = $00010000;
const WTPF_IGNOREREVOCATIONONTS = $00020000;
const WTPF_ALLOWONLYPERTRUST = $00040000;

{ Trust errors }
const TRUSTERROR_STEP_WVTPARAMS = 0;
const TRUSTERROR_STEP_FILEIO = 2;
const TRUSTERROR_STEP_SIP = 3;
const TRUSTERROR_STEP_SIPSUBJINFO = 5;
const TRUSTERROR_STEP_CATALOGFILE = 6;
const TRUSTERROR_STEP_CERTSTORE = 7;
const TRUSTERROR_STEP_MESSAGE = 8;
const TRUSTERROR_STEP_MSG_SIGNERCOUNT = 9;
const TRUSTERROR_STEP_MSG_INNERCNTTYPE = 10;
const TRUSTERROR_STEP_MSG_INNERCNT = 11;
const TRUSTERROR_STEP_MSG_STORE = 12;
const TRUSTERROR_STEP_MSG_SIGNERINFO = 13;
const TRUSTERROR_STEP_MSG_SIGNERCERT = 14;
const TRUSTERROR_STEP_MSG_CERTCHAIN = 15;
const TRUSTERROR_STEP_MSG_COUNTERSIGINFO = 16;
const TRUSTERROR_STEP_MSG_COUNTERSIGCERT = 17;
const TRUSTERROR_STEP_VERIFY_MSGHASH = 18;
const TRUSTERROR_STEP_VERIFY_MSGINDIRECTDATA = 19;
const TRUSTERROR_STEP_FINAL_WVTINIT = 30;
const TRUSTERROR_STEP_FINAL_INITPROV = 31;
const TRUSTERROR_STEP_FINAL_OBJPROV = 32;
const TRUSTERROR_STEP_FINAL_SIGPROV = 33;
const TRUSTERROR_STEP_FINAL_CERTPROV = 34;
const TRUSTERROR_STEP_FINAL_CERTCHKPROV = 35;
const TRUSTERROR_STEP_FINAL_POLICYPROV = 36;
const TRUSTERROR_STEP_FINAL_UIPROV = 37;
const TRUSTERROR_MAX_STEPS = 38;


//#define WVT_OFFSETOF(t,f)   ((ULONG)((ULONG_PTR)(&((t*)0)->f)))

// WOB #1251526 -- macro must check whether _all_ bytes of the member
// lie within the struct size specified
//#define WVT_ISINSTRUCT(structtypedef, structpassedsize, member) \
//  ((WVT_OFFSETOF(structtypedef, member) + sizeof(((structtypedef *) 0)->member) <= structpassedsize) ? TRUE : FALSE)


//#define WVT_IS_CBSTRUCT_GT_MEMBEROFFSET(structtypedef, structpassedsize, member) \
//                    WVT_ISINSTRUCT(structtypedef, structpassedsize, member)


{ Cryptographic provider data subject choices }
const CPD_CHOICE_SIP = 1;

{ Cryptographic provider data provider flags }
const CPD_USE_NT5_CHAIN_FLAG = $80000000;
const CPD_REVOCATION_CHECK_NONE = $00010000;
const CPD_REVOCATION_CHECK_END_CERT = $00020000;
const CPD_REVOCATION_CHECK_CHAIN = $00040000;
const CPD_REVOCATION_CHECK_CHAIN_EXCLUDE_ROOT = $00080000;

{ Cryptographic provider data UI state flags }
const CPD_UISTATE_MODE_PROMPT = $00000000;
const CPD_UISTATE_MODE_BLOCK = $00000001;
const CPD_UISTATE_MODE_ALLOW = $00000002;
const CPD_UISTATE_MODE_MASK = $00000003;

{ Signer types }
const SGNR_TYPE_TIMESTAMP = $00000010;

{ Certificate confidence flags }
const CERT_CONFIDENCE_SIG = $10000000;
const CERT_CONFIDENCE_TIME = $01000000;
const CERT_CONFIDENCE_TIMENEST = $00100000;
const CERT_CONFIDENCE_AUTHIDEXT = $00010000;
const CERT_CONFIDENCE_HYGIENE = $00001000;
const CERT_CONFIDENCE_HIGHEST = $11111000;

{ Current trust version }
const WT_CURRENT_VERSION = $00000200;

{ Trust provider names }
const WT_PROVIDER_DLL_NAME = 'WINTRUST.DLL';
const WT_PROVIDER_CERTTRUST_FUNCTION = 'WintrustCertificateTrust';

{ WintrustAddActionID() flags }
const WT_ADD_ACTION_ID_RET_RESULT_FLAG = $00000001;

{ WintrustGetDefaultForUsage() actions }
const DWACTION_ALLOCANDFILL = 1;
const DWACTION_FREE = 2;

{ Trust object identifiers }
const szOID_TRUSTED_CODESIGNING_CA_LIST = '1.3.6.1.4.1.311.2.2.1';
const szOID_TRUSTED_CLIENT_AUTH_CA_LIST = '1.3.6.1.4.1.311.2.2.2';
const szOID_TRUSTED_SERVER_AUTH_CA_LIST = '1.3.6.1.4.1.311.2.2.3';

{ Special object identifiers }
const SPC_COMMON_NAME_OBJID = szOID_COMMON_NAME;
const SPC_TIME_STAMP_REQUEST_OBJID = '1.3.6.1.4.1.311.3.2.1';
const SPC_INDIRECT_DATA_OBJID = '1.3.6.1.4.1.311.2.1.4';
const SPC_SP_AGENCY_INFO_OBJID = '1.3.6.1.4.1.311.2.1.10';
const SPC_STATEMENT_TYPE_OBJID = '1.3.6.1.4.1.311.2.1.11';
const SPC_SP_OPUS_INFO_OBJID = '1.3.6.1.4.1.311.2.1.12';
const SPC_CERT_EXTENSIONS_OBJID = '1.3.6.1.4.1.311.2.1.14';
const SPC_PE_IMAGE_DATA_OBJID = '1.3.6.1.4.1.311.2.1.15';
const SPC_RAW_FILE_DATA_OBJID = '1.3.6.1.4.1.311.2.1.18';
const SPC_STRUCTURED_STORAGE_DATA_OBJID = '1.3.6.1.4.1.311.2.1.19';
const SPC_JAVA_CLASS_DATA_OBJID = '1.3.6.1.4.1.311.2.1.20';
const SPC_INDIVIDUAL_SP_KEY_PURPOSE_OBJID = '1.3.6.1.4.1.311.2.1.21';
const SPC_COMMERCIAL_SP_KEY_PURPOSE_OBJID = '1.3.6.1.4.1.311.2.1.22';
const SPC_CAB_DATA_OBJID = '1.3.6.1.4.1.311.2.1.25';
const SPC_GLUE_RDN_OBJID = '1.3.6.1.4.1.311.2.1.25';
const SPC_MINIMAL_CRITERIA_OBJID = '1.3.6.1.4.1.311.2.1.26';
const SPC_FINANCIAL_CRITERIA_OBJID = '1.3.6.1.4.1.311.2.1.27';
const SPC_LINK_OBJID = '1.3.6.1.4.1.311.2.1.28';
const SPC_SIGINFO_OBJID = '1.3.6.1.4.1.311.2.1.30';
const SPC_PE_IMAGE_PAGE_HASHES_V1_OBJID = '1.3.6.1.4.1.311.2.3.1';
const SPC_PE_IMAGE_PAGE_HASHES_V2_OBJID = '1.3.6.1.4.1.311.2.3.2';

{ Catalog object identifiers }
const CAT_NAMEVALUE_OBJID = '1.3.6.1.4.1.311.12.2.1';
const CAT_MEMBERINFO_OBJID = '1.3.6.1.4.1.311.12.2.2';

{ Special data structure identifiers }
const SPC_SP_AGENCY_INFO_STRUCT: PAnsiChar = PAnsiChar(2000);
const SPC_MINIMAL_CRITERIA_STRUCT: PAnsiChar = PAnsiChar(2001);
const SPC_FINANCIAL_CRITERIA_STRUCT: PAnsiChar = PAnsiChar(2002);
const SPC_INDIRECT_DATA_CONTENT_STRUCT: PAnsiChar = PAnsiChar(2003);
const SPC_PE_IMAGE_DATA_STRUCT: PAnsiChar = PAnsiChar(2004);
const SPC_LINK_STRUCT: PAnsiChar = PAnsiChar(2005);
const SPC_STATEMENT_LINK_STRUCT: PAnsiChar = PAnsiChar(2006);
const SPC_SP_OPUS_INFO_STRUCT: PAnsiChar = PAnsiChar(2007);
const SPC_CAB_DATA_STRUCT: PAnsiChar = PAnsiChar(2008);
const SPC_JAVA_CLASS_DATA_STRUCT: PAnsiChar = PAnsiChar(2009);
const SPC_SIGINFO_STRUCT: PAnsiChar = PAnsiChar(2130);

{ Catalog data structure identifiers }
const CAT_NAMEVALUE_STRUCT: PAnsiChar = PAnsiChar(2221);
const CAT_MEMBERINFO_STRUCT: PAnsiChar = PAnsiChar(2222);

{ Special UUID length }
const SPC_UUID_LENGTH = 16;

{ Special serialized object attributes class identifier }

const SpcSerializedObjectAttributesClassId: TGUID = (D1:$D586B5A6; D2:$A1B4; D3:$6624; D4:($AE,$05,$A2,$17,$DA,$8E,$60,$D6));

{ Special link choices }
const SPC_URL_LINK_CHOICE = 1;
const SPC_MONIKER_LINK_CHOICE = 2;
const SPC_FILE_LINK_CHOICE = 3;

{ Windows certificate revisions }
const WIN_CERT_REVISION_1_0 = $0100;
const WIN_CERT_REVISION_2_0 = $0200;

{ Windows certificate types }
const WIN_CERT_TYPE_X509 = $0001;
const WIN_CERT_TYPE_PKCS_SIGNED_DATA = $0002;
const WIN_CERT_TYPE_TS_STACK_SIGNED = $0004;

{ Trust subject types }
const
  WIN_TRUST_SUBJTYPE_RAW_FILE:     TGUID =  '{959dc450-8d9e-11cf-8736-00aa00a485eb}';
  WIN_TRUST_SUBJTYPE_PE_IMAGE:     TGUID =  '{43c9a1e0-8da0-11cf-8736-00aa00a485eb}';
  WIN_TRUST_SUBJTYPE_JAVA_CLASS:   TGUID =  '{08ad3990-8da1-11cf-8736-00aa00a485eb}';
  WIN_TRUST_SUBJTYPE_CABINET:      TGUID =  '{d17c5374-a392-11cf-9df5-00aa00c184e0}';
  WIN_TRUST_SUBJTYPE_RAW_FILEEX:   TGUID =  '{6F458110-C2F1-11CF-8A69-00AA006C3706}';
  WIN_TRUST_SUBJTYPE_PE_IMAGEEX:   TGUID =  '{6F458111-C2F1-11CF-8A69-00AA006C3706}';
  WIN_TRUST_SUBJTYPE_JAVA_CLASSEX: TGUID =  '{6F458113-C2F1-11CF-8A69-00AA006C3706}';
  WIN_TRUST_SUBJTYPE_CABINETEX:    TGUID =  '{6F458114-C2F1-11CF-8A69-00AA006C3706}';
  WIN_TRUST_SUBJTYPE_OLE_STORAGE:  TGUID =  '{C257E740-8DA0-11CF-8736-00AA00A485EB}';
  WIN_SPUB_ACTION_TRUSTED_PUBLISHER: TGUID = '{66426730-8DA1-11CF-8736-00AA00A485EB}';
  WIN_SPUB_ACTION_NT_ACTIVATE_IMAGE: TGUID = '{8BC96B00-8DA1-11CF-8736-00AA00A485EB}';
  WIN_SPUB_ACTION_PUBLISHED_SOFTWARE:TGUID = '{64B9D180-8DA2-11CF-8736-00AA00A485EB}';


{ OpenPersonalTrustDBDialogEx() flags }
const WT_TRUSTDBDIALOG_NO_UI_FLAG = $00000001;
const WT_TRUSTDBDIALOG_ONLY_PUB_TAB_FLAG = $00000002;
const WT_TRUSTDBDIALOG_WRITE_LEGACY_REG_FLAG = $00000100;
const WT_TRUSTDBDIALOG_WRITE_IEAK_STORE_FLAG = $00000200;

const WTD_UI_CONTEXT_EXECUTE = 0;
const WTD_UI_CONTEXT_INSTALL = 1;



{ Trust file information }
type
  PWintrustFileInfo = ^TWintrustFileInfo;
  PWINTRUST_FILE_INFO = ^WINTRUST_FILE_INFO_;
  WINTRUST_FILE_INFO_ = packed record
    cbStruct: DWORD;
    pcwszFilePath: LPCWSTR;
    hFile: THandle;
    pgKnownSubject: PGUID;
  end;
  WINTRUST_FILE_INFO = WINTRUST_FILE_INFO_;
  TWintrustFileInfo = WINTRUST_FILE_INFO_;


{ Trust catalog information }
type
  PWintrustCatalogInfo = ^TWintrustCatalogInfo;
  PWINTRUST_CATALOG_INFO = ^WINTRUST_CATALOG_INFO_;
  WINTRUST_CATALOG_INFO_ = packed record
    cbStruct: DWORD;
    dwCatalogVersion: DWORD;
    pcwszCatalogFilePath: LPCWSTR;
    pcwszMemberTag: LPCWSTR;
    pcwszMemberFilePath: LPCWSTR;
    hMemberFile: THandle;
    pbCalculatedFileHash: PByte;
    cbCalculatedFileHash: DWORD;
    pcCatalogContext: PCCTL_CONTEXT;
  end;
  WINTRUST_CATALOG_INFO = WINTRUST_CATALOG_INFO_;
  TWintrustCatalogInfo = WINTRUST_CATALOG_INFO_;

{ Trust blob information }
type
  PWintrustBlobInfo = ^TWintrustBlobInfo;
  PWINTRUST_BLOB_INFO = ^WINTRUST_BLOB_INFO_;
  WINTRUST_BLOB_INFO_ = packed record
    cbStruct: DWORD;
    gSubject: TGUID;
    pcwszDisplayName: LPCWSTR;
    cbMemObject: DWORD;
    pbMemObject: PByte;
    cbMemSignedMsg: DWORD;
    pbMemSignedMsg: PByte;
  end;
  WINTRUST_BLOB_INFO = WINTRUST_BLOB_INFO_;
  TWintrustBlobInfo = WINTRUST_BLOB_INFO_;

{ Trust signer information }
type
  PWintrustSgnrInfo = ^TWintrustSgnrInfo;
  PWINTRUST_SGNR_INFO = ^WINTRUST_SGNR_INFO_;
  WINTRUST_SGNR_INFO_ = packed record
    cbStruct: DWORD;
    pcwszDisplayName: LPCWSTR;
    psSignerInfo: PCMSG_SIGNER_INFO;
    chStores: DWORD;
    pahStores: PHCERTSTORE;
  end;
  WINTRUST_SGNR_INFO = WINTRUST_SGNR_INFO_;
  TWintrustSgnrInfo = WINTRUST_SGNR_INFO_;

{ Trust certificate information }
type
  PWintrustCertInfo = ^TWintrustCertInfo;
  PWINTRUST_CERT_INFO = ^WINTRUST_CERT_INFO_;
  WINTRUST_CERT_INFO_ = packed record
    cbStruct: DWORD;
    pcwszDisplayName: LPCWSTR;
    psCertContext: PCERT_CONTEXT;
    chStores: DWORD;
    pahStores: PHCERTSTORE;
    dwFlags: DWORD;
    psftVerifyAsOf: PFILETIME;
  end;
  WINTRUST_CERT_INFO = WINTRUST_CERT_INFO_;
  TWintrustCertInfo = WINTRUST_CERT_INFO_;


{ Trust data }
type
  PWintrustData = ^TWintrustData;
  PWINTRUST_DATA = ^WINTRUST_DATA;
  _WINTRUST_DATA = packed record
    cbStruct: DWORD;
    pPolicyCallbackData: Pointer;
    pSIPClientData: Pointer;
    dwUIChoice: DWORD;
    fdwRevocationChecks: DWORD;
    dwUnionChoice: DWORD;
    InfoUnion: record
      case Integer of
        0: (pFile: PWintrustFileInfo);
        1: (pCatalog: PWintrustCatalogInfo);
        2: (pBlob: PWintrustBlobInfo);
        3: (pSgnr: PWintrustSgnrInfo);
        4: (pCert: PWintrustCertInfo);
    end;
    dwStateAction: DWORD;
    hWVTStateData: THandle;
    pwszUrlReference: LPCWSTR;
    dwProvFlags: DWORD;
    dwUIContext: DWORD;
  end;
  WINTRUST_DATA = _WINTRUST_DATA;
  TWintrustData = _WINTRUST_DATA;



{ Cryptographic provider data }
type

  PCRYPT_PROVIDER_PRIVDATA = ^CRYPT_PROVIDER_PRIVDATA;
  PCRYPT_PROVIDER_FUNCTIONS = ^CRYPT_PROVIDER_FUNCTIONS;
  PCRYPT_PROVIDER_SGNR = ^CRYPT_PROVIDER_SGNR;
  PPROVDATA_SIP = ^PROVDATA_SIP;
  PCRYPT_PROVUI_FUNCS = ^CRYPT_PROVUI_FUNCS;
  PCRYPT_PROVUI_DATA = ^CRYPT_PROVUI_DATA;
  PCRYPT_PROVIDER_DEFUSAGE = ^CRYPT_PROVIDER_DEFUSAGE;


  PCryptProviderData = ^TCryptProviderData;
  PCRYPT_PROVIDER_DATA = ^CRYPT_PROVIDER_DATA;
  _CRYPT_PROVIDER_DATA = packed record
    cbStruct: DWORD;
    pWintrustData: PWINTRUST_DATA;
    fOpenedFile: Bool;
    hWndParent: HWND;
    pgActionID: PGUID;
    hProv: HCRYPTPROV;
    dwError: DWORD;
    dwRegSecuritySettings: DWORD;
    dwRegPolicySettings: DWORD;
    psPfns: PCRYPT_PROVIDER_FUNCTIONS;
    cdwTrustStepErrors: DWORD;
    padwTrustStepErrors: PDWORD;
    chStores: DWORD;
    pahStores: PHCERTSTORE;
    dwEncoding: DWORD;
    hMsg: HCRYPTMSG;
    csSigners: DWORD;
    pasSigners: PCRYPT_PROVIDER_SGNR;
    csProvPrivData: DWORD;
    pasProvPrivData: PCRYPT_PROVIDER_PRIVDATA;
    dwSubjectChoice: DWORD;
    pPDSIP: PPROVDATA_SIP; //in C a union with one member
    pszUsageOID: PAnsiChar;
    fRecallWithState: BOOL;
    sftSystemTime: FILETIME;
    pszCTLSignerUsageOID: PAnsiChar;
    dwProvFlags: DWORD;
    dwFinalError: DWORD;
    pRequestUsage: PCERT_USAGE_MATCH;
    dwTrustPubSettings: DWORD;
    dwUIStateFlags: DWORD;
  end;
  CRYPT_PROVIDER_DATA = _CRYPT_PROVIDER_DATA;
  TCryptProviderData = _CRYPT_PROVIDER_DATA;

{ Cryptographic provider private data }
  _CRYPT_PROVIDER_PRIVDATA = packed record
    cbStruct: DWORD;
    gProviderID: GUID;
    cbProvData: DWORD;
    pvProvData: Pointer;
  end;
  CRYPT_PROVIDER_PRIVDATA = _CRYPT_PROVIDER_PRIVDATA;

{ Provider data SIP }

  _PROVDATA_SIP = packed record
    cbStruct: DWORD;
    gSubject: GUID;
    { These values are defined in "mssip.h" but I don't currently
    { have a pascal version - so generic pointer versions used here }
    // pSip: PSIP_DISPATCH_INFO_;
    // pCATSip: PSIP_DISPATCH_INFO_;
    // psSipSubjectInfo: PSIP_SUBJECTINFO_;
    // struct SIP_SUBJECTINFO_*psSipCATSubjectInfo: Integer;
    // psIndirectData: PSIP_INDIRECT_DATA_;
    pSip: Pointer;
    pCATSip: Pointer;
    psSipSubjectInfo: Pointer;
    psSipCATSubjectInfo: Pointer;
    psIndirectData: Pointer;
  end;
  PROVDATA_SIP = _PROVDATA_SIP;


{ Cryptographic provider certificate }

  PCRYPT_PROVIDER_CERT = ^CRYPT_PROVIDER_CERT;
  _CRYPT_PROVIDER_CERT = packed record
    cbStruct: DWORD;
    pCert: PCCERT_CONTEXT;
    fCommercial: BOOL;
    fTrustedRoot: BOOL;
    fSelfSigned: BOOL;
    fTestCert: BOOL;
    dwRevokedReason: DWORD;
    dwConfidence: DWORD;
    dwError: DWORD;
    pTrustListContext: PCTL_CONTEXT;
    fTrustListSignerCert: BOOL;
    pCtlContext: PCCTL_CONTEXT;
    dwCtlError: DWORD;
    fIsCyclic: BOOL;
    pChainElement: PCERT_CHAIN_ELEMENT;
  end;
  CRYPT_PROVIDER_CERT = _CRYPT_PROVIDER_CERT;

{ Cryptographic provider signer }

  _CRYPT_PROVIDER_SGNR = packed record
    cbStruct: DWORD;
    sftVerifyAsOf: FILETIME;
    csCertChain: DWORD;
    pasCertChain: PCRYPT_PROVIDER_CERT;
    dwSignerType: DWORD;
    psSigner: PCMSG_SIGNER_INFO;
    dwError: DWORD;
    csCounterSigners: DWORD;
    pasCounterSigners: PCRYPT_PROVIDER_SGNR;
    pChainContext: PCCERT_CHAIN_CONTEXT;
  end;
  CRYPT_PROVIDER_SGNR = _CRYPT_PROVIDER_SGNR;



{ Cryptographic trust registration entry }

  PCRYPT_TRUST_REG_ENTRY = ^CRYPT_TRUST_REG_ENTRY;
  _CRYPT_TRUST_REG_ENTRY = packed record
    cbStruct: DWORD;
    pwszDLLName: PWCHAR;
    pwszFunctionName: PWCHAR;
  end;
  CRYPT_TRUST_REG_ENTRY = _CRYPT_TRUST_REG_ENTRY;


{ Cryptographic register action identifier }

  PCRYPT_REGISTER_ACTIONID = ^CRYPT_REGISTER_ACTIONID;
  _CRYPT_REGISTER_ACTIONID = packed record
    cbStruct: DWORD;
    sInitProvider: CRYPT_TRUST_REG_ENTRY;
    sObjectProvider: CRYPT_TRUST_REG_ENTRY;
    sSignatureProvider: CRYPT_TRUST_REG_ENTRY;
    sCertificateProvider: CRYPT_TRUST_REG_ENTRY;
    sCertificatePolicyProvider: CRYPT_TRUST_REG_ENTRY;
    sFinalPolicyProvider: CRYPT_TRUST_REG_ENTRY;
    sTestPolicyProvider: CRYPT_TRUST_REG_ENTRY;
    sCleanupProvider: CRYPT_TRUST_REG_ENTRY;
  end;
  CRYPT_REGISTER_ACTIONID = _CRYPT_REGISTER_ACTIONID;



{ Cryptographic provider data callbacks }
 PFN_CPD_MEM_ALLOC = function(Arg1: DWORD): pointer;

 PFN_CPD_MEM_FREE = procedure(Arg1: pointer);
 PFN_CPD_ADD_STORE = function(Arg1: PCRYPT_PROVIDER_DATA; Arg2: HCERTSTORE): BOOL;
 PFN_CPD_ADD_SGNR = function(Arg1: PCRYPT_PROVIDER_DATA; Arg2: BOOL; Arg3: DWORD; Arg4: PCRYPT_PROVIDER_SGNR): BOOL;
 PFN_CPD_ADD_CERT = function(Arg1: PCRYPT_PROVIDER_DATA; Arg2: DWORD; Arg3: BOOL; Arg4: DWORD; Arg5: PCCERT_CONTEXT): BOOL;
 PFN_CPD_ADD_PRIVDATA = function(Arg1: PCRYPT_PROVIDER_DATA; Arg2: PCRYPT_PROVIDER_PRIVDATA): BOOL;

{ Provider callbacks }
 PFN_PROVIDER_INIT_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_OBJTRUST_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_SIGTRUST_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_CERTTRUST_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_FINALPOLICY_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_TESTFINALPOLICY_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_CLEANUP_CALL = function(Arg1: PCRYPT_PROVIDER_DATA): HRESULT;
 PFN_PROVIDER_CERTCHKPOLICY_CALL = function(Arg1: PCRYPT_PROVIDER_DATA; Arg3: DWORD; Arg4: BOOL; Arg5: DWORD): BOOL;


{ Cryptographic provider functions }
 _CRYPT_PROVIDER_FUNCTIONS = packed record
    cbStruct: LongInt;
    pfnAlloc: PFN_CPD_MEM_ALLOC;
    pfnFree: PFN_CPD_MEM_FREE;
    pfnAddStore2Chain: PFN_CPD_ADD_STORE;
    pfnAddSgnr2Chain: PFN_CPD_ADD_SGNR;
    pfnAddCert2Chain: PFN_CPD_ADD_CERT;
    pfnAddPrivData2Chain: PFN_CPD_ADD_PRIVDATA;
    pfnInitialize: PFN_PROVIDER_INIT_CALL;
    pfnObjectTrust: PFN_PROVIDER_OBJTRUST_CALL;
    pfnSignatureTrust: PFN_PROVIDER_SIGTRUST_CALL;
    pfnCertificateTrust: PFN_PROVIDER_CERTTRUST_CALL;
    pfnFinalPolicy: PFN_PROVIDER_FINALPOLICY_CALL;
    pfnCertCheckPolicy: PFN_PROVIDER_CERTCHKPOLICY_CALL;
    pfnTestFinalPolicy: PFN_PROVIDER_TESTFINALPOLICY_CALL;
    psUIpfns: PCRYPT_PROVUI_FUNCS;
    pfnCleanupPolicy: PFN_PROVIDER_CLEANUP_CALL;
  end;
  CRYPT_PROVIDER_FUNCTIONS = _CRYPT_PROVIDER_FUNCTIONS;

{ Provider UI callback }
 PFN_PROVUI_CALL = function(Arg1: HWND; Arg2: PCRYPT_PROVIDER_DATA): BOOL;

{ Cryptographic provider UI functions }

 _CRYPT_PROVUI_FUNCS = packed record
    cbStruct: LongInt;
    psUIData: PCRYPT_PROVUI_DATA;
    pfnOnMoreInfoClick: PFN_PROVUI_CALL;
    pfnOnMoreInfoClickDefault: PFN_PROVUI_CALL;
    pfnOnAdvancedClick: PFN_PROVUI_CALL;
    pfnOnAdvancedClickDefault: PFN_PROVUI_CALL;
  end;
  CRYPT_PROVUI_FUNCS = _CRYPT_PROVUI_FUNCS;



{ Cryptographic provider UI data }

  _CRYPT_PROVUI_DATA = packed record
    cbStruct: LongInt;
    dwFinalError: LongInt;
    pYesButtonText: PWCHAR;
    pNoButtonText: PWCHAR;
    pMoreInfoButtonText: PWCHAR;
    pAdvancedLinkText: PWCHAR;
    pCopyActionText: PWCHAR;
    pCopyActionTextNoTS: PWCHAR;
    pCopyActionTextNotSigned: PWCHAR;
  end;
  CRYPT_PROVUI_DATA = _CRYPT_PROVUI_DATA;


{ Default usage callbacks }
 PFN_ALLOCANDFILLDEFUSAGE = function(const Arg1: Pchar; Arg2: PCRYPT_PROVIDER_DEFUSAGE): BOOL;
 PFN_FREEDEFUSAGE = function(const Arg1: PChar; Arg2: PCRYPT_PROVIDER_DEFUSAGE): BOOL;

{ Cryptographic provider registration default usage }

  PCRYPT_PROVIDER_REGDEFUSAGE = ^CRYPT_PROVIDER_REGDEFUSAGE;
  _CRYPT_PROVIDER_REGDEFUSAGE = packed record
    cbStruct: DWORD;
    pgActionID: PGUID;
    pwszDllName: PWCHAR;
    pwszLoadCallbackDataFunctionName: PChar;
    pwszFreeCallbackDataFunctionName: PChar;
  end;
  CRYPT_PROVIDER_REGDEFUSAGE = _CRYPT_PROVIDER_REGDEFUSAGE;

{ Cryptographic provider default usage }

  _CRYPT_PROVIDER_DEFUSAGE = packed record
    cbStruct: LongInt;
    gActionID: GUID;
    pDefPolicyCallbackData: PVOID;
    pDefSIPClientData: PVOID;
  end;
  CRYPT_PROVIDER_DEFUSAGE = _CRYPT_PROVIDER_DEFUSAGE;

{ Special UUID data type }
type SPC_UUID = array [0..SPC_UUID_LENGTH-1] of Byte;

{ Special serialized object }
type
  PSPC_SERIALIZED_OBJECT = ^SPC_SERIALIZED_OBJECT;
  _SPC_SERIALIZED_OBJECT = packed record
    ClassId: SPC_UUID;
    SerializedData: CRYPT_DATA_BLOB;
  end;
  SPC_SERIALIZED_OBJECT = _SPC_SERIALIZED_OBJECT;


{ Special signature information }
type
  PSPC_SIGINFO = ^SPC_SIGINFO;
  SPC_SIGINFO_ = packed record
    dwSipVersion: LongInt;
    gSIPGuid: GUID;
    dwReserved1: LongInt;
    dwReserved2: LongInt;
    dwReserved3: LongInt;
    dwReserved4: LongInt;
    dwReserved5: LongInt;
  end;
  SPC_SIGINFO = SPC_SIGINFO_;


{ Special link }
type
  PSPC_LINK = ^SPC_LINK;
  SPC_LINK_ = packed record
    dwLinkChoice: LongInt;
        pwszUrl: PWideChar;
    Moniker: SPC_SERIALIZED_OBJECT;
    pwszFile: PWideChar;
  end;
  SPC_LINK = SPC_LINK_;
  PSPC_LINK_ = ^SPC_LINK_;


{ Special PE image data }
type
  PSPC_PE_IMAGE_DATA = ^SPC_PE_IMAGE_DATA;
  _SPC_PE_IMAGE_DATA = packed record
    Flags: CRYPT_BIT_BLOB;
    pFile: PSPC_LINK;
  end {_SPC_PE_IMAGE_DATA};
  SPC_PE_IMAGE_DATA = _SPC_PE_IMAGE_DATA;


{ Special indirect data content }
type
  PSPC_INDIRECT_DATA_CONTENT = ^SPC_INDIRECT_DATA_CONTENT;
  _SPC_INDIRECT_DATA_CONTENT = packed record
    Data: CRYPT_ATTRIBUTE_TYPE_VALUE;
    DigestAlgorithm: CRYPT_ALGORITHM_IDENTIFIER;
    Digest: CRYPT_HASH_BLOB;
  end {_SPC_INDIRECT_DATA_CONTENT};
  SPC_INDIRECT_DATA_CONTENT = _SPC_INDIRECT_DATA_CONTENT;


{ Special financial criteria }
type
  PSPC_FINANCIAL_CRITERIA = ^SPC_FINANCIAL_CRITERIA;
  _SPC_FINANCIAL_CRITERIA = packed record
    fFinancialInfoAvailable: Bool;
    fMeetsCriteria: Bool;
  end {_SPC_FINANCIAL_CRITERIA};
  SPC_FINANCIAL_CRITERIA = _SPC_FINANCIAL_CRITERIA;


{ Special image }
type
  PSPC_IMAGE = ^SPC_IMAGE;
  _SPC_IMAGE = packed record
    pImageLink: PSPC_LINK_;
    Bitmap: CRYPT_DATA_BLOB;
    Metafile: CRYPT_DATA_BLOB;
    EnhancedMetafile: CRYPT_DATA_BLOB;
    GifFile: CRYPT_DATA_BLOB;
  end {_SPC_IMAGE};
  SPC_IMAGE = _SPC_IMAGE;



{ Special SP agency information }
type
  PSPC_SP_AGENCY_INFO = ^SPC_SP_AGENCY_INFO;
  _SPC_SP_AGENCY_INFO = packed record
    pPolicyInformation: PSPC_LINK_;
    pwszPolicyDisplayText: PWideChar;
    pLogoImage: PSPC_IMAGE;
    pLogoLink: PSPC_LINK_;
  end {_SPC_SP_AGENCY_INFO};
  SPC_SP_AGENCY_INFO = _SPC_SP_AGENCY_INFO;


{ Special statement type }
type
  PSPC_STATEMENT_TYPE = ^SPC_STATEMENT_TYPE;
  _SPC_STATEMENT_TYPE = packed record
    cKeyPurposeId: LongInt;
    rgpszKeyPurposeId: PPChar;
  end {_SPC_STATEMENT_TYPE};
  SPC_STATEMENT_TYPE = _SPC_STATEMENT_TYPE;


{ Special SP opus information }
  PSPC_SP_OPUS_INFO = ^SPC_SP_OPUS_INFO;
  _SPC_SP_OPUS_INFO = packed record
    pwszProgramName: PWideChar;
    pMoreInfo: PSPC_LINK_;
    pPublisherInfo: PSPC_LINK_;
  end {_SPC_SP_OPUS_INFO};
  SPC_SP_OPUS_INFO = _SPC_SP_OPUS_INFO;



{ Catalog name value }
type
  PCAT_NAMEVALUE = ^CAT_NAMEVALUE;
  _CAT_NAMEVALUE = packed record
    pwszTag: PWideChar;
    fdwFlags: LongInt;
    Value: CRYPT_DATA_BLOB;
  end {_CAT_NAMEVALUE};
  CAT_NAMEVALUE = _CAT_NAMEVALUE;

{ Catalog member information }
type
  PCAT_MEMBERINFO = ^CAT_MEMBERINFO;
  _CAT_MEMBERINFO = packed record
    pwszSubjGuid: PWideChar;
    dwCertVersion: LongInt;
  end {_CAT_MEMBERINFO};
  CAT_MEMBERINFO = _CAT_MEMBERINFO;


{ Windows certificate }
type
  LPWIN_CERTIFICATE = ^WIN_CERTIFICATE;
  _WIN_CERTIFICATE = packed record
    dwLength: LongInt;
    wRevision: Word;
    wCertificateType: Word;
    bCertificate: array [0..ANYSIZE_ARRAY-1] of BYTE;
  end {_WIN_CERTIFICATE};
  WIN_CERTIFICATE = _WIN_CERTIFICATE;



{ Trust action data context with subject }
type
  LPWIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT = ^WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT;
  _WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT = packed record
    hClientToken: THandle;
    SubjectType: PGUID;
    Subject: WIN_TRUST_SUBJECT;
  end {_WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT};
  WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT = _WIN_TRUST_ACTDATA_CONTEXT_WITH_SUBJECT;



{ Trust action data subject only }
type
  LPWIN_TRUST_ACTDATA_SUBJECT_ONLY = ^WIN_TRUST_ACTDATA_SUBJECT_ONLY;
  _WIN_TRUST_ACTDATA_SUBJECT_ONLY = packed record
    SubjectType: PGUID;
    Subject: WIN_TRUST_SUBJECT;
  end {_WIN_TRUST_ACTDATA_SUBJECT_ONLY};
  WIN_TRUST_ACTDATA_SUBJECT_ONLY = _WIN_TRUST_ACTDATA_SUBJECT_ONLY;

{ Trust subject file }
type
  LPWIN_TRUST_SUBJECT_FILE = ^WIN_TRUST_SUBJECT_FILE;
  _WIN_TRUST_SUBJECT_FILE = packed record
    hFile: THandle;
    lpPath: PWideChar;
  end {_WIN_TRUST_SUBJECT_FILE};
  WIN_TRUST_SUBJECT_FILE = _WIN_TRUST_SUBJECT_FILE;


{ Trust subject file and display }
type
  LPWIN_TRUST_SUBJECT_FILE_AND_DISPLAY = ^WIN_TRUST_SUBJECT_FILE_AND_DISPLAY;
  _WIN_TRUST_SUBJECT_FILE_AND_DISPLAY = packed record
    hFile: THandle;
    lpPath: PWideChar;
    lpDisplayName: PWideChar;
  end {_WIN_TRUST_SUBJECT_FILE_AND_DISPLAY};
  WIN_TRUST_SUBJECT_FILE_AND_DISPLAY = _WIN_TRUST_SUBJECT_FILE_AND_DISPLAY;


{ Trusted publisher data }
type
  LPWIN_SPUB_TRUSTED_PUBLISHER_DATA = ^WIN_SPUB_TRUSTED_PUBLISHER_DATA;
  _WIN_SPUB_TRUSTED_PUBLISHER_DATA = packed record
    hClientToken: THandle;
    lpCertificate: LPWIN_CERTIFICATE;
  end {_WIN_SPUB_TRUSTED_PUBLISHER_DATA};
  WIN_SPUB_TRUSTED_PUBLISHER_DATA = _WIN_SPUB_TRUSTED_PUBLISHER_DATA;


{ Functions in WINTRUST.DLL }

function OpenPersonalTrustDBDialog(hwndParent: HWND): BOOL;  stdcall; 
function OpenPersonalTrustDBDialogEx(hwndParent: HWND; dwFlags: DWORD; pvReserved: PVOID): Bool;  stdcall; 

function WTHelperCertCheckValidSignature(pProvData: PCRYPT_PROVIDER_DATA): HRESULT;  stdcall; 
function WTHelperCertIsSelfSigned(dwEncoding: DWORD; pCert: PCERT_INFO): BOOL;  stdcall; 
function WTHelperGetProvSignerFromChain(pProvData: PCRYPT_PROVIDER_DATA; idxSigner: DWORD; fCounterSigner: BOOL; idxCounterSigner: DWORD): PCRYPT_PROVIDER_SGNR; stdcall;
function WTHelperGetProvPrivateDataFromChain(pProvData: PCRYPT_PROVIDER_DATA; const pgProviderId: TGUID): PCRYPT_PROVIDER_PRIVDATA; stdcall;

function WTHelperGetProvCertFromChain(pSgnr: PCRYPT_PROVIDER_SGNR; idxCert: DWORD): PCRYPT_PROVIDER_CERT; stdcall;
function WTHelperProvDataFromStateData(hStateData: THandle): PCRYPT_PROVIDER_DATA; stdcall;
function WinVerifyTrust(hwnd: HWND; pgActionID: PGUID; var pWVTData: _WINTRUST_DATA): LongInt;  stdcall;

function WinVerifyTrustEx(hwnd: HWND; pgActionID: PGUID; pWinTrustData: PWINTRUST_DATA): HRESULT;  stdcall; 
function WintrustAddActionID(pgActionID: PGUID; fdwFlags: DWORD; psProvInfo: PCRYPT_REGISTER_ACTIONID): BOOL;  stdcall; 
function WintrustAddDefaultForUsage(const pszUsageOID: PChar; psDefUsage: PCRYPT_PROVIDER_REGDEFUSAGE): BOOL;  stdcall; 
function WintrustGetDefaultForUsage(dwAction: DWORD; const pszUsageOID: PChar; psUsage: PCRYPT_PROVIDER_DEFUSAGE): BOOL;  stdcall; 
procedure WintrustGetRegPolicyFlags(var pdwPolicyFlags: DWORD); stdcall;
function WintrustLoadFunctionPointers(pgActionID: PGUID; pPfns: PCRYPT_PROVIDER_FUNCTIONS): Bool  stdcall;
function WintrustRemoveActionID(pgActionID: PGUID): Bool  stdcall; 
procedure WintrustSetDefaultIncludePEPageHashes(fIncludePEPageHashes: Bool);  stdcall;
function WintrustSetRegPolicyFlags(dwPolicyFlags: DWORD): Bool  stdcall; 

implementation


function OpenPersonalTrustDBDialog; external 'WINTRUST.DLL';
function OpenPersonalTrustDBDialogEx; external 'WINTRUST.DLL';

function WTHelperCertCheckValidSignature; external 'WINTRUST.DLL';
function WTHelperCertIsSelfSigned; external 'WINTRUST.DLL';
function WTHelperGetProvSignerFromChain; external 'WINTRUST.DLL';
function WTHelperGetProvPrivateDataFromChain; external 'WINTRUST.DLL';

function WTHelperGetProvCertFromChain; external 'WINTRUST.DLL';
function WTHelperProvDataFromStateData; external 'WINTRUST.DLL';
function WinVerifyTrust; external 'WINTRUST.DLL';

function WinVerifyTrustEx; external 'WINTRUST.DLL';
function WintrustAddActionID; external 'WINTRUST.DLL';
function WintrustAddDefaultForUsage; external 'WINTRUST.DLL';
function WintrustGetDefaultForUsage; external 'WINTRUST.DLL';
procedure WintrustGetRegPolicyFlags; external 'WINTRUST.DLL';
function WintrustLoadFunctionPointers; external 'WINTRUST.DLL';
function WintrustRemoveActionID; external 'WINTRUST.DLL';
procedure WintrustSetDefaultIncludePEPageHashes; external 'WINTRUST.DLL';
function WintrustSetRegPolicyFlags; external 'WINTRUST.DLL';

//function WVT_ISINSTRUCT; external 'WINTRUST.DLL';

end.
