unit ClipboardLib;

interface

uses Windows,Graphics;

procedure ClipWriteFormat(Format, Data: string; Clear: boolean);
function ClipReadFormat(Format: string; var Data: string): boolean;

function GetClipboardDescription(Format: integer): string;

implementation

uses Clipbrd;


{--------------- Display Routines ---------------------------------------------}


function GetClipFormatName(Format: integer): string;
{Get clipboard format name for non-standard formats}
var L: integer;
begin
SetLength(Result,255);
L:=GetClipboardFormatName(Format, PChar(Result), 255);
SetLength(Result,L);
end;



function GetClipboardDescription(Format: integer): string;
{Get standard and non-standard format description}
begin
case Format of
 CF_TEXT: Result:='Text';
 CF_BITMAP: Result:='Bitmap';
 CF_METAFILEPICT: Result:='Meta File Pict';
 CF_SYLK: Result:='Sylk';
 CF_DIF: Result:='Dif';
 CF_TIFF: Result:='TIFF';
 CF_OEMTEXT: Result:='Oem Text';
 CF_DIB: Result:='DIB';
 CF_PALETTE: Result:='Palette';
 CF_PENDATA: Result:='Pen Data';
 CF_RIFF: Result:='RIFF';
 CF_WAVE: Result:='Wave';
 CF_UNICODETEXT: Result:='Unicode Text';
 CF_ENHMETAFILE: Result:='Enh Meta File';
 CF_HDROP: Result:='Hdrop';
 CF_LOCALE: Result:='Locale';
 CF_MAX: Result:='Max';
 CF_OWNERDISPLAY: Result:='Owner Display';
 CF_DSPTEXT: Result:='DSP Text';
 CF_DSPBITMAP: Result:='DSP Bitmap';
 CF_DSPMETAFILEPICT: Result:='DSP Meta File Pict';
 CF_DSPENHMETAFILE: Result:='DSP Enh Meta File';
 else if (Format>=CF_PRIVATEFIRST) and (Format<=CF_PRIVATELAST) then Result:='Private'
 else if (Format>=CF_GDIOBJFIRST) and (Format<=CF_GDIOBJLAST) then Result:='GDI Object'
 else Result:=GetClipFormatName(Format);
 end;
end;



{--------------- Read and Write to/from Clipboard -----------------------------}



procedure SetBuffer(Format: Word; var Buffer; Size: Integer);
{Copy specified buffer to clipboard using specified format}
{Does not open clipboard - should be done at a higher}
{ level so multiple format can be saved}
var Data: THandle;
var DataPtr: Pointer;
begin
Data := GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, Size);
try
DataPtr := GlobalLock(Data);
try
Move(Buffer, DataPtr^, Size);
Clipboard.SetAsHandle(Format, Data);
finally GlobalUnlock(Data); end;
except GlobalFree(Data); raise; end;
end;



procedure ClipWriteFormat(Format, Data: string; Clear: boolean);
{Does not open clipboard - should be done at a higher}
{ level so multiple format can be saved}
var ClipFormat: integer;
begin
ClipFormat:=RegisterClipboardFormat(PChar(Format));
if Clear then Clipboard.Clear;
SetBuffer(ClipFormat, PChar(Data)^, Length(Data) + 1);
end;



{------------------------------------------------------------------------------}



function ClipReadFormat(Format: string; var Data: string): boolean;
var Handle: THandle;
var TextPtr: PChar;
var ClipFormat: integer;
begin
Result:=False;
ClipFormat:=RegisterClipboardFormat(PChar(Format));

ClipBoard.Open;
Handle:= Clipboard.GetAsHandle(ClipFormat);
TextPtr:= GlobalLock(Handle);

Data:=TextPtr;
Clipboard.Close;
GlobalUnlock(ClipFormat);
Result:=True;
end;



end.
