{ Assortment of support routines for Delphi programs }

unit DSupport;

interface

uses Windows,Graphics,StdCtrls,SysUtils,Types,IniFiles,Classes,Grids,Forms,StrUtils,
     Controls,Registry,Math,SysConst,ShellApi,MMSystem,ClipBrd,Harmonizer;

{Notes:

1. To get a message for the last Windows errors use: SysErrorMessage(GetLastError)
}


type TASCIICharSet = set of #0..#$FF;	{A set that includes all ASCII chars}

type TDoubleArray = array[0..MaxInt div SizeOf(Double)-1] of Double;
type PDoubleArray = ^TDoubleArray;

type TDynByteArray = array of byte;
type TDynWordArray = array of Word;
type TDynDWordArray = array of DWord;

type PDynByteArray =  ^TDynByteArray;
type PDynWordArray =  ^TDynWordArray;
type PDynDWordArray = ^TDynDWordArray;


{Arithmetic and Transendental functions }
function Log(X: extended): extended;
function Power(X,Y:double): double;
function AntiLog(X: double): double;
function Sign(X: double): double;
function CopySign(V,S: double): double;
function ArcTan2(Y,X:double) : double;
function Mag3DVector(X,Y,Z:double):double;
function ClipFraction(V:double; N:integer): double;
function FindMin(V1,V2: double): double;
function FindMax(V1,V2: double): double;
function LinInterpolate(V1,V2,D: double): double;
function NearestMultipleUp(V, N: double): double;
function NearestMultipleDown(V, N: double): double;
function NearestMultiple(V,N: double;Low: boolean): double; overload;
function NearestMultiple(V,N: double): double; overload;
function FloatMod(X,Y: extended): extended;
function FloatMod2(X,Y: extended): extended;
function DivMod(Numerator, Denominator: integer; var Remainder: integer): integer; overload;
function DivMod(Numerator, Denominator: Int64; var Remainder: Int64): Int64; overload;
function DivMod(Numerator, Denominator: double; var Remainder: double): double; overload;
function DivMod(Numerator, Denominator: extended; var Remainder: extended): integer; overload;
function ExtendSign(B: byte): integer;
function IntPower2(P: integer): integer;
function IntExponent(A,B: integer): integer;
function CenteredRandom(Range,Center: double): double;
function GetRandomRange(Low,High: double): double;
function Get8087Status(Clear: Boolean): Word; assembler;
procedure Clear8087Status; assembler;
function FastFloor(F: double): integer;
function FastCeil(F: double): integer;
function FastTrunc(F: double): integer;
function FastRound(F: double): integer;
function TriMin(a, b, c: Integer): Integer;
function WrappedDifference(V1,V2,Range: double): double;
procedure ModuloRange(var X: integer; LowEnd,HighEnd: integer); overload;
procedure ModuloRange(var X: double; LowEnd,HighEnd: double); overload;
function ModuloTruncate(V: integer; M: integer): integer;


{Number manipulation routines }
function FloatToEngineerStr(D: extended; Digits: integer): string;
procedure RealFormat(L,R: integer);
function GetXPLRealStr(V: double): string;
function PadInteger(V: integer; Width: integer): string;
function IntToStrThousands(Int: int64): string;
function ExtractNumber(Str: string; var Ptr1: integer; var Num: double): boolean;
function GetNumberValue(S: string; var Inx: integer; Def: double): double; overload;
function GetNumberValue(S: string): double;  overload;
function StrToFloatDef(const S: string; Default: Extended): Extended;

function ValidDateString(D:string): boolean;
procedure FixDate(var M,D,Y: integer);
function GetShortDateEditMask: string;
function ConvertStrToDate(S: string): TDate;
function GetMonthNumber(Date: TDate): integer;
function GetMonthCount(D2,D1: double): integer;

function IsValidFloatStr(S: string): boolean;
function IsValidNumber(S: string): boolean;
function IsIntegerStr(S: string): boolean;
function IsUnsigedIntegerStr(S: string): boolean;
function UnroundedFloat(V: extended; Digits: integer): string;
function LeftTruncateInteger(Value,Digit: int64): int64;
function RightZeroInteger(Value,Digit: int64): int64;
function GetIntegerDigit(Value,Digit: int64): int64;
function NormalizeRange(V,Min,Max: integer): integer; overload;
function NormalizeRange(V,Min,Max: double): double; overload;
function IntToOctal(Value : Integer): string;
function GetIntMagStr(V: int64; Digits: integer): string;

{Number I/O routines }
function IntegerIn(var IFile: TextFile): integer;
procedure RealOut(var OFile: TextFile; V: double);
function RealIn(var IFile: TextFile): double;


{Character and string functions}
function IsDigit(C: char): boolean;
function IsHexDigit(C: char): boolean;
function IsUpperHexDigit(C: char): boolean;
function IsAlpha(C:char): boolean;
function IsAlphaNumeric(C:char): boolean;
function IsUpperCaseChar(C: char): boolean;
function IsLowerCaseChar(C: char): boolean;
function IsASCIIChar(C: char): boolean;
function FindChar(const C: Char; const S: string; Start: integer): integer;
function FindLastOccurrence(Str: string; C: Char): integer;
function FindString(const Substr,S: string; Start: integer): integer;
function PosNoCase(const SubStr,S: ansistring): integer;
function PosWildCard(SubStr,S: string; Wild: char): integer;
function PosAfterIndex(const SubStr,Str: string; var Index: integer): integer;
function PosWildNoCase(SubString,S: string;Wild:char): integer;
function FindStr(SubStr, S: string; var Loc: integer; WholeWord,MatchCase: boolean): boolean;
function WildCompare(S1,S2: string; Wild: char): boolean; overload;
function WildCompare(S1,S2: string): boolean; overload;
function ExactCompare(S1,S2: string): boolean;

function IsWhiteSpace(C: char): boolean;
function IsExtendedAlpha(C: char): boolean;
function IsWordBreak(C: Char; CrossHyphen: boolean): boolean;
function IsWordBreakEx(C,Adj: Char; CrossHyphen: boolean): boolean;
function IsWholeWord(S: string; FirstInx,LastInx: integer): boolean;



function IsAllDigits(S: string): boolean;
procedure GetDigitCount(S: string; var Total, MaxContig: integer);

function UpperChar(C: char): char;
function LowerChar(C: char): char;
function RemoveMatchingStr(SubStr, S: string; MatchCase: boolean): string;
function PadLeftString(Str: string; Count: integer; C: char): string;
function PadRightString(Str: string; Count: integer; C: char): string;
function GetTabStrRight(Str: string; Tab: integer): string;
function GetTabStrLeft(Str: string; Tab: integer): string;
function TabRightInteger(I,Tab: longint): string;
function TabLeftInteger(I,Tab: longint): string;
function ExtractToken(S: string; Sep: char; var P: integer): string; overload;
function ExtractToken(S: string; Sep: TASCIICharSet; var P: integer): string; overload;
procedure ExtractWordsFromString(S: string; SL: TStringList);
procedure ParseStringArgs(S: string; Sep,Quote: char; var SA: TStringDynArray);
procedure GetStringArgs(S: string; Sep,Quote: char; var SA: TStringDynArray);
procedure GetArgsInBlock(S: string; Token1,Token2,Sep: char; var SL: TStringList);
function ReturnEnclosedBlock(S: string; Inx: integer; BeginToken,EndToken: char): string;
function RemoveEnclosedText(S: string; BeginToken,EndToken: char): string;
function GetEndOfBlock(S: string; Inx: integer; BeginToken,EndToken: char): integer;

procedure ReplaceChars(var S: string; Target,Replacement: Char); overload;
procedure ReplaceChars(var S: string; Target: TASCIICharSet; Replacement: Char); overload;
function ReplaceStrChars(S: string; Target,Replacement: Char): string;

function ReplaceStr(const AText, AFromText, AToText: string): string;
function RemoveChars(const S: string; C: char): string;
function RemoveControls(var S: string): boolean;
function RemoveSpaces(const S: string): string;
function RemoveNonAlphaNum(S: string): string;
function TrimNonNumerics(const S: string): string;
function MatchCount(S1,S2: string; NoCase: boolean): integer;
procedure EatLeadingSpaces(var S: string);
function EatTrailingDigits(S: string): string;
procedure EatLeadingChars(var S: string; const C: char); overload;
procedure EatTrailingChars(var S: string; const C: char); overload;
procedure EatLeadingChars(var S: string; const CS: TASCIICharSet); overload;
procedure EatTrailingChars(var S: string; const CS: TASCIICharSet); overload;

procedure EatLeadTrailChars(var S: string; const C: char); overload;
procedure EatLeadTrailChars(var S: string; const CS: TASCIICharSet); overload;
procedure WrapString(var S: string; BreakChar: char; Column: integer);
function FillString(C: char; N: integer): string;
function ExtractStringLine(var Source: string; var Destination: string; var Inx: integer): char;
function CountCharMatches(S: string; C: Char): integer;
function IntToBin(IValue: Int64; NumBits: word; LeadZero: boolean) : string;
function BinToInt(Value: string): integer;
function DottedString(MaxDots: integer): string;
function CountColums(S: string; TabSpaces: integer): integer;
function DeleteSubstring(Substring: string; var S: string; CaseSensitive: boolean): boolean;
function DeleteAllSubStrings(Substring: string; var S: string; CaseSensitive: boolean): integer;
function CopyRange(var S: string; Inx1,Inx2: integer): string;
procedure DeleteRange(var S: string; Inx1,Inx2: integer);
function EditDistance(Target, Test: string): Integer;



{Character and string I/O}
procedure TabStringOut(var OFile: TextFile; Str: string; TabCount:integer);
procedure CRLFOut(var OFile: TextFile);
procedure SpaceOut(var F: TextFile; Num: integer);

{File Management Routines}
function IsFileOpen(var F): boolean;
function IsFileAssigned(var F): boolean;
function ValidateFilename(var S: string): boolean;
function GetFileSizeByName(const FileName: string): Int64;
function FileDateTime(const FileName: string): System.TDateTime;
function GetFilePosition(var F: TextFile): Longint;
function ExtractParentDirName(Path: string): string;
function ExtractParentPath(var Path: string): string;
function ExtractNamePart(FileName: string): string;
function RemoveDriveLetter(Path: string): string;
function RelPathToAbsPath(const RelPath, BasePath: string): string;
function AbsPathToRelPath(const AbsPath, BasePath: string): string;
function NormalizeFilename(Filename,DefaultExt: string): string;
function ExpandEnvVars(const Str: string): string;


function CreateUniqueFilename(Path,Name,Ext: string): string;
procedure ChangeDefaultDir(Name: string);
function GetDefaultDir: string;
function IsFileInUse(FName: string): boolean;
procedure FileCopy(const Source, Destin:String);
function ClearRemoveFolder(const Path: string): Boolean;

procedure OpenWindowsExplorer(Path: string);
procedure OpenDOSWindow(Path: string);
function GetIniPathName(SubDir: string; Create: boolean): string;
function IsOutputRedirected: boolean;
function IsInputRedirected: boolean;

function GetUsedDiskSpace(Path: string; var Used: Int64): boolean;


{Time/Degrees decoding routines}
procedure HourToHMSM(Hours: extended; var Hour,Min,Sec,MSec: integer);
function HourMinuteSecStr(Hours: extended; DoDegrees,DoMilliseconds: boolean; Digits: integer): string;
function DaysHourMinuteSecStr(Hours: extended; DoMilliseconds: boolean; Digits: integer): string;


{Logical Routines}
function PickString(B: boolean; S1,S2: string): string;
function PickInteger(B: boolean; V1,V2: Integer): Integer;
function PickFloat(B: boolean; V1,V2: Extended): Extended;
function PickColor(B: boolean; C1,C2: TColor): TColor;
function PickBool(B: boolean; B1,B2: boolean): boolean;
function MatchIntToStr(V: integer; T: array of integer; S: array of string): string;

function SwapIntegerArgs(Swap: boolean; var A1,A2: integer): boolean;
procedure SwapIntegers(var A1,A2: integer);
procedure SwapColors(var C1,C2: TColor);
function SwapStringArgs(Swap: boolean; var A1,A2: string): boolean;
procedure SwapRectangles(var R1,R2: TRect);
function SwapDoubleArgs(Swap: boolean; var A1,A2: double): boolean;
procedure SwapDouble(var A1,A2: double);

{Bit handling routines}
function IsBitSet(Value,Bit: dword): boolean;
function AreAllBitClear(Value,Bits: dword): boolean;
function AreAllBitSet(Value,Bits: dword): boolean;
function ClearBits(Value,Bits: DWord): DWord;
function SetBits(Value,Bits: DWord): DWord;
function SetClearBits(DoSet: boolean; Value,Bits: DWord): DWord;

{Misc. Routines}
function StringToHex(S: string): string;
function GetByteArrayStr(BA: TDynByteArray): string; overload;
function GetByteArrayStr(BA: array of byte): string; overload;
procedure FillPattern(Dest,Pattern: Pointer; DestCnt,PatternCnt: integer);
procedure ParseComLine(var Switch,Name: string);
procedure SetInternational(Flag: boolean);
procedure SetInternationalFoot(V: boolean);
function ReadBigIniString(INIFile: TIniFile; const Section, Ident, Default: string): string;
function ReadRealIni(Ini: TIniFile;const Section, Ident: string; Default: double): double;
procedure WriteRealIni(Ini: TIniFile;const Section, Ident: string; Value: double);
function ReadPointIni(Ini: TIniFile; const Section, Ident: string; Default: TPoint): TPoint;
procedure WritePointIni(Ini: TIniFile;const Section, Ident: string; Value: TPoint);

procedure SetHelpfilePath;
function GetBuildNumber: string;
function EncodeHexStr(S: string): string;
function DecodeHexStr(S,Default: string): string;
procedure EncodeINIString(INIFile: TIniFile; const Section, Ident, Value: string);
function DecodeINIString(INIFile: TIniFile; const Section, Ident, Default: string): string;

procedure SaveStringsIni(Filename,Section: string; Strs: TStrings);
procedure LoadStringsIni(Filename,Section: string; Strs: TStrings);
procedure UpdateAndSaveComboBox(IniFilename,Section: string; Combo: TComboBox);


function IsValidHexString(S: string): boolean;
function GetShiftStateStr(Shift: TShiftState): string;

function GetHexDumpStr(var Buffer: array of byte; LabelOffSet: integer = 0): string; overload;
function GetHexDumpStr(P: PByteArray; Size: integer; LabelOffSet: integer = 0): string; overload;
function GetHexDumpStr(S: string; LabelOffSet: integer = 0): string; overload;

function IntToHexBytes(V: cardinal; MinBytes: integer): string;


function ReadRawClipboardStr: string;
procedure SetClipboardOEMText(const Value: string);
procedure SetWideClipboardText(const Text: WideString);
procedure SetClipboardAsHTML(const Value: string);


procedure RaiseLastOSErrorMsg(Msg: string);
function SystemIdleTime: DWord;

procedure SetProcessorAfinity(NewMask: cardinal);



var CPUSpeed: int64;	{Current CPU speed from registry (May not be accurate)}

{Flags we are using system regional settings - always starts True}

var InternationalFlag: boolean = true;


{Delay and timer routines}

type TTimeoutData = record
 MSRemaining: integer;
 end;

type TAbortCallback =function(TD: TTimeoutData): boolean;



procedure DoTimeDelay(MS: dword);
procedure DelayProcessMessages(MS: dword);
procedure DelayProcessMessagesAbort(MS: dword; var Abort: boolean);
procedure DelayProcessMessagesRun(MS: dword; var Run: boolean);
procedure DelayProcessMessagesCallback(MS: dword; AbortCallBack: TAbortCallback);
procedure DelaySleepProcess(MS: dword);
procedure DoHiResDelay(US: int64);
procedure HiResDelayProcessMessages(US: int64);
function GetHiResTick: dword;
function GetHiResTick64: int64;
function GetHiResSeconds: extended;
function RDTSC : int64;
function GetSuperResSeconds: extended;
function GetSuperResTick64: int64;

function GetProcessorSpeed: int64;

{Graphics Routines }
procedure LineToLastPixel(H: HDC; X1,Y1,X2,Y2: integer);
function GetLastPixelPos(X1,Y1: integer; var X2,Y2: integer): boolean;

{Delphi VCL Support}
procedure ScaleGrid(Grid: TCustomGrid;M,D: integer);
procedure CenterScrollBox(SB: TScrollBox);

var ThousandsComma: boolean=False;	{Insert comma between thousands}

var LastChar: char; 		{Last character read by TextFile routines}

const FilenameMax=255;		{ Maximum characters in a filename }

const DesignRes = 96;		{Current design resolution}

const
 CtrlA= #$01;	CtrlB= #$02;	CtrlC= #$03;	CtrlD= #$04;
 CtrlE= #$05;	CtrlF= #$06;	CtrlG= #$07;	CtrlH= #$08;
 CtrlI= #$09;	CtrlJ= #$0A;	CtrlK= #$0B;	CtrlL= #$0C;
 CtrlM= #$0D;	CtrlN= #$0E;	CtrlO= #$0F;	CtrlP= #$10;
 CtrlQ= #$11;	CtrlR= #$12;	CtrlS= #$13;	CtrlT= #$14;
 CtrlU= #$15;	CtrlV= #$16;	CtrlW= #$17;	CtrlX= #$18;
 CtrlY= #$19;	CtrlZ= #$1A;


const
 SP_CHAR=#$20;  ESC_CHAR=#$1B;  EOF_CHAR=#$1A;  FF_CHAR=#$0C;
 CR_CHAR=#$0D;  LF_CHAR=#$0A;   TAB_CHAR=#$09;  CRLF_CHAR=#$0D#$0A;
 Bell_Char=#$07; Apost_Char = #$27;


{All these constants work by multiplying}

var Feet2Meters: double;	{feet to meters}
var Inches2Meters: double;	{Inches to Meters}
var Inches2Millimeters: double;	{Inches to Millimeters}
var Inches2Centimeters: double;

var Meters2Feet: double;	{Meters to Feet}
var Meters2Inches: double;      {Meters to Inches}
var Millimeters2Inches: double;	{Millimeters to Inches}
var Centimeters2Inches: double;	{Centimeters to Inches}

var SqrMeters2SqrFeet: double;	{Square Meters to Square Feet}
var SqrFeet2SqrMeters: double;	{Square Feet to Square Meters}

var CubeMeters2CubeFeet: double;	{Cubic Meters to Cubic Feet}
var CubeFeet2CubeMeters: double;	{Cubic Feet to Cubic Meters}



const Degree2Radian=PI/180;	{Degrees to Radians}
const Radian2Degree=180/PI;	{Radians to Degrees}

const HalfPi = PI/2;		{90 degress in radians}
const TwoPI = PI*2;		{360 degrees in radians}


{Object to parse text into string pairs }
{Delimit1 = the top delimiter character - separates the text into individual string-pairs}
{Delimit2 = the secondary delimiter character - splits strings pairs into individual strings }
{Assign the text to be parsed to the Text property}

type TStringPairs = class(TObject)
 private
  FText: string;
  procedure ParseText(const Value: string);
  function GetCount: integer;
 protected
 public
  Strings1,Strings2: TStringList;
  Delimit1,Delimit2: char;
  TrimChars: TASCIICharSet;
  Trim: boolean;
  constructor Create;
  destructor Destroy; override;
  property Text: string read FText write ParseText;
  property Count: integer read GetCount;
 end;






implementation

uses Version;

var FLeft,FRight: integer;		{Real format information }


{----------------- Local Windows Format settings ---------------------------}


type TFormatSettings = record
 CurrencyString: string;
 CurrencyFormat: Byte;
 CurrencyDecimals: Byte;
 DateSeparator: Char;
 TimeSeparator: Char;
 ListSeparator: Char;
 ShortDateFormat: string;
 LongDateFormat: string;
 TimeAMString: string;
 TimePMString: string;
 ShortTimeFormat: string;
 LongTimeFormat: string;
 ShortMonthNames: array[1..12] of string;
 LongMonthNames: array[1..12] of string;
 ShortDayNames: array[1..7] of string;
 LongDayNames: array[1..7] of string;
 ThousandSeparator: Char;
 DecimalSeparator: Char;
 TwoDigitYearCenturyWindow: Word;
 NegCurrFormat: Byte;
 end;



{$IF CompilerVersion < 17}
var FormatSettings: TFormatSettings;
type  DWORD_PTR = cardinal;
{$IFEND}



procedure CopyStringArray(var Dest: array of string; const Source: array of string);
var I: integer;
begin
for I:=Low(Dest) to High(Dest) do
 Dest[I]:= Source[I];
end;



procedure GetFormatSettings;
{Used to make DSupport compatible with later versions of Delphi}
begin
{$IF CompilerVersion < 17}
FormatSettings.CurrencyString:=CurrencyString;
FormatSettings.CurrencyFormat:=CurrencyFormat;
FormatSettings.CurrencyDecimals:=CurrencyDecimals;
FormatSettings.DateSeparator:=DateSeparator;
FormatSettings.TimeSeparator:=TimeSeparator;
FormatSettings.ListSeparator:=ListSeparator;
FormatSettings.ShortDateFormat:= ShortDateFormat;
FormatSettings.LongDateFormat:=LongDateFormat;
FormatSettings.TimeAMString:= TimeAMString;
FormatSettings.TimePMString:= TimePMString;
FormatSettings.ShortTimeFormat:= ShortTimeFormat;
FormatSettings.LongTimeFormat:=LongTimeFormat;
CopyStringArray(FormatSettings.ShortMonthNames,ShortMonthNames);

CopyStringArray(FormatSettings.ShortMonthNames,ShortMonthNames);
CopyStringArray(FormatSettings.LongMonthNames,LongMonthNames);
CopyStringArray(FormatSettings.ShortDayNames,ShortDayNames);
CopyStringArray(FormatSettings.LongDayNames, LongDayNames);

FormatSettings.ThousandSeparator:=ThousandSeparator;
FormatSettings.DecimalSeparator:=DecimalSeparator;
FormatSettings.TwoDigitYearCenturyWindow:=TwoDigitYearCenturyWindow;
FormatSettings.NegCurrFormat:=NegCurrFormat;
{$IFEND}
end;


{----------------- Logic Routines ---------------------------}


function PickString(B: boolean; S1,S2: string): string;
{Return S1 if B is true else S2}
begin
if B then Result:=S1 else Result:=S2;
end;


function MatchIntToStr(V: integer; T: array of integer; S: array of string): string;
{Locate V in T and return corresponding string}
var I: integer;
begin
Result:='';
for I:=0 to High(T) do
 if (V=T[I]) and (I<Length(S)) then
	begin
	Result:=S[I];
	break;
	end;
end;


function PickInteger(B: boolean; V1,V2: Integer): Integer;
{Return V1 if B is true else V2}
begin
if B then Result:=V1 else Result:=V2;
end;


function PickColor(B: boolean; C1,C2: TColor): TColor;
{Return C1 if B is true else V2}
begin
if B then Result:=C1 else Result:=C2;
end;


function PickFloat(B: boolean; V1,V2: Extended): Extended;
{Return V1 if B is true else V2}
begin
if B then Result:=V1 else Result:=V2;
end;



function PickBool(B: boolean; B1,B2: boolean): boolean;
{Return B1 if B is true else B2}
begin
if B then Result:=B1 else Result:=B2;
end;



procedure SwapIntegers(var A1,A2: integer);
{Swap integer arguments}
var T: integer;
begin
T:=A1; A1:=A2; A2:=T;
end;


procedure SwapColors(var C1,C2: TColor);
{Swap integer arguments}
var T: TColor;
begin
T:=C1; C1:=C2; C2:=T;
end;



function SwapIntegerArgs(Swap: boolean; var A1,A2: integer): boolean;
{Swap integer arguments based Swap boolean}
var T: integer;
begin
Result:=Swap;
if not Swap then exit;
T:=A1; A1:=A2; A2:=T;
end;



function SwapStringArgs(Swap: boolean; var A1,A2: string): boolean;
{Swap string arguments based Swap boolean}
var T: string;
begin
Result:=Swap;
if not Swap then exit;
T:=A1; A1:=A2; A2:=T;
end;



function SwapDoubleArgs(Swap: boolean; var A1,A2: double): boolean;
{Swap double arguments based Swap boolean}
var T: double;
begin
Result:=Swap;
if not Swap then exit;
T:=A1; A1:=A2; A2:=T;
end;



procedure SwapDouble(var A1,A2: double);
var T: double;
begin
T:=A1; A1:=A2; A2:=T;
end;


procedure SwapRectangles(var R1,R2: TRect);
{Swap specified rectangles}
var T: TRect;
begin
T:=R1; R1:=R2; R2:=T;
end;



{----------------- Bit manipulation routines ----------------------------------}

function IsBitSet(Value,Bit: dword): boolean;
{Returns true if any of the specifed bits are ones}
begin
Result:=(Value and Bit)<>0;
end;


function AreAllBitSet(Value,Bits: dword): boolean;
{Returns true if all of the specifed bits are ones}
begin
Result:=(Value and Bits)=Bits;
end;



function AreAllBitClear(Value,Bits: dword): boolean;
{Returns true if all of the specifed bits are Zeros}
begin
Result:=(Value and Bits)=0;
end;




function SetBits(Value,Bits: DWord): DWord;
{Set specified  bits by "ORing" }
{so other bits are not changed}
begin
Result:=Value or Bits;
end;



function ClearBits(Value,Bits: DWord): DWord;
{Clear specified bits by "ANDing"}
{ so other bits are not changed}
begin
Result:=Value and not Bits;
end;




function SetClearBits(DoSet: boolean; Value,Bits: DWord): DWord;
{Set or Clear the specified bits depending on whether DoSet is True }
begin
if DoSet then Result:=SetBits(Value,Bits)
else Result:=ClearBits(Value,Bits);
end;



{----------------- Miscellanious Routines ---------------------------}



procedure FillPattern(Dest,Pattern: Pointer; DestCnt,PatternCnt: integer);
{Fill Dest buffer with the bytes specified in pattern}
var I: integer;
begin
{$R-}
for I:=0 to DestCnt-1 do
	begin
	{if Range check is enabled, this will give run time error if over 32K}
	PByteArray(Dest)[I]:=PByteArray(Pattern)[I mod PatternCnt];
	end;
{$R+}
end;




procedure SetInternationalFoot(V: boolean);
{Set international vs US survey foot}
begin
if V then Feet2Meters:=0.3048
else Feet2Meters:=0.304800609601;
Meters2Feet:=1/Feet2Meters;
Meters2Inches:=Meters2Feet * 12;
Millimeters2Inches:=Meters2Inches/1000;
Centimeters2Inches:=Meters2Inches/100;

Inches2Meters:=1/Meters2Inches;
Inches2Millimeters:=Inches2Meters * 1000;
Inches2Centimeters:=Inches2Meters * 100;

SqrMeters2SqrFeet:=Sqr(Meters2Feet);
SqrFeet2SqrMeters:=Sqr(Feet2Meters);

CubeMeters2CubeFeet:=Meters2Feet * Meters2Feet * Meters2Feet;
CubeFeet2CubeMeters:=Feet2Meters * Feet2Meters * Feet2Meters;
end;



procedure SetInternational(Flag: boolean);
{Enable/Disable International Support }
var DefaultLCID: Integer;
begin
InternationalFlag:=Flag;
DefaultLCID := GetThreadLocale;
if Flag then
	begin
	{This gets a "platform" warning}
	{$WARNINGS OFF}
	IndSystemData.DecimalSeparator := GetLocaleChar(DefaultLCID, LOCALE_SDECIMAL, '.');
	IndSystemData.ThousandSeparator := GetLocaleChar(DefaultLCID, LOCALE_STHOUSAND, ',');
	{$WARNINGS ON}
	end
else
	begin
	IndSystemData.DecimalSeparator:='.';
	{No thousands separator so we can parse comma separated data}
	IndSystemData.ThousandSeparator:=#0;
	end;
end;



procedure ParseComLine(var Switch,Name: string);
{ Extract filename and all switches from command line }
var P: integer;
var I: Word;
begin
Name:='';
Switch:='';
for I := 1 to ParamCount do Name:=Name+ParamStr(I);

P:=Pos('/',Name);
while P>0 do
	begin
        if (P+1)<=Length(Name) then Switch:=Switch+Name[P+1];
        Delete(Name,P,2);
	P:=Pos('/',Name);
	end;
Switch:=UpperCase(Switch);
end;



function ReadBigIniString(INIFile: TIniFile; const Section, Ident, Default: string): string;
{Delphi limits INI file string item to 2048 bytes}
{This routine reads strings up to 16 K}
var Buffer: array[0..1024*16] of Char;
begin
SetString(Result, Buffer, GetPrivateProfileString(PChar(Section),
    PChar(Ident), PChar(Default), Buffer, SizeOf(Buffer), PChar(INIFile.FileName)));
end;



function ReadRealIni(Ini: TIniFile;const Section, Ident: string; Default: double): double;
{ Read real from specified INI file}
var S: string;
var T: char;
begin
T:=IndSystemData.DecimalSeparator;
IndSystemData.DecimalSeparator:='.';
S:=Ini.ReadString(Section,Ident,FloatToStr(Default));
Result:=StrToFloat(S);
IndSystemData.DecimalSeparator:=T;
end;


procedure WriteRealIni(Ini: TIniFile;const Section, Ident: string; Value: double);
{ Write real to specified INI file}
var T: char;
begin
T:=IndSystemData.DecimalSeparator;
IndSystemData.DecimalSeparator:='.';
Ini.WriteString(Section,Ident,FloatToStr(Value));
IndSystemData.DecimalSeparator:=T;
end;


function ReadPointIni(Ini: TIniFile; const Section, Ident: string; Default: TPoint): TPoint;
{ Read TPoint from specified INI file}
begin
Result.X:=Ini.ReadInteger(Section,Ident+'X',Default.X);
Result.Y:=Ini.ReadInteger(Section,Ident+'Y',Default.Y);
end;


procedure WritePointIni(Ini: TIniFile;const Section, Ident: string; Value: TPoint);
{ Write TPoint to specified INI file}
begin
Ini.WriteInteger(Section,Ident+'X',Value.X);
Ini.WriteInteger(Section,Ident+'Y',Value.Y);
end;



procedure DoTimeDelay(MS: dword);
{ Delay specified number of miliseconds}
{ Resolution 1 ms in 95, 5 ms in NT}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do;
end;



procedure DelayProcessMessages(MS: dword);
{ Delay specified number of miliseconds}
{Process messages at the same time}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do Application.ProcessMessages;
end;



procedure DelayProcessMessagesAbort(MS: dword; var Abort: boolean);
{ Delay specified number of miliseconds}
{Process messages at the same time}
{Exit early if Abort flag is true - (varaible passed by reference)}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do
	begin
	Application.ProcessMessages;
	if Abort then break;
	end;
end;


procedure DelayProcessMessagesRun(MS: dword; var Run: boolean);
{ Delay specified number of miliseconds}
{Process messages at the same time}
{Exit early if Run flag is False - (varaible passed by reference)}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do
	begin
	Application.ProcessMessages;
	if not Run then break;
	end;
end;



procedure DelayProcessMessagesCallback(MS: dword; AbortCallBack: TAbortCallback);
{ Delay specified number of miliseconds}
{Process messages at the same time}
{Exit early if Run flag is False - (varaible passed by reference)}
var Stop: DWord;
var TD: TTimeoutData;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do
	begin
	Application.ProcessMessages;
	TD.MSRemaining:=Stop-timeGetTime;
	if Assigned(AbortCallBack) then
	 if AbortCallBack(TD) then break;
	end;
end;



procedure DelaySleepProcess(MS: dword);
{ Delay specified number of miliseconds}
{Process messages and Sleep at the same time}
var Stop: DWord;
begin
Stop:=timeGetTime+MS;
while timeGetTime<Stop do
	begin
	Application.ProcessMessages;
	Sleep(1);
	end;
end;

{------------------------------------------------------------------------------}





function GetHiResTick: dword;
{Returns high resolution ticks in usec}
var C,Freq: Int64;
begin
QueryPerformanceFrequency(Freq);
QueryPerformanceCounter(C);
C:=C * 1000000 div Freq;
Result:=ULARGE_INTEGER(C).LowPart;
end;



function GetHiResTick64: int64;
{Returns high resolution ticks in usec}
{For a 2.6Ghz processor the math will overflow at about 2 hours}
var C,Freq: Int64;
begin
QueryPerformanceFrequency(Freq);
QueryPerformanceCounter(C);
C:=C * 1000000 div Freq;
Result:=C;
end;



function GetHiResSeconds: extended;
{Returns high resolution ticks in seconds}
{For a 2.6Ghz processor the math will overflow at about 2 hours}
var C,Freq: Int64;
begin
QueryPerformanceFrequency(Freq);
QueryPerformanceCounter(C);
Result:=C / Freq;
end;




procedure DoHiResDelay(US: int64);
{Do High resolutio delay in microseconds}
var Stop: int64;
begin
Stop:=GetHiResTick+US;
while GetHiResTick<Stop do;
end;




procedure HiResDelayProcessMessages(US: int64);
{Do High resolutio delay in microseconds}
var Stop,Cnt: int64;
begin
Stop:=GetHiResTick+US;
while true do
	begin
	Cnt:=GetHiResTick;
	if Cnt>=Stop then break;
	if (Cnt mod 1000)=0 then Application.ProcessMessages;
	end;
end;



function RDTSC : int64; register;
{Read hardware CPU timer}
{Should be counting at CPU speed}
asm  db   $0F, $31;
end;



function GetSuperResSeconds: extended;
{Get times down into nano second resolution}
var Speed: int64;
begin
Result:=RDTSC/CPUSpeed;
end;


function GetSuperResTick64: int64;
{Get tick count with nano second resolution}
var C: int64;
begin
C:=RDTSC;
C:=C * 1000000000 div CPUSpeed;
Result:=C;
end;




function GetProcessorSpeed: int64;
{Get processor speed from registry}
{Value is in Hertz}
var Reg: TRegistry;
begin
Result:=0;
{The argument "KEY_READ or $0100" allows keys}
{to be read on both 32 & 64-bit machines}
Reg := TRegistry.Create(KEY_READ or $0100);
try
Reg.RootKey := HKEY_LOCAL_MACHINE;
if Reg.OpenKey('Hardware\description\system\CentralProcessor\0', False) then
	begin
	{Note: Win98 will get illegal instruciton error if you try to read a nonexistent value}
	if Reg.ValueExists('~Mhz') then Result:=Reg.ReadInteger('~Mhz');
	{This is necessary or else you'll get a negative value}
	Result:=Result * 1000000;
	Reg.CloseKey;
	end;
finally Reg.Free; end;
end;


{------------------------------------------------------------------------------}



procedure SetHelpfilePath;
{Take the design-time help filename and add the path for the executable}
var S: string;
begin
S:=ExtractFilename(Application.helpFile);
Application.helpFile:=ExtractFilePath(Application.ExeName) + S;
end;


function GetBuildNumber: string;
{Get build number from version info}
var I: integer;
var VersionInfo: TVersionInfo;
begin
Result:='';
VersionInfo:=TVersionInfo.Create(nil);
Result:=VersionInfo.FileVersion;
if Pointer(Result) = nil then exit;
I:=Length(Result);
while (Result[I]<>'.') and (I>1) do Dec(I);
Delete(Result,1,I);
VersionInfo.Free;
end;


function EncodeHexStr(S: string): string;
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do Result:=Result+IntToHex(byte(S[I]),2);
end;



function StringToHex(S: string): string;
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do
	begin
	if I<>1 then Result:=Result+',';
	Result:=Result+'$'+IntToHex(byte(S[I]),2);
	end;
end;


function GetByteArrayStr(BA: TDynByteArray): string;
var I: integer;
begin
Result:='';
for I:=0 to High(BA) do
	begin
	if I<>0 then Result:=Result+',';
	Result:=Result+IntToHex(BA[I],2);
	end;
end;



function GetByteArrayStr(BA: array of byte): string;
var I: integer;
begin
Result:='';
for I:=0 to High(BA) do
	begin
	if I<>0 then Result:=Result+',';
	Result:=Result+IntToHex(BA[I],2);
	end;
end;



procedure EncodeINIString(INIFile: TIniFile; const Section, Ident, Value: string);
{Encode an ini string so it can contain control characters}
var HexString: string;
begin
HexString:=EncodeHexStr(Value);
IniFile.WriteString(Section,Ident,HexString);
end;




function IsValidHexString(S: string): boolean;
{Checks all chars in string for valid hex}
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do
 if not IsHexDigit(S[I]) then exit;
Result:=True;
end;



function DecodeHexStr(S,Default: string): string;
var I: integer;
var BS: string[3];
begin
Result:=Default;
if not IsValidHexString(S) then exit;

Result:='';
for I:=0 to (Length(S) div 2)-1 do
	begin
	BS:='$'+Copy(S,I*2+1,2);
	Result:=Result+char(StrToInt(BS));
	end;
end;




function DecodeINIString(INIFile: TIniFile; const Section, Ident, Default: string): string;
{Decode an ini string that contains control characters}
var HexString: string;
begin
HexString:=ReadBigIniString(IniFile,Section,Ident,'');
Result:=DecodeHexStr(HexString,Default);
end;




procedure SaveStringsIni(Filename,Section: string; Strs: TStrings);
{Save TStrings to an INI file}
var I: integer;
var IniFile: TIniFile;
var Path: string;
begin
Filename:=ExpandEnvVars(Filename);
Path:=ExtractFilePath(Filename);
ForceDirectories(Path);
IniFile:=TIniFile.Create(Filename);
try
IniFile.EraseSection(Section);
IniFile.WriteInteger(Section,'Count',Strs.Count);
for I:=0 to Strs.Count-1 do
	begin
	IniFile.WriteString(Section,'String'+IntToStr(I),Strs[I]);
	end;
finally IniFile.Free; end;
end;



procedure LoadStringsIni(Filename,Section: string; Strs: TStrings);
{Load TStrings to an INI file}
var I,Cnt: integer;
var IniFile: TIniFile;
begin
Filename:=ExpandEnvVars(Filename);
IniFile:=TIniFile.Create(Filename);
try
Cnt:=IniFile.ReadInteger(Section,'Count',0);
Strs.Clear;
for I:=0 to Cnt-1 do
	begin
	Strs.Add(IniFile.ReadString(Section,'String'+IntToStr(I),''));
	end;
finally IniFile.Free; end;
end;


function StringsIndexOf(Strings: TStrings; const Target: string): Integer;
begin
for Result:=0 to Strings.Count-1 do
 if Strings[Result]=Target then exit;
Result := -1;
end;



procedure UpdateAndSaveComboBox(IniFilename,Section: string; Combo: TComboBox);
{Copy text to drop down list and then save to Inifile}
var Inx: integer;
begin
if Combo.Text='' then exit;
Inx:=StringsIndexOf(Combo.Items,Combo.Text);
if Inx<0 then
	begin
	{Not in list, so insert}
	Combo.Items.Insert(0,Combo.Text);
	while Combo.Items.Count>30 do
	 Combo.Items.Delete(Combo.Items.Count-1);
	end
else
	begin
	{Already in list, so move to the top}
	Combo.Items.Exchange(0,Inx);
	end;
Combo.ItemIndex:=0;
SaveStringsIni(IniFilename,Section,Combo.Items);
end;





function GetShiftStateStr(Shift: TShiftState): string;
{Get string that corresponds to shift state}
begin
if ssShift in Shift then Result:=Result+'ssShift,';
if ssAlt in Shift then  Result:=Result+'ssAlt,';
if ssCtrl in Shift then Result:=Result+'ssCtrl,';
if ssLeft in Shift then Result:=Result+'ssLeft,';
if ssRight in Shift then Result:=Result+'ssRight,';
if ssMiddle in Shift then Result:=Result+'ssMiddle,';
if ssDouble in Shift then Result:=Result+'ssDouble,';
if (Length(Result)>0) and (Result[Length(Result)]=',') then Delete(Result,Length(Result),1);
end;



function ReadRawClipboardStr: string;
{Reads clipboard string including $00 characters}
var Size: integer;
var MyHandle: THandle;
var TextPtr: PChar;
begin
ClipBoard.Open;
try
MyHandle:=Clipboard.GetAsHandle(CF_OEMTEXT);
Size:=GlobalSize(MyHandle);
TextPtr:=GlobalLock(MyHandle);
SetLength(Result,Size-1);
Move(TextPtr^, Result[1], Size-1);
GlobalUnlock(MyHandle);
finally Clipboard.Close; end;
end;



procedure SetClipboardOEMText(const Value: string);
{Properly set OEM text data including control chars }
var Data: THandle;
var DataPtr: Pointer;
begin
Clipboard.Open;
Clipboard.Clear;
try
Data := GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, Length(Value) + 1);
try
DataPtr := GlobalLock(Data);
try
Move(PChar(Value)^, DataPtr^, Length(Value) + 1);
SetClipboardData(CF_TEXT, Data);
SetClipboardData(CF_OEMTEXT, Data);
finally GlobalUnlock(Data); end;
except
 GlobalFree(Data);
 raise;
 end;
 finally Clipboard.Close; end;
end;


procedure SetWideClipboardText(const Text: WideString);
{Send Wide String to Clipboard}
var Count: Integer;
var Handle: HGLOBAL;
var Ptr: Pointer;
begin
Clipboard.Open;
try
Clipboard.Clear;
Count := (Length(Text)+1)*SizeOf(WideChar);
Handle := GlobalAlloc(GMEM_MOVEABLE, Count);
try
Win32Check(Handle<>0);
Ptr := GlobalLock(Handle);
Win32Check(Assigned(Ptr));
Move(PWideChar(Text)^, Ptr^, Count);
GlobalUnlock(Handle);
Clipboard.SetAsHandle(CF_UNICODETEXT, Handle);
finally GlobalFree(Handle); end;
finally Clipboard.Close; end;
end;


procedure SetClipboardAsHTML(const Value: string);
{Properly copy HTML data to clipboard }
var Data: THandle;
var DataPtr: Pointer;
var CF_HTML: word;
begin
CF_HTML := RegisterClipboardFormat('HTML Format');
Clipboard.Open;
Clipboard.Clear;
try
Data := GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, Length(Value) + 1);
try
DataPtr := GlobalLock(Data);
try
Move(PChar(Value)^, DataPtr^, Length(Value) + 1);
SetClipboardData(CF_HTML, Data);
finally GlobalUnlock(Data); end;
except
 GlobalFree(Data);
 raise;
 end;
 finally Clipboard.Close; end;
end;


{------------------- String Handling Routines -------------------------}


function IsDigit(C: char): boolean;
{ Return true if char is a digit }
begin
Result:= (C>='0') and (C<='9');
end;


function IsHexDigit(C: char): boolean;
{ Return true if char is a Hex digit }
begin
case C of
 '0'..'9','a'..'f','A'..'F': Result:=True;
 else Result:=False;
 end;
end;



function IsUpperHexDigit(C: char): boolean;
{ Return true if char is a Upper case Hex digit }
begin
case C of
 '0'..'9','A'..'F': Result:=True;
 else Result:=False;
 end;
end;


function IsAlpha(C:char): boolean;
{ Return True If Ascii Alpha }
begin
Result:= ((C>='A')  and  (C<='Z'))  or  ((C>='a')  and  (C<='z'));
end;


function IsAlphaNumeric(C:char): boolean;
{ Return True If Ascii AlphaNumeric }
begin
Result:=(C>='0') and (C<='9') or
	((C>='A')  and  (C<='Z'))  or
	((C>='a')  and  (C<='z'));
end;


function IsUpperCaseChar(C: char): boolean;
{Returns true if character is upper case}
begin
Result:=(C >= 'A') and (C <='Z');
end;



function IsLowerCaseChar(C: char): boolean;
{Returns true if character is lower case}
begin
Result:=(C >= 'a') and (C <='z');
end;



function IsASCIIChar(C: char): boolean;
{Returns true if char is non-control, non-extened character}
begin
Result:=(C>=' ') and (C<=#$7E);
end;




function UpperChar(C: char): char;
{ Convert character to upper case }
begin
Result:=C;
if (C>='a') and (C<='z') then Result:=char(byte(Result)-$20);
end;


function LowerChar(C: char): char;
{ Convert character to lower case }
begin
Result:=C;
if (C>='A') and (C<='Z') then Result:=char(byte(Result)+$20);
end;



function FindChar(const C: Char; const S: string; Start: integer): integer;
{Find specified character starting at Start}
begin
for Result:=Start to Length(S) do if S[Result]=C then exit;
Result:=0;
end;


function FindLastOccurrence(Str: string; C: Char): integer;
{Find the last occurence of the specified Char}
begin
for Result:=Length(Str) downto 1 do if Str[Result]=C then exit;
Result:=0;
end;



function FindString(const Substr,S: string; Start: integer): integer;
{Find specified Substr in S starting at Start}
var I,I2: integer;
begin
Result:=0;
for I:=Start to Length(S) do
	begin
	if (I+Length(Substr))>Length(S) then exit;
	for I2:=1 to Length(Substr) do
		begin
		if Substr[I2]<>S[I+I2-1] then break;
		if I2=Length(Substr) then
			begin
			Result:=I;
			exit;
			end;
		end;
	end;
end;





function WildCompare(S1,S2: string; Wild: char): boolean;
{ Compare Strings Wild char match anything }
{ Wild Card char Must Be In S1 }
var I: integer;
begin
Result:=False;
if Length(S1)<>Length(S2) then exit;
for I:=1 to Length(S1) do
 if (S1[I]<>S2[I]) and (S1[I]<>Wild) then exit;
Result:=True;
end;



function WildCompare(S1,S2: string): boolean;
{ Compare Strings, ? And * Are Wild Cards }
{ Wild Card String Must Be In S1 }
var P,L,L1,L2: Integer;
begin
L1:=Length(S1);
L2:=Length(S2);
if L1<L2 then L:=L1 else L:=L2;
for P:=1 to L do
	begin
	if S1[P]='*' then  begin Result:= True; exit; end;
	if (S1[P]<>S2[P]) and (S1[P]<>'?') then  begin Result:= False; exit; end;
	end;
if L2>L1 then begin Result:=False; exit; end;
if L1<>L2 then
 for P:=L+1 to L1 do
	begin
	if S1[P]='*' then  begin Result:= True; exit; end;
	if S1[P]<>'?' then  begin Result:= False; exit; end;
	end;
Result:=True;
end;



function ExactCompare(S1,S2: string): boolean;
{Compare strings, strings must be same length}
begin
if Length(S1)=Length(S2) then Result:=WildCompare(S1,S2)
else if S1[Length(S1)]='*' then Result:=WildCompare(S1,S2)
else Result:=False;
end;



{Lookup table for converting chars to upper case}
{Bytes $61 to $7A have been changed to $41 to $5A}

type TCaseTable = array [AnsiChar] of AnsiChar;

const CaseTable: TCaseTable = (
#$00,#$01,#$02,#$03,#$04,#$05,#$06,#$07,#$08,#$09,#$0A,#$0B,#$0C,#$0D,#$0E,#$0F,
#$10,#$11,#$12,#$13,#$14,#$15,#$16,#$17,#$18,#$19,#$1A,#$1B,#$1C,#$1D,#$1E,#$1F,
#$20,#$21,#$22,#$23,#$24,#$25,#$26,#$27,#$28,#$29,#$2A,#$2B,#$2C,#$2D,#$2E,#$2F,
#$30,#$31,#$32,#$33,#$34,#$35,#$36,#$37,#$38,#$39,#$3A,#$3B,#$3C,#$3D,#$3E,#$3F,
#$40,#$41,#$42,#$43,#$44,#$45,#$46,#$47,#$48,#$49,#$4A,#$4B,#$4C,#$4D,#$4E,#$4F,
#$50,#$51,#$52,#$53,#$54,#$55,#$56,#$57,#$58,#$59,#$5A,#$5B,#$5C,#$5D,#$5E,#$5F,
#$60,#$41,#$42,#$43,#$44,#$45,#$46,#$47,#$48,#$49,#$4A,#$4B,#$4C,#$4D,#$4E,#$4F,
#$50,#$51,#$52,#$53,#$54,#$55,#$56,#$57,#$58,#$59,#$5A,#$7B,#$7C,#$7D,#$7E,#$7F,
#$80,#$81,#$82,#$83,#$84,#$85,#$86,#$87,#$88,#$89,#$8A,#$8B,#$8C,#$8D,#$8E,#$8F,
#$90,#$91,#$92,#$93,#$94,#$95,#$96,#$97,#$98,#$99,#$9A,#$9B,#$9C,#$9D,#$9E,#$9F,
#$A0,#$A1,#$A2,#$A3,#$A4,#$A5,#$A6,#$A7,#$A8,#$A9,#$AA,#$AB,#$AC,#$AD,#$AE,#$AF,
#$B0,#$B1,#$B2,#$B3,#$B4,#$B5,#$B6,#$B7,#$B8,#$B9,#$BA,#$BB,#$BC,#$BD,#$BE,#$BF,
#$C0,#$C1,#$C2,#$C3,#$C4,#$C5,#$C6,#$C7,#$C8,#$C9,#$CA,#$CB,#$CC,#$CD,#$CE,#$CF,
#$D0,#$D1,#$D2,#$D3,#$D4,#$D5,#$D6,#$D7,#$D8,#$D9,#$DA,#$DB,#$DC,#$DD,#$DE,#$DF,
#$E0,#$E1,#$E2,#$E3,#$E4,#$E5,#$E6,#$E7,#$E8,#$E9,#$EA,#$EB,#$EC,#$ED,#$EE,#$EF,
#$F0,#$F1,#$F2,#$F3,#$F4,#$F5,#$F6,#$F7,#$F8,#$F9,#$FA,#$FB,#$FC,#$FD,#$FE,#$FF);




function PosNoCase(const SubStr, S: ansistring): Integer;
{Locate String S in SubString, not case sensitive}
var I,SubLen,StrLen: Integer;
label Fail;
begin
Result:=0;
if (S='') or (SubStr='') then exit;
SubLen:=Length(SubStr);
StrLen:=Length(s);
if SubLen > StrLen then exit;
Result:=1;
while SubLen <= (StrLen-Result+1) do
	begin
	for I:=1 to SubLen do
	 if CaseTable[SubStr[I]] <> CaseTable[S[Result+I-1]] then
		begin
                Inc(Result);
		break;
		end
	 else if I>=SubLen then exit;
	end;
Result:=0;
end;



function PosWildCard(SubStr,S: string; Wild: char): integer;
{Locate SubString in string S, with wild cards}
var P2,L1,L2: integer;

	function Match: boolean;
	var P1,Inx3: integer;
	begin
	Result:=False;
	for P1:=1 to L1 do
		begin
		Inx3:=P2 + P1 -1;
		if (Inx3>L2) or ((SubStr[P1]<>S[Inx3]) and (SubStr[P1]<>Wild)) then exit;
		end;
	Result:=True;
	end;

begin
Result:=0;
L1:=Length(SubStr);
L2:=Length(S);
if L1>L2 then exit;
for P2:=1 to L2 do
  if (Substr[1]=S[P2]) or (Substr[1]=Wild) then
    if Match then begin Result:=P2; exit; end;
end;



function PosWildNoCase(SubString,S: string; Wild: char): integer;
{Locate String S in SubString, not case sensitive, wild cards}
begin
Result:=PosWildCard(UpperCase(SubString),UpperCase(S),Wild);
end;





function FindStr(SubStr, S: string; var Loc: integer; WholeWord,MatchCase: boolean): boolean;
{Find substring in S, starting at position Loc, with WholeWord and IgnoreCase options}
var Options: TStringSearchOptions;
var PC: PChar;
begin
Options:=[soDown];
if WholeWord then Options:=Options+[soWholeWord];
if MatchCase then Options:=Options+[soMatchCase];
PC:=SearchBuf(PChar(S),Length(S),Loc-1,0,SubStr,Options);
Result:=PC<>nil;
if Result then Loc:=PC - PChar(S)+1
else Loc:=0;
end;




function PosAfterIndex(const SubStr,Str: string; var Index: integer): integer;
{Find the Position of SubStr in Str starting at Index}
{Index is left pointing one character past end of the target}
{If SubStr is not found, Result will be 0 and Index = Length+1}
var SPInx,P: PChar;
begin
if Index<1 then Index:=1;
SPInx:=@Str[Index];
P:=StrPos(SPInx,PChar(SubStr));
if P<> nil then
	begin
	Result:=Integer(P)-Integer(PChar(Str))+1;
        Index:=Result + Length(SubStr);
	end
else
	begin
	Index:=Length(Str)+1;
	Result:=0;
	end;
end;


function IsWhiteSpace(C: char): boolean;
{Returns True if specified character is "white space"}
const WhiteSpaceChars = [CR_CHAR, SP_CHAR, TAB_CHAR];
begin
Result:=C in WhiteSpaceChars;
end;



function IsExtendedAlpha(C: char): boolean;
{Return true if is extended ASCII character that functions as an Alpha i.e. ""}
const ExtendedChars = [#$C0..#$C6,#$C8..#$CF,#$D0..#$D6,#$D8..#$DF,#$E0..#$E6,#$E8..#$EF,#$F0..#$F6,#$F8..#$FF];
begin
Result:=C in ExtendedChars;
end;



function IsWordBreak(C: Char; CrossHyphen: boolean): boolean;
{Returns True if specified character is a word break}
{if CrossHyphen is true consider Hyphen part of word}
const WordChars = ['A'..'Z','a'..'z','0'..'9','_',''''];
begin
Result:=not (C in WordChars);
Result:=Result and not IsExtendedAlpha(C);
if CrossHyphen then Result:=Result and (C<>'-');
end;




function IsWordBreakEx(C,Adj: Char; CrossHyphen: boolean): boolean;
{Test for normal word break, plus differentiates between}
{appostrophy and single quote by looking at the adjacent char}
{if CrossHyphen is true considers Hyphen part of word}
begin
Result:=True;
if (C='''') and not IsAlphaNumeric(Adj) then exit;
Result:=IsWordBreak(C,CrossHyphen);
end;



function IsWholeWord(S: string; FirstInx,LastInx: integer): boolean;
{Returns true if word included in FirstInx and LastInx is a whole word}
var C1,C2,C3,C4: char;
begin
Result:=False;
if (FirstInx<1) or (LastInx>Length(S)) then exit;
if FirstInx>1 then C1:=S[FirstInx-1] else C1:=' ';
C2:=S[FirstInx];
C3:=S[LastInx];
if LastInx<Length(S) then C4:=S[LastInx+1] else C4:=' ';
Result:=IsWordBreakEx(C2,C1,False) and IsWordBreakEx(C3,C4,False);
end;



function IsAllDigits(S: string): boolean;
{Test to see if all chars in the string are digits}
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do
 if not IsDigit(S[I]) then exit;
Result:=True;
end;



procedure GetDigitCount(S: string; var Total, MaxContig: integer);
{Return the number of digits and maximum contiguous digits}
var I,T: integer;
begin
Total:=0;
MaxContig:=0;
T:=0;
for I:=1 to Length(S) do
 if IsDigit(S[I]) then
	begin
	Inc(Total);
	Inc(T);
	end
 else
	begin
	if T>MaxContig then MaxContig:=T;
	T:=0;
	end;
if T>MaxContig then MaxContig:=T;
end;




function RemoveMatchingStr(SubStr, S: string; MatchCase: boolean): string;
var Inx: integer;
begin
Result:=S;
if MatchCase then Inx:=Pos(SubStr,Result)
else Inx:=PosNoCase(SubStr,Result);
if Inx>0 then Delete(Result,Inx,Length(SubStr));
end;



function  PadLeftString(Str: string; Count: integer; C: char): string;
{ Pad left side of string to specified length, with specified char}
var I : Integer;
begin
Result:='';
for I:=1 to Count-Length(Str) do Result:=Result+C;
Result:=Result+Str;
end;



function PadRightString(Str: string; Count: integer; C: char): string;
{ Pad right side of string to specified length, with specified char}
var I: Integer;
begin
Result:=Str;
for I:=1 to Count-Length(Str) do Result:=Result+C;
end;



function  GetTabStrRight(Str:string; Tab: integer): string;
{ Return String Right Justified }
begin
Result:=PadLeftString(Str,Tab,SP_Char);
end;



function GetTabStrLeft(Str: string; Tab: integer): string;
{ Return String Left Justified }
begin
Result:=PadRightString(Str,Tab,SP_Char);
end;



function TabRightInteger(I,Tab: longint):string;
{ Return rigth justified integer }
begin
Result:=GetTabStrRight(IntToStr(I),Tab);
end;


function TabLeftInteger(I,Tab: longint):string;
{ Return left justified integer }
begin
Result:=GetTabStrLeft(IntToStr(I),Tab);
end;


function ExtractToken(S: string; Sep: char; var P: integer): string;
{Extract token from S, starting at P up to but not including Sep}
{Terminates with P pointing past Sep or past end of string}
var C: char;
begin
Result:='';
while P<=Length(S) do
	begin
	C:=S[P]; Inc(P);
	if C=Sep then break
	else Result:=Result+C;
	end;
end;


procedure ParseStringArgs(S: string; Sep,Quote: char; var SA: TStringDynArray);
{Parse string into arguments and put into a string array}
{ Converts multiple separators into empty fields}
{Sep = separator to between args, Ignores Sep in Quote chars}
var I: integer;
var InQuotes: boolean;
begin
InQuotes:=False;
for I:=1 to Length(S) do
	begin
	if (I<>1) and (S[I-1]=Quote) then InQuotes:=not InQuotes;
	if (S[I]<>Sep) or InQuotes then
		begin
		if (I=1) or (S[I-1]=Sep) and not InQuotes then
			begin
			SetLength(SA,Length(SA)+1);
            		SA[High(SA)]:='';
			end;
		SA[High(SA)]:=SA[High(SA)]+S[I];
		end
	else if ((I=1) or (S[I-1]=Sep)) and (S[I]=Sep) then
		{Two separators in a row is an empty field}
		{The character before the first character is alway}
		begin

		SetLength(SA,Length(SA)+1);
		SA[High(SA)]:='';
		end;
	end;
end;



procedure GetStringArgs(S: string; Sep,Quote: char; var SA: TStringDynArray);
{Parse string into arguments and put into a string array}
{Only includes non-empty strings, ingnores all separators}
{Sep = separator to between args, Ignores Sep in Quote chars}
var I: integer;
var InQuotes: boolean;
begin
InQuotes:=False;
for I:=1 to Length(S) do
	begin
	if (I<>1) and (S[I-1]=Quote) then InQuotes:=not InQuotes;
	if (S[I]<>Sep) or InQuotes then
		begin
		if (I=1) or (S[I-1]=Sep) and not InQuotes then
			begin
			SetLength(SA,Length(SA)+1);
            		SA[High(SA)]:='';
			end;
		SA[High(SA)]:=SA[High(SA)]+S[I];
		end;
	end;
end;




function ExtractToken(S: string; Sep: TASCIICharSet; var P: integer): string;
{Extract token from S, starting at P up to but not including Sep}
{Terminates with P pointing past Sep or past end of string}
var C: char;
begin
Result:='';
while P<=Length(S) do
	begin
	C:=S[P]; Inc(P);
	if C in Sep then break
	else Result:=Result+C;
	end;
end;


procedure ExtractWordsFromString(S: string; SL: TStringList);
{Extract individual words from a stings and place them in a list}
{Uses standard word-breaks criteria for finding words}
var WordStr: string;
var I,Inx: integer;
var O: TObject;
const TextChars = ['A'..'Z','a'..'z','0'..'9'];
begin
SL.Clear;
WordStr:='';
for I:=1 to Length(S) do
	begin
	if not (S[I] in TextChars) or IsExtendedAlpha(S[I]) then
		begin
		if (WordStr<>'') then
			begin
			SL.Add(WordStr);
			WordStr:='';
			end;
		end
	else WordStr:=WordStr+LowerChar(S[I]);
	end;
if (WordStr<>'') then SL.Add(WordStr);
end;



procedure GetArgsInBlock(S: string; Token1,Token2,Sep: char; var SL: TStringList);
{Get args inside block defined by Token1 and Token2 and separted by "Sep"}
var Inx1,Inx2,I: integer;
var ST,Arg: string;
begin
SL.Clear;
Inx1:=Pos(Token1,S);
Inx2:=Pos(Token2,S);
if (Inx1=0) or (Inx2=0) or (Inx1>=Inx2) then exit;
ST:=Copy(S,Inx1+1, (Inx2-Inx1)-1);
I:=1;
Arg:=ExtractToken(ST,',',I);
while Arg<>'' do
	begin
	SL.Add(Arg);
	Arg:=ExtractToken(ST,',',I);
	end
end;



function ReturnEnclosedBlock(S: string; Inx: integer; BeginToken,EndToken: char): string;
{Returns block enclosed by Begin and End Token - Handles nesting }
{Assumes index is pointing at begining token}
var I: integer;
var TC: integer;
begin
Result:='';
TC:=0;
for I:=Inx to Length(S) do
	begin
	Result:=Result+S[I];
	if S[I] = EndToken then
		begin
		Dec(TC);
		if TC<=0 then break;
		end
	else if S[I] = BeginToken then Inc(TC);
	end;
end;



function RemoveEnclosedText(S: string; BeginToken,EndToken: char): string;
{Remove text enclosed between the two token}
var I: integer;
var InBlock: boolean;
begin
InBlock:=False;
for I:=1 to Length(S) do
	begin
	if S[I]= BeginToken then InBlock:=True
	else if S[I]=EndToken then InBlock:=False;
	if not InBlock then Result:=Result + S[I];
	end;
end;


function GetEndOfBlock(S: string; Inx: integer; BeginToken,EndToken: char): integer;
{Returns final block enclosed by Begin and End Token - Handles nesting }
{Assumes index is pointing at begin token}
var I: integer;
var TC: integer;
begin
Result:=0;
TC:=0;
for I:=Inx to Length(S) do
	begin
	if S[I] = EndToken then
		begin
		Dec(TC);
		if TC<=0 then
			begin
			Result:=I;
			break;
			end;
		end
	else if S[I] = BeginToken then Inc(TC);
	end;
end;


procedure ReplaceChars(var S: string; Target,Replacement: Char);
{Replace all target chars with the Replacement}
var I: integer;
begin
for I:=1 to Length(S) do
 if S[I]=Target then S[I]:=Replacement;
end;

procedure ReplaceChars(var S: string; Target: TASCIICharSet; Replacement: Char);
{Replace all target chars with the Replacement}
var I: integer;
begin
for I:=1 to Length(S) do
 if S[I] in Target then S[I]:=Replacement;
end;


function ReplaceStrChars(S: string; Target,Replacement: Char): string;
{Replace all target chars with the Replacement}
begin
Result:=S;
ReplaceChars(Result,Target,Replacement);
end;



function ReplaceStr(const AText, AFromText, AToText: string): string;
{Replace FromText with AToStext in AText}
begin
Result := StringReplace(AText, AFromText, AToText, [rfReplaceAll]);
end;




function RemoveControls(var S: string): boolean;
{Remove control characters}
var P1,P2: integer;
begin
P2:=0;
for P1:=1 to Length(S) do
   if S[P1]>=#$20 then
	begin Inc(P2); S[P2]:=S[P1]; end;
Result:=P2<>Length(S);
SetLength(S,P2);
end;


function RemoveSpaces(const S: string): string;
{Remove space characters}
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do if S[I]<>#$20 then Result:=Result+S[I];
end;



function RemoveChars(const S: string; C: char): string;
{Remove all occurences of the specified character}
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do if S[I]<>C then Result:=Result+S[I];
end;



function RemoveNonAlphaNum(S: string): string;
{ Remove All Non AlphaNumeric From String }
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do
 if IsAlphanumeric(S[I]) then Result:=Result+S[I];
end;



function TrimNonNumerics(const S: string): string;
{Trim all non-numerics from beginning and end of string}
var I, L: Integer;
begin
L:=Length(S);
I:=1;
while (I <= L) and not IsDigit(S[I]) do Inc(I);
if I > L then Result:='' else
	begin
	while not IsDigit(S[L]) do Dec(L);
	Result:=Copy(S, I, L - I + 1);
	end;
end;


procedure EatLeadingChars(var S: string; const C: char);
{Remove leading chars from a string}
begin
while (Length(S)>0) and (S[1] = C) do Delete(S,1,1);
end;



procedure EatTrailingChars(var S: string; const C: char);
{Remove trailing char from a string}
begin
while (Length(S)>0) and (S[Length(S)] = C) do Delete(S,Length(S),1);
end;


procedure EatLeadingChars(var S: string; const CS: TASCIICharSet);
{Remove leading chars from a string}
begin
while (Length(S)>0) and (S[1] in CS) do Delete(S,1,1);
end;



procedure EatTrailingChars(var S: string; const CS: TASCIICharSet);
{Remove trailing char from a string}
begin
while (Length(S)>0) and (S[Length(S)] in CS) do Delete(S,Length(S),1);
end;




procedure EatLeadTrailChars(var S: string; const C: char);
{Remove trailing and leading char from a string}
begin
EatLeadingChars(S,C);
EatTrailingChars(S,C);
end;



procedure EatLeadTrailChars(var S: string; const CS: TASCIICharSet);
{Remove trailing and leading char from a string}
begin
EatLeadingChars(S,CS);
EatTrailingChars(S,CS);
end;



function EatTrailingDigits(S: string): string;
{Remove all trailing numerical digits from string}
begin
Result:=S;
while IsDigit(Result[Length(Result)]) do Delete(Result,Length(Result),1);
end;



procedure EatLeadingSpaces(var S: string);
{Remove leading spaces from a string}
begin
while Pos(' ',S)=1 do Delete(S,1,1);
end;



function MatchCount(S1,S2: string; NoCase: boolean): integer;
{Count the number of matching characters in S1 and S2}
var P,L: integer;
begin
Result:=0;
if NoCase then begin S1:=UpperCase(S1); S2:=UpperCase(S2); end;
if Length(S1)<Length(S2) then L:=Length(S1) else L:=Length(S2);
for P:=1 to L do
 if S1[P]=S2[P] then Inc(Result)
 else break;
end;



procedure WrapString(var S: string; BreakChar: char; Column: integer);
{Wrap string at specified break char past specifed column}
var I: integer;
begin
I:=1;
while true do
	begin
	Inc(I,Column);
	while true do
		begin
		if I>Length(S) then exit;
		if S[I]=BreakChar then
			begin
			Delete(S,I,1);
			Insert(CRLF_Char,S,I);
			break;
			end;
		Inc(I);
		end;
	end;
end;



function FillString(C: char; N: integer): string;
{Fill string with N number of the specified character}
var I: integer;
begin
Result:='';
for I:=1 to N do Result:=Result + C;
end;



function ExtractStringLine(var Source: string; var Destination: string; var Inx: integer): char;
{Fast routine for extracting lines from strings}
{Call with index set to 1 and then repeat call until all strings extracted}
{Index is updated as lines are found }
var P,Start: PChar;
var I: integer;
begin
P:=Pointer(Source);
Inc(P,Inx-1);
Start := P;
for I:=Inx to Length(Source) do
 if (P^ in [LF_Char, CR_Char]) then break else Inc(P);
SetString(Destination, Start, P - Start);
Result:=P^;
if P^ = CR_Char then Inc(P);
if P^ = LF_Char then Inc(P);
Inx:=Inx + (integer(P) - integer(Start));
end;



function CountCharMatches(S: string; C: Char): integer;
{Return the number of time C is found in string}
var I: integer;
begin
Result:=0;
for I:=1 to Length(S) do
 if S[I]=C then Inc(Result);
end;



{ ------------------ Math Routines ------------------------------------}



function IntegerIn(var IFile: TextFile): integer;
{ Read integer from text file}
var S: String[128];
begin
S:='';
Read(IFile,LastChar);
while (LastChar<'0') or (LastChar>'9') do
	begin
	if Eof(IFile) then begin Result:=0; exit; end;
	if (LastChar = '-') or (LastChar = '+') then S:=S+LastChar
	else S:='';
	Read(IFile,LastChar);
	end;
while (LastChar>='0') and (LastChar<='9') do
	begin
	S:=S+LastChar;
	Read(IFile,LastChar);
	if Eof(IFile) then break;
	end;
Result:=StrToInt(S);
end;




function RealIn(var IFile: TextFile): double;
{ Read a real from the input chanel }
var S: String[128];
begin
S:='';
Read(IFile,LastChar);
while ((LastChar<'0') or (LastChar>'9')) and (LastChar<>'.') do
	begin
	if Eof(IFile) then begin Result:=0; exit; end;
	if (LastChar = '-') or (LastChar = '+') then S:=S+LastChar
	else S:='';
	Read(IFile,LastChar);
	end;
while (LastChar>='0') and (LastChar<='9') or
	(LastChar='.') or (LastChar='E') or (LastChar='e') or
	(LastChar='+') or (LastChar='-') do
	begin
	S:=S+LastChar;
	Read(IFile,LastChar);
	if Eof(IFile) then break;
	end;
Result:=StrToFloat(S);
end;



function ExtractNumber(Str: string; var Ptr1: integer; var Num: double): boolean;
{Extract a number from a string starting at Ptr1 }
var Len: integer;
var NStr: string;
var DecFlg: boolean;
begin
Result:=False;
DecFlg:=False;
NStr:='';
Num:=0;
Len:=Length(Str);
if Len=0 then exit;
{ Look for leading +, -, . or digit}
while True do
	begin
	if Ptr1>Len then exit;
	case Str[Ptr1] of
	 '-': if (Length(NStr)=0) or (not DecFlg and (NStr[Length(NStr)]<>'-')) then NStr:=NStr+Str[Ptr1];
	 else if (Str[Ptr1]=IndSystemData.DecimalSeparator) and not DecFlg  then
		begin
		NStr:=NStr+Str[Ptr1];
		DecFlg:=True;
		end
	 else if IsDigit(Str[Ptr1]) then break
	 else
		begin
		NStr:='';
		DecFlg:=False;
		end;
	 end;
	Ptr1:=Ptr1+1;
	end;
Result:=True;

while (Ptr1<=Len) and
      (IsDigit(Str[Ptr1]) or
      (Str[Ptr1]=IndSystemData.DecimalSeparator) or
      (Str[Ptr1]=IndSystemData.ThousandSeparator)) do
	begin
	if Str[Ptr1]=IndSystemData.DecimalSeparator then
	  if DecFlg then break else DecFlg:=True;
	if Str[Ptr1]<>IndSystemData.ThousandSeparator then NStr:=NStr+Str[Ptr1];
	Inc(Ptr1);
	end;
if (Ptr1<=Len) then
 if (Str[Ptr1]='E') or (Str[Ptr1]='e') then
	begin
	NStr:=NStr+Str[Ptr1]; Inc(Ptr1);
	while (Ptr1<=Len) and (Str[Ptr1]='-') do
		begin
		if NStr[Length(NStr)]<>'-' then NStr:=NStr+Str[Ptr1];
		Inc(Ptr1);
		end;
	while (Ptr1<=Len) and (IsDigit(Str[Ptr1])) do
		begin
		NStr:=NStr+Str[Ptr1];
		Inc(Ptr1);
		end;
	end;

Num:=StrToFloat(NStr);
end;




function GetNumberValue(S: string; var Inx: integer; Def: double): double;
{ Get real from string, return 0 if invalid string}
var Tmp: double;
begin
if ExtractNumber(S,Inx,Tmp) then Result:=Tmp
else Result:=Def;
end;



function GetNumberValue(S: string): double;
{ Get real from string, return 0 if invalid string}
var P: integer;
var Tmp: double;
begin
P:=1;
if ExtractNumber(S,P,Tmp) then Result:=Tmp
else Result:=0;
end;



function IsValidFloatStr(S: string): boolean;
{ Verify string contains valid floating value }
var P:integer; Tmp: double;
begin
P:=1;
Result:=ExtractNumber(S,P,Tmp);
end;



function IsValidNumber(S: string): boolean;
{Test to see if string is exclusively a number}
var I,Dots: integer;
begin
Result:=False;
Dots:=0;
for I:=1 to Length(S) do
	begin
	{Leading +/- must be in first charater}
	if (I<>1) and (S[I] in ['+','-']) then exit;
	if S[I] = '.' then Inc(Dots);
	if Dots>1 then exit;
	if not (S[I] in ['0'..'9','.','+','-']) then exit;
	end;
Result:=True;
end;



function IsUnsigedIntegerStr(S: string): boolean;
{Test to see if string is exclusively an unsigned integer}
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do
	begin
	if not IsDigit(S[I]) then exit;
	end;
Result:=True;
end;



function IsIntegerStr(S: string): boolean;
{Test to see if string is exclusively an unsigned integer}
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do
	begin
	if (S[I] in ['+','-']) and (I<>1) then exit;
	if not IsDigit(S[I]) then exit;
	end;
Result:=True;
end;



function FloatToEngineerStr(D: extended; Digits: integer): string;
{Convert real value to Engineering notation string}
var Exp: integer;
begin
if IsInfinite(D) or IsNAN(D) then
	begin
	Result:=FloatToStr(D);
	exit;
	end;
Exp:=0;
if D<>0 then
	begin
	{Multiply number by one thousand until it's not a fraction (i.e. not < 1.0)}
	while abs(D)<1 do
		begin
		D:=D * 1000;
		Exp:=Exp-3;
		end;
	{Divide by one thousand until number is less than a thousand}
	while abs(D)>=1000 do
		begin
		D:=D / 1000;
		Exp:=Exp+3;
		end;
	end;
{Output number followed by exponent created by above notation adjustments}
Result:=FloatToStrF(D,ffFixed,19,Digits);
Result:=Result+FillString('0',Digits-18);
Result:=Result+'E';
if Exp>=0 then Result:=Result+'+' else Result:=Result+'-';
Result:=Result+PadInteger(Abs(Exp),3);
end;



procedure RealFormat(L,R: integer);
{Set real format }
begin
FLeft:=L; FRight:=R;
end;




procedure FloatToXPLStr(V: extended; var FS: string);
{Convert real to XPL formated string }
var Left,DP: integer;
var LS,RS: string;
begin
if FLeft<0 then
	begin
	{Handle engineering notation}
	FS:=FloatToEngineerStr(V,FRight);
	Left:=4;
	end
else if FLeft=0 then
	begin
	{Handle scientific notation}
	FS:=FloatToStrF(V,ffExponent,FRight+1,3);
	Left:=2;
	end
else
	begin
	{Handle fixed point notation}
	if ThousandsComma then FS:=FloatToStrF(V,ffNumber,18,FRight)
	else FS:=FloatToStrF(V,ffFixed,18,FRight);
	Left:=FLeft;
	end;
{Find position of decimal point}
DP:=Pos(IndSystemData.DecimalSeparator,FS);
{Split number into parts left and right of decimal point }
if DP=0 then
	begin
	{Handle case where there is no decimal point}
	LS:=FS; RS:='';
	end
else
	begin
	LS:=LeftStr(FS,DP-1);
	RS:=RightStr(FS,Length(FS)-(DP-1));
	end;

{Pad left of decimal with spaces and combine left and right }
FS:=PadLeftString(LS,Left,' ')+RS;
end;




function StrToFloatDef(const S: string; Default: Extended): Extended;
{Floating point version of Delphi's StrToIntDef}
var E: Integer;
begin
Val(S, Result, E);
if E <> 0 then Result := Default;
end;



function UnroundedFloat(V: extended; Digits: integer): string;
{Returnes float as string without rounding past decimal point}
var Inx,DPI: integer;
begin
Result:=FloatToStrF(V,ffFixed,25,25);
if Pos('E',Result)>0 then exit;
Inx:=Pos('.',Result);
if Inx=0 then exit;
if Digits=0 then DPI:=0 else DPI:=1;
Delete(Result,Inx+Digits+DPI,Length(Result));
end;


function GetXPLRealStr(V: double): string;
{Get a real as an XPL formated number }
begin
FloatToXPLStr(V,Result);
end;


function PadInteger(V: integer; Width: integer): string;
{ Add leading zeros to fill string to width}
{ Only works properly for positive numbers}
var I : Integer;
begin
Result:=IntToStr(V);
for I:=1 to Width - Length(Result) do Result:='0'+Result;
end;


function IntToStrThousands(Int: int64): string;
{Convert integer to string and add thousands separators}
var S: string;
var I,I1: integer;
begin
S:=IntToStr(Int);
for I:=1 to Length(S) do
	begin
	I1:=Length(S)-I;
	Result:=Result+S[I];
	if ((I1 mod 3)=0) and (I1<>0) then Result:=Result+',';
	end;
end;



procedure RealOut(var OFile: TextFile; V: double);
{Output a real as an XPL formated number }
var STmp: string;
begin
FloatToXPLStr(V,STmp);
Write(OFile,STmp);
end;


function Log(X: extended): extended;
{Calculate log 10 of X }
begin
Result:=Ln(X)/2.302585092994045680;
end;


function Sign(X: double) : double;
{ Return 1 or -1 depending on sign of X}
begin
if X < 0.0  then Result:= -1.0 else Result:= 1.0;
end;


function CopySign(V,S: double): double;
{Copy the sign of S into V and return the result}
begin
if S<0 then Result:=-V else Result:=V;
end;


function ArcTan2(Y,X:double) : double;
{ Return four quadrant ArcTangent }
begin
if X > 0.0 then Result:= ArcTan(Y/X)
else if X < 0.0  then  Result:= ArcTan(Y/X) + PI * Sign(Y)
else Result:= PI/2.0 * Sign(Y);
end;



function Power(X,Y:double): double;
{ X raise to the Y power }
begin
if X=0 then Result:=0
else
	begin
	Result:= Exp(Y * Ln(abs(X)));
	if X<0 then Result:=-Result;
	end;
end;



function AntiLog(X: double): double;
{Find anti log base 10}
begin
Result:=Exp(X * 2.30258509299404568401799145468436);
end;


function Mag3DVector(X,Y,Z:double): double;
{Calculate magnitude of a 3D vector}
begin
Result:=SQRT(X*X + Y*Y + Z*Z);
end;


function FindMin(V1,V2: double): double;
{Find minimum value}
begin
if V1<V2 then Result:=V1 else Result:=V2;
end;


function FindMax(V1,V2: double): double;
{Find maximum value}
begin
if V1>V2 then Result:=V1 else Result:=V2;
end;


function LinInterpolate(V1,V2,D: double): double;
{Linear interpolation}
begin
Result:=((V2-V1)*D)+V1;
end;


function NearestMultipleUp(V, N: double): double;
{Round up to the nearest multiple above}
var R: double;
begin
Result:=V;
if V=0 then exit;
R:=FloatMod(Abs(V),N);
if R=0 then exit;
if V < 0 then Result:= -(abs(V) - R)
else Result:=V + N - R;
end;



function NearestMultipleDown(V, N: double): double;
{Round down to the nearest multiple below}
var R: double;
begin
Result:=V;
if V=0 then exit;
R:=FloatMod(Abs(V),N);
if R=0 then exit;
if V >= 0 then Result:= abs(V) - R
else Result:=V - N + R;
end;





function NearestMultiple(V,N: double; Low: boolean): double;
{Return the nearest multiple to V, modulo N}
{Low controls whether the result is higher or lower then V}
begin
if Low then Result:=NearestMultipleDown(V,N)
else Result:=NearestMultipleUp(V,N);
end;



function NearestMultiple(V,N: double): double;
{Return the nearest multiple to V, modulo N}
begin
Result:=Round(V/N)*N;
exit;
if Frac(V/N)>0.5 then Result:=NearestMultipleUp(V,N)
else Result:=NearestMultipleDown(V,N);
end;



function FloatMod(X,Y: extended): extended;
{Return real remainder of X/Y }
begin
Result:= Frac(X/Y) * Y;
end;



function FloatMod2(X,Y: extended): extended;
{Return real remainder of X/Y }
begin
Result:=X - floor(X/Y) * Y;
end;


function DivMod(Numerator, Denominator: integer; var Remainder: integer): integer; overload;
{Do 32-bit divide returning dividend and remainder}
begin
Result:=Numerator div Denominator;
Remainder:= Numerator mod Denominator;
end;



function DivMod(Numerator, Denominator: Int64; var Remainder: Int64): Int64; overload;
{Do 64-bit divide returning dividend and remainder}
begin
Result:=Numerator div Denominator;
Remainder:= Numerator mod Denominator;
end;



function DivMod(Numerator, Denominator: extended; var Remainder: extended): integer; overload;
{Do floating point divide returning dividend and remainder}
begin
Result:=Trunc(Numerator / Denominator);
Remainder:= FloatMod(Numerator,Denominator);
end;



function DivMod(Numerator, Denominator: double; var Remainder: double): double; overload;
{Do floating point divide returning dividend and remainder}
begin
Result:=Trunc(Numerator / Denominator);
Remainder:= FloatMod(Numerator,Denominator);
end;


function ExtendSign(B: byte): integer;
{Extend sign bit of byte to integer}
begin
Result:=ShortInt(Lo(B));
end;


function IntPower2(P: integer): integer;
{Return integer power of two}
begin
if P<0 then Result:=0
else Result:=1 shl P;
end;




function IntExponent(A,B: integer): integer;
{Recursively raise A to B power}
begin
if B = 0 then Result:=1
else if (B mod 2) = 1 then Result:=A * IntExponent(A, B - 1)
else
	begin
        Result:=IntExponent(A, B div 2);
        Result:=Result * Result;
	end;
end;




function CenteredRandom(Range,Center: double): double;
begin
Result:=(Random - 0.5) * Range;
Result:=Result + Center;
end;



function GetRandomRange(Low,High: double): double;
var D: double;
begin
D:=High - Low;
Result:=(Random * D) + Low;
end;


function ClipFraction(V:double;N:integer): double;
{ Clip fraction part to N digits }
var S: string;
begin
S:=FloatToStrF(V,ffFixed,18,N);
Result:=StrToFloat(S);
end;


function ValidDateString(D: string): boolean;
{ Test for a valid date string }
begin
Result:=True;
try ConvertStrToDate(D);
 except Result:=False;
 end;
end;



procedure FixDate(var M,D,Y: integer);
{Repair defective date}
const DaysPerMonth: array[1..12] of Integer =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
var C: integer;
begin
if Y<=0 then Y:=1;
if M<=0 then M:=1;
if M>12 then M:=12;
C:=DaysPerMonth[M];
if (M=2) and (Y mod 4 = 0) and
   ((Y mod 100 <> 0) or (Y mod 400 = 0)) then C:=C+1;
if D>C then D:=C;
if D<=0 then D:=1;
end;




function GetShortDateEditMask: string;
{Get format to be used in "EditMask" based on ShortDateFormat }
var I: integer;
begin
Result:=ReplaceStr(DateToStr(EncodeDate(9999,11,11)),'1','9');
for I:=1 to Length(Result) do
 if IsAlpha(Result[I]) then Result[I]:='L';
end;




function ConvertStrToDate(S: string): TDate;
{Convert string into TDate using current Short Date Format}
var I,Cnt: integer;
var DS,FS: TStringList;
var Year,Month,Day: integer;


	function TwoToFourDigitYear(Year: integer): integer;
	var CenturyBase: Integer;
	begin
	Result:=Year;
	CenturyBase := CurrentYear - Formatsettings.TwoDigitYearCenturyWindow;
	Inc(Result, CenturyBase div 100 * 100);
	if (Formatsettings.TwoDigitYearCenturyWindow > 0) and (Result < CenturyBase) then
	Inc(Result, 100);
	end;




	function DecodeMonth(S: string; MonthName: array of string): integer;
	var I: integer;
	begin
	Result:=0;
	S:=LowerCase(S);
	for I:=Low(MonthName) to High(MonthName) do
	 if S=LowerCase(MonthName[I]) then
		begin
		Result:=I + 1;
		exit;
		end;
	end;



	procedure DecodeDateItem(Item, Format: string; var Year,Month,Day: integer);
	{Interpret a date item according to specified format}

	begin
	Format:=LowerCase(Format);
	if Pos('d',Format)>0 then Day:=StrToInt(Item)
	else if Format = 'm' then Month:=StrToInt(Item)
	else if Format = 'mm' then Month:=StrToInt(Item)
	else if Format = 'mmm' then Month:=DecodeMonth(Item,Formatsettings.ShortMonthNames)
	else if Format = 'mmmm' then Month:=DecodeMonth(Item,Formatsettings.LongMonthNames)
	else if Format = 'yy' then Year:=TwoToFourDigitYear(StrToInt(Item))
	else if Format = 'yyyy' then Year:=StrToInt(Item);
	end;


begin
DS:=TStringList.Create;
try
FS:=TStringList.Create;
try
Year:=1900; Month:=1; Day:=1;
{prepare lists to split up data}
DS.Delimiter:=Formatsettings.DateSeparator;
FS.Delimiter:=Formatsettings.DateSeparator;
{Split date string data into parts}
DS.DelimitedText:=S;
{Split format into parts}
FS.DelimitedText:=Formatsettings.ShortDateFormat;
Cnt:=DS.Count;
if FS.Count<Cnt then Cnt:=FS.Count;
{Parse the string according to specified format}
for I:=0 to Cnt-1 do
	begin
	DecodeDateItem(DS[I], FS[I], Year,Month,Day);
	end;
Result:=EncodeDate(Year,Month,Day);
finally FS.Free; end;
finally DS.Free; end;
end;




function GetMonthNumber(Date: TDate): integer;
{Return the number of months since 1900}
var Year, Month, Day: Word;
begin
DecodeDate(Date,Year, Month, Day);
Result:=Month+((Year - 1900) * 12);
end;



function GetMonthCount(D2,D1: double): integer;
{Get count of months between two dates}
begin
Result:=GetMonthNumber(D2)-GetMonthNumber(D1);
end;



{==============================================================================}




function RightZeroInteger(Value,Digit: int64): int64;
{Return value with Digit number of digits zeroed}
var D,I: integer;
begin
D:=1;
for I:=1 to Digit do D:=D * 10;
Result:=(Value div D) * D;
end;




function LeftTruncateInteger(Value,Digit: int64): int64;
{Truncate all but Digits number of digits on left}
var T: integer;
begin
T:=RightZeroInteger(Value,Digit);
Result:=Value - T;
end;


function GetIntegerDigit(Value,Digit: int64): int64;
{Return one integer digit specified by Digit}
{0 = the ones digit}
var I: integer;
begin
Result:=Value;
for I:=1 to Digit do Result:=Result div 10;
Result:=Result - ((Result div 10) * 10);
end;



function NormalizeRange(V,Min,Max: integer): integer; overload;
{Normalize a number with in the specified range}
{Wraps numbers around if they exeed the specified Min and Max}
var D: integer;
begin
D:=Max-Min;
Result:=((V-Min) mod D)+Min;
if Result<Min then Result:=Result+D;
end;



function NormalizeRange(V,Min,Max: double): double; overload;
{Normalize a number with in the specified range}
{Wraps numbers around if they exeed the specified Min and Max}
var D: double;
begin
D:=Max-Min;
Result:=FloatMod(V-Min,D)+Min;
if Result<Min then Result:=Result+D;
end;

{
FPU Busy =                      $8000
Condition Code C3 =             $4000
Top Of Stack Pointer B2 =       $2000
Top Of Stack Pointer B1 =       $1000
Top Of Stack Pointer B0 =       $0800
Condition Code C2 =             $0400
Condition Code C1 =             $0200
Condition Code C0 =             $0100
Error Summary Status =          $0080
Stack Fault =                   $0040
Precision Exception =           $0020
Underflow Exception =           $0010
Overflow Exception =            $0008
Zero Divide Exception =         $0004
Denormalized Operand Exception = $0002
Invalid Operation Exception =   $0001
}

function Get8087Status(Clear: Boolean): Word; assembler;
{Get 8087 Status Word. Clear status word if Clear is true}
asm
	TEST    AX,AX
	JE      @NoClr
	FNSTSW   AX
	FNINIT		{Clear cooprocessor}
	RET
@NoClr:	FNSTSW  AX
end;


procedure Clear8087Status; assembler;
{Clear 8087 status word}
asm
	FNINIT		{Clear cooprocessor}
end;



function FastFloor(f: double): integer;
{fast FPU floor function}
var T: single;
begin
T:=-0.5;
asm
	FLD	F
	FADD	ST,ST(0)
	FADD	T
	FISTP	Result
	SAR	Result, 1
	end;
end;


function FastCeil(F: double): integer;
{fast FPU ceiling function}
var T : double;
begin
T:=-0.5 ;
asm
	FLD	F
	FADD	ST,ST(0)
	FSUBR	T
	FISTP	Result
	SAR	Result,1
	end;
end;



function FastTrunc(F: double): integer;
{fast FPU Trunc function}
var T: double;
begin
T:=-0.5 ;
asm
	FLD	F
	FADD	ST,ST(0)
	FABS
	FADD	T
	FISTP	Result
	SAR	Result,1
end;
if F<0 then Result:=-Result;
end;



function FastRound(F: double): integer;
{fast FPU Round function}
var T: double;
begin
T:=0.5;
	asm
	FLD	F
	FADD	ST,ST(0)
	FADD	T
	FISTP	Result
	SAR	Result, 1
	end;
end;




function TriMin(a, b, c: Integer): Integer;
{Find minimum value of A,B, and C}
begin
if B < A then Result := B else Result := A;
if C < Result then Result := C;
end;




function WrappedDifference(V1,V2,Range: double): double;
{Find the shortest difference between V1 and V2, wrapped over Range}
var RawDiff,ModDiff: double;
begin
RawDiff:=PickFloat(V1 > V2,V1 - V2, V2 - V1);
ModDiff:=FloatMod(RawDiff, Range);
Result:=PickFloat(ModDiff > Range/2,Range - ModDiff, ModDiff);
end;



procedure ModuloRange(var X: integer; LowEnd,HighEnd: integer);
{Forcd X to be in the range between LowEnd and HighEnd}
{In same modulo of that range as in the original}
var Delta: integer;
begin
Delta:=HighEnd-LowEnd;
while X<LowEnd do Inc(X,Delta);
while X>=HighEnd do Dec(X,Delta);
end;



procedure ModuloRange(var X: double; LowEnd,HighEnd: double);
{Forcd X to be in the range between LowEnd and HighEnd}
{In same modulo of that range as in the original}
var Delta: double;
begin
Delta:=HighEnd-LowEnd;
while X<LowEnd do X:=X+Delta;
while X>=HighEnd do X:=X-Delta;
end;




function ModuloTruncate(V: integer; M: integer): integer;
{Truncate to the next lower modulo value}
begin
Result:=V-(V mod M);
end;



function IntToOctal(Value: integer): string;
const OctDigits: array[0..7] of AnsiChar = AnsiString('01234567');
begin
if Value = 0 then Result:='0'
else
	begin
	Result:='';
	while Value > 0 do
		begin
		Result:=OctDigits[Value and 7] + Result;
		Value := Value shr 3;
		end;
	end;
end;



function GetIntMagStr(V: int64; Digits: integer): string;
{Get integer is with KB, MB, GB type notation}
const Sizes: array [0..5] of String =('B','KB','MB','GB','TB','EB');
var I: integer;
var D,T,R: int64;
begin
T:=V;
D:=1;
for I:=0 to High(Sizes) do
	begin
	if T<1024 then break;
	D:=D shl 10;
	T:=T div 1024;
	end;
R:=V mod D;
Result:=FloatToStrF(V/D,ffFixed,18,Digits)+' '+Sizes[I];
end;

{------------------- IO Routines --------------------------------------}

function ValidFileChar(C: char): boolean;
{ Return True if valid filename character}
var P: integer;
var V: string[20];
begin
V:='._^$~!#%&-{}()`''';
Result:=False;
     if IsAlpha(C) then Result:=True
else if IsDigit(C) then Result:=True
else for P:=1 to Length(V) do if char(V[P])=C then
	begin Result:=True; Exit; end;
end;



function ValidateFilename(var S: string): boolean;
{ Convert S to valid filename}
var P,C: integer;
begin
Result:=False;
if Length(S)<1 then exit;
C:=0;
for P:=Length(S) downto 1 do
 if S[P]='.' then
	begin
	Inc(C);
	if C>1 then S[P]:='-';
	end;
P:=1;
while P<Length(S) do
	begin
	if not ValidFileChar(S[P]) then Delete(S,P,1)
	else Inc(P);
	end;
Result:=True;
end;



function IsFileOpen(var F): boolean;
{Is delphi style file open }
var M: word;
begin
Result:=False;
if Pointer(F)=nil then exit;
M:=TFileRec(F).Mode;
Result:=(M=fmInput) or (M=fmOutput) or (M=fmInOut);
end;


function IsFileAssigned(var F): boolean;
{Is delphi style file assigned }
var M: word;
begin
M:=TFileRec(F).Mode;
Result:=(M=fmInput) or (M=fmOutput) or (M=fmInOut) or (M=fmClosed);
end;


function IsFileInUse(FName: string): boolean;
{Returns true if file is open by any application}
var H: integer;
begin
{This gets a "platform" warning}
{$WARNINGS OFF}
H:=FileOpen(FName,fmShareDenyRead or fmOpenRead);
{$WARNINGS ON}
Result:=H<0;
FileClose(H);
end;


procedure FileCopy(const Source, Destin:String);
{Copy file from Source to Destin}
var S, T: TFileStream;
begin
try Screen.Cursor:=crHourGlass;
S:=TFileStream.Create(Source,fmOpenRead);
try
T:=TFileStream.Create(Destin,fmOpenWrite or fmCreate);
try T.CopyFrom(S,S.Size);
finally T.Free; end;
finally S.Free; end;
finally Screen.Cursor:=crDefault; end;
end;




function ClearRemoveFolder(const Path: string): Boolean;
{Delete all subfiles/subdirectories and remove directory}
var SHFileOpStruct: TSHFileOpStruct;
var DirBuf: array[0..255] of Char;
begin
Result:=False;
if not DirectoryExists(Path) then exit;
FillChar(SHFileOpStruct, Sizeof(SHFileOpStruct), 0);
FillChar(DirBuf, Sizeof(DirBuf), 0);
StrPCopy(DirBuf, Path);
SHFileOpStruct.Wnd := 0;
SHFileOpStruct.pFrom := @DirBuf;
SHFileOpStruct.wFunc := FO_DELETE;
SHFileOpStruct.fFlags := FOF_NOCONFIRMATION or FOF_SILENT;
Result := SHFileOperation(SHFileOpStruct) = 0;
end;


function GetFilePosition(var F:TextFile): Longint;
{Get position of file pointer for text file}
var H: word;
begin
H:=TTextRec(F).Handle;
Result:=FileSeek(H,0,1);
end;


function ExtractNamePart(FileName: string): string;
{Extract the name part of filename }
var I: Integer;
begin
Result:=ExtractFileName(FileName);
I := Length(Result);
while (I > 0) and not (Result[I] in ['.']) do Dec(I);
if (I > 0) then SetLength(Result,I-1);
end;



function RemoveDriveLetter(Path: string): string;
{Remove the drive letter and colon part of path }
begin
Result:=Path;
if (Length(Result)>1) and
   IsAlpha(Result[1]) and
   (Result[2]=':') then Delete(Result,1,2);
end;



function ExtractParentDirName(Path: string): string;
{Extract the parent directory name from path}
var Inx: integer;
begin
Result:=ExtractFilePath(Path);
Inx:=Pos(':',Result);
if Inx>0 then Delete(Result,1,Inx);
if (Length(Result)>0) and (Result[Length(Result)]='\') then Delete(Result,Length(Result),1);
Inx:=FindLastOccurrence(Result,'\');
if Inx>0 then Delete(Result,1,Inx);
end;



function ExtractParentPath(var Path: string): string;
{Extract the path to the parent directory including drive letter and all parent directories}
var Inx: integer;
begin
Result:=ExcludeTrailingBackslash(Path);
Inx:=FindLastOccurrence(Result,'\');
if Inx>0 then Delete(Result,Inx+1,(Length(Result)-Inx)+1);
if (Result='\\') or ((Length(Result)>0) and (Result[Length(Result)]=':')) then Result:=''
else Result:=IncludeTrailingBackSlash(Result);
end;



{DLL function headers}

function PathRelativePathTo(pszPath: PChar; pszFrom: PChar; dwAttrFrom: DWORD; pszTo: PChar; dwAtrTo: DWORD): LongBool; stdcall; external 'shlwapi.dll' name 'PathRelativePathToA';
function PathCanonicalize(lpszDst: PChar; lpszSrc: PChar): LongBool; stdcall; external 'shlwapi.dll' name 'PathCanonicalizeA';


function AbsPathToRelPath(const AbsPath, BasePath: string): string;
{Converts absolute path to relative path }
{by removing the BasePath part and replacing "..\" }
var Path: array[0..MAX_PATH-1] of char;
begin
PathRelativePathTo(@Path[0], PChar(BasePath), FILE_ATTRIBUTE_DIRECTORY, PChar(AbsPath), 0);
result := Path;
end;


function RelPathToAbsPath(const RelPath, BasePath: string): string;
{Convert Relative Path to Absolute path}
{Works by replacing "../" and "./" with appropirate parts of Base Path}
var Dst: array[0..MAX_PATH-1] of char;
begin
PathCanonicalize(@Dst[0], PChar(IncludeTrailingBackslash(BasePath) + RelPath));
Result := Dst;
end;


function NormalizeFilename(Filename,DefaultExt: string): string;
{Make sure file has correct extension and }
{the path the correct and fully expanded}
var Ext,Path: string;
begin
Result:=Trim(Filename);
Ext:=ExtractFileExt(Result);
if Ext='' then Result:=ChangeFileExt(Result,DefaultExt);
Path:=ExtractFilePath(Result);
if Path='' then Result:=ExpandFileName(Result);
end;





function ExpandEnvVars(const Str: string): string;
{Expand environmental variables in a string e.g. %LOCALAPPDATA%}
var BufSize: Integer;
begin
{ Get required buffer size}
BufSize := ExpandEnvironmentStrings(PChar(Str), nil, 0);
if BufSize > 0 then
	begin
	{ Read expanded string into result string}
	SetLength(Result, BufSize - 1);
	ExpandEnvironmentStrings(PChar(Str),PChar(Result), BufSize);
	Result:=Trim(Result);
	end
else Result := '';
end;



function GetFileSizeByName(const FileName: string): Int64;
{Find the size of a file without opening it}
{ Returns file size in bytes or -1 if not found}
var SR: TSearchRec;
begin
{This gets a "platform" warning}
{$WARNINGS OFF}
if FindFirst(Filename,faAnyFile,SR) = 0 then  Result:=Int64(SR.FindData.nFileSizeHigh) shl Int64(32) + Int64(SR.FindData.nFileSizeLow)
else result := -1;
{$WARNINGS ON}
FindClose(SR) ;
end;



procedure ChangeDefaultDir(Name: string);
{Change default directory based on fully qualified filename}
var Path: string;
var Len: integer;
begin
Path:=ExtractFilePath(Name);
Len:=Length(Path);
if Len<1 then exit;
if Path[Len]='\' then Delete(Path,Len,1);
ChDir(Path);
end;



function GetDefaultDir: string;
{Get current default Directory}
begin
GetDir(0,Result);
end;



function FileDateTime(const FileName: string): System.TDateTime;
{Get the time/date for specified file }
begin
Result := FileDateToDateTime(FileAge(FileName));
end;




function CreateUniqueFilename(Path,Name,Ext: string): string;
{Create a Unique filane using Path, Name and Ext}
var Inx: integer;
begin
Inx:=1;
repeat
	begin
	Result:=Path+Name+IntToStr(Inx)+'.'+Ext;
	Inc(Inx);
	end
until not FileExists(Result);
end;


procedure OpenWindowsExplorer(Path: string);
{Open Windows Explorer at the specified path}
begin
ShellExecute(Application.Handle, PChar('explore'), PChar(Path), nil, nil, SW_SHOWNORMAL);
end;





procedure OpenDOSWindow(Path: string);
{Open DOS prompt at the specified path}
begin
ShellExecute(Application.Handle, 'open',PChar('cmd.exe'), PChar(Path), nil, SW_SHOWNORMAL);
end;



function GetIniPathName(SubDir: string; Create: boolean): string;
{Get the best path name for an INI file based OS type}
var Version: TOSVersionInfo;
begin
if Length(SubDir)>0 then
	begin
	if SubDir[1]<>'\' then SubDir:='\'+SubDir;
	if SubDir[Length(SubDir)]<>'\' then SubDir:=SubDir+'\';
	end;
Version.dwOSVersionInfoSize:=SizeOf(Version);
GetVersionEx(Version);
if Version.dwPlatformId>=VER_PLATFORM_WIN32_NT then
	begin
	{Handle NT, XP, Vista, Win7, etc.}
	Result:=GetEnvironmentVariable('APPDATA')+SubDir;
	CreateDir(Result);
	end
else
	begin
	{Handle Win98 and below}
	Result:=GetEnvironmentVariable('windir')+'\';
	end;
end;


procedure  SpaceOut(var F:TextFile;Num:integer);
{ Output A Specified Number Of Spaces }
var C : Integer;
begin
for C:=1 to NUM do Write(F,SP_CHAR);
end;


procedure CRLFOut(var OFile: TextFile);
{ Send CRLF to file }
begin
Write(OFile, CR_CHAR, LF_CHAR);
end;


procedure  TabStringOut(var OFile: TextFile;Str:string; TabCount:integer);
{Output String With Tabulated Spaces }
var I : Integer;
var STmp: string;
begin
STmp:='';
for I:=1 to TabCount-Length(STR) do STmp:=STmp+' ';
Write(OFile,STmp,Str);
end; { Of Tabstr }



function IntToBin(IValue: Int64; NumBits: word; LeadZero: boolean) : string;
{Convert integer to binary string}
var I: integer;
begin
Result:='';
I:=IntPower2(NumBits-1);
while I <> 0 do
	begin
	if (IValue and I)<>0 then Result:=Result + '1'
	else if LeadZero or (Length(Result)>0) then Result:=Result + '0';
	I:=I shr 1;
	end;
if Result='' then Result:='0';
end;





function BinToInt(Value: string): integer;
var I, Count: integer;
begin
Result := 0;
Count:=Length(Value);
 for i := Count downto 1 do
    if Value[I] = '1' then Result := Result + (1 shl (Count - i));
end;



var DotCount: integer;

function DottedString(MaxDots: integer): string;
{Create a dotted string that increases with every call}
{MaxDots specifies the number of dots before it starts over}
var I: integer;
begin
Inc(DotCount);
Result:='';
for I:=1 to DotCount mod MaxDots do
 Result:=Result+'.';
end;


function CountColums(S: string; TabSpaces: integer): integer;
{Count columns expanding tabs}
var I: integer;
begin
Result:=0;
for I:=1 to Length(S) do
	begin
	if (S[I]=TAB_CHAR) then Result:=Result + (TabSpaces - (Result mod TabSpaces))
	else Inc(Result);
	end;
end;



function DeleteSubstring(Substring: string; var S: string; CaseSensitive: boolean): boolean;
{Find target in specified string and delete if found}
var I: integer;
begin
Result:=False;
if CaseSensitive then I:=Pos(Substring,S)
else I:=PosNoCase(Substring,S);
if I=0 then exit;
Delete(S,I,Length(Substring));
Result:=True;
end;



function DeleteAllSubStrings(Substring: string; var S: string; CaseSensitive: boolean): integer;
{Delete all substrings matching the specified substring}
{Returns a count of the number deleted}
begin
Result:=0;
while DeleteSubstring(Substring, S, CaseSensitive) do Inc(Result);
end;



procedure DeleteRange(var S: string; Inx1,Inx2: integer);
{Delete chars in string between Inx1 and Inx2 inclusive}
begin
Delete(S,Inx1,Inx2-Inx1+1);
end;



function CopyRange(var S: string; Inx1,Inx2: integer): string;
{Copy chars in string between Inx1 and Inx2 inclusive}
begin
Result:=Copy(S,Inx1,Inx2-Inx1+1);
end;



function EditDistance(Target, Test: string): Integer;
{ Compute Levenshtein distance between two strings }
var TgtSize, TstSize, I, J, Cost: integer;
var Matrix: array [0..255,0..255] of integer;
begin
TgtSize:= Length(Target);
TstSize:= Length(Test);
if TgtSize = 0 then begin Result:= TstSize; exit; end;
if TstSize = 0 then begin Result:= TgtSize; exit; end;

for I:= 0 to TgtSize do Matrix[I, 0]:= I;
for J:= 0 to TstSize do Matrix[0, J]:= J;

for I:= 1 to TgtSize do
	begin
	for J:= 1 to TstSize do
		begin
		if Target[I] = Test[J] then Cost:= 0 else Cost:= 1;
		Matrix[I, J]:= TriMin(Matrix[I - 1,J] + 1, Matrix[I,J - 1] + 1, Matrix[I-1,J-1] + Cost);
		end;
	end;
Result:= Matrix[TgtSize, TstSize];
end;



function IsOutputRedirected: boolean;
{Test if STDOOUT console output is redirect}
var hOutput: THandle;
var FileType: dword;
begin
hOutput:=GetStdHandle(STD_OUTPUT_HANDLE);
FileType:=GetFileType(hOutput);
Result:=FileType = FILE_TYPE_DISK;
end;




function IsInputRedirected: boolean;
{Test if STDIN console input is redirect}
var hInput: THandle;
var FileType: dword;
begin
hInput:=GetStdHandle(STD_INPUT_HANDLE);
FileType:=GetFileType(hInput);
Result:=(FileType = FILE_TYPE_DISK) or (FileType = FILE_TYPE_PIPE);
end;



function GetUsedDiskSpace(Path: string; var Used: Int64): boolean;
{Get the number of bytes used on a disk}
var TotalSpace, Available : Int64;
var TotalFree: TLargeInteger;
begin
Result:=GetDiskFreeSpaceEx(PChar(Path), Available, TotalSpace, @TotalFree);
if Result then Used:=TotalSpace-TotalFree
else Used:=0;
end;


{------------------ Time/degree routines ----------------------------}


procedure HourToHMSM(Hours: extended; var Hour,Min,Sec,MSec: integer);
{Converts Time to Hours, Minutes, Sec, and MSec integers}
{Time is expressed as a real number of hours  }
var Rem: integer;
var Negative: boolean;
begin
Negative:=Hours<0;
if Negative then Hours:=-Hours;
Hour:=DivMod(round(Hours*3600000), 3600000, Rem);
Min:=DivMod(Rem, 60000, Rem);
Sec:=DivMod(Rem, 1000,MSec);
if Negative then
	begin
	if Hour<>0 then Hour:=-Hour
	else if Min<>0 then Min:=-Min
	else if Sec<>0 then Sec:=-Sec
	else MSec:=-MSec;
	end;
end;


function HourMinuteSecStr(Hours: extended; DoDegrees,DoMilliseconds: boolean; Digits: integer): string;
{Return hour, minute and second string.  If DoDegrees is true then return Degrees minutes seconds}
{If DoMilliseconds is true display Milliseconds. Digits controls decimals for seconds.}
var Hour,Min,Sec,MSec: integer;
var S,S1,S2,S3,DS: string;
var Negative: boolean;
begin
HourToHMSM(Hours,Hour,Min,Sec,MSec);
Negative:=(Hour<0) or (Min<0) or (Sec<0) or (MSec<0);
Hour:=Abs(Hour); Min:=Abs(Min); Sec:=Abs(Sec); MSec:=Abs(MSec);
if DoDegrees then begin S1:=#$B0; S2:='''';  S3:='"';end
else begin S1:=':'; S2:=':'; S3:=''; end;
if Negative then Result:='-' else Result:='';
Result:=Result+IntToStr(Hour)+S1+PadInteger(Min,2)+S2+PadInteger(Sec,2);
if DoDegrees then
	begin
	DS:= FloatToStrF(MSec/1000,ffFixed,18,18);
	Delete(DS,1,2);
	if Digits>0 then Result:=Result + '.' + LeftStr(DS,Digits);
	Result:=Result+S3;
	end
else
	begin
	if DoMilliseconds then Result:=Result + ' ' + PadInteger(MSec,3)
	else if Digits>0 then
		begin
		S:=FloatToStrF(MSec/1000,ffFixed,0,Digits);
                Delete(S,1,1);
		Result:=Result+ S;
		end;
	end;
end;


function DaysHourMinuteSecStr(Hours: extended; DoMilliseconds: boolean; Digits: integer): string;
{Return Days hour, minute and second string.}
{If DoMilliseconds is true display Milliseconds. Digits controls decimals for seconds.}
var Days: integer;
var Rem: extended;
begin
Days:=DivMod(Hours,24,Rem);
Result:=IntToStr(Days)+' Days '+HourMinuteSecStr(Rem,False,DoMilliseconds,Digits);
end;



{------------------------------------------------------------------------------}


function GetHexDumpStr(var Buffer: array of byte; LabelOffSet: integer = 0): string;
{Dump contents of byte buffer as both Hex and ASCII}
var I: integer;
var S0,S1,S2: string;
const SpecialChars = [0,$09];
begin
Result:='';
for I:=0 to High(Buffer) do
	begin
	if (I mod 16) = 0 then
		begin
		if I<>0 then
			begin
			Result:=Result+S0+' '+S1+' '+S2+CRLF_Char;
			end;
		S0:=IntToHex(I+LabelOffset,8);
		S1:=''; S2:='';
		end;
	S1:=S1+IntToHex(Buffer[I],2)+' ';
	S2:=S2+PickString((Buffer[I]<$20) or (Buffer[I]>$7F),' ',Char(Buffer[I]));
	end;
{Fill in the last row if it not an even multiple of 16}
for I:=Length(S2) to 16-1 do
	begin
	S1:=S1 + '.. ';
	S2:=S2 + '.';
	end;
Result:=Result+S0+' '+S1+' '+S2+CRLF_Char;
end;


function GetHexDumpStr(P: PByteArray; Size: integer; LabelOffSet: integer = 0): string;
var Buffer: array of byte;
var I: integer;
begin
SetLength(Buffer,Size);
for I:=0 to High(Buffer) do Buffer[I]:=P[I];
Result:=GetHexDumpStr(Buffer,LabelOffset);
end;



function GetHexDumpStr(S: string; LabelOffSet: integer = 0): string;
var Buffer: array of byte;
var I: integer;
begin
SetLength(Buffer,Length(S));
for I:=1 to Length(S) do Buffer[I-1]:=Byte(S[I]);
Result:=GetHexDumpStr(Buffer,LabelOffset);
end;



function IntToHexBytes(V: cardinal; MinBytes: integer): string;
{Returns Hex string, in least number of bytes digits }
{MinBytes specifies the minimum number bytes digits}
var Nibbles,MinNibbles,I: integer;
const Two8 = 256; Two16 = 65536; Two24 = 16777216;
begin
MinNibbles:=MinBytes * 2;
if V>=Two24 then Nibbles:=8
else if V>=Two16 then Nibbles:=6
else if V>=Two8 then Nibbles:=4
else Nibbles:=2;
if Nibbles<MinNibbles then Nibbles:=MinNibbles;
Result:=IntToHex(V,Nibbles);
end;




procedure RaiseLastOSErrorMsg(Msg: string);
var LastError: Integer;
var Error: EOSError;
begin
LastError := GetLastError;
if LastError <> 0 then Error := EOSError.CreateResFmt(@SOSError, [LastError, SysErrorMessage(LastError)])
else Error := EOSError.CreateRes(@SUnkOSError);
Error.Message:=Msg+': '+Error.Message;
Error.ErrorCode := LastError;
raise Error;
end;



function SystemIdleTime: DWord;
{Return the since last mouse movement or key input}
var liInfo: TLastInputInfo;
begin
liInfo.cbSize := SizeOf(TLastInputInfo) ;
GetLastInputInfo(liInfo) ;
Result := (GetTickCount - liInfo.dwTime) div 1000;
end;




procedure SetProcessorAfinity(NewMask: cardinal);
{Set processor affinity for current process}
var ProcAFMask: DWORD_PTR;
var SysAFMask: DWORD_PTR;
begin
GetProcessAffinityMask(GetCurrentProcess, ProcAFMask, SysAFMask);
if SysAFMask<>NewMask then
	begin
	SysAFMAsk := NewMask;
	SetProcessAffinityMask(GetCurrentProcess, SysAFMAsk);
	end;
end;



{----------------- Graphics Routines --------------------------------}


function GetLastPixelPos(X1,Y1: integer; var X2,Y2: integer): boolean;
{Calculate the position of the last pixel for a line running from X1,Y1, to X2,Y2}
var DX,DY: integer;
begin
Result:=False;
DX:=X2-X1; DY:=Y2-Y1;
if (DX=0) and (DY=0) then exit;
if abs(DX)>abs(DY) then
	begin
	if DX=0 then Inc(Y2,round(DY div abs(DY)))
	else begin Inc(X2,round(DX div abs(DX))); Inc(Y2,round(DY div abs(DX))); end;
	end
else
	begin
	if DY=0 then Inc(X2,round(DX div abs(DX)))
	else begin Inc(Y2,round(DY div abs(DY))); Inc(X2,round(DX div abs(DY))); end;
	end;
Result:=True;
end;




procedure LineToLastPixel(H: HDC; X1,Y1,X2,Y2: integer);
{Draws a line from X1,Y1 to X2,Y2 - including the last pixel}
begin
if not GetLastPixelPos(X1,Y1,X2,Y2) then exit;
MoveToEx(H,X1,Y1,Nil);
LineTo(H,X2,Y2);
end;

{----------------- Delphi VCL Routines --------------------------------}


procedure ScaleGrid(Grid: TCustomGrid; M,D: integer);
{Scale rows and column sizes of a grid}
var I:integer;
var M2,D2: double;
begin
M2:=TForm(Grid.Owner).Canvas.TextHeight('0');
D2:=M2*D/M;
with TDrawGrid(Grid) do
 begin
 for I:=0 to ColCount-1 do
	begin
	ColWidths[I]:=round(ColWidths[I]*M2/D2);
	end;
 for I:=0 to RowCount-1 do
	begin
	RowHeights[I]:=round(RowHeights[I]*M2/D2);
	end;
 end;
end;


procedure CenterScrollBox(SB: TScrollBox);
{Center scrollbar positions}
begin
SB.HorzScrollBar.Position:=(SB.HorzScrollBar.Range-SB.ClientWidth) div 2;
SB.VertScrollBar.Position:=(SB.VertScrollBar.Range-SB.ClientHeight) div 2;
end;


{------------------------------------------------------------------------------}




{ TStringPairs }

constructor TStringPairs.Create;
begin
inherited;
Strings1:=TStringList.Create;
Strings2:=TStringList.Create;
Delimit1:=',';
Delimit2:=':';
TrimChars:=['"'];
Trim:=True;
end;


destructor TStringPairs.Destroy;
begin
Strings1.Free;
Strings2.Free;
inherited;
end;



procedure TStringPairs.ParseText(const Value: string);
var I,Inx: integer;
var SA: array of String;
var ST,S1,S2: string;
begin
FText := Value;
SetLength(SA,0);
{Get individual string-pairs}
I:=1;
while true do
	begin
	ST:=ExtractToken(Value,Delimit1,I);
	if ST='' then break;
	SetLength(SA,Length(SA)+1);
	SA[High(SA)]:=ST;
	end;
Strings1.Clear;
Strings2.Clear;
{Get individual string}
for I:=0 to High(SA) do
	begin
	Inx:=Pos(Delimit2,SA[I]);
	S1:=Copy(SA[I],1,Inx-1);
	S2:=Copy(SA[I],Inx+1,Length(SA[I])-Inx);
	if Trim then
		begin
		EatLeadTrailChars(S1,TrimChars);
		EatLeadTrailChars(S2,TrimChars);
		end;
	Strings1.Add(S1);
	Strings2.Add(S2);
	end;
end;



function TStringPairs.GetCount: integer;
begin
Result:=Strings1.Count;
end;




initialization
GetFormatSettings;
CPUSpeed:=GetProcessorSpeed;
SetInternationalFoot(True);
FLeft:=18;
FRight:=2;
end.

