unit FileLibrary;

interface

uses Windows,Classes,SysUtils;


type TProgressProc=procedure(Percent: integer);

procedure RelaxFileAttributes(FName: string);
function BackupFile(BaseName,Extension: string): boolean;
procedure MultiLevelBackup(BaseName,Extension: string; Levels: integer);
function GetFileAttributeStr(Attrib: cardinal): string;
function SecureErase(FileName: string; Progress: TProgressProc = nil): boolean;

implementation



procedure AddCommaListItem(var S: string; Item: string);
{Add item to comma delimited list}
begin
if Length(S)>0 then S:=S+', ';
S:=S+Item;
end;



function GetFileAttributeStr(Attrib: cardinal): string;
begin
Result:='';
if (Attrib and FILE_ATTRIBUTE_ARCHIVE)<>0 then AddCommaListItem(Result,'Archive');
if (Attrib and FILE_ATTRIBUTE_COMPRESSED)<>0 then AddCommaListItem(Result,'Compressed');
if (Attrib and FILE_ATTRIBUTE_DIRECTORY)<>0 then AddCommaListItem(Result,'Directory');
if (Attrib and FILE_ATTRIBUTE_HIDDEN)<>0 then AddCommaListItem(Result,'Hidden');
if (Attrib and FILE_ATTRIBUTE_NORMAL)<>0 then AddCommaListItem(Result,'Normal');
if (Attrib and FILE_ATTRIBUTE_OFFLINE)<>0 then AddCommaListItem(Result,'Offline');
if (Attrib and FILE_ATTRIBUTE_READONLY)<>0 then AddCommaListItem(Result,'Readonly');
if (Attrib and FILE_ATTRIBUTE_SYSTEM)<>0 then AddCommaListItem(Result,'System');
if (Attrib and FILE_ATTRIBUTE_TEMPORARY)<>0 then AddCommaListItem(Result,'Temporary');
end;



procedure RelaxFileAttributes(FName: string);
{Remove the ReadOnly attribute from a file}
{ and set Archive attribute because it has been modified}
var Attrib: integer;
begin
Attrib:=FileGetAttr(FName);
Attrib:=Attrib and (not faReadOnly);
Attrib:=Attrib or faArchive;
FileSetAttr(FName,Attrib);
end;




function BackupFile(BaseName,Extension: string): boolean;
{ Covert old file name to backup name}
{ Extension does not require dot}
var BackupName: string;
begin
Result:=False;
if (Length(Extension)>0) and (Extension[1]<>'.') then Insert('.',Extension,1);
if FileExists(BaseName) then
	begin
	BackupName:=ChangeFileExt(BaseName,Extension);
	if FileExists(BackupName) then
		begin
		RelaxFileAttributes(BackupName);
		if not DeleteFile(BackupName) then raise Exception.Create('Cannot Delete Old Backup File');
		end;
	RelaxFileAttributes(BaseName);
	if not RenameFile(BaseName,BackupName) then raise Exception.Create('Cannot Rename File');
	Result:=True;
	end;
end;



procedure MultiLevelBackup(BaseName,Extension: string; Levels: integer);
{Do multiple level backup Extension does not require dot}
var I: integer;
var Ext1,Ext2,FileTmp: string;
begin
if (Length(Extension)>0) and (Extension[1]<>'.') then Insert('.',Extension,1);
for I:=Levels downto 2 do
	begin
	Ext1:=Extension+IntToStr(I-1);
	Ext2:=Extension+IntToStr(I);
	FileTmp:=ChangeFileExt(BaseName,Ext1);
	BackupFile(FileTmp,Ext2);
	end;
BackupFile(BaseName,Ext1);
end;



function SecureErase(FileName: string; Progress: TProgressProc = nil): boolean;
{Do secure erase by filling file with random data}
var Buffer: array [0..4095] of Byte;
var NewName: string;
var Max, N: LongInt;
var i: Integer;
var fs: TFileStream;


	procedure RandomizeBuffer;
	var i: Integer;
	begin
	for I := Low(Buffer) to High(Buffer) do
	 buffer[I] := Random(256);
	end;


begin
try
Result:=False;
FS := TFilestream.Create(FileName, fmOpenReadWrite or fmShareExclusive);
try
RandomizeBuffer;
max := fs.Size;
fs.Position := 0;
while max > 0 do
	begin
	if max > SizeOf(buffer) then n := SizeOf(buffer)
	else n := max;
	fs.Write(Buffer, n);
	max := max - n;
	if Assigned(Progress) then Progress(MulDiv(100,FS.Size - Max,FS.Size));
	end;
FlushFileBuffers(fs.Handle);
finally fs.Free; end;
NewName:=ExtractFilePath(Filename)+'azbycxdw.evfu';
RenameFile(FileName,NewName);
Result:=Deletefile(NewName);
except end;
end;



end.
