unit FirewallLib;

interface

uses Classes, SysUtils, ActiveX, ComObj, Variants, DSupport;

const
   NET_FW_SCOPE_ALL = 0;
   NET_FW_SCOPE_LOCAL_SUBNET = 1;
   NET_FW_SCOPE_CUSTOM = 2;
   NET_FW_SCOPE_MAX = 3;

const
   NET_FW_PROFILE_DOMAIN = 0;
   NET_FW_PROFILE_STANDARD = 1;
   NET_FW_PROFILE_CURRENT = 2;
   NET_FW_PROFILE_TYPE_MAX = 3;



const
  NET_FW_IP_VERSION_V4 = 0;
  NET_FW_IP_VERSION_V6 = 1;
  NET_FW_IP_VERSION_ANY = 2;
  NET_FW_IP_VERSION_MAX = 3;


const
   NET_FW_IP_PROTOCOL_TCP = 6;
   NET_FW_IP_PROTOCOL_UDP = 17;


const
  NET_FW_SERVICE_FILE_AND_PRINT = 0;
  NET_FW_SERVICE_UPNP = 1;
  NET_FW_SERVICE_REMOTE_DESKTOP = 2;
  NET_FW_SERVICE_NONE = 3;
  NET_FW_SERVICE_TYPE_MAX = 4;




type TPortType =(ptTCP,ptUDP);



procedure EnableFireWall(Enable: boolean);
function IsFireWallEnabled: Boolean;
function IsAppRulePresent(const Filename: string) : Boolean;
function IsIncomingPingAllowed: Boolean;
procedure SetIncomingPingAllowed(Value: Boolean);
function GetRemoteAdminAdress: string;
procedure SetRemoteAdminAdress(Value: string);
function IsRemoteAdminAllowed: boolean;
procedure SetRemoteAdminAllowed(Value: Boolean);
procedure SetICMPSetting(Redirect: boolean);
procedure ModifyFirewallProperties(NoExceptions,NoNotifications,NoMulticaseResponse: boolean);
function AreExceptionsAllowed: boolean;
procedure OpenFirewallPort(Port: integer;  PortType: TPortType; Enable: boolean);
procedure RestoretheDefaultSettings(List: TStrings);
procedure AddPortToFirewall(EntryName: string; PortType: TPortType; PortNumber: Cardinal);
procedure AddApplicationToFirewall(EntryName: string; Filename: string);
procedure DeleteApplicationFromFirewall(const Filename: string);

procedure ListAuthorizedApps(List: TStrings);
procedure ListGlobalOpenPorts(List: TStrings);
procedure ListFirewallProperties(List: TStrings);
procedure ListFirewallServiceProperties(List: TStrings);
procedure ListICMPSettings(List: TStrings);
procedure ListRemoteAdministrationSettings(List: TStrings);
procedure ListStandardProfileProperties(List: TStrings);



implementation


function GetBooleanStr(B: boolean): string;
begin
Result:=PickString(B,'Yes','No');
end;




function ScopeToStr(Scope: integer): string;
begin
case Scope of
 NET_FW_SCOPE_ALL: Result:='All';
 NET_FW_SCOPE_LOCAL_SUBNET: Result:='Local Subnet';
 NET_FW_SCOPE_CUSTOM: Result:='Custom';
 NET_FW_SCOPE_MAX: Result:='Max';
 else Result:='Unknown';
 end;
end;



function VersionToStr(Version: integer): string;
begin
case Version of
 NET_FW_IP_VERSION_V4: Result:='V4';
 NET_FW_IP_VERSION_V6: Result:='V6';
 NET_FW_IP_VERSION_ANY: Result:='Any';
 NET_FW_IP_VERSION_MAX: Result:='Max';
 else Result:='Unknown';
 end;
end;



function ProtocolToStr(Proto: integer): string;
begin
case Proto of
 NET_FW_IP_PROTOCOL_TCP: Result:='TCP';
 NET_FW_IP_PROTOCOL_UDP: Result:='UDP';
 else Result:='Unknown';
 end;
end;



function ProfileTypeToStr(Profile: integer): string;
begin
case Profile of
 NET_FW_PROFILE_DOMAIN: Result:='Domain';
 NET_FW_PROFILE_STANDARD: Result:='Standard';
 NET_FW_PROFILE_CURRENT: Result:='Current';
 NET_FW_PROFILE_TYPE_MAX: Result:='Max';
 else Result:='Unknown';
 end;
end;


function ServiceTypeToStr(Service: integer): string;
begin
case Service of
 NET_FW_SERVICE_FILE_AND_PRINT: Result:='File and Print';
 NET_FW_SERVICE_UPNP: Result:='UPNP';
 NET_FW_SERVICE_REMOTE_DESKTOP: Result:='Remote Desktop';
 NET_FW_SERVICE_NONE: Result:='None';
 NET_FW_SERVICE_TYPE_MAX: Result:='Max';
 else Result:='Unknown';
 end;
end;




{----------- Routines to set, clear and read Firewall state -------------------}


procedure EnableFireWall(Enable: boolean);
{Enable/Disable Firewall for current profile}
var objFirewall, objPolicy: OleVariant;
begin
objFirewall:= CreateOLEObject('HNetCfg.FwMgr');
objPolicy:= objFirewall.LocalPolicy.CurrentProfile;
objPolicy.FirewallEnabled:=Enable;
end;



function IsFireWallEnabled: Boolean;
var objFirewall, objPolicy: OleVariant;
begin
objFirewall:= CreateOLEObject('HNetCfg.FwMgr');
objPolicy:= objFirewall.LocalPolicy.CurrentProfile;
Result := objPolicy.FirewallEnabled;
end;



function IsAppRulePresent(const Filename: string) : Boolean;
var App: OleVariant;
var fwMgr: OleVariant;
var profile: OleVariant;
begin
fwMgr := CreateOLEObject('HNetCfg.FwMgr');
profile := fwMgr.LocalPolicy.CurrentProfile;

App := profile.AuthorizedApplications.Item(Filename);
Result :=App <> unassigned;
end;




function IsIncomingPingAllowed: Boolean;
var objFirewall,objPolicy: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
Result := objPolicy.IcmpSettings.AllowInboundEchoRequest;
end;



procedure SetIncomingPingAllowed(Value: Boolean);
var objFirewall,objPolicy: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
objPolicy.IcmpSettings.AllowInboundEchoRequest := Value;
end;





function GetRemoteAdminAdress: string;
var objFirewall, objPolicy, objAdminSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
objAdminSettings:=objPolicy.RemoteAdminSettings;
Result := objAdminSettings.RemoteAddresses;
end;



procedure SetRemoteAdminAdress(Value: string);
var objFirewall, objPolicy, objAdminSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
objAdminSettings:=objPolicy.RemoteAdminSettings;
objAdminSettings.RemoteAddresses:=Value;
end;




function IsRemoteAdminAllowed: boolean;
var objFirewall, objPolicy, objAdminSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
objAdminSettings:=objPolicy.RemoteAdminSettings;
Result := objAdminSettings.Enabled;
end;



procedure SetRemoteAdminAllowed(Value: Boolean);
var objFirewall, objPolicy, objAdminSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
objAdminSettings:=objPolicy.RemoteAdminSettings;
objAdminSettings.Enabled := Value;
end;



procedure SetICMPSetting(Redirect: boolean);
{Modifies a Windows Firewall ICMP setting for the current profile.}
var objFirewall, objPolicy,objICMPSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;

objICMPSettings:=objPolicy.ICMPSettings;
objICMPSettings.AllowRedirect:= Redirect;
end;



procedure ModifyFirewallProperties(NoExceptions,NoNotifications,NoMulticaseResponse: boolean);
{Modifies Windows Firewall properties for the current profile. }
var objFirewall, objPolicy: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;

objPolicy.ExceptionsNotAllowed := NoExceptions;
objPolicy.NotificationsDisabled := NoNotifications;
objPolicy.UnicastResponsestoMulticastBroadcastDisabled := NoMulticaseResponse;
end;




function AreExceptionsAllowed: boolean;
var objFirewall, objPolicy: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
Result :=not objPolicy.ExceptionsNotAllowed;
end;





procedure OpenFirewallPort(Port: integer;  PortType: TPortType; Enable: boolean);
{Opens/close port for the Windows Firewall current profile. }
var objFirewall, objPolicy, colPorts, objPort: OleVariant;
var Proto: integer;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
colPorts:=objPolicy.GloballyOpenPorts;
if PortType = ptTCP then Proto := NET_FW_IP_PROTOCOL_TCP
else Proto := NET_FW_IP_PROTOCOL_UDP;

objPort:=colPorts.Item(Port,Proto);
objPort.Enabled:=Enable;
end;



procedure RestoretheDefaultSettings(List: TStrings);
{Restore the Windows Firewall default settings. }
var objFirewall: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objFirewall.RestoreDefaults;
end;




{------- Routines to add/delete entries in fireware exception tables  ---------}


procedure AddPortToFirewall(EntryName: string; PortType: TPortType; PortNumber: Cardinal);
{Add }
var fwMgr,port:OleVariant;
var profile:OleVariant;
begin
fwMgr := CreateOLEObject('HNetCfg.FwMgr');
profile := fwMgr.LocalPolicy.CurrentProfile;
port := CreateOLEObject('HNetCfg.FWOpenPort');
port.Name := EntryName;
if PortType = ptTCP then port.Protocol := NET_FW_IP_PROTOCOL_TCP
else port.Protocol := NET_FW_IP_PROTOCOL_UDP;
port.Port := PortNumber;
port.Scope := NET_FW_SCOPE_ALL;
port.Enabled := true;
profile.GloballyOpenPorts.Add(port);
end;




procedure AddApplicationToFirewall(EntryName: string; Filename: string);
var fwMgr,app: OleVariant;
var profile: OleVariant;
begin
fwMgr := CreateOLEObject('HNetCfg.FwMgr');
profile := fwMgr.LocalPolicy.CurrentProfile;
app := CreateOLEObject('HNetCfg.FwAuthorizedApplication');
app.ProcessImageFileName := Filename;
app.Name := EntryName;
app.Scope := NET_FW_SCOPE_ALL;
app.IpVersion := NET_FW_IP_VERSION_ANY;
app.Enabled :=true;
profile.AuthorizedApplications.Add(app);
end;


procedure DeleteApplicationFromFirewall(const Filename: string);
var fwMgr,app: OleVariant;
var profile: OleVariant;
begin
fwMgr := CreateOLEObject('HNetCfg.FwMgr');
profile := fwMgr.LocalPolicy.CurrentProfile;
profile.AuthorizedApplications.Remove(Filename);
end;


{------- Routines to list Firewall properties --------------------------------}



procedure ListAuthorizedApps(List: TStrings);
{List all authorized applications}
var objFirewall, objPolicy, colApplications, objApplication: OleVariant;
var Enum: IEnumVariant;
var I,Cnt: cardinal;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
colApplications:=objPolicy.AuthorizedApplications;
Enum:=IUnknown(colApplications._NewEnum) as IEnumVariant;
I:=0;
while Enum.Next(1, objApplication, Cnt) = S_OK do
	begin
        List.Add('Authorized application(' + IntToStr(I) + '): ' + objApplication.Name);
        List.Add('     Application enabled: ' + GetBooleanStr(objApplication.Enabled));
        List.Add('     Application IP version: ' + VersionToStr(objApplication.IPVersion));
        List.Add('     Application process image file name: ' +  objApplication.ProcessImageFileName);
        List.Add('     Application remote addresses: ' +  objApplication.RemoteAddresses);
        List.Add('     Application scope: ' + ScopeToStr(objApplication.Scope));
	Inc(I);
        end;
end;



procedure ListGlobalOpenPorts(List: TStrings);
{List all globally open ports}
var objFirewall,objPolicy, colPorts, objPort: OleVariant;
var Enum: IEnumVariant;
var I,Cnt: cardinal;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
colPorts:=objPolicy.GloballyOpenPorts;
Enum:=IUnknown(colPorts._NewEnum) as IEnumVariant;

while Enum.Next(1, objPort, Cnt) = S_OK do
	begin
	List.Add('Port name: ' + objPort.Name);
	List.Add('     Port number: ' + IntToStr(objPort.Port));
	List.Add('     Port IP version: ' + VersionToStr(objPort.IPVersion));
	List.Add('     Port protocol: ' + ProtocolToStr(objPort.Protocol));
	List.Add('     Port scope: ' + ScopeToStr(objPort.Scope));
	List.Add('     Port remote addresses: ' + objPort.RemoteAddresses);
	List.Add('     Port enabled: ' + GetBooleanStr(objPort.Enabled));
	List.Add('     Port built-in: ' + GetBooleanStr(objPort.Builtin));
	end;
end;


procedure ListFirewallProperties(List: TStrings);
{Lists Windows Firewall properties for the current profile.}
var objFirewall,objPolicy: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;
List.Add('Current profile type: ' + ProfileTypeToStr(objFirewall.CurrentProfileType));

List.Add('Firewall enabled: ' + GetBooleanStr(objPolicy.FirewallEnabled));
List.Add('Exceptions not allowed: ' + GetBooleanStr(objPolicy.ExceptionsNotAllowed));
List.Add('Notifications disabled: ' + GetBooleanStr(objPolicy.NotificationsDisabled));
List.Add('Unicast responses to multicast broadcast disabled: ' + GetBooleanStr(objPolicy.UnicastResponsestoMulticastBroadcastDisabled));
end;


procedure ListFirewallServiceProperties(List: TStrings);
{Lists service properties for the Windows Firewall current profile. }
var objFirewall, objPolicy, colServices, objService, colPorts, objPort: OleVariant;
var Enum1,Enum2: IEnumVariant;
var I,Cnt: cardinal;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;

colServices:=objPolicy.Services;

Enum1:=IUnknown(colServices._NewEnum) as IEnumVariant;
while Enum1.Next(1, objService, Cnt) = S_OK do
	begin
	List.Add('Service name: ' + objService.Name);
	List.Add('     Service enabled: ' + GetBooleanStr(objService.Enabled));
	List.Add('     Service type: ' + ServiceTypeToStr(objService.Type));
	List.Add('     Service IP version: ' + VersionToStr(objService.IPVersion));
	List.Add('     Service scope: ' + ScopeToStr(objService.Scope));
	List.Add('     Service remote addresses: ' + objService.RemoteAddresses);
	List.Add('     Service customized: ' + GetBooleanStr(objService.Customized));
	colPorts := objService.GloballyOpenPorts;
	Enum2:=IUnknown(colPorts._NewEnum) as IEnumVariant;
	while Enum2.Next(1, objPort, Cnt) = S_OK do
		begin
		List.Add('          Port name: ' + objPort.Name);
		List.Add('          Port number: ' + IntToStr(objPort.Port));
		List.Add('          Port enabled: ' + GetBooleanStr(objPort.Enabled));
		List.Add('          Port built-in: ' + GetBooleanStr(objPort.BuiltIn));
		List.Add('          Port IP version: ' + VersionToStr(objPort.IPVersion));
		List.Add('          Port protocol: ' + ProtocolToStr(objPort.Protocol));
		List.Add('          Port remote addresses: ' + objPort.RemoteAddresses);
		List.Add('          Port scope: ' + ScopeToStr(objPort.Scope));
		end;
	end;
end;



procedure ListICMPSettings(List: TStrings);
{Lists ICMP settings for the Windows Firewall current profile. }
var objFirewall,objPolicy,objICMPSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;

objICMPSettings:=objPolicy.ICMPSettings;

List.Add('Allow inbound echo request: ' + GetBooleanStr(objICMPSettings.AllowInboundEchoRequest));
List.Add('Allow inbound mask request: ' + GetBooleanStr(objICMPSettings.AllowInboundMaskRequest));
List.Add('Allow inbound router request: ' + GetBooleanStr(objICMPSettings.AllowInboundRouterRequest));
List.Add('Allow inbound timestamp request: ' + GetBooleanStr(objICMPSettings.AllowInboundTimestampRequest));
List.Add('Allow outbound destination unreachable: ' + GetBooleanStr(objICMPSettings.AllowOutboundDestinationUnreachable));
List.Add('Allow outbound packet too big: ' + GetBooleanStr(objICMPSettings.AllowOutboundPacketTooBig));
List.Add('Allow outbound parameter problem: ' + GetBooleanStr(objICMPSettings.AllowOutboundParameterProblem));
List.Add('Allow outbound source quench: ' + GetBooleanStr(objICMPSettings.AllowOutboundSourceQuench));
List.Add('Allow outbound time exceeded: ' + GetBooleanStr(objICMPSettings.AllowOutboundTimeExceeded));
List.Add('Allow redirect: ' + GetBooleanStr(objICMPSettings.AllowRedirect));
end;




procedure ListRemoteAdministrationSettings(List: TStrings);
{Lists remote administration settings for the Windows Firewall current profile. }
var objFirewall, objPolicy, objAdminSettings: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy.CurrentProfile;

objAdminSettings:=objPolicy.RemoteAdminSettings;
List.Add('Remote administration settings enabled: ' + GetBooleanStr(objAdminSettings.Enabled));
List.Add('Remote administration addresses: ' + objAdminSettings.RemoteAddresses);
List.Add('Remote administration scope: ' + ScopeToStr(objAdminSettings.Scope));
List.Add('Remote administration IP version: ' + VersionToStr(objAdminSettings.IPVersion));
end;





procedure ListStandardProfileProperties(List: TStrings);
{List information about the Windows Firewall standard profile. }
var objFirewall, objPolicy, objProfile: OleVariant;
begin
objFirewall:=CreateOLEObject('HNetCfg.FwMgr');
objPolicy:=objFirewall.LocalPolicy;
objProfile:=objPolicy.GetProfileByType(1);

List.Add('Firewall enabled: ' + GetBooleanStr(objProfile.FirewallEnabled));
List.Add('Exceptions not allowed: ' + GetBooleanStr(objProfile.ExceptionsNotAllowed));
List.Add('Notifications disabled: ' + GetBooleanStr(objProfile.NotificationsDisabled));
List.Add('Unicast responses to multicast broadcast disabled: ' + GetBooleanStr(objProfile.UnicastResponsestoMulticastBroadcastDisabled));
end;





end.


