unit Fourier;

interface

uses DSupport;



type TFastFourier=class(TObject)
 private
 protected
 public
  constructor Create;
  procedure CartesianToPolar(Reals,Imagins: array of double; var Mags,Phase: array of double; Size: integer);
  procedure Fourier(var Reals,Imagins: array of double; Size : Integer; Invert: boolean);
  procedure DoFourier(var Reals,Imagins: array of double; Size : Integer);
  procedure DoInverseFourier(var Reals,Imagins: array of double; Size : Integer);
 end;

implementation


{--------------- Fourier Transform Object ----------------------------------}

constructor TFastFourier.Create;
begin
inherited Create;
end;




procedure TFastFourier.CartesianToPolar(Reals,Imagins: array of double; var Mags,Phase: array of double; Size: integer);
var I: integer;
var TR: double;
begin
for I:=0 to Size-1 do
	begin
	Mags[I]:=sqrt(sqr(Reals[I])+sqr(Imagins[I]))/Size;
	if Reals[I]=0 then TR:=1.0E-300 else TR:=Reals[I];
	Phase[I]:=ArcTan2(Imagins[I],TR);
	end;
end;


procedure TFastFourier.Fourier(var Reals,Imagins: array of double; Size : Integer; Invert: boolean);
var TR,TI,WR,WI: double;
var MR,M,L,NN,STEP,A : Integer;
var  I,J : Integer;
begin
MR:=0;
NN:=Size-1;
for M:=1 to NN do
	begin
	L:=Size;
	repeat L:=L div 2 until MR+L<=NN;
	MR:=(MR mod L)+L;
	if MR>M then
		begin
		TR:=Reals[M];
		Reals[M]:=Reals[MR];
		Reals[MR]:=TR;
		TI:=Imagins[M];
		Imagins[M]:=Imagins[MR];
		Imagins[MR]:=TI;
		end;
	end;

L:=1;
while L<Size do
	begin
	STEP:=2*L;
	for M:=0 to L-1 do
		begin
		A:=(Size div STEP)*(-M);
		if Invert then A:=-A;
		WR:=-Sin(6.283185308*(A-Size div 4)/Size);
		WI:=Sin(6.283185308*A/Size);
		I:=M;
		repeat
			begin
			J:=I+L;
			TR:=(WR*Reals[J]-WI*Imagins[J]);
			TI:=(WR*Imagins[J]+WI*Reals[J]);
			Reals[J]:=Reals[I]-TR;
			Imagins[J]:=Imagins[I]-TI;
			Reals[I]:=Reals[I]+TR;
			Imagins[I]:=Imagins[I]+TI;
			I:=I+STEP
			end
		until I>=Size-1;
		end;
	L:=STEP;
	end;

end;



procedure TFastFourier.DoFourier(var Reals,Imagins: array of double; Size : Integer);
begin
Fourier(Reals,Imagins,Size,False);
end;



procedure TFastFourier.DoInverseFourier(var Reals,Imagins: array of double; Size : Integer);
var I: integer;
begin
Fourier(Reals,Imagins,Size,True);
for I:=0 to 511 do Reals[I]:=Reals[I]/Size;
for I:=0 to 511 do Imagins[I]:=Imagins[I]/Size;
end;









end.
