unit Georef;

interface

uses SysUtils,DSupport,StdCtrls,Math,Vector;


procedure LongLatToUTM(LON,LAT:double;var X,Y:double;var Zone: integer;DatumIndex: integer);
procedure UTMToLongLat(X,Y:double;Zone: integer;var LON,LAT: double;DatumIndex: integer);
procedure SetGeoCombo(var Combo: TComboBox);
function LongLatConvergence(Long,Lat: double;DatumIndex: integer): double;
function UTMConvergence(X,Y:double;Zone: integer;DatumIndex: integer): double;
procedure UTMEastWestLimits(North: double; Zone, DatumIndex: integer; var Min,Max: double);
function ValidateUTM(East,North: double;Zone,DatumIndex: integer): boolean;
procedure ChangeDatums(var Longitude,Latitude,Height: double; FromInx,ToInx: integer);
function GetZoneWidth(Lat: double): double;
procedure GetDatumParams(DatumInx: integer; var Major,Minor,Flat,Eccent: double; var Shift: T3DVector);
function LatLongDistance(Lat1, Long1, Lat2, Long2: double; DatumIndex: integer; var Bear1,Bear2,Dist: double): boolean;
procedure MakeZoneEquivalent(OldEast,OldNorth: double; ProperZone,ExtendedZone: integer; DatumIndex: integer; var NewEast,NewNorth: double);

{ Ellipsoid constants }

type
 TEllipsoidParams =record
 Major,Minor: double;
 Name: String;
 end;

const EllipseCount=22;

const
 Ellipsoids: array [0..EllipseCount-1] of TEllipsoidParams =(
 (Major: 6378206.4;    Minor: 6356583.8;     Name: 'Clarke 1866'),              {0}
 (Major: 6378249.145;  Minor: 6356514.86955; Name: 'Clarke 1880'),              {1}
 (Major: 6377397.155;  Minor: 6356078.96284; Name: 'Bessel'),                   {2}
 (Major: 6378157.5;    Minor: 6356772.2;     Name: 'International 1967'),       {3}
 (Major: 6378388.0;    Minor: 6356911.94613; Name: 'International 1909'),       {4}
 (Major: 6378135.0;    Minor: 6356750.519915;Name: 'WGS 72'),                   {5}
 (Major: 6377276.3452; Minor: 6356075.4133;  Name: 'Everest'),                  {6}
 (Major: 6378145.0;    Minor: 6356759.769356;Name: 'WGS 66'),                   {7}
 (Major: 6378137.0;    Minor: 6356752.31414; Name: 'GRS 1980'),                 {8}
 (Major: 6378137.0;    Minor: 6356752.31414; Name: 'WGS 84'),                   {9}
 (Major: 6377563.396;  Minor: 6356256.91;    Name: 'Airy'),                     {10}
 (Major: 6377304.063;  Minor: 6356103.039;   Name: 'Modified Everest'),         {11}
 (Major: 6377340.189;  Minor: 6356034.448;   Name: 'Modified Airy'),            {12}
 (Major: 6378137.0;    Minor: 6356752.314245;Name: 'Walbeck'),                  {13}
 (Major: 6378155.0;    Minor: 6356773.3205;  Name: 'Southeast Asia'),           {14}
 (Major: 6378160.0;    Minor: 6356774.719;   Name: 'Australian National'),      {15}
 (Major: 6378245.0;    Minor: 6356863.0188;  Name: 'Krassovsky'),               {16}
 (Major: 6378270.0;    Minor: 6356794.343479;Name: 'Hough'),                    {17}
 (Major: 6378166.0;    Minor: 6356784.283666;Name: 'Mercury 1960'),             {18}
 (Major: 6378150.0;    Minor: 6356768.337303;Name: 'Modified Mercury 1968'),    {19}
 (Major: 6378160.0;    Minor: 6356774.71920; Name: 'South American 1969'),      {20}
 (Major: 6370997.0;    Minor: 6370997.0;     Name: 'Sphere of Radius 6370997m')){21};



type
 TGeoParams = record
 Index: integer;
 Shift: T3DVector;
 Name: String;
 end;

const GeoCount=23;

const
 GeoParams: array [0..GeoCount-1] of TGeoParams =(
   (Index:  1; Shift:(X: -162.0; Y:  -12.0; Z:  206.0); Name: 'Adindan'),		{0}
   (Index:  1; Shift:(X: -143.0; Y:  -90.0; Z: -294.0); Name: 'Arc 1950'),		{1}
   (Index:  1; Shift:(X: -160.0; Y:   -8.0; Z: -300.0); Name: 'Arc 1960'),		{2}
   (Index: 15; Shift:(X: -133.0; Y:  -48.0; Z:  148.0); Name: 'Australian 1966'),	{3}
   (Index: 15; Shift:(X: -134.0; Y:  -48.0; Z:  149.0); Name: 'Australian 1984'),	{4}
   (Index:  3; Shift:(X: -104.0; Y: -129.0; Z:  230.0); Name: 'Camp Area Astro'),	{5}
   (Index:  1; Shift:(X: -136.0; Y: -108.0; Z: -292.0); Name: 'Cape'),			{6}
   (Index:  4; Shift:(X:  -87.0; Y:  -98.0; Z: -121.0); Name: 'European 1950'),		{7}
   (Index:  3; Shift:(X:  -86.0; Y:  -98.0; Z: -119.0); Name: 'European 1979'),		{8}
   (Index:  3; Shift:(X:   84.0; Y:  -22.0; Z:  209.0); Name: 'Geodetic 1949'),		{9}
   (Index:  3; Shift:(X: -156.0; Y: -271.0; Z: -189.0); Name: 'Hong Kong 1963'),	{10}
   (Index:  3; Shift:(X: -634.0; Y: -549.0; Z: -201.0); Name: 'Hu Tzu Shan'),		{11}
   (Index:  6; Shift:(X:  289.0; Y:  734.0; Z:  257.0); Name: 'Indian'),		{12}
   (Index:  0; Shift:(X:   -8.0; Y:  160.0; Z:  176.0); Name: 'North American 1927'),	{13}
   (Index:  8; Shift:(X:    0.0; Y:    0.0; Z:    0.0); Name: 'North American 1983'),	{14}
   (Index:  1; Shift:(X: -346.0; Y:   -1.0; Z:  224.0); Name: 'Oman'),			{15}
   (Index: 10; Shift:(X:  375.0; Y: -111.0; Z:  431.0); Name: 'Ordnance Survey 1936'),	{16}
   (Index: 16; Shift:(X:   27.0; Y: -135.0; Z:  -89.0); Name: 'Pulkovo 1942'),		{17}
   (Index:  3; Shift:(X: -288.0; Y:  175.0; Z: -376.0); Name: 'South American 1956'),	{18}
   (Index: 20; Shift:(X:  -57.0; Y:    1.0; Z:  -41.0); Name: 'South American 1969'),	{19}
   (Index:  2; Shift:(X: -128.0; Y:  481.0; Z:  664.0); Name: 'Tokyo'),			{20}
   (Index:  5; Shift:(X:    0.0; Y:    0.0; Z:   -4.5); Name: 'WGS 1972'),		{21}
   (Index:  9; Shift:(X:    0.0; Y:    0.0; Z:    0.0); Name: 'WGS 1984'));		{22}

const DefaultGeo=21;

implementation


var
  MajorAxis,MinorAxis,		{Earth Axes dimensions}
  ScaleFact,			{Scale Factor }
  CMeridian,			{Central Meridian}
  NorthOff,			{Offset}
  E2,EPS,A0,A1,A2,ERR: double;


procedure SetGeoCombo(var Combo: TComboBox);
{Fill combobox with Ellipsoid types}
var I: integer;
begin
Combo.Clear;
for I:=0 to GeoCount-1 do Combo.Items.Add(GeoParams[I].Name);
Combo.ItemIndex:=DefaultGeo;
end;



procedure GetDatumAxes(DatumIndex: integer; var Major,Minor: double; var Shift: T3DVector);
{Get Axes values from the Ellipsoid Tables}
var EllipsIndex: integer;
begin
if (DatumIndex<0) or (DatumIndex>=GeoCount) then DatumIndex:=DefaultGeo;
EllipsIndex:=GeoParams[DatumIndex].Index;
Shift:=GeoParams[DatumIndex].Shift;
Major:=Ellipsoids[EllipsIndex].Major;
Minor:=Ellipsoids[EllipsIndex].Minor;
end;



procedure SetAxesFromDatum(DatumIndex: integer);
{Get Axis setting from the Ellipsoid Tables}
var V: T3DVector;
begin
GetDatumAxes(DatumIndex,MajorAxis,MinorAxis,V);
end;



procedure InitGeoConstants(DatumIndex: integer);
var E4,E6,E8,AE: double;
begin
SetAxesFromDatum(DatumIndex);
ScaleFact:=0.9996;

E2:=(MajorAxis*MajorAxis-MinorAxis*MinorAxis)/(MajorAxis*MajorAxis);
EPS:=(MajorAxis*MajorAxis-MinorAxis*MinorAxis)/(MinorAxis*MinorAxis);
E4:=E2*E2; E6:=E4*E2; E8:=E6*E2;
AE:=MajorAxis*(1-E2);
A0:=AE*(1+(3/4)*E2+(45/64)*E4+(175/256)*E6+(11025/16384)*E8);
A1:=-AE/2*((3/4)*E2+(15/16)*E4+(525/512)*E6+(2205/2048)*E8);
A2:=AE/4*((15/64)*E4+(105/256)*E6+(2205/4096)*E8);
ERR:=-AE/6*((35/512)*E6+(315/2048)*E8);
end;


procedure GetDatumParams(DatumInx: integer; var Major,Minor,Flat,Eccent: double; var Shift: T3DVector);
{Calculate Datum based parameters}
begin
GetDatumAxes(DatumInx,Major,Minor,Shift);
Flat:=(Major-Minor)/Major;
Eccent:=1 - Minor * Minor/ Major/Major;
end;

function LatToNorth(PHI: double): double;
{Calculate the distance from the equator along the meridian}
var LAT: double;
begin
LAT:=PHI*Degree2Radian;
Result:=(A0*LAT + A1*SIN(2*LAT) + A2*SIN(4*LAT) + ERR*SIN(6*LAT));
end;


procedure  UTMToLongLat(X,Y:double; Zone: integer; var LON,LAT: double; DatumIndex: integer);
{Convert UTM to Lat/Long}
var BaseLatR: double;
var PHI,OLD,S,C,T,T2,N,N2,Y0,AA0,AA1: double;
var AA2,T3,T4,T5,T6,N4,N6,N8,AA3,AA4: double;
var I: integer;

	function NorthToLat(Y,PHI:double): double;
	{Calculate latitude from distance north of equator}
	var DY,C,D,C1,P1,P2: double;
	begin
	Y0:=LatToNorth(PHI*Radian2Degree);
	DY:=Y-Y0;
	C:=MajorAxis*(1-E2);
	D:=(1-E2*SIN(PHI)*SIN(PHI));
	C1:=-3*E2/(C*C);
	P1:=Power(D,1.5)/C;
	P2:=C1*D*D*COS(PHI);
	Result:=(PHI+DY*P1+DY*DY*P2);
	end;

begin
InitGeoConstants(DatumIndex);
BaseLatR:=0;
CMeridian:=((6*abs(Zone)) - 183)*Degree2Radian;
if Zone<0 then NorthOff:=10000000 else NorthOff:=0;
X:= (X-500000)/ScaleFact;
Y:=(Y-NorthOff)/ScaleFact;

{ Iterate to compute meridianal distance}
PHI:=NorthToLat(Y,BaseLatR);
OLD:=Y;
for I:= 1 to 10 do
	begin
	PHI:=NorthToLat(Y,PHI);
	if OLD = Y0 then break;
	OLD:=Y0;
	end;
if PHI>(PI/2) then PHI:=PI/2;
if PHI<(-PI/2) then PHI:=-PI/2;

S:=SIN(PHI); C:=COS(PHI);
T:=Tan(Phi);
T2:=T*T;
N2:=EPS*C*C;
N:=MajorAxis/SQRT(1-E2*S*S);
AA0:=X/N;
AA1:=Power(AA0,3)*(1+2*T2+N2)/6;
AA2:=Power(AA0,5)*(5+28*T2+24*T2*T2+6*N2+8*T2*N2)/120;

{ Compute Longitude (Radians)}

LON:=CMeridian+(1/C)*(AA0-AA1+AA2);

T3:=T*T2; T4:=T*T3; T5:=T*T4; T6:=T*T5;
N4:=N2*N2; N6:=N4*N2; N8:=N6*N2;
AA1:=T*(1+N2)*AA0*AA0;
AA2:=T*(1+N2)*Power(AA0,4)*(5+3*T2+N2-4*N4-9*N2*T2);
AA3:=T*(1+N2)*Power(AA0,6)*(61+90*T2+46*N2+45*T4-252*T2*N2-3*N4+100*N6-66*T2*N4-90*T4*N2+88*N8+225*T4*N4+84*T2*N6-192*T2*N8);
AA4:=T*(1+N2)*Power(AA0,8)*(1385+3633*T2+4095*T4+1575*T6);

{ Compute Latitude (Radians)}
LAT:=PHI-AA1/2+AA2/24-AA3/720+AA4/40320;

{ Convert To Degrees}
LAT:=LAT *Radian2Degree;
LON:=LON *Radian2Degree;
end;


procedure  LongLatToUTM(LON,LAT:double; var X,Y:double; var Zone: integer;DatumIndex: integer);
{Compute UTM from Lat/Long}
var D,D2,D3,D4,D5,D6,D7,D8: double;
var S,C,T,T2,T4,T6,C2,C3,C4,C5,C6,C7: double;
var N2,N4,N,AA0,AA1,AA2,AA3: double;
var B0,B1,B2,B3,B4: double;
begin
InitGeoConstants(DatumIndex);
Zone:= trunc((LON + 180)/6 + 1);
if LAT < 0 then Zone:=-Zone;
if Zone<0 then NorthOff:=10000000 else NorthOff:=0;
CMeridian:=((6*abs(Zone)) - 183)*Degree2Radian;

D:=(LON*Degree2Radian)-CMeridian;
D2:=D*D; D3:=D2*D; D4:=D3*D; D5:=D4*D;
D6:=D5*D; D7:=D6*D; D8:=D7*D;

S:=SIN(LAT*Degree2Radian);
C:=COS(LAT*Degree2Radian);
if C=0 then C:=1e-40;
T:=S/C;
T2:=T*T;
T4:=T2*T2;
T6:=T4*T2;
C2:=C*C; C3:=C2*C; C4:=C3*C; C5:=C4*C; C6:=C5*C; C7:=C6*C;

N2:=EPS*C*C;
N4:=N2*N2;
N:=MajorAxis/SQRT(1-E2*S*S);
AA0:=N*C;
AA1:=N*C3*(1-T2+N2);
AA2:=N*C5*(5-18*T2+T4+14*N2-58*T2*N2);
AA3:=N*S*C7*(61-479*T2+179*T4-T6);
B0:=LatToNorth(LAT);
B1:=N*S*C;
B2:=N*S*C3*(5-T2+9*N2+4*N4);
B3:=N*S*C5*(61-58*T2+T4+270*N2-330*T2*N2);
B4:=N*S*C7*(1385-3111*T2+543*T4-T6);

X:=ScaleFact*(AA0*D+AA1*D3/6+AA2*D5/120+AA3*D7/5040)+500000;
Y:=ScaleFact*(B0+B1*D2/2+B2*D4/24+B3*D6/720+B4*D8/40320)+NorthOff;
end;



function LongLatConvergence(Long,Lat: double; DatumIndex: integer): double;
{ Calculate the UTM Convergence angle based on Long, Lat}
var P,P1,P3,ES,ESP,SinPhi,CosPhi,C2,Con,N,Q1,Q2: double;
begin
SetAxesFromDatum(DatumIndex);
CMeridian:=((6*abs(trunc((Long + 180)/6 + 1))) - 183);
P:=(Long - CMeridian);
P1:=Abs(P/1000);
P3:=P1*P1*P1;
SinPhi:=Sin(Lat*Degree2Radian);
CosPhi:=Cos(Lat*Degree2Radian);
C2:=CosPhi*CosPhi;
ES:=1 - MinorAxis*MinorAxis/MajorAxis/MajorAxis;
ESP:=ES/(1- ES);
Con:=1 - ES * SinPhi*SinPhi;
N:=MajorAxis / sqrt(con);
Q1:=P*SinPhi;
Q2:=P3*N*SinPhi*C2*(1 + 3*ESP*C2 + 2 + ESP*ESP*C2*C2)/3600;
{use Survey convention, not Gauss-Bomford}
Result:=(Q1 + Q2);
end;



function UTMConvergence(X,Y:double; Zone: integer; DatumIndex: integer): double;
{ Calculate the UTM Convergence angle based on X,Y UTM coordinates}
var Long,Lat: double;
begin
UTMToLongLat(X,Y,Zone,Long,Lat,DatumIndex);
Result:=LongLatConvergence(Long,Lat,DatumIndex);
end;




procedure UTMEastWestLimits(North: double; Zone, DatumIndex: integer; var Min,Max: double);
{Return the minimum and maximum UTM values for a particular north}
var Long,Lat: double;
begin
UTMToLongLat(500000,North,Zone,Long,Lat,DatumIndex);
LongLatToUTM(0,Lat,Min,North,Zone,DatumIndex);
Max:=1000000-Min;
end;



function ValidateUTM(East,North: double; Zone,DatumIndex: integer): boolean;
{ Return True if UTM coordinates and zone are valid}
var Min,Max: double;
begin
Result:=False;
if (Zone<-60) or (Zone>60) or (Zone=0) then exit;
UTMEastWestLimits(North,Zone,DatumIndex,Min,Max);
if (East<Min) or (East>Max) then exit;
if (North<0) or (North>10000000) then exit;
Result:=True;
end;


{---------------- Find distance between to Lat/Long Points  ---------------}

function LatLongDistance(Lat1, Long1, Lat2, Long2: double; DatumIndex: integer; var Bear1,Bear2,Dist: double): boolean;
{Get distance and bearing for twol Lat/Long points - Bear1 = from angle, Bear2 =  To angle}
var Major,Minor,Flat,Eccent: double;
var Shift: T3DVector;
var A1,B1: double;
var L: double;
var U1,U2: double;
var sinU1,cosU1,sinU2,cosU2: double;
var lambda,lambdaP: double;
var cosLambda,sinSigma,sinLambda,cosSigma,sigma,sinAlpha,cosSqAlpha,cos2SigmaM,C: double;
var uSq, deltaSigma : double;
var iterLimit: integer;
begin
Result:=False;
GetDatumParams(DatumIndex, Major,Minor,Flat,Eccent,Shift);
L:=(Long2-Long1) * Degree2Radian;
U1:=arctan((1-Flat) * tan(Lat1 * Degree2Radian));
U2:=arctan((1-Flat) * tan(Lat2 * Degree2Radian));
sinU1:=sin(U1);
cosU1:=cos(U1);
sinU2:=sin(U2);
cosU2:=cos(U2);
lambda:=L;
iterLimit:=100;
repeat
	begin
	sinLambda:=sin(lambda);
	cosLambda:=cos(lambda);
	sinSigma:=sqrt((cosU2*sinLambda) * (cosU2*sinLambda) +  (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
	if (sinSigma=0) then raise Exception.Create('Co-incident points');
	cosSigma:=sinU1*sinU2 + cosU1*cosU2*cosLambda;
	sigma:=arctan2(sinSigma, cosSigma);
	sinAlpha:=cosU1 * cosU2 * sinLambda / sinSigma;
	cosSqAlpha:=1 - sinAlpha*sinAlpha;
	if cosSqAlpha=0 then cosSqAlpha:=1e-12;
	cos2SigmaM:=cosSigma - 2*sinU1*sinU2/cosSqAlpha;
	if isNaN(cos2SigmaM) then cos2SigmaM:=0;  // equatorial line: cosSqAlpha:=0 (6)
	C:=Flat/16*cosSqAlpha*(4+Flat*(4-3*cosSqAlpha));
	lambdaP:=lambda;
	lambda:=L + (1-C) * Flat * sinAlpha * (sigma + C*sinSigma*(cos2SigmaM+C*cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)));
	Dec(iterLimit);
	end
until (abs(lambda-lambdaP) < 1e-12) or (iterLimit<=0);
if (iterLimit=0) then raise Exception.Create('formula failed to converge');

uSq:=cosSqAlpha * (Major*Major - Minor*Minor) / (Minor*Minor);
A1:=1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
B1:=uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
deltaSigma:=B1*sinSigma*(cos2SigmaM+B1/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)- B1/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
Dist:=Minor*A1*(sigma-deltaSigma);
Bear1:= arctan2(cosU2*Sin(Lambda), cosU1*sinU2 - sinU1*cosU2*cos(Lambda));
Bear2:= arctan2(cosU1*Sin(Lambda), -sinU1*cosU2 + cosU1*sinU2*cos(Lambda));
Result:=True;
end;




procedure  LongLatToZoneUTM(LON,LAT: double; var X,Y: double; Zone: integer; DatumIndex: integer);
{Compute UTM for a specific Zone from Lat/Long - Used for zone crossing calculations}
var D,D2,D3,D4,D5,D6,D7,D8: double;
var S,C,T,T2,T4,T6,C2,C3,C4,C5,C6,C7: double;
var N2,N4,N,AA0,AA1,AA2,AA3: double;
var B0,B1,B2,B3,B4: double;
begin
InitGeoConstants(DatumIndex);

if LAT < 0 then Zone:=-abs(Zone) else Zone:=abs(Zone);
if Zone<0 then NorthOff:=10000000 else NorthOff:=0;
CMeridian:=((6*abs(Zone)) - 183)*Degree2Radian;

D:=(LON*Degree2Radian)-CMeridian;
D2:=D*D; D3:=D2*D; D4:=D3*D; D5:=D4*D;
D6:=D5*D; D7:=D6*D; D8:=D7*D;

S:=SIN(LAT*Degree2Radian);
C:=COS(LAT*Degree2Radian);
if C=0 then C:=1e-40;
T:=S/C;
T2:=T*T;
T4:=T2*T2;
T6:=T4*T2;
C2:=C*C; C3:=C2*C; C4:=C3*C; C5:=C4*C; C6:=C5*C; C7:=C6*C;

N2:=EPS*C*C;
N4:=N2*N2;
N:=MajorAxis/SQRT(1-E2*S*S);
AA0:=N*C;
AA1:=N*C3*(1-T2+N2);
AA2:=N*C5*(5-18*T2+T4+14*N2-58*T2*N2);
AA3:=N*S*C7*(61-479*T2+179*T4-T6);
B0:=LatToNorth(LAT);
B1:=N*S*C;
B2:=N*S*C3*(5-T2+9*N2+4*N4);
B3:=N*S*C5*(61-58*T2+T4+270*N2-330*T2*N2);
B4:=N*S*C7*(1385-3111*T2+543*T4-T6);

X:=ScaleFact*(AA0*D+AA1*D3/6+AA2*D5/120+AA3*D7/5040)+500000;
Y:=ScaleFact*(B0+B1*D2/2+B2*D4/24+B3*D6/720+B4*D8/40320)+NorthOff;
end;




procedure MakeZoneEquivalent(OldEast,OldNorth: double; ProperZone,ExtendedZone: integer; DatumIndex: integer; var NewEast,NewNorth: double);
{Convert points in two UTM Zones into the equivalent point in the first zone}
var Long,Lat: double;
begin
UTMToLongLat(OldEast,OldNorth,ProperZone,Long,Lat,DatumIndex);
LongLatToZoneUTM(Long,Lat, NewEast,NewNorth,ExtendedZone,DatumIndex);
end;



{---------------- Routines to handle Conversion between Datums ---------------}


procedure MolodenskyTransform(var Longitude,Latitude,Height: double;
			   FromMajor, FromFlatten, FromEccent,
			   DeltaMajorAxis, DeltaFlatten, dx,dy,dz: double);
{Convert between Datums using the Molodensky Transform}
{Longitude,Latitude,Height:	The geodetic position to be translated.
FromMajor:			The semi-major axis of the "from" ellipsoid.
FromFlatten:			Flattening of the "from" ellipsoid.
FromEccent:			Eccentricity-squared of the "from" ellipsoid.

DeltaMajorAxis:			Change in semi-major axis length (meters); "to" minus "from"
DeltaFlatten:			Change in flattening; "to" minus "from"
dx:				Change in x between "from" and "to" datum.
dy:				Change in y between "from" and "to" datum.
dz:				Change in z between "from" and "to" datum.}
var SinLat: double;
var CosLat: double;
var SinLong: double;
var CosLong: double;
var SinLatSquared: double;
var MajorMinorRatio: double;
var DeltaLat, DeltaLong, DeltaHeight: double;
var VerticalRadius: double;
var PrimeRadius: double;
begin
SinLat:=sin(Latitude);
CosLat:=cos(Latitude);
SinLong:=sin(Longitude);
CosLong:=cos(Longitude);
SinLatSquared:=SinLat * SinLat;
MajorMinorRatio:=1 / (1 - FromFlatten);  { A/B = 1(1 - F) }

{Radius of curvature in prime vertical}
VerticalRadius:=FromMajor / sqrt(1 - FromEccent * SinLatSquared);
{Radius of curvature in prime meridian}
PrimeRadius:=FromMajor * (1 - FromEccent) / Power ((1 - FromEccent * SinLatSquared),1.5 );

{Calculate Latitude shift}
DeltaLat:=(((((-dx * SinLat * CosLong - dy * SinLat * SinLong) + dz * CosLat)
      + (DeltaMajorAxis * ((VerticalRadius * FromEccent * SinLat * CosLat) / FromMajor)))
      + (DeltaFlatten * (PrimeRadius * MajorMinorRatio + VerticalRadius / MajorMinorRatio) * SinLat * CosLat))) / (PrimeRadius + Height);

{Calcualte Longitude shift}
DeltaLong:=(-dx * SinLong + dy * CosLong) / ((VerticalRadius + Height) * CosLat);

{Calculate Height shift}
DeltaHeight:=(dx * CosLat * CosLong) + (dy * CosLat * SinLong) + (dz * SinLat)
     - (DeltaMajorAxis * (FromMajor / VerticalRadius)) + ((DeltaFlatten * VerticalRadius * SinLatSquared) / MajorMinorRatio);

{Add shifts to starting values}
Longitude:=Longitude + DeltaLong;
Latitude:=Latitude + DeltaLat;
Height:=Height + DeltaHeight;
end;




procedure ChangeDatums(var Longitude,Latitude,Height: double; FromInx,ToInx: integer);
{Change datums based on the datums specified by From and To indexes}
var FromMajor,FromMinor,FromFlat,FromEccent: double;
var ToMajor,ToMinor,ToFlat,ToEccent: double;
var DeltaMajorAxis,DeltaFlatten: double;
var FromShift,ToShift,DS: T3DVector;
begin
if FromInx = ToInx then exit;
{Convert to radians}
Longitude:=DegToRad(Longitude);
Latitude:=DegToRad(Latitude);

{Get From and To Major, Minor, Flattening, Eccentricty and Shift}
GetDatumParams(FromInx,FromMajor,FromMinor,FromFlat,FromEccent,FromShift);
GetDatumParams(ToInx,ToMajor,ToMinor,ToFlat,ToEccent,ToShift);

{Calculate the Delta parameters}
DeltaMajorAxis:=ToMajor - FromMajor;
DeltaFlatten:=ToFlat -FromFlat;
DS:=VectorSubtract3D(FromShift,ToShift);

{Do transform}
MolodenskyTransform(Longitude,Latitude,Height, FromMajor, FromFlat, FromEccent,
			   DeltaMajorAxis, DeltaFlatten,DS.X,DS.Y,DS.Z);
{Back to degrees}
Longitude:=RadToDeg(Longitude);
Latitude:=RadToDeg(Latitude);
end;


function GetZoneWidth(Lat: double): double;
var East1,East2,North: double;
var Zone: integer;
begin
LongLatToUTM(0,Lat,East1,North,Zone,14);
LongLatToUTM(5.999999999999999,Lat,East2,North,Zone,14);
Result:=East2 - East1;
end;



end.
