unit Hugemem;


interface

uses Classes,SysUtils,WinProcs,WinTypes;

type
    TMemoryBlock = class(TObject)
    private
	Handle: THandle;
	MemPointer: Pointer;
    protected
    public
      MemSize: LongInt;
      constructor Create(H:THandle);
      destructor Destroy; override;
      procedure ReAllocMem(NewMaxItems: LongInt);
      procedure TestReAlloc(Size: longint);
      function GetItemPtr(Index: LongInt): Pointer;
      procedure Put(Index: Longint;const Item: byte);
      function Get(Index: Longint): byte;
      property Value[Index: Longint]: byte read Get write Put; default;
    end;

implementation
 


procedure TMemoryBlock.ReAllocMem(NewMaxItems: LongInt);
{ Reallocate memory for new number of items }
var NewHandle: THandle;
var TmpPtr: Pointer;
begin
NewHandle := GlobalReAlloc(Handle, NewMaxItems, GMEM_MOVEABLE);
if NewHandle = 0 then OutOfMemoryError;

try
   TmpPtr := GlobalLock(NewHandle);
   if TmpPtr = nil then OutOfMemoryError;
   MemSize := GlobalSize(NewHandle);
   Handle := NewHandle;
   MemPointer:=TmpPtr;
except
   if Handle <> NewHandle then
	begin
	GlobalUnlock(NewHandle);
	GlobalFree(NewHandle);
	end;
    raise;
    end;
end;


procedure TMemoryBlock.TestReAlloc(Size: longint);
begin
if Size>MemSize then ReAllocMem(MemSize+100);
end;


constructor TMemoryBlock.Create(H:THandle);
{Create a memory block item}
begin
inherited Create;
Handle:=H;
MemPointer := GlobalLock(Handle);
MemSize:=GlobalSize(Handle);
end;



destructor TMemoryBlock.Destroy;
{clean up memory block array item}
begin
if Handle <> 0 then GlobalUnlock(Handle);
inherited Destroy;
end;

function TMemoryBlock.GetItemPtr(Index: LongInt): Pointer;
{ Return point to item based on index }
begin
Result:=Pointer(Integer(MemPointer) + Index);
end;

procedure TMemoryBlock.Put(Index: Longint;const Item: byte);
{Insert byte at index}
var PTR: ^byte;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function TMemoryBlock.Get(Index: Longint): byte;
{Get byte at the index}
var PTR: ^byte;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


end.
