unit HugeNumberLib;

interface

uses SysUtils;

{Define structures to represent 128-bit number}

type TInt128 = array[0..3] of cardinal;

{Define structure for dissection 128-bit number}

type  Int128Rec = packed record
case Integer of
 0: (Lo, Hi: Int64);
 1: (Int64s: array [0..1] of Int64);
 2: (Words: array [0..7] of Word);
 3: (Bytes: array [0..15] of Byte);
 end;

{Public subrotines}

function Neg128(I: TInt128): TInt128;
function Abs128(I: TInt128): TInt128;
function Int64ToInt128(I: Int64): TInt128;
function Int128ToInt64(I: TInt128): Int64;


function AddInt128(X,Y: TInt128): TInt128;
function SubInt128(x,y: TInt128): TInt128;

function UnsignedMul64_128(X,Y: int64): TInt128;
function SignedMul64_128(X,Y: int64): TInt128;


function UnsignedDiv128_64(Numerator: TInt128; Divider: int64): TInt128;
function SignedDiv128_64(Numerator: TInt128; Divider: int64): TInt128;
function MulDiv64(Value, Multiplier, Divider: Int64): Int64;


implementation



{-------------- Basic 128 bit operations ---------------------------------------}


function Neg128(I: TInt128): TInt128;
{Change sign of a 128-bit}
begin
Result:=I;
Int128Rec(Result).Lo:=0-Int128Rec(Result).Lo;
if Int128Rec(I).Lo<>0 then Inc(Int128Rec(Result).Hi);
Int128Rec(Result).Hi:=0-Int128Rec(Result).Hi;
end;



function IsNeg128(I: TInt128): boolean;
{Test if 128-bit number is negative}
begin
Result:=(I[3] and $8000)<>0;
end;



function Abs128(I: TInt128): TInt128;
{Return the absolution value of th 128-bit value}
begin
Result:=I;
if IsNeg128(I) then Result:=Neg128(Result);
end;



function Int64ToInt128(I: Int64): TInt128;
{Convert 64-bit number to 128-bits}
begin
Int128Rec(Result).Lo:=I;
if I<0 then Int128Rec(Result).Hi:=-1 else Int128Rec(Result).Hi:=0;
end;



function Int128ToInt64(I: TInt128): Int64;
{Convert 128-bit number to 64-bits}
var Negative: boolean;
begin
Negative:=IsNeg128(I);
I:=Abs128(I);
Result:=Int128Rec(I).Lo;
if Negative then Result:=-Result;
end;


{------------------- Do Add Routines ------------------------------------------}

function AddWithCarry(X,Y: integer; var Carry: Boolean): integer;
{Do 32-bit add and flag if there is a carry}
var I: Int64;
begin
I:=X+Y;
if Carry then Inc(I);
Carry:=Int64Rec(I).Hi<>0;
Result:=I;
end;




function AddInt128(X,Y: TInt128): TInt128;
{Add two 128-bit numbers}
var Carry: Boolean;
begin
Carry := False;
Result[0] := AddWithCarry(X[0], Y[0], Carry);
Result[1] := AddWithCarry(X[1], Y[1], Carry);
Result[2] := AddWithCarry(X[2], Y[2], Carry);
Result[3] := AddWithCarry(X[3], Y[3], Carry);
end;


{-------------------- Do Subtract Routines ------------------------------------}

function SubtractWithBorrow(X,Y: integer; var Borrow: Boolean): integer;
var I: Int64;
{Do 32-bit subtract and set flag if there is borrow }
begin
I:=X - Y;
if Borrow then Dec(I);
Borrow:=Int64Rec(I).Hi<>0;
Result:=I;
end;


function SubInt128(x,y: TInt128): TInt128;
{Subtract two 128-bit numbers}
var Borrow: Boolean;
begin
Borrow := False;
Result[0] := SubtractWithBorrow(x[0], y[0], Borrow);
Result[1] := SubtractWithBorrow(x[1], y[1], Borrow);
Result[2] := SubtractWithBorrow(x[2], y[2], Borrow);
Result[3] := SubtractWithBorrow(x[3], y[3], Borrow);
end;


{------------------ Mixed 64/128 operations ----------------------------------}

{Diagram of Multiply:

         Yh,Yl
         Xh,Xl
--------------
   R3,R2,R1,R0

1. Xl * Yl ---- R0 = M, C = O
2. Xl * Yh ---- R1 = M + C, R2 = O

3. Xh * Yl ---- R1 = R1 + M, C = O
4. Xh * Yh ---- R2 = M + C, R3 = O

}

function MulWithCarry(X,Y: cardinal; var Overflow: cardinal): cardinal;
{Do 32-bit multiply, flagging overflow}
var T: Int64;
begin
T:=Int64(X) * Int64(Y) + Overflow;
Overflow:=Int64Rec(T).Hi;
Result:=Int64Rec(T).Lo;
end;



function UnsignedMul64_128(X,Y: int64): TInt128;
{Do Unsigned 64x64 Multiply with 128 result}
var Overflow: cardinal;
begin
OverFlow:=0;
Result[0]:=MulWithCarry(Int64Rec(X).Lo, Int64Rec(Y).Lo,Overflow);
Result[1]:=MulWithCarry(Int64Rec(X).Lo, Int64Rec(Y).Hi, Overflow);
Result[2]:=Overflow;

OverFlow:=0;
Result[1]:=Result[1]+MulWithCarry(Int64Rec(X).Hi, Int64Rec(Y).Lo,Overflow);
Result[2]:=Result[2]+MulWithCarry(Int64Rec(X).Hi, Int64Rec(Y).Hi, Overflow);
Result[3]:=Overflow;
end;



function SignedMul64_128(X,Y: int64): TInt128;
{Do Signed 64x64 Multiply with 128 result}
var Negative: boolean;
begin
Negative:=(X<0) xor (Y<0);
X:=Abs(X);
Y:=Abs(Y);
Result:=UnsignedMul64_128(X,Y);
if Negative then Result:=Neg128(Result);
end;


{-----------------------------------------------------------------------------}


function UnsignedDiv128_64(Numerator: TInt128; Divider: int64): TInt128;
{Do 128 by 64-bit unsigned divide}
var Remainder: TInt128;
begin
{Zero remainder}
Remainder[0]:=0; Remainder[1]:=0; Remainder[2]:=0; Remainder[3]:=0;

asm	MOV	ECX, 128
@Loop1:
{ Compute Remainder:Numerator =	Remainder:Numerator shl	1 }
	SHL	DWORD PTR [Numerator+ 0], 1
	RCL	DWORD PTR [Numerator+ 4], 1
	RCL	DWORD PTR [Numerator+ 8], 1
	RCL	DWORD PTR [Numerator+12], 1
	RCL	DWORD PTR [Remainder+ 0], 1
	RCL	DWORD PTR [Remainder+ 4], 1
	RCL	DWORD PTR [Remainder+ 8], 1
	RCL	DWORD PTR [Remainder+12], 1

{ Test (Remainder >= Divider) }
	XOR	EAX, EAX
	CMP	DWORD PTR [Remainder+12], EAX
	JA	@IfTrue
	JB	@IfFalse

	CMP	DWORD PTR [Remainder+ 8], EAX
	JA	@IfTrue
	JB	@IfFalse

	MOV	EAX, DWORD PTR [Remainder+ 4]
	CMP	EAX, DWORD PTR [Divider+4]
	JA	@IfTrue
	JB	@IfFalse

	MOV	EAX, DWORD PTR [Remainder+ 0]
	CMP	EAX, DWORD PTR [Divider+0]
	JB	@IfFalse

{ Remainder = Remainder	- Divider }
@IfTrue:
	MOV	EAX, DWORD PTR [Divider+0]
	SUB	DWORD PTR [Remainder+ 0], EAX
	MOV	EAX, DWORD PTR [&Divider+4]
	SBB	DWORD PTR [Remainder+ 4], EAX
	XOR	EAX, EAX
	SBB	DWORD PTR [Remainder+ 8], EAX
	SBB	DWORD PTR [Remainder+12], EAX

{ Quotient = Quotient +1  }
	ADD	DWORD PTR [Numerator+ 0], 1
	ADC	DWORD PTR [Numerator+ 4], 0
	ADC	DWORD PTR [Numerator+ 8], 0
	ADC	DWORD PTR [Numerator+12], 0

{ Loop size = 101 bytes, is less than 127 so loop is possible }
@IfFalse:
	LOOP	@Loop1
	end;
Result:=Numerator;
end;


function SignedDiv128_64(Numerator: TInt128; Divider: int64): TInt128;
var Negative: boolean;
begin
Negative:=IsNeg128(Numerator) xor (Divider<0);
Numerator:=Abs128(Numerator);
Divider:=Abs(Divider);
Result:=UnsignedDiv128_64(Numerator,Divider);
if Negative then Result:=Neg128(Result);
end;



function Modulo128_64(Numerator: TInt128; Divider: int64; var Remainder: int64): TInt128;
{Do 128/64 modulo function}
var T: TInt128;
begin
Result:=UnsignedDiv128_64(Numerator,Divider);
T:=Int64ToInt128(Divider);
Remainder:=Int128Rec(Numerator).Lo - Int128Rec(Result).Lo * Divider;
end;



{------------------------------------------------------------------------------}



function MulDiv64(Value, Multiplier, Divider: Int64): Int64;
{Multiplies two 64-bit values and then divides the 128-bit result by a third 64-bit value}
var T:TInt128;
begin
T:=SignedMul64_128(Value,Multiplier);
T:=SignedDiv128_64(T,Divider);
Result:=Int128ToInt64(T);
end;






end.
