unit HullLibrary;

interface

uses Vector;

procedure GetConvexHull(Points: T2DVectorArray; var Hull: T2DVectorArray);
procedure GetConvexHull2(Points: T2DVectorArray; var Hull: T2DVectorArray);

implementation



{------------------------------------------------------------------------------}

procedure LexQuickSort(var SortList: T2DVectorArray; L, R: Integer);
{Do lexical quick sort by X and then by Y values}
var I, J: Integer;
var  P, T: T2DVector;

	function SCompare(Item1, Item2: T2DVector): double;
	{Compare returns < 0 if Item1 is less and Item2,
	0 if they are equal and > 0 if Item1 is greater than Item2.}
	begin
	Result:=Item1.X - Item2.X;
	if Result=0 then Result:=Item1.Y - Item2.Y;
	end;


begin
repeat
	begin
	I := L;
	J := R;
	P := SortList[(L + R) shr 1];
	repeat
		begin
		while SCompare(SortList[I], P) < 0 do Inc(I);
		while SCompare(SortList[J], P) > 0 do Dec(J);
		if I <= J then
			begin
			T := SortList[I];
			SortList[I] := SortList[J];
			SortList[J] := T;
			Inc(I);
			Dec(J);
			end;
		end
	until I > J;
	if L < J then LexQuickSort(SortList, L, J);
	L := I;
	end
until I >= R;
end;



{------------------------------------------------------------------------------}

{ Convex hull based on Andrew's monotone chain algorithm}

procedure GetConvexHull(Points: T2DVectorArray; var Hull: T2DVectorArray);
{Converts Points in a Convex Hull}
{ Hull is in counter-clockwise order - assuming +Y is up and -Y is down}
{ Thus on a Windows screen, the order will appear clockwise}
{The last point is the same as the first}
var I,T: integer;

	function GetTriDir(P1,P2,P3: T2DVector): double;
	{Determine the point order of P1,P2,P3
	{ uses determint area of triangle}
	{ Result > 0 = Counter-clockwise }
	{ Result < 0 = Clockwise }
	{ Result = 0 = colinear }
	begin
	Result:= (P2.x - P1.X)*(P3.Y - P1.Y) - (P2.Y - P1.Y)*(P3.X - P1.X);
	end;


	procedure DelteItem;
	begin
	SetLength(Hull,Length(Hull)-1);
	end;


	procedure InsertItem(P: T2DVector);
	begin
	SetLength(Hull,Length(Hull)+1);
	Hull[High(Hull)]:=P;
	end;


begin
SetLength(Hull,0);
if Length(Points)<1 then exit;
LexQuickSort(Points,0,High(Points));
{Do lower hull}
for I:=0 to High(Points) do
	begin
	while (Length(Hull) >= 2) and
	      (GetTriDir(hull[Length(Hull)-2], hull[Length(Hull)-1], points[I]) <= 0) do DelteItem;
	InsertItem(points[i]);
	end;

{Do upper hull }
T:=Length(Hull)+1;
for I:=High(Points)-1 downto 0 do
	begin
	while (Length(Hull) >=T) and
	       (GetTriDir(hull[Length(Hull)-2], hull[Length(Hull)-1], points[i]) <= 0) do DelteItem;
	InsertItem(points[i]);
	end;
end;


procedure GetConvexHull2(Points: T2DVectorArray; var Hull: T2DVectorArray);
{Return Convex Hull - except last point is not the same as the first point}
begin
GetConvexHull(Points,Hull);
if Length(Hull)>1 then SetLength(Hull,Length(Hull)-1);
end;



end.
