unit ImageEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, intspin, Buttons, Menus,Clipbrd, Spin,
  ComCtrls, DSupport, Reopen, ImageLib,
  ExtDlgs, subtimer,BitmapLib, SpinButtons,CircularBuffer;
                                             
type
  TImageEditDlg = class(TForm)
    Splitter1: TSplitter;
    Panel2: TPanel;
    Panel3: TPanel;
    InScrollBox: TScrollBox;
    PageControl: TPageControl;
    ResizePage: TTabSheet;
    SharpGroup: TRadioGroup;
    GroupBox1: TGroupBox;
    IncRadio: TRadioGroup;
    UnitsRadio: TRadioGroup;
    ResetSizeBtn: TButton;
    Panel5: TPanel;
    StretchBtn: TSpeedButton;
    ClipBtn: TSpeedButton;
    TabSheet1: TTabSheet;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    BrightnessBar: TTrackBar;
    ContrastBar: TTrackBar;
    BrightDis: TPanel;
    ContrastDis: TPanel;
    GammaDis: TPanel;
    ResetBCGBtn: TButton;
    GammaBar: TTrackBar;
    DimenDis: TLabel;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    WidthSpin: TIntegerSpin;
    HeightSpin: TIntegerSpin;
    IncUnitsDis: TLabel;
    PercentDis: TPanel;
    IncSpin: TSpinButton;
    AspectChk: TCheckBox;
    InputImage: TImage;
    Panel1: TPanel;
    UndoBtn: TSpeedButton;
    CopyBtn: TSpeedButton;
    PasteBtn: TSpeedButton;
    Panel4: TPanel;
    Button1: TButton;
    Button2: TButton;
    HelpBtn: TBitBtn;
    procedure ClipBtnClick(Sender: TObject);
    procedure InputImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure InputImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure InputImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure StretchMnuClick(Sender: TObject);
    procedure ResetSizeBtnClick(Sender: TObject);
    procedure IncSpinUpClick(Sender: TObject);
    procedure IncSpinDownClick(Sender: TObject);
    procedure UnitsRadioClick(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ResetMeasureBtnClick(Sender: TObject);
    procedure SearchHelpMnuClick(Sender: TObject);
    procedure BrightnessBarChange(Sender: TObject);
    procedure ResetBCGBtnClick(Sender: TObject);
    procedure WidthSpinChange(Sender: TObject);
    procedure HeightSpinChange(Sender: TObject);
    procedure UndoBtnClick(Sender: TObject);
    procedure CopyBtnClick(Sender: TObject);
    procedure PasteBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ImageEditDlg: TImageEditDlg;


var SelectBox,OldBox: TRect;
var LeftDown: boolean;

var NewSizeX,NewSizeY: integer;

var BaseSizeX,BaseSizeY: integer;

var DeltaSize: integer;

var AspectRatio: double;
var FirstActivate: boolean;
var BrightnessChange: boolean;

var FileUndo: TFileUndo;
var BaseBitmap: TBitmap;

function DoEditImage(Bitmap: TBitmap; HelpContext: integer): boolean;
procedure InitializeImage;


implementation

uses resizeBM,BitmapProg;

{$R *.DFM}

{------------------- Handle Undo Process -------------------------------------}

procedure SetUndoItem;
begin
BaseBitmap.SaveToFile(FileUndo.AddUndoItem);
BaseBitmap.Assign(ImageEditDlg.InputImage.Picture.Bitmap);
end;

procedure RestoreUndoItem;
var S: string;
begin
S:=FileUndo.UndoItem;
if S='' then exit;
BaseBitmap.LoadFromFile(S);
ImageEditDlg.InputImage.Picture.Assign(BaseBitmap);
DeleteFile(S);
InitializeImage;
end;



procedure HandleBrightnessChange;
begin
if BrightnessChange then
	begin
	SetUndoItem;
	BaseBitmap.Assign(ImageEditDlg.InputImage.Picture.Bitmap);
	BrightnessChange:=False;
	end;
end;


{------------------ Handle Brightness, Contrast and Gamma --------------------}

procedure CenterTrackBar(TrackBar: TTrackBar);
var E: TNotifyEvent;
begin
E:=TrackBar.OnChange;
TrackBar.OnChange:=nil;
try
TrackBar.Position:=(TrackBar.Max + TrackBar.Min) div 2;
finally TrackBar.OnChange:=E; end;
end;




procedure GetBarSettings(var Bright,Contrast,Gamma: double);
begin
Bright:=ImageEditDlg.BrightnessBar.Position/100;
Contrast:=ImageEditDlg.ContrastBar.Position/100;
Gamma:=ImageEditDlg.GammaBar.Position/100;
end;


procedure DisplayBarValues(var Bright,Contrast,Gamma: double);
begin
ImageEditDlg.GammaDis.Caption:=FloatToStrF(Gamma,ffFixed,18,1);
ImageEditDlg.BrightDis.Caption:=FloatToStr(Bright);
ImageEditDlg.ContrastDis.Caption:=FloatToStr(Contrast);
end;



procedure ResetBrightContrastGamma(UpdateImage: Boolean);
begin
CenterTrackBar(ImageEditDlg.BrightnessBar);
CenterTrackBar(ImageEditDlg.ContrastBar);
CenterTrackBar(ImageEditDlg.GammaBar);
if UpdateImage then ImageEditDlg.BrightnessBarChange(ImageEditDlg);
end;



{---------------------------------------------------------------------}


function GetPixelString(BM: TBitmap): string;
begin
case BM.PixelFormat of
 pfDevice: Result:= 'Device';
 pf1bit: Result:= '1-bit';
 pf4bit: Result:= '4-bit';
 pf8bit: Result:= '8-bit';
 pf15bit: Result:= '15-bit';
 pf16bit: Result:= '16-bit';
 pf24bit: Result:= '24-bit';
 pf32bit: Result:= '32-bit';
 pfCustom: Result:= 'Custom';
 else Result:= 'Unknown';
 end;
end;

{-------------------Handle Size Setting ----------------------}



procedure GetInputSize;
{Get basic input image size}
begin
BaseSizeX:=ImageEditDlg.InputImage.Picture.Width;
BaseSizeY:=ImageEditDlg.InputImage.Picture.Height;
NewSizeX:=BaseSizeX;
NewSizeY:=BaseSizeY;
DeltaSize:=0;
if NewSizeY = 0 then AspectRatio:=1
else AspectRatio:=NewSizeX/NewSizeY;
end;


procedure ShowInputSize;
{Display current size of input image}
var X,Y: integer;
var S: string;
begin
X:=ImageEditDlg.InputImage.Picture.Width;
Y:=ImageEditDlg.InputImage.Picture.Height;

S:='Image Size: '+IntToStr(X)+' X '+IntToStr(Y);
if (ImageEditDlg.InputImage.Picture.Graphic<> nil) and not ImageEditDlg.InputImage.Picture.Graphic.Empty then
       	S:=S+' Pixels: '+GetPixelString(ImageEditDlg.InputImage.Picture.Bitmap);
ImageEditDlg.DimenDis.Caption:=S;
end;


procedure DisplaySize;
{Display what size will be after resizing}
begin
with ImageEditDlg do
 begin
 WidthSpin.NoEventValue:=NewSizeX;
 HeightSpin.NoEventValue:=NewSizeY;
 if UnitsRadio.ItemIndex=0 then IncUnitsDis.Caption:='Percent'
 else IncUnitsDis.Caption:='Pixels';
 PercentDis.Caption:=' '+IntToStr(DeltaSize);
 end;
end;



procedure NormalizeSize;
{Normalize aspect ratio and display new size}
begin
with ImageEditDlg do
 begin
 if UnitsRadio.ItemIndex=0 then NewSizeX:=BaseSizeX + round(BaseSizeX * (DeltaSize/100))
 else NewSizeX:=BaseSizeX + DeltaSize;
 NewSizeY:=round(NewSizeX / AspectRatio);
 DisplaySize;
 end;
end;


procedure InitializeImage;
{Handle newly loaded image}
begin
GetInputSize;
ShowInputSize;
NormalizeSize;
SelectBox:=Rect(0,0,0,0);
end;



procedure TImageEditDlg.ResetSizeBtnClick(Sender: TObject);
{Reset sizing increments and displays}
begin
GetInputSize;
NormalizeSize;
end;



procedure IncrementDimensions(Sign: integer);
{Symetrically increment the image dimensions}
var I: integer;
begin
with ImageEditDlg do
 begin
 I:=StrToInt(IncRadio.Items[IncRadio.ItemIndex]);
 I:=I * Sign;
 DeltaSize:=DeltaSize + I;
 if DeltaSize<=-100 then DeltaSize:=-99;
 end;
end;



procedure TImageEditDlg.IncSpinUpClick(Sender: TObject);
{Handle 'Up' spin button}
begin
IncrementDimensions(1);
NormalizeSize;
end;


procedure TImageEditDlg.IncSpinDownClick(Sender: TObject);
{Handle 'Down' spin button}
begin
IncrementDimensions(-1);
NormalizeSize;
end;

{-------------------Handle Drawing Clipping Box --------------------}


type TBoxCorners=(bcNone,bcTopLeft,bcTopRight,bcBottomLeft,bcBottomRight,bcCenter);

var BoxDragMode: TBoxCorners;


function CornerHitTest(P: TPoint): TBoxCorners;
{Test if point falls in one of the corners}
const Size=5;
begin
Result:=bcNone;
if PtInRect(Rect(SelectBox.Left-Size,SelectBox.Top-Size,SelectBox.Left+Size,SelectBox.Top+Size),P) then Result:=bcTopLeft
else if PtInRect(Rect(SelectBox.Right-Size,SelectBox.Top-Size,SelectBox.Right+Size,SelectBox.Top+Size),P) then Result:=bcTopRight
else if PtInRect(Rect(SelectBox.Left-Size,SelectBox.Bottom-Size,SelectBox.Left+Size,SelectBox.Bottom+Size),P) then Result:=bcBottomLeft
else if PtInRect(Rect(SelectBox.Right-Size,SelectBox.Bottom-Size,SelectBox.Right+Size,SelectBox.Bottom+Size),P) then Result:=bcBottomRight
else if PtInRect(SelectBox,P) then Result:=bcCenter;
end;


procedure DrawRectangle(Canvas: TCanvas; R: TRect);
{Draw a rectangle on specified canvas.}
begin
Canvas.MoveTo(R.Left,R.Top);
Canvas.LineTo(R.Right,R.Top);
Canvas.LineTo(R.Right,R.Bottom);
Canvas.LineTo(R.Left,R.Bottom);
Canvas.LineTo(R.Left,R.Top);
end;



procedure DrawHandle(Canvas: TCanvas; P: TPoint; Corner: TBoxCorners);
{Draw one of the corner handles}
var R: TRect;
const Size=3;
begin
R:=Rect(P.X-Size,P.Y-Size,P.X+Size,P.Y+Size);

Canvas.Brush.Style:=bsSolid;
Canvas.Brush.Color:=clLime;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=Canvas.Brush.Color;
Canvas.Pen.Mode:=pmNotXor;
Canvas.Rectangle(R.Left,R.Top,R.Right,R.Bottom);
end;


procedure DrawBox(Canvas: TCanvas; R: TRect);
{Draw current clipping box}
var R2: TRect;
begin
Canvas.Pen.Style:=psDot;
Canvas.Pen.Color:=clBlack;
Canvas.Pen.Mode:=pmXor;
DrawRectangle(Canvas,R);

R2:=R;
InflateRect(R2,1,1);
DrawRectangle(Canvas,R2);
if (R.Left=0) and (R.Top=0) and (R.Right=0) and (R.Bottom=0) then exit;

DrawHandle(Canvas,R.TopLeft,bcTopLeft);
DrawHandle(Canvas,R.BottomRight,bcBottomRight);
DrawHandle(Canvas,Point(R.Right,R.Top),bcTopRight);
DrawHandle(Canvas,Point(R.Left,R.Bottom),bcBottomLeft);
end;


procedure CursorScroll(Image: TImage; ScrollBox: TScrollBox; X,Y: integer);
{Scroll box based on position X,Y}
var RX,RY: integer;
var MovX,MovY: integer;
begin
RX:=X+Image.Left;
RY:=Y+Image.Top;
if RX<0 then MovX:=-RX
else if RX>ScrollBox.ClientWidth then MovX:=RX-ScrollBox.ClientWidth
else MovX:=0;

if RY<0 then MovY:=-RY
else if RY>ScrollBox.ClientHeight then MovY:=RY-ScrollBox.ClientHeight
else MovY:=0;

if RX<0 then ScrollBox.HorzScrollBar.Position:=ScrollBox.HorzScrollBar.Position-MovX;
if RX>ScrollBox.ClientWidth then ScrollBox.HorzScrollBar.Position:=ScrollBox.HorzScrollBar.Position+MovX;
if RY<0 then ScrollBox.VertScrollBar.Position:=ScrollBox.VertScrollBar.Position-MovY;
if RY>ScrollBox.ClientHeight then ScrollBox.VertScrollBar.Position:=ScrollBox.VertScrollBar.Position+MovY;
end;


procedure UpdateBox(Image: TImage; ScrollBox: TScrollBox; X1,Y1,X2,Y2: integer);
{Erase old box and draw new box}
begin
DrawBox(Image.Canvas,SelectBox);
SelectBox:=Rect(X1,Y1,X2,Y2);
DrawBox(Image.Canvas,SelectBox);
end;


procedure HideBox;
{Hide current box}
begin
OldBox:=SelectBox;
UpdateBox(ImageEditDlg.InputImage,ImageEditDlg.InScrollBox,0,0,0,0);
end;


procedure RestoreBox;
{Restore hidden box}
begin
UpdateBox(ImageEditDlg.InputImage,ImageEditDlg.InScrollBox,OldBox.Left,OldBox.Top,OldBox.Right,OldBox.Bottom);
end;

{-------------------Do Image Clipping -----------------------------------------}



function ClipBitmap(BM: TBitmap; R: TRect): boolean;
{Clip bitmap to rectangle}
var R2: TRect;
var T: integer;
begin
BM.Dormant;
BM.FreeImage;
Result:=False;
if R.Left=R.Right then exit;
if R.Top=R.Bottom then exit;

if R.Left>R.Right then begin T:=R.Left; R.Left:=R.Right; R.Right:=T; end;
if R.Top>R.Bottom then begin T:=R.Top; R.Top:=R.Bottom; R.Bottom:=T; end;

if R.Right>BM.Width then R.Right:=BM.Width;
if R.Left<0 then R.Left:=0;
if R.Bottom>BM.Height then R.Bottom:=BM.Height;
if R.Top<0 then R.Top:=0;

R2:=R;
OffsetRect(R2,-R.Left,-R.Top);
try
BM.Canvas.CopyRect(R2,BM.Canvas,R);
BM.Width:=R.Right-R.Left;
BM.Height:=R.Bottom-R.Top;
Result:=True;
except end;
end;


{---------------------Clipping Box Event Handlers -------------------}

var StartX,StartY: integer;

procedure BoxMouseDown(X,Y: integer);
begin
LeftDown:=True;
BoxDragMode:=CornerHitTest(Point(X,Y));
if BoxDragMode=bcNone then
UpdateBox(ImageEditDlg.InputImage,ImageEditDlg.InScrollBox,X,Y,X,Y);
StartX:=X; StartY:=Y;
end;


procedure BoxMouseMove(X,Y: integer);
var Rect: TRect;
begin
with ImageEditDlg do
 begin
 if LeftDown then
	begin
	CursorScroll(InputImage,InScrollBox,X,Y);
	case BoxDragMode of
	 bcNone,bcBottomRight: UpdateBox(InputImage,InScrollBox,SelectBox.Left,SelectBox.Top,X,Y);
	 bcTopLeft: UpdateBox(InputImage,InScrollBox,X,Y,SelectBox.Right,SelectBox.Bottom);
	 bcTopRight: UpdateBox(InputImage,InScrollBox,SelectBox.Left,Y,X,SelectBox.Bottom);
	 bcBottomLeft: UpdateBox(InputImage,InScrollBox,X,SelectBox.Top,SelectBox.Right,Y);
	 bcCenter:
		begin
		Rect:=SelectBox;
		OffsetRect(Rect,X-StartX,Y-StartY);
		UpdateBox(InputImage,InScrollBox,Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
		end;
	 end;
	end;
 end;
StartX:=X; StartY:=Y;
end;

procedure BoxMouseUp;
begin
LeftDown:=False;
end;


{-----------------Input Image Mouse event handlers ---------------------}


procedure TImageEditDlg.InputImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
BoxMouseDown(X,Y);
end;

procedure TImageEditDlg.InputImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
BoxMouseMove(X,Y);
end;

procedure TImageEditDlg.InputImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
BoxMouseUp;
end;



{------------------Routine to resize image ---------------------------}


function GetSharpness: integer;
{Get sharpness Index from control}
begin
case ImageEditDlg.SharpGroup.ItemIndex of
 0: Result:=4;
 1: Result:=5;
 2: Result:=6;
 else Result:=6;
 end;
end;




procedure ResizeImage;
{Resize InputImage and put in Output Image}
begin
with ImageEditDlg do
 begin
 if (NewSizeX<0) or (NewSizeY<0) then
	begin
	MessageDlg('Negative Image Size',mtWarning,[mbOk],0);
	exit;
	end;
 DisplayProgress('Resizing Image');
 try
 HandleBrightnessChange;
 ResetBrightContrastGamma(False);
 StretchBitmap(InputImage.Picture.Bitmap,NewSizeX,NewSizeY, GetSharpness, DoProgress);
 SetUndoItem;
 InputImage.Invalidate;
 InitializeImage;
 finally HideProgress; end;
 end;
end;


{------------------ General Event Handlers ----------------------------}




procedure TImageEditDlg.FormCreate(Sender: TObject);
begin
FirstActivate:=True;
InitializeImage;
PageControl.ActivePage:=ResizePage;
FileUndo:=TFileUndo.Create(10);
BaseBitmap:=TBitmap.Create;
InScrollBox.DoubleBuffered:=True;
end;




procedure TImageEditDlg.FormDestroy(Sender: TObject);
begin
FileUndo.Free;
BaseBitmap.Free;
end;



{------------------ Menu/Button Event Handlers ----------------------------}




procedure TImageEditDlg.StretchMnuClick(Sender: TObject);
begin
HideBox;
ResizeImage;
InitializeImage;
end;


procedure TImageEditDlg.ClipBtnClick(Sender: TObject);
begin
HideBox;
HandleBrightnessChange;
if ClipBitmap(InputImage.Picture.Bitmap,OldBox) then
	begin
	SetUndoItem;
	InitializeImage;
	end
else RestoreBox;
end;



procedure TImageEditDlg.ResetMeasureBtnClick(Sender: TObject);
begin
end;


{-------------------------------------------------------------}



procedure TImageEditDlg.UnitsRadioClick(Sender: TObject);
begin
NormalizeSize;
end;

procedure TImageEditDlg.Splitter1Moved(Sender: TObject);
begin
if Splitter1.Left>144 then PageControl.Width:=144;
end;


procedure TImageEditDlg.SearchHelpMnuClick(Sender: TObject);
begin
Application.HelpCommand(HELP_PARTIALKEY,Longint(PChar('')));
end;


procedure TImageEditDlg.BrightnessBarChange(Sender: TObject);
var Brightness,Contrast,Gamma: double;
begin
HideBox;
BrightnessChange:=True;
GetBarSettings(Brightness,Contrast,Gamma);
DisplayBarValues(brightness,Contrast,Gamma);
AdjustBrightnessContrastGamma(BaseBitmap,InputImage.Picture.Bitmap,Brightness,Contrast,Gamma);
InputImage.Invalidate;
InitializeImage;
end;



procedure TImageEditDlg.ResetBCGBtnClick(Sender: TObject);
begin
ResetBrightContrastGamma(True);
end;


procedure TImageEditDlg.WidthSpinChange(Sender: TObject);
begin
if not AspectChk.Checked then exit;
HeightSpin.Value:=trunc(WidthSpin.Value/AspectRatio);
end;

procedure TImageEditDlg.HeightSpinChange(Sender: TObject);
begin
if not AspectChk.Checked then exit;
WidthSpin.NoEventValue:=trunc(HeightSpin.Value*AspectRatio);
end;



procedure TImageEditDlg.UndoBtnClick(Sender: TObject);
begin
RestoreUndoItem;
end;


procedure TImageEditDlg.CopyBtnClick(Sender: TObject);
var MyFormat : Word;
var AData : Thandle;
var APalette: HPALETTE;
begin
HideBox;
InputImage.Picture.SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
RestoreBox;
end;


procedure TImageEditDlg.PasteBtnClick(Sender: TObject);
begin
HandleBrightnessChange;
InputImage.Picture.LoadFromClipBoardFormat(cf_BitMap,ClipBoard.GetAsHandle(cf_Bitmap),0);
SetUndoItem;
InitializeImage;
end;



function DoEditImage(Bitmap: TBitmap; HelpContext: integer): boolean;
begin
Result:=False;
BrightnessChange:=False;
ImageEditDlg.HelpBtn.HelpContext:=HelpContext;
ImageEditDlg.HelpBtn.Visible:=HelpContext>0;
ImageEditDlg.InputImage.AutoSize:=False;
ImageEditDlg.InputImage.Picture.Assign(Bitmap);
ImageEditDlg.InputImage.AutoSize:=True;
BaseBitmap.Assign(Bitmap);
InitializeImage;
ImageEditDlg.ShowModal;
if ImageEditDlg.ModalResult = mrOK then
	begin
	HideBox;
	Bitmap.Assign(ImageEditDlg.InputImage.Picture);
	Result:=True;
	end;
end;


end.
