unit IEDodge;

interface

uses Windows,SysUtils,Graphics,Types,Classes,Math,DSupport,IntVector,BitmapLib;

type TDodgeBurnMode = (dmHighlight,dmMidtones,dmShadows);


procedure DoDodge(Start: boolean; X,Y,Size: integer; Dodge: boolean; Mode: TDodgeBurnMode; Expose: double);
procedure DoSmudge(Start: boolean; X,Y,Size: integer; Depth: double);
procedure DoEnhance(Start: boolean; X,Y,Size: integer; Bright,Contrast,Gamma,Hue,Saturation: double);


implementation

uses ImageEditor, IEClipper, IEPolygonLib;

var TBMP: TBitmap;


{-------------- Dodge/Burn Code ====------------------------------------------}

{ https://losingfight.com/blog/2007/09/27/how-to-implement-dodge-and-burn-tools/ }





function DodgeHighlights(C, Expose: double): double;
begin
Result:=C + Expose * (Exp(C) - 1);
end;



function DodgeMidtones(C, Expose: double): double;
var Factor: double;
begin
Factor:=Expose * 0.25;
Result:=C + Factor * Sin(C * Pi);
end;



function DodgeShadows(C, Expose: double): double;
var Factor: double;
begin
Factor:=(1 - Expose * 0.5);
Result:=factor * C + (1.0 - factor);
end;



function BurnHighlights(C, Expose: double): double;
var Factor: double;
begin
Factor:= (1 - Expose * 0.75);
Result:= factor * C;
end;



function BurnMidtones(C, Expose: double): double;
var Factor: double;
begin
Factor:=Expose * 0.25;
Result:= C - Factor * Sin(C * Pi);
end;


function BurnShadows(C, Expose: double): double;
var Factor: double;
begin
Factor:=Expose;
Result:=C + factor * (1.0 - exp(1.0 - C));
end;


function DodgeBurnValue(V: byte; Dodge: boolean; Mode: TDodgeBurnMode; Expose: double): byte;
var C: double;
begin
C:=V / 255;
if Dodge then
	begin
	case Mode of
	 dmHighlight: C:=DodgeHighlights(C, Expose);
	 dmMidtones: C:=DodgeMidtones(C, Expose);
	 dmShadows: C:=DodgeShadows(C, Expose);
	 end;
	end
else
	begin
	case Mode of
	 dmHighlight: C:=BurnHighlights(C, Expose);
	 dmMidtones: C:=BurnMidtones(C, Expose);
	 dmShadows: C:=BurnShadows(C, Expose);
	 end;
	end;
C:=C *255;
if C>255 then C:=255;
if C<0 then C:=0;
Result:=round(C);
end;


type TDBColor = (dbRed,dbGreen,dbBlue);

type TDBColors = set of TDBColor;


procedure DodgeBurn(BMP: TBitmap; MouseX, MouseY,Size: integer; Dodge: boolean; Colors: TDBColors; Mode: TDodgeBurnMode; Expose: double);
var X,Y,X1,Y1: integer;
var PixelColor: TRGBTriple;
begin
BMP.PixelFormat:=pf24Bit;
for Y:=0 to Size-1 do
	begin
	Y1:=MouseY + Y;
	Y1:=EnsureRange(Y1,0,BMP.Height-1);
	for x:=0 to Size-1 do
		begin
		X1:=MouseX + X;
		X1:=EnsureRange(X1,0,BMP.Width-1);
		PixelColor:=ColorToTriple(BMP.Canvas.Pixels[X1,Y1]);
		if dbRed in Colors then PixelColor.rgbtRed:=DodgeBurnValue(PixelColor.rgbtRed, Dodge, Mode, Expose);
		if dbGreen in Colors then PixelColor.rgbtGreen:=DodgeBurnValue(PixelColor.rgbtGreen, Dodge, Mode, Expose);
		if dbBlue in Colors then PixelColor.rgbtBlue:=DodgeBurnValue(PixelColor.rgbtBlue, Dodge, Mode, Expose);
		BMP.Canvas.Pixels[X1,Y1]:=BitmapLib.TripleToColor(PixelColor);
		end;
	end;
end;



var LastX,LastY: integer;

procedure DoDodge(Start: boolean; X,Y,Size: integer; Dodge: boolean; Mode: TDodgeBurnMode; Expose: double);
var T: double;
var R: TRect;
var C: TDBColors;
begin
if Start then
	begin
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
        end;

C:=[];
if ImageEditorDlg.RedChk.Checked then C:=C + [dbRed];
if ImageEditorDlg.GreenChk.Checked then C:=C + [dbGreen];
if ImageEditorDlg.BlueChk.Checked then C:=C + [dbBlue];


if (LastX=X) and (LastY=Y) then exit;
PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
DodgeBurn(ImageEditorDlg.Image.Picture.Bitmap, X,Y,Size, Dodge, C, Mode, Expose);
PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

ImageEditorDlg.Image.Invalidate;
LastX:=X;
LastY:=Y;
end;


{--------------- Smudge Code --------------------------------------------------}


{https://openframeworks.cc/documentation/math/ofMath/#show_ofDist}

{https://github.com/ptterb/SmudgeTest}



function Distance(X1,Y1,X2,Y2: double): double;
var DX,DY: double;
begin
DX:=X2 - X1;
DY:=Y2 - Y1;
Result:=Sqrt(Sqr(DX) + Sqr(DY));
end;


function MapValue(Value,InputMin,InputMax,OutputMin,OutputMax: double; Clamp: boolean): double;
{Map value in the InputMin to InputMax range to OutputMin to OutputMax range}
var OutVal: double;
const FLT_EPSILON = 1.19209290E-07;
begin
if abs(InputMin - InputMax) < FLT_EPSILON then Result:=outputMin
else
	begin
	outVal:=((value - inputMin) / (inputMax - inputMin) * (outputMax - outputMin) + outputMin);
	if  clamp  then
		begin
		if outputMax < outputMin then
			begin
			if  outVal < outputMax  then outVal:=outputMax
			else if  outVal > outputMin  then outVal:=outputMin;
			end
		else
			begin
			if  outVal > outputMax  then outVal:=outputMax
			else if  outVal < outputMin  then outVal:=outputMin;
			end;
		end;
	Result:=outVal;
	end;
end;


function ofLerp(start, stop, amt: double): byte;
begin
Result:=Trunc(start + (stop-start) * amt);
end;


procedure Smudge(mouseX, mouseY: integer; Brush,BackGround: TBitmap; Depth: double);
var X,Y,X2,Y2: integer;
var brushColor,BackColor: TRGBTriple;
var dist,c2,blurdist: double;
var BrushScan,BackScan: pRGBTripleArray;
var HW,HH: double;
begin
HW:=(Brush.Width+1)/2-1;
HH:=(Brush.Height+1)/2-1;
c2:=sqrt(Sqr(HW) + Sqr(HH) );
for Y:=0 to Brush.Height-1 do
	begin
	Y2:=MouseY + Y;
	Y2:=EnsureRange(Y2,0,Background.Height-1);
	BrushScan:=Brush.ScanLine[Y];
	BackScan:=Background.ScanLine[Y2];
	for x:=0 to Brush.Width-1 do
		begin
		X2:=MouseX + X;
		if X2>=Background.Width then X2:=Background.Width-1;
		brushColor:=BrushScan[x];
		BackColor:=BackScan[X2];

		dist:=Distance(x, y, HW, HH);

		if Brush.Width<=1 then BlurDist:=Depth
		else blurDist:=MapValue(dist, 0.0, c2 , 0.0, 1.0, True) * Depth;


		BackColor.rgbtRed:=ofLerp(BrushColor.rgbtRed, BackColor.rgbtRed, 1-blurDist);
		BackColor.rgbtGreen:=ofLerp(BrushColor.rgbtGreen, BackColor.rgbtGreen, 1-blurDist);
		BackColor.rgbtBlue:=ofLerp(BrushColor.rgbtBlue, BackColor.rgbtBlue, 1-blurDist);
		BackGround.Canvas.Pixels[X2,Y2]:=TripleToColor(BackColor);
		BrushScan[X]:=BackColor;
		end;
	end;
end;

{------------------------------------------------------------------------------}

procedure DoSmudge(Start: boolean; X,Y,Size: integer; Depth: double);
var TargetRect,BMPRect: TRect;
begin
BMPRect:=Rect(0,0,Size,Size);
TargetRect:=BMPRect;
OffsetRect(TargetRect,X,Y);
if Start then
	begin
        ImageEditorDlg.Image.Picture.Bitmap.PixelFormat:=pf24Bit;
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
	TBMP.Width:=Size; TBMP.Height:=Size;
	SetStretchBltMode(TBMP.Canvas.Handle, HALFTONE);
	TBMP.Canvas.CopyMode:=cmSrcCopy;
	TBMP.Canvas.CopyRect(BMPRect,ImageEditorDlg.Image.Canvas,TargetRect);
        end
else
	begin
	PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
	Smudge(X, Y,TBMP,ImageEditorDlg.Image.Picture.Bitmap,Depth);
	PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

	ImageEditorDlg.Image.Invalidate;
	end;
end;



{----------------- Enhance Code -----------------------------------------------}


procedure Enhance(BMP: TBitmap; MouseX, MouseY,Size: integer; Brightness,Contrast,Gamma,Hue,Saturation: double);
var X,Y,X1,Y1: integer;
var PixelColor: TRGBTriple;
var HSB: THSBRecord;
var Scale: double;

	function GetAdjust(V: integer): integer;
	var Tmp: double;
	const Gray = 128;
	begin
	Tmp:=Power(V,Gamma) * Scale;
	Tmp:=((Tmp - Gray) * Contrast) + Gray;
	Result:=trunc(EnsureRange(Tmp * Brightness,0,255));
	end;

begin
{Change Brightness/Contrast ranges from 0 - 1 to 0.1 to 10 }
Brightness:=Power(10,(Brightness - 0.5)*2);
Contrast:=Power(10,(Contrast - 0.5)*2);

{Change Gamma range from 0 - 1 to 0.1 to 10 }
Gamma:=Power(10,(Gamma - 0.50) * 2);
if Gamma<=0 then Gamma:=100 else Gamma:=1/Gamma;

{Used to scale gamma values back to 0 to 255 range}
Scale:=256/power(256,Gamma);


BMP.PixelFormat:=pf24Bit;
for Y:=0 to Size-1 do
	begin
	Y1:=MouseY + Y;
	Y1:=EnsureRange(Y1,0,BMP.Height-1);
	for x:=0 to Size-1 do
		begin
		X1:=MouseX + X;
		X1:=EnsureRange(X1,0,BMP.Width-1);
		PixelColor:=ColorToTriple(BMP.Canvas.Pixels[X1,Y1]);
		{Handle Hue}
		HSB:=RGBToHSB(PixelColor);
		HSB.Hue:=HSB.Hue + Hue;
		while HSB.Hue>360 do HSB.Hue:=HSB.Hue -360;
		while HSB.Hue<0 do HSB.Hue:=HSB.Hue +360;

		{Handle saturation}
		HSB.Sat:=HSB.Sat + Saturation;
		if HSB.Sat>1 then HSB.Sat:=1;
		if HSB.Sat<0 then HSB.Sat:=0;

		PixelColor:=HSBtoRGB(HSB);

                PixelColor.rgbtRed:=GetAdjust(PixelColor.rgbtRed);
                PixelColor.rgbtGreen:=GetAdjust(PixelColor.rgbtGreen);
                PixelColor.rgbtBlue:=GetAdjust(PixelColor.rgbtBlue);

		BMP.Canvas.Pixels[X1,Y1]:=TripleToColor(PixelColor);
		end;
	end;
end;



procedure DoEnhance(Start: boolean; X,Y,Size: integer; Bright,Contrast,Gamma,Hue,Saturation: double);
var T: double;
var R: TRect;
var C: TDBColors;
begin
if Start then
	begin
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
        end;

if (LastX=X) and (LastY=Y) then exit;

PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

Enhance(ImageEditorDlg.Image.Picture.Bitmap, X,Y,Size, Bright,Contrast,Gamma,Hue,Saturation);
PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

ImageEditorDlg.Image.SaveImage;
ImageEditorDlg.Image.Invalidate;
LastX:=X;
LastY:=Y;
end;


{------------------------------------------------------------------------------}



initialization
TBMP:=TBitmap.Create;
TBMP.PixelFormat:=pf24bit;
finalization
TBMP.Free;
end.
