unit IEEnhance;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, ComCtrls,BitmapLib, SpinButtons, Realspin;

type
  TEnhancementDlg = class(TForm)
    CancelBtn: TButton;
    BlackWhiteGroup: TGroupBox;
    Panel10: TPanel;
    Label1: TLabel;
    ThresholdBar: TTrackBar;
    BlackWhiteChk: TCheckBox;
    InvertChk: TCheckBox;
    ColorGroup: TGroupBox;
    Label9: TLabel;
    Label10: TLabel;
    Label8: TLabel;
    ResetBCGBtn: TButton;
    BitBtn1: TBitBtn;
    Label6: TLabel;
    Label7: TLabel;
    Panel1: TPanel;
    HueBar: TTrackBar;
    Panel2: TPanel;
    HueSpin: TRealSpin;
    Panel3: TPanel;
    Panel4: TPanel;
    BrightSpin: TRealSpin;
    Panel5: TPanel;
    Panel6: TPanel;
    SatSpin: TRealSpin;
    Panel7: TPanel;
    Panel8: TPanel;
    ContrastSpin: TRealSpin;
    Panel9: TPanel;
    Panel11: TPanel;
    GammaSpin: TRealSpin;
    SaturationBar: TTrackBar;
    ContrastBar: TTrackBar;
    BrightnessBar: TTrackBar;
    GammaBar: TTrackBar;
    procedure BrightnessBarChange(Sender: TObject);
    procedure ResetBCGBtnClick(Sender: TObject);
    procedure BlackWhiteChkClick(Sender: TObject);
    procedure ThresholdBarChange(Sender: TObject);
    procedure HueSpinChange(Sender: TObject);
    procedure SatSpinChange(Sender: TObject);
    procedure BrightSpinChange(Sender: TObject);
    procedure ContrastSpinChange(Sender: TObject);
    procedure GammaSpinChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var EnhancementDlg: TEnhancementDlg;


procedure DoEnhanceDlg;

procedure ResetBrightContrastGamma(UpdateImage: Boolean);
procedure CenterTrackBar(TrackBar: TTrackBar);

implementation

uses ImageEditor, IEClipper, IEPolygonLib;

{$R *.dfm}


var BaseBitmap: TBitmap;


{------------------ Handle Brightness, Contrast and Gamma --------------------}


procedure CenterTrackBar(TrackBar: TTrackBar);
var E: TNotifyEvent;
begin
E:=TrackBar.OnChange;
TrackBar.OnChange:=nil;
try
TrackBar.Position:=(TrackBar.Max + TrackBar.Min) div 2;
finally TrackBar.OnChange:=E; end;
end;



procedure GetBarSettings(var Bright,Contrast,Gamma: double);
begin
Bright:=EnhancementDlg.BrightnessBar.Position/100;
Contrast:=EnhancementDlg.ContrastBar.Position/100;
Gamma:=EnhancementDlg.GammaBar.Position/100;
end;


procedure DisplayBarValues(var Bright,Contrast,Gamma: double);
begin
EnhancementDlg.GammaSpin.NoEventValue:=Gamma;
EnhancementDlg.BrightSpin.NoEventValue:=Bright;
EnhancementDlg.ContrastSpin.NoEventValue:=Contrast;
end;



procedure ResetBrightContrastGamma(UpdateImage: Boolean);
begin
CenterTrackBar(EnhancementDlg.BrightnessBar);
CenterTrackBar(EnhancementDlg.ContrastBar);
CenterTrackBar(EnhancementDlg.GammaBar);
CenterTrackBar(EnhancementDlg.HueBar);
CenterTrackBar(EnhancementDlg.SaturationBar);

if UpdateImage then EnhancementDlg.BrightnessBarChange(EnhancementDlg);
end;


procedure TEnhancementDlg.BrightnessBarChange(Sender: TObject);
var Brightness,Contrast,Gamma: double;
var BMP: TBitmap;
var Hue,Saturation: double;
begin
GetBarSettings(Brightness,Contrast,Gamma);
DisplayBarValues(brightness,Contrast,Gamma);

Hue:=HueBar.Position/10;
HueSpin.NoEventValue:=Hue;

Saturation:=SaturationBar.Position;
Saturation:=Saturation/100;
SatSpin.NoEventValue:=Saturation;

BMP:=PolyClipper.InitializeBMPClip(ImageEditorDlg.Image.Picture.Bitmap);
AdjustHue(BaseBitmap,BMP,Hue);
AdjustSaturation(BMP,BMP,Saturation);
AdjustBrightnessContrastGamma(BMP,BMP,Brightness,Contrast,Gamma);


PolyClipper.DoBMPClip(ImageEditorDlg.Image.Picture.Bitmap);

ImageEditorDlg.Image.Invalidate;
end;






procedure TEnhancementDlg.ResetBCGBtnClick(Sender: TObject);
begin
ResetBrightContrastGamma(True);
end;


procedure TEnhancementDlg.BlackWhiteChkClick(Sender: TObject);
begin
if BlackWhiteChk.Checked then
	begin
	MakeBlackAndWhite(BaseBitmap,PolyClipper.InitializeBMPClip(ImageEditorDlg.Image.Picture.Bitmap),InvertChk.Checked,ThresholdBar.Position/1000);
	PolyClipper.DoBMPClip(ImageEditorDlg.Image.Picture.Bitmap);
	end
else ImageEditorDlg.Image.Picture.Bitmap.Assign(BaseBitmap);
ImageEditorDlg.Image.Invalidate;
end;



procedure TEnhancementDlg.ThresholdBarChange(Sender: TObject);
begin
if BlackWhiteChk.Checked then BlackWhiteChkClick(Self);
end;





procedure DoEnhanceDlg;
begin
BaseBitmap.Assign(ImageEditorDlg.Image.Picture.Bitmap);
EnhancementDlg.ShowModal;
BaseBitmap.ReleaseHandle;
BaseBitmap.Width:=0;
BaseBitmap.Height:=0;
end;



procedure TEnhancementDlg.HueSpinChange(Sender: TObject);
begin
HueBar.Position:=trunc(HueSpin.Value * 10);
end;



procedure TEnhancementDlg.SatSpinChange(Sender: TObject);
begin
SaturationBar.Position:=trunc(SatSpin.Value * 100);
end;



procedure TEnhancementDlg.BrightSpinChange(Sender: TObject);
begin
BrightnessBar.Position:=trunc(BrightSpin.Value * 100);
end;



procedure TEnhancementDlg.ContrastSpinChange(Sender: TObject);
begin
ContrastBar.Position:=trunc(ContrastSpin.Value * 100);
end;





procedure TEnhancementDlg.GammaSpinChange(Sender: TObject);
begin
GammaBar.Position:=trunc(GammaSpin.Value * 100);
end;



initialization
BaseBitmap:=TBitmap.Create;
finalization
BaseBitmap.Free;
end.
