unit IEPenDrawing;

interface

uses Windows,Classes,Types,Graphics,SysUtils,ExtCtrls,DSupport,IntVector,BitmapLib,ImageEx,DottedBox;

procedure DrawOnImage(Start: boolean; X,Y,Size: integer; Scale: double;  C: TColor);

procedure DrawPenBox1(Image: TImageEx; X,Y,Size: integer; Scale: double; Modulo: boolean);
procedure DrawPenBox2(Image: TImageEx; X,Y,Size: integer; Scale: double; Modulo: boolean);
procedure GetModuloPixel(X,Y: integer; var X2,Y2: integer; Size: integer; Scale: double);

procedure SamplePixel(X,Y: integer; Shift: TShiftState);
procedure SprayDraw(Start: boolean; X,Y,Dense,Size: integer; C: TColor);


implementation

uses ImageEditor, IEPolygonLib, IEClipper;


{----------------------------------------------------------------------------}


procedure SamplePixel(X,Y: integer; Shift: TShiftState);
var C: TColor;
begin
C:=ImageEditorDlg.Image.Canvas.Pixels[X,Y];
if ssLeft in Shift then ImageEditorDlg.LeftColorDis.Color:=C
else if ssRight in Shift then ImageEditorDlg.RightColorDis.Color:=C;
end;



{------------------------------------------------------------------------------}



procedure GetBoxModulo(var X,Y: integer; Size: integer; Scale: double);
{Make X,Y shift by even multiples of the pixel size at this scale}
{Shift takes place 1/2 pixel from the center of the pen-size}
{Tis is used for the drawing of box that represents the pen size}
var Off: double;
begin
Off:=(Size-1)/2;
X:=trunc(trunc((X / Scale)-Off) * Scale);
Y:=trunc(trunc((Y / Scale)-Off) * Scale);
end;


procedure GetModuloPixel(X,Y: integer; var X2,Y2: integer; Size: integer; Scale: double);
{Make unscaled coordinates shift to match the scaled version above}
begin
GetBoxModulo(X,Y,Size,Scale);
X2:=round(X/Scale); Y2:=round(Y/Scale);
end;



function GetPenBox(X,Y,Size: integer; Scale: double; Modulo: boolean): TRect;
{Get rectangle that describes pen size}
var ScaledSize: integer;
begin
if Modulo then GetBoxModulo(X,Y,Size,Scale);
ScaledSize:=Trunc(Size*Scale);
Result:=Rect(X,Y,X+ScaledSize,Y+ScaledSize);
end;




procedure DrawPenBox(Box: TDottedBox; Image: TImageEx; X,Y,Size: integer; Scale: double; Modulo: boolean);
begin
if Size<2 then exit;
Box.HideRectangle;
Box.Box:=GetPenBox(X,Y,Size,Scale,Modulo);
Box.ShowRectangle;
end;



procedure DrawPenBox1(Image: TImageEx; X,Y,Size: integer; Scale: double; Modulo: boolean);
begin
DrawPenBox(ImageEditorDlg.DottedBox1, Image, X,Y,Size, Scale,Modulo);
end;




procedure DrawPenBox2(Image: TImageEx; X,Y,Size: integer; Scale: double; Modulo: boolean);
var R: TRect;
begin
DrawPenBox(ImageEditorDlg.DottedBox2, Image, X,Y,Size, Scale,Modulo);
end;


{------------------------------------------------------------------------------}


procedure SketchDraw(Start: boolean; X,Y,Size: integer; C: TColor);
begin
if Start then
	begin
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
        end;
PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

ContinuousDraw(ImageEditorDlg.Image,Start,X,Y,Size,C);
PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
ImageEditorDlg.Image.Invalidate;
end;


{-----------------------------------------------------------------------------}


var RubberStart,RubberEnd: TPoint;
var LastSize: integer;


procedure DrawRubberLine(C: TColor; Size: integer);
var L: double;
begin
with ImageEditorDlg do
 begin
 Image.Canvas.Pen.Color:=C;
 Image.Canvas.Pen.Width:=Size;
 Image.Canvas.MoveTo(RubberStart.X,RubberStart.Y);
 Image.Canvas.LineTo(RubberEnd.X,RubberEnd.Y);
 LastSize:=Size;
 end;
end;


procedure HideOldRubberLine;
var R: TRect;
begin
R.TopLeft:=RubberStart;
R.BottomRight:=RubberEnd;
R:=NormalizeRect(R);
InflateRect(R,LastSize,LastSize);
ImageEditorDlg.Image.RestoreRectangle(R);
end;


procedure SetRubberStart(X,Y: integer; C: TColor; Size: integer);
begin
RubberStart:=Point(X,Y);
RubberEnd:=Point(X,Y);
DrawRubberLine(C, Size);
end;


procedure SetRubberEnd(X,Y: integer; C: TColor; Size: integer);
begin
HideOldRubberLine;
RubberEnd:=Point(X,Y);
DrawRubberLine(C, Size);
end;



procedure RubberBandDraw(Start: boolean; X,Y,Size: integer; C: TColor);
begin
if Start then
	begin
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
	end;

PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
if Start then SetRubberStart(X,Y,C,Size)
else SetRubberEnd(X,Y,C,Size);
PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

ImageEditorDlg.Image.Invalidate;
end;


{-----------------------------------------------------------------------------}





procedure DrawOnImage(Start: boolean; X,Y,Size: integer; Scale: double;  C: TColor);
begin
if ImageEditorDlg.RubberChk.Checked then RubberBandDraw(Start,X,Y,Size,C)
else SketchDraw(Start,X,Y,Size,C);
end;


{------------------------------------------------------------------------------}


procedure ScatterPixels(Image: TImage; X,Y, Dense, Size: integer; C: TColor);
var OX,OY,X1,Y1: integer;
var S1,S2: integer;
begin
if Size<=Dense then Size:=Dense+1;
for OY:=0 to Size-1 do
 for OX:=0 to Size-1 do
  if (OX mod Dense)=0 then
  if (OY mod Dense)=0 then
	begin
	X1:=X+OX; Y1:=Y+OY;
	if (X1>=0) and (X1<Image.Width) or
	   (Y1>=0) and (Y1<Image.Height) then Image.Canvas.Pixels[X1,Y1]:=C;
	end;
end;


procedure SprayDraw(Start: boolean; X,Y,Dense,Size: integer; C: TColor);
begin
if Start then
	begin
	ImageEditorDlg.Image.RestoreImage;
	ImageEditorDlg.ImageUndoer.StoreUndo;
        end;
PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);

ScatterPixels(ImageEditorDlg.Image,X,Y,Dense,Size,C);

PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
ImageEditorDlg.Image.Invalidate;
end;





end.
