unit IEPolygonLib;

interface

uses Windows,Classes,Types,ExtCtrls,Graphics,GLPolygons,Vector,HugeArry,Math,GraphicsLib,BitmapLib;


type THandleType = (htNone,htPoint,htSpline,htLathe);

type THitType = (htPolygon,htHandle,htControl1,htControl2,htMove);

type THitPoint = record
 Valid: boolean;
 PolyInx,PointInx: integer;
 HitType: THitType;
 end;



type TLineInfo = record
 PolyInx,PntInx1: integer;
 Distance: double;
 Intercept: T3DVector;
 end;



var PolygonArray: TRealPolygonArray;
var HitPoint,SelectPoint: THitPoint;

procedure DrawSplinePolygon(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer; Close: boolean); overload;
procedure DrawSplinePolygon(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer); overload;
procedure DrawSplineLine(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer);
procedure DrawOpenPolygon(Canvas: TCanvas; PG: TRealPolygon; HandleType: THandleType; Selected: boolean; Segments: integer);

function PolygonLineHitTest(Pos: T2DVector; PG: TRealPolygon; var LineInfo: TLineInfo; Segments: integer; Closed: boolean): boolean; overload;
function PolygonLineHitTest(Pos: T2DVector; PG: TRealPolygon; var LineInfo: TLineInfo; Segments: integer): boolean; overload;

procedure DrawPolygonHandles(Canvas: TCanvas; RP: TRealPolygon);
procedure DrawControlHandles(Canvas: TCanvas; RP: TRealPolygon);
procedure ScaleRealPolygon(PG: TRealPolygon; Scale: double);
procedure DrawPolygonLoop(Canvas: TCanvas; PG: TRealPolygon; HandleType: THandleType; Selected: boolean; Segments: integer);
procedure HighlightSelectedPoint(Canvas: TCanvas; PG: TRealPolygon; Inx: integer);

procedure MoveControlPoints(Points,Opposites: THuge2DVector; Inx: integer; Pos,V: T2DVector; Shift: TShiftState);

procedure ClearPaintBox(PaintBox: TPaintBox);


implementation




procedure ClearPaintBox(PaintBox: TPaintBox);
begin
PaintBox.Canvas.Brush.Color:=clSilver;
PaintBox.Canvas.Pen.Color:=clBlack;
PaintBox.Canvas.Rectangle(PaintBox.ClientRect);
end;

{-------------------- Draw Polygons and Point Handles -------------------------}



function VectorArray2DToPolygon(VA: T2DVectorArray): TPolygon;
var I: integer;
begin
SetLength(Result,Length(VA));
for I:=0 to High(VA) do Result[I]:=Vector2DToPoint(VA[I]);
end;



procedure DrawSplinePolygon(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer; Close: boolean);
var Poly: T2DVectorArray;
var I,J,Count: integer;
begin
Canvas.Brush.Style:=bsClear;
Canvas.Pen.Color:=C;
if Close then Count:=RP.Count-1 else Count:=RP.Count-2;
for I:=0 to Count do
	begin
	GetSplineLine(I,RP,Steps,Poly);
	Canvas.Polyline(VectorArray2DToPolygon(Poly));
	end;
end;


procedure DrawSplinePolygon(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer);
begin
DrawSplinePolygon(canvas,RP,C,Steps,True);
end;



procedure DrawSplineLine(Canvas: TCanvas; RP: TRealPolygon; C: TColor; Steps: integer);
begin
DrawSplinePolygon(canvas,RP,C,Steps,False);
end;



procedure DrawPolygonHandles(Canvas: TCanvas; RP: TRealPolygon);
var I: integer;
begin
for I:=0 to RP.Count-1 do
	begin
	DrawSquareHandle(Canvas,Vector2DToPoint(RP[I]),clBlack);
	end;
end;



procedure DrawControlHandles(Canvas: TCanvas; RP: TRealPolygon);
var I: integer;

	procedure DrawHandleLine(V1,V2: T2DVector);
	var P1,P2: TPoint;
	begin
	P1:=Vector2DToPoint(V1); P2:=Vector2DToPoint(VectorAdd2D(V1,V2));
	Canvas.MoveTo(P1.X,P1.Y); Canvas.LineTo(P2.X,P2.Y);
	DrawRoundHandle(Canvas,P2,clLime);
	end;


begin
Canvas.Pen.Color:=clLime;
for I:=0 to RP.Count-1 do
	begin
	DrawHandleLine(RP[I],RP.ControlPoints1[I]);
	DrawHandleLine(RP[I],RP.ControlPoints2[I]);
	end;
end;



procedure ScaleRealPolygon(PG: TRealPolygon; Scale: double);
var J: integer;
begin
for J:=0 to PG.Count-1 do
	begin
	PG[J]:=ScalarProduct2D(PG[J],Scale);
	PG.ControlPoints1[J]:=ScalarProduct2D(PG.ControlPoints1[J],Scale);
	PG.ControlPoints2[J]:=ScalarProduct2D(PG.ControlPoints2[J],Scale);
	end;
end;



procedure DrawPolygonLoop(Canvas: TCanvas; PG: TRealPolygon; HandleType: THandleType; Selected: boolean; Segments: integer);
var C: TColor;
begin
if Selected then C:=clRed
else C:=clBlue;
DrawSplinePolygon(Canvas,PG,C,Segments);
if Selected then
        begin
        case HandleType of
         htNone: ;
         htPoint: DrawPolygonHandles(Canvas,PG);
         htSpline: DrawControlHandles(Canvas,PG);
         end;
        end;
end;





procedure DrawOpenPolygon(Canvas: TCanvas; PG: TRealPolygon; HandleType: THandleType; Selected: boolean; Segments: integer);
var C: TColor;
begin
if Selected then C:=clRed
else C:=clBlue;
DrawSplineLine(Canvas,PG,C,Segments);
if Selected then
        begin
        case HandleType of
         htNone: ;
         htPoint: DrawPolygonHandles(Canvas,PG);
         htSpline: DrawControlHandles(Canvas,PG);
         end;
        end;
end;


procedure HighlightSelectedPoint(Canvas: TCanvas; PG: TRealPolygon; Inx: integer);
var P1,P2: TPoint;
var J: integer;
begin
if Inx>=PG.Count-1 then J:=0 else J:=Inx+1;
P1:=PG.Points[Inx]; P2:=PG.Points[J];
DrawSquareHandle(Canvas,P1,clRed);
end;



function SignedVectorAngle2D(V: T2DVector): double;
begin
V:=Vector.UnitVector2D(V);
Result:=ArcTan2(V.Y,V.X);
end;




procedure MoveControlPoints(Points,Opposites: THuge2DVector; Inx: integer; Pos,V: T2DVector; Shift: TShiftState);
var A,L: double;
var V1: T2DVector;
begin
Points[Inx]:=VectorSubtract2D(Pos,V);
if (ssCtrl in Shift) or (ssShift in Shift) then
	begin
	if (ssCtrl in Shift) then L:=VectorABS2D(Opposites[Inx])
	else L:=VectorABS2D(Points[Inx]);
	A:=SignedVectorAngle2D(Points[Inx]);
	A:=1.5 * Pi - A;
	V1:=MakeVector2D(0,1);
	Vector.RotatePoint2D(V1,MakeVector2D(0,0),RadToDeg(A));
	Opposites[Inx]:=ScalarProduct2D(V1,L);
	end;
end;




{----------------- Line Intercept Testing Routines ----------------------------}




procedure PointLineDistance(P,L1,L2: TPoint; var Distance: double; var Intercept: T3DVector);
var P3D,L3D1,L3D2: T3DVector;
begin
P3D:=PointToVector3D(P);
L3D1:=PointToVector3D(L1);
L3D2:=PointToVector3D(L2);
PointLineSegDist3D(P3D,L3D1,L3D2,Distance,Intercept);
end;





procedure PointSplineDistance(I: integer; RP: TRealPolygon; P: T2DVector; var Distance: double; var Intercept: T3DVector; Segments: integer);
var P3D,L3D1,L3D2: T3DVector;
var Poly: T2DVectorArray;
var J: integer;
var Int: T3DVector;
var Dist: double;
begin
Distance:=1E9;
P3D:=Vector2DToVector3D(P);
GetSplineLine(I,RP,Segments,Poly);
for J:=0 to High(Poly)-1 do
	begin
	L3D1:=Vector2DToVector3D(Poly[J]);
	L3D2:=Vector2DToVector3D(Poly[J+1]);
	PointLineSegDist3D(P3D,L3D1,L3D2,Dist,Int);
	if Dist<Distance then
		begin
		Distance:=Dist;
		Intercept:=Int;
		end;
	end;
end;



function PolygonLineHitTest(Pos: T2DVector; PG: TRealPolygon; var LineInfo: TLineInfo; Segments: integer; Closed: boolean): boolean;
var J: integer;
var D: double;
var Intercept: T3DVector;
var Cnt: integer;
begin
Result:=False;
if Closed then Cnt:=PG.Count-1 else Cnt:=PG.Count-2;
for J:=0 to Cnt do
	begin
	PointSplineDistance(J,PG,Pos,D,Intercept,Segments);
	if D<LineInfo.Distance then
		begin
		LineInfo.Distance:=D;
		LineInfo.PntInx1:=J;
		LineInfo.Intercept:=Intercept;
		Result:=True;
		end;
	end;
end;



function PolygonLineHitTest(Pos: T2DVector; PG: TRealPolygon; var LineInfo: TLineInfo; Segments: integer): boolean;
begin
Result:=PolygonLineHitTest(Pos,PG,LineInfo,Segments,True);
end;



initialization
PolygonArray:=TRealPolygonArray.Create;
finalization
PolygonArray.Free;
end.
