unit IntVector;

interface

uses Windows,Types,Math,SysUtils,Vector,DSupport;

{Integer Lines routines}

type TLine = record
 P1,P2: TPoint;
 end;

function Line(P1,P2: TPoint): TLine; overload;
function Line(X1,Y1,X2,Y2: integer): TLine; overload;

{3D point routines }

type TPoint3D = packed record
 X,Y,Z: integer;
 end;



function Point3D(X,Y,Z: Integer): TPoint3D;
function Point3DToVector3D(P: TPoint3D): T3DVector;
function I3DVectorAdd(V1,V2: TPoint3D): TPoint3D;
function ArePointsEqual3D(P1,P2: TPoint3D): boolean;

{2D point routines }

function IntDotProduct(V1,V2: TPoint): double;
function IntCrossProduct(V1,V2: TPoint): double;
function IntVectorABS(V: TPoint): double;
function IntUnitVector(V: TPoint): TPoint;
function IntVectorAngle(V1,V2: TPoint): double;
function IntScalarAdd(V: TPoint; S: double): TPoint;
function IntScalarSubtract(V: TPoint; S: double): TPoint;
function IntScalarProduct(V: TPoint; S: double): TPoint;
function IntScalarDivide(V: TPoint; S: double): TPoint;
function IntVectorSubtract(V1,V2: TPoint): TPoint;
function IntVectorAdd(V1,V2: TPoint): TPoint;
function IntVectorMultiply(V1,V2: TPoint): TPoint;
function IntVectorDivide(V1,V2: TPoint): TPoint;
procedure IntSwapComponents(var V: TPoint);
procedure SwapPoints(var P1,P2: TPoint);
function PointToString(V: TPoint; Labeled: boolean = True): string;
procedure RotatePoint(var P: TPoint; C: TPoint; Angle: double);
function IntUnsignedVector(V: TPoint): TPoint;
function IntVectorDistance(P1,P2: TPoint): double;
function ArePointsEqual(P1,P2: TPoint): boolean;
function SizeToPoint(S: TSize): TPoint;
function IntPointLineDist(const Point,LineStart,LineEnd: TPoint; var Distance: double; var Intercept: TPoint): boolean; overload;
function IntPointLineDist(const Point,LineStart,LineEnd: TPoint): double; overload;

{Retangle routines }

function GetBottomLeft(R: TRect): TPoint;
function GetTopRight(R: TRect): TPoint;
function FindRectCenter(R: TRect): TPoint;
procedure CenterRectOnPoint(var R: TRect; Center: TPoint);
function CenterRectOnRect(Source,Center: TRect): TRect;
function MakeCenteredRect(X,Y,SizeX,SizeY: integer; Trunc: boolean): TRect;
function NormalizeRect(R: TRect): TRect;
function RectInflate(R1,R2: TRect): TRect;
function RectDeflate(R1,R2: TRect): TRect;
function RectMinMax(R: TRect; P: TPoint): TRect; overload;
function RectMinMax(R1, R2: TRect): TRect; overload;
function GetRectStr(R: TRect; Labeled: boolean): string;
function DoRectsOverlap(R1,R2: TRect): boolean;

{Polygon Routines }

type TPointPolygon = array of TPoint;

function CopyPolygon(Poly: TPointPolygon): TPointPolygon;
procedure RotatePolygon(var PT: TPointPolygon; Center: TPoint; Angle: double);
function GetBoundingBox(A: TPointPolygon): TRect;
procedure OffsetPolygon(var PT: TPointPolygon; Offset: TPoint);
function PointInPolygon(Poly: TPointPolygon; P: TPoint): boolean;
function PolygonIntersect(P1,P2: TPointPolygon): boolean;

implementation



{------------ Integer/Point Vector math routines --------------------}


function IntDotProduct(V1,V2: TPoint): double;
{Do dot product of two vectors}
begin
Result:=(V1.X * V2.X)+(V1.Y*V2.Y);
end;


function IntVectorAdd(V1,V2: TPoint): TPoint;
{Add V1 and V2}
begin
Result.X:= V1.X+V2.X;
Result.Y:= V1.Y+V2.Y;
end;



function IntVectorMultiply(V1,V2: TPoint): TPoint;
{Multiply vector by vector}
begin
Result.X:=round(V1.X * V2.X);
Result.Y:=round(V1.Y * V2.Y);
end;



function IntVectorDivide(V1,V2: TPoint): TPoint;
{Divide vector by vector}
begin
Result.X:=round(V1.X / V2.X);
Result.Y:=round(V1.Y / V2.Y);
end;


function IntVectorSubtract(V1,V2: TPoint): TPoint;
{Subtract V2 from V1}
begin
Result.X:= V1.X-V2.X;
Result.Y:= V1.Y-V2.Y;
end;

function IntCrossProduct(V1,V2: TPoint): double;
{Do crosss product of two vectors - only returns Z value}
begin
Result:= ((V1.X*V2.Y)-(V1.Y*V2.X));
end;

function IntScalarAdd(V: TPoint; S: double): TPoint;
{Add scalar to vector }
begin
Result.X:=Trunc(V.X+S);
Result.Y:=Trunc(V.Y+S);
end;

function IntScalarSubtract(V: TPoint; S: double): TPoint;
{Add scalar to vector }
begin
Result.X:=Trunc(V.X-S);
Result.Y:=Trunc(V.Y-S);
end;

function IntScalarProduct(V: TPoint; S: double): TPoint;
{Multiply vector by scalar}
begin
Result.X:=Trunc(V.X*S);
Result.Y:=Trunc(V.Y*S);
end;


function IntScalarDivide(V: TPoint; S: double): TPoint;
{Divide vector by scalar}
begin
Result.X:=Trunc(V.X/S);
Result.Y:=Trunc(V.Y/S);
end;


function IntVectorABS(V: TPoint): double;
{Find ABS of vector. The T2DVector forces }
{a call to floating point verson of Sqr}
var V1: T2DVector;
begin
V1:=PointToVector2D(V);
Result:=Sqrt(Sqr(V1.X) + Sqr(V1.Y));
end;


function IntUnitVector(V: TPoint): TPoint;
{Return unit vector}
var L: double;
begin
L:=IntVectorABS(V);
if L=0.0 then L:=1E-99;
Result.X:=Trunc(V.X/L);
Result.Y:=Trunc(V.Y/L);
end;



function IntVectorDistance(P1,P2: TPoint): double;
{Subtract P1 - P2 and return distance}
var D: TPoint;
begin
D:=IntVectorSubtract(P1,P2);
Result:=IntVectorABS(D);
end;



function IntUnsignedVector(V: TPoint): TPoint;
{Return Unsiged form of vector}
begin
Result.X:=Abs(V.X);
Result.Y:=Abs(V.Y);
end;



function IntVectorAngle(V1,V2: TPoint): double;
{Return angle of two vectors}
var T,D: double;
begin
T:=IntVectorABS(V1)*IntVectorABS(V2);
if T=0 then Result:=0
else
	begin
	D:=IntDotProduct(V1,V2)/T;
	if abs(D)>1 then Result:=0
	else Result:=ArcCos(D);
	end;
end;



function Vector2DToPoint(V: TPoint): TPoint;
{Convert 2D Vector to a point}
begin
Result.X:=trunc(V.X);
Result.Y:=trunc(V.Y);
end;



function PointToString(V: TPoint; Labeled: boolean = True): string;
{Return point as string.}
begin
if Labeled then Result:='X: '+IntToStr(V.X)+' Y: '+IntToStr(V.Y)
else Result:=IntToStr(V.X)+','+IntToStr(V.Y);
end;



procedure RotatePoint(var P: TPoint; C: TPoint; Angle: double);
{  Rotate point XP,YP around XC,YC  }
var DX,DY: double;
var SinTheta,CosTheta : double;
begin
CosTheta:=cos(Angle * Degree2Radian);
SinTheta:=sin(Angle * Degree2Radian);
DX:=P.X - C.X; DY:=P.Y - C.Y;
P.Y:=Trunc((SinTheta  * -DX) + (CosTheta  * DY) + C.Y);
P.X:=Trunc((CosTheta  * DX) + (SinTheta  * DY) + C.X);
end;



function ArePointsEqual(P1,P2: TPoint): boolean;
{Are the points specified equal?}
begin
Result:=(P1.X = P2.X) and (P1.Y = P2.Y);
end;



function SizeToPoint(S: TSize): TPoint;
{Convert TSize variable to TPoint}
begin
Result.X:=S.CX;
Result.Y:=S.CY;
end;



procedure IntSwapComponents(var V: TPoint);
{Swap vector components}
var T: integer;
begin
T:=V.X; V.X:=V.Y; V.Y:=T;
end;


procedure SwapPoints(var P1,P2: TPoint);
{Swap Points}
var T: TPoint;
begin
T:=P1; P1:=P2; P2:=T;
end;



function IntPointLineDist(const Point,LineStart,LineEnd: TPoint; var Distance: double; var Intercept: TPoint): boolean;
{Get distance between point and line. Returns true if intersect within line segment}
var U,LineMag: double;
var LineLength,StartDist: TPoint;
begin
Result:=False;
LineLength:=IntVectorSubtract(LineEnd,LineStart);
StartDist:=IntVectorSubtract(Point,LineStart);
LineMag:=IntVectorABS(LineLength);

U:=((StartDist.X * LineLength.X) + (StartDist.Y * LineLength.Y)) / ( LineMag * LineMag );
{ Does closest point fall within the line segment }
if U<0 then Intercept:=LineStart
else if U>1 then Intercept:=LineEnd
else
	begin
	Result:=True;
	Intercept:=IntVectorAdd(LineStart,IntScalarProduct(LineLength,U));
	end;
Distance:= IntVectorABS(IntVectorSubtract(Point, Intercept));
end;


function IntPointLineDist(const Point,LineStart,LineEnd: TPoint): double;
var Intercept: TPoint;
begin
IntPointLineDist(Point,LineStart,LineEnd,Result,Intercept);
end;

{--------------- Integer Line handling routines -------------------------------}



function Line(P1,P2: TPoint): TLine;
{Make line from points}
begin
Line.P1:=P1;
Line.P2:=P2;
end;


function Line(X1,Y1,X2,Y2: integer): TLine;
{Make line from integers}
begin
Result:=Line(Point(X1,Y1),Point(X2,Y2));
end;


{------------------ 3D Point handling routines -------------------------------}


function Point3D(X,Y,Z: Integer): TPoint3D;
begin
Result.X:=X;
Result.Y:=Y;
Result.Z:=Z;
end;


function I3DVectorAdd(V1,V2: TPoint3D): TPoint3D;
{Add V1 and V2}
begin
Result.X:= V1.X+V2.X;
Result.Y:= V1.Y+V2.Y;
Result.Z:= V1.Z+V2.Z;
end;


function Point3DToVector3D(P: TPoint3D): T3DVector;
begin
Result.X:=P.X;
Result.Y:=P.Y;
Result.Z:=P.Z;
end;


function ArePointsEqual3D(P1,P2: TPoint3D): boolean;
{Are the points specified equal?}
begin
Result:=(P1.X = P2.X) and (P1.Y = P2.Y) and (P1.Z = P2.Z);
end;


{------------------ Rectanlge handling routines -------------------------------}



function GetBottomLeft(R: TRect): TPoint;
{Get point for bottom left corner of rectangle}
begin
Result.X:=R.Left; Result.Y:=R.Bottom;
end;


function GetTopRight(R: TRect): TPoint;
{Get point for Top right corner of rectangle}
begin
Result.X:=R.Right; Result.Y:=R.Top;
end;


function FindRectCenter(R: TRect): TPoint;
{Find point at center of specified rectangle}
begin
Result:=IntScalarDivide(IntVectorAdd(R.BottomRight,R.TopLeft),2);
end;


procedure CenterRectOnPoint(var R: TRect; Center: TPoint);
{Place center of specified rectangle on the specified point}
var C,D: TPoint;
begin
C:=FindRectCenter(R);
D:=IntVectorSubtract(Center,C);
OffsetRect(R,D.X,D.Y);
end;



function CenterRectOnRect(Source,Center: TRect): TRect;
{Center Source Rect and Center Rect}
var C: TPoint;
begin
Result:=Source;
C:=FindRectCenter(Center);
CenterRectOnPoint(Result,C);
end;



function MakeCenteredRect(X,Y,SizeX,SizeY: integer; Trunc: boolean): TRect;
{Make a Rectangle centered on Specified point of the specified size}
{Trunc controls how to handle sizes that are not evenly divisible by 2}
var SX2,SY2: integer;
var Rem: TPoint;
begin
Rem.X:=SizeX mod 2;
Rem.Y:=SizeY mod 2;
SX2:=SizeX div 2;
SY2:=SizeY div 2;
Result:=Rect(X,Y,X,Y);
InflateRect(Result,SX2,SY2);
if not Trunc then Result.BottomRight:=IntVectorAdd(Result.BottomRight,Rem)
end;


function NormalizeRect(R: TRect): TRect;
{ This routine normalizes a rectangle. It makes sure that the Left,Top}
{ coords are always above and to the left of the Bottom,Right coords.}
begin
Result:=R;
if R.Left > R.Right then SwapIntegerArgs(R.Left > R.Right,Result.Left,Result.Right);
if R.Top > R.Bottom then SwapIntegerArgs(R.Top > R.Bottom,Result.Top,Result.Bottom);
end;


function RectInflate(R1,R2: TRect): TRect;
{Inflate R1 by the amount in R2 }
begin
Result.Left:=R1.Left - R2.Left;
Result.Right:=R1.Right + R2.Right;
Result.Bottom:=R1.Bottom + R2.Bottom;
Result.Top:=R1.Top - R2.Top;
end;


function RectDeflate(R1,R2: TRect): TRect;
{Inflate R1 by the amount in R2 }
begin
Result.Left:=R1.Left + R2.Left;
Result.Right:=R1.Right - R2.Right;
Result.Bottom:=R1.Bottom - R2.Bottom;
Result.Top:=R1.Top + R2.Top;
end;


function RectMinMax(R: TRect; P: TPoint): TRect;
{Compare point to retangle and }
{return the largest combined Rectangle}
begin
Result.Left:=Min(P.X,R.Left);
Result.Top:=Min(P.Y,R.Top);
Result.Right:=Max(P.X,R.Right);
Result.Bottom:=Max(P.Y,R.Bottom);
end;


function RectMinMax(R1, R2: TRect): TRect;
{Compare Rectangle to retangle and }
{return the largest combined Rectangle}
begin
Result:=RectMinMax(R1,R2.TopLeft);
Result:=RectMinMax(Result,R2.BottomRight);
end;

function GetRectStr(R: TRect; Labeled: boolean): string;
{Get string of Rectangle coordinates}
begin
if Labeled then Result:=Format('Left: %d Top: %d Right: %d Bottom: %d',[R.Left,R.Top,R.Right,R.Bottom])
else Result:=Format('%d, %d, %d, %d',[R.Left,R.Top,R.Right,R.Bottom]);
end;



function DoRectsOverlap(R1,R2: TRect): boolean;
{Do rectangles overlap?}
begin
Result:=False;
if R1.BottomRight.x < R2.TopLeft.x then exit; { R1 is left of R2}
if R1.TopLeft.x > R2.BottomRight.x then exit; { R1 is right of R2}
if R1.BottomRight.y < R2.TopLeft.y then exit; { R1 is above R2}
if R1.TopLeft.y > R2.BottomRight.y then exit; { R1 is below R2}
Result:=True;
end;


{============ Polygon Routines ===============================================}




function CopyPolygon(Poly: TPointPolygon): TPointPolygon;
var I: integer;
begin
SetLength(Result, Length(Poly));
for I:=0 to High(Poly) do Result[I]:=Poly[I];
end;


procedure RotatePolygon(var PT: TPointPolygon; Center: TPoint; Angle: double);
var I: integer;
begin
for I:=0 to High(PT) do RotatePoint(PT[I],Point(0,0),Angle);
end;


procedure OffsetPolygon(var PT: TPointPolygon; Offset: TPoint);
var I: integer;
begin
for I:=0 to High(PT) do PT[I]:=IntVectorAdd(PT[I],Offset);
end;



function GetBoundingBox(A: TPointPolygon): TRect;
{Get box that encloses all points in specified array}
var I: integer;
begin
Result:=Rect(0,0,0,0);
if Length(A)<1 then exit;
Result:=Rect(High(integer),High(integer),Low(integer),Low(integer));
for I:=0 to High(A) do
	begin
	if A[I].X<Result.Left then Result.Left:=A[I].X;
	if A[I].X>Result.Right then Result.Right:=A[I].X;
	if A[I].Y<Result.Top then Result.Top:=A[I].Y;
	if A[I].Y>Result.Bottom then Result.Bottom:=A[I].Y;
	end;
end;


function PointInPolygon(Poly: TPointPolygon; P: TPoint): boolean;
{Integer version}
var I,J: integer;
var OddNodes: boolean;
begin
OddNodes:=False;
J:=High(Poly);
for I:=0 to High(Poly) do
	begin
	if ((Poly[i].Y<P.Y) and (Poly[j].Y>=P.Y))  or
	   ((Poly[j].Y<P.Y) and (Poly[i].Y>=P.Y)) then
		begin
		if (Poly[I].X+(P.Y-Poly[I].Y) / (Poly[j].Y-Poly[i].Y) * (Poly[j].X-Poly[i].X) < P.X) then
			begin
			OddNodes:=not oddNodes;
			end;
		end;
	J:=I;
	end;
Result:=OddNodes;
end;



function PolygonIntersect(P1,P2: TPointPolygon): boolean;
{Return true if the specified polygons intersect?}
var I: integer;
begin
Result:=True;
for I:=0 to High(P1) do
 if PointInPolygon(P2,P1[I]) then exit;
for I:=0 to High(P2) do
 if PointInPolygon(P1,P2[I]) then exit;
Result:=False;
end;


end.
