unit LowDiskIO;

interface

uses Windows,Classes,SysUtils, ComObj,Variants, ActiveX, OleCtnrs,AxCtrls,
     StdVcl, DSupport;

{Windows definitions for DeviceIoControl data}

type MEDIA_TYPE = (
    Unknown,                // Format is unknown
    F5_1Pt2_512,            // 5.25", 1.2MB,  512 bytes/sector
    F3_1Pt44_512,           // 3.5",  1.44MB, 512 bytes/sector
    F3_2Pt88_512,           // 3.5",  2.88MB, 512 bytes/sector
    F3_20Pt8_512,           // 3.5",  20.8MB, 512 bytes/sector
    F3_720_512,             // 3.5",  720KB,  512 bytes/sector
    F5_360_512,             // 5.25", 360KB,  512 bytes/sector
    F5_320_512,             // 5.25", 320KB,  512 bytes/sector
    F5_320_1024,            // 5.25", 320KB,  1024 bytes/sector
    F5_180_512,             // 5.25", 180KB,  512 bytes/sector
    F5_160_512,             // 5.25", 160KB,  512 bytes/sector
    RemovableMedia,         // Removable media other than floppy
    FixedMedia,             // Fixed hard disk media
    F3_120M_512,            // 3.5", 120M Floppy
    F3_640_512,             // 3.5" ,  640KB,  512 bytes/sector
    F5_640_512,             // 5.25",  640KB,  512 bytes/sector
    F5_720_512,             // 5.25",  720KB,  512 bytes/sector
    F3_1Pt2_512,            // 3.5" ,  1.2Mb,  512 bytes/sector
    F3_1Pt23_1024,          // 3.5" ,  1.23Mb, 1024 bytes/sector
    F5_1Pt23_1024,          // 5.25",  1.23MB, 1024 bytes/sector
    F3_128Mb_512,           // 3.5" MO 128Mb   512 bytes/sector
    F3_230Mb_512,           // 3.5" MO 230Mb   512 bytes/sector
    F8_256_128,             // 8",     256KB,  128 bytes/sector
    F3_200Mb_512,           // 3.5",   200M Floppy (HiFD)
    F3_240M_512,            // 3.5",   240Mb Floppy (HiFD)
    F3_32M_512);            // 3.5",   32Mb Floppy

type TDiskGeometry = record
 Cylinders: TLargeInteger;
 MediaType: MEDIA_TYPE;
 TracksPerCylinder: DWORD;
 SectorsPerTrack: DWORD;
 BytesPerSector: DWORD;
 end;

const METHOD_BUFFERED   = 0;
const FILE_DEVICE_DISK  = $00000007;
const FILE_ANY_ACCESS = 0;
const IOCTL_DISK_BASE = FILE_DEVICE_DISK;
const IOCTL_DISK_GET_DRIVE_GEOMETRY = ((IOCTL_DISK_BASE shl 16) or (FILE_ANY_ACCESS shl 14) or ($0000 shl 2) or METHOD_BUFFERED);



type TLogDriveInfo = record
 VolumeName: string;
 FileSystemName: string;
 FileSystemFlags: DWORD;
 SerialNumber: DWORD;
 end;

{Structure to hold disk drive information}

type TLogicalDisk = record
 Name: string;
 Size: int64;
 Sectors: int64;
 DriveInfo: TLogDriveInfo;
 end;

type TLogicalDisks = array of TLogicalDisk;

type TPartition = record
 PartitionID: string;
 PartNumber: integer;
 LogicalDisks: TLogicalDisks;
 end;

type TPartitions = array of TPartition;

type TPhysicalDrive = record
 DriveNumber: integer;
 DeviceID: string;
 Geometry: TDiskGeometry;
 Size: int64;
 Sectors: int64;
 Partitions: TPartitions;
 end;

type TDriveArray = array of TPhysicalDrive;




const BytesPerSector = 512;

type TSector = array[0..BytesPerSector-1] of Byte;

type TSectorBlock = array of TSector;


{------------------------------------------------------------------------------}


type TSectorStream = class(THandleStream)
 private
  FSize, FPosition: int64;
  FSectorCount: int64;
  SectorCache: TSectorBlock;
  CacheStart,CacheSize: int64;
  CacheCalls,CacheHits: int64;
  procedure AnalyzeDrive(Handle: THandle);
  procedure LoadCache(Start, Size: int64);
    function GetCacheHitPercent: double;
 protected
  function Seek(const Offset: Int64; Origin: TSeekOrigin): Int64; overload; override;
 public
  Geometry: TDiskGeometry;
  constructor Create(PhysicalDrive: integer); overload;
  constructor Create(LogicalDrive: char); overload;
  destructor Destroy; override;
  procedure SetSize(NewSize: Longint); override;
  procedure SetSize(const NewSize: Int64); override;
  function Read(var Buffer; Count: Integer): Longint; override;
  function ReadSector(Sector: int64; var SectorData: TSector): boolean;
  property DriveSize: int64 read FSize;
  property SectorCount: int64 read FSectorCount;
  procedure ResetStats;
  property CacheHitPercent: double read GetCacheHitPercent;
 end;


{------------------------------------------------------------------------------}

function DOS_IOErrToStr(ret:Integer):String;
function DOS_25h_DiskRead(Drive:Integer; Buffer:PChar; StartSector,Count: Integer; var ret: Integer): Boolean;
function DOS_26h_DiskWrite(Drive:Integer; Buffer:PChar; StartSector,Count: Integer; var ret: Integer): Boolean;
procedure LowReadSector(Drive: string; Sector: integer; var SectorData: TSector);


function GetLogicalHandle(DriveLetter: string): THandle;
function GetPhysicalHandle(DriveNumber: integer): THandle;


procedure ReadLogicalDriveSector(DriveLetter: string; Sector: int64; var SectorData: TSector);
procedure ReadPhysicalDriveSector(DriveNumber: integer; Sector: int64; var SectorData: TSector);


function GetDriveGeometry(PathName: string; var Geometry: TDiskGeometry): boolean; overload;
function GetDriveGeometry(Drive: integer; var Geometry: TDiskGeometry): boolean; overload;

procedure GetDriveData(var List: TDriveArray);
function GetDriveDataStr(Data: TDriveArray): string;

procedure ReadLogicalSectorBlock(DriveLetter: string; Sector: int64; var SectorBlock: TSectorBlock);
procedure ReadPhysicalSectorBlock(DriveNumber: integer; Sector: int64; var SectorBlock: TSectorBlock);

implementation







function GetMediaTypeStr(MT: MEDIA_TYPE): string;
begin
case MT of
 Unknown: Result:='Format is unknown';
 F5_1Pt2_512: Result:='5.25", 1.2MB, 512 bytes/sector';
 F3_1Pt44_512: Result:='3.5", 1.44MB, 512 bytes/sector';
 F3_2Pt88_512: Result:='3.5", 2.88MB, 512 bytes/sector';
 F3_20Pt8_512: Result:='3.5", 20.8MB, 512 bytes/sector';
 F3_720_512: Result:='3.5", 720KB, 512 bytes/sector';
 F5_360_512: Result:='5.25", 360KB, 512 bytes/sector';
 F5_320_512: Result:='5.25", 320KB, 512 bytes/sector';
 F5_320_1024: Result:='5.25", 320KB, 1024 bytes/sector';
 F5_180_512: Result:='5.25", 180KB, 512 bytes/sector';
 F5_160_512: Result:='5.25", 160KB, 512 bytes/sector';
 RemovableMedia: Result:='Removable media other than floppy';
 FixedMedia: Result:='Fixed hard disk media';
 F3_120M_512: Result:='3.5", 120M Floppy';
 F3_640_512: Result:='3.5" , 640KB, 512 bytes/sector';
 F5_640_512: Result:='5.25", 640KB, 512 bytes/sector';
 F5_720_512: Result:='5.25", 720KB, 512 bytes/sector';
 F3_1Pt2_512: Result:='3.5" , 1.2Mb, 512 bytes/sector';
 F3_1Pt23_1024: Result:='3.5" , 1.23Mb, 1024 bytes/sector';
 F5_1Pt23_1024: Result:='5.25", 1.23MB, 1024 bytes/sector';
 F3_128Mb_512: Result:='3.5" MO 128Mb  512 bytes/sector';
 F3_230Mb_512: Result:='3.5" MO 230Mb  512 bytes/sector';
 F8_256_128: Result:='8",   256KB, 128 bytes/sector';
 F3_200Mb_512: Result:='3.5",  200M Floppy (HiFD)';
 F3_240M_512: Result:='3.5",  240Mb Floppy (HiFD)';
 F3_32M_512: Result:='3.5",  32Mb Floppy';
 else Result:='Unknown';
 end;
end;



function GetDriveGeometry(PathName: string; var Geometry: TDiskGeometry): boolean;
{Get drive geomtry for specified drive - PathName should be in the form:}
{'\\.\PHYSICALDRIVE0' where the number at the end is the physical drive #}
var hDevice: THandle;
var bResult: boolean;
var junk: cardinal;
begin
Result:=False;
hDevice:=INVALID_HANDLE_VALUE;
Junk:= 0;
hDevice := CreateFile(PChar(PathName), 0, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, 0, 0);
if hDevice = INVALID_HANDLE_VALUE then exit;

Result := DeviceIoControl(hDevice, IOCTL_DISK_GET_DRIVE_GEOMETRY, nil, 0, @Geometry, sizeof(TDiskGeometry), junk, nil);
CloseHandle(hDevice);
end;



function GetDriveGeometry(Drive: integer; var Geometry: TDiskGeometry): boolean;
{Get drive geomtry for specified drive number}
begin
Result:=GetDriveGeometry('\\.\PHYSICALDRIVE'+IntToStr(Drive), Geometry);
end;




function GetLogDriveInfo(Drive: char): TLogDriveInfo;
var TempBool: Boolean;
var VolumeName: array [0..255] of Char;
var VolumeNameLength: Integer;
var SerialNumber: Cardinal;
var MaxComponentLength: Cardinal;
var FileSystemFlags: Cardinal;
var FileSystemName: array [0..255] of Char;
var FileSystemNameLength: Integer;
var Success: boolean;
begin
{ Prep the variables that need prepping}
Result.SerialNumber := 0;
Result.VolumeName := '';
Result.FileSystemName := '';
VolumeNameLength := 255;
FileSystemNameLength := 255;
MaxComponentLength := 0;

{ Yank the info from the drive via API call}
Success:=GetVolumeInformation(PAnsiChar(Drive + ':\'),
				      @VolumeName, VolumeNameLength,
				      @SerialNumber,
				      MaxComponentLength,
				      FileSystemFlags,
				      @FileSystemName, FileSystemNameLength);
if Success then
	begin
	{ Populate the result type}
	Result.VolumeName := VolumeName;
	Result.SerialNumber := SerialNumber;
	Result.FileSystemName := FileSystemName;
	Result.FileSystemFlags:=FileSystemFlags;

	if Result.VolumeName='' then Result.VolumeName:='Local Disk ('+Drive+')';
	end
else raise Exception.Create('Error ' + SysErrorMessage(GetLastError));
end;

{-------------------------------------------------------------------------------}



procedure SortDriveData(var List: TDriveArray);
{Bubble sort drive data}
var I,Swaps: integer;
var T: TPhysicalDrive;
begin
repeat
	begin
	Swaps:=0;
	for I:=0 to High(List)-1 do
		begin
		if List[I].DriveNumber>List[I+1].DriveNumber then
			begin
			T:=List[I];
			List[I]:=List[I+1];
			List[I+1]:=T;
			Inc(Swaps);
			end;
		end;
	end
until Swaps=0;
end;





procedure GetDriveData(var List: TDriveArray);
{Get information about installed drives}
var Locator, WMIService, DiskDrives : Variant;
var colLogicalDisks, colPartitions: Variant;
var objdiskDrive, objPartition, objLogicalDisk: OLEVariant;
var oEnumDiskDrive, oEnumPartition, oEnumLogical: IEnumvariant;
var DeviceID: widestring;
var S: widestring;
var T: string;
var Size,Sectors: int64;
var N: LongWord;
var Drive: ^TPhysicalDrive;
var Part: ^TPartition;

	function ExtractDriveNumber(ID: string): integer;
	{Extract drive number from device ID}
	begin
	if pos('\\.\PHYSICALDRIVE',ID)>0 then
		Result:=StrToIntDef(ID[Length(ID)],-1);
	end;


	function ExtractPartNumber(ID: string): integer;
	{Extract partition number from device ID}
	begin
	if pos('Partition #',ID)>0 then
		Result:=StrToIntDef(ID[Length(ID)],-1);
	end;


	function GetDiskSize(ID: string): int64;
	begin
	if Length(ID)>0 then Result:=DiskSize(byte(ID[1])-$40)
	else Result:=0;
	end;





begin
SetLength(List,0);
Locator  := CreateOleObject('WbemScripting.SWbemLocator');
WMIService  := Locator.ConnectServer('localhost', 'root\CIMV2', '', '');
DiskDrives  := WMIService.ExecQuery('SELECT DeviceID FROM Win32_DiskDrive', 'WQL');
oEnumDiskDrive := IUnknown(DiskDrives._NewEnum) as IEnumVariant;
while oEnumDiskDrive.Next(1, objdiskDrive, n) = 0 do
	begin
	SetLength(List,Length(List)+1);
	Drive:=@List[High(List)];
	Drive.DeviceID:=objdiskDrive.DeviceID;
	Drive.DriveNumber:=ExtractDriveNumber(Drive.DeviceID);
	GetDriveGeometry(Drive.DeviceID,Drive.Geometry);
	Drive.Sectors:=Drive.Geometry.Cylinders * Drive.Geometry.TracksPerCylinder * Drive.Geometry.SectorsPerTrack;
	Drive.Size:=Drive.Sectors * Drive.Geometry.BytesPerSector;

	{Escape the `\` chars in the DeviceID value because the '\' is a reserved character in WMI.}
	DeviceID := StringReplace(objdiskDrive.DeviceID,'\','\\',[rfReplaceAll]);

	{link the Win32_DiskDrive class with the Win32_DiskDriveToDiskPartition class}
	s:=Format('ASSOCIATORS OF {Win32_DiskDrive.DeviceID="%s"} WHERE AssocClass = Win32_DiskDriveToDiskPartition',[DeviceID]);
	colPartitions  := WMIService.ExecQuery(s, 'WQL');
	oEnumPartition := IUnknown(colPartitions._NewEnum) as IEnumVariant;
	SetLength(Drive.Partitions,0);
	while oEnumPartition.Next(1, objPartition, n) = 0 do
		begin
		if not VarIsNull(objPartition.DeviceID) then
			begin
			SetLength(Drive.Partitions,Length(Drive.Partitions)+1);
			Part:=@Drive.Partitions[High(Drive.Partitions)];
			SetLength(Part.LogicalDisks,0);
			Part.PartitionID:=objPartition.DeviceID;
			Part.PartNumber:=ExtractPartNumber(Part.PartitionID);

			{link the Win32_DiskPartition class with theWin32_LogicalDiskToPartition class.}
			s:='ASSOCIATORS OF {Win32_DiskPartition.DeviceID="'+VarToStr(objPartition.DeviceID)+'"} WHERE AssocClass = Win32_LogicalDiskToPartition';
			colLogicalDisks := WMIService.ExecQuery(s);
			oEnumLogical := IUnknown(colLogicalDisks._NewEnum) as IEnumVariant;
			while oEnumLogical.Next(1, objLogicalDisk, n) = 0 do
				begin
				SetLength(Part.LogicalDisks,Length(Part.LogicalDisks)+1);
				T:=objLogicalDisk.DeviceID;
				Part.LogicalDisks[High(Part.LogicalDisks)].Name:=T;
				Part.LogicalDisks[High(Part.LogicalDisks)].DriveInfo:=GetLogDriveInfo(T[1]);
				Sectors:=0;
				Size:=GetDiskSize(T);
				Part.LogicalDisks[High(Part.LogicalDisks)].Size:=Size;
                                if Drive.Geometry.BytesPerSector<>0 then Sectors:=Size div Drive.Geometry.BytesPerSector;

				Part.LogicalDisks[High(Part.LogicalDisks)].Sectors:=Sectors;
				objLogicalDisk:=Unassigned;
				end;
			end;
		objPartition:=Unassigned;
		end;
	objdiskDrive:=Unassigned;
	end;
SortDriveData(List);
end;


function GetDriveDataStr(Data: TDriveArray): string;
{Extract drive data into a string for display}
var I,J,K: integer;
var Drive: TPhysicalDrive;
var Part: TPartition;
var Disk: TLogicalDisk;
const Gigabyte = 1024 * 1024 * 1024;


	function GetGigaBytes(V: int64): extended;
	begin
	Result:=V / GigaByte;
	end;


begin
Result:='';
for I:=0 to High(Data) do
	begin
	Drive:=Data[I];
	Result:=Result+'Drive #: '+IntToStr(Drive.DriveNumber)+CRLF_Char;
	Result:=Result+'Drive ID#: '+Drive.DeviceID+CRLF_Char;
	Result:=Result+'       Media Type: '+GetMediaTypeStr(Drive.Geometry.MediaType)+CRLF_Char;
	Result:=Result+'       Cylinders: '+IntToStr(Drive.Geometry.Cylinders)+CRLF_Char;
	Result:=Result+'       Track/Cylinder: '+IntToStr(Drive.Geometry.TracksPerCylinder)+CRLF_Char;
	Result:=Result+'       Sectors/Track: '+IntToStr(Drive.Geometry.SectorsPerTrack)+CRLF_Char;
	Result:=Result+'       Bytes/Sector: '+IntToStr(Drive.Geometry.BytesPerSector)+CRLF_Char;
	Result:=Result+'       Size: '+IntToStr(Drive.Size)+CRLF_Char;
	Result:=Result+'       Disk size: '+FloatToStrF(GetGigaBytes(Drive.Size),ffNumber,18,3)+CRLF_Char;
	Result:=Result+'       Sectors: '+IntToStr(Drive.Sectors)+CRLF_Char;

	for J:=0 to High(Drive.Partitions) do
		begin
		Part:=Drive.Partitions[J];
		Result:=Result+'   Part ID#: '+Part.PartitionID+CRLF_Char;
		Result:=Result+'   Part #: '+IntToStr(Part.PartNumber)+CRLF_Char;
		for K:=0 to High(Part.LogicalDisks) do
			begin
			Disk:=Part.LogicalDisks[K];
			Result:=Result+'           Logical Disk ID#: '+Disk.Name+CRLF_Char;
			Result:=Result+'           Size: '+FloatToStrF(GetGigaBytes(Disk.Size),ffNumber,18,3)+CRLF_Char;
			Result:=Result+'           Sectors: '+IntToStr(Disk.Sectors)+CRLF_Char;
			Result:=Result+'           Name: '+Disk.DriveInfo.VolumeName+CRLF_Char;
			Result:=Result+'           Serial #: '+IntToStr(Disk.DriveInfo.SerialNumber)+CRLF_Char;
			Result:=Result+'           Flags: '+IntToHex(Disk.DriveInfo.FileSystemFlags,8)+CRLF_Char;
			Result:=Result+'           File Sys: '+Disk.DriveInfo.FileSystemName+CRLF_Char;
			end;
		end;
	end;
end;
 

{------------------------------------------------------------------------------}



type TCPURegsters = packed record
 reg_ebx: DWORD;
 reg_edx: DWORD;
 reg_ecx: DWORD;
 reg_eax: DWORD;
 reg_edi: DWORD;
 reg_esi: DWORD;
 reg_cflag: DWORD;
 end;

const WIN95_IOCTL_DEV = '\\.\vwin32';

const WIN32_DOS_IOCTL = 1;
const WIN32_DOS_INT25 = 2;
const WIN32_DOS_INT26 = 3;


{----------------- Old Win 95 level code, probably obsolete ------------------}


function WIN95_IOCTL(Cmd: Integer; var regs: TCPURegsters): boolean;
var hd: THandle; ret: DWORD; ans: Boolean; 
begin 
Result:= False; 
try 
{Open WIN95_IOCTL_DEV} 
hd:= 0; 
hd:= CreateFile(WIN95_IOCTL_DEV,0,0,NIL,0,FILE_FLAG_DELETE_ON_CLOSE,0); 
if hd = 0 then exit; 
{Device I/O Control} 
ans:= DeviceIoControl(hd,cmd,@regs,sizeof(regs),@regs,sizeof(regs),ret,NIL); 
if not ans then  exit; 
Result:= True; 
 finally 
 {Close WIN95_IOCTL_DEV} 
 CloseHandle(hd); 
 end; 
end; 

 
 
function DOS_IOErrToStr(ret: Integer): String; 
begin 
case ret of 
 $00: Result:= 'disk is write-protected'; 
 $01: Result:= 'bad disk unit'; 
 $02: Result:= 'drive is not ready'; 
 $03: Result:= 'invalid disk command'; 
 $04: Result:= 'CRC error'; 
 $05: Result:= 'invalid length (disk operation)'; 
 $06: Result:= 'seek error'; 
 $07: Result:= 'wrong disk'; 
 $08: Result:= 'sector not found'; 
 $09: Result:= 'out of paper'; 
 $0a: Result:= 'write fault'; 
 $0b: Result:= 'read fault'; 
 $0c: Result:= 'general fault'; 
 $0f: Result:= 'invalid disk change'; 
 else  Result:= 'unknown'; 
 end; 
end; 
 
 
function DOS_25h_DiskRead(Drive: Integer; Buffer: PChar;  StartSector,Count: Integer; var ret: Integer): Boolean; 
var regs: TCPURegsters;
begin 
regs.reg_EAX:= DWORD(Drive);		// AL: 0:A, 1:B..
regs.reg_EBX:= DWORD(Buffer);		// buffer
regs.reg_ECX:= DWORD(Count);		// read sector count.
regs.reg_EDX:= DWORD(StartSector);	// start sector
regs.reg_cflag:= DWORD($00000001);	// clear cflg.
 
if (not win95_ioctl(WIN32_DOS_INT25,regs)) then
	begin 
	Result:= False; 
	ret:= -1; 
	exit; 
	end; 
if (regs.reg_cflag and $00000001) <> 0 then
	begin 
	Result:= False; 
	ret:= Integer(regs.reg_EAX and $0000000f); 
	exit; 
	end; 
Result:= True; 
end; 
 
 

function DOS_26h_DiskWrite(Drive: Integer; Buffer: PChar; StartSector,Count: Integer; var ret: Integer): Boolean; 
var regs: TCPURegsters;
begin
regs.reg_EAX:= DWORD(Drive);		// AL: 0:A, 1:B..
regs.reg_EBX:= DWORD(Buffer);		// buffer
regs.reg_ECX:= DWORD(Count);		// write sector count.
regs.reg_EDX:= DWORD(StartSector);	// start sector 
regs.reg_cflag:= DWORD($00000001);	// clear cflg. 
 
if (not win95_ioctl(WIN32_DOS_INT26,regs)) then
	begin 
	Result:= False; 
	ret:= -1; 
	exit; 
	end; 
if (regs.reg_cflag and $00000001) <> 0 then
	begin 
	Result:= False; 
	ret:= Integer(regs.reg_EAX and $0000000f); 
	exit; 
	end; 
Result:= True; 
end;




function ReadSectorWin95(Drive: string; Sector: integer; var SectorData: TSector): boolean;
var Ret: integer;
var D: integer;
begin
Drive:=UpperCase(Drive);
D:=byte(Drive[1])-$41;
if not DOS_25h_DiskRead(D,@SectorData,Sector,1,Ret) then raise Exception.Create('Unable to access Drive: '+Drive);
end;



function IsWindowsNT: boolean;
var Version: TOSVersionInfo;
begin
Version.dwOSVersionInfoSize:=SizeOf(Version);
GetVersionEx(Version);
Result:=Version.dwPlatformId=VER_PLATFORM_WIN32_NT;
end;



{----------------- Modern, Windows NT-Level code ------------------------------}



function GetLogicalHandle(DriveLetter: string): THandle;
begin
{Note: in C you'll see \\\\.\\. In C, backslash is a meta char and has to be escaped}
Result:= CreateFile(PChar(Format('\\.\%s:', [UpperCase(DriveLetter)])), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
if Result = INVALID_HANDLE_VALUE then raise Exception.Create(SysErrorMessage(GetLastError));
end;



function GetPhysicalHandle(DriveNumber: integer): THandle;
var S: string;
begin
{Note: in C you'll see \\\\.\\. In C, backslash is a meta char and has to be escaped}
S:='\\.\PhysicalDrive'+IntToStr(DriveNumber);
Result:= CreateFile(PChar(S), GENERIC_READ, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
if Result = INVALID_HANDLE_VALUE then raise Exception.Create(SysErrorMessage(GetLastError));
end;


procedure ReadLogicalDriveSector(DriveLetter: string; Sector: int64; var SectorData: TSector);
{Read sector by drive letter}
var BytesRead: Cardinal;
var Offset: Int64;
var H: THandle;
begin
H:=GetLogicalHandle(DriveLetter);
try
{Sector must be a cardinal or high part will be negative}
Offset:=Sector * BytesPerSector;
SetFilePointer(H,ULARGE_INTEGER(Offset).LowPart, @ULARGE_INTEGER(Offset).HighPart, FILE_BEGIN);
if not ReadFile(H, SectorData, SizeOf(SectorData), BytesRead, nil) then raise Exception.Create(SysErrorMessage(GetLastError));
finally  CloseHandle(H) end;
end;



procedure ReadPhysicalDriveSector(DriveNumber: integer; Sector: int64; var SectorData: TSector);
{Read sector by hardware drive number}
var S: string;
var BytesRead: Cardinal;
var Offset: Int64;
var H: THandle;
begin
H:=GetPhysicalHandle(DriveNumber);
try
{Sector must be a cardinal or high part will be negative}
Offset:=Sector * BytesPerSector;
SetFilePointer(H,ULARGE_INTEGER(Offset).LowPart, @ULARGE_INTEGER(Offset).HighPart, FILE_BEGIN);
if not ReadFile(H, SectorData, SizeOf(SectorData), BytesRead, nil) then raise Exception.Create(SysErrorMessage(GetLastError));
finally  CloseHandle(H) end;
end;

{------------------------------------------------------------------------------}



procedure ReadLogicalSectorBlock(DriveLetter: string; Sector: int64; var SectorBlock: TSectorBlock);
{Read block of ssector by drive letter}
var BytesRead: Cardinal;
var Offset: Int64;
var H: THandle;
begin
H:=GetLogicalHandle(DriveLetter);
try
{Sector must be a cardinal or high part will be negative}
Offset:=Sector * BytesPerSector;
SetFilePointer(H,ULARGE_INTEGER(Offset).LowPart, @ULARGE_INTEGER(Offset).HighPart, FILE_BEGIN);
if not ReadFile(H, SectorBlock, SizeOf(TSector)*Length(SectorBlock), BytesRead, nil) then raise Exception.Create(SysErrorMessage(GetLastError));
finally  CloseHandle(H) end;
end;



procedure ReadPhysicalSectorBlock(DriveNumber: integer; Sector: int64; var SectorBlock: TSectorBlock);
{Read sector by hardware drive number}
var BytesRead: Cardinal;
var Offset: Int64;
var H: THandle;
begin
H:=GetPhysicalHandle(DriveNumber);
try
{Sector must be a cardinal or high part will be negative}
Offset:=Sector * BytesPerSector;
SetFilePointer(H,ULARGE_INTEGER(Offset).LowPart, @ULARGE_INTEGER(Offset).HighPart, FILE_BEGIN);
if not ReadFile(H, SectorBlock[0], SizeOf(TSector)*Length(SectorBlock), BytesRead, nil) then raise Exception.Create(SysErrorMessage(GetLastError));
finally  CloseHandle(H) end;
end;


{------------------------------------------------------------------------------}



procedure LowReadSector(Drive: string; Sector: integer; var SectorData: TSector);
begin
if IsWindowsNT then ReadLogicalDriveSector(Drive,Sector,SectorData)
else ReadSectorWin95(Drive,Sector,SectorData);
end;

{------------------------------------------------------------------------------}

{ TSectorStream }


procedure TSectorStream.AnalyzeDrive(Handle: THandle);
var junk: cardinal;
begin
DeviceIoControl(Handle, IOCTL_DISK_GET_DRIVE_GEOMETRY, nil, 0, @Geometry, sizeof(TDiskGeometry), junk, nil);
FSectorCount:=Geometry.Cylinders * Geometry.TracksPerCylinder * Geometry.SectorsPerTrack;
FSize:=FSectorCount * Geometry.BytesPerSector;
CacheStart:=-1;
CacheSize:=-1;
end;




constructor TSectorStream.Create(PhysicalDrive: integer);
var H: THandle;
begin
H:=GetPhysicalHandle(PhysicalDrive);
AnalyzeDrive(H);
inherited Create(H);
end;



constructor TSectorStream.Create(LogicalDrive: char);
var H: THandle;
begin
H:=GetLogicalHandle(LogicalDrive);
AnalyzeDrive(H);
inherited Create(H);
end;



destructor TSectorStream.Destroy;
begin
if FHandle >= 0 then FileClose(FHandle);
inherited Destroy;
end;



procedure TSectorStream.SetSize(NewSize: Integer);
begin
{Hardrive cannot be resized}
end;



procedure TSectorStream.SetSize(const NewSize: Int64);
begin
{Hardrive cannot be resized}
end;



function TSectorStream.Seek(const Offset: Int64; Origin: TSeekOrigin): Int64;
{Warning you must use TSeekOrigin style constants like soBeginnig}
{And not soFromBeginnig or the 32-bit version will be called}
begin
case Ord(Origin) of
 soFromBeginning: FPosition := Offset;
 soFromCurrent: Inc(FPosition, Offset);
 soFromEnd: FPosition := FSize + Offset;
 end;
Result := FPosition;
end;



function TSectorStream.ReadSector(Sector: int64; var SectorData: TSector): boolean;
var Offset: Int64;
var BytesRead: Cardinal;
begin
Result:=False;
Offset:=Sector * BytesPerSector;
SetFilePointer(Handle,ULARGE_INTEGER(Offset).LowPart, @ULARGE_INTEGER(Offset).HighPart, FILE_BEGIN);
if not ReadFile(Handle, SectorData, SizeOf(SectorData), BytesRead, nil) then raise Exception.Create(SysErrorMessage(GetLastError));
Result:=True;
end;




procedure TSectorStream.LoadCache(Start, Size: int64);
var Inx: int64;
var I: integer;
begin
Inc(CacheCalls);
if (Start=CacheStart) and (Size=CacheSize) then
	begin
	Inc(CacheHits);
	exit;
	end;
SetLength(SectorCache,Size);
Inx:=Start;
for I:=0 to High(SectorCache) do
	begin
	ReadSector(Inx,SectorCache[I]);
	Inc(Inx);
	end;
CacheStart:=Start;
CacheSize:=Size;
end;




function TSectorStream.Read(var Buffer; Count: Longint): Longint;
var EndSize: int64;

	procedure GetSectorOffset(Pos: int64; var Sector,Offset: int64);
	begin
	Sector:=Pos div BytesPerSector;
	OffSet:=Pos mod BytesPerSector;
	end;


	function CopyData: int64;
	var Sect1,Off1,Sect2,Off2: int64;
	var Inx: int64;
	var I: integer;
	var Size,SectorCnt: cardinal;
	begin
	GetSectorOffset(FPosition,Sect1,Off1);
	GetSectorOffset(FPosition+Count-1,Sect2,Off2);
	SectorCnt:=(Sect2-Sect1)+1;
	LoadCache(Sect1, SectorCnt);
	Size:=(SectorCnt*BytesPerSector)-(Off1 + (BytesPerSector-Off2))+1;
	Move(Pointer(integer(@SectorCache[0])+Off1)^,Buffer,Size);
	Result:=Size;
	end;


begin
if (FPosition >= 0) and (Count >= 0) then
	begin
	EndSize := FSize - FPosition;
	if EndSize > 0 then
		begin
		if Count>EndSize then Result:=EndSize
		else Result := Count;
		CopyData;
		Inc(FPosition, Result);
		exit;
		end
	else Result:=0;
	end;
Result := 0;
end;



procedure TSectorStream.ResetStats;
begin
CacheCalls:=0;
CacheHits:=0;
end;



function TSectorStream.GetCacheHitPercent: double;
begin
Result:=CacheHits/CacheCalls;
end;



initialization
CoInitialize(nil);
finalization
CoUninitialize;
end.
