unit FFBookMarks;

interface

uses SysUtils, IniFiles;

type TBookMark = record
 Name: string;
 MarkTime: int64;
 end;

type TBookMarkArray = array of TBookMark;

var BookMarks: TBookMarkArray;


procedure AddBookMark(Name: string; MarkTime: Int64);
procedure ClearBookMarks;
function DeleteBookMark(Inx: integer): boolean;

function MoveBookMarkUp(Inx: integer): boolean;
function MoveBookMarkDown(Inx: integer): boolean;

procedure LoadBookMarkData(Filename: string);
procedure SaveBookMarkData(Filename: string);



implementation


procedure ClearBookMarks;
begin
SetLength(BookMarks,0);
end;

function DeleteBookMark(Inx: integer): boolean;
var I: integer;
begin
Result:=False;
if Length(BookMarks)<1 then exit;
if (Inx<0) and (Inx>High(BookMarks)) then exit;
for I:=Inx to High(BookMarks)-1 do
	begin
	BookMarks[I]:=BookMarks[I+1];
	end;
SetLength(BookMarks, Length(BookMarks)-1);
Result:=True;
end;



procedure AddBookMark(Name: string; MarkTime: Int64);
begin
SetLength(BookMarks,Length(BookMarks)+1);
BookMarks[High(BookMarks)].Name:=Name;
BookMarks[High(BookMarks)].MarkTime:=MarkTime;
end;



function MoveBookMarkUp(Inx: integer): boolean;
var Tmp: TBookMark;
begin
Result:=False;
if Length(BookMarks)<1 then exit;
if (Inx<=0) or (Inx>High(BookMarks)) then exit;
Tmp:=BookMarks[Inx-1];
BookMarks[Inx-1]:=BookMarks[Inx];
BookMarks[Inx]:=Tmp;
Result:=True;
end;



function MoveBookMarkDown(Inx: integer): boolean;
var Tmp: TBookMark;
begin
Result:=False;
if Length(BookMarks)<1 then exit;
if (Inx<0) or (Inx>=High(BookMarks)) then exit;
Tmp:=BookMarks[Inx+1];
BookMarks[Inx+1]:=BookMarks[Inx];
BookMarks[Inx]:=Tmp;
Result:=True;
end;

{======================================================================}


procedure SaveBookMarkData(Filename: string);
{Save BookMark data }
var IniFile: TIniFile;
var I: integer;
var T: TBookMark;
begin
IniFile:=TIniFile.Create(Filename);
IniFile.EraseSection('BookMarks');
IniFile.WriteInteger('BookMarks','Count',Length(BookMarks));
for I:=0 to High(BookMarks) do
	begin
	T:=BookMarks[I];
	IniFile.WriteString('BookMarks','Name'+IntToStr(I),T.Name);
	IniFile.WriteInteger('BookMarks','MarkTime'+IntToStr(I),T.MarkTime);
	end;
IniFile.Free;
end;


procedure LoadBookMarkData(Filename: string);
{Load BookMark data }
var IniFile: TIniFile;
var I,Cnt: integer;
var T: TBookMark;
begin
ClearBookMarks;
IniFile:=TIniFile.Create(Filename);
Cnt:=IniFile.ReadInteger('BookMarks','Count',0);
for I:=0 to Cnt-1 do
	begin
	T.Name:=IniFile.ReadString('BookMarks','Name'+IntToStr(I),'');
	T.MarkTime:=IniFile.ReadInteger('BookMarks','MarkTime'+IntToStr(I),0);
	AddBookMark(T.Name,T.MarkTime);
	end;
IniFile.Free;
end;

end.
