unit GLAnimator;

interface

uses Vector,HugeArry,GLQuatLib,DSupport;


type TGLPosition = record
 Rot: TEulerAngles;
 Pos: T3DVector;
 end;


{------------------------------------------------------------}
{-------- Declaration for Animation Array object ------------}
{------------------------------------------------------------}

type TAnimationArray = class(THugeArray)
 private
  procedure Put(Index: integer; Item: TGLPosition);
  function Get(Index: integer): TGLPosition;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: TGLPosition): integer; reintroduce; virtual;
  function AddPointer(Item: TGLPosition; Ptr: integer): Integer;
  procedure Insert(Value: TGLPosition; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: TGLPosition read Get write Put; default;
 end;


type TGLAnimator = class(TObject)
  private
 public
  AnimationArray: TAnimationArray;
  constructor Create;
  destructor Destroy; override;
  procedure Assign(Animator: TGLAnimator); virtual;
  function GetStepPosition(Step, TotalSteps: integer; Circular: boolean; var NewPos: TGLPosition): boolean;
 end;



implementation



{------------------------------------------------------------------------}
{  Implementation for huge animation array object based on huge array object}
{------------------------------------------------------------------------}

constructor TAnimationArray.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(TGLPosition));
end;


function TAnimationArray.Add(Item: TGLPosition): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function TAnimationArray.AddPointer(Item: TGLPosition; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure TAnimationArray.Put(Index: integer; Item: TGLPosition);
{Insert Double at index}
var PTR: ^TGLPosition;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function TAnimationArray.Get(Index: integer): TGLPosition;
{Get Double at the index}
var PTR: ^TGLPosition;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure TAnimationArray.Insert(Value: TGLPosition; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure TAnimationArray.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure TAnimationArray.Move(CurIndex, NewIndex: Integer);
var T: TGLPosition;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


{ TGLAnimator }

procedure TGLAnimator.Assign(Animator: TGLAnimator);
begin
Self.AnimationArray.Assign(Animator.AnimationArray);
end;

constructor TGLAnimator.Create;
begin
inherited;
AnimationArray:=TAnimationArray.Create(5);
end;

destructor TGLAnimator.Destroy;
begin
AnimationArray.Free;
inherited;
end;



function NormalizeAngle(A: double): double;
begin
Result:=A;
while Result<0 do Result:=Result + 360;
while Result>360 do Result:=Result - 360;
end;




procedure ShortestDistance(var A2,A1: double);
{Convert angles to shortest distance subrtracting A2 - A1}
var D: double;
begin
A1:=NormalizeAngle(A1);
A2:=NormalizeAngle(A2);
D:=A2-A1;
if abs(D)<=180 then exit;
if A1<180 then A1:=A1 + 360 else A2:=A2 + 360;
end;




procedure NormalizePair(var P1,P2: TGLPosition);
begin
ShortestDistance(P2.Rot.Pitch,P1.Rot.Pitch);
ShortestDistance(P2.Rot.Yaw,P1.Rot.Yaw);
ShortestDistance(P2.Rot.Roll,P1.Rot.Roll);
end;



function InterpolatePositions(P1,P2: TGLPosition; Percent: double): TGLPosition;
begin
Result.Pos.X:=LinInterpolate(P1.Pos.X,P2.Pos.X,Percent);
Result.Pos.Y:=LinInterpolate(P1.Pos.Y,P2.Pos.Y,Percent);
Result.Pos.Z:=LinInterpolate(P1.Pos.Z,P2.Pos.Z,Percent);

NormalizePair(P1,P2);
Result.Rot:= InterpolateAngles(P1.Rot, P2.Rot, Percent);
end;




function TGLAnimator.GetStepPosition(Step,TotalSteps: integer; Circular: boolean; var NewPos: TGLPosition): boolean;
{Move object to position specified by step}
var StepsPerEntry,Percent: double;
var Start,Stop: integer;
var Pos1,Pos2: TGLPosition;
begin
Result:=False;
if (AnimationArray.Count<1) or (Step>=TotalSteps) then exit;
if Circular then StepsPerEntry:=(AnimationArray.Count)/(TotalSteps)
else StepsPerEntry:=(AnimationArray.Count-1)/(TotalSteps-1);
Percent:=Frac(StepsPerEntry * Step);
Start:=trunc(StepsPerEntry * Step);
Stop:=Start +1;
if Stop>=AnimationArray.Count then
	begin
	if Circular then Stop:=0
	else Stop:=AnimationArray.Count-1;
	end;
Pos1:=AnimationArray[Start];
Pos2:=AnimationArray[Stop];
NewPos:=InterpolatePositions(Pos1,Pos2,Percent);
Result:=True;
end;




end.
