unit GLFastMesh;

{Fast mesh loads and displays about seven times faster than the regular mesh}
{But doesn't support crease angles, sepearate normals/colors for shared vertices}
{The files is OpenGL-SuperArrays folder is faster and supports all those things}

interface

uses Windows,Classes,Graphics,SysUtils,Contnrs,Math,Hash,Vector,HugeArry,VectorHash,
     OpenGL15,GLRenderer,GLOctTree,Huge2D,GLLibrary,GLTexture,GLMatrix,GLArrays,GLCustomMesh;



type TLine = record
 P1,P2: integer;
 end;


{
Organizational Structure

Mesh
 Vertices
  AdjacentFaces
  Normals
  Colors
 Faces
  Normals
  Colors
}


type TGLFastMesh = class(TGLCustomMesh)
 private
  FNormalLength: double;
  procedure SetFaceNormals;
  function GetWeightedNormal(Inx: integer; CreaseAngle: double): T3DVector;
  function CalculateVertexNormal(Inx: integer; CreaseAngle: double): T3DVector;
  procedure SetVertexNormals(CreaseAngle: double);
  procedure ConvertAndAppend(Destination, Source: THuge3DVector; Matrix: TGLMatrix);
  procedure MatrixMultiplyVertices(Vertices: THuge3DVector; Matrix: TGLMatrix);
  procedure RenderNormal;
  procedure PrepareMesh;
  procedure DoWireTriangles;
  procedure DrawWireTriangle(Inx, Cell: integer);
  procedure SetWireTriangle(I: integer);
  procedure ShowFaceColor(I: integer);
  procedure DoFlatTriangles;
  procedure DrawFlatTriangle(Inx, Cell: integer);
  procedure SetFlatTriangle(I: integer);
  procedure DisplayNormals;
  procedure DrawNormalLine(Inx, Cell: integer);
  procedure FastSmoothTriangles;
  procedure DoSmoothTriangles;
  procedure DisplayFaceNormals;
  procedure DrawFaceNormalLine(Inx,Cell: integer);
  procedure HandleTextureChange(Sender: TObject);
  procedure SetupOctalTree;
  procedure HandleArrayChange(Sender: TObject);
  function GetAdjacent(X, Y: integer): integer;
 protected
  VertexArray: TVertexArray;
  FaceArray: TFaceList;

  procedure FindAdjacentTriangles(Target: T3DVector; var TA: THugeInteger);
  function GetExtentMax: T3DVector; override;
  procedure SetExtentMax(const Value: T3DVector); override;
  function GetExtentMin: T3DVector; override;
  procedure SetExtentMin(const Value: T3DVector); override;

  function GetFaceIndices(Index: integer): TFace; override;
  procedure SetFaceIndices(Index: integer; const Value: TFace); override;

  procedure Change; override;
  function GetFaceCount: integer; override;
  function GetFaceNormal(Index: integer): T3DVector; override;
  procedure SetFaceNormal(Index: integer; const Value: T3DVector); override;
  function GetFaceTriangles(Index: integer): TTriangle; override;
  procedure SetFaceTriangles(Index: integer; const Value: TTriangle); override;
  function GetGLColor(Index: integer): TGLFaceColor; override;
  procedure SetGLColor(Index: integer; const Value: TGLFaceColor); override;
  procedure SetRenderMode(const Value: TRenderMode); override;
  procedure SetTransparency(const Value: double); override;
  function GetVertex(Index: integer): T3DVector; override;
  procedure SetVertex(Index: integer; const Value: T3DVector); override;
  function GetVertexColor(Index: integer): TColor; override;
  procedure SetVertexColor(Index: integer; const Value: TColor); override;
  function GetVertexCount: integer; override;
  procedure SetVertexCount(const Value: integer); override;
  function GetVertexNormal(Index: integer): T3DVector; override;
  procedure SetVertexNormal(Index: integer; const Value: T3DVector); override;
 public
  Triangles: TTriInterface;
  OptimizedNormals,UseArea: boolean;
  constructor Create; override;
  destructor Destroy; override;
  procedure Assign(GLObject: TGLObject); override;
  procedure Append(GLObject: TGLObject); override;
  procedure PrepareTexture; override;
  procedure CalculateMinMax; override;

  procedure SetAllNormals(DoVertex: boolean); overload; override;
  procedure SetAllNormals; overload; override;
  procedure SetAllNormals(DoVertex: boolean; CreaseAngle: double); overload; override;
  procedure SetAllNormals(CreaseAngle: double); overload; override;

  procedure SetAdjacentNormals(Target: T3DVector; CreaseAngle: double); override;
  procedure RemoveDegenerateTriangles; override;
  procedure RemoveAdjacentDegenerates(Target: T3DVector); override;
  function RemoveTriangle(I: integer): T3DVector; override;
  procedure FindAdjacentEdge(L: T3DLine; Inx: integer; var Edge: TEdgeInfo); override;

  function AddTriangle(T: TTriangle; GLC: TGLColor): integer; overload; override;
  function AddTriangle(T: TTriangle; C: TColor): integer; overload; override;
  function AddTriangle(P1,P2,P3: T3DVector; C: TColor): integer; overload; override;
  procedure DeleteTriangle(Index: integer); override;
  procedure AddRectangle(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double; C:TColor); overload; override;
  procedure AddRectangle(P1,P2: T3DVector; C: TColor); overload; override;
  procedure AddRectangle(P1,P2,P3,P4: T3DVector; C: TColor); overload; override;
  procedure AddRectangle(R: T3DRectangle; C: TColor); overload; override;
  procedure AddPrism(P1,P2,P3,P4,P5,P6,P7,P8: T3DVector; C: TColor); overload; override;
  procedure AddPrism(Min,Max: T3DVector; C: TColor); overload; override;
  procedure AddPrism(Min,Max: T3DVector; XCount,YCount,ZCount: integer; C: TColor); overload; override;
  procedure AddHexagon(P1, P2, P3, P4, P5, P6: T3DVector; C: TColor); override;
  procedure AddPentagon(P1, P2, P3, P4, P5: T3DVector; C: TColor); override;
  procedure NormalizeScale; override;
  procedure CenterMesh; override;
  procedure MeshClear; override;

  procedure Clear; override;
  procedure ClearData; override;
  procedure Render; override;
  procedure SetColor(const Color: TColor); override;
  function GetColor: TColor; override;
  procedure SetFaceColors(Inx: integer; C1,C2,C3: TColor); override;
  function FaceToTriangle(FaceInx: integer): TTriangle; override;
  procedure FindAdjacentVertices(Old: T3DVector; var VA: TIntegerHashTable); override;
  function FindFirstDuplicate(Target: T3DVector): integer; override;
  procedure FindDuplicateVertices(Target: T3DVector; var VA: TIntegerHashTable); override;
  procedure ChangeAllDuplicateVertices(Old, New: T3DVector); override;

  property RenderMode;
  property OnChange;
  property OnNormalProgress;
  property ShowNormals;
  property NormalColor;
  property ShowFaceNormals;
  property FaceNormalColor;
  property Vertex;
  property VertexColor;
  property VertexCount;
  property VertexNormal;
  property FaceCount;
  property FaceTriangles;
  property FaceColors;
  property FaceNormal;


 {New functions - I think??}

  function AddVertexNormal(V: T3DVector): integer;
  function AddVertexColor(C: TColor): integer;
  function AddVertex(V: T3DVector): integer;

  function AdjacentCols(Row: integer): integer;
  property AdjacentFaces[X,Y: integer]: integer read GetAdjacent;
  procedure VertexDistribution(var Dist: THugeInteger);
end;


{ TMeshList }


type  TGLFastMeshList = class(TGLObjectList)
  private
    function GetTriangles: integer;
  protected
    function GetItem(Index: Integer): TGLFastMesh;
    procedure SetItem(Index: Integer; AObject: TGLFastMesh);
  public
    function Add(var Mesh: TGLFastMesh): integer; overload;
    function Add: integer; overload;
    function Insert(Inx: integer; var Mesh: TGLFastMesh): integer; overload;
    function Insert(Inx: integer): integer; overload;
    property Items[Index: Integer]: TGLFastMesh read GetItem write SetItem; default;
    procedure Assign(ObjectList: TObjectList);
    property Triangles: integer read GetTriangles;
  end;

function GetEdgePoints(Edge: integer; Tri: TTriangle): T3DLine;

function VectorCompare(V1,V2: T3DVector): integer;
function VectorsEqual(V1,V2: T3DVector): boolean;


implementation


{----------------- Support Routines -------------------------------------------}



function NotEquals(V1,V2: double): boolean;
begin
Result:=Abs(V1-V2)>0.00000000001;
end;


function VectorCompare(V1,V2: T3DVector): integer;
{Compare 3D vectors. Returns 0 if equal, <0 if }
{Returns negative if V1<V2 and positive if V1>V2}
begin
if NotEquals(V1.X,V2.X) then Result:=Math.Sign(V1.X-V2.X)
else if NotEquals(V1.Y,V2.Y) then Result:=Math.Sign(V1.Y-V2.Y)
else if NotEquals(V1.Z,V2.Z) then Result:=Math.Sign(V1.Z-V2.Z)
else Result:=0;
end;


function VectorsEqual(V1,V2: T3DVector): boolean;
{Returns true if vectors are equal}
begin
Result:=VectorCompare(V1,V2)=0;
end;




{-------------------------------------------------------------------------}
{                Implementation for TGLFastMesh object                   }
{-------------------------------------------------------------------------}



procedure TGLFastMesh.HandleTextureChange(Sender: TObject);
begin
TextureBuilt:=False;
end;


procedure TGLFastMesh.Change;
begin
MeshBuilt:=False;
TextureBuilt:=False;
if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TGLFastMesh.HandleArrayChange(Sender: TObject);
begin
Change;
end;


constructor TGLFastMesh.Create;
begin
inherited ;
VertexArray:=TVertexArray.Create(5);
FaceArray:=TFaceList.Create(5);
Triangles:=TTriInterface.Create(FaceArray,VertexArray);
VertexArray.BoxPower:=20;
VertexArray.OnChange:=HandleArrayChange;
FaceArray.OnChange:=HandleArrayChange;

Texture.OnChange:=HandleTextureChange;
OptimizedNormals:=False;
UseArea:=False;
end;



destructor TGLFastMesh.Destroy;
begin
Triangles.Free;
FaceArray.Free;
VertexArray.Free;
inherited;
end;



procedure TGLFastMesh.Assign(GLObject: TGLObject);
begin
Clear;
inherited Assign(GLObject);
if GLObject is TGLFastMesh then
	begin
	VertexArray.Assign(TGLFastMesh(GLObject).VertexArray);
	FaceArray.Assign(TGLFastMesh(GLObject).FaceArray);
	Change;
	end;
end;


{------------------------------------------------------------------------------}


procedure TGLFastMesh.MatrixMultiplyVertices(Vertices: THuge3DVector; Matrix: TGLMatrix);
var I: integer;
begin
for I:=0 to Vertices.Count-1 do
	begin
	Vertices[I]:=MatrixVectorMult(Matrix,Vertices[I]);
	end;
end;


procedure TGLFastMesh.ConvertAndAppend(Destination,Source: THuge3DVector; Matrix: TGLMatrix);
var VT: THuge3DVector;
begin
VT:=THuge3DVector.Create(1);
try
VT.Assign(Source);
MatrixMultiplyVertices(VT,Matrix);
Destination.Append(VT);
finally VT.Free; end;
end;



procedure AppendFaces(Dest,Source: TGLFastMesh; Matrix: TGLMatrix);
var I,Inx: integer;
var T: TTriangle;
begin
for I:=0 to Source.Triangles.Count-1 do
	begin
	T:=Source.Triangles[I];
	T.P1:=MatrixVectorMult(Matrix,T.P1);
	T.P2:=MatrixVectorMult(Matrix,T.P2);
	T.P3:=MatrixVectorMult(Matrix,T.P3);
	Inx:=Dest.AddTriangle(T,Source.FaceArray.GLColors[I]);
	Dest.VertexColor[Dest.FaceArray[Inx].P1]:=Source.VertexColor[Source.FaceArray[I].P1];
	Dest.VertexColor[Dest.FaceArray[Inx].P2]:=Source.VertexColor[Source.FaceArray[I].P2];
	Dest.VertexColor[Dest.FaceArray[Inx].P3]:=Source.VertexColor[Source.FaceArray[I].P3];

	Dest.VertexNormal[Dest.FaceArray[Inx].P1]:=MatrixVectorMult(Matrix,Source.VertexNormal[Source.FaceArray[I].P1]);
	Dest.VertexNormal[Dest.FaceArray[Inx].P2]:=MatrixVectorMult(Matrix,Source.VertexNormal[Source.FaceArray[I].P2]);
	Dest.VertexNormal[Dest.FaceArray[Inx].P3]:=MatrixVectorMult(Matrix,Source.VertexNormal[Source.FaceArray[I].P3]);
	Dest.FaceNormal[Inx]:=MatrixVectorMult(Matrix,Source.FaceNormal[I]);
	end;
end;



procedure TGLFastMesh.Append(GLObject: TGLObject);
var VT: THuge3DVector;
var M: TGLMatrix;
begin
VT:=THuge3DVector.Create(1);
try
inherited Append(GLObject);
if GLObject is TGLFastMesh then
	begin
	{Put both meshes in the same relative position by applying each of their matrices}
	MatrixMultiplyVertices(VertexArray.Items,LastMatrix);
	MatrixMultiplyVertices(VertexArray.Normals,LastMatrix);
	MatrixMultiplyVertices(FaceArray.Normals,LastMatrix);

	AppendFaces(Self,TGLFastMesh(GLObject),TGLFastMesh(GLObject).LastMatrix);
	//ConvertAndAppend(VertexArray.Items,TGLFastMesh(GLObject).VertexArray.Items,TGLFastMesh(GLObject).LastMatrix);
	{Now rotate back to the neutral position}
	M:=InvertGLMatrix(LastMatrix);
	MatrixMultiplyVertices(VertexArray.Items,M);
	MatrixMultiplyVertices(VertexArray.Normals,M);
	MatrixMultiplyVertices(FaceArray.Normals,M);

	{Now rotate the new normals into their starting position and then back to the neutral position}
	M:=MatrixMultiply(TGLFastMesh(GLObject).LastMatrix,M);
	M:=RemoveGLMatrixScaling(RemoveGLMatrixPosition(M));
	//ConvertAndAppend(FaceArray.Normals,TGLFastMesh(GLObject).FaceArray.Normals,M);
       	//ConvertAndAppend(VertexArray.Normals,TGLFastMesh(GLObject).VertexArray.Normals,M);
	//VertexArray.GLColors.Append(TGLFastMesh(GLObject).VertexArray.GLColors);
	CalculateMinMax;
	Change;
	end;
finally VT.Free; end;
end;




procedure TGLFastMesh.CalculateMinMax;
var I: integer;
var V: T3DVector;
begin
ExtentMin:=MakeVector3D(MaxDouble,MaxDouble,MaxDouble);
ExtentMax:=MakeVector3D(-MaxDouble,-MaxDouble,-MaxDouble);
for I:=0 to VertexArray.Count-1 do
	begin
	V:=VertexArray[I];
	ExtentMin:=GetMinimumVector(ExtentMin,V);
	ExtentMax:=GetMaximumVector(ExtentMax,V);
	end;
end;




{------------------------------------------------------------------------------}



function TGLFastMesh.GetWeightedNormal(Inx: integer; CreaseAngle: double): T3DVector;
{Returns the normal for a vertex specified by Inx}
{Calculated by averaging the face normals for all}
{faces adjacent to the specified vertex}
var I,TriInx: integer;
var W: double;
var N: T3DVector;


	function GetWeight(Face: TFace; Inx: integer): double;
	var V1,V2: T3DVector;
	begin
	if Face.P1=Inx then
		begin
		V1:=VectorSubtract3D(VertexArray[Face.P2],VertexArray[Face.P1]);
		V2:=VectorSubtract3D(VertexArray[Face.P3],VertexArray[Face.P1]);
		end
	else if Face.P2=Inx then
		begin
		V1:=VectorSubtract3D(VertexArray[Face.P1],VertexArray[Face.P2]);
		V2:=VectorSubtract3D(VertexArray[Face.P3],VertexArray[Face.P2]);
		end
	else
		begin
		V1:=VectorSubtract3D(VertexArray[Face.P1],VertexArray[Face.P3]);
		V2:=VectorSubtract3D(VertexArray[Face.P2],VertexArray[Face.P3]);
		end;
	Result:=VectorAngle3D(V2,V1);
	if UseArea then Result:=Result*GetTriangleArea(MakeTriangle(VertexArray[Face.P1],VertexArray[Face.P2],VertexArray[Face.P3]));
	end;


begin
Result:=MakeVector3D(0,0,0);
{Get the value of the specified vertex}
for I:=0 to VertexArray.AdjacentFaces.Cols[Inx]-1 do
	begin
	TriInx:=VertexArray.AdjacentFaces[I,Inx];
	W:=GetWeight(FaceArray[TriInx],Inx);
	N:=ScalarProduct3D(FaceArray.Normals[TriInx],W);
	Result:=VectorAdd3D(Result,N);
	end;
Result:=UnitVector3D(Result);
end;





function TGLFastMesh.CalculateVertexNormal(Inx: integer; CreaseAngle: double): T3DVector;
{Returns the normal for a vertex specified by Inx}
{Calculated by averaging the face normals for all}
{faces adjacent to the specified vertex}
var I,TriInx: integer;
begin
Result:=MakeVector3D(0,0,0);
{Get the value of the specified vertex}
for I:=0 to VertexArray.AdjacentFaces.Cols[Inx]-1 do
	begin
	TriInx:=VertexArray.AdjacentFaces[I,Inx];
	Result:=VectorAdd3D(Result,FaceArray.Normals[TriInx]);
	end;
Result:=UnitVector3D(Result);
end;




procedure TGLFastMesh.SetVertexNormals(CreaseAngle: double);
{Build vertex normal list}
var N: T3DVector;
var I: integer;
begin
if OptimizedNormals then
	begin
	for I:=0 to VertexArray.Count-1 do
		begin
		N:=GetWeightedNormal(I,CreaseAngle);
		VertexArray.Normals[I]:=N;
		DisplayProgress('Vertex Normals',I,VertexArray.Count);
		end;
	end
else                         
	begin
	for I:=0 to VertexArray.Count-1 do
		begin
		N:=CalculateVertexNormal(I,CreaseAngle);
		VertexArray.Normals[I]:=N;
		DisplayProgress('Vertex Normals',I,VertexArray.Count);
		end;
	end;
end;





procedure TGLFastMesh.SetAllNormals(DoVertex: boolean; CreaseAngle: double);
begin
DisplayProgress('Setting Face Normals',0,1);
if DoVertex then
	begin
	DisplayProgress('Setting Vertices Normals',7,10);
	SetVertexNormals(CreaseAngle);
	end;
DisplayProgress('Done',1,1);
end;




procedure TGLFastMesh.SetFaceNormals;
{Sets all face normals}
var I: integer;
var T: TTriangle;
begin
for I:=0 to FaceArray.Count-1 do
	begin
	T:=Triangles[I];
	FaceArray.Normals[I]:=GetTriangleNormal(T);
	DisplayProgress('Face Normals',I,FaceArray.Count);
	end;
end;


procedure TGLFastMesh.SetAllNormals(DoVertex: boolean);
begin
SetAllNormals(DoVertex,89);
end;



procedure TGLFastMesh.SetAllNormals;
begin
SetAllNormals(True);
end;



procedure TGLFastMesh.SetAllNormals(CreaseAngle: double);
begin
SetAllNormals(True,CreaseAngle);
end;



procedure TGLFastMesh.FindAdjacentTriangles(Target: T3DVector; var TA: THugeInteger);
{Find all triangles adjacent to Target point}
var TInx: integer;
begin
TInx:=VertexArray.Search(Target);
if TInx>=0 then TA.Assign(VertexArray.AdjacentFaces.Row[TInx]);
end;



function TGLFastMesh.FaceToTriangle(FaceInx: integer): TTriangle;
begin
Result:=inherited FaceToTriangle(FaceInx);
Result:=MakeTriangle(VertexArray[FaceArray[FaceInx].P1], VertexArray[FaceArray[FaceInx].P2], VertexArray[FaceArray[FaceInx].P3]);
end;



procedure TGLFastMesh.SetAdjacentNormals(Target: T3DVector; CreaseAngle: double);
{Set normals for triangles that are immediately adjacent to Target}
var I,FInx: integer;
var VA: THugeInteger;
var T: TTriangle;
begin
VA:=THugeInteger.Create(5);
try
FindAdjacentTriangles(Target,VA);
for I:=0 to VA.Count-1 do
	begin
	FInx:=VA[I];
	T:=FaceToTriangle(FInx);
	FaceArray.Normals[FInx]:=GetTriangleNormal(T);
	VertexArray[FaceArray[FInx].P1]:=CalculateVertexNormal(FaceArray[FInx].P1,CreaseAngle);
	end;
Change;
finally VA.Free; end;
end;




procedure TGLFastMesh.RemoveDegenerateTriangles;
{Remove all triangles that have two duplicate points (makes a line)}
var I,Cnt: integer;
begin
Cnt:=0;
for I:=FaceArray.Count-1 downto 0 do
	begin
	if (FaceArray[I].P1=FaceArray[I].P2) or
           (FaceArray[I].P1=FaceArray[I].P3) or
           (FaceArray[I].P2=FaceArray[I].P3) then
		begin
		FaceArray.Delete(I);
		Inc(Cnt)
		end;
	end;
if Cnt<>0 then Change;
end;



procedure TGLFastMesh.RemoveAdjacentDegenerates(Target: T3DVector);
{Remove degenerate triangles that are immediately adjacent to Target}
var I,Cnt: integer;
var VA: THugeInteger;
begin
Cnt:=0;
VA:=THugeInteger.Create(5);
try
FindAdjacentTriangles(Target,VA);
for I:=VA.Count-1 downto 0 do
	begin
	if (FaceArray[VA[I]].P1=FaceArray[VA[I]].P2) or
           (FaceArray[VA[I]].P1=FaceArray[VA[I]].P3) or
           (FaceArray[VA[I]].P2=FaceArray[VA[I]].P3) then
		begin
		FaceArray.Delete(I);
		Inc(Cnt)
		end;
	end;
if Cnt<>0 then Change;
finally VA.Free; end;
end;



function TGLFastMesh.RemoveTriangle(I: integer): T3DVector;
{Remove triangle by collapsing to single point and moving adjacent vertices}
var T: TTriangle;
var F: TFace;
begin
F:=FaceArray[I];
T:=FaceToTriangle(I);
Result:=ScalarDivide3D(VectorAdd3D(VectorAdd3D(T.P1,T.P2),T.P3),3);
VertexArray.Items[F.P1]:=Result;
VertexArray.Items[F.P2]:=Result;
VertexArray.Items[F.P3]:=Result;
FaceArray.Delete(I);
Change;
end;



function GetEdgePoints(Edge: integer; Tri: TTriangle): T3DLine;
begin
case Edge of
 0: begin Result.P1:=Tri.P1; Result.P2:=Tri.P2; end;
 1: begin Result.P1:=Tri.P2; Result.P2:=Tri.P3; end;
 2: begin Result.P1:=Tri.P3; Result.P2:=Tri.P1; end;
 end;
end;





procedure TGLFastMesh.FindAdjacentEdge(L: T3DLine; Inx: integer; var Edge: TEdgeInfo);
var I,FInx,TInx: integer;
var Face: TFace;
var Tri: TTriangle;
var VA: THugeInteger;
begin
VA:=THugeInteger.Create(5);
try
Edge.Inx:=-1;
FindAdjacentTriangles(L.P1,VA);
for I:=0 to VA.Count-1 do
	begin
	FInx:=VA[I];
	if FInx=Inx then continue;
	Face:=FaceArray[FInx];
	Tri:=FaceToTriangle(FInx);
	{Ignore degenerate triangles}
	if (Face.P1 = Face.P2) or (Face.P2 = Face.P3) or (Face.P3 = Face.P1) then continue;
	if (VectorsEqual(Tri.P1, L.P1) and VectorsEqual(Tri.P2,L.P2)) or
	   (VectorsEqual(Tri.P1, L.P2) and VectorsEqual(Tri.P2, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=0;
		exit;
		end
	else if (VectorsEqual(Tri.P2, L.P1) and VectorsEqual(Tri.P3, L.P2)) or
	   (VectorsEqual(Tri.P2, L.P2) and VectorsEqual(Tri.P3, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=1;
		exit;
		end
	else if (VectorsEqual(Tri.P3, L.P1) and VectorsEqual(Tri.P1, L.P2)) or
	   (VectorsEqual(Tri.P3, L.P2) and VectorsEqual(Tri.P1, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=2;
		exit;
		end;
	end;
finally VA.Free; end;
end;


{------------------------------------------------------------------------------}




procedure TGLFastMesh.FindAdjacentVertices(Old: T3DVector; var VA: TIntegerHashTable);
{Find all vertices that are connected by a triangle}
var I,TS: integer;
var Inx: integer;
var VInx,FInx: integer;
begin
VInx:=VertexArray.Search(Old);
for I:=0 to VertexArray.AdjacentFaces.Cols[VInx]-1 do
	begin
	FInx:=VertexArray.AdjacentFaces[I,VInx];
	VA.ADDUnique(FaceArray[FInx].P1);
	VA.ADDUnique(FaceArray[FInx].P2);
	VA.ADDUnique(FaceArray[FInx].P3);
	end;
end;




procedure TGLFastMesh.NormalizeScale;
var I: integer;
begin
for I:=0 to VertexArray.Count-1 do
	VertexArray[I]:=VectorMultiply3D(VertexArray[I],Scale3D);
Scale3D:=MakeVector3D(1,1,1);
Change;
end;



procedure TGLFastMesh.CenterMesh;
var I: integer;
var C: T3DVector;
begin
C:=ScalarDivide3D(VectorAdd3D(ExtentMin,ExtentMax),2);
for I:=0 to VertexArray.Count-1 do
	VertexArray[I]:=VectorSubtract3D(VertexArray[I],C);
Change;
Self.RotationCenter:=MakeVector3D(0,0,0);
end;




procedure TGLFastMesh.MeshClear;
{Clear Mesh only leaving position, rotation and scaling}
begin
VertexArray.Clear;
FaceArray.Clear;
MeshBuilt:=false;
TextureBuilt:=False;
end;


{------------------------------------------------------------------------------}


procedure TGLFastMesh.Clear;
begin
inherited Clear;
MeshClear;
Change;
end;




procedure TGLFastMesh.ClearData;
{Clear data without resetting transforms}
begin
MeshClear;
end;



{----------------- Routines to add/Edit various polygons ----------------------}

function TGLFastMesh.AddTriangle(T: TTriangle; GLC: TGLColor): integer;
{Add colored triangle}
var Face: TFace;
begin
Face.P1:=VertexArray.AddUnique(T.P1,GLC);
Face.P2:=VertexArray.AddUnique(T.P2,GLC);
Face.P3:=VertexArray.AddUnique(T.P3,GLC);
Result:=FaceArray.Add(Face,GLC,GetTriangleNormal(T));

VertexArray.AddFaceLink(Face.P1,Result);
VertexArray.AddFaceLink(Face.P2,Result);
VertexArray.AddFaceLink(Face.P3,Result);
Change;
end;




function TGLFastMesh.AddTriangle(T: TTriangle; C: TColor): integer;
{Add colored triangle}
var Face: TFace;
var GLC: TGLColor;
begin
GLC:=ColorToGLColor(C);
Result:=AddTriangle(T,GLC);
end;



function TGLFastMesh.AddTriangle(P1,P2,P3: T3DVector; C: TColor): integer;
{Add a triangle to the mesh}
var T: TTriangle;
begin
T.P1:=P1; T.P2:=P2; T.P3:=P3;
Result:=AddTriangle(T,C);
end;



procedure TGLFastMesh.AddRectangle(P1,P2,P3,P4: T3DVector; C: TColor);
{Add a rectangle to the mesh}
{assumes 1..4 are counter clockwise facing face}
var T: TTriangle;
begin
T.P1:=P1; T.P2:=P2; T.P3:=P4;
AddTriangle(T,C);
T.P1:=P2; T.P2:=P3; T.P3:=P4;
AddTriangle(T,C);
end;



procedure TGLFastMesh.AddRectangle(R: T3DRectangle; C: TColor);
{Add a rectangle to the mesh}
{assumes 1..4 are counter clockwise facing face}
begin
AddRectangle(R.P1,R.P2,R.P3,R.P4,C);
end;



procedure TGLFastMesh.AddRectangle(P1,P2: T3DVector; C: TColor);
{Add a rectangle to the mesh}
var P3,P4: T3DVector;
begin
P3:=P2;
P2.Y:=P1.Y;
P4:=P1;
P4.Y:=P3.Y;
AddRectangle(P1,P2,P3,P4,C);
end;


procedure TGLFastMesh.AddRectangle(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double; C:TColor);
{Build rectangle based on individual points}
{assumes 1..4 are counter clockwise facing face}
var P1,P2,P3,P4: T3DVector;
begin
P1:=MakeVector3D(X1,Y1,Z1);
P2:=MakeVector3D(X2,Y2,Z2);
P3:=MakeVector3D(X3,Y3,Z3);
P4:=MakeVector3D(X4,Y4,Z4);
AddRectangle(P1,P2,P3,P4,C);
end;





procedure TGLFastMesh.AddHexagon(P1,P2,P3,P4,P5,P6: T3DVector; C: TColor);
var Cnt: T3DVector;
begin
Cnt:=VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(P1,P2),P3),P4),P5),P6);
Cnt:=ScalarDivide3D(Cnt,6);
AddTriangle(Cnt,P1,P2,C);
AddTriangle(Cnt,P2,P3,C);
AddTriangle(Cnt,P3,P4,C);
AddTriangle(Cnt,P4,P5,C);
AddTriangle(Cnt,P5,P6,C);
AddTriangle(Cnt,P6,P1,C);
end;



procedure TGLFastMesh.AddPentagon(P1,P2,P3,P4,P5: T3DVector; C: TColor);
var Cnt: T3DVector;
begin
Cnt:=VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(P1,P2),P3),P4),P5);
Cnt:=ScalarDivide3D(Cnt,5);
AddTriangle(Cnt,P1,P2,C);
AddTriangle(Cnt,P2,P3,C);
AddTriangle(Cnt,P3,P4,C);
AddTriangle(Cnt,P4,P5,C);
AddTriangle(Cnt,P5,P1,C);
end;



procedure TGLFastMesh.AddPrism(P1,P2,P3,P4,P5,P6,P7,P8: T3DVector; C: TColor);
{Add prism based on points, assumes 1..4 are counter clockwise facing one face }
{ and 5..8 are counter clockwise facing the oposite face}
begin
AddRectangle(P1,P2,P3,P4,C);
AddRectangle(P5,P6,P7,P8,C);
AddRectangle(P1,P6,P5,P2,C);
AddRectangle(P2,P5,P8,P3,C);
AddRectangle(P4,P7,P6,P1,C);
AddRectangle(P4,P3,P8,P7,C);
end;



procedure TGLFastMesh.AddPrism(Min,Max: T3DVector; C: TColor);
var P1,P2,P3,P4,P5,P6,P7,P8: T3DVector;
begin
P1:=MakeVector3D(Min.X,Min.Y,Max.Z);
P2:=MakeVector3D(Max.X,Min.Y,Max.Z);
P3:=MakeVector3D(Max.X,Max.Y,Max.Z);
P4:=MakeVector3D(Min.X,Max.Y,Max.Z);

P5:=MakeVector3D(Max.X,Min.Y,Min.Z);
P6:=MakeVector3D(Min.X,Min.Y,Min.Z);
P7:=MakeVector3D(Min.X,Max.Y,Min.Z);
P8:=MakeVector3D(Max.X,Max.Y,Min.Z);

AddPrism(P1,P2,P3,P4,P5,P6,P7,P8,C);
end;




procedure TGLFastMesh.AddPrism(Min,Max: T3DVector; XCount,YCount,ZCount: integer; C: TColor);
var X,Y,Z: integer;
var P1,P2,Step: T3DVector;
begin
Step:=VectorDivide3D(VectorSubtract3D(Max,Min),MakeVector3D(XCount,YCount,ZCount));
for Z:=0 to ZCount-1 do
 for Y:=0 to YCount-1 do
  for X:=0 to XCount-1 do
	begin
	P1:=VectorAdd3D(Min,VectorMultiply3D(MakeVector3D(X,Y,Z),Step));
	P2:=VectorAdd3D(Min,VectorMultiply3D(MakeVector3D(X+1,Y+1,Z+1),Step));
	AddPrism(P1,P2,C);
	end;
end;



procedure TGLFastMesh.SetFaceColors(Inx: integer; C1,C2,C3: TColor);
var I1,I2,I3: integer;
begin
inherited;
VertexArray.Colors[FaceArray[Inx].P1]:=C1;
VertexArray.Colors[FaceArray[Inx].P2]:=C2;
VertexArray.Colors[FaceArray[Inx].P3]:=C3;
end;



function TGLFastMesh.GetGLColor(Index: integer): TGLFaceColor;
begin
Result.C1:=VertexArray.Colors[FaceArray[Index].P1];
Result.C2:=VertexArray.Colors[FaceArray[Index].P2];
Result.C2:=VertexArray.Colors[FaceArray[Index].P3];
end;


procedure TGLFastMesh.SetGLColor(Index: integer; const Value: TGLFaceColor);
begin
VertexArray.Colors[FaceArray[Index].P1]:=Value.C1;
VertexArray.Colors[FaceArray[Index].P2]:=Value.C2;
VertexArray.Colors[FaceArray[Index].P3]:=Value.C3;
end;


{==================== Rendering Routines ======================================}



procedure TGLFastMesh.ShowFaceColor(I: integer);
{Tell OpenGL about the color of specified face}
var R,G,B,A: single;
var C: TGLColor;
begin
C:=FaceArray.GLColors[I];
glColor4f(C.R/255,C.G/255,C.B/255,C.A/255);
end;


{--------------- Wire Triangle Routines ---------------------------------------}



procedure TGLFastMesh.SetWireTriangle(I: integer);
{Tell OpenGL about a face that will be rendered as a wire frame}
var T: TTriangle;
begin
T:=Triangles[I];
glBegin(GL_LINE_LOOP);
glVertex3dv(@T.P1); glVertex3dv(@T.P2); glVertex3dv(@T.P3);
glEnd;
end;



procedure TGLFastMesh.DrawWireTriangle(Inx,Cell: integer);
begin
ShowFaceColor(Inx);
SetWireTriangle(Inx);
end;


procedure TGLFastMesh.DoWireTriangles;
{Render whole mesh as wire frame}
var I: integer;
begin
GLShadeModel(GL_FLAT);
OctalTree.IterateFaces(DrawWireTriangle);
{glEnd not needed here}
end;


{--------------- Flat Triangle Routines ---------------------------------------}



procedure TGLFastMesh.SetFlatTriangle(I: integer);
{Tell OpenGL about a face that will be rendered flat}
var T: TTriangle;
begin
glNormal3dv(FaceArray.Normals.GetItemPtr(I));
T:=Triangles[I];
if Texture.TextureMode=tmManual then glTexCoord2d(Texture.UVCoordinates[FaceArray[I].P1].X, Texture.UVCoordinates[FaceArray[I].P1].Y);
glVertex3dv(@T.P1);
if Texture.TextureMode=tmManual then glTexCoord2d(Texture.UVCoordinates[FaceArray[I].P2].X, Texture.UVCoordinates[FaceArray[I].P2].Y);
glVertex3dv(@T.P2);
if Texture.TextureMode=tmManual then glTexCoord2d(Texture.UVCoordinates[FaceArray[I].P3].X, Texture.UVCoordinates[FaceArray[I].P3].Y);
glVertex3dv(@T.P3);
end;



procedure TGLFastMesh.DrawFlatTriangle(Inx,Cell: integer);
begin
ShowFaceColor(Inx);
SetFlatTriangle(Inx);
end;



procedure TGLFastMesh.DoFlatTriangles;
{Render whole mesh as flat triangles}
var I: integer;
begin
GLShadeModel(GL_FLAT);
glBegin(GL_TRIANGLES);
OctalTree.IterateFaces(DrawFlatTriangle);
glEnd;
end;


{--------------- Smooth Triangle Routines -------------------------------------}


procedure TGLFastMesh.FastSmoothTriangles;
{Render whole mesh as smooth triangles at high speed}
{Only supports vertex colors not face colors }
{So all shared vertices will have the same color }
begin
glPushAttrib(GL_ALL_ATTRIB_BITS);
try
GLShadeModel(GL_SMOOTH);
glEnableClientState(GL_VERTEX_ARRAY);
glEnableClientState(GL_NORMAL_ARRAY);
glEnableClientState(GL_COLOR_ARRAY);
if Texture.TextureMode=tmManual then
	begin
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glTexCoordPointer(2,GL_double,0,Texture.UVCoordinates.GetItemPtr(0));
	end;
glVertexPointer(3, GL_double, 0, VertexArray.Items.GetItemPtr(0));
glNormalPointer(GL_double,0,VertexArray.Normals.GetItemPtr(0));
glColorPointer(4,GL_UNSIGNED_BYTE,0,VertexArray.GLColors.GetItemPtr(0));

glDrawElements(GL_TRIANGLES, FaceArray.Count * 3, GL_UNSIGNED_INT, FaceArray.GetItemPtr(0));
if Texture.TextureMode=tmManual then glDisableClientState(GL_TEXTURE_COORD_ARRAY);
glDisableClientState(GL_COLOR_ARRAY);
glDisableClientState(GL_NORMAL_ARRAY);
glDisableClientState(GL_VERTEX_ARRAY);
finally glPopAttrib; end;
end;





procedure TGLFastMesh.DoSmoothTriangles;
{Render whole mesh a smooth triangles}
begin
FastSmoothTriangles;
end;

{-----------------------------------------------------------------------------}



procedure TGLFastMesh.RenderNormal;
{Handle displaying normal GL mode}
begin
case FRenderMode of
 rmWireFrame: DoWireTriangles;
 rmFlatFace: DoFlatTriangles;
 rmSmoothFace: DoSmoothTriangles;
 end;
end;




{------------------Routines to Handle Displaying Normals ----------------------}



procedure TGLFastMesh.DrawNormalLine(Inx,Cell: integer);
var T,N: TTriangle;
var Face: TFace;
begin
Face:=FaceArray[Inx];
T:=Triangles[Inx];

N.P1:=VectorAdd3D(T.P1,ScalarProduct3D(VertexArray.Normals[Face.P1],FNormalLength));
N.P2:=VectorAdd3D(T.P2,ScalarProduct3D(VertexArray.Normals[Face.P2],FNormalLength));
N.P3:=VectorAdd3D(T.P3,ScalarProduct3D(VertexArray.Normals[Face.P3],FNormalLength));

glBegin(GL_LINES);
glVertex3dv(@T.P1); glVertex3dv(@N.P1);
glVertex3dv(@T.P2); glVertex3dv(@N.P2);
glVertex3dv(@T.P3); glVertex3dv(@N.P3);
glEnd;
end;



procedure TGLFastMesh.DisplayNormals;
{Handle displaying normals as lines}
var R,G,B: single;
var I: integer;
begin
FNormalLength:=VectorABS3D(VectorSubtract3D(ExtentMax,ExtentMin));
FNormalLength:=FNormalLength * 0.1;
GetRGB(NormalColor,R,G,B);
glColor4f(R,G,B,1);
glDisable(GL_LIGHTING);
OctalTree.IterateLines(DrawNormalLine);
//for I:=0 to Vertices.Count-1 do DrawNormalLine(I,0);
{glEnd not needed here}
end;


{-----------------------------------------------------------------------------}



procedure TGLFastMesh.DrawFaceNormalLine(Inx,Cell: integer);
var PC,P1,P2,P3,PN: T3DVector;
begin
P1:=VertexArray[FaceArray[Inx].P1];
P2:=VertexArray[FaceArray[Inx].P2];
P3:=VertexArray[FaceArray[Inx].P3];
PC:=ScalarDivide3D(VectorAdd3D(VectorAdd3D(P1,P2),P3),3);

PN:=FaceArray.Normals[Inx];
PN:=VectorAdd3D(PC,ScalarProduct3D(PN,FNormalLength));
glBegin(GL_LINES);
glVertex3dv(@PC); glVertex3dv(@PN);
glEnd;
end;



procedure TGLFastMesh.DisplayFaceNormals;
{Handle displaying normals as lines}
var R,G,B: single;
var I: integer;
begin
FNormalLength:=VectorABS3D(VectorSubtract3D(ExtentMax,ExtentMin));
FNormalLength:=FNormalLength * 0.1;
GetRGB(FaceNormalColor,R,G,B);
glColor4f(R,G,B,1);
glDisable(GL_LIGHTING);
OctalTree.IterateLines(DrawFaceNormalLine);
{glEnd not needed here}
end;




{-----------------------------------------------------------------------------}




procedure TGLFastMesh.SetupOctalTree;
{Point all triangles in the boxes}
{This is only done when the mesh changed}
var I: integer;
begin
OctalTree.ResetBoxes;
OctalTree.MeshDimensions:=MakeBox(ExtentMin,ExtentMax);
for I:=0 to Triangles.Count-1 do
	begin
	{If triangles span boxes, we'll use the box for the first point}
	OctalTree.PutTriangleInBox(Triangles[I],I);
	end;
end;



procedure TGLFastMesh.PrepareMesh;
begin
SetupOctalTree;
MeshBuilt:=True;
end;



procedure TGLFastMesh.PrepareTexture;
begin
Texture.SetupTextureVertices(Triangles,ExtentMin,ExtentMax);
TextureBuilt:=True;
end;


procedure TGLFastMesh.Render;
{Render whole mesh}
begin
glPushAttrib(GL_ALL_ATTRIB_BITS);
try
if not MeshBuilt then PrepareMesh;
if not TextureBuilt then PrepareTexture;
inherited Render;
if (FRenderMode<>rmSmoothFace) or ShowNormals or ShowFaceNormals then OctalTree.FlagBoxesInFrustum;
RenderNormal;
if ShowNormals then DisplayNormals;
if ShowFaceNormals then DisplayFaceNormals;
finally glPopAttrib; end;
end;



procedure TGLFastMesh.SetColor(const Color: TColor);
var I: integer;
begin
inherited SetColor(Color);
for I:=0 to VertexArray.GLColors.Count-1 do
 VertexArray.Colors[I]:=Color;
for I:=0 to FaceArray.Count-1 do FaceArray.GLColors[I]:=ColorToGLColor(Color);
end;



function TGLFastMesh.GetColor: TColor;
begin
inherited GetColor;
if VertexArray.GLColors.Count>0 then Result:=VertexArray.Colors[0]
else Result:=clWhite;
end;

{------------------ Property Setting Routines ---------------------------------}



procedure TGLFastMesh.SetTransparency(const Value: double);
{Set current render mode}
begin
if VertexArray<>nil then VertexArray.GLColors.SetTransparency((100-Value)/100);
if FaceArray<>nil then FaceArray.GLColors.SetTransparency((100-Value)/100);
Inherited;
end;



function TGLFastMesh.GetExtentMax: T3DVector;
begin
Result:=VertexArray.ExtentMax;
end;


function TGLFastMesh.GetExtentMin: T3DVector;
begin
Result:=VertexArray.ExtentMin;
end;



procedure TGLFastMesh.SetExtentMax(const Value: T3DVector);
begin
inherited;
VertexArray.ExtentMax:=Value;
end;



procedure TGLFastMesh.SetExtentMin(const Value: T3DVector);
begin
inherited;
VertexArray.ExtentMin:=Value;
end;





function TGLFastMesh.GetFaceNormal(Index: integer): T3DVector;
begin
Result:=FaceArray.Normals[Index];
end;



procedure TGLFastMesh.SetFaceNormal(Index: integer; const Value: T3DVector);
begin
FaceArray.Normals[Index]:=Value;
end;


function TGLFastMesh.GetFaceCount: integer;
begin
Result:=FaceArray.Count;
end;



function TGLFastMesh.GetFaceTriangles(Index: integer): TTriangle;
begin
Result:=MakeTriangle(VertexArray[FaceArray[Index].P1], VertexArray[FaceArray[Index].P2], VertexArray[FaceArray[Index].P3]);
end;



procedure TGLFastMesh.SetFaceTriangles(Index: integer; const Value: TTriangle);
begin
VertexArray[FaceArray[Index].P1]:=Value.P1;
VertexArray[FaceArray[Index].P2]:=Value.P2;
VertexArray[FaceArray[Index].P3]:=Value.P3;
end;



function TGLFastMesh.GetFaceIndices(Index: integer): TFace;
begin
Result:=FaceArray[Index];
end;



procedure TGLFastMesh.SetFaceIndices(Index: integer; const Value: TFace);
begin
FaceArray[Index]:=Value;
end;




procedure TGLFastMesh.ChangeAllDuplicateVertices(Old, New: T3DVector);
var Inx: integer;
begin
inherited;
Inx:=VertexArray.Search(Old);
if Inx>=0 then VertexArray[Inx]:=New;
end;




procedure TGLFastMesh.FindDuplicateVertices(Target: T3DVector; var VA: TIntegerHashTable);
var Inx,Start: integer;
begin
inherited;
Inx:=VertexArray.Search(Target);
if Inx>=0 then VA.AddUnique(Inx);
end;




function TGLFastMesh.FindFirstDuplicate(Target: T3DVector): integer;
begin
inherited FindFirstDuplicate(Target);
Result:=VertexArray.Search(Target);
end;


{------------------------------------------------------------------------------}


function TGLFastMesh.GetVertex(Index: integer): T3DVector;
begin
Result:=VertexArray[Index];
end;



procedure TGLFastMesh.SetVertex(Index: integer; const Value: T3DVector);
begin
VertexArray[Index]:=Value;
end;



function TGLFastMesh.GetVertexColor(Index: integer): TColor;
begin
Result:=VertexArray.Colors[Index];
end;


procedure TGLFastMesh.SetVertexColor(Index: integer; const Value: TColor);
begin
VertexArray.Colors[Index]:=Value;
end;


function TGLFastMesh.GetVertexCount: integer;
begin
Result:=VertexArray.Count;
end;



procedure TGLFastMesh.SetVertexCount(const Value: integer);
begin
VertexArray.Count:=Value;
end;



procedure TGLFastMesh.SetVertexNormal(Index: integer; const Value: T3DVector);
begin
VertexArray.Normals[Index]:=Value;
end;



function TGLFastMesh.GetVertexNormal(Index: integer): T3DVector;
begin
Result:=VertexArray.Normals[Index];
end;



function TGLFastMesh.AddVertexNormal(V: T3DVector): integer;
begin
VertexArray.Normals.Add(V);
end;



function TGLFastMesh.AddVertex(V: T3DVector): integer;
begin
Result:=VertexArray.Add(V);
end;



function TGLFastMesh.AddVertexColor(C: TColor): integer;
begin
Result:=VertexArray.GLColors.Add(ColorToGLColor(C));
end;


function TGLFastMesh.AdjacentCols(Row: integer): integer;
begin
Result:=VertexArray.AdjacentFaces.Cols[Row];
end;



function TGLFastMesh.GetAdjacent(X, Y: integer): integer;
begin
Result:=VertexArray.AdjacentFaces[X,Y];
end;


procedure TGLFastMesh.VertexDistribution(var Dist: THugeInteger);
begin
VertexArray.GetDistribution(Dist);
end;



procedure TGLFastMesh.SetRenderMode(const Value: TRenderMode);
begin
if FRenderMode<>Value then MeshBuilt:=False;
Inherited ;
end;



procedure TGLFastMesh.DeleteTriangle(Index: integer);
begin
inherited;
FaceArray.Delete(Index);
end;


{ TMeshList }

function TGLFastMeshList.Add(var Mesh: TGLFastMesh): integer;
begin
Mesh:=TGLFastMesh.Create;
Mesh.Name:='Mesh-'+IntToStr(Count);
Result:=inherited Add(Mesh);
end;



function TGLFastMeshList.Add: integer;
var Mesh: TGLFastMesh;
begin
Result:=Add(Mesh);
end;




function TGLFastMeshList.Insert(Inx: integer; var Mesh: TGLFastMesh): integer;
begin
Mesh:=TGLFastMesh.Create;
Mesh.Name:='Mesh-'+IntToStr(Count);
inherited Insert(Inx,Mesh);
Result:=Inx;
end;



function TGLFastMeshList.Insert(Inx: integer): integer;
var Mesh: TGLFastMesh;
begin
Result:=Insert(Inx,Mesh);
end;



procedure TGLFastMeshList.Assign(ObjectList: TObjectList);
var I: integer;
var O: TGLFastMesh;
begin
if ObjectList is TGLFastMeshList then
	begin
	Clear;
	for I:=0 to ObjectList.Count-1 do
		begin
		Add(O);
		O.Assign(TGLFastMesh(ObjectList[I]));
		end;
	end
else inherited Assign(ObjectList);
end;




function TGLFastMeshList.GetItem(Index: Integer): TGLFastMesh;
begin
Result := TGLFastMesh(inherited GetItem(Index));
end;



function TGLFastMeshList.GetTriangles: integer;
var I: integer;
begin
Result:=0;
for I:=0 to Count-1 do Result:=Result + Self[I].FaceArray.Count;
end;



procedure TGLFastMeshList.SetItem(Index: Integer; AObject: TGLFastMesh);
begin
{This is done so user can replace}
inherited SetItem(Index, AObject);
end;


end.
