unit GLFog;

interface

uses Graphics,OpenGL15,GLLibrary;


type TFogMode = (fmLinear,fmExponential,fmExponentialSquared);
type TFogHint = (fhFast,fhBest);

type TGLFog = class(TObject)
 private
  FFogMode: TFogMode;
  FEnabled: boolean;
  FStop: double;
  FDensity: double;
  FStart: double;
  FColor: TColor;
  FHint: TFogHint;
  procedure SetFogMode(const Value: TFogMode);
  procedure SetColor(const Value: TColor);
  procedure SetDensity(const Value: double);
  procedure SetEnabled(const Value: boolean);
  procedure SetHint(const Value: TFogHint);
  procedure SetStart(const Value: double);
  procedure SetStop(const Value: double);
  procedure ApplyFog;
 protected
 public
  procedure Reset;
  constructor Create;
  property FogMode: TFogMode read FFogMode write SetFogMode;
  property Enabled: boolean read FEnabled write SetEnabled;
  property Density: double read FDensity write SetDensity;
  property Color: TColor read FColor write SetColor;
  property Hint: TFogHint read FHint write SetHint;
  property Start: double read FStart write SetStart;
  property Stop: double read FStop write SetStop;
 end;


implementation


{ TGLFog }



procedure TGLFog.Reset;
begin
FogMode:=fmExponentialSquared;
Density:=0.1;
Color:=clWhite;
Hint:=fhBest;
Start:=0; Stop:=1;
end;


function GetGLFogMode(FM: TFogMode): integer;
begin
case FM of
 fmLinear: Result:=GL_LINEAR;
 fmExponential: Result:=GL_EXP;
 fmExponentialSquared: Result:=GL_EXP2;
 end;
end;



function GetGLFogHint(FH: TFogHint): integer;
begin
case FH of
 fhFast: Result:=GL_FASTEST;
 fhBest: Result:=GL_NICEST;
 end;
end;



procedure TGLFog.ApplyFog;
var fogColor: TGLArrayf4;
begin
FogColor:=ColorToGLRGBA(Color);
if Enabled then glEnable(GL_FOG)
else glDisable(GL_FOG);
glFogi(GL_FOG_MODE, GetGLFogMode(FogMode));
glFogf(GL_FOG_DENSITY, Density);
glFogfv(GL_FOG_COLOR, @fogColor);
glFogf(GL_FOG_START,Start);
glFogf(GL_FOG_END,Stop);
glHint(GL_FOG_HINT, GetGLFogHint(Hint));

end;



constructor TGLFog.Create;
begin
Reset;
end;



procedure TGLFog.SetFogMode(const Value: TFogMode);
begin
if FFogMode<>Value then
	begin
	FFogMode:=Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetColor(const Value: TColor);
begin
if FColor <> Value then
	begin
	FColor := Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetDensity(const Value: double);
begin
if FDensity <> Value then
	begin
	FDensity := Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetEnabled(const Value: boolean);
begin
if FEnabled <> Value then
	begin
	FEnabled := Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetHint(const Value: TFogHint);
begin
if FHint <> Value then
	begin
	FHint := Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetStart(const Value: double);
begin
if FStart <> Value then
	begin
	FStart := Value;
	ApplyFog;
	end;
end;



procedure TGLFog.SetStop(const Value: double);
begin
if FStop<> Value then
	begin
	FStop:= Value;
	ApplyFog;
	end;
end;



end.
