unit GLOutliner;

interface

uses Windows,Types,Graphics,Math,DSupport,BitmapLib,Vector,IntVector,GLPolygons,HugeArry;



type TOutlineProgress = procedure(Percent: integer);



procedure FindImageOutline(Bitmap: TBitmap; var Lines: TPolygonArray; ProgressProc: TOutlineProgress); overload;
procedure FindImageOutline(Bitmap: TBitmap; Optimize: boolean; var RPA: TRealPolygonArray; ProgressProc: TOutlineProgress; Clear: boolean); overload;
procedure SmoothPolygonArray(var Lines: TRealPolygonArray);
procedure ConcatonateLines(var Lines: TRealPolygonArray);
procedure ConcatonatePolygon(var P: TRealPolygon);


implementation



{------------------------------------------------------------------------------}


function IsOnLine(P1,P2,P3: T2DVector): boolean;
var A: double;
var V1,V2: T2DVector;
begin
V1:=VectorSubtract2D(P2,P1);
V2:=VectorSubtract2D(P3,P2);
A:=VectorAngle2D(V1,V2) * Radian2Degree;
Result:=abs(A)<1;
end;




procedure ConcatonatePoints(var Line: TRealPolygon);
var PA: THuge2DVector;
var First,Middle,Last: integer;
var I: integer;
begin
if Line.Count<3 then exit;
PA:=THuge2DVector.Create(1);
try
First:=0; Middle:=0;
while true do
	begin
	Middle:=Middle+1; Last:=Middle+1;
	if Last>=Line.Count then
		begin
		PA.Add(Line[First]);
		PA.Add(Line[Middle]);
		break;
		end;
	if not IsOnLine(Line[First],Line[Middle],Line[Last]) then
		begin
		PA.Add(Line[First]);
		First:=Middle; Middle:=Last;
		if Last>=Line.Count-1 then break;
		end;
     	end;
PA.Add(Line[Middle]);
Line.Count:=PA.Count;
for I:=0 to PA.Count-1 do Line[I]:=PA[I];

finally PA.Free; end;
end;



procedure RemoveStairSteps(var Line: TRealPolygon);
var RP: TRealPolygon;
var H,I,J: integer;
var V1,V2,VT: T2DVector;
var A: double;
begin
RP:=TRealPolygon.Create(Line.Count);
try
for I:=0 to Line.Count-1 do
	begin
	if I=0 then H:=Line.Count-1 else H:=I-1;
	if I=Line.Count-1 then J:=0 else J:=I+1;
	V1:=VectorSubtract2D(Line[H],Line[I]);
	V2:=VectorSubtract2D(Line[J],Line[I]);
	A:=RadToDeg(VectorAngle2D(V1,V2));
	if (Abs(VectorABS2D(V1)-1)>0.1) or
	   (Abs(VectorABS2D(V2)-1)>0.1) or
	   (Abs(A - 90)>0.1) then RP.Add(Line[I])
	else
		begin
		VT:=ScalarDivide2D(VectorAdd2D(Line[H],Line[J]),2);
		VT:=ScalarDivide2D(VectorAdd2D(Line[I],VT),2);
		RP.Add(VT);
		end;
	end;
Line.Assign(RP);
finally RP.Free; end;
end;


procedure SmoothPolygon(var Line: TRealPolygon);
var RP: TRealPolygon;
var G,H,I,J,K: integer;
var Ave: T2DVector;
var A: double;
begin
if Line.Count<4 then exit;
RP:=TRealPolygon.Create(Line.Count);
try
for I:=0 to Line.Count-1 do
	begin
        G:=I-2; H:=I-1;
	J:=I+1; K:=I+2;
	if G<0 then G:=Line.Count+G;
	if H<0 then H:=Line.Count+H;
	if J>=Line.Count then J:=J-Line.Count;
	if K>=Line.Count then K:=K-Line.Count;
	Ave:=VectorAdd2D(Line[G],Line[H]);
	Ave:=VectorAdd2D(Ave,Line[I]);
	Ave:=VectorAdd2D(Ave,Line[J]);
	Ave:=VectorAdd2D(Ave,Line[K]);
	Ave:=ScalarDivide2D(Ave,5);
	Ave:=ScalarDivide2D(VectorAdd2D(Ave,Line[I]),2);
	RP.Add(Ave)
	end;
Line.Assign(RP);
finally RP.Free; end;
end;


procedure SmoothPolygonArray(var Lines: TRealPolygonArray);
var I: integer;
var P: TRealPolygon;
begin
for I:=0 to Lines.Count-1 do
	begin
	P:=Lines[I];
	SmoothPolygon(P);
	end;
end;


procedure ConcatonatePolygon(var P: TRealPolygon);
begin
RemoveStairSteps(P);
ConcatonatePoints(P);
end;



procedure ConcatonateLines(var Lines: TRealPolygonArray);
var I: integer;
var P: TRealPolygon;
begin
for I:=0 to Lines.Count-1 do
	begin
	P:=Lines[I];
	ConcatonatePolygon(P);
	end;
end;



{------------------------------------------------------------------------------}


function FindEdgePoint(BM: TBitmap; var X,Y: integer): boolean;
{Scan columns for first bloack point (note: this is oposite direction 'Potrace')}
var X1,Y1: integer;
begin
Result:=True;
for X1:=X to BM.Width-2 do
 for Y1:=0 to BM.Height-1 do
	begin
	if BM.Canvas.Pixels[X1,Y1]=clBlack then
		begin
		X:=X1; Y:=Y1;
		exit;
		end;
	end;
Result:=False;
end;


function Majority(bm : TBitmap; X,Y: integer): boolean;
var i, a, ct: integer;
begin
Result:=False;
for I:=2 to 4 do
	begin  {  check at "radius" i  }
	ct := 0;
	for a:=-i+1 to i-1 do
		begin
		if BM.Canvas.Pixels[x+a, y+i-1]=clBlack then Inc(CT) else Dec(CT);
		if BM.Canvas.Pixels[x+i-1, y+a-1]=clBlack then Inc(CT) else Dec(CT);
		if BM.Canvas.Pixels[x+a-1, y-i]=clBlack then Inc(CT) else Dec(CT);
		if BM.Canvas.Pixels[x-i, y+a]=clBlack then Inc(CT) else Dec(CT);
		end;
	if  (ct>0)  then
		begin 
		Result:=True;
		exit;
		end
	else if (ct<0) then exit;
	end;

end;



type TTurnPolicy = (tpBlack,tpWhite,tpLeft,tpRight,tpMinority,tpMajority,tpRandom);

function FollowLine(BM: TBitmap; Start: TPoint; Sign: boolean; TurnPolicy: TTurnPolicy; Points: THugePoint): double;
var NextRight, NextLeft: boolean;
var Dir,P: TPoint;

	function DoTurn(P: TPoint; Left: boolean): TPoint;
	begin
	Result.X:=P.Y;
	Result.Y:=P.X;
	if Left then Result.X:=-Result.X
	else Result.Y:=-Result.Y;
	end;
begin
P:=Start;
Dir:=Point(1,0);
Result:=0;

while true do
	begin
	Points.Add(P);
	{  move to next point  }
	P:=IntVectorAdd(P,Dir);
	Result:=Result + P.x * Dir.Y;

	{  path complete?  }
	if  (P.x=Start.x) and (P.y=Start.y)  then break;

	{  determine next direction  }
	NextRight := BM.Canvas.Pixels[ P.x + (Dir.X+Dir.Y-1) div 2, P.y + (Dir.Y-Dir.X-1) div 2]=clBlack;
	NextLeft := BM.Canvas.Pixels[ P.x + (Dir.X-Dir.Y-1) div 2, P.y + (Dir.Y+Dir.X-1) div 2]=clBlack;
        {  ambiguous turn  }
	if (NextRight and not NextLeft) then
		begin                
		if  (turnpolicy = tpRight)   or
		    ((turnpolicy = tpBlack) and  sign)  or
		    ((turnpolicy = tpWhite) and not sign)   or
		    ((turnpolicy = tpRandom) and (Random(2)=1))  or
		    ((turnpolicy = tpMajority) and  majority(bm, P.x, P.y)) or
		    ((turnpolicy = tpMinority) and   not majority(bm, P.x, P.y))  then
 			begin Dir:=DoTurn(Dir,False);
			end
		else Dir:=DoTurn(Dir,True);
    		end
	else if NextRight then Dir:=DoTurn(Dir,False)
	else if not NextLeft then Dir:=DoTurn(Dir,True);
	end;
end;

var LocalPath: THugePoint;

function SortProc(Item1, Item2: Pointer): Integer;
var Inx1,Inx2: integer;
begin
Inx1:=Integer(Item1^);
Inx2:=Integer(Item2^);
Result:=LocalPath[Inx1].Y-LocalPath[Inx2].Y;
if Result=0 then Result:=LocalPath[Inx1].X-LocalPath[Inx2].X;
end;




procedure XORPathInterior(BM: TBitmap; Path: THugePoint);
{XOR the interior image by XOR all points in the polygon all the way to the edge}
{Each pixel may get toggled several time but in the end, only the interior is inverted}
var I: integer;
var X,Y,LastY: integer;
var P: TPoint;
begin
if Path.Count<1 then exit;
LastY:=Path[Path.Count-1].Y;
for I:=0 to Path.Count-1 do
	begin
	P:=Path[I];
	if P.Y<>LastY then
		begin
		for X:=P.X to BM.Width-1 do
			begin
			Y:=Min(P.Y,LastY);
			if BM.Canvas.Pixels[X,Y]=clWhite then BM.Canvas.Pixels[X,Y]:=clBlack
			else BM.Canvas.Pixels[X,Y]:=clWhite;
			end;
		LastY:=P.Y;
		end;
	end;
end;


procedure FindImageOutline(Bitmap: TBitmap; var Lines: TPolygonArray; ProgressProc: TOutlineProgress);
{Outline black on white object in the bitmap}
{Returns a list of lines encircling each object}
var X,Y: integer;
var DelinatedPoints: THugePoint;
var BM: TBitmap;
var Sign: boolean;


	procedure StoreOneLineSegment(var Lines: TPolygonArray; LineList: THugePoint);
	var I: integer;
	begin
	if (LineList.Count<3) then exit;
	SetLength(Lines,Length(Lines)+1);
	SetLength(Lines[High(Lines)],LineList.Count);
	for I:=0 to LineList.Count-1 do Lines[High(Lines),I]:=LineList[I];
	end;


	procedure HandleProgress;
	var Pos,Area: integer;
	begin
	if not Assigned(ProgressProc) then exit;
	Area:=BM.Width * BM.Height;
	Pos:=BM.Height * X + Y;
	ProgressProc(MulDiv(100,Pos,Area));
	end;


begin
BM:=TBitmap.Create;
DelinatedPoints:=THugePoint.Create(1);
try
BM.Assign(Bitmap);
X:=0; Y:=0;

while FindEdgePoint(BM,X,Y) do
	begin
	HandleProgress;
	Sign:=Bitmap.Canvas.Pixels[X,Y]=clBlack;
	if FollowLine(BM,Point(X,Y),Sign,tpBlack,DelinatedPoints)>3 then
		StoreOneLineSegment(Lines,DelinatedPoints);
	XORPathInterior(BM,DelinatedPoints);
	DelinatedPoints.Clear;
	end;
finally
 DelinatedPoints.Free;
 BM.Free;
 end;
end;




procedure FindImageOutline(Bitmap: TBitmap; Optimize: boolean; var RPA: TRealPolygonArray; ProgressProc: TOutlineProgress; Clear: boolean);
var LineArray: TPolygonArray;
var RPA2: TRealPolygonArray;
begin
RPA2:=TRealPolygonArray.Create;
try
FindImageOutline(Bitmap,LineArray,ProgressProc);
LineArrayToRealPolygons(LineArray,RPA2,1,Clear);
if Optimize then ConcatonateLines(RPA2);
if Clear then RPA.Assign(RPA2) else RPA.Append(RPA2);
finally RPA2.Free; end;
end;





end.
