unit GLPolygons;

interface

uses Windows,Types,Classes,SysUtils,Math,DSupport,OpenGL15,IntVector,Vector,HugeArry;


type TPolygon = array of TPoint;
type TPolygonArray = array of TPolygon;

type TGLLineArray = array of TGLArrayd3;
type TGLPolygonArray = array of TGLLineArray;



type TRealPolygon = class(THuge2DVector)
 private
  function GetPoint(Inx: integer): TPoint;
  procedure SetPoint(Inx: integer; const Value: TPoint);
 protected
  function GetPolygon: TPolygon;
 public
  ControlPoints1: THuge2DVector;
  ControlPoints2: THuge2DVector;
  constructor Create(Number: integer);
  destructor Destroy; override;
  function Add(Item: T2DVector): integer; reintroduce;
  function AddPointer(Item: T2DVector; Ptr: integer): Integer; override;
  procedure Insert(Value: T2DVector; Index: integer); override;
  procedure Delete(Index: integer); override;
  procedure Move(CurIndex, NewIndex: Integer); override;
  procedure Clear; override;
  property Polygon: TPolygon read GetPolygon;
  property Points[Inx: integer]: TPoint read GetPoint write SetPoint;
  procedure Assign(Source: TRealPolygon);
 end;


type TRealPolygonArray = class(TList)
 private
  FOnChange: TNotifyEvent;
  function GetPolygon(Inx: integer): TRealPolygon;
 protected
  procedure Notify(Ptr: Pointer; Action: TListNotification); override;
  procedure ChangeHandler(Sender: TObject);
 public
  function Add: integer;
  procedure Append(PA: TRealPolygonArray);
  procedure Assign(PA: TRealPolygonArray);
  procedure Clear; override;
  property Polygon[Inx: integer]: TRealPolygon read GetPolygon; default;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
 end;

procedure ReversePolygon(var Poly: TPolygon);
procedure NormalizePolygons(var Polygons: TPolygonArray);

function PolygonArea(LA: TGLLineArray): double; overload;
function PolygonArea(Poly: TPolygon): double; overload;
function PolygonArea(Poly: TRealPolygon): double; overload;
function PolygonArea(Poly: T2DVectorArray): double; overload;


function IsCLockwise(LA: TGLLineArray): boolean; overload;
function IsCLockwise(LA: TRealPolygon): boolean; overload;
function IsCLockwise(Poly: TPolygon): boolean; overload;

function PointInPolygon(LA: TGLLineArray; Point: TGLArrayd3): boolean; overload;
function PointInPolygon(Poly: TPolygon; P: TPoint): boolean; overload;
function PointInPolygon(Poly: TRealPolygon; P: T2DVector): boolean; overload;
function PointInPolygon(Poly: TRealPolygon; P: TPoint): boolean; overload;
function PointInPolygon(Poly: T2DVectorArray; P: T2DVector): boolean; overload;
function FindPolygonCenter(RP: THuge2DVector): T2DVector;
function FindRayPolygonIntercept(RP: THuge2DVector; A: double): T2DVector;

function PolygonIntersect(P1,P2: T2DVectorArray): boolean;

procedure ReverseLines(var Lines: TGLLineArray); overload;
procedure ReverseLines(Lines: TRealPolygon); overload;


function GetWindingNumber(PA: TGLPolygonArray; Inx: integer): integer; overload;
function GetWindingNumber(PA: TRealPolygonArray; Inx: integer): integer; overload;
function GetWindingNumber(PA: TPolygonArray; Inx: integer): integer; overload;


function IsExteriorPoint(PA: TRealPolygonArray; Inx: integer): boolean; overload;
function IsExteriorPoint(PA: TGLPolygonArray; Inx: integer): boolean; overload;
function IsExteriorPoint(PA: TPolygonArray; Inx: integer): boolean; overload;

procedure LineArrayToRealPolygons(LineArray: TPolygonArray; var Polygons: TRealPolygonArray; Scale: double); overload;
procedure LineArrayToRealPolygons(LineArray: TPolygonArray; var Polygons: TRealPolygonArray; Scale: double; Clear: boolean); overload;

procedure LineArrayToPolygons(LineArray: TPolygonArray; var Polygons: TGLPolygonArray; Scale: double);
procedure RealPolygonArrayToGLPolygons(RPA: TRealPolygonArray; var Polygons: TGLPolygonArray; Scale: double);
function CenterPolygons(PA: TPolygonArray): TPolygonArray; overload;
procedure CenterPolygons(var PA: TRealPolygonArray); overload;
procedure InsertPoints(var PG: TPolygon; Inx: integer; NewPts: TPolygon); overload;
procedure InsertPoints(var PG: TRealPolygon; Inx: integer; NewPts: TPolygon); overload;

procedure GetSplineLine(I: integer; RP: TRealPolygon; Steps: integer; var PG: T2DVectorArray); overload;
procedure GetSplineLine(I: integer; RP: TRealPolygon; Steps: integer; ClearArray: boolean; var PG: T2DVectorArray); overload;
procedure GetSplinePolygon(RP: TRealPolygon; Steps: integer; Closed: boolean; var PG: T2DVectorArray);

procedure InflatePolygon(Points,Points2: TRealPolygon; IgnoreWinding,TrimLoops: boolean; Delta: double);
function GetPolygonCenter(Points: TRealPolygon): T2DVector;
procedure ScalePolygon(Points: TRealPolygon;  Scale: double);
procedure RotatePolygon(Points: TRealPolygon;  Angle: double);
procedure GetPolygonSize(Points: TRealPolygon; var Min,Max: T2DVector);
procedure ScalePolygonArray(A: TRealPolygonArray; Scale: double);
procedure GetPolygonArraySize(A: TRealPolygonArray; var Min,Max: T2DVector);
procedure OffsetPolygonArray(A: TRealPolygonArray; Offset: T2DVector);
procedure OffsetPolygon(Points: TRealPolygon; Offset: T2DVector);

function GetPolyPolyRegion(Polys: TRealPolygonArray): HRgn;
function GetSplinePolyPolyRegion(Polys: TRealPolygonArray; Steps: integer): HRgn;

implementation

uses SplineLib;



{ TRealPolygon }

constructor TRealPolygon.Create(Number: integer);
begin
inherited Create(Number);
ControlPoints1:=THuge2DVector.Create(Number);
ControlPoints2:=THuge2DVector.Create(Number);
end;


destructor TRealPolygon.Destroy;
begin
ControlPoints1.Destroy;
ControlPoints2.Destroy;
inherited Destroy;
end;


function TRealPolygon.Add(Item: T2DVector): integer;
begin
ControlPoints1.Add(MakeVector2D(0,0));
ControlPoints2.Add(MakeVector2D(0,0));
Result:=Inherited Add(Item);
end;


function TRealPolygon.AddPointer(Item: T2DVector; Ptr: integer): Integer;
begin
ControlPoints1.Add(MakeVector2D(0,0));
ControlPoints2.Add(MakeVector2D(0,0));
Result:=Inherited AddPointer(Item,PTr);
end;


procedure TRealPolygon.Delete(Index: integer);
begin
inherited Delete(Index);
ControlPoints1.Delete(Index);
ControlPoints2.Delete(Index);
end;


procedure TRealPolygon.Insert(Value: T2DVector; Index: integer);
begin
inherited;
ControlPoints1.Insert(MakeVector2D(0,0),Index);
ControlPoints2.Insert(MakeVector2D(0,0),Index);
end;


procedure TRealPolygon.Move(CurIndex, NewIndex: Integer);
begin
inherited Move(CurIndex,NewIndex);
ControlPoints1.Move(CurIndex,NewIndex);
ControlPoints2.Move(CurIndex,NewIndex);
end;



function TRealPolygon.GetPolygon: TPolygon;
var I: integer;
begin
SetLength(Result,Self.Count);
for I:=0 to Self.Count-1 do Result[I]:=Vector2DToPoint(Self[I]);
end;


function TRealPolygon.GetPoint(Inx: integer): TPoint;
begin
Result:=Vector2DToPoint(Self[Inx]);
end;

procedure TRealPolygon.SetPoint(Inx: integer; const Value: TPoint);
begin
Self[Inx]:=PointToVector2D(Value);
end;

procedure TRealPolygon.Clear;
begin
ControlPoints1.Clear;
ControlPoints2.Clear;
inherited;
end;


procedure TRealPolygon.Assign(Source: TRealPolygon);
begin
inherited Assign(Source);
ControlPoints1.Assign(Source.ControlPoints1);
ControlPoints2.Assign(Source.ControlPoints2);
end;


{ TRealPolygonArray }

function TRealPolygonArray.Add: integer;
var Poly: TRealPolygon;
begin
Poly:=TRealPolygon.Create(5);
Result:=inherited Add(Poly);
Poly.OnChange:=ChangeHandler;
end;



procedure TRealPolygonArray.Append(PA: TRealPolygonArray);
var I,Inx: integer;
begin
Capacity:=Capacity+PA.Capacity;
for I := 0 to PA.Count - 1 do
	begin
	Inx:=Self.Add;
	Self[Inx].Assign(PA[I]);
	Self[Inx].ControlPoints1.Assign(PA[I].ControlPoints1);
	Self[Inx].ControlPoints2.Assign(PA[I].ControlPoints2);
	end;
ChangeHandler(Self);
end;



procedure TRealPolygonArray.Assign(PA: TRealPolygonArray);
var I,Inx: integer;
begin
Clear;
Append(PA);
end;




procedure TRealPolygonArray.ChangeHandler(Sender: TObject);
begin
if Assigned(FOnChange) then FOnChange(Self);
end;



procedure TRealPolygonArray.Clear;
var I: integer;
begin
for I:=0 to Self.Count-1 do
 if Self[I]<>nil then Self[I].Free;
Self.Count:=0;
ChangeHandler(Self);
inherited;
end;



function TRealPolygonArray.GetPolygon(Inx: integer): TRealPolygon;
begin
Result:=TRealPolygon(Items[Inx]);
end;






procedure TRealPolygonArray.Notify(Ptr: Pointer;
  Action: TListNotification);
begin
inherited;
ChangeHandler(Self);
end;




{------------------------------------------------------------------------------}




procedure GetSplineLine(I: integer; RP: TRealPolygon; Steps: integer; ClearArray: boolean; var PG: T2DVectorArray);
var J: integer;
var L1,L2: T2DLine;
begin
if I=RP.Count-1 then J:=0 else J:=I+1;
L1.P1:=RP[I]; L1.P2:=VectorAdd2D(L1.P1,RP.ControlPoints1[I]);
L2.P2:=RP[J]; L2.P1:=VectorAdd2D(L2.P2,RP.ControlPoints2[J]);
DoCubicSplineLine(Steps,L1,L2,ClearArray,PG);
end;



procedure GetSplineLine(I: integer; RP: TRealPolygon; Steps: integer; var PG: T2DVectorArray);
begin
GetSplineLine(I,RP,Steps,True,PG);
end;




procedure GetSplinePolygon(RP: TRealPolygon; Steps: integer; Closed: boolean; var PG: T2DVectorArray);
{Generate all splines for all points in polygon taking care of overlap}
var Cnt,I: integer;
begin
SetLength(PG,0);
if Closed then Cnt:=RP.Count-1 else Cnt:=RP.Count-2;
for I:=0 to Cnt do
	begin
	GetSplineLine(I,RP,Steps,False,PG);
	if I<>Cnt then SetLength(PG,High(PG));
	end;
end;



{------ Library routines for OpenGL Array -------------------------------------}



procedure NormalizePolygons(var Polygons: TPolygonArray);
{Normalize polygons so interior and exterior polygons are counter clockwise and interiors are clockwise}
{Note this routines assumes screen coordinate where Y is reversed from opengl}
var I: integer;
begin
for I:=0 to High(Polygons) do
	begin
	if Length(Polygons[I])<1 then continue;
	if IsExteriorPoint(Polygons,I) then
		begin
		if not IsClockwise(Polygons[I]) then ReversePolygon(Polygons[I]);
		end
	else
		begin
		if IsClockwise(Polygons[I]) then ReversePolygon(Polygons[I]);
		end;
	end;
end;


procedure ReversePolygon(var Poly: TPolygon);
{Reverse the order of a polygon}
var T: TPoint;
var I,I2: integer;
begin
I2:=High(Poly);
for I:=0 to High(Poly) div 2 do
	begin
        T:=Poly[I2]; Poly[I2]:=Poly[I]; Poly[I]:=T;
        Dec(I2);
        end;
end;

function PolygonArea(LA: TGLLineArray): double;
{Return polygon areay - GL version}
{K = [(x1y2 + x2y3 + x3y4 + ... + xny1) - (x2y1 + x3y2 + x4y3 + ... + x1yn)]/2}
var I,J: integer;
var Sum1,Sum2: double;
begin
Sum1:=0; Sum2:=0;
for I:=0 to High(LA) do
	begin
	if I=High(LA) then J:=0 else J:=I+1;
	Sum1:=Sum1+ LA[I,0] * LA[J,1];
	Sum2:=Sum2+ LA[J,0] * LA[I,1];
	end;

Result:=(Sum1 - Sum2)/2;
end;


function IsCLockwise(LA: TGLLineArray): boolean;
var A: double;
begin
A:=PolygonArea(LA);
Result:=A>=0;
end;



function IsCLockwise(Poly: TPolygon): boolean;
var A: double;
begin
A:=PolygonArea(Poly);
Result:=A>=0;
end;



function PointInPolygon(LA: TGLLineArray; Point: TGLArrayd3): boolean;
{Returns true if point is inside polygon }
{Warning: assumes last point is Not same as first}
var I,J: integer;
begin
Result:=False;
for I:=0 to High(LA) do
	begin
	if I=High(LA) then J:=0
	else J:=I+1;
	if ((((LA[i,1] <= Point[1]) and (Point[1] < LA[j,1])) or
             ((LA[j,1] <= Point[1]) and (Point[1] < LA[i,1]))) and
            (Point[0] < (LA[j,0] - LA[i,0]) * (Point[1] - LA[i,1]) / (LA[j,1] - LA[i,1]) + LA[i,0]))
            then Result:=not Result;
	end;
end;



function GetWindingNumber(PA: TGLPolygonArray; Inx: integer): integer;
var I: integer;
var Point: TGLArrayd3;
begin
Result:=0;
Point:=PA[Inx,0];
for I:=0 to High(PA) do
 if (I<>Inx) and PointInPolygon(PA[I],Point) then Inc(Result);
end;



function GetWindingNumber(PA: TPolygonArray; Inx: integer): integer;
var I: integer;
var Point: TPoint;
begin
Result:=0;
Point:=PA[Inx,0];
for I:=0 to High(PA) do
 if (I<>Inx) and PointInPolygon(PA[I],Point) then Inc(Result);
end;



function IsExteriorPoint(PA: TGLPolygonArray; Inx: integer): boolean;
var WN: integer;
begin
WN:=GetWindingNumber(PA,Inx);
Result:=(WN and 1)=0;
end;



function IsExteriorPoint(PA: TPolygonArray; Inx: integer): boolean;
var WN: integer;
begin
WN:=GetWindingNumber(PA,Inx);
Result:=(WN and 1)=0;
end;





procedure ReverseLines(var Lines: TGLLineArray);
var T: TGLArrayd3;
var I,I2: integer;
begin
I2:=High(Lines);
for I:=0 to High(Lines) div 2 do
	begin
        T:=Lines[I2]; Lines[I2]:=Lines[I]; Lines[I]:=T;
        Dec(I2);
        end;
end;

{------ Library routines for Ordinary Windows Point Arrays --------------------}


function PolygonArea(Poly: TPolygon): double;
{Return polygon areay - Integer version}
{K = [(x1y2 + x2y3 + x3y4 + ... + xny1) - (x2y1 + x3y2 + x4y3 + ... + x1yn)]/2}
var I,J: integer;
var Sum1,Sum2: double;
begin
Sum1:=0; Sum2:=0;
for I:=0 to High(Poly) do
	begin
	if I=High(Poly) then J:=0 else J:=I+1;
	Sum1:=Sum1+ Poly[I].X * Poly[J].Y;
	Sum2:=Sum2+ Poly[J].X * Poly[I].Y;
	end;
Result:=(Sum1 - Sum2)/2;
end;




function PointInPolygon(Poly: TPolygon; P: TPoint): boolean;
{Integer version}
var I,J: integer;
var OddNodes: boolean;
begin
OddNodes:=False;
J:=High(Poly);
for I:=0 to High(Poly) do
	begin
	if ((Poly[i].Y<P.Y) and (Poly[j].Y>=P.Y))  or
	   ((Poly[j].Y<P.Y) and (Poly[i].Y>=P.Y)) then
		begin
		if (Poly[I].X+(P.Y-Poly[I].Y) / (Poly[j].Y-Poly[i].Y) * (Poly[j].X-Poly[i].X) < P.X) then
			begin
			OddNodes:=not oddNodes;
			end;
		end;
	J:=I;
	end;
Result:=OddNodes;
end;


procedure LineArrayToPolygons(LineArray: TPolygonArray; var Polygons: TGLPolygonArray; Scale: double);
var I,J: integer;
var GA: TGLArrayd3;
begin
SetLength(Polygons,Length(LineArray));
for I:=0 to High(LineArray) do
	begin
	SetLength(Polygons[I],Length(LineArray[I]));
	for J:=0 to High(LineArray[I]) do
		begin
		GA[0]:=LineArray[I,J].X/Scale;
		GA[1]:=-1*LineArray[I,J].Y/Scale;
		GA[2]:=0;
		Polygons[I,J]:=GA;
		end;
	end;
end;




function CopyPolygonArray(PA: TPolygonArray): TPolygonArray;
var I,J: integer;
begin
SetLength(Result,Length(PA));
for I:=0 to High(PA) do SetLength(Result[I],Length(PA[I]));
for I:=0 to High(PA) do
 for J:=0 to High(PA[I]) do
	Result[I,J]:=PA[I,J];
end;




function CenterPolygons(PA: TPolygonArray): TPolygonArray;
{Adjust polygon so all points are centered around center}
var I,J,Cnt: integer;
var C: TPoint;
begin
Result:=CopyPolygonArray(PA);
Cnt:=0;
C:=Point(0,0);
for I:=0 to High(Result) do
 for J:=0 to High(Result[I]) do
	begin
	C:=IntVectorAdd(C,Result[I,J]);
	Inc(Cnt);
	end;
C:=IntScalarDivide(C,Cnt);
for I:=0 to High(Result) do
 for J:=0 to High(Result[I]) do
	begin
	Result[I,J]:=IntVectorSubtract(Result[I,J],C);
	end;
end;



procedure InsertPoints(var PG: TPolygon; Inx: integer; NewPts: TPolygon);
var I,IP: integer;
var NewCnt: integer;
begin
IP:=Inx+1;
NewCnt:=Length(NewPts);
SetLength(PG,Length(PG)+NewCnt);
for I:=High(PG)-NewCnt downto IP do
	begin
	PG[I+NewCnt]:=PG[I];
	end;
for I:=0 to NewCnt-1 do
PG[I+IP]:=NewPts[I];
end;


{------ Library routines for Dynamic 2dVector Arrays --------------------------}




function PointInPolygon(Poly: T2DVectorArray; P: T2DVector): boolean;
{Integer version}
var I,J: integer;
var OddNodes: boolean;
begin
OddNodes:=False;
J:=High(Poly);
for I:=0 to High(Poly) do
	begin
	if ((Poly[i].Y<P.Y) and (Poly[j].Y>=P.Y))  or
	   ((Poly[j].Y<P.Y) and (Poly[i].Y>=P.Y)) then
		begin
		if (Poly[I].X+(P.Y-Poly[I].Y) / (Poly[j].Y-Poly[i].Y) * (Poly[j].X-Poly[i].X) < P.X) then
			begin
			OddNodes:=not oddNodes;
			end;
		end;
	J:=I;
	end;
Result:=OddNodes;
end;



function PolygonIntersect(P1,P2: T2DVectorArray): boolean;
{Return true if the specified polygons intersect?}
var I: integer;
begin
Result:=True;
for I:=0 to High(P1) do
 if PointInPolygon(P2,P1[I]) then exit;
for I:=0 to High(P2) do
 if PointInPolygon(P1,P2[I]) then exit;
Result:=False;
end;



function PolygonArea(Poly: T2DVectorArray): double;
{Return polygon area - Dynamic 2D Vector version}
{K = [(x1y2 + x2y3 + x3y4 + ... + xny1) - (x2y1 + x3y2 + x4y3 + ... + x1yn)]/2}
var I,J: integer;
var Sum1,Sum2: double;
begin
Sum1:=0; Sum2:=0;
for I:=0 to High(Poly) do
	begin
	if I=High(Poly) then J:=0 else J:=I+1;
	Sum1:=Sum1+ Poly[I].X * Poly[J].Y;
	Sum2:=Sum2+ Poly[J].X * Poly[I].Y;
	end;
Result:=(Sum1 - Sum2)/2;
end;



function FindPolygonCenter(RP: THuge2DVector): T2DVector;
var I: integer;
begin
Result:=MakeVector2D(0,0);
for I:=0 to RP.Count-1 do Result:=VectorAdd2D(Result,RP[I]);
Result:=ScalarDivide2D(Result,RP.Count);
end;




function FindRayPolygonIntercept(RP: THuge2DVector; A: double): T2DVector;
var I,I2: integer;
var C: T2DVector;
var L1,L2: T2DLine;
begin
C:=FindPolygonCenter(RP);
L1:=CompassToLine2D(C,A,1E6);
for I:=0 to RP.Count-1 do
	begin
	I2:=I+1;
	if I2>=RP.Count then I2:=0;
	L2.P1:=RP[I]; L2.P2:=RP[I2];
	if DoLinesIntersect2D(L1,L2,Result) then break;
	end;
end;

{------ Library routines for Huge Real Point Arrays --------------------------}



function PolygonArea(Poly: TRealPolygon): double;
{Return polygon area Real Polygon Version}
{K = [(x1y2 + x2y3 + x3y4 + ... + xny1) - (x2y1 + x3y2 + x4y3 + ... + x1yn)]/2}
var I,J: integer;
var Sum1,Sum2: double;
begin
Sum1:=0; Sum2:=0;
for I:=0 to Poly.Count-1 do
	begin
	if I=(Poly.Count-1) then J:=0 else J:=I+1;
	Sum1:=Sum1+ Poly[I].X * Poly[J].Y;
	Sum2:=Sum2+ Poly[J].X * Poly[I].Y;
	end;
Result:=(Sum1 - Sum2)/2;
end;




function IsCLockwise(LA: TRealPolygon): boolean;
var A: double;
begin
A:=PolygonArea(LA);
Result:=A>=0;
end;



function PointInPolygon(Poly: TRealPolygon; P: T2DVector): boolean;
{Integer version}
var I,J: integer;
var OddNodes: boolean;
begin
OddNodes:=False;
J:=Poly.Count-1;
for I:=0 to Poly.Count-1 do
	begin
	if ((Poly[i].Y<P.Y) and (Poly[j].Y>=P.Y))  or
	   ((Poly[j].Y<P.Y) and (Poly[i].Y>=P.Y)) then
		begin
		if (Poly[I].X+(P.Y-Poly[I].Y) / (Poly[j].Y-Poly[i].Y) * (Poly[j].X-Poly[i].X) < P.X) then
			begin
			OddNodes:=not oddNodes;
			end;
		end;
	J:=I;
	end;
Result:=OddNodes;
end;




function PointInPolygon(Poly: TRealPolygon; P: TPoint): boolean;
begin
Result:=PointInPolygon(Poly,PointToVector2D(P));
end;






procedure RealPolygonArrayToGLPolygons(RPA: TRealPolygonArray; var Polygons: TGLPolygonArray; Scale: double);
var I,J: integer;
var GA: TGLArrayd3;
begin
SetLength(Polygons,RPA.Count);
for I:=0 to RPA.Count-1 do
	begin
	SetLength(Polygons[I],RPA[I].Count);
	for J:=0 to RPA[I].Count-1 do
		begin
		GA[0]:=RPA[I][J].X/Scale;
		GA[1]:=-1*RPA[I][J].Y/Scale;
		GA[2]:=0;
		Polygons[I,J]:=GA;
		end;
	end;
end;



procedure CenterPolygons(var PA: TRealPolygonArray);
{Adjust polygon so all points are centered around center}
var I,J,Cnt: integer;
var C: T2DVector;
begin
Cnt:=0;
C:=MakeVector2D(0,0);
for I:=0 to PA.Count-1 do
 for J:=0 to PA[I].Count-1 do
	begin
	C:=VectorAdd2D(C,PA[I][J]);
	Inc(Cnt);
	end;
C:=ScalarDivide2D(C,Cnt);
for I:=0 to PA.Count-1 do
 for J:=0 to PA[I].Count-1 do
	begin
	PA[I][J]:=VectorSubtract2D(PA[I][J],C);
	end;
end;



procedure InsertPoints(var PG: TRealPolygon; Inx: integer; NewPts: TPolygon);
var I,IP: integer;
var NewCnt: integer;
begin
IP:=Inx+1;
NewCnt:=Length(NewPts);
PG.Count:=PG.Count+NewCnt;
for I:=(PG.Count-1)-NewCnt downto IP do
	begin
	PG[I+NewCnt]:=PG[I];
	end;
for I:=0 to NewCnt-1 do
	PG[I+IP]:=PointToVector2D(NewPts[I]);
end;



function GetWindingNumber(PA: TRealPolygonArray; Inx: integer): integer;
var I: integer;
var Point: T2DVector;
begin
Result:=0;
Point:=PA[Inx][0];
for I:=0 to PA.Count-1 do
 if (I<>Inx) and PointInPolygon(PA[I],Point) then Inc(Result);
end;



function IsExteriorPoint(PA: TRealPolygonArray; Inx: integer): boolean;
var WN: integer;
begin
WN:=GetWindingNumber(PA,Inx);
Result:=(WN and 1)=0;
end;


procedure ReverseLines(Lines: TRealPolygon);
var T: T2DVector;
var I,I2: integer;
begin
I2:=Lines.Count-1;
for I:=0 to (Lines.Count-1) div 2 do
	begin
        T:=Lines[I2]; Lines[I2]:=Lines[I]; Lines[I]:=T;
        Dec(I2);
        end;
end;


procedure LineArrayToRealPolygons(LineArray: TPolygonArray; var Polygons: TRealPolygonArray; Scale: double; Clear: boolean);
var I,J,Inx: integer;
var RP: TRealPolygon;
begin
if Clear then Polygons.Clear;
for I:=0 to High(LineArray) do
	begin
	Inx:=Polygons.Add;
	RP:=Polygons[Inx];
	for J:=0 to High(LineArray[I]) do
		begin
		RP.Add(MakeVector2D(LineArray[I,J].X/Scale,LineArray[I,J].Y/Scale));
		end;
	end;
end;



procedure LineArrayToRealPolygons(LineArray: TPolygonArray; var Polygons: TRealPolygonArray; Scale: double);
begin
LineArrayToRealPolygons(LineArray,Polygons,Scale,True);
end;




{------------------------------------------------------------------------------}




function AngledLineFromLine(L: T2DLine; P:T2DVector; Dist:extended; alpha: extended):T2DLine;
{compute a line from point, P, on line, L, for a specified distance, dist
 at angle, alpha.  }
var theta, newangle:extended;
begin
if L.p1.x<>L.p2.x then theta:=arctan2((L.p1.y-L.p2.y),(L.p2.x-L.p1.x)) {make sure slope is not infinite}
else if L.p2.y<L.p1.y then theta:=pi/2 else theta:=-pi/2; {vertical line}

Result.p1:=p;
newangle:=theta+alpha;
Result.p2.x:=p.x+(dist*cos(newangle));
Result.p2.y:=p.y-(dist*sin(newangle));
end;


function HandleColinearLines(L1,L2: T2DLine): T2DVector;
begin
{Do they intersect at end points?}
if AreEqual2D(L1.P1,L2.P1) or AreEqual2D(L1.P1,L2.P2) then Result:=L1.P1
else if AreEqual2D(L1.P2,L2.P1) or AreEqual2D(L1.P2,L2.P2) then Result:=L1.P2
else Result:=L1.P2;
end;



procedure DoTrimLoops(Points: TRealPolygon);
{Trim loops by putting by setting all points in the loop to the intereset point}
var I,J,K,L,M: integer;
var L1,L2: T2DLine;
var Intercept: T2DVector;
var Loops: array of TPoint;
var Intercepts: array of T2DVector;
var Start,Stop: integer;
begin
for I:=Points.Count-2 downto 0 do
	begin
	J:=I+1;
	for K:=J to Points.Count-1 do
		begin
		if K>=Points.Count-1 then L:=0 else L:=K+1;
		L1:=Make2DLine(Points[I],Points[J]);
		L2:=Make2DLine(Points[K],Points[L]);
		if DoLinesIntersect2D(L1,L2,Intercept) then
			begin
			SetLength(Loops,Length(Loops)+1);
			Loops[High(Loops)]:=Point(J,K);
			SetLength(Intercepts,Length(Intercepts)+1);
			Intercepts[High(Intercepts)]:=Intercept;
			end;
		end;
	end;
for I:=0 to High(Loops) do
	begin
	Start:=Loops[I].X; Stop:=Loops[I].Y;
	if ((Points.Count-Stop) + Start)<(Stop-Start) then
		begin
                for M:=Stop to Points.Count-1 do Points[M]:=Intercepts[I];
                for M:=0 to Start do Points[M]:=Intercepts[I];
		end
	else for M:=Start to Stop do Points[M]:=Intercepts[I];
	end;
end;




procedure InflatePolygon(Points,Points2: TRealPolygon; IgnoreWinding,TrimLoops: boolean; Delta: double);
{Inflate rectangle by specified amount - if IgnoreWinding is true, positive deltas }
{always expand. if it is false, CCW polygons expand and CW polygons shrink }
var H,I,J: integer;
var clockwise: boolean;
var V: double;
var IP: T2DVector;
var L1,L2: T2DLine;
var PL1,PL2: T2DLine;
begin
for i:=0 to Points.Count-1 do
	begin
	Points2.Add(Points[i]);
	Points2.ControlPoints1[I]:=Points.ControlPoints1[I];
	Points2.ControlPoints2[I]:=Points.ControlPoints2[I];
	end;
Clockwise:=IsClockWise(Points);
V:=-Delta;
if IgnoreWinding and not Clockwise then v:=Delta;

for  I:=0 to Points.Count-1 do
	begin
	{Find the point perpendicular to each edge at given inflate
	distance, V, for each end of the edge}
	if I<1 then H:=Points.Count-1 else H:=I-1;
	if I>=Points.Count-1 then J:=0 else J:=I+1;
	L1.P1:=Points[H]; L1.P2:=Points[I];
	L2.P1:=Points[I]; L2.P2:=Points[J];
	PL1.P1:=AngledLineFromLine(L1,L1.p1,V,-Pi/2).P2;
	PL1.P2:=AngledLineFromLine(L1,L1.p2,V,-Pi/2).P2;
	PL2.P1:=AngledLineFromLine(L2,L2.p1,V,-Pi/2).P2;
	PL2.P2:=AngledLineFromLine(L2,L2.p2,V,-Pi/2).P2;
	if IntersectLines2D(PL1,PL2,IP) then Points2[I]:=IP
	else Points2[I]:=HandleColinearLines(PL1,PL2);
        end;

if TrimLoops then DoTrimLoops(Points2);
end;



function GetPolygonCenter(Points: TRealPolygon): T2DVector;
var I: integer;
var T: T2DVector;
begin
T:=MakeVector2D(0,0);
for I:=0 to Points.Count-1 do
	begin
	T:=VectorAdd2D(Points[I],T);
	end;
Result:=ScalarDivide2D(T,Points.Count);
end;



procedure GetPolygonSize(Points: TRealPolygon; var Min,Max: T2DVector);
var I: integer;
var T: T2DVector;
begin
Min:=MakeVector2D(1E9,1E9);
Max:=MakeVector2D(-1E9,-1E9);
for I:=0 to Points.Count-1 do     
	begin
	T:=Points[I];
	Min:=GetMinimumVector2D(Min,T);
	Max:=GetMaximumVector2D(Max,T);
	end;
end;



procedure GetPolygonArraySize(A: TRealPolygonArray; var Min,Max: T2DVector);
var I: integer;
var Min2,Max2: T2DVector;
begin
Min:=MakeVector2D(1E9,1E9);
Max:=MakeVector2D(-1E9,-1E9);
for I:=0 to A.Count-1 do
	begin
	GetPolygonSize(A[I],Min2,Max2);
	Min:=GetMinimumVector2D(Min,Min2);
	Max:=GetMaximumVector2D(Max,Max2);
	end;
end;



procedure ScalePolygon(Points: TRealPolygon;  Scale: double);
{Scale polygon around its Center}
var I: integer;
var C,T: T2DVector;
begin
C:=GetPolygonCenter(Points);
for I:=0 to Points.Count-1 do
	begin
	T:=VectorSubtract2D(Points[I],C);
        T:=ScalarProduct2D(T,Scale);
	T:=VectorAdd2D(T,C);
	Points[I]:=T;
	end;
end;




procedure ScalePolygonArray(A: TRealPolygonArray; Scale: double);
{Scale all polygon around their Centers}
var I: integer;
begin
for I:=0 to A.Count-1 do
	begin
	ScalePolygon(A[I],Scale);
	end;
end;



procedure OffsetPolygon(Points: TRealPolygon; Offset: T2DVector);
{Offset polygon }
var I: integer;
begin
for I:=0 to Points.Count-1 do
	begin
	Points[I]:=VectorAdd2D(Points[I],Offset);
	end;
end;




procedure OffsetPolygonArray(A: TRealPolygonArray; Offset: T2DVector);
{Offset all polygons a}
var I: integer;
begin
for I:=0 to A.Count-1 do
	begin
	OffsetPolygon(A[I],Offset);
	end;
end;



procedure RotatePolygon(Points: TRealPolygon;  Angle: double);
{Rotate polygon around its Center to angle specified in degrees}
var I: integer;
var C,T,T1,T2: T2DVector;
begin
C:=GetPolygonCenter(Points);
for I:=0 to Points.Count-1 do
	begin
	{Set control points relative to points and rotate}
	T1:=VectorAdd2D(Points[I],Points.ControlPoints1[I]);
        RotatePoint2D(T1,C,Angle);
	T2:=VectorAdd2D(Points[I],Points.ControlPoints2[I]);
        RotatePoint2D(T2,C,Angle);

	{Rotate points}
	T:=Points[I];
        RotatePoint2D(T,C,Angle);
	Points[I]:=T;
	{Make controls points relative again}
	Points.ControlPoints1[I]:=VectorSubtract2D(T1,Points[I]);
	Points.ControlPoints2[I]:=VectorSubtract2D(T2,Points[I]);
	end;
end;



function GetPolyPolyRegion(Polys: TRealPolygonArray): HRgn;
var I, J, K: integer;
var ALength: integer;
var PointList: array of TPoint;
var CountList: array of integer;
var Poly: TRealPolygon;
var PT: TPoint;
begin
SetLength(PointList, 0);
SetLength(CountList, 0);
for I:=0 to Polys.Count-1 do
	begin
	Poly:=Polys[I];
	SetLength(CountList,Length(CountList)+1);
	CountList[High(CountList)]:=Poly.Count;
	for J:=0 to Poly.Count-1 do
		begin
		PT:=Poly.Points[J];
		SetLength(PointList,Length(PointList)+1);
		PointList[High(PointList)]:=PT;
		end;
	end;
Result := CreatePolyPolygonRgn(PointList[0], CountList[0], Length(CountList), ALTERNATE);
end;





function GetSplinePolyPolyRegion(Polys: TRealPolygonArray; Steps: integer): HRgn;
var I, J, K: integer;
var ALength: integer;
var PointList: array of TPoint;
var CountList: array of integer;
var Poly: TRealPolygon;
var PT: TPoint;
var PG: T2DVectorArray;
begin
SetLength(PG,0);
SetLength(PointList, 0);
SetLength(CountList, 0);
for I:=0 to Polys.Count-1 do
	begin
	Poly:=Polys[I];
	GetSplinePolygon(Poly,Steps, true, PG);
	SetLength(CountList,Length(CountList)+1);
	CountList[High(CountList)]:=Length(PG);
	for J:=0 to High(PG) do
		begin
		PT:=Vector2DToPoint(PG[J]);
		SetLength(PointList,Length(PointList)+1);
		PointList[High(PointList)]:=PT;
		end;
	end;
Result := CreatePolyPolygonRgn(PointList[0], CountList[0], Length(CountList), ALTERNATE);
end;

end.
