unit GLTextOutline;

interface

uses Windows,SysUtils,Graphics,GLPolygons,HugeArry,GraphicsLib;



type TFontInfo = record
 Name: string;
 Style: TFontStyles;
 Size: integer;
 Color: TColor;
 end;


procedure GetStringOutline(FontInfo: TFontInfo; S: string; var Path: TPolygonArray); overload;
procedure GetStringOutline(Text: string; Canvas: TCanvas; var Outline: TPolygonArray); overload;
procedure GetStringOutline(Text: string; Name: TFontName; Style: TFontStyles; Size: integer; var Outline: TPolygonArray); overload;

implementation


function EqualPoints(P1,P2: TPoint): boolean;
begin
Result:=(P1.X=P2.X) and (P1.Y=P2.Y);
end;


procedure GetLineArray(DC: HDC; var Path: TPolygonArray);
var Cnt: integer;
var Vertices: array of TPoint;
var Types: array of byte;
var I,J: integer;
var PA: THugePoint;
var DummyPtr: pointer;
begin
PA:=THugePoint.Create(5);
try
DummyPtr := nil;
Cnt:=GetPath(DC, DummyPtr, DummyPtr, 0);
if Cnt<1 then exit;
SetLength(Vertices, Cnt);
SetLength(Types, Cnt);
GetPath(DC, Vertices[0], Types[0], Cnt);

for I:=4 to Cnt-1 do
	begin
	if Types[I] = (PT_CLOSEFIGURE or PT_LINETO)  then
		begin
		SetLength(Path,Length(Path)+1);
		SetLength(Path[High(Path)],PA.Count);
		for J:=0 to PA.Count-1 do Path[High(Path),J]:=PA[J];
		PA.Clear;
		end
	else if (PA.Count<1) or not EqualPoints(PA[0],Vertices[I]) then PA.Add(Vertices[I]);
	end;
finally PA.Free; end;
end;


procedure GetStringOutline(FontInfo: TFontInfo; S: string; var Path: TPolygonArray);
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.Canvas.Font.Size:=FontInfo.Size;
BM.Canvas.Font.Name:=FontInfo.Name;
BM.Canvas.Font.Style:=FontInfo.Style;
BeginPath(BM.Canvas.Handle);
if not IsTrueTypeFont(BM.Canvas) then raise Exception.Create('TrueType font required');
TextOut(BM.Canvas.Handle, 0, 0, @S[1], Length(S));
EndPath(BM.Canvas.Handle);
FlattenPath(BM.Canvas.Handle);
GetLineArray(BM.Canvas.Handle,Path);
finally BM.Free; end;
end;



procedure GetStringOutline(Text: string; Canvas: TCanvas; var Outline: TPolygonArray);
var FontInfo: TFontInfo;
begin
FontInfo.Name:=Canvas.Font.Name;
FontInfo.Style:=Canvas.Font.Style;
FontInfo.Size:=Canvas.Font.Size;
GetStringOutline(FontInfo,Text,Outline);
end;


procedure GetStringOutline(Text: string; Name: TFontName; Style: TFontStyles; Size: integer; var Outline: TPolygonArray);
var FontInfo: TFontInfo;
begin
FontInfo.Name:=Name;
FontInfo.Style:=Style;
FontInfo.Size:=Size;
GetStringOutline(FontInfo,Text,Outline);
end;


end.
