unit GLXFileBinToText;

interface

uses Windows,Classes,SysUtils,Vector,HugeArry,DSupport,GLViewer,GLXFileTextReader,GLXFileGlobal;

procedure BinToTextXStream(Stream: TStream; var XText: string; ProgInfo: TProgressInfo);

implementation

uses StreamLib;

var IndentLevel: integer;



var ObjectTag,ObjectName: string;


{------------------ Progress handling routines -------------------------------}


var StreamProgress: TStreamProgress;



{----------------- Low level support routines ---------------------------------}


procedure OutputXFileStr(S: string; StringList: TStringList);
{Copy S into XText addin appropriate indentation}
var I: integer;
begin
for I:=1 to IndentLevel do S:=' '+S;
StringList.Add(S);
end;



function GetGUIDStr(G: TGUID): string;
{Read GUID and convert ot string}
var I: integer;
begin
Result:=IntToHex(G.D1,8)+'-';
Result:=Result+IntToHex(G.D2,4)+'-';
Result:=Result+IntToHex(G.D3,4)+'-';
for I:=0 to 3 do Result:=Result+IntToHex(G.D4[I],2);
Result:=Result+'-';
for I:=4 to 7 do Result:=Result+IntToHex(G.D4[I],2);
end;



function GetTokenName(Stream: TStream): string;
{Get current token name}
var Length: integer;
begin
Length:=GetStreamDWord(Stream);
Result:=GetStreamString(Stream,Length);
end;




function RealListToStr(List: THugeDouble; First,Last: integer; Sep: char): string;
{Output specified integer list using specified separator}
var I: integer;
begin
Result:='';
for I:=First to Last do Result:=Result+FloatToStrF(List[I],ffFixed,18,6)+Sep;
end;




function GetGUIDString(Stream: TStream): string;
{Read GUID and convert to properly formatted string}
var I: integer;
var GUID: TGUID;
begin
GUID.D1:=GetStreamDWord(Stream);
GUID.D2:=GetStreamWord(Stream);
GUID.D3:=GetStreamWord(Stream);
for I:=0 to 7 do GUID.D4[I]:=GetStreamByte(Stream);
Result:=GetGUIDStr(GUID);
end;




{------------------- Low Level Token handling routines ------------------------}




function GetSpecifiedToken(Stream: TStream; Target: word): boolean;
{Read until we find specified token}
var Token: word;
begin
Result:=True;
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	if Token=Target then exit;
	end;
Result:=False;
end;



function GetNextOpenBrace(Stream: TStream): boolean;
{Read until we find next open brace token}
begin
Result:=GetSpecifiedToken(Stream,TOKEN_OBRACE);
end;




procedure ReadOpenBrace(Stream: TStream);
{Read open brace and raise exception if not found}
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_OBRACE then raise Exception.Create('Missing Open Brace');
Inc(IndentLevel);
end;




procedure ReadCloseBrace(Stream: TStream; StringList: TStringList);
{Read close brace and raise exception if not found}
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_CBRACE then raise Exception.Create('Syntax Error: } missing');
Dec(IndentLevel);
OutputXFileStr('}',StringList);
end;


{------------------------------------------------------------------------------}


function ReadOneListInteger(Stream: TStream): integer;
{Read an integer from a one item list}
var Length: integer;
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_INTEGER_LIST then raise Exception.Create('Syntax Error: Integer List missing');
Length:=GetStreamDWord(Stream);
if Length>1 then raise Exception.Create('Syntax Error: Expected Single Integer');
Result:=GetStreamDWord(Stream);
end;



procedure ReadIntegerList(Stream: TStream; List: THugeInteger);
{Read integer list and store it in a huge array}
var Length: integer;
var I: integer;
begin
Length:=GetStreamDWord(Stream);
List.Clear;
for I:=0 to Length-1 do List.Add(GetStreamDWord(Stream));
end;




procedure GetIntegerList(Stream: TStream; List: THugeInteger);
{Make sure next toke is a}
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_INTEGER_LIST then raise Exception.Create('Syntax Error: Integer List missing');
ReadIntegerList(Stream,List);
end;



function GetReal(Stream: TStream): double;
{Return one real, using the proper type based on the file format}
begin
if FileInfo.NumberType=ntSingle then Result:=GetStreamSingle(Stream)
else Result:=GetStreamDouble(Stream);
end;




procedure ReadRealList(Stream: TStream; List: THugeDouble);
{Read a list of real reals}
var Length: integer;
var I: integer;
begin
List.Clear;
Length:=GetStreamDWord(Stream);
for I:=0 to Length-1 do List.Add(GetReal(Stream));
end;



procedure GetRealList(Stream: TStream; List: THugeDouble);
{Read a list of reals, making sure the proper token is there}
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_REALNUM_LIST then raise Exception.Create('Syntax Error: Real List missing');
ReadRealList(Stream,List);
end;



procedure ReadVectorList(Stream: TStream; List: THuge3DVector);
{Read a list of reals as 3D Vectors}
var Length: integer;
var I: integer;
var V: T3DVector;
var Token: Word;
begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_REALNUM_LIST then raise Exception.Create('Syntax Error: Integer List missing');
List.Clear;
Length:=GetStreamDWord(Stream);
for I:=0 to (Length div 3)-1 do
	begin
	V.X:=GetReal(Stream);
	V.Y:=GetReal(Stream);
	V.Z:=GetReal(Stream);
	List.Add(V);
	StreamProgress.CheckStreamPosition(Stream);
	end;
end;




procedure SkipObject(Stream: TStream; StringList: TStringList);
{Skip of XFile object keeping track of nesting to all subobjects are skipped}
var Token: word;
var Nesting: integer;
var Name: string;
var IL: THugeInteger;
var RL: THugeDouble;
begin
if not GetNextOpenBrace(Stream) then exit;

IL:=THugeInteger.Create(5);
RL:=THugeDouble.Create(5);
try
OutputXFileStr('//Skipping: '+ObjectTag+' '+ObjectName,StringList);
Nesting:=1;
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_OBRACE: Inc(Nesting);
	 TOKEN_CBRACE: Dec(Nesting);
	 TOKEN_NAME,TOKEN_STRING: Name:=GetTokenName(Stream);
	 TOKEN_INTEGER_LIST: ReadIntegerList(Stream,IL);
	 TOKEN_REALNUM_LIST: ReadRealList(Stream,RL);
	 end;
	if Nesting<1 then exit;
	end;
finally
 IL.Free;
 RL.Free;
 end;
end;


function GetObjectInfo(Stream: TStream; StringList: TStringList): TXObjectToken;
{Get Object Tag name and if available, an Object Name}
var Token: word;
begin
ObjectName:='';
ObjectTag:=GetTokenName(Stream);
Token:=PeekStreamWord(Stream);
if Token=TOKEN_NAME then
	begin
	{Handle peek look ahead }
	Token:=GetStreamWord(Stream);
	ObjectName:=GetTokenName(Stream);
	end;
Result:=GetObjectID(ObjectTag);
end;




procedure ReadVertexList(Stream: TStream; VertexList: TVertexList);
{Read generic vertex list - i.e. face vertices or normal vertices}
var Vertices: integer;
begin
VertexList.Clear;
Vertices:=ReadOneListInteger(Stream);
ReadVectorList(Stream,VertexList.Vertices);

GetIntegerList(Stream,VertexList.Indices);
VertexList.FaceCount:=VertexList.Indices[0];
VertexList.Indices.Delete(0);
end;




procedure DisplayVertexList(Stream: TStream; VertList: TVertexList; StringList: TStringList);
{Display whole vertex list}
var I,J,Inx,PntCnt: integer;
var S: string;
begin
OutputXFileStr(IntToStr(VertList.Vertices.Count)+';',StringList);
for I:=0 to VertList.Vertices.Count-1 do
	begin
	S:=Vector3DToString(VertList.Vertices[I],6,';')+';';
	if I=(VertList.Vertices.Count-1) then S:=S+';' else S:=S+',';
	OutputXFileStr(S,StringList);
	StreamProgress.CheckStreamPosition(Stream);
	end;
OutputXFileStr(IntToStr(VertList.FaceCount)+';',StringList);


Inx:=0;
for I:=0 to VertList.FaceCount-1 do
	begin
	PntCnt:=VertList.Indices[Inx];
	Inc(Inx);
	S:=IntToStr(PntCnt)+';';
	for J:=0 to PntCnt-1 do
		begin
		S:=S+IntToStr(VertList.Indices[Inx]);
		Inc(Inx);
		if J=(PntCnt-1) then S:=S+';' else S:=S+',';
		end;
	if I=(VertList.FaceCount-1) then S:=S+';' else S:=S+',';
	OutputXFileStr(S,StringList);
	StreamProgress.CheckStreamPosition(Stream);
	end;
end;



{--------------- Read Header Object -------------------------------------------}



procedure ReadHeader(Stream: TStream; StringList: TStringList);
{Read and display header object}
var List: THugeInteger;
var I: integer;
begin
List:=THugeInteger.Create(5);
try
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
ReadOpenBrace(Stream);
GetIntegerList(Stream,List);
for I:=0 to List.Count-1 do OutputXFileStr(IntToStr(List[I])+';',StringList);
ReadCloseBrace(Stream,StringList);
finally List.Free; end;
end;


{---------------- Handle Material List Objects --------------------------------}



procedure ReadTextureFilename(Stream: TStream; StringList: TStringList);
{Read and display Texture Filename}
var Token: word;
var Filename: string;
begin
ReadOpenBrace(Stream);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
Token:=GetStreamWord(Stream);
if Token<>TOKEN_STRING then raise Exception.Create('Syntax Error: String missing');
Filename:=GetTokenName(Stream);
OutputXFileStr('"'+Filename+'"',StringList);
Token:=GetStreamWord(Stream);
if Token<>TOKEN_SEMICOLON then raise Exception.Create('Syntax Error: Semicolon missing');
ReadCloseBrace(Stream,StringList);
end;




procedure ReadMaterialSubobject(Stream: TStream; StringList: TStringList);
{Parse Material sub-objects}
var ObjectToken: TXObjectToken;
begin
ObjectToken:=GetObjectInfo(Stream,StringList);
case ObjectToken of
 xfTextureFilename: ReadTextureFilename(Stream,StringList);
 else SkipObject(Stream,StringList);
 end;
end;



procedure ReadMaterial(Stream: TStream; StringList: TStringList);
{Read and display Material object}
var Token: word;
var List: THugeDouble;
begin
List:=THugeDouble.Create(5);
try
ReadOpenBrace(Stream);
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
GetRealList(Stream,List);
if List.Count<>11 then raise Exception.Create('Error: Insufficient Reals in material');
OutputXFileStr(RealListToStr(List,0,3,';')+';',StringList);
OutputXFileStr(RealListToStr(List,4,4,';'),StringList);
OutputXFileStr(RealListToStr(List,5,7,';')+';',StringList);
OutputXFileStr(RealListToStr(List,8,10,';')+';',StringList);

while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_NAME: ReadMaterialSubobject(Stream,StringList);
	 TOKEN_CBRACE: break;
	 else raise Exception.Create('Syntax Error: Name or "}" expected');
	 end;
	end;

Dec(IndentLevel);
OutputXFileStr('}',StringList);
finally List.Free; end;
end;




procedure ReadMaterialsSubobject(Stream: TStream; StringList: TStringList);
{Parse Materials sub-objects}
var ObjectToken: TXObjectToken;
begin
ObjectToken:=GetObjectInfo(Stream,StringList);
case ObjectToken of
 xfMaterial: ReadMaterial(Stream,StringList);
 else SkipObject(Stream,StringList);
 end;
end;




procedure ReadSymbolName(Stream: TStream; StringList: TStringList);
{Read and display Texture Filename}
var Token: word;
var SymbolName: string;
begin
Inc(IndentLevel);
OutputXFileStr('{',StringList);
Token:=GetStreamWord(Stream);
if (Token<>TOKEN_STRING) and (Token<>TOKEN_NAME) then raise Exception.Create('Syntax Error: String missing');
SymbolName:=GetTokenName(Stream);
OutputXFileStr(SymbolName,StringList);
ReadCloseBrace(Stream,StringList);
end;



procedure ReadMaterialList(Stream: TStream; StringList: TStringList);
{Read and display Material List objects}
var Token: word;
var List: THugeInteger;
var I: integer;
begin
List:=THugeInteger.Create(5);
try
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
ReadOpenBrace(Stream);
GetIntegerList(Stream,List);
for I:=0 to List.Count-1 do
	begin
	OutputXFileStr(IntToStr(List[I])+';',StringList);
	StreamProgress.CheckStreamPosition(Stream);
	end;

while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_NAME: ReadMaterialsSubobject(Stream,StringList);
	 TOKEN_OBRACE: ReadSymbolName(Stream,StringList);
	 TOKEN_CBRACE: break;
	 else raise Exception.Create('Syntax Error: Name or "}" expected');
	 end;
	end;

Dec(IndentLevel);
OutputXFileStr('}',StringList);
finally List.Free; end;
end;


{------------------ Handle Meshes --------------------------------------------}



procedure ReadNormals(Stream: TStream; StringList: TStringList);
{Read and display Mesh Normals objects}
var VertList: TVertexList;
begin
VertList:=TVertexList.Create;
try
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
ReadOpenBrace(Stream);
ReadVertexList(Stream,VertList);
DisplayVertexList(Stream,VertList,StringList);
ReadCloseBrace(Stream,StringList);
finally VertList.Free; end;
end;



procedure ReadTextureCoords(Stream: TStream; StringList: TStringList);
{Read and display Texture Coordinates objects}
var List: THugeDouble;
var I,Inx,Count: integer;
var S: string;
begin
List:=THugeDouble.Create(5);
try
ReadOpenBrace(Stream);
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
Count:=ReadOneListInteger(Stream);
OutputXFileStr(IntToStr(Count)+';',StringList);
GetRealList(Stream,List);
for I:=0 to (List.Count div 2)-1 do
	begin
	Inx:=I*2;
	S:=RealListToStr(List,Inx,Inx+1,';');
	if I=((List.Count div 2)-1) then S:=S+';' else S:=S+',';
	OutputXFileStr(S,StringList);
	StreamProgress.CheckStreamPosition(Stream);
	end;
ReadCloseBrace(Stream,StringList);
finally List.Free; end;
end;



procedure ReadVertexColors(Stream: TStream; StringList: TStringList);
{Read and output Vertex Colors}
var List: THugeDouble;
var Token: DWord;
var I,Inx,T,Count: integer;
var S: string;
const ColorCnt = 4;
begin
List:=THugeDouble.Create(5);
try
ReadOpenBrace(Stream);
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);

Token:=GetStreamWord(Stream);
if Token<>TOKEN_INTEGER_LIST then raise Exception.Create('Syntax Error: Integer List missing');
T:=GetStreamDWord(Stream);
Count:=GetStreamDWord(Stream);
OutputXFileStr(IntToStr(Count)+';',StringList);
for I:=0 to Count-1 do
	begin
	if I<>0 then
		begin
		Token:=GetStreamWord(Stream);
		if Token<>TOKEN_INTEGER_LIST then raise Exception.Create('Syntax Error: Integer List missing');
		T:=GetStreamDWord(Stream);
		end;
	Inx:=GetStreamDWord(Stream);
	GetRealList(Stream,List);
	S:=IntToStr(Inx)+'; '+RealListToStr(List,0,ColorCnt-1,';');
	if I=(ColorCnt-1) then S:=S+';' else S:=S+',';
	OutputXFileStr(S,StringList);
	StreamProgress.CheckStreamPosition(Stream);
	end;
ReadCloseBrace(Stream,StringList);
finally List.Free; end;
end;




procedure ReadMeshSubObject(Stream: TStream; StringList: TStringList);
{Decode Mesh subobjects}
var ObjectToken: TXObjectToken;
begin
ObjectToken:=GetObjectInfo(Stream,StringList);
case ObjectToken of
 xfMeshMaterialList: ReadMaterialList(Stream,StringList);
 xfMeshNormals: ReadNormals(Stream,StringList);
 xfMeshTextureCoords: ReadTextureCoords(Stream,StringList);
 xfMeshVertexColors: ReadVertexColors(Stream,StringList);
 else SkipObject(Stream,StringList);
 end;
end;



procedure ParseSubMesh(Stream: TStream; StringList: TStringList);
{Parse sub-mesh objects}
var Token: word;
begin
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_NAME: ReadMeshSubObject(Stream,StringList);
	 TOKEN_CBRACE: exit;
	 else raise Exception.Create('Syntax Error: Name or "}" expected');
	 end;
	end;
end;





procedure ReadMesh(Stream: TStream; StringList: TStringList);
{Read and display Mesh objects}
var VertList: TVertexList;
begin
VertList:=TVertexList.Create;
try
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
ReadOpenBrace(Stream);
ReadVertexList(Stream,VertList);
DisplayVertexList(Stream,VertList,StringList);
ParseSubMesh(Stream,StringList);
Dec(IndentLevel);
OutputXFileStr('}',StringList);
finally VertList.Free; end;
end;



{----------------- Handle Templates ------------------------------------------}




procedure ReadTemplate(Stream: TStream; StringList: TStringList);
{Read and display Template objects}
var Token: word;
var Name,GUID: string;
var MemberStr: string;


	procedure AddMemberItem(S: string; Pad: boolean);
	{Add template member to string handling separators}
	begin
	if Pad and
	   (Length(MemberStr)>0) and
	   not (MemberStr[Length(MemberStr)] in ['<','(','[']) then MemberStr:=MemberStr+' ';
	MemberStr:=MemberStr+S;
	end;


	procedure DiplayMemberStr;
	{Display member string, clearing string when done}
	begin
	OutputXFileStr(MemberStr,StringList);
	MemberStr:='';
	end;


begin
Token:=GetStreamWord(Stream);
if Token<>TOKEN_NAME then raise Exception.Create('Syntax Error: Name Missing');
Name:=GetTokenName(Stream);
OutputXFileStr('template '+Name+' {',StringList);
ReadOpenBrace(Stream);
Token:=GetStreamWord(Stream);
if Token<>TOKEN_GUID then raise Exception.Create('Syntax Error: GUID Missing');
GUID:=GetGUIDString(Stream);
OutputXFileStr('<'+GUID+'>',StringList);

MemberStr:='';
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_NAME: AddMemberItem(GetTokenName(Stream),True);
	 TOKEN_WORD: AddMemberItem('WORD',True);
	 TOKEN_DWORD: AddMemberItem('DWORD',True);
	 TOKEN_FLOAT: AddMemberItem('FLOAT',True);
	 TOKEN_DOUBLE: AddMemberItem('DOUBLE',True);
	 TOKEN_CHAR: AddMemberItem('CHAR',True);
	 TOKEN_UCHAR: AddMemberItem('UCHAR',True);
	 TOKEN_SWORD: AddMemberItem('SWORD',True);
	 TOKEN_SDWORD: AddMemberItem('SDWORD',True);
	 TOKEN_OPAREN: AddMemberItem('(',False);
	 TOKEN_CPAREN: AddMemberItem(')',False);
	 TOKEN_OBRACKET: AddMemberItem('[',False);
	 TOKEN_CBRACKET:
		begin
		AddMemberItem(']',False);
		if MemberStr[1]='[' then DiplayMemberStr;
		end;
	 TOKEN_OANGLE: AddMemberItem('<',False);
	 TOKEN_CANGLE: AddMemberItem('>',False);
	 TOKEN_DOT: AddMemberItem('.',False);
	 TOKEN_COMMA: AddMemberItem(',',False);
	 TOKEN_SEMICOLON:
		begin
		AddMemberItem(';',False);
		DiplayMemberStr;
		end;
	 TOKEN_CBRACE: break;
	 end;
	end;
Dec(IndentLevel);
OutputXFileStr('}',StringList);
OutputXFileStr('',StringList);
end;


{---------------- Read Frame Data ---------------------------------------------}



procedure ReadTransform(Stream: TStream; StringList: TStringList);
{Read and display Frame Transform object}
var List: THugeDouble;
var Cnt: integer;
var S: string;
begin
List:=THugeDouble.Create(5);
try
ReadOpenBrace(Stream);
OutputXFileStr('',StringList);
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
Cnt:=0;
S:='';
repeat
	begin
	GetRealList(Stream,List);
	Cnt:=Cnt + List.Count;
	S:=S+RealListToStr(List,0,List.Count-1,',');
	end
until Cnt>=16;
S:=S+';';
OutputXFileStr(S,StringList);
ReadCloseBrace(Stream,StringList);
finally List.Free; end;
end;


procedure ReadFrame(Stream: TStream; StringList: TStringList); forward;
{Forward declaration for recursion}


procedure ReadFrameSubObject(Stream: TStream; StringList: TStringList);
{Decode frame sub-object}
var ObjectToken: TXObjectToken;
begin
ObjectToken:=GetObjectInfo(Stream,StringList);
case ObjectToken of
 xfMesh: ReadMesh(Stream,StringList);
 xfFrame: ReadFrame(Stream,StringList);
 xfFrameTransformMatrix: ReadTransform(Stream,StringList);
 else SkipObject(Stream,StringList);
 end;
end;



procedure ReadFrame(Stream: TStream; StringList: TStringList);
{Read and display frame object}
var Token: word;
begin
OutputXFileStr(ObjectTag+' '+ObjectName+' {',StringList);
ReadOpenBrace(Stream);
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	Token:=GetStreamWord(Stream);
	case Token of
	 TOKEN_NAME: ReadFrameSubObject(Stream,StringList);
	 TOKEN_CBRACE: break;
	 else raise Exception.Create('Syntax Error: Name or "}" expected');
	 end;
	end;
Dec(IndentLevel);
OutputXFileStr('}',StringList);
end;



{------------------------------------------------------------------------------}


procedure DecodeObject(Stream: TStream; StringList: TStringList);
{Decode top-level objects}
var Token: word;
var ObjectToken: TXObjectToken;
begin
Token:=GetStreamWord(Stream);
case Token of
 TOKEN_NAME:
	begin
	ObjectToken:=GetObjectInfo(Stream,StringList);
	case ObjectToken of
	 xfHeader: ReadHeader(Stream,StringList);
	 xfMesh: ReadMesh(Stream,StringList);
	 xfFrame: ReadFrame(Stream,StringList);
	 xfMaterial: ReadMaterial(Stream,StringList);
	 xfUnknown: SkipObject(Stream,StringList);
	 else SkipObject(Stream,StringList);
	 end;
	end;
 TOKEN_TEMPLATE: ReadTemplate(Stream,StringList);
 else raise Exception.Create('Syntax Error: Object name missing');
 end;
end;




procedure AddTextHeader(Stream: TStream; StringList: TStringList);
{Create and a text header}
begin
StringList.Add('xof 0302txt 0064');
end;




procedure BinToTextXStream(Stream: TStream; var XText: string; ProgInfo: TProgressInfo);
var StringList: TStringList;
begin
StringList:=TStringList.Create;
try
StreamProgress.ProgInfo:=ProgInfo;
IndentLevel:=0;
StringList.Clear;
Stream.Seek(0,soFromBeginning);
if not ReadXFileHeader(Stream,FileInfo) then raise Exception.Create('Invalid File');
if FileInfo.FileType<>ffBinary then raise Exception.Create('Only Binary Files Support');
AddTextHeader(Stream,StringList);
while StreamProgress.CheckStreamPosition(Stream) do
	begin
	DecodeObject(Stream,StringList);
	end;
XText:=StringList.Text;
finally StringList.Free; end;
end;



initialization
StreamProgress:=TStreamProgress.Create;
finalization
StreamProgress.Free;
end.
