unit GLXFileGlobal;

interface

uses Windows,Classes,Graphics,StreamLib,HugeArry,GLMesh,SysUtils;

{Binary Record Tokens}
{ Record-bearing } 

const TOKEN_NAME=1;
const TOKEN_STRING=2;
const TOKEN_INTEGER=3;
const TOKEN_GUID=5;
const TOKEN_INTEGER_LIST=6;
const TOKEN_REALNUM_LIST=7;

{ Stand-alone} 

const TOKEN_OBRACE=10;
const TOKEN_CBRACE=11;
const TOKEN_OPAREN=12;
const TOKEN_CPAREN=13;
const TOKEN_OBRACKET=14;
const TOKEN_CBRACKET=15;
const TOKEN_OANGLE=16;
const TOKEN_CANGLE=17;
const TOKEN_DOT=18;
const TOKEN_COMMA=19;
const TOKEN_SEMICOLON=20;
const TOKEN_TEMPLATE=31;
const TOKEN_WORD=40;
const TOKEN_DWORD=41;
const TOKEN_FLOAT=42;
const TOKEN_DOUBLE=43;
const TOKEN_CHAR=44;
const TOKEN_UCHAR=45;
const TOKEN_SWORD=46;
const TOKEN_SDWORD=47;

const TOKEN_VOID=48;
const TOKEN_LPSTR=49;
const TOKEN_UNICODE=50;
const TOKEN_CSTRING=51;
const TOKEN_ARRAY=52;


{Tokens for various names}

type TXObjectToken = (xfWhiteSpace,xfComment,xfOpenBrace,xfCloseBrace,xfError,xfUnknown,
		    xfTemplate,xfFrameTransFormMatrix,xfFrame,xfSkinMeshHeader,
		    xfMeshTextureCoords,xfMeshMaterialList,xfMeshNormals,xfMesh,xfMeshVertexColors,
		    xfMaterial,xfSkinWeights,xfTextureFilename,xfAnimationSet,
		    xfAnimationKey,xfAnimation,xfHeader);


const MaxBlockID = 16;

type TBlockID = record
 Name: string;
 Token: TXObjectToken;
 end;



const BlockIDList: array [0..MaxBlockID-1] of TBlockID = (
	(Name: 'template'; Token: xfTemplate),
	(Name: 'FrameTransformMatrix'; Token: xfFrameTransformMatrix),
	(Name: 'Frame'; Token: xfFrame),
	(Name: 'XSkinMeshHeader'; Token: xfSkinMeshHeader),
	(Name: 'MeshTextureCoords'; Token: xfMeshTextureCoords),
	(Name: 'MeshMaterialList'; Token: xfMeshMaterialList),
	(Name: 'MeshNormals'; Token: xfMeshNormals),
	(Name: 'Mesh'; Token: xfMesh),
	(Name: 'MeshVertexColors'; Token: xfMeshVertexColors),
	(Name: 'Material'; Token: xfMaterial),
	(Name: 'SkinWeights'; Token: xfSkinWeights),
	(Name: 'TextureFilename'; Token: xfTextureFilename),
	(Name: 'AnimationSet'; Token: xfAnimationSet),
	(Name: 'AnimationKey'; Token: xfAnimationKey),
	(Name: 'Animation'; Token: xfAnimation),
	(Name: 'Header'; Token: xfHeader)
	);


{File format information }

type TFileFormat = (ffText,ffBinary,ffCompress,ffBZip,ffTZip,ffUnknown);
type TNumberType = (ntSingle,ntDouble);

const FileFormatStrings: array [0..4] of string = ('Text','Binary','Compress','BZip','ffUnknown');

type TXFileInfo = record
 FileType: TFileFormat;
 NumberType: TNumberType;
 Version: string;
 end;


type TVertexList = class(TObject)
 public
  Vertices: THuge3DVector;
  Indices: THugeInteger;
  FaceCount: integer;
  Name: string;
  constructor Create;
  destructor destroy; override;
  procedure Clear;
 end;

{Progress handling object}

type TProgressProcedure= procedure(Percent: integer);


type TProgressInfo = record
  Proc: TProgressProcedure;
  Multiplier,Offset: integer;
 end;


type TStreamProgress = class(TObject)
 private
  FProgInfo: TProgressInfo;
  FCount: integer;
  procedure HandleProgressCallback(Percent: integer);
  procedure SetProgInfo(Value: TProgressInfo);
 public
  constructor Create;
  property ProgInfo: TProgressInfo read FProgInfo write SetProgInfo;
  function CheckStreamPosition(Stream: TStream): boolean;
 end;



var FileInfo: TXFileInfo;

function ReadXFileHeader(Stream: TStream; var Info: TXFileInfo): boolean;
function XFileFormatToStr(FT: TFileFormat): string;
function GetObjectID(ID: string): TXObjectToken;


implementation



{--------- Object ot handle progress operations -------------------------------}

procedure TStreamProgress.HandleProgressCallback(Percent: integer);
begin
if Assigned(ProgInfo.Proc) then ProgInfo.Proc(Percent);
end;



procedure TStreamProgress.SetProgInfo(Value: TProgressInfo);
begin
FProgInfo:=Value;
end;


function TStreamProgress.CheckStreamPosition(Stream: TStream): boolean;
begin
Result:=Stream.Position<Stream.Size;
Inc(FCount);
if (FCount mod 100)=0 then HandleProgressCallBack(MulDiv(ProgInfo.Multiplier,Stream.Position,Stream.Size)+ProgInfo.Offset);
end;



constructor TStreamProgress.Create;
begin
FProgInfo.Proc:=nil;
FProgInfo.Multiplier:=100;
FProgInfo.Offset:=0;
end;



{------------- Object to handle Vertex List -----------------------------------}

procedure TVertexList.Clear;
begin
Vertices.Clear;
Indices.Clear;
end;



constructor TVertexList.Create;
begin
inherited;
Vertices:=THuge3DVector.Create(5);
Indices:=THugeInteger.Create(5);
end;



destructor TVertexList.destroy;
begin
Vertices.Free;
Indices.Free;
inherited;
end;


{------------ General support routines ----------------------------------------}

function XFileFormatToStr(FT: TFileFormat): string;
begin
case FT of
 ffText: Result:='Text';
 ffBinary: Result:='Binary';
 ffCompress: Result:='Compressed';
 ffBZip: Result:='BZip';
 ffTZip: Result:='BTip';
 ffUnknown: Result:='Unknown';
 end;
end;



function ReadXFileHeader(Stream: TStream; var Info: TXFileInfo): boolean;
var S: string;
begin
Result:=False;
{Read magic number}
S:=GetStreamString(Stream,4);
if S<>'xof ' then exit;
Info.Version:=GetStreamString(Stream,4);

S:=GetStreamString(Stream,4);
if S='txt ' then Info.FileType:=ffText
else if S='bin ' then Info.FileType:=ffBinary
else if S='com ' then Info.FileType:=ffCompress
else if S='bzip' then Info.FileType:=ffBZip
else if S='tzip' then Info.FileType:=ffTZip
else Info.FileType:=ffUnknown;

S:=GetStreamString(Stream,4);
if S='0032' then Info.NumberType:=ntSingle
else if S='0064' then Info.NumberType:=ntDouble;
Result:=True;
end;




function GetObjectID(ID: string): TXObjectToken;
{Decode block ID string to Block token}
var I: integer;
begin
Result:=xfUnknown;
for I:=0 to High(BlockIDList) do
 if SameText(ID,BlockIDList[I].Name) then
	begin
	Result:=BlockIDList[I].Token;
	Break;
	end;
end;


end.
