unit GLXMLReader;

interface

uses Windows,Forms,Classes,Graphics,SysUtils,OpenGL15,XMLIntf,XMLDoc,
     DSupport,Vector,HugeArry,GLRenderer,GLFrame,GLQuatLib,GLCustomMesh,GLMesh,GLFastMesh,GLLibrary,
     GLAnimator,GLTexture,GLColorArray,GLLines,GLFonts,GLCubeMap,GLXMLGlobal,XMLDocLibrary;

function LoadGLXMLFile(Frame: TGLFrame; Filename: string; ProgressHandler: TXMLProgressProc): boolean;


implementation



function ReadGLArrayf4(Node: IXMLNode): TGLArrayf4;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
if GetChildNodeValue(Node,'Float4',V) then
	begin
	S:=V;
	Inx:=1;
	Result[0]:=StrToFloat(ExtractToken(S,',',Inx));
	Result[1]:=StrToFloat(ExtractToken(S,',',Inx));
	Result[2]:=StrToFloat(ExtractToken(S,',',Inx));
	Result[3]:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;



function ReadQuaternion(Node: IXMLNode): TQuaternion;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
if GetChildNodeValue(Node,'Quaternion',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Y:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Z:=StrToFloat(ExtractToken(S,',',Inx));
	Result.W:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;


{------------------------------------------------------------------------------}




procedure Load3DVectorArray(Node: IXMLNode; VectorArray: THuge3DVector);
var I,Inx,Count: integer;
var ChildNode: IXMLNode;
var S: string;
var V: T3DVector;
begin
ChildNode:= Node.ChildNodes.First;
if ChildNode.LocalName<>'Vector3DArray' then raise Exception.Create('Expected: Vector3DArray. Found: '+ChildNode.LocalName);
Count:=ChildNode.Attributes['Count'];
VectorArray.Clear;
if Count<1 then exit;
S:=ChildNode.NodeValue;
Inx:=1;
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	V.X:=StrToFloat(Trim(ExtractToken(S,',',Inx)));
	V.Y:=StrToFloat(Trim(ExtractToken(S,',',Inx)));
	V.Z:=StrToFloat(Trim(ExtractToken(S,',',Inx)));
	VectorArray.Add(V);
	end;
end;



procedure LoadColorArray(Node: IXMLNode; ColorArray: TGLColorArray);
var I,Inx,Count: integer;
var ChildNode: IXMLNode;
var S: string;
var C: TGLFaceColor;
begin
ChildNode:= Node.ChildNodes.First;
if ChildNode.LocalName<>'ColorArray' then raise Exception.Create('Expected: ColorArray. Found: '+ChildNode.LocalName);
Count:=ChildNode.Attributes['Count'];
ColorArray.Clear;
if Count<1 then exit;
S:=ChildNode.NodeValue;
Inx:=1;
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	C.C1:=StrToInt(Trim(ExtractToken(S,',',Inx)));
	C.C2:=StrToInt(Trim(ExtractToken(S,',',Inx)));
	C.C3:=StrToInt(Trim(ExtractToken(S,',',Inx)));
	ColorArray.Add(C);
	end;
end;




procedure LoadByteArray(Node: IXMLNode; ByteArray: THugeByte);
var I,Inx,Count: integer;
var ChildNode: IXMLNode;
var S: string;
begin
ChildNode:= Node.ChildNodes.First;
if ChildNode.LocalName<>'ByteArray' then raise Exception.Create('Expected: ByteArray. Found: '+ChildNode.LocalName);
Count:=ChildNode.Attributes['Count'];
ByteArray.Clear;
if Count<1 then exit;
S:=ChildNode.NodeValue;
Inx:=1;
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	ByteArray.Add(StrToInt(Trim(ExtractToken(S,',',Inx))));
	end;
end;




procedure LoadIntegerArray(Node: IXMLNode; IntegerArray: THugeInteger);
var I,Inx,Count: integer;
var ChildNode: IXMLNode;
var S: string;
begin
ChildNode:= Node.ChildNodes.First;
if ChildNode.LocalName<>'IntegerArray' then raise Exception.Create('Expected: IntegerArray. Found: '+ChildNode.LocalName);
Count:=ChildNode.Attributes['Count'];
IntegerArray.Clear;
if Count<1 then exit;
S:=ChildNode.NodeValue;
Inx:=1;
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	IntegerArray.Add(StrToInt(Trim(ExtractToken(S,',',Inx))));
	end;
end;

{------------------------------------------------------------------------------}



function ReadPosition(XMLNode: IXMLNode): TGLPosition;
var ChildNode: IXMLNode;
begin
ChildNode:=XMLNode.ChildNodes.First;
if ChildNode.LocalName<>'Position' then raise Exception.Create('Expected: Position. Found: '+ChildNode.LocalName);
Result.Pos:=ReadXMLVector3D(ChildNode);
ChildNode:=ChildNode.NextSibling;
if ChildNode.LocalName<>'Rotation' then raise Exception.Create('Expected: Rotation. Found: '+ChildNode.LocalName);
Result.Rot.V:=ReadXMLVector3D(ChildNode);
end;


procedure ReadAnimationArray(XMLNode: IXMLNode; AnimationArray: TAnimationArray);
var I,Count: integer;
var ChildNode: IXMLNode;
begin
Count:=XMLNode.Attributes['Count'];
AnimationArray.Clear;
if Count<1 then exit;
ChildNode:= XMLNode.ChildNodes.First;
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	if ChildNode.LocalName<>'AnimationKey' then
		raise Exception.Create('Expected: AnimationKey. Found: '+ChildNode.LocalName);
	AnimationArray.Add(ReadPosition(ChildNode));
	ChildNode:=ChildNode.NextSibling;
	end;
end;


{------------------------------------------------------------------------------}




procedure LoadGLAnimator(XMLNode: IXMLNode; Animator: TGLAnimator);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='Array' then ReadAnimationArray(ChildNode,Animator.AnimationArray);
	ChildNode:=ChildNode.NextSibling;
	end;
end;



procedure LoadGLObject(XMLNode: IXMLNode; GLObject: TGLObject);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='Name' then GLObject.Name:=ReadXMLString(ChildNode)
	else if ChildNode.LocalName='Animator' then LoadGLAnimator(ChildNode,GLObject.Animator)
	else if ChildNode.LocalName='ExtentMin' then GLObject.ExtentMin:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='ExtentMax' then GLObject.ExtentMax:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='Quaternion' then GLObject.Quaternion:=ReadQuaternion(ChildNode)
	else if ChildNode.LocalName='Position' then GLObject.Position:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='ResetPos' then GLObject.ResetPos:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='RotationCenter' then GLObject.RotationCenter:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='FlyThrough' then GLObject.FlyThrough:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='Scale' then GLObject.Scale:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='Scale3D' then GLObject.Scale3D:=ReadXMLVector3D(ChildNode);
	ChildNode:=ChildNode.NextSibling;
	end;
end;




{------------------------------------------------------------------------------}


procedure LoadImageArray(Node: IXMLNode; BM: TGLBaseBitmap);
var ChildNode: IXMLNode;
begin
ChildNode:= Node.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='Image' then LoadByteArray(ChildNode,BM.ByteArray)
	else if ChildNode.LocalName='Width' then BM.Width:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Height' then BM.Height:=ReadXMLInteger(ChildNode);
	ChildNode:=ChildNode.NextSibling;
	end;
end;




procedure LoadGLCubeMap(XMLNode: IXMLNode; CubeMap: TGLCubeMap);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='LeftMap' then LoadImageArray(ChildNode,CubeMap.LeftMap)
	else if ChildNode.LocalName='RightMap' then LoadImageArray(ChildNode,CubeMap.RightMap)
	else if ChildNode.LocalName='TopMap' then LoadImageArray(ChildNode,CubeMap.TopMap)
	else if ChildNode.LocalName='BottomMap' then LoadImageArray(ChildNode,CubeMap.BottomMap)
	else if ChildNode.LocalName='FrontMap' then LoadImageArray(ChildNode,CubeMap.FrontMap)
	else if ChildNode.LocalName='BackMap' then LoadImageArray(ChildNode,CubeMap.BackMap)
	else if ChildNode.LocalName='Scale' then CubeMap.Scale:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='Offset' then CubeMap.Offset:=ReadXMLVector3D(ChildNode)
	else if ChildNode.LocalName='RPlane' then CubeMap.RPlane:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='SPlane' then CubeMap.SPlane:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='TPlane' then CubeMap.TPlane:=ReadGLArrayf4(ChildNode);
	ChildNode:=ChildNode.NextSibling;
	end;
end;





procedure LoadGLTexture(XMLNode: IXMLNode; Texture: TGLObjectTexture);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='Image' then LoadByteArray(ChildNode,Texture.ByteArray)
	else if ChildNode.LocalName='Width' then Texture.Width:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Height' then Texture.Height:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Enabled' then Texture.Enabled:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='Highlight' then Texture.Highlight:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='TextureMode' then Texture.TextureMode:=TTextureMode(ReadXMLInteger(ChildNode))
	else if ChildNode.LocalName='AutoTextureMap' then Texture.AutoTextureMap:=TAutoTextureMap(ReadXMLInteger(ChildNode))
	else if ChildNode.LocalName='ManualTextureMap' then Texture.ManualTextureMap:=TManualTextureMap(ReadXMLInteger(ChildNode))
	else if ChildNode.LocalName='Scale' then Texture.Scale:=ReadXMLVector2D(ChildNode)
	else if ChildNode.LocalName='Offset' then Texture.Offset:=ReadXMLVector2D(ChildNode)
	else if ChildNode.LocalName='Yaw' then Texture.Yaw:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='Pitch' then Texture.Pitch:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='SPlane' then Texture.SPlane:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='TPlane' then Texture.TPlane:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='CubeMap' then LoadGLCubeMap(ChildNode,Texture.CubeMap);
	ChildNode:=ChildNode.NextSibling;
	end;
end;

{------------------------------------------------------------------------------}


procedure LoadGLRenderer(XMLNode: IXMLNode; Renderer: TGLRenderer);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='GLObject' then LoadGLObject(ChildNode,Renderer)
	else if ChildNode.LocalName='Animator' then LoadGLAnimator(ChildNode,Renderer.Animator)
	else if ChildNode.LocalName='DoAlphaBlend' then Renderer.DoAlphaBlend:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='Transparency' then Renderer.Transparency:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='BlendBackground' then Renderer.BlendBackground:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='TransparentDepth' then Renderer.TransparentDepth:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='LightEmission' then Renderer.LightEmission:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='LightSpecular' then Renderer.LightSpecular:=ReadGLArrayf4(ChildNode)
	else if ChildNode.LocalName='LightShininess' then Renderer.LightShininess:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='Texture' then LoadGLTexture(ChildNode,Renderer.Texture)
	else if ChildNode.LocalName='Color' then Renderer.Color:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='RenderMode' then Renderer.RenderMode:=TRenderMode(ReadXMLInteger(ChildNode))

	else if ChildNode.LocalName='DoTwoSidedLighting' then Renderer.DoTwoSidedLighting:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='DoFrontBackColor' then Renderer.DoFrontBackColor:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='FrontColor' then Renderer.FrontColor:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='BackColor' then Renderer.BackColor:=ReadXMLInteger(ChildNode);

	ChildNode:=ChildNode.NextSibling;
	end;
end;



{------------------------------------------------------------------------------}


procedure LoadGLMesh(XMLNode: IXMLNode; Mesh: TGLMesh);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='GLRenderer' then LoadGLRenderer(ChildNode,Mesh)
	else if ChildNode.LocalName='Vertices' then Load3DVectorArray(ChildNode,Mesh.Vertices)
	else if ChildNode.LocalName='FaceNormals' then Load3DVectorArray(ChildNode,Mesh.FaceNormals)
	else if ChildNode.LocalName='VertexNormals' then Load3DVectorArray(ChildNode,Mesh.VertexNormals)
	else if ChildNode.LocalName='FaceColor' then LoadColorArray(ChildNode,Mesh.FaceColor);
	ChildNode:=ChildNode.NextSibling;
	end;
Mesh.CalculateMinMax;
end;


{------------------------------------------------------------------------------}


procedure LoadGLMeshList(XMLNode: IXMLNode; List: TGLMeshList);
var Count,I: integer;
var ChildNode: IXMLNode;
var Mesh: TGLMesh;
begin
Count:=XMLNode.Attributes['Count'];
if Count<1 then exit;
ChildNode:= XMLNode.ChildNodes.First;
if ChildNode=nil then raise Exception.Create('Empty Mesh list. Expected: '+IntToStr(Count));
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	if ChildNode=nil then raise Exception.Create('Expected '+IntToStr(Count)+'Meshes. Found: '+IntToStr(I));
	if ChildNode.LocalName='GLMesh' then
		begin
		List.Add(Mesh);
		LoadGLMesh(ChildNode,Mesh);
		end;
	ChildNode:=ChildNode.NextSibling;
	end;
end;


{------------------------------------------------------------------------------}


procedure LoadGLLines(XMLNode: IXMLNode; Lines: TGLLines);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='GLRenderer' then LoadGLRenderer(ChildNode,Lines)
	else if ChildNode.LocalName='Vertices' then Load3DVectorArray(ChildNode,Lines.Vertices)
	else if ChildNode.LocalName='LineColor' then LoadIntegerArray(ChildNode,Lines.LineColor);
	ChildNode:=ChildNode.NextSibling;
	end;
end;



procedure LoadGLLineList(XMLNode: IXMLNode; List: TGLLineList);
var Count,I: integer;
var ChildNode: IXMLNode;
var Lines: TGLLines;
begin
Count:=XMLNode.Attributes['Count'];
if Count<1 then exit;
ChildNode:= XMLNode.ChildNodes.First;
if ChildNode=nil then raise Exception.Create('Empty Line list. Expected: '+IntToStr(Count));
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	if ChildNode=nil then raise Exception.Create('Expected '+IntToStr(Count)+'Lines. Found: '+IntToStr(I));
	if ChildNode.LocalName='GLLines' then
		begin
		List.Add(Lines);
		LoadGLLines(ChildNode,Lines);
		end;
	ChildNode:=ChildNode.NextSibling;
	end;
end;



{------------------------------------------------------------------------------}




procedure LoadGL3DFont(XMLNode: IXMLNode; Font: TGL3DFont);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='DelphiFont' then ReadXMLFont(ChildNode,Font.Font)
	else if ChildNode.LocalName='Depth' then Font.FontDepth:=ReadXMLDouble(ChildNode);
	ChildNode:=ChildNode.NextSibling;
	end;
end;




procedure LoadGL3DString(XMLNode: IXMLNode; String3D: TGL3DString);
var ChildNode: IXMLNode;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='GLRenderer' then LoadGLRenderer(ChildNode,String3D)
	else if ChildNode.LocalName='Font3D' then LoadGL3DFont(ChildNode,String3D.GL3DFont)
	else if ChildNode.LocalName='Text' then String3D.Text:=ReadXMLString(ChildNode)
	else if ChildNode.LocalName='Justify' then String3D.Justify:=TTextJustify(ReadXMLInteger(ChildNode))
	else if ChildNode.LocalName='LineSpacing' then String3D.LineSpacing:=ReadXMLDouble(ChildNode)
	else if ChildNode.LocalName='ShowLineBreaks' then String3D.ShowLineBreaks:=ReadXMLBoolean(ChildNode)
	else if ChildNode.LocalName='RenderMode' then String3D.RenderMode:=TRenderMode(ReadXMLInteger(ChildNode))
	else if ChildNode.LocalName='Color' then String3D.Color:=ReadXMLInteger(ChildNode);
	ChildNode:=ChildNode.NextSibling;
	end;
end;



procedure LoadGLStringList3d(XMLNode: IXMLNode; List: TGLStringList3D);
var Count,I: integer;
var ChildNode: IXMLNode;
var String3D: TGL3DString;
begin
Count:=XMLNode.Attributes['Count'];
if Count<1 then exit;
ChildNode:= XMLNode.ChildNodes.First;
if ChildNode=nil then raise Exception.Create('Empty Line list. Expected: '+IntToStr(Count));
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	if ChildNode=nil then raise Exception.Create('Expected '+IntToStr(Count)+'Lines. Found: '+IntToStr(I));
	if ChildNode.LocalName='GL3DString' then
		begin
		List.Add(String3D);
		LoadGL3DString(ChildNode,String3D);
		end;
	ChildNode:=ChildNode.NextSibling;
	end;
end;


{------------------------------------------------------------------------------}


procedure LoadFrame(XMLNode: IXMLNode; Frame: TGLFrame); forward




procedure LoadGLFrameList(XMLNode: IXMLNode; List: TGLFrameList);
var Count,I: integer;
var ChildNode: IXMLNode;
var Frame: TGLFrame;
begin
Count:=XMLNode.Attributes['Count'];
if Count<1 then exit;
ChildNode:= XMLNode.ChildNodes.First;
if ChildNode=nil then raise Exception.Create('Empty Frame list. Expected: '+IntToStr(Count));
for I:=0 to Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Count));
	if ChildNode=nil then raise Exception.Create('Expected '+IntToStr(Count)+'Lines. Found: '+IntToStr(I));
	if ChildNode.LocalName='GLFrame' then
		begin
		List.Add(Frame);
		LoadFrame(ChildNode,Frame);
		end;
	ChildNode:=ChildNode.NextSibling;
	end;
end;


{------------------------------------------------------------------------------}



procedure LoadFrame(XMLNode: IXMLNode; Frame: TGLFrame);
var ChildNode: IXMLNode;
begin
ChildNode:=XMLNode.ChildNodes.First;
while ChildNode <> nil do
	begin
	if ChildNode.LocalName='GLObject' then LoadGLObject(ChildNode, Frame)
	else if ChildNode.LocalName='GLMeshList' then LoadGLMeshList(ChildNode, Frame.MeshList)
	else if ChildNode.LocalName='GLLineList' then LoadGLLineList(ChildNode, Frame.LineList)
	else if ChildNode.LocalName='GLStringList3D' then LoadGLStringList3d(ChildNode, Frame.StringList3D)
	else if ChildNode.LocalName='GLFrameList' then LoadGLFrameList(ChildNode, Frame.FrameList);
	ChildNode:= ChildNode.NextSibling;
	end;
end;




procedure LoadFrameFromStream(Frame: TGLFrame;   Stream: TStream);
{Insert specified XML stream at specified Tree node }
var XMLNode: IXMLNode;
var XMLDoc: TXMLDocument;
begin
XMLDoc:=TXMLDocument.Create(Application);
try
XMLDoc.Options:=XMLDoc.Options + [doAttrNull];
XMLDoc.LoadFromStream(Stream);
XMLDoc.Active:= True;

if XMLDoc.ChildNodes.First = nil then raise Exception.Create('Empty OpenGL File');

XMLNode:= XMLDoc.DocumentElement.ChildNodes.First;
LoadFrame(XMLNode,Frame);
XMLDoc.Active:= False;
finally XMLDoc.Free; end;
end;





function LoadGLXMLFile(Frame: TGLFrame; Filename: string; ProgressHandler: TXMLProgressProc): boolean;
{Load specified file into tree}
var Stream: TMemoryStream;
begin
ProgressProc:=ProgressHandler;
Stream:=TMemoryStream.Create;
Stream.LoadFromFile(Filename);
try
LoadFrameFromStream(Frame,Stream);
finally Stream.Free; end;
end;



end.
