unit GLXMLWriter;

interface


uses Windows,Classes,Graphics,SysUtils,DSupport,GLRenderer,GLFrame,GLQuatLib,
     XMLIntf,XMLDoc,Vector,HugeArry,GLCustomMesh, GLMesh,GLFastMesh, OpenGL15,GLLibrary,GLAnimator,GLTexture,
     GLColorArray,GLLines,GLFonts,GLCubeMap,GLXMLGlobal,XMLDocLibrary,ActiveX;

function SaveGLXMLFile(Frame: TGLFrame; Filename: string; ProgressHandler: TXMLProgressProc): boolean;


implementation






procedure SaveGLArrayf4(Node: IXMLNode; GA: TGLArrayf4);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Float4');
ChildXMLNode.NodeValue:=GLArrayf4ToStr(GA);
end;


procedure SaveTagGLArrayf4(Node: IXMLNode; Tag: string;  GA: TGLArrayf4);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveGLArrayf4(ChildXMLNode,GA);
end;


{------------------------------------------------------------------------------}


procedure SaveByteArray(Node: IXMLNode; HB: THugeByte);
var I,L: integer;
var ChildXMLNode: IXMLNode;
var S,InStr: string;
begin
ChildXMLNode:= Node.AddChild('ByteArray');
ChildXMLNode.Attributes['Count']:=HB.Count;
L:=GetXMLNestingLevel(ChildXMLNode);
InStr:=CRLF_Char+GetXMLIndentStr(L*2);
S:='';
for I:=0 to HB.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,HB.Count));
	if I<>0 then S:=S+',';
	if (I mod 25)=0 then S:=S+InStr;
	S:=S+IntToStr(HB[I]);
	end;
ChildXMLNode.NodeValue:=S;
end;



procedure SaveTagByteArray(Node: IXMLNode; Tag: string; HB: THugeByte);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveByteArray(ChildXMLNode,HB);
end;

{------------------------------------------------------------------------------}


procedure Save3DVectorArray(Node: IXMLNode; VA: THuge3DVector);
var I,L: integer;
var ChildXMLNode: IXMLNode;
var S,InStr: string;
var SL: TStringList;
begin
SL:=TStringList.Create;
ChildXMLNode:= Node.AddChild('Vector3DArray');
ChildXMLNode.Attributes['Count']:=VA.Count;
L:=GetXMLNestingLevel(ChildXMLNode);
InStr:=GetXMLIndentStr(L*2);
S:='';
for I:=0 to VA.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,VA.Count));
	if I<>0 then S:=S+',';
	if (I mod 10)=0 then
		begin
		SL.Add(S);
		S:=InStr;
		end;
	S:=S+Vector3DToString(VA[I]);
	end;
SL.Add(S);
ChildXMLNode.NodeValue:=SL.Text;
SL.Free;
end;


procedure SaveTag3DVectorArray(Node: IXMLNode; Tag: string; VA: THuge3DVector);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
Save3DVectorArray(ChildXMLNode,VA);
end;

{------------------------------------------------------------------------------}



procedure SaveVertices(Node: IXMLNode; Tag: string; Mesh: TGLCustomMesh);
var TagNode,VectorNode: IXMLNode;
var SL: TStringList;
var S,InStr: string;
var I,L: integer;
begin
SL:=TStringList.Create;
try
InStr:=GetXMLIndentStr(L*2);
TagNode:= Node.AddChild(Tag);
VectorNode:= TagNode.AddChild('Vector3DArray');
VectorNode.Attributes['Count']:=Mesh.VertexCount;
S:='';
for I:=0 to Mesh.VertexCount-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Mesh.VertexCount));
	if I<>0 then S:=S+',';
	if (I mod 10)=0 then
		begin
		SL.Add(S);
		S:=InStr;
		end;
	S:=S+Vector3DToString(Mesh.Vertex[I]);
	end;
SL.Add(S);
VectorNode.NodeValue:=SL.Text;
finally SL.Free; end;
end;



procedure SaveFaceNormals(Node: IXMLNode; Tag: string; Mesh: TGLCustomMesh);
var TagNode,VectorNode: IXMLNode;
var SL: TStringList;
var S,InStr: string;
var I,L: integer;
begin
SL:=TStringList.Create;
try
InStr:=GetXMLIndentStr(L*2);
TagNode:= Node.AddChild(Tag);
VectorNode:= TagNode.AddChild('Vector3DArray');
VectorNode.Attributes['Count']:=Mesh.FaceCount;
S:='';
for I:=0 to Mesh.VertexCount-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Mesh.FaceCount));
	if I<>0 then S:=S+',';
	if (I mod 10)=0 then
		begin
		SL.Add(S);
		S:=InStr;
		end;
	S:=S+Vector3DToString(Mesh.FaceNormal[I]);
	end;
SL.Add(S);
VectorNode.NodeValue:=SL.Text;
finally SL.Free; end;
end;



procedure SaveVertexNormals(Node: IXMLNode; Tag: string; Mesh: TGLCustomMesh);
var TagNode,VectorNode: IXMLNode;
var SL: TStringList;
var S,InStr: string;
var I,L: integer;
begin
SL:=TStringList.Create;
try
InStr:=GetXMLIndentStr(L*2);
TagNode:= Node.AddChild(Tag);
VectorNode:= TagNode.AddChild('Vector3DArray');
VectorNode.Attributes['Count']:=Mesh.VertexCount;
S:='';
for I:=0 to Mesh.VertexCount-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Mesh.VertexCount));
	if I<>0 then S:=S+',';
	if (I mod 10)=0 then
		begin
		SL.Add(S);
		S:=InStr;
		end;
	S:=S+Vector3DToString(Mesh.VertexNormal[I]);
	end;
SL.Add(S);
VectorNode.NodeValue:=SL.Text;
finally SL.Free; end;
end;



procedure SaveSaveFaceColors(Node: IXMLNode; Tag: string; Mesh: TGLCustomMesh);
var TagNode,VectorNode: IXMLNode;
var SL: TStringList;
var S,InStr: string;
var I,L: integer;
begin
SL:=TStringList.Create;
try
InStr:=GetXMLIndentStr(L*2);
TagNode:= Node.AddChild(Tag);
VectorNode:= TagNode.AddChild('ColorArray');
VectorNode.Attributes['Count']:=Mesh.FaceCount;
S:='';
for I:=0 to Mesh.VertexCount-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Mesh.FaceCount));
	if I<>0 then S:=S+',';
	if (I mod 10)=0 then
		begin
		SL.Add(S);
		S:=InStr;
		end;
	S:=S+GLColorToString(Mesh.FaceColors[I]);
	end;
SL.Add(S);
VectorNode.NodeValue:=SL.Text;
finally SL.Free; end;
end;


{------------------------------------------------------------------------------}


procedure SaveIntegerArray(Node: IXMLNode; VA: THugeInteger);
var I,L: integer;
var ChildXMLNode: IXMLNode;
var S,InStr: string;
begin
ChildXMLNode:= Node.AddChild('IntegerArray');
ChildXMLNode.Attributes['Count']:=VA.Count;
L:=GetXMLNestingLevel(ChildXMLNode);
InStr:=CRLF_Char+GetXMLIndentStr(L*2);
S:='';
for I:=0 to VA.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,VA.Count));
	if I<>0 then S:=S+',';
	if (I mod 15)=0 then S:=S+InStr;
	S:=S+IntToStr(VA[I]);
	end;
ChildXMLNode.NodeValue:=S;
end;



procedure SaveTagIntegerArray(Node: IXMLNode; Tag: string; VA: THugeInteger);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveIntegerArray(ChildXMLNode,VA);
end;



{------------------------------------------------------------------------------}


procedure SaveAnimationPos(Node: IXMLNode; Inx: integer; Pos: TGLPosition);
var KeyNode,ChildNode: IXMLNode;
begin
KeyNode:= Node.AddChild('AnimationKey');
KeyNode.Attributes['Index']:=Inx;
ChildNode:= KeyNode.AddChild('Position');
SaveXML3DVector(ChildNode,Pos.Pos);
ChildNode:= KeyNode.AddChild('Rotation');
SaveXML3DVector(ChildNode,Pos.Rot.V);
end;


procedure SaveGLAnimator(Node: IXMLNode; Animator: TGLAnimator);
var Node1,Node2: IXMLNode;
var I: integer;
begin
Node1:= Node.AddChild('Animator');
Node2:= Node1.AddChild('Array');
Node2.Attributes['Count']:=Animator.AnimationArray.Count;
for I:=0 to Animator.AnimationArray.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,Animator.AnimationArray.Count));
	SaveAnimationPos(Node2,I,Animator.AnimationArray[I]);
	end;
end;
       

procedure SaveGLObject(Node: IXMLNode; GLObject: TGLObject);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLObject');
SaveTagXMLString(ChildXMLNode,'Name',GLObject.Name);
SaveTagXML3DVector(ChildXMLNode,'ExtentMin',GLObject.ExtentMin);
SaveTagXML3DVector(ChildXMLNode,'ExtentMax',GLObject.ExtentMax);
SaveTagXMLQuat(ChildXMLNode,'Quaternion',GLObject.Quaternion);
SaveTagXML3DVector(ChildXMLNode,'Position',GLObject.Position);
SaveTagXML3DVector(ChildXMLNode,'ResetPos',GLObject.ResetPos);
SaveTagXML3DVector(ChildXMLNode,'RotationCenter',GLObject.RotationCenter);


SaveTagXMLBoolean(ChildXMLNode,'FlyThrough',GLObject.FlyThrough);
SaveTagXMLDouble(ChildXMLNode,'Scale',GLObject.Scale);
SaveTagXML3DVector(ChildXMLNode,'Scale3D',GLObject.Scale3D);
SaveGLAnimator(ChildXMLNode,GLObject.Animator);
end;


{------------------------------------------------------------------------------}



procedure SaveTagImageArray(Node: IXMLNode; Tag: string; BM: TGLBaseBitmap);

var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveTagXMLInteger(ChildXMLNode,'Width',BM.Width);
SaveTagXMLInteger(ChildXMLNode,'Height',BM.Height);
SaveTagByteArray(ChildXMLNode,'Image',BM.ByteArray);
end;



procedure SaveGLCubeMap(Node: IXMLNode; CubeMap: TGLCubeMap);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('CubeMap');
SaveTagImageArray(ChildXMLNode,'LeftMap',CubeMap.LeftMap);
SaveTagImageArray(ChildXMLNode,'RightMap',CubeMap.RightMap);
SaveTagImageArray(ChildXMLNode,'TopMap',CubeMap.TopMap);
SaveTagImageArray(ChildXMLNode,'BottomMap',CubeMap.BottomMap);
SaveTagImageArray(ChildXMLNode,'FrontMap',CubeMap.FrontMap);
SaveTagImageArray(ChildXMLNode,'BackMap',CubeMap.BackMap);
SaveTagXMLInteger(ChildXMLNode,'MapMode',integer(CubeMap.MapMode));
SaveTagXML3DVector(ChildXMLNode,'Scale',CubeMap.Scale);
SaveTagXML3DVector(ChildXMLNode,'Offset',CubeMap.Offset);

SaveTagGLArrayf4(ChildXMLNode,'RPlane',CubeMap.RPlane);
SaveTagGLArrayf4(ChildXMLNode,'SPlane',CubeMap.SPlane);
SaveTagGLArrayf4(ChildXMLNode,'TPlane',CubeMap.TPlane);
end;




procedure SaveGLTexture(Node: IXMLNode; Texture: TGLObjectTexture);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Texture');
SaveTagByteArray(ChildXMLNode,'Image',Texture.ByteArray);
SaveTagXMLInteger(ChildXMLNode,'Width',Texture.Width);
SaveTagXMLInteger(ChildXMLNode,'Height',Texture.Height);
SaveTagXMLBoolean(ChildXMLNode,'Enabled',Texture.Enabled);
SaveTagXMLBoolean(ChildXMLNode,'Highlight',Texture.Highlight);

SaveTagXMLInteger(ChildXMLNode,'TextureMode',integer(Texture.TextureMode));
SaveTagXMLInteger(ChildXMLNode,'AutoTextureMap',integer(Texture.AutoTextureMap));
SaveTagXMLInteger(ChildXMLNode,'ManualTextureMap',integer(Texture.ManualTextureMap));

SaveTagXML2DVector(ChildXMLNode,'Scale',Texture.Scale);
SaveTagXML2DVector(ChildXMLNode,'Offset',Texture.Offset);

SaveTagXMLDouble(ChildXMLNode,'Yaw',Texture.Yaw);
SaveTagXMLDouble(ChildXMLNode,'Pitch',Texture.Pitch);
SaveTagGLArrayf4(ChildXMLNode,'SPlane',Texture.SPlane);
SaveTagGLArrayf4(ChildXMLNode,'TPlane',Texture.TPlane);
SaveGLCubeMap(ChildXMLNode,Texture.CubeMap);
end;



procedure SaveGLRenderer(Node: IXMLNode; Renderer: TGLRenderer);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLRenderer');
SaveGLObject(ChildXMLNode,Renderer);

SaveTagXMLBoolean(ChildXMLNode,'DoAlphaBlend',Renderer.DoAlphaBlend);
SaveTagXMLDouble(ChildXMLNode,'Transparency',Renderer.Transparency);
SaveTagXMLBoolean(ChildXMLNode,'BlendBackground',Renderer.BlendBackground);
SaveTagXMLBoolean(ChildXMLNode,'TransparentDepth',Renderer.TransparentDepth);
SaveTagGLArrayf4(ChildXMLNode,'LightEmission',Renderer.LightEmission);
SaveTagGLArrayf4(ChildXMLNode,'LightSpecular',Renderer.LightSpecular);
SaveTagXMLDouble(ChildXMLNode,'LightShininess',Renderer.LightShininess);
SaveGLTexture(ChildXMLNode,Renderer.Texture);
SaveTagXMLInteger(ChildXMLNode,'Color',Renderer.Color);
SaveTagXMLInteger(ChildXMLNode,'RenderMode',integer(Renderer.RenderMode));

SaveTagXMLBoolean(ChildXMLNode,'DoTwoSidedLighting',Renderer.DoTwoSidedLighting);
SaveTagXMLBoolean(ChildXMLNode,'DoFrontBackColor',Renderer.DoFrontBackColor);
SaveTagXMLInteger(ChildXMLNode,'FrontColor',Renderer.FrontColor);
SaveTagXMLInteger(ChildXMLNode,'BackColor',Renderer.BackColor);
end;

{------------------------------------------------------------------------------}


procedure SaveGLMesh(Node: IXMLNode; ID,Index: string; Mesh: TGLCustomMesh);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLMesh');
ChildXMLNode.Attributes['ID']:=ID;
ChildXMLNode.Attributes['Index']:=Index;

SaveGLRenderer(ChildXMLNode,Mesh);
SaveVertices(ChildXMLNode,'Vertices',Mesh);
SaveFaceNormals(ChildXMLNode,'FaceNormals',Mesh);
SaveVertexNormals(ChildXMLNode,'VertexNormals',Mesh);
SaveSaveFaceColors(ChildXMLNode,'FaceColor',Mesh);
end;



procedure SaveGLMeshList(Node: IXMLNode; List: TGLMeshList);
var I: integer;
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLMeshList');
ChildXMLNode.Attributes['Count']:=List.Count;
for I:=0 to List.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,List.Count));
	SaveGLMesh(ChildXMLNode,'ListMesh',IntToStr(I),List[I]);
	end;
end;


{------------------------------------------------------------------------------}


procedure SaveGLLines(Node: IXMLNode; ID,Index: string; Lines: TGLLines);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLLines');
ChildXMLNode.Attributes['ID']:=ID;
ChildXMLNode.Attributes['Index']:=Index;

SaveGLRenderer(ChildXMLNode,Lines);
SaveTag3DVectorArray(ChildXMLNode,'Vertices',Lines.Vertices);
SaveTagIntegerArray(ChildXMLNode,'LineColor',Lines.LineColor);
end;



procedure SaveGLLineList(Node: IXMLNode; List: TGLLineList);
var I: integer;
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLLineList');
ChildXMLNode.Attributes['Count']:=List.Count;
for I:=0 to List.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,List.Count));
	SaveGLLines(ChildXMLNode,'ListLines',IntToStr(I),List[I]);
	end;
end;


{------------------------------------------------------------------------------}



procedure SaveGL3DFont(Node: IXMLNode; F: TGL3DFont);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Font3D');
SaveXMLTagFont(ChildXMLNode,'DelphiFont',F.Font);
SaveTagXMLDouble(ChildXMLNode,'Depth',F.FontDepth);
end;



procedure SaveGL3DString(Node: IXMLNode; ID,Index: string; String3D: TGL3DString);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GL3DString');
ChildXMLNode.Attributes['ID']:=ID;
ChildXMLNode.Attributes['Index']:=Index;

SaveGLRenderer(ChildXMLNode,String3D);
SaveGL3DFont(ChildXMLNode,String3D.GL3DFont);
SaveTagXMLString(ChildXMLNode,'Text',String3D.Text);
SaveTagXMLInteger(ChildXMLNode,'Justify',Integer(String3D.Justify));
SaveTagXMLDouble(ChildXMLNode,'LineSpacing',String3D.LineSpacing);
SaveTagXMLBoolean(ChildXMLNode,'ShowLineBreaks',String3D.ShowLineBreaks);
SaveTagXMLInteger(ChildXMLNode,'RenderMode',Integer(String3D.RenderMode));
SaveTagXMLInteger(ChildXMLNode,'Color',Integer(String3D.Color));
end;



procedure SaveGL3DStringList(Node: IXMLNode; List: TGLStringList3D);
var I: integer;
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLStringList3D');
ChildXMLNode.Attributes['Count']:=List.Count;
for I:=0 to List.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,List.Count));
	SaveGL3DString(ChildXMLNode,'ListString3D',IntToStr(I),List[I]);
	end;
end;



{------------------------------------------------------------------------------}


procedure SaveGLFrame(Node: IXMLNode; ID,Index: string; Frame: TGLFrame); forward;



procedure SaveFrameList(Node: IXMLNode; List: TGLFrameList);
var I: integer;
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('GLFrameList');
ChildXMLNode.Attributes['Count']:=List.Count;
for I:=0 to List.Count-1 do
	begin
	if (I mod ProgressMod)=0 then HandleProgress(MulDiv(100,I,List.Count));
	SaveGLFrame(ChildXMLNode,'GLListFrame',IntToStr(I),List[I]);
	end;
end;


procedure SaveGLFrame(Node: IXMLNode; ID,Index: string; Frame: TGLFrame);
var ChildXMLNode: IXMLNode;

begin
ChildXMLNode:=Node.AddChild('GLFrame');
ChildXMLNode.Attributes['ID']:=ID;
ChildXMLNode.Attributes['Index']:=Index;

SaveGLObject(ChildXMLNode,Frame);
SaveGLMeshList(ChildXMLNode, Frame.MeshList);
SaveGLLineList(ChildXMLNode,Frame.LineList);
SaveGL3DStringList(ChildXMLNode,Frame.StringList3D);
SaveFrameList(ChildXMLNode,Frame.FrameList);
end;


{------------------------------------------------------------------------------}


function SaveGLXMLFile(Frame: TGLFrame; Filename: string; ProgressHandler: TXMLProgressProc): boolean;
{Save specified TreeNode to specified XML file}
var XMLDoc: TXMLDocument;
var XMLNode: IXMLNode;
begin
ProgressProc:=ProgressHandler;
Result:=False;
XMLDoc:= TXMLDocument.Create(nil);
XMLDoc.Options:=[doNodeAutoCreate,doNodeAutoIndent,doAttrNull,doAutoPrefix,doNamespaceDecl];
XMLDoc.Active:= True;
XMLNode:= XMLDoc.AddChild('OpenGLXML');
XMLNode.Attributes['Version']:= '1.0';
SaveGLFrame(XMLNode,'BaseFrame','0',Frame);

XMLDoc.SaveToFile(Filename);
Result:=True;
end;



initialization
CoInitialize(nil);
finalization
CoUnInitialize;
end.
