unit PrintLib;

interface

uses Windows,Types,Graphics,Printers,Math,SysUtils;


type TPrintProgress = procedure(Percent: integer);

procedure PrintFitToPage(BM: TBitmap; Progress: TPrintProgress; Expand,Shrink: boolean);
procedure GetPageParameters(BM: TBitmap; var XPages,YPages,PWidth,PHeight,Total: integer);
procedure PrintTiled(BM: TBitmap; Progress: TPrintProgress);

implementation

const PageBorder=5;
const PageBorder2 =PageBorder * PageBorder;




procedure PrintFitToPage(BM: TBitmap; Progress: TPrintProgress; Expand,Shrink: boolean);
{Print Bitmap with Fit-To-Page options}
{If Expand and bitmap<Page Size - will expand the bitmap to fit the page}
{If Shrink and bitmap>Page Size - will shrink the bitmap to fit the page}
{Note: make sure the bitmap is a DIB}
var Dest,Source: TRect;
var ShrinkWidth,ShrinkHeight: integer;
var ClientWidth,ClientHeight: integer;
var SY1,SY2,DY1,DY2: integer;
const BandHeight = 100;
var BandHeight2: integer;

	procedure GetShrinkSize;
	var Ratio1,Ratio2: Double;
	begin
	Ratio1:=BM.Height/BM.Width;
	Ratio2:=Printer.PageHeight/Printer.PageWidth;
	if Ratio1>Ratio2 then
		begin
		ShrinkHeight:=ClientHeight;
		ShrinkWidth:=trunc(ShrinkHeight / Ratio1);
		end
	else
		begin
		ShrinkWidth:=ClientWidth;
		ShrinkHeight:=trunc(ShrinkWidth * Ratio1);
		end;
	end;


	procedure GetExpandSize;
	var WidthGap,HeightGap: integer;
	var Ratio: double;
	begin
	Ratio:=BM.Height/BM.Width;
	WidthGap:=ClientWidth-BM.Width;
	HeightGap:=ClientHeight-BM.Height;
	if WidthGap<HeightGap then
		begin
		ShrinkWidth:=ClientWidth;
		ShrinkHeight:=Trunc(ShrinkWidth * Ratio);
		end
	else
		begin
		ShrinkHeight:=ClientHeight;
		ShrinkWidth:=Trunc(ShrinkHeight / Ratio);
		end;
	end;


begin
ClientWidth:=Printer.PageWidth-PageBorder2;
ClientHeight:=Printer.PageHeight-PageBorder2;
ShrinkWidth:=BM.Width; ShrinkHeight:=BM.Height;

if (BM.Width<ClientWidth) and (BM.Height<ClientHeight) then
	begin
	if Expand then GetExpandSize;
	end
else
	begin
	if Shrink then GetShrinkSize;
	end;

BandHeight2:=MulDiv(BandHeight,ShrinkWidth,BM.Width);
Printer.Title:='Test';
BM.Canvas.Lock;
Printer.BeginDoc;
try
SelectPalette(Printer.Canvas.Handle,BM.Palette, True);
RealizePalette(Printer.Canvas.Handle);

{Do banding so you don't overflow memory limitations in Win98}
SY1:=0; SY2:=SY1+BandHeight;
DY1:=PageBorder; DY2:=DY1+BandHeight2;
repeat
	begin
	if Assigned(Progress) then Progress(trunc((SY1/BM.Height)*100));
	Source:=Rect(0,SY1,BM.Width,SY2);
	Dest:=Rect(PageBorder,DY1,ShrinkWidth,DY2);
	Printer.Canvas.CopyMode:=cmSrcCopy;
	Printer.Canvas.CopyRect(Dest,BM.Canvas,Source);
	SY1:=SY2; SY2:=SY1+BandHeight;
	DY1:=DY2; DY2:=DY1+BandHeight2;
	end
until SY1>=BM.Height;
Printer.EndDoc;

finally
 BM.Canvas.Unlock;
 end;
end;



procedure GetPageParameters(BM: TBitmap; var XPages,YPages,PWidth,PHeight,Total: integer);
begin
PWidth:=Printer.PageWidth-PageBorder2;
PHeight:=Printer.PageHeight-PageBorder2;
XPages:=ceil(BM.Width/PWidth);
YPages:=ceil(BM.Height/PHeight);
Total:=XPages*YPages;
end;



procedure DrawPageBoarder(Canvas: TCanvas; X,Y,XLimit,YLimit: integer; Rect: TRect);
begin
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Color:=clBlack;
Canvas.Pen.Width:=2;
if X=0 then
	begin
	Printer.Canvas.MoveTo(Rect.Left,Rect.Top);
	Printer.Canvas.LineTo(Rect.Left,Rect.Bottom);
	end;
if X=(XLimit-1) then
	begin
	Printer.Canvas.MoveTo(Rect.Right,Rect.Top);
	Printer.Canvas.LineTo(Rect.Right,Rect.Bottom);
	end;
if Y=0 then
	begin
	Printer.Canvas.MoveTo(Rect.Left,Rect.Top);
	Printer.Canvas.LineTo(Rect.Right,Rect.Top);
	end;
if Y=(YLimit-1) then
	begin
	Printer.Canvas.MoveTo(Rect.Left,Rect.Bottom);
	Printer.Canvas.LineTo(Rect.Right,Rect.Bottom);
	end;
end;


procedure PrintTiled(BM: TBitmap; Progress: TPrintProgress);
{Note: make sure the bitmap is a DIB}
var Dest,Source: TRect;
var X,Y,PWidth,PHeight: integer;
var XPages,YPages,TotalPages: integer;
var W,H: integer;
var P: double;
begin
GetPageParameters(BM,XPages,YPages,PWidth,PHeight,TotalPages);
try
for Y:=0 to YPages-1 do
 for X:=0 to XPages-1 do
	begin
	P:=((XPages*Y)+X)/TotalPages;
	if Assigned(Progress) then Progress(trunc(P*100));
	Source:=Rect(0,0,PWidth,PHeight);
	OffsetRect(Source,X*PWidth,Y*PHeight);
	if Source.Right>BM.Width then Source.Right:=BM.Width;
	if Source.Bottom>BM.Height then Source.Bottom:=BM.Height;
	W:=Source.Right-Source.Left;
	H:=Source.Bottom-Source.Top;
	Dest:=Rect(PageBorder,PageBorder,W,H);
	Printer.Title:=IntToStr(X)+' '+IntToStr(Y);
	Printer.BeginDoc;
	Printer.Canvas.CopyMode:=cmSrcCopy;
	Printer.Canvas.CopyRect(Dest,BM.Canvas,Source);
	DrawPageBoarder(Printer.Canvas,X,Y,XPages,YPages,Dest);
	Printer.EndDoc;
	end;
finally end;
end;



end.
