unit RASLib;

interface

uses Windows,StdCtrls,ComCtrls,SysUtils,Dialogs,RAS;


const RasMaxEntries = 100;


type TPhoneBookArray = record
  Count: integer;
  Entries: array [0..RasMaxEntries-1] of TRasEntryName;
  end;


type TConnArray = record
  Count: integer;
  Entries: array [0..RasMaxEntries-1] of TRasConn;
  end;

function GetStatusString(state: TRasConnState; Error: integer): String;
function GetPhoneBook(var PhoneArray: TPhoneBookArray): boolean;
procedure FillPhoneListBox(ListBox: TListBox; PhoneArray: TPhoneBookArray);
function GetActiveConnections(var ConnArray: TConnArray): DWord;
procedure FillConnectionList(List: TListView; ConnArray: TConnArray);
procedure GetUsernamePassword(Connection: string; var UserName,Password: string);
function SetUsernamePassword(ConnName,Username,Password: string): boolean;


implementation



function GetStateStatus(state: TRasConnState): string;
begin
case State of
 RASCS_OpenPort: Result:='Opening port';
 RASCS_PortOpened: Result:='Port opened';
 RASCS_ConnectDevice: Result:='Connecting device';
 RASCS_DeviceConnected: Result:='Device connected';
 RASCS_AllDevicesConnected: Result:='All devices connected';
 RASCS_Authenticate: Result:='Start authenticating';
 RASCS_AuthNotify: Result:='Authentication: notify';
 RASCS_AuthRetry: Result:='Authentication: retry';
 RASCS_AuthCallback: Result:='Authentication: callback';
 RASCS_AuthChangePassword: Result:='Authentication: change password';
 RASCS_AuthProject: Result:='Authentication: projecting';
 RASCS_AuthLinkSpeed: Result:='Authentication: link speed';
 RASCS_AuthAck: Result:='Authentication: acknowledge';
 RASCS_ReAuthenticate: Result:='Authentication: reauthenticate';
 RASCS_Authenticated: Result:='Authenticated';
 RASCS_PrepareForCallback: Result:='Preparing for callback';
 RASCS_WaitForModemReset: Result:='Waiting for modem reset';
 RASCS_WaitForCallback: Result:='Waiting for callback';
 RASCS_Projected: Result:='Projected';
 RASCS_StartAuthentication: Result:='Start authentication';
 RASCS_CallbackComplete: Result:='Callback complete';
 RASCS_LogonNetwork: Result:='Logging on network';

 RASCS_Interactive: Result:='Interactive';
 RASCS_RetryAuthentication: Result:='Retry Authentication';
 RASCS_CallbackSetByCaller: Result:='Callback set by caller';
 RASCS_PasswordExpired: Result:='Password expired';

 RASCS_Connected: Result:='Connected';
 RASCS_Disconnected: Result:='Disconnected';
 end;
end;



function GetStatusString(state: TRasConnState; Error: integer): String;
var CharStr: Array[0..100] of Char;
begin
if Error <> 0 then
	begin
	RasGetErrorString(Error,CharStr, 100);
	Result:=CharStr;
	end
else Result:=GetStateStatus(State);
end;


{----------------- Routines to Handle Phone Book Entries ----------------------}

function GetPhoneBook(var PhoneArray: TPhoneBookArray): boolean;
{Read phone book entries into array}
var bufsize: integer;
var Return: integer;
begin
PhoneArray.Entries[0].dwSize:=SizeOf(TRasEntryName);
bufsize:=SizeOf(TRasEntryName) * RasMaxEntries;
Return:=RasEnumEntries(nil, nil, @PhoneArray.Entries[0], bufsize, PhoneArray.Count);
Result:=(Return = 0) and (PhoneArray.Count>0);
end;




procedure FillPhoneListBox(ListBox: TListBox; PhoneArray: TPhoneBookArray);
{Put phone book entries into a list box}
var I: integer;
begin
ListBox.Items.Clear;
for I:=0 to PhoneArray.Count-1 do ListBox.Items.Add(PhoneArray.Entries[I].szEntryName);
ListBox.ItemIndex:=0;
end;


{------------------ Routines To Handle Active Connections ---------------------}



function GetActiveConnections(var ConnArray: TConnArray): DWord;
{Get active connections}
var bufsize: integer;
begin
ConnArray.Entries[0].dwSize:=SizeOf(TRasConn);
bufsize:=SizeOf(TRasConn) * RasMaxEntries;
Result:=RasEnumConnections(@ConnArray.Entries[0], bufsize, ConnArray.Count);
end;



procedure FillConnectionList(List: TListView; ConnArray: TConnArray);
{Fill a connection list view}
var stat: TRasConnStatus;
var I: integer;
var ListItem: TListItem;
begin
for I:=0 to ConnArray.Count-1 do
	begin
	ListItem:=List.Items.Add;
	LIstItem.Data:=Pointer(ConnArray.Entries[I].hrasconn);
	ListItem.Caption:=ConnArray.Entries[I].szEntryName;
	RasGetConnectStatus(ConnArray.Entries[I].hrasconn, stat);
	ListItem.SubItems.Add(GetStatusString(stat.rasconnstate, stat.dwError));
	ListItem.SubItems.Add(ConnArray.Entries[I].szDeviceType);
	ListItem.SubItems.Add(ConnArray.Entries[I].szDeviceName);
	end;
end;



{------------------------------------------------------------------------------}


procedure GetUsernamePassword(Connection: string; var UserName,Password: string);
var DialParams: TRasDialParams;
var fp: LongBool;
var r: integer;
var c: Array[0..100] of Char;
begin
FillChar(DialParams, SizeOf(TRasDialParams), 0);
DialParams.dwSize:=Sizeof(TRasDialParams);
StrPCopy(DialParams.szEntryName, Connection);

r:=RasGetEntryDialParams(nil, DialParams, fp);
if r = 0 then
 	begin
 	UserName:=DialParams.szUserName;
 	if fp then Password:=DialParams.szPassword;
 	end
 else
	begin
	RasGetErrorString(r, c, 100);
	ShowMessage('RasGetEntryDialParams failed: ' + c);
	end;
end;




function SetUsernamePassword(ConnName,Username,Password: string): boolean;
var DialParams: TRasDialParams;
begin
DialParams.dwSize:=Sizeof(TRasDialParams);
StrPCopy(DialParams.szEntryName, ConnName);
StrPCopy(DialParams.szUserName, UserName);
StrPCopy(DialParams.szPassword, Password);
{Note: phone number cannot be set by this function}
StrPCopy(DialParams.szPhoneNumber, '111');
if RasSetEntryDialParams(nil, DialParams, False) <> 0 then ShowMessage('RasSetEntryDialParams failed.');
end;




end.
