unit SVGLibrary;

interface

uses Types, Graphics, SysUtils, GLPolygons,GLTextOutline,DSupport,Vector, BitmapLib;


function GetSVGColorStr(C: TColor): string;

function TrueTypeToSVG(Text,Attrib: string; Info: TFontInfo; Offset: T2DVector): string; overload;
function TrueTypeToSVG(Text,Attrib: string; Canvas: TCanvas; Offset: T2DVector): string; overload;


implementation




function GetSVGColorStr(C: TColor): string;
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
Result:='rgb('+IntToStr(R)+','+IntToStr(G)+','+IntToStr(B)+')';
end;





function TrueTypeToSVG(Text,Attrib: string; Info: TFontInfo; Offset: T2DVector): string;
{Convert TrueType font to an SVG Path}
{Attrib adds any additional attributes you want.}
var Outline: TPolygonArray;
var I,J,LastEOL: integer;
var Scale: double;
const BaseSize = 1000;

	function GetPointStr(Com: string; P: TPoint): string;
	{Scale and output point to a string}
	begin
	Result:=Com+FloatToStrF(P.X * Scale,ffFixed,18,2)+' '+FloatToStrF(P.Y * Scale,ffFixed,18,2);
	end;


	function GetTranslationStr(OffX,OffY: double): string;
	begin
	Result:='transform="translate(';
	Result:=Result+FloatToStrF(OffX,ffFixed,18,2)+' ';
	Result:=Result+FloatToStrF(OffY,ffFixed,18,2)+')"';
	end;

	procedure TestNewLine;
	begin
	if (Length(Result)-LastEOL)>60 then
		begin
		Result:=Result+CRLF_Char;
		LastEOL:=Length(Result);
		end;
	end;



begin
LastEOL:=0;
Result:='<path '+Attrib;
Result:=Result+GetTranslationStr(Offset.X,Offset.Y);
Result:=Result+' fill="'+GetSVGColorStr(Info.Color);
TestNewLine;
Result:=Result+'" d="';
Scale:=Abs(Info.Size / BaseSize);
Info.Size:=BaseSize;
GetStringOutline(Info,Text,Outline);
for J:=0 to High(OutLine) do
 if Length(OutLine[J])>0 then
	begin
	{Start polygon}
	Result:=Result+GetPointStr('M ', OutLine[J,0]);
	{Insert points in polygon}
	for I:=1 to High(OutLine[J]) do
		begin
		Result:=Result+GetPointStr(' L ', OutLine[J,I]);
		TestNewLine;
		end;
	{Close polygon}
	Result:=Result+' Z';
	end;
{Close Path}
Result:=Result+'"/>'+CRLF_Char;
end;


function TrueTypeToSVG(Text,Attrib: string; Canvas: TCanvas; Offset: T2DVector): string;
{Convert TrueType font from a canvas to an SVG Path}
var Info: TFontInfo;
begin
Info.Name:=Canvas.Font.Name;
Info.Style:=Canvas.Font.Style;
Info.Size:=Canvas.Font.Size;
Info.Color:=Canvas.Font.Color;
Result:=TrueTypeToSVG(Text,Attrib,Info,Offset);
end;



end.
