unit TextLibrary;

interface

uses Windows,Classes,SysUtils, Graphics,VCLLibrary,DSupport;


{ Delphi incorrectly imports this function, so we must reimport it
 manually to obtain the full functionality of this function}

function TabbedTextOut(hDC: HDC; X, Y: Integer; lpString: PChar; nCount, nTabPositions: Integer;
  lpnTabStopPositions: Pointer; nTabOrigin: Integer): Longint; stdcall;

{ Delphi incorrectly imports this function, so we must reimport it
 manually to obtain the full functionality of this function}

function GetTabbedTextExtent(hDC: HDC; lpString: PChar;
  nCount, nTabPositions: Integer; lpnTabStopPositions: Pointer): DWORD; stdcall;


type TTextAttribute = record
 UseDefault: boolean;
 BackColor,TextColor: TColor;
 Style: TFontStyles;
 EndInx: integer;
 end;

var List: TList;


type TTextWriter = class(TObject)
 private
  TabStops: ^integer;		{Empty tab stop defaults to 8 time average char width}
  Attributes: TList;
  function GetXExtent(Canvas: TCanvas; Text: string): integer;
  procedure TabbedText(Canvas: TCanvas; X, Y: integer; S: string);
 protected
 public
  constructor Create;
  procedure Clear;
  function GetAttribStr: string;
  procedure AddAttribute(EndInx: integer; BackColor,TextColor: TColor; Style: TFontStyles); overload;
  procedure AddAttribute(Attribute: TTextAttribute); overload;
  procedure AddDefaultAttib(EndInx: integer);
  procedure WriteText(Canvas: TCanvas; S: string; X, Y: integer);
 end;


implementation


{reimport the function}
function GetTabbedTextExtent; external user32 name 'GetTabbedTextExtentA';

{reimport the function}
function TabbedTextOut; external user32 name 'TabbedTextOutA';


{ TTextWriter }


procedure TTextWriter.AddAttribute(Attribute: TTextAttribute);
var A: ^TTextAttribute;
begin
New(A);
A^:=Attribute;
Attributes.Add(A);
end;



procedure TTextWriter.AddAttribute(EndInx: integer; BackColor, TextColor: TColor; Style: TFontStyles);
var A: TTextAttribute;
begin
A.UseDefault:=False;
A.EndInx:=EndInx;
A.BackColor:=BackColor;
A.TextColor:=TextColor;
A.Style:=Style;
AddAttribute(A);
end;


procedure TTextWriter.AddDefaultAttib(EndInx: integer);
var A: TTextAttribute;
begin
A.UseDefault:=True;
A.BackColor:=clWhite;
A.TextColor:=clBlack;
A.Style:=[];
A.EndInx:=EndInx;
AddAttribute(A);
end;

procedure TTextWriter.Clear;
var I: integer;
begin
for I:=0 to Attributes.Count-1 do
	Dispose(Attributes[I]);
end;


constructor TTextWriter.Create;
begin
Attributes:=TList.Create;
end;

function SortCompare(Item1, Item2: Pointer): integer;
begin
Result:=TTextAttribute(Item1^).EndInx-TTextAttribute(Item2^).EndInx;
end;

procedure TTextWriter.TabbedText(Canvas: TCanvas; X,Y: integer; S: string);
begin
TabbedTextOut(Canvas.Handle, X,Y, PChar(S), Length(S),0, @TabStops,  X);
end;

function TTextWriter.GetXExtent(Canvas: TCanvas; Text: string): integer;
begin
Result:=GetTabbedTextExtent(Canvas.Handle,PChar(Text),Length(Text),0,@Tabstops) and $FFFF;
end;



function GetCurrentAttrib(Canvas: TCanvas): TTextAttribute;
begin
Result.Style:=Canvas.Font.Style;
Result.TextColor:=Canvas.Font.Color;
Result.BackColor:=Canvas.Brush.Color;
end;


function TTextWriter.GetAttribStr: string;
var I: integer;
var A: TTextAttribute;
begin
Result:='';
for I:=0 to Attributes.Count-1 do
	begin
	A:=TTextAttribute(Attributes[I]^);
	Result:=Result+IntToStr(I);
	Result:=Result+' Def:'+PickString(A.UseDefault,'Yes','No ');
	Result:=Result+' Back:$'+IntToHex(A.BackColor,8);
	Result:=Result+' Text:$'+IntToHex(A.TextColor,8);
	Result:=Result+' Style:'+GetFontStyleString(A.Style);
	Result:=Result+' EndInx:'+IntToStr(A.EndInx);
	Result:=Result+CRLF_Char;
	end;
end;



procedure TTextWriter.WriteText(Canvas: TCanvas; S: string; X,Y: integer);
var I,Inx1,Inx2: integer;
var S1: string;
var A,Default: TTextAttribute;

	procedure SetAttributes(A: TTextAttribute);
	begin
	Canvas.Font.Style:=A.Style;
	Canvas.Font.Color:=A.TextColor;
	Canvas.Brush.Color:=A.BackColor;
	end;


begin
Default:=GetCurrentAttrib(Canvas);
Attributes.Sort(SortCompare);
Inx1:=1;
I:=0;
while Inx1<Length(S) do
	begin
	if I<Attributes.Count then
		begin
		A:=TTextAttribute(Attributes[I]^);
		Inx2:=A.EndInx;
		if A.UseDefault then A:=Default;
		Inc(I);
		end
	else
		begin
		Inx2:=Length(S)+1;
                A:=Default;
		end;
	S1:=Copy(S,Inx1, (Inx2-Inx1));
	SetAttributes(A);
	TabbedText(Canvas,X,Y,S1);
	X:=X+GetXExtent(Canvas,S1);
	Inx1:=Inx2;
	end;
SetAttributes(Default);
end;


end.
