unit TimerLib;

interface

uses Windows,MMSystem;

{ This Object creates a realtime counter object that
  simulates the operation of the old hardware generated
  interrupt based clock. Clock is read and set using the
  "Value" property. }

type
 TRealCounter = class(TObject)
 private
  FBase: int64;
  FScale: int64;
  function GetCount: dword;
  procedure SetCount(V: dword);
 public
  constructor Create;
  property Value: dword read GetCount write SetCount;
  property Scale: int64 read FScale write FScale;
 end;


{ Object used to create "Timeout"  objects which can be
 used to cancel a long operation if the hardware fails to
 respond. Pass a time delay in ms. to the "Start" method to
 start the timer. Check the "IsDone"  property repeatedly
 to see if the timeout time has completed. }

type
 TTimeout = class(TObject)
 private
  FEndTime: dword;
  FRunning: boolean;
 public
  constructor Create;
  procedure Start(V: dword);
  function IsDone: boolean;
 end;


implementation


{--------------- TRealCounter Methods -------------------------------}

constructor TRealCounter.Create;
begin
inherited Create;
FScale:=5;
SetCount(0);
end;

function TRealCounter.GetCount: dword;
begin
Result:=(timeGetTime -FBase) div FScale;
end;

procedure TRealCounter.SetCount(V: dword);
begin
FBase:=timeGetTime - (V * FScale);
end;


{--------------- TTimeOut object Methods -----------------------}


constructor TTimeout.Create;
begin
inherited Create;
FRunning:=False;
end;

procedure TTimeout.Start(V: dword);
begin
FEndTime:=timeGetTime+V;
FRunning:=True;
end;

function TTimeout.IsDone: boolean;
begin
if FRunning and (timeGetTime>FEndTime) then FRunning:=False;
Result:=not FRunning;
end;




end.
