unit USBLibrary;

interface

uses Windows, Messages, SysUtils, Classes, Forms, Registry, Masks, DSupport;

type DEV_BROADCAST_HDR = packed record
 dbch_size: DWORD;
 dbch_devicetype: DWORD;
 dbch_reserved: DWORD;
 end;
type PDevBroadcastHdr = ^DEV_BROADCAST_HDR;



type DEV_BROADCAST_DEVICEINTERFACE = record
 dbcc_size: DWORD;
 dbcc_devicetype: DWORD;
 dbcc_reserved: DWORD;
 dbcc_classguid: TGUID;
    dbcc_name: char;
 end;
  PDevBroadcastDeviceInterface  = ^DEV_BROADCAST_DEVICEINTERFACE;
type PDEV_BROADCAST_DEVICEINTERFACE = ^DEV_BROADCAST_DEVICEINTERFACE;


type DEV_BROADCAST_PORT = record
 dbcp_size: DWORD;
 dbcp_devicetype: DWORD;
 dbcp_reserved: DWORD;
 dbcp_name: array [0..0] of AnsiChar;
 end;
type PDEV_BROADCAST_PORT = ^DEV_BROADCAST_PORT;



const GUID_DEVINTERFACE_USB_DEVICE: TGUID = '{A5DCBF10-6530-11D2-901F-00C04FB951ED}';


const DBT_APPYBEGIN =			$0000;
const DBT_APPYEND =			$0001;
const DBT_DEVNODES_CHANGED =		$0007;
const DBT_QUERYCHANGECONFIG =		$0017;
const DBT_CONFIGCHANGED =		$0018;
const DBT_CONFIGCHANGECANCELED =	$0019;
const DBT_MONITORCHANGE =		$001B;
const DBT_SHELLLOGGEDON =		$0020;
const DBT_CONFIGMGAPI32 =		$0022;
const DBT_VXDINITCOMPLETE =		$0023;
const DBT_USERDEFINED =			$FFFF;

const DBT_DEVICEARRIVAL =		$8000;		{ system detected a new device}
const DBT_CONFIGMGPRIVATE =		$7FFF;
const DBT_DEVICEQUERYREMOVE =		$8001;		{ wants to remove, may fail}
const DBT_DEVICEQUERYREMOVEFAILED =	$8002;		{ removal aborted}
const DBT_DEVICEREMOVEPENDING =		$8003;		{ about to remove, still avail.}
const DBT_DEVICEREMOVECOMPLETE =	$8004;		{ device is gone}
const DBT_DEVICETYPESPECIFIC =		$8005;		{ type specific event}
const DBT_CUSTOMEVENT =			$8006;		{ user-defined event}

const DBT_VOLLOCKQUERYLOCK =		$8041;
const DBT_VOLLOCKLOCKTAKEN =		$8042;
const DBT_VOLLOCKLOCKFAILED =		$8043;
const DBT_VOLLOCKQUERYUNLOCK =		$8044;
const DBT_VOLLOCKLOCKRELEASED =		$8045;
const DBT_VOLLOCKUNLOCKFAILED =		$8046;
const DBT_VPOWERDAPI =			$8100;

const DBT_DEVTYP_OEM =			$00000000;	{ oem-defined device type}
const DBT_DEVTYP_DEVNODE =		$00000001;	{ devnode number}
const DBT_DEVTYP_VOLUME =		$00000002;	{ logical volume}
const DBT_DEVTYP_PORT =			$00000003;	{ serial, parallel}
const DBT_DEVTYP_NET =			$00000004;	{ network resource}
const DBT_DEVTYP_DEVICEINTERFACE =	$00000005;	{ device interface class}
const DBT_DEVTYP_HANDLE =		$00000006;	{ file system handle}

const DBT_NO_DISK_SPACE =		0047;
const DBT_LOW_DISK_SPACE =		0048;



{-------------------------------------------------------------------------------}

{Component to generate notify events when a USB component has been inserted or removed}

type TDeviceType = (dtStorage,dtSerial,dtUnknown);


type TUSBInfo = record
 DeviceType: TDeviceType;
 MainKey,SubKey: string;
 BaseDecription: string;
 DeviceDescription: string;
 PortName: string;
 ClassName: string;
 Service: string;
 FriendlyName: string;
 end;


{ Event Types }

type TOnUsbChangeEvent = procedure(AObject: TObject; const USBInfo: TUSBInfo) of object;

  { USB Class }
type TUSBNotifier = class(TObject)
 private
  FNotifyHandle: HDEVNOTIFY;
  FHandle: HWND;
  FOnUsbRemoval,
  FOnUsbInsertion: TOnUsbChangeEvent;
  procedure GetUsbInfo(const ADeviceString: string; var USBInfo: TUSBInfo);
  procedure WinMethod(var AMessage: TMessage);
  procedure RegisterUsbHandler;
  procedure WMDeviceChange(var AMessage: TMessage);
  function TestUSBSTORSubkeys(Reg: TRegistry; Key, Key2: string; var USBInfo: TUSBInfo): boolean;
  function TestUSBStoreDevice(var oReg: TRegistry; Key2: string; var USBInfo: TUSBInfo): boolean;
    function TestUSBSerialPort(var oReg: TRegistry; MainKey: string;
      var USBInfo: TUSBInfo): boolean;
 public
  constructor Create;
  destructor Destroy; override;
  property OnUsbInsertion: TOnUsbChangeEvent read FOnUsbInsertion write FOnUsbInsertion;
  property OnUsbRemoval: TOnUsbChangeEvent read FOnUsbRemoval write FOnUsbRemoval;
  end;

{-------------------------------------------------------------------------------}


function GetBroadcastTypeStr(Msg: TMessage): string;
function GetEventTypeStr(Msg: TMessage): string;
function GetDeviceTypeStr(DT: TDeviceType): string;


implementation


{---------------- Support Routines --------------------------------------------}




function GetBroadcastTypeStr(Msg: TMessage): string;
var Datos: PDevBroadcastHdr;
begin
Datos:=PDevBroadcastHdr(Msg.lParam);
case Datos.dbch_devicetype of
 DBT_DEVTYP_OEM: Result:='OEM';
 DBT_DEVTYP_DEVNODE: Result:='Dev Node';
 DBT_DEVTYP_VOLUME: Result:='Volume';
 DBT_DEVTYP_PORT: Result:='Port';
 DBT_DEVTYP_NET: Result:='Net';
 DBT_DEVTYP_DEVICEINTERFACE: Result:='Device Interface';
 DBT_DEVTYP_HANDLE: Result:='Handle';
 else Result:='Unknown: $'+IntToHex(Msg.LParam,8);
 end;
end;



function GetEventTypeStr(Msg: TMessage): string;
begin
case Msg.WParam of
 DBT_DEVNODES_CHANGED: Result:='Dev Nodes Changed';
 DBT_DEVICEARRIVAL: Result:='Device Arrived';
 DBT_CONFIGMGPRIVATE: Result:='Config MG Private';
 DBT_DEVICEQUERYREMOVE: Result:='Device Query Remove';
 DBT_DEVICEQUERYREMOVEFAILED: Result:='Device Query Remove Failed';
 DBT_DEVICEREMOVEPENDING: Result:='Device Remove Pending';
 DBT_DEVICEREMOVECOMPLETE: Result:='Device Remove Complete';
 DBT_DEVICETYPESPECIFIC: Result:='Device Type Specific';
 DBT_CUSTOMEVENT: Result:='Custom Event';
 DBT_NO_DISK_SPACE: Result:='No Disk Space';
 DBT_LOW_DISK_SPACE: Result:='Low Disk Space';
 else Result:='Unknown: $'+IntToHex(Msg.WParam,8);
 end;
end;


function GetDeviceTypeStr(DT: TDeviceType): string;
begin
case DT of
 dtStorage: Result:='Storage';
 dtSerial: Result:='Serial';
 dtUnknown: Result:='Unknown';
 else Result:='Invalide'
 end;
end;



{--------------- USB Notifier Object ------------------------------------------}




const GUID_DEVINTF_USB_DEVICE: TGUID = '{A5DCBF10-6530-11D2-901F-00C04FB951ED}';
const USB_INTERFACE                = $00000005; { Device interface class}
const USB_INSERTION                = $8000;     { System detected a new device}
const USB_REMOVAL                  = $8004;     { Device is gone}

  { Registry Keys}
const USBKEY  = 'SYSTEM\CurrentControlSet\Enum\USB\%s\%s';
const USBSTORKEY = 'SYSTEM\CurrentControlSet\Enum\USBSTOR';
const SUBKEY1  = USBSTORKEY + '\%s';
const SUBKEY2  = SUBKEY1 + '\%s';



constructor TUSBNotifier.Create;
begin
inherited Create;
{Create a non-windowed window to catch messages}
FHandle:=AllocateHWnd(WinMethod);
RegisterUsbHandler;
end;



destructor TUSBNotifier.Destroy;
begin
UnregisterDeviceNotification(FNotifyHandle);
DeallocateHWnd(FHandle);
inherited Destroy;
end;



function GetKeys(DeviceString: string; var Key1,Key2: string): boolean;
{ Extract the portions of the Device string to construct registry keys}
var Inx: integer;
var T1,T2,T3,T4: string;
begin
Inx:=1;
T1:=ExtractToken(DeviceString,'#',Inx);
T2:=ExtractToken(DeviceString,'#',Inx);
T3:=ExtractToken(DeviceString,'#',Inx);
T4:=ExtractToken(DeviceString,'#',Inx);
Key1:=T2; Key2:=T3;
end;


{------------------- Test USB Storage devices ---------------------------------}

function TUSBNotifier.TestUSBSTORSubkeys(Reg: TRegistry; Key, Key2: string; var USBInfo: TUSBInfo): boolean;
{Search USBSTOR for subkey that matches the specified Key and Key2 }
{Key = Device key under USBSTOR, and Key2 = represents the target subkey under Key}
var SubKeyList: TStringList;
var SK: string;
var J: integer;
begin
Result:=False;
if not Reg.OpenKeyReadOnly(Format(SUBKEY1,[Key])) then exit;
SubKeyList:=TStringList.Create;
try
Reg.GetKeyNames(SubKeyList);
Reg.CloseKey;

for J:=0 to SubKeyList.Count - 1 do
	begin
	if MatchesMask(SubKeyList[J],Key2 + '*') then
		begin
		{Get subkey}
		SK:=Format(SUBKEY2,[Key,SubKeyList[J]]);
		{ Got a match?, get the actual Description and Friendly Name}
		if Reg.OpenKeyReadOnly(SK) then
			begin
			USBInfo.SubKey:=SK;
			{Get relevant information}
			USBInfo.DeviceDescription:=Reg.ReadString('DeviceDesc');
			USBInfo.FriendlyName:=Reg.ReadString('FriendlyName');
			Reg.CloseKey;
			end;
		USBInfo.DeviceType:=dtStorage;
		Result:=true;
		end;
	end;
finally SubKeyList.Free; end;
end;



function TUSBNotifier.TestUSBStoreDevice(var oReg: TRegistry; Key2: string; var USBInfo: TUSBInfo): boolean;
{ Get list of keys in USBSTOR and enumerate each key}
{ looking for a key that matches our Key2}
var KeyList: TStringList;
var I: integer;
begin
Result:=False;
KeyList:=TStringList.Create;
try
if oReg.OpenKeyReadOnly(USBSTORKEY) then
	begin
	oReg.GetKeyNames(KeyList);
	oReg.CloseKey;

	{ Iterate through list to find our sKey2}
	for I:=0 to KeyList.Count - 1 do
		begin
		Result:=TestUSBSTORSubkeys(oReg,KeyList[I],Key2,USBInfo);
                if Result then break;
		end;
	end;
finally KeyList.Free; end;
end;


{------------------------------------------------------------------------------}





function TUSBNotifier.TestUSBSerialPort(var oReg: TRegistry; MainKey: string; var USBInfo: TUSBInfo): boolean;
{ Get list of keys in USBSTOR and enumerate each key}
{ looking for a key that matches our Key2}
var KeyList: TStringList;
var I: integer;
begin
Result:=False;
KeyList:=TStringList.Create;
try
if not oReg.OpenKeyReadOnly(MainKey+'\Device Parameters') then exit;
if not oReg.ValueExists('PortName') then exit;
USBInfo.DeviceType:=dtSerial;
USBInfo.PortName:=oReg.ReadString('PortName');
oReg.CloseKey;
finally KeyList.Free; end;
end;


{------------------------------------------------------------------------------}


procedure TUSBNotifier.GetUSBInfo(const ADeviceString: string; var USBInfo: TUSBInfo);
{Get information about newly inserted or removed device based on Info Windows sends}
var MainKey, sKey1,sKey2: string;
var oReg: TRegistry;
begin
USBInfo.MainKey:='';
USBInfo.SubKey:='';
USBInfo.DeviceType:=dtUnknown;
USBInfo.DeviceDescription:='';
USBInfo.FriendlyName:='';

oReg:=TRegistry.Create;
try
{exit if there is no data}
if ADeviceString='' then exit;

{Setup Registry keys based on WM_DeviceChange info}
oReg.RootKey:=HKEY_LOCAL_MACHINE;
GetKeys(ADeviceString,sKey1,sKey2);

{Get main USB key}
MainKey:=Format(USBKEY,[skey1,sKey2]);
{ Check USB device section for specified key - Exit if it is not a registered USB device }
if not oReg.OpenKeyReadOnly(MainKey) then exit;
USBInfo.MainKey:=MainKey;
USBInfo.BaseDecription:=oReg.ReadString('DeviceDesc');
USBInfo.ClassName:=oReg.ReadString('Class');
USBInfo.Service:=oReg.ReadString('Service');
{This may get overwritten later by USBSTOR data}
USBInfo.FriendlyName:=oReg.ReadString('FriendlyName');
oReg.CloseKey;

if TestUSBStoreDevice(oReg,sKey2,USBInfo) then exit;
if TestUSBSerialPort(oReg,MainKey,USBInfo) then exit;
finally oReg.Free; end;
end;




procedure TUSBNotifier.WMDeviceChange(var AMessage: TMessage);
{Recieves WM_DeviceChange messages indicating a device has changed}
var USBInfo: TUSBInfo;
var iDevType: integer;
var pData: PDevBroadcastDeviceInterface;
begin
if (AMessage.wParam = USB_INSERTION) or
   (AMessage.wParam = USB_REMOVAL) then
   	begin
   	pData:=PDevBroadcastDeviceInterface(AMessage.LParam);
   	iDevType:=pData^.dbcc_devicetype;

	{ Is it a USB Interface Device ?}
	if iDevType = USB_INTERFACE then
		begin
		USBInfo.BaseDecription:=PChar(@pData^.dbcc_name);
		GetUsbInfo(USBInfo.BaseDecription,USBInfo);

		{ Trigger Events if assigned}
		if (AMessage.wParam = USB_INSERTION) and
		    Assigned(FOnUsbInsertion) then FOnUsbInsertion(self,USBInfo);
		if (AMessage.wParam = USB_REMOVAL) and
		Assigned(FOnUsbRemoval) then FOnUsbRemoval(self,USBInfo);
		end;
	end;
end;



procedure TUSBNotifier.WinMethod(var AMessage: TMessage);
{Procedure the receives windows messages for non-windowed window}
begin
if (AMessage.Msg = WM_DEVICECHANGE) then WMDeviceChange(AMessage)
else AMessage.Result:=DefWindowProc(FHandle,AMessage.Msg, AMessage.wParam,AMessage.lParam);
end;


procedure TUSBNotifier.RegisterUsbHandler;
{Tell Windows to notify us when a USB device changes}
var rDbi: DEV_BROADCAST_DEVICEINTERFACE;
var iSize: integer;
begin
iSize:=SizeOf(DEV_BROADCAST_DEVICEINTERFACE);
ZeroMemory(@rDbi,iSize);
rDbi.dbcc_size:=iSize;
rDbi.dbcc_devicetype:=USB_INTERFACE;
rDbi.dbcc_reserved:=0;
rDbi.dbcc_classguid:=GUID_DEVINTF_USB_DEVICE;
rDbi.dbcc_name:=#0;
FNotifyHandle:=RegisterDeviceNotification(FHandle,@rDbi,DEVICE_NOTIFY_WINDOW_HANDLE);
end;




end.
