unit VCLLibrary;

interface

uses Windows,Messages,Classes,SysUtils, Forms,Controls,ExtCtrls,ComCtrls,Types, Graphics,
     CommDlg, Grids, StdCtrls, Dialogs,StrUtils, IntVector, DSupport,Vector,RichEdit,
     ClipBrd;

type THJustify = (hjLeft,hjCenter,hjRight);
type TVJustify = (vjTop,vjCenter,vjBottom);

var DefHorzJust: THJustify=hjLeft;
var DefVertJust: TVJustify=vjCenter;


type THinter = class(TComponent)
 private
  FHint: THintWindow;
  FTimer: TTimer;
  FFontSize: integer;
  procedure HandleTimer(Sender: TObject);
  function GetVisible: boolean;
  function GetColor: TColor;
  procedure SetColor(const Value: TColor);
 protected
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure ShowHint(Target: TControl; X,Y: integer; S: string; LifeMS: integer);
  procedure KillHint;
  property Visible: boolean read GetVisible;
  property Color: TColor read GetColor write SetColor;
  property FontSize: integer read FFontSize write FFontSize;
 published
 end;

{Creates a two-level list similar to the structure of an InI File}
{It contains a list of sections with list of entries in each section}

type TSectionList = class(TStringList)
 private
  function GetSection(Index: Integer): TStringList;
 protected
 public
  function Add(const S: string): integer; override;
  procedure Clear; override;
  procedure Delete(Index: Integer); override;
  destructor Destroy; override;
  property Sections[Index: Integer]: TStringList read GetSection; default;
 end;


{TAutoString is TStringList derrivative that can be configured to automatically
{put certain object types into the object array. The object type is specified as}
{in the argument to the Create method. Once it is specified, the every time the}
{"Add" method is called, a new object is added of the type specified. It}
{ automatically adds objects of the specified type everytime the "ADD" method is used}
{Automatically Frees an object when it is deleted or when the array is cleared}

{Warning - the object must be a simple object with no constructor.
The constructor won't get executed because TObjects Create method
is not virtual}


type TAutoString = class(TStringList)
 private
  ObjectClass: TClass;
 protected
 public
  constructor Create(ObjClass: TClass);
  destructor Destroy; override;
  function Add(const S: string): Integer; override;
  procedure Delete(Index: Integer); override;
  procedure Clear; override;
 published
 end;



procedure OwnerDrawItem(Canvas: TCanvas; S: string; Rect: TRect; HorzJust: THJustify; VertJust: TVJustify); overload;
procedure OwnerDrawItem(Canvas: TCanvas; S: string; Rect: TRect); overload;
procedure OwnerDrawInteger(Canvas: TCanvas; I: integer; Rect: TRect);
procedure OwnerDrawFloat(Canvas: TCanvas; F: double; Rect: TRect; Format: TFloatFormat; Digits: integer); overload;
procedure OwnerDrawFloat(Canvas: TCanvas; F: double; Rect: TRect; Digits: integer); overload;
procedure OwnerDrawAlign(Canvas: TCanvas; S: string; AlignLeft: boolean; Col, Row: Longint; Rect: TRect);




procedure CenterOnWorkArea(Form: TForm);
procedure ClearStringGrid(Grid: TStringGrid; ClearFixed: boolean);
procedure GetScrollPercent(SB: TScrollBox; var X,Y: double);
procedure SetScrollPercent(SB: TScrollBox; X,Y: double);
procedure AdjustScrollBox(SB: TScrollBox; BaseSize: TPoint; NewScale: double);
procedure ZoomImageScrollBox(SB: TScrollBox; Image: TImage;  NewScale: double);


procedure SearchTree(Tree: TTreeView; FindDialog: TFindDialog);
procedure SearchMemo(var FindDlg: TFindDialog; Memo: TMemo);
procedure StartSearch(FindDlg: TFindDialog);
procedure HandleOpenDialogChange(Dialog: TOpenDialog; Def: string; Update: boolean);

function FindChildControls(WinControl: TWinControl; Name: string): TControl;
procedure EnableChildControls(Control: TWinControl; Enable: boolean);

procedure EnableRadioItem(RG: TRadioGroup; Inx: integer; Enable: boolean);
procedure ShowRadioItem(RG: TRadioGroup; Inx: integer; Show: boolean);
function IsRadioItemAccessable(RG: TRadioGroup; Inx: integer): boolean;

function MoveListItemUp(List: TListBox): boolean;
function MoveListItemDown(List: TListBox): boolean;
function DeleteSelectedList(List: TListBox): boolean;
procedure ClearAllSelected(List: TListBox);
procedure SetAllSelected(List: TListBox; Value: boolean);
function ListBoxDragOver(Sender, Source: TObject; List: TListBox; P: TPoint): boolean;
procedure ListBoxDrop(Sender,Source: TObject; List: TListBox; P: TPoint);

procedure FreeObjects(const Strings: TStrings) ;
function CaseSensitiveIndexOf(const Strings: TStrings; const S: string): Integer;
function StringToFontStyle(Style: string): TFontStyles;
function GetFontStyleString(Style: TFontStyles): string;


procedure SetMinimumFormHeight(Form: TForm);

procedure ScrollMemoToEnd(Memo: TMemo);
procedure SetMemoLine(Memo: TMemo; LineNo: integer);
function GetMemoLine(Memo: TMemo): integer;
function GetMemoFirstLine(Memo: TMemo): integer;
procedure SetMemoFirstLine(Memo: TMemo; LineNo: integer);

procedure RichEditSearch(Memo: TRichEdit; FindDialog: TFindDialog);
procedure AutoSizeRadio(Radio: TRadioGroup);

procedure LimitedMemoAdd(S: string; Memo: TMemo; Limit: integer);
procedure CenterTrackBar(TrackBar: TTrackBar);

type TValidateFunc = function(Row: integer): boolean;

procedure SelectiveRowChange(Grid: TStringGrid; ACol, ARow: Integer; Validate: TValidateFunc; var CanSelect: Boolean);
function GetGridCells(Grid: TStringGrid): string;
procedure CopyStringGrid(Grid: TStringGrid);

implementation


{ THinter }

procedure THinter.KillHint;
begin
FTImer.Enabled:=False;
if Assigned(FHint) then
	begin
	FHint.ReleaseHandle;
	FreeAndNil(FHint);
	end;
end;



procedure THinter.HandleTimer(Sender: TObject);
begin
FTImer.Enabled:=False;
KillHint;
end;


constructor THinter.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FTimer:=TTimer.Create(nil);
FTimer.Interval:=1000;
FTimer.Enabled:=False;
FTimer.OnTimer:=HandleTimer;
FontSize:=9;
end;



destructor THinter.Destroy;
begin
FTimer.Free;
inherited;
end;


procedure THinter.ShowHint(Target: TControl; X, Y: integer; S: string; LifeMS: integer);
var R: TRect;
var O: TPoint;
begin
KillHint;
FHint:=THintWindow.Create(Owner);
FHint.Canvas.Font.Size:=FontSize;
Color:=RGB(255,255,225);
R:=FHint.CalcHintRect(MulDiv(Screen.Width,2,3), S, nil);
if R.Bottom>MulDiv(Screen.Height,2,9) then R.Bottom:=MulDiv(Screen.Height,2,9);
O:=Target.ClientToScreen(Point(X,Y));
OffsetRect(R, O.X, O.Y);
FHint.ActivateHint(R,S);
FTimer.Interval:=LifeMS;
FTimer.Enabled:=True;
end;


function THinter.GetVisible: boolean;
begin
Result:=FTimer.Enabled;
end;


function THinter.GetColor: TColor;
begin
Result:=FHint.Color;
end;


procedure THinter.SetColor(const Value: TColor);
begin
FHint.Color:=Value;
end;



{------------------------------------------------------------------------------}


procedure OwnerDrawItem(Canvas: TCanvas; S: string; Rect: TRect; HorzJust: THJustify; VertJust: TVJustify);
{Display the string in the grid inside the rectangle}
var Size,Delta: TPoint;
var Left,Top: integer;
const Border = 3;
begin
Size:=TPoint(Canvas.TextExtent(S));
Delta:=IntVectorSubtract(Rect.BottomRight,Rect.TopLeft);
Delta:=IntVectorSubtract(Delta,Size);
Delta:=IntScalarDivide(Delta,2);
case HorzJust of
 hjLeft: Left:=Rect.Left + Border;
 hjCenter: Left:=Rect.Left +Delta.X;
 hjRight: Left:=Rect.Right - (Size.X + Border);
 else  Left:=Rect.Left + Border;
 end;
case VertJust of
 vjTop: Top:=Rect.Top + Border;
 vjCenter: Top:=Rect.Top +Delta.Y;
 vjBottom: Top:=Rect.Bottom - (Size.Y + Border);
 else  Top:=Rect.Top + Border;
 end;

Canvas.TextRect(Rect,Left,Top,S);
end;


procedure OwnerDrawItem(Canvas: TCanvas; S: string; Rect: TRect);
begin
OwnerDrawItem(Canvas,S,Rect,DefHorzJust,DefVertJust);
end;

procedure OwnerDrawAlign(Canvas: TCanvas; S: string; AlignLeft: boolean; Col, Row: Longint; Rect: TRect);
begin
if AlignLeft then Canvas.TextRect(Rect,Rect.Left+3,Rect.Top,S)
else Canvas.TextRect(Rect,Rect.Right-2-Canvas.TextWidth(S),Rect.Top+2,S);
end;


procedure OwnerDrawInteger(Canvas: TCanvas; I: integer; Rect: TRect);
begin
OwnerDrawItem(Canvas,IntToStr(I),Rect);
end;


procedure OwnerDrawFloat(Canvas: TCanvas; F: double; Rect: TRect; Format: TFloatFormat; Digits: integer);
begin
OwnerDrawItem(Canvas,FloatToStrF(F,Format,18,Digits),Rect);
end;

procedure OwnerDrawFloat(Canvas: TCanvas; F: double; Rect: TRect; Digits: integer);
begin
OwnerDrawFloat(Canvas,F,Rect,ffFixed,Digits);
end;




procedure CenterOnWorkArea(Form: TForm);
{Center form on available work space}
var Rect: TRect;
var ScreenSize,FormSize,Delta,Offset: TPoint;
begin
SystemParametersInfo(SPI_GETWORKAREA, 0, @Rect, 0);
ScreenSize:=IntVectorSubtract(Rect.BottomRight,Rect.TopLeft);
FormSize:=Point(Form.Width,Form.Height);
Delta:=IntVectorSubtract(ScreenSize,FormSize);
Offset:=IntScalarDivide(Delta,2);
Form.Left:=Offset.X + Rect.Left;
Form.Top:=Offset.Y + Rect.Top;
end;



procedure ClearStringGrid(Grid: TStringGrid; ClearFixed: boolean);
var XStart,YStart: integer;
var X,Y: integer;
begin
if ClearFixed then
	begin
	XStart:=0;
	YStart:=0;
	end
else
	begin
	XStart:=Grid.FixedCols;
	YStart:=Grid.FixedRows;
	end;
for Y:=YStart to Grid.RowCount-1 do
for X:=XStart to Grid.ColCount-1 do Grid.Cells[X,Y]:='';
end;


{------------------------------------------------------------------------------}



procedure GetScrollPercent(SB: TScrollBox; var X,Y: double);
{Get scroll box position by percent }
begin
X:=SB.HorzScrollBar.Position/(SB.HorzScrollBar.Range-SB.ClientWidth);
Y:=SB.VertScrollBar.Position/(SB.VertScrollBar.Range-SB.ClientHeight);
end;


procedure SetScrollPercent(SB: TScrollBox; X,Y: double);
{Set scroll box position by percent }
begin
SB.HorzScrollBar.Position:=round(X*(SB.HorzScrollBar.Range-SB.ClientWidth));
SB.VertScrollBar.Position:=round(Y*(SB.VertScrollBar.Range-SB.ClientHeight));
end;



procedure AdjustScrollBox(SB: TScrollBox; BaseSize: TPoint; NewScale: double);
{Center scrollbar positions}
var PH,PV: double;
var Size: TPoint;
begin
Size:=IntVectorSubtract(SB.ClientRect.BottomRight,SB.ClientRect.TopLeft);

PH:=SB.HorzScrollBar.Position + (Size.X/2);
PH:=PH/SB.HorzScrollBar.Range;
PV:=SB.VertScrollBar.Position + (Size.Y/2);
PV:=PV/SB.VertScrollBar.Range;
SB.HorzScrollBar.Position:=round((BaseSize.X * PH * NewScale)-(Size.X/2));
SB.VertScrollBar.Position:=round((BaseSize.Y * PV * NewScale)-(Size.Y/2));
end;

procedure ZoomControlScrollBox(SB: TScrollBox; Image: TControl;  NewScale: double);
{Center scrollbar positions}
var BaseSize: TPoint;
var PH,PV: double;
var Size: TPoint;
begin
Size:=IntVectorSubtract(SB.ClientRect.BottomRight,SB.ClientRect.TopLeft);
BaseSize:=Point(Image.Width,Image.Height);
PH:=SB.HorzScrollBar.Position + (Size.X/2);
PH:=PH/SB.HorzScrollBar.Range;
PV:=SB.VertScrollBar.Position + (Size.Y/2);
PV:=PV/SB.VertScrollBar.Range;

Image.Width:=Trunc(Image.Width*NewScale);
Image.Height:=Trunc(Image.Height*NewScale);
SB.HorzScrollBar.Position:=round((BaseSize.X * PH * NewScale)-(Size.X/2));
SB.VertScrollBar.Position:=round((BaseSize.Y * PV * NewScale)-(Size.Y/2));
Image.Invalidate;
end;



procedure ZoomImageScrollBox(SB: TScrollBox; Image: TImage;  NewScale: double);
{Center scrollbar positions}
var BaseSize: TPoint;
var PH,PV: double;
var Size: TPoint;
begin
Size:=IntVectorSubtract(SB.ClientRect.BottomRight,SB.ClientRect.TopLeft);
BaseSize:=Point(Image.Picture.Bitmap.Width,Image.Picture.Bitmap.Height);
PH:=SB.HorzScrollBar.Position + (Size.X/2);
PH:=PH/SB.HorzScrollBar.Range;
PV:=SB.VertScrollBar.Position + (Size.Y/2);
PV:=PV/SB.VertScrollBar.Range;

Image.AutoSize:=False;
Image.Stretch:=True;
Image.Width:=Trunc(Image.Picture.Bitmap.Width*NewScale);
Image.Height:=Trunc(Image.Picture.Bitmap.Height*NewScale);
SB.HorzScrollBar.Position:=round((BaseSize.X * PH * NewScale)-(Size.X/2));
SB.VertScrollBar.Position:=round((BaseSize.Y * PV * NewScale)-(Size.Y/2));
Image.Invalidate;
end;






{-----------------------------------------------------------------------------}

var SearchPos: integer;

procedure SearchMemo(var FindDlg: TFindDialog; Memo: TMemo);
var S: string;
var startpos : integer;
begin
{If the stored position is 0 this cannot be a find next. }
if SearchPos = 0 then FindDlg.Options:= FindDlg.Options - [frFindNext];

{ Figure out where to start the search and get the corresponding text from the memo. }
if frfindNext in FindDlg.Options then
	begin
	{ This is a find next, start after the end of the last found word. }
	StartPos:= SearchPos + Length(FindDlg.Findtext);
	S:= Copy(Memo.Lines.Text, StartPos, MaxInt);
	end
else
	begin
	{ This is a find first, start at the, well, start. }
	S:=Memo.Lines.Text;
	StartPos:= 1;
	end;
{ Perform a globalsearch for FindText in S }
if frMatchCase in FindDlg.Options then SearchPos:= Pos(FindDlg.FindText, S)
else SearchPos:= PosNoCase(FindDlg.FindText, S);
if SearchPos > 0 then
	begin
	{ Found something, correct position for the location of the start of search. }
	SearchPos:= SearchPos + StartPos - 1;
	Memo.SelStart:= SearchPos - 1;
	Memo.SelLength:= Length(FindDlg.FindText);
	Memo.SetFocus;
	end
else
	begin
	S:='Could not find: '+FindDlg.FindText;
	MessageDlgPos(S, mtError, [mbOK], 0,FindDlg.Left+360,FindDlg.Top+125);
	end;
end;


procedure StartSearch(FindDlg: TFindDialog);
begin
SearchPos:=0;
FindDlg.Execute;
end;


{-----------------------------------------------------------------------------}



procedure GetFileExtensions(S: String; SL: TStringList; Def: string);
{Extract all extensions from Filter List in TOpenDialog and descendants}
var I,Inx: integer;
begin
Dsupport.ReplaceChars(S,'|',CR_Char);
SL.Text:=S;
for I:=SL.Count-1 downto 0 do
 if (I mod 2)=0 then SL.Delete(I);
for I:=0 to SL.Count-1 do
	begin
	S:=SL[I];
	Inx:=Pos(';',S);
	if Inx>0 then S:=Def
	else Delete(S,1,2);
	SL[I]:=S;
	end;
end;




procedure HandleOpenDialogChange(Dialog: TOpenDialog; Def: string; Update: boolean);
{Hand TypeChange event in TOpenDialog and descendents}
var SL: TStringList;
var S: string;
const CB_FILENAME_ID = 1148;
begin
SL:=TStringList.Create;
try
GetFileExtensions(Dialog.Filter,SL,Def);
Dialog.DefaultExt:=SL[Dialog.FilterIndex-1];
if Update then
	begin
	S:=ChangeFileExt(Dialog.FileName,'.'+Dialog.DefaultExt);
	S:=ExtractFileName(S);
	SendMessage(GetParent(Dialog.Handle),CDM_SETCONTROLTEXT, CB_FILENAME_ID, integer(pchar(S)));
	end;
finally SL.Free; end;
end;



{-----------------------------------------------------------------------------}


function FindChildControls(WinControl: TWinControl; Name: string): TControl;
{Recursively Find Child Controls}
var I: Integer;
begin
Result:=nil;
for I := 0 to WinControl.ControlCount - 1 do
	begin
	if WinControl.Controls[I].Name = Name then
		begin
		Result:=WinControl.Controls[I];
		break;
		end;
	if WinControl.Controls[I] is TWinControl then
		begin
		Result:=FindChildControls(TWinControl(WinControl.Controls[I]),Name);
		if Result<>nil then break;
		end;
	end;
end;


procedure EnableChildControls(Control: TWinControl; Enable: boolean);
{Enable/Disable child control of specified Control}
var I: Integer;
var ChildControl: TControl;
begin
for I:= 0 to Control.ControlCount -1 do
	begin
	ChildControl := Control.Controls[I];
	{Handle subcontrols of other WinControls by recursing}
	if ChildControl is TWinControl then EnableChildControls(TWinControl(ChildControl),Enable);
	ChildControl.Enabled:=Enable;
	end;
end;



procedure EnableRadioItem(RG: TRadioGroup; Inx: integer; Enable: boolean);
{Enable/Disable a Radio Group Item}
begin
if (Inx<0) or (Inx>=RG.ControlCount) then exit;
TRadioButton(RG.Controls[Inx]).Enabled:=Enable;
end;



procedure ShowRadioItem(RG: TRadioGroup; Inx: integer; Show: boolean);
{Enable/Disable a Radio Group Item}
begin
if (Inx<0) or (Inx>=RG.ControlCount) then exit;
TRadioButton(RG.Controls[Inx]).Visible:=Show;
end;



function IsRadioItemAccessable(RG: TRadioGroup; Inx: integer): boolean;
{Enable/Disable a Radio Group Item}
begin
Result:=False;
if (Inx<0) or (Inx>=RG.ControlCount) then exit;
Result:=TRadioButton(RG.Controls[Inx]).Visible and TRadioButton(RG.Controls[Inx]).Enabled;
end;





{-------------- Tree Search Routines ------------------------------------------}

procedure SearchTree(Tree: TTreeView; FindDialog: TFindDialog);
{Searh Tree according to parameters specified in FindDialog}
var Inx,Found: integer;
var Target: string;
var WholeWord,MatchCase: boolean;


	function Compare(SubStr,S: string): boolean;
	var SO: TStringSearchOptions;
	begin
	SO:=[soDown];
	if MatchCase then SO:=SO+[soMatchCase];
	if WholeWord then SO:=SO+[soWholeWord];
	Result:=SearchBuf(PChar(S), Length(S), 0, 0, SubStr, SO)<>nil;
	end;



	function ForwardSearchTree(Tree: TTreeView; Target: string; Start: integer): integer;
	{Search forward from the specified "Start" location}
	var S: string;
	begin
	for Result:=Start to Tree.Items.Count-1 do
		begin
		S:=Tree.Items.Item[Result].Text;
		if Compare(Target,S) then exit;
		end;
	Result:=0;
	end;



	function BackwardSearchTree(Tree: TTreeView; Target: string; Start: integer): integer;
	{Search backward from specified "Start" location}
	var S: string;
	begin
	for Result:=Start-1 downto 0 do
		begin
		S:=Tree.Items.Item[Result].Text;
		if Compare(Target,S) then exit;
		end;
	Result:=0;
	end;


begin
Inx:=Tree.Selected.AbsoluteIndex;
Target:=FindDialog.FindText;
WholeWord:=frWholeWord in FindDialog.Options;
MatchCase:=frMatchCase in FindDialog.Options;
if frDown in FindDialog.Options then Found:=ForwardSearchTree(Tree,Target,Inx+1)
else Found:=BackwardSearchTree(Tree,Target,Inx);
if Found>0 then Tree.Selected:=Tree.Items.Item[Found];
end;


{--------------- TListBox Routines -------------------------------------------}



function MoveListItemUp(List: TListBox): boolean;
{Move List Item up one slot}
var I: integer;
begin
Result:=False;
I:=List.ItemIndex;
if I>0 then
	begin
	List.Items.Exchange(I,I-1);
	List.ItemIndex:=I-1;
	List.Selected[List.ItemIndex]:=True;
	Result:=True;
	end;
end;



function MoveListItemDown(List: TListBox): boolean;
{Move List Item down one slot}
var I: integer;
begin
Result:=False;
I:=List.ItemIndex;
if (I>-1) and (I<List.Items.Count-1) then
	begin
	List.Items.Exchange(I,I+1);
	List.ItemIndex:=I+1;
	List.Selected[List.ItemIndex]:=True;
	Result:=True;
	end;
end;


function DeleteSelectedList(List: TListBox): boolean;
{Delete Selected List Item}
var I: integer;
begin
Result:=False;
I:=List.ItemIndex;
if (I>-1) and (I<List.Items.Count) then
	begin
	Result:=True;
	List.Items.Delete(I);
	if List.Items.Count<1 then exit;
	if (I>=List.Items.Count) then List.ItemIndex:=I-1
	else if I<0 then List.ItemIndex:=I+1
	else List.ItemIndex:=I;
	end;
end;



procedure SetAllSelected(List: TListBox; Value: boolean);
{Set All Selected List Items to speficied value}
var I: integer;
begin
for I:=0 to List.Count-1 do
	begin
	List.Selected[I]:=Value;
	end;
end;



procedure ClearAllSelected(List: TListBox);
{clear all Selected List Items}
begin
SetAllSelected(List,False);
end;




function ListBoxDragOver(Sender, Source: TObject; List: TListBox; P: TPoint): boolean;
{Test if we are dragging over a good drop target}
var Inx1,Inx2: integer;
begin
Result := False;
if (Source=List) then
	begin
	Inx1:=List.ItemIndex;
	Inx2:=List.ItemAtPos(P,True);
	if Inx1<>Inx2 then Result:= True;
	end;
end;




procedure ListBoxDrop(Sender,Source: TObject; List: TListBox; P: TPoint);
{Handle inserting item after drop}
var Inx1,Inx2: integer;
begin
if Sender<>Source then exit;
Inx1:=List.ItemIndex;
Inx2:=List.ItemAtPos(P,True);
if (Inx1<0) or (Inx2<0) then exit;
List.Items.Move(Inx1,Inx2);
List.ItemIndex:=Inx2;
end;



{----------------- TStrings Routines ------------------------------------------}


procedure FreeObjects(const Strings: TStrings) ;
var Inx: integer;
begin
for Inx:= 0 to strings.Count-1 do
	begin
	Strings.Objects[Inx].Free;
	Strings.Objects[Inx]:= nil;
	end;
end;


function CaseSensitiveIndexOf(const Strings: TStrings; const S: string): Integer;
{Case sensitive IndexOf function}
begin
for Result:=0 to Strings.Count-1 do
 if Strings[Result]=S then Exit;
Result := -1;
end;


{------------------------------------------------------------------------------}

procedure SetMinimumFormHeight(Form: TForm);
{Note: doesn't work if any controls are client aligned}
{Because they go to fill up all the space}
var I: Integer;
var ChildControl: TControl;
var Lowest,L: integer;
begin
Lowest:=0;
for I:= 0 to Form.ControlCount -1 do
	begin
	ChildControl := Form.Controls[I];
	if not ChildControl.Visible then continue;
	L:=ChildControl.Top+ChildControl.Height;
	if L>Lowest then Lowest:=L;
	end;
Form.ClientHeight:=Lowest+10;
end;



{------------------------------------------------------------------------------}



function TSectionList.Add(const S: string): integer;
begin
AddObject(S,TStringList.Create);
end;



procedure TSectionList.Clear;
var I: integer;
begin
for I:=0 to Count-1 do TStringList(Objects[I]).Free;
inherited Clear;
end;


procedure TSectionList.Delete(Index: Integer);
begin
TStringList(Objects[Index]).Free;
inherited Delete(Index);
end;



destructor TSectionList.Destroy;
begin
Clear;
inherited;
end;



function TSectionList.GetSection(Index: Integer): TStringList;
begin
Result:=TStringList(Objects[Index]);
end;



{ TAutoString }

function TAutoString.Add(const S: string): Integer;
begin
Result:=inherited Add(S);
Objects[Result]:=ObjectClass.Create;
end;



procedure TAutoString.Clear;
var I: integer;
begin
for I:=0 to Count-1 do Objects[I].Free;
inherited;
end;

constructor TAutoString.Create(ObjClass: TClass);
begin
inherited Create;
ObjectClass:=ObjClass;
end;



destructor TAutoString.Destroy;
begin
Clear;
inherited;
end;


procedure TAutoString.Delete(Index: Integer);
begin
Objects[Index].Free;
inherited;
end;


{------------------------------------------------------------------------------}


procedure AppendStringComma(var Destination: string; Source: string);
begin
if Destination<>'' then Destination:=Destination+',';
Destination:=Destination+Source;
end;



function GetFontStyleString(Style: TFontStyles): string;
begin
Result:='';
if fsBold in Style then AppendStringComma(Result,'fsBold');
if fsItalic in Style then AppendStringComma(Result,'fsItalic');
if fsUnderline in Style then AppendStringComma(Result,'fsUnderline');
if fsStrikeOut in Style then AppendStringComma(Result,'fsStrikeOut');
end;



function StringToFontStyle(Style: string): TFontStyles;
begin
Result:=[];
if Pos('fsBold',Style)>0 then Result:=Result + [fsBold];
if Pos('fsItalic',Style)>0 then Result:=Result + [fsItalic];
if Pos('fsUnderline',Style)>0 then Result:=Result + [fsUnderline];
if Pos('fsStrikeOut',Style)>0 then Result:=Result + [fsStrikeOut];
end;


procedure ScrollMemoToEnd(Memo: TMemo);
begin
SendMessage(Memo.Handle, EM_LINESCROLL, 0,Memo.Lines.Count);
end;


procedure SetMemoLine(Memo: TMemo; LineNo: integer);
{Set memo so specified line is on the screen}
var Inx: integer;
begin
Inx:= Memo.Perform(EM_LINEINDEX, LineNo, 0);
Memo.SelLength:=0;
Memo.SelStart := Inx;
Memo.Perform(EM_SCROLLCARET, 0, 0);
Memo.Invalidate;
end;



function GetMemoLine(Memo: TMemo): integer;
{Get the line number of caret position}
begin
Result:=Memo.Perform(EM_LINEFROMCHAR, Memo.SelStart, 0);
end;



procedure SetMemoFirstLine(Memo: TMemo; LineNo: integer);
{Set the first line of memo to the specified positon - irrespective of caret post}
var FirstLine: integer;
begin
FirstLine:=Memo.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
Memo.Perform(EM_LineScroll, 0, LineNo - FirstLine);
end;



function GetMemoFirstLine(Memo: TMemo): integer;
{Vet the postion of the first line of the memor - irrespective of caret post}
begin
Result:=Memo.Perform(EM_GETFIRSTVISIBLELINE, 0, 0);
end;



{-----------------------------------------------------------------------------}


procedure RichEditSearch(Memo: TRichEdit; FindDialog: TFindDialog);
{Routine to handle TRichEdit Search using TFindDialog}
var Flags,LineNum: Integer;
var FS : string;
var FoundAt : Integer;
var FT : TFindText;

	function FindForward: integer;
	begin
	if Memo.SelLength <> 0 then FT.chrg.cpMin:=Memo.SelStart + Memo.SelLength else FT.chrg.cpMin := 0;
	FT.chrg.cpMax := Length(Memo.Text);
	Result:=Memo.Perform(EM_FindText,Flags,integer(@FT));
	end;


	function FindBackward: integer;
	begin
	FT.chrg.cpMax:=Memo.SelStart;
	FT.chrg.cpMin:=Memo.SelStart - Length(FS);

	Result := -1;
	while (FT.chrg.cpMin > -1) and (Result = -1) do
		begin
		Result:=Memo.Perform(EM_FindText,Flags,integer(@FT));
		Dec(FT.chrg.cpMin);
		end;
	end;


begin
FS := FindDialog.FindText;
FT.lpstrText := PChar(fs);

Flags:=0;
if frWholeWord in FindDialog.Options then Flags:=Flags or FR_WHOLEWORD;
If frMatchCase in FindDialog.Options then Flags:=Flags or FR_MATCHCASE;

if frDown in FindDialog.Options then FoundAt := FindForward
else FoundAt := FindBackward;
if FoundAt > -1 then                                 
	begin
	Memo.SetFocus;
	Memo.SelStart := FoundAt;
	Memo.SelLength := length(FS);
	{The version of EM_SCROLLCARET in Messages is correct}
	{The version of EM_SCROLLCARET in RichEdit is wrong}
	Memo.Perform(Messages.EM_SCROLLCARET,0,0);
	end
else ShowMessage(fs+' Not Found');
Memo.SetFocus;
end;




procedure AutoSizeRadio(Radio: TRadioGroup);
var Cnt,H: integer;
var Canvas: TCanvas;
begin
Canvas:=TCanvas.Create;
try
Canvas.Handle:=GetDC(Radio.Handle);
H:=Canvas.TextHeight('ABC');
Cnt := (Radio.Items.Count + Radio.Columns - 1) div Radio.Columns;
Radio.Height:=H + Cnt * MulDiv(H,6,5);
finally Canvas.Free; end;
end;



procedure LimitedMemoAdd(S: string; Memo: TMemo; Limit: integer);
{Add string to TMemo but limit to a maximum number of items}
{Uses intermediate TStringList hecause of a memory leak in Windows/VCL that}
{ causes memory useage to expand when the deleting items from start of list}
var SL: TStringList;
begin
SL:=TStringList.Create;
try
SL.Assign(Memo.Lines);
SL.Add(S);
while SL.Count>=Limit do SL.Delete(0);
Memo.Text:=SL.Text;
finally SL.Free; end;
end;



procedure CenterTrackBar(TrackBar: TTrackBar);
var E: TNotifyEvent;
begin
E:=TrackBar.OnChange;
TrackBar.OnChange:=nil;
try
TrackBar.Position:=(TrackBar.Max + TrackBar.Min) div 2;
finally TrackBar.OnChange:=E; end;
end;


{-------------------------------------------------------------------------------}

var MouseUpEvent, LastState: boolean;

procedure SelectiveRowChange(Grid: TStringGrid; ACol, ARow: Integer; Validate: TValidateFunc; var CanSelect: Boolean);
{Handles Row change by moving to the next valid row}
{Call from the SelectCell event and pass a Validate }
var Down: boolean;


	function IsLButtonDown: boolean;
	{Detect if the Mouse is down}
	begin
	{Test if user has swapped right/left buttons and get left mouse state}
	if boolean(GetSystemMetrics(SM_SWAPBUTTON)) then
		Result:=(GetAsyncKeyState(VK_RBUTTON) and $8000)<>0
	else Result:=(GetAsyncKeyState(VK_LBUTTON) and $8000)<>0;
	{Detect if there was a MouseUp event}
	MouseUpEvent:=not Result and LastState;
	LastState:=Result;
	end;


	procedure FindNextDown;
	{Search down for next available cell}
	var I: integer;
	begin
	for I:=ARow+1 to Grid.RowCount-1 do
	 if Validate(I) then
		begin
		Grid.Row:=I;
		exit;
		end;
	end;

	procedure FindNextUp;
	{Search up for next available cell}
	var I: integer;
	begin
	for I:=ARow-1 downto 1 do
	 if Validate(I) then
		begin
		Grid.Row:=I;
		exit;
		end;
	end;


begin
{Is the new Row valid?}
CanSelect:=Validate(ARow);
{Exit if valid}
if CanSelect then exit;
{We only care about arrow-key movments}
{So exit if Left Button down or the was just release}
if IsLButtonDown or MouseUpEvent then exit;
{Get the direction of the movement}
Down:=ARow>=Grid.Row;
{Move to next valid row in the proper direction}
if Down then FindNextDown
else FindNextUp;
end;



function GetGridCells(Grid: TStringGrid): string;
{Return all string grid cells, tab separated}
var X,Y: integer;
begin
Result:='';
for Y:=0 to Grid.RowCount-1 do
	begin
	for X:=0 to Grid.ColCount-1 do
		begin
		if X>0 then Result:=Result+Tab_Char;
		Result:=Result+Grid.Cells[X,Y];
		end;
	Result:=Result+CRLF_Char;
	end;
end;



procedure CopyStringGrid(Grid: TStringGrid);
{Copy all cells of string grid to clipboard}
var S: string;
begin
S:=GetGridCells(Grid);
ClipBoard.Open;
ClipBoard.AsText:=S;
ClipBoard.Close;
end;




end.
