unit Vector;

{Routines to handle vector math}

interface

uses Types,Graphics,IniFiles, SysUtils ,Math, DSupport;


{Vector, rectangle, line and triangle structures}

type T3DVector=packed record
  X,Y,Z: double;
  end;
type P3DVector = ^T3DVector;


type T2DVector=packed record
  X,Y: double;
  end;
type P2DVector = ^T2DVector;


type T2DRect = packed record
 case Integer of
  0: (Left, Top, Right, Bottom: double);
  1: (TopLeft, BottomRight: T2DVector);
  end;


type T2DLine = packed record
 P1,P2: T2DVector;
 end;


type T3DLine = packed record
 P1,P2: T3DVector;
 end;

type P3DLine = ^T3DLine;


type TTriangle = packed record
P1,P2,P3: T3DVector;
end;

type PTriangle = ^TTriangle;



type T3DRectangle = record
 P1,P2,P3,P4: T3DVector;
 end;


type T2DVectorArray = array of T2DVector;
type T3DVectorArray = array of T3DVector;
type T2DLineArray = array of T2DLine;

type T2DPolygon = array of T2DVector;

const ZeroPoint3D: T3DVector = (X:0; Y:0; Z:0);


{3D Plane object defined by azimuth and inclination vector}

type
 TPlane = class(TObject)
 private
  Con: T3DVector;
 protected
 public
  Azm,Dip: double;
  procedure SetPlane(Azm,Dip: double);
  function GetPlaneZ(X,Y: double): double;
  procedure GetAzmDip(var A,D: double);
  procedure SetEquation(V: T3DVector);
  function GetEquation: T3DVector;
  constructor Create;
  property Equation: T3DVector read GetEquation write SetEquation;
 end;

{ 3D Functions }

function DotProduct3D(const V1,V2: T3DVector): double;
function CrossProduct3D(const V1,V2: T3DVector): T3DVector;
function RightCrossProduct3D(const V1,V2: T3DVector): T3DVector;
function VectorABS3D(const V: T3DVector): double;
function UnitVector3D(const V: T3DVector): T3DVector;
function VectorAngle3D(const V1,V2: T3DVector): double;
function PolarToVector3D(const Point: T3DVector; const Angle1,Angle2,Length: double): T3DVector;
procedure VectorToPolar3D(P1,P2: T3DVector; var Azimuth,Inclination,Length: double);
function ScalarAdd3D(const V: T3DVector; const S: double): T3DVector;
function ScalarSubtract3D(const V: T3DVector; const S: double): T3DVector;
function ScalarProduct3D(const V: T3DVector; const S: double): T3DVector;
function VectorMultiply3D(const V1,V2: T3DVector): T3DVector;
function VectorSqrRoot3D(const V: T3DVector): T3DVector;
function VectorFraction3D(const V: T3DVector): T3DVector;
function VectorSquare3D(const V: T3DVector): T3DVector;
function VectorDivide3D(const V1,V2: T3DVector): T3DVector;
function ScalarDivide3D(const V: T3DVector; const S: double): T3DVector;
function VectorSubtract3D(const V1,V2: T3DVector): T3DVector;
function VectorAdd3D(const V1,V2: T3DVector): T3DVector;
function InvertVector3D(const V: T3DVector): T3DVector;
function MakeVector3D(const X,Y,Z: double): T3DVector; overload;
function MakeVector3D(const V: T2DVector; const Z: double): T3DVector; overload;
function MakeVector3DY(const V: T2DVector): T3DVector;
function RandomVector3D(const Range: T3DVector): T3DVector;
function GetMinimumVector(const V1,V2: T3DVector): T3DVector;
function GetMaximumVector(const V1,V2: T3DVector): T3DVector;
function RangeCheckVectorMin(const V: T3DVector; const Min: double): T3DVector;  overload;
function RangeCheckVectorMin(const V,Min: T3DVector): T3DVector; overload;
function LineLength3D(const A, B: T3DVector): double; overload;
function LineLength3D(const L: T3DLine): double; overload;
function NegateVector3D(const V: T3DVector): T3DVector;
function AreEqual3D(const V1,V2: T3DVector): boolean;
function VectorNearestMultiple3D(const V: T3DVector; const N: double; const Low: boolean): T3DVector; overload;
function VectorNearestMultiple3D(const V,N: T3DVector; const Low: boolean): T3DVector; overload;
procedure RotatePoint3D(var P: T3DVector; const Center: T3DVector; const Yaw,Pitch: double); overload;
procedure RotatePoint3D(var P: T3DVector; const Center: T3DVector; const Pitch,Yaw,Roll: double); overload;
procedure VectorToUpRight(Dir: T3DVector; var Up,Right: T3DVector);

function CardinalBasisVector(V: T3DVector): T3DVector;
procedure NormalToPlaneVectors(Normal: T3DVector; var V1,V2: T3DVector);

function InterpolateVector3D(V1,V2: T3DVector; D: double): T3DVector;

function RangeCheckVectorMax(const V,Max: T3DVector): T3DVector; overload;
function RangeCheckVectorMax(const V: T3DVector; const Max: double): T3DVector;  overload;
procedure ExchangeVectors3D(var V1,V2: T3DVector);

function PointToVector3D(const P: TPoint): T3DVector;
function Vector3DToPoint(const V: T3DVector): TPoint;
function Vector3DtoPointZ(const V: T3DVector; const UseY: boolean): TPoint;
function Vector3DTo2D(const V: T3DVector): T2DVector;
function Vector3Dto2DZ(const V: T3DVector; const UseY: boolean): T2DVector;

function Vector3DToString(const V: T3DVector; const Digits: integer): string; overload;
function Vector3DToString(const V: T3DVector; const Digits: integer; const Separator: char): string; overload;
function Vector3DToString(const V: T3DVector): string; overload;



{ 2D Functions }

function DotProduct2D(const V1,V2: T2DVector): double;
function CrossProduct2D(const V1,V2: T2DVector): T3DVector;
function VectorABS2D(const V: T2DVector): double;
function UnitVector2D(const V: T2DVector): T2DVector;
function NegateVector2D(const V: T2DVector): T2DVector;
function VectorAngle2D(const V1,V2: T2DVector): double;
function SignedVectorAngle2D(const V1,V2: T2DVector): double;
function ScalarAdd2D(const V: T2DVector; const S: double): T2DVector;
function ScalarSubtract2D(const V: T2DVector; const S: double): T2DVector;
function ScalarProduct2D(const V: T2DVector; const S: double): T2DVector;
function ScalarDivide2D(const V: T2DVector; const S: double): T2DVector;
function VectorSubtract2D(const V1,V2: T2DVector): T2DVector;
function VectorAdd2D(const V1,V2: T2DVector): T2DVector;
function VectorDivide2D(const V,S: T2DVector): T2DVector;
function VectorMultiply2D(const V,S: T2DVector): T2DVector;
function VectorTripleMult2D(const V1,V2,V3: T2DVector): T2DVector;
function MakeVector2D(const X,Y: double): T2DVector;
function VectorSign2D(const V: T2DVector): T2DVector;
function UnsignVector2D(const V: T2DVector): T2DVector;
function VectorSquare2D(const V: T2DVector): T2DVector;
function VectorFraction2D(const V: T2DVector): T2DVector;
function VectorSquareRoot2D(const V: T2DVector): T2DVector;
function GetMinimumVector2D(const V1,V2: T2DVector): T2DVector;
function GetMaximumVector2D(const V1,V2: T2DVector): T2DVector;
function Vector2DToString(const V: T2DVector; const Digits: integer; const Labeled: boolean): string; overload;
function Vector2DToString(const V: T2DVector): string; overload;
function Vector2DToPoint(const V: T2DVector): TPoint;
function Vector2DToVector3D(const V: T2DVector): T3DVector; overload;
function Vector2DToVector3D(const V: T2DVector; const Z: double): T3DVector; overload;
function Vector2DToVector3DZ(const V: T2DVector): T3DVector;
function PointToVector2D(const P: TPoint): T2DVector;
procedure ExchangeVectors2D(var V1,V2: T2DVector);
procedure RotatePoint2D(var P: T2DVector; Center: T2DVector; Angle: double);
function PolarToVector2D(const Point: T2DVector; Angle,Length: double): T2DVector;
function CompassToVector2D(const Point: T2DVector; Angle,Length: double): T2DVector;
function AreEqual2D(const V1,V2: T2DVector): boolean;
function VectorNearestMultiple2D(const V: T2DVector; const N: double; const Low: boolean): T2DVector; overload;
function VectorNearestMultiple2D(const V,N: T2DVector; const Low: boolean): T2DVector; overload;
function GetNormal2D(const V: T2DVector): T2DVector;
function GetUnitNormal2D(const Line: T2DLine): T2DVector; overload;
function GetUnitNormal2D(const V: T2DVector): T2DVector; overload;
procedure WriteVector2DIni(Ini: TIniFile;const Section, Ident: string;  const Value: T2DVector);
function ReadVector2DIni(const Ini: TIniFile; const Section, Ident: string; const Default: T2DVector): T2DVector;
function OffsetVector2D(V: T2DVector; X,Y: double): T2DVector;
function InterpolateVector2D(V1,V2: T2DVector; D: double): T2DVector;
function BilinearInterpolate2D(X,Y,Width,Height: integer; NW,NE,SW,SE: T2DVector): T2DVector;




{Routines for 2D rectangles}

function Rect2DToRect(const R: T2DRect): TRect;
procedure OffsetRect2D(var Rect: T2DRect;  const Offset: T2DVector);
function GetOffsetRect2D(Rect: T2DRect; const Offset: T2DVector): T2DRect;
procedure InflateRect2D(var Rect: T2DRect; const Delta: T2DVector);
function MakeRect2D(const X1,Y1,X2,Y2: double): T2DRect;  overload;
function MakeRect2D(const TopLeft,BottomRight: T2DVector): T2DRect; overload;
procedure ExpandRect2D(var R: T2DRect; A: T2DVector);
procedure RotateRectangleCenter2D(var Rect: T2DRect; Center: T2DVector; Angle: double);
function RectToRect2D(const Rect: TRect): T2DRect;
function MaxRotatedRect2D(const Rect: T2DRect; const Center: T2DVector; const Angle: double): T2DRect;
function PointInRect2D(const P: T2DVector; R: T2DRect): boolean;
function DoRectsOverlap2D(R1,R2: T2DRect): boolean;

{Routines for 2D Lines}

function Make2DLine(const P1,P2: TPoint): T2DLine; overload;
function Make2DLine(const P1,P2: T2DVector): T2DLine; overload;
function OffsetLine2D(Line: T2DLine; X,Y: double): T2DLine;
function PolarToLine2D(const Point: T2DVector; const Angle,Length: double): T2DLine;
function CompassToLine2D(const Point: T2DVector; Angle,Length: double): T2DLine;
procedure RotateLine2D(var Line: T2DLine; Center: T2DVector; Angle: double);
function IntersectLines2D(L1,L2: T2DLine; var Point: T2DVector): Boolean;
function DoLinesIntersect2D(L1,L2: T2DLine; var Point: T2DVector): boolean; overload;
function DoLinesIntersect2D(L1,L2: T2DLine): boolean; overload;
function LineLength2D(const P1,P2: T2DVector) : double; overload;
function LineLength2D(const L: T2DLine) : double; overload;
function LineCenter2D(const L: T2DLine): T2DVector;
procedure ForceIntersect2D(var L1,L2: T2DLine);
function ExtendLine2D(const L1: T2DLine; const Len: double): T2DVector; overload;
function ExtendLine2D(const P1,P2: T2DVector; const Len: double): T2DVector; overload;
function ChangeLineLength2D(var Line: T2DLine; const Len: double): T2DLine;
function RightTurn2D(const L1, L2: T2DLine): Boolean;
procedure DrawLine2D(Canvas: TCanvas; L: T2DLine; C: TColor);

function PointLineExtDist2D(const Point,LineStart,LineEnd: T2DVector; var Intercept: T2DVector): double; overload;
function PointLineExtDist2D(const Point: T2DVector; const Line: T2DLine; var Intercept: T2DVector): double; overload;
function PointLineExtDist2D(const Point: T2DVector; const Line: T2DLine): double; overload;
function PointLineSegDist2D(const Point: T2DVector; const Line: T2DLine; var Distance: double; var Intercept: T2DVector): boolean; overload;
function PointLineSegDist2D(const Point: T2DVector; const Line: T2DLine): double; overload;
function LineSegLineSegDist2D(S1,S2: T2DLine): double;

procedure WrapLine2D(ClipWin: T2DRect; Line: T2DLine; var WrappedLines: T2DLineArray; WrapHorizontal,WrapVertical: boolean);
function WrapPoint2D(ClipWin: T2DRect; V: T2DVector; WrapHorizontal,WrapVertical: boolean): T2DVector;

{2D Figures}

function CircleIntersect(Center1,Center2: T2DVector; Radius1,Radius2: double; var Intersect1,Intersect2: T2DVector): boolean;
function IntersectCenteredEllipse(A,B: double; P: T2DVector): T2DVector;
function IntersectEllipse(R: T2DRect; P: T2DVector): T2DVector;


{ 3D Line Functions}

function GetDirectedVector3D(const P1,P2: T3DVector; const Length: double): T3DVector; overload;
function GetDirectedVector3D(const Line: T3DLine; const Len: double): T3DVector; overload;
function ExtendLine3D(const Line: T3DLine; const Len: double): T3DVector; overload;
function ExtendLine3D(const P1,P2: T3DVector; const Len: double): T3DVector; overload;


procedure TrimStartLine3D(var L: T3DLine; const Amount: double);
procedure TrimEndLine3D(var L: T3DLine; const Amount: double);
function MakeLine3D(const P1,P2: T3DVector): T3DLine;
function Line2DToLine3D(L: T2DLine): T3DLine;

function PointLineExtDist3D(const Point,LineStart,LineEnd: T3DVector; var Intercept: T3DVector): double; overload;
function PointLineExtDist3D(const Point: T3DVector; const Line: T3DLine; var Intercept: T3DVector): double; overload;
function PointLineSegDist3D(const Point,LineStart,LineEnd: T3DVector; var Distance: double; var Intercept: T3DVector): boolean; overload;
function PointLineSegDist3D(const Point: T3DVector; const Line: T3DLine; var Distance: double; var Intercept: T3DVector): boolean; overload;
function LineSegLineSegDist3D(S1,S2: T3DLine): double;

{ Triangle Functions }

function MakeTriangle(const P1,P2,P3: T3DVector): TTriangle;
function GetTriangleNormal(const Tri: TTriangle): T3DVector; overload;
function GetTriangleNormal(const P1,P2,P3: T3DVector): T3DVector; overload;
function ReverseWinding(const T: TTriangle): TTriangle;
function RayTriangleIntersect(const Orig, Dir,  Vert0, Vert1, Vert2: T3DVector; const CullFaces: boolean; var T, U, V: double): boolean;  overload;
function RayTriangleIntersect(const Ray: T3DLine; const Triangle: TTriangle; const CullFaces: boolean): boolean; overload;
function RayTriangleIntersect(const Ray: T3DLine; Triangle: TTriangle; const CullFaces: boolean; var Intercept: T3DVector): boolean; overload;
function BaryCentricInterpolation(var T: TTriangle; const U,V: double): T3DVector;
function GetTriangleArea(const T: TTriangle): double;
function GetTriangleCenter(T: TTriangle): T3DVector;
function GetTrianglePerimeter(T: TTriangle): double;
function GetLongestTriangleEdge(T: TTriangle): double;
function IsFacingTriangle(T: TTriangle): boolean;

function MakeRectangle(const P1,P2,P3,P4: T3DVector): T3DRectangle;


implementation

uses GLMatrix,GLQuatLib;

{------------ 2D Vector math routines --------------------}


function DotProduct2D(const V1,V2: T2DVector): double;
{Do dot product of two vectors}
begin
Result:=(V1.X * V2.X) + (V1.Y * V2.Y);
end;


function VectorAdd2D(const V1,V2: T2DVector): T2DVector;
{Add V1 and V2}
begin
Result.X:= V1.X + V2.X;
Result.Y:= V1.Y + V2.Y;
end;


function VectorSubtract2D(const V1,V2: T2DVector): T2DVector;
{Subtract V2 from V1}
begin
Result.X:= V1.X - V2.X;
Result.Y:= V1.Y - V2.Y;
end;

function CrossProduct2D(const V1,V2: T2DVector): T3DVector;
{Do crosss product of two vectors}
begin
Result.X:=0;
Result.Y:=0;
Result.Z:= ((V1.X * V2.Y) - (V1.Y * V2.X));
end;

function ScalarAdd2D(const V: T2DVector; const S: double): T2DVector;
{Add scalar to vector }
begin
Result.X:=V.X + S;
Result.Y:=V.Y + S;
end;



function ScalarSubtract2D(const V: T2DVector; const S: double): T2DVector;
{Subtract scalar from vector }
begin
Result.X:=V.X - S;
Result.Y:=V.Y - S;
end;


function ScalarProduct2D(const V: T2DVector; const S: double): T2DVector;
{Multiply vector by scalar}
begin
Result.X:=V.X * S;
Result.Y:=V.Y * S;
end;


function ScalarDivide2D(const V: T2DVector; const S: double): T2DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S;
Result.Y:=V.Y / S;
end;


function VectorDivide2D(const V,S: T2DVector): T2DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S.X;
Result.Y:=V.Y / S.Y;
end;


function VectorMultiply2D(const V,S: T2DVector): T2DVector;
{Multiply vector by vector}
begin
Result.X:=V.X * S.X;
Result.Y:=V.Y * S.Y;
end;


function VectorTripleMult2D(const V1,V2,V3: T2DVector): T2DVector;
{Multiply three vectors by each other}
begin
Result:=VectorMultiply2D(VectorMultiply2D(V1,V2),V3);
end;




function VectorABS2D(const V: T2DVector): double;
{Find ABS of vector}
begin
Result:=Sqrt(Sqr(V.X) + Sqr(V.Y));
end;


function UnitVector2D(const V: T2DVector): T2DVector;
{Return unit vector}
var L: double;
begin
L:=VectorABS2D(V);
if L=0.0 then L:=1E-99;
Result.X:=V.X / L;
Result.Y:=V.Y / L;
end;


function NegateVector2D(const V: T2DVector): T2DVector;
{Return negation of vector}
begin
Result.X:=-V.X;
Result.Y:=-V.Y;
end;


function VectorSquare2D(const V: T2DVector): T2DVector;
{Return square of vector}
begin
Result:=VectorMultiply2D(V,V);
end;


function VectorSquareRoot2D(const V: T2DVector): T2DVector;
{Return square of vector}
begin
Result.X:=Sqrt(V.X);
Result.Y:=Sqrt(V.Y);
end;


function VectorFraction2D(const V: T2DVector): T2DVector;
{Return fractional part of vector components}
begin
Result.X:=Frac(V.X);
Result.Y:=Frac(V.Y);
end;


function VectorAngle2D(const V1,V2: T2DVector): double;
{Return angle of two vectors}
var T,D: double;
begin
T:=VectorABS2D(V1)*VectorABS2D(V2);
if T=0 then Result:=0
else
	begin
	D:=DotProduct2D(V1,V2) / T;
	if abs(D)>1 then Result:=0
	else Result:=ArcCos(D);
	end;
end;



function SignedVectorAngle2D(const V1,V2: T2DVector): double;
{Return signed angle between the two vectors}
var PerpDot: double;
begin
PerpDot:= v1.X * v2.Y - v1.Y * v2.X;
Result:=Arctan2(perpDot, DotProduct2D(V1, V2));
end;


function MakeVector2D(const X,Y: double): T2DVector;
{Create 2D Vector from X and Y}
begin
Result.X:=X;
Result.Y:=Y;
end;



function VectorSign2D(const V: T2DVector): T2DVector;
{Returns -1 or +1 depending on whether the value is positive or negative}
begin
Result.X:=PickFloat(V.X>=0,1,-1);
Result.Y:=PickFloat(V.Y>=0,1,-1);
end;


function UnsignVector2D(const V: T2DVector): T2DVector;
{Returns the abs of the X,Y components of the the vector}
begin
Result.X:=abs(V.X);
Result.Y:=abs(V.Y);
end;


function Vector2DToPoint(const V: T2DVector): TPoint;
{Convert 2D Vector to a point}
begin
Result.X:=round(V.X);
Result.Y:=round(V.Y);
end;



function Vector2DToVector3D(const V: T2DVector): T3DVector;
{Convert 2D Vector to a 3D Vector}
begin
Result.X:=V.X;
Result.Y:=V.Y;
Result.Z:=0;
end;


function Vector2DToVector3D(const V: T2DVector; const Z: double): T3DVector;
{Convert 2D Vector to a 3D Vector}
begin
Result.X:=V.X;
Result.Y:=V.Y;
Result.Z:=Z;
end;



function Vector2DToVector3DZ(const V: T2DVector): T3DVector;
{Convert 2D Vector to a 3D Vector using Y as the Z value}
begin
Result.X:=V.X;
Result.Y:=V.Y;
Result.Z:=V.Y;
end;





function PointToVector2D(const P: TPoint): T2DVector;
{Convertpoint to a 2D Vector }
begin
Result.X:=P.X;
Result.Y:=P.Y;
end;


function Vector3Dto2DZ(const V: T3DVector; const UseY: boolean): T2DVector;
{Convert 3D coordinate to 2D}
begin
Result.X:=V.X;
if UseY then Result.Y:=V.Y else Result.Y:=V.Z;
end;


function Vector3DtoPointZ(const V: T3DVector; const UseY: boolean): TPoint;
{Convert 3D coordinate to 2D}
begin
Result.X:=round(V.X);
if UseY then Result.Y:=round(V.Y) else Result.Y:=round(V.Z);
end;



function Vector3DTo2D(const V: T3DVector): T2DVector;
{Convert 3D coordinate to 2D}
begin
Result.X:=V.X;
Result.Y:=V.Y;
end;



procedure ExchangeVectors2D(var V1,V2: T2DVector);
{Swap Vectors V1 and V2}
var T: T2DVector;
begin
T:=V1; V1:=V2; V2:=T;
end;


function GetMinimumVector2D(const V1,V2: T2DVector): T2DVector;
{Return minimum components of two vectors}
begin
Result.X:=Min(V1.X,V2.X);
Result.Y:=Min(V1.Y,V2.Y);
end;




function GetMaximumVector2D(const V1,V2: T2DVector): T2DVector;
{Return minimum components of two vectors}
begin
Result.X:=Max(V1.X,V2.X);
Result.Y:=Max(V1.Y,V2.Y);
end;


function AreEqual2D(const V1,V2: T2DVector): boolean;
{Return True if vectors are equal}
begin
Result:=(V1.X=V2.X) and (V1.Y=V2.Y);
end;



function Vector2DToString(const V: T2DVector; const Digits: integer; const Labeled: boolean): string;
{Return 2d vector as string. Digits past decimal in Digits}
var S1,S2: string;
begin
if Labeled then begin S1:='X: '; S2:=' Y: '; end
else begin S1:=''; S2:=' , '; end;
Result:=S1+FloatToStrF(V.X,ffFixed,18,Digits)+
        S2+FloatToStrF(V.Y,ffFixed,18,Digits);
end;



function Vector2DToString(const V: T2DVector): string;
{Return 2d vector as string}
begin
Result:=FloatToStrF(V.X,ffGeneral,18,4)+','+FloatToStrF(V.Y,ffGeneral,18,4);
end;



procedure RotatePoint2D(var P: T2DVector; Center: T2DVector; Angle: double);
{ Rotate point P around Center }
var D: T2DVector;
var SinTheta,CosTheta : double;
begin
CosTheta:=cos(Angle * Degree2Radian);
SinTheta:=sin(Angle * Degree2Radian);
D:=VectorSubtract2D(P,Center);
P.Y:=(SinTheta * -D.X) + (CosTheta * D.Y) + Center.Y;
P.X:=(CosTheta * D.X) + (SinTheta * D.Y) + Center.X;
end;



function PolarToVector2D(const Point: T2DVector; Angle,Length: double): T2DVector;
{Convert point length and angle to vector}
begin
Angle:=Angle * Degree2Radian;
Result.X:=Length * Cos(Angle);
Result.Y:=Length * Sin(Angle);
Result:=VectorAdd2D(Result,Point);
end;



function CompassToVector2D(const Point: T2DVector; Angle,Length: double): T2DVector;
{Convert point length and angle to vector}
{Uses compass angle where 0 degrees = north or up}
begin
Angle:=Angle + 90;
Result:=PolarToVector2D(Point,Angle,Length);
end;



function VectorNearestMultiple2D(const V,N: T2DVector; const Low: boolean): T2DVector;
begin
Result.X:=NearestMultiple(V.X,N.X,Low);
Result.Y:=NearestMultiple(V.Y,N.Y,Low);
end;



function VectorNearestMultiple2D(const V: T2DVector; const N: double; const Low: boolean): T2DVector;
begin
Result:=VectorNearestMultiple2D(V,MakeVector2D(N,N),Low);
end;


function GetNormal2D(const V: T2DVector): T2DVector;
{Returns perpendicular vector the same magnitude as the input}
begin
Result:=MakeVector2D(-V.Y, V.X);
end;


function GetUnitNormal2D(const V: T2DVector): T2DVector; overload;
{Returns perpendicular unit vector}
begin
Result:=UnitVector2D(MakeVector2D(-V.Y, V.X));
end;


function GetUnitNormal2D(const Line: T2DLine): T2DVector; overload;
{Returns perpendicular unit vector to specified line}
var D: T2DVector;
begin
D:=VectorSubtract2D(Line.P2,Line.P1);
Result:=UnitVector2D(MakeVector2D(-D.Y, D.X));
end;


function OffsetVector2D(V: T2DVector; X,Y: double): T2DVector;
begin
Result:=VectorAdd2D(V,MakeVector2D(X,Y));
end;


function InterpolateVector2D(V1,V2: T2DVector; D: double): T2DVector;
{Linear interpolation between two points}
begin
Result.X:=((V2.X-V1.X)*D)+V1.X;
Result.Y:=((V2.Y-V1.Y)*D)+V1.Y;
end;



function BilinearInterpolate2D(X,Y,Width,Height: integer; NW,NE,SW,SE: T2DVector): T2DVector;
{Binlinearly interpolate the coordinate on a grid}
{The mapping coordinates of corners of the grid are defined by NW, NE, SW, and SE}
{X1,Y1 and X2,Y2 are the grid index coordinates of the SW ane NE corners}
var SI,NI: T2DVector;
var Step: T2DVector;
begin
Step:=ScalarDivide2D(VectorSubtract2D(SE,SW),Width-1);
SI:=VectorAdd2D(ScalarProduct2D(Step,X),SW);
Step:=ScalarDivide2D(VectorSubtract2D(NE,NW),Width-1);
NI:=VectorAdd2D(ScalarProduct2D(Step,X),NW);

Step:=ScalarDivide2D(VectorSubtract2D(NI,SI),Height-1);
Result:=VectorAdd2D(ScalarProduct2D(Step,Y),SI);
end;



function ReadVector2DIni(const Ini: TIniFile; const Section, Ident: string; const Default: T2DVector): T2DVector;
{ Read T2DVector from specified INI file}
begin
Result.X:=Ini.ReadFloat(Section,Ident+'X',Default.X);
Result.Y:=Ini.ReadFloat(Section,Ident+'Y',Default.Y);
end;


procedure WriteVector2DIni(Ini: TIniFile;const Section, Ident: string; const Value: T2DVector);
{ Write T2DVector to specified INI file}
begin
Ini.WriteFloat(Section,Ident+'X',Value.X);
Ini.WriteFloat(Section,Ident+'Y',Value.Y);
end;


{----------------- Routines for 2D Lines -------------------------------------}

function Make2DLine(const P1,P2: TPoint): T2DLine; overload;
{Convert Points to 2D lines}
begin
Result.P1:=PointToVector2D(P1);
Result.P2:=PointToVector2D(P2);
end;


function Make2DLine(const P1,P2: T2DVector): T2DLine; overload;
{Convert 2D Vectors to 2D lines}
begin
Result.P1:=P1;
Result.P2:=P2;
end;



Function OffsetLine2D(Line: T2DLine; X,Y: double): T2DLine;
{Offset both points in Line by X and Y}
begin
Result.P1:=OffsetVector2D(Line.P1,X,Y);
Result.P2:=OffsetVector2D(Line.P2,X,Y);
end;




procedure RotateLine2D(var Line: T2DLine; Center: T2DVector; Angle: double);
{ Rotate Line around Center }
begin
RotatePoint2D(Line.P1,Center,Angle);
RotatePoint2D(Line.P2,Center,Angle);
end;



function PolarToLine2D(const Point: T2DVector; const Angle,Length: double): T2DLine;
{Convert point length and angle to 2D Line}
begin
Result.P1:=Point;
Result.P2:=PolarToVector2D(Point,Angle,Length);
end;



function CompassToLine2D(const Point: T2DVector; Angle,Length: double): T2DLine;
{Convert point length and angle to vector}
{Uses compass angle where 0 degrees = north or up}
begin
Angle:=Angle + 90;
Result:=PolarToLine2D(Point,Angle,Length);
end;





function IntersectLines2D(L1,L2: T2DLine; var Point: T2DVector): boolean;
{Finds the interesection point and will extend if necessary}
var distAB, theCos, theSin, newX, ABpos: double;
begin
Result:=False;
{ Fail if either line is undefined.}
if ((L1.P1.X=L1.P2.X) and (L1.P1.y=L1.P2.Y)) or
   ((L2.P1.X=L2.P2.X) and (L2.P1.Y=L2.P2.Y)) then exit;

{ (1) Translate the system so that point A is on the origin.}
L1.P2.X:=L1.P2.X-L1.P1.X; L1.P2.Y:=L1.P2.Y-L1.P1.y;
L2.P1.X:=L2.P1.X-L1.P1.X; L2.P1.Y:=L2.P1.Y-L1.P1.y;
L2.P2.X:=L2.P2.X-L1.P1.X; L2.P2.Y:=L2.P2.Y-L1.P1.y;

{ Discover the length of segment A-B.}
distAB:=sqrt(L1.P2.X*L1.P2.X+L1.P2.Y*L1.P2.Y);

{ (2) Rotate the system so that point B is on the positive X L1.P1.Xis.}
theCos:=L1.P2.X/distAB;
theSin:=L1.P2.Y/distAB;
newX:=L2.P1.X*theCos+L2.P1.Y*theSin;
L2.P1.Y :=L2.P1.Y*theCos-L2.P1.X*theSin; L2.P1.X:=newX;
newX:=L2.P2.X*theCos+L2.P2.Y*theSin;
L2.P2.Y :=L2.P2.Y*theCos-L2.P2.X*theSin; L2.P2.X:=newX;

{ Fail if the lines are parallel.}
if L2.P1.Y=L2.P2.Y then exit;

{ (3) Discover the position of the intersection point along line A-B.}
ABpos:=L2.P2.X+(L2.P1.X-L2.P2.X)*L2.P2.Y/(L2.P2.Y-L2.P1.Y);

{ (4) Apply the discovered position to line A-B in the original coordinate system.}
Point.X:=L1.P1.X+ABpos*theCos;
Point.Y:=L1.P1.y+ABpos*theSin;

{ Success.}
Result:=True;
end;





function DoLinesIntersect2D(L1,L2: T2DLine; var Point: T2DVector): boolean;
{Finds intersect point only if the lines actually intersect}
var distAB, theCos, theSin, newX, ABpos: double;
begin
Result:=False;
{  Fail if either line segment is zero-length.}
if (L1.P1.X=L1.P2.X) and (L1.P1.Y=L1.P2.Y) or
   (L2.P1.X=L2.P2.X) and (L2.P1.Y=L2.P2.Y) then exit;

{  Fail if the segments share an end-point.}
if (L1.P1.X=L2.P1.X) and (L1.P1.Y=L2.P1.Y) or
   (L1.P2.X=L2.P1.X) and (L1.P2.Y=L2.P1.Y) or
   (L1.P1.X=L2.P2.X) and (L1.P1.Y=L2.P2.Y) or
   (L1.P2.X=L2.P2.X) and (L1.P2.Y=L2.P2.Y) then exit;

{  (1) Translate the system so that point A is on the origin.}
L1.P2.X:=L1.P2.X-L1.P1.X; L1.P2.Y:=L1.P2.Y-L1.P1.Y;
L2.P1.X:=L2.P1.X-L1.P1.X; L2.P1.Y:=L2.P1.Y-L1.P1.Y;
L2.P2.X:=L2.P2.X-L1.P1.X; L2.P2.Y:=L2.P2.Y-L1.P1.Y;

{  Discover the length of segment A-B.}
distAB:=sqrt(L1.P2.X*L1.P2.X+L1.P2.Y*L1.P2.Y);

{  (2) Rotate the system so that point B is on the positive X L1.P1.Xis.}
theCos:=L1.P2.X/distAB;
theSin:=L1.P2.Y/distAB;
newX:=L2.P1.X*theCos+L2.P1.Y*theSin;
L2.P1.Y  :=L2.P1.Y*theCos-L2.P1.X*theSin; L2.P1.X:=newX;
newX:=L2.P2.X*theCos+L2.P2.Y*theSin;
L2.P2.Y  :=L2.P2.Y*theCos-L2.P2.X*theSin; L2.P2.X:=newX;

{  Fail if segment C-D doesn't cross line A-B.}
if (L2.P1.Y<0) and (L2.P2.Y<0) or (L2.P1.Y>=0) and (L2.P2.Y>=0) then exit;

{  (3) Discover the position of the intersection point along line A-B.}
ABpos:=L2.P2.X+(L2.P1.X-L2.P2.X)*L2.P2.Y/(L2.P2.Y-L2.P1.Y);

{  Fail if segment C-D crosses line A-B outside of segment A-B.}
if (ABpos<0) or (ABpos>distAB) then exit;

{  (4) Apply the discovered position to line A-B in the original coordinate system.}
Point.X:=L1.P1.X+ABpos*theCos;
Point.Y:=L1.P1.Y+ABpos*theSin;
Result:=True;
end;



function DoLinesIntersect2D(L1,L2: T2DLine): boolean;
{Return true if lines intersect}
var Point: T2DVector;
begin
Result:=DoLinesIntersect2D(L1,L2,Point);
end;

function LineLength2D(const P1,P2: T2DVector) : double; overload;
{  Find length of a line defined by P1 and P2  }
begin
Result:=VectorABS2D(VectorSubtract2D(P2,P1));
end;



function LineLength2D(const L: T2DLine) : double; overload;
{  Find length of a line defined by P1 and P2  }
begin
Result:=LineLength2D(L.P2,L.P1);
end;




Function LineCenter2D(const L: T2DLine): T2DVector;
{  Find the center of line L  }
begin
Result.X := ((L.P2.X - L.P1.X)/2) + L.P1.X;
Result.Y := ((L.P2.Y - L.P1.Y)/2) + L.P1.Y;
end;



procedure ForceIntersect2D(var L1,L2: T2DLine);
{  Extend lines until they intersect  }
var Point: T2DVector;
begin
if not IntersectLines2D(L1,L2,Point) then exit;
if LineLength2D(L1.P2,Point) < LineLength2D(L1.P1,L1.P2) then
	begin
	L1.P2:=Point;
	L2.P1 := Point;
	end;
end;



function ExtendLine2D(const L1: T2DLine; const Len: double): T2DVector;
{ Return a point that extends line L1 by Len  }
var Len1,UX,UY : double;
begin
Len1 := LineLength2D(L1)+1E-9;
UX := (L1.P2.X - L1.P1.X)/Len1;
UY := (L1.P2.Y - L1.P1.Y)/Len1;
Result.X := L1.P2.X +(UX *Len);
Result.Y := L1.P2.Y+(UY *Len);
end;


function ExtendLine2D(const P1,P2: T2DVector; const Len: double): T2DVector;
{  Return a point that extends line L1 by Len  }
begin
Result:=ExtendLine2D(Make2DLine(P1,P2),Len);
end;


function ChangeLineLength2D(var Line: T2DLine; const Len: double): T2DLine;
{Extend or truncate line symetrically for the center of the line}
begin
Result.P1:=ExtendLine2D(Line.P2,Line.P1,Len/2);
Result.P2:=ExtendLine2D(Line.P1,Line.P2,Len/2);
end;


procedure DrawLine2D(Canvas: TCanvas; L: T2DLine; C: TColor);
{Draw Line on specified canvas}
begin
Canvas.Pen.Color:=C;
Canvas.MoveTo(Trunc(L.P1.X),Trunc(L.P1.Y));
Canvas.LineTo(Trunc(L.P2.X),Trunc(L.P2.Y));
end;



function RightTurn2D(const L1, L2: T2DLine): Boolean;
{  return true if angle A2 goes right from A1 }
var T,A1,A2 : double;
begin
A1 := ArcTan2(L1.P2.X - L1.P1.X,L1.P2.Y - L1.P1.Y - 1E-6);
A2 := ArcTan2(L2.P2.X - L2.P1.X,L2.P2.Y - L2.P1.Y - 1E-6);

T := A2-A1;
if T>PI then T:=T-TwoPI else if T < -PI then T:=T+TwoPI;
Result:=(T>=0);
end;


{-----------------------------------------------------------------------------}


function PointLineExtDist2D(const Point,LineStart,LineEnd: T2DVector; var Intercept: T2DVector): double;
{Get distance between point and extended line}
var Inter: T3DVector;
begin
Result:=PointLineExtDist3D(Vector2DToVector3D(Point),Vector2DToVector3D(LineStart),Vector2DToVector3D(LineEnd),Inter);
end;



function PointLineExtDist2D(const Point: T2DVector; const Line: T2DLine; var Intercept: T2DVector): double;
{Find distance between a point and extended line}
var Inter: T3DVector;
begin
Result:=PointLineExtDist3D(Vector2DToVector3D(Point),Vector2DToVector3D(Line.P1),Vector2DToVector3D(Line.P2),Inter);
end;



function PointLineExtDist2D(const Point: T2DVector; const Line: T2DLine): double;
{Find distance between a point and a extended line}
var Intercept: T2DVector;
begin
Result:=PointLineExtDist2D(Point,Line,Intercept);
end;



function PointLineSegDist2D(const Point: T2DVector; const Line: T2DLine; var Distance: double; var Intercept: T2DVector): boolean;
{Find distance between a point and a line segment}
var Inter: T3DVector;
begin
Result:=PointLineSegDist3D(Vector2DToVector3D(Point),Vector2DToVector3D(Line.P1),Vector2DToVector3D(Line.P2),Distance,Inter);
Intercept:=Vector3DTo2D(Inter);
end;



function PointLineSegDist2D(const Point: T2DVector; const Line: T2DLine): double;
{Find distance between a point and a line segment}
var Intercept: T2DVector;
begin
PointLineSegDist2D(Point,Line,Result,Intercept);
end;




function LineSegLineSegDist2D(S1,S2: T2DLine): double;
{Find distance between two line segments}
begin
Result:=LineSegLineSegDist3D(Line2DtoLine3D(S1),Line2DtoLine3D(S2));
end;

{------------------- 2D Figures------------------------------------------------}


function CircleIntersect(Center1,Center2: T2DVector; Radius1,Radius2: double; var Intersect1,Intersect2: T2DVector): boolean;
{Find the intersection between two circles}
{ Returns if there is no intersection }
var Delta,R,P2: T2DVector;
var  a, d, h: double;
begin
Result:=False;
{ Delta.X and Delta.Y are the vertical and horizontal distances between the circle centers. }
Delta:=VectorSubtract2D(Center2,Center1);

{ Determine the straight-line distance between the centers. }
d := VectorAbs2D(Delta);

{ Check for solvability. }
if d > (Radius1 + Radius2) then exit;	{ no solution. circles do not intersect. }
if d < abs(Radius1 - Radius2) then exit;	{ no solution. one circle is contained in the other }

{ Point 2'is the point where the line through the circle}
{intersection points crosses the line between the circle centers.}

{ Determine the distance from point 0 to point 2. }
a := ((Radius1*Radius1) - (Radius2*Radius2) + (d*d)) / (2.0 * d) ;

{ Determine the coordinates of point 2. }
P2.X := Center1.X + (Delta.X * a/d);
P2.Y := Center1.Y + (Delta.Y * a/d);

{ Determine the distance from point 2 to either of the intersection points. }

h := sqrt((Radius1*Radius1) - (a*a));

{ Now determine the offsets of the intersection points from point 2. }
R.X := -Delta.Y * (h/d);
R.Y := Delta.X * (h/d);

{ Determine the absolute intersection points. }
Intersect1.X := P2.X + R.X;
Intersect2.X := P2.X - R.X;
Intersect1.Y := P2.Y + R.Y;
Intersect2.Y := P2.Y - R.Y;
Result:=True;
end;



function IntersectCenteredEllipse(A,B: double; P: T2DVector): T2DVector;
{ Intersect ellipse centered at the origin widths/ehights of a and b }
{with a line passing through the origin and point-P. Result is +/- times the value}
var T: double;
begin
T:=(A * B) / Sqrt(sqr(A) * sqr(P.Y) + sqr(B) * sqr(P.X));
Result:=ScalarProduct2D(P,T);
end;



function IntersectEllipse(R: T2DRect; P: T2DVector): T2DVector;
{Intersect uncentered ellipse with line defined by the center and Point-P}
{Since there are two possible intersect, it chooses the correct one for P }
var C,Size: T2DVector;
begin
{Find the center of the ellipse}
C:=ScalarDivide2D(VectorAdd2D(R.TopLeft,R.BottomRight),2);
{Center around origin}
P:=VectorSubtract2D(P,C);
{Get size of Ellipse}
Size:=VectorSubtract2D(R.BottomRight,R.TopLeft);
{Find the interesect between ellipse and line}
Result:=IntersectCenteredEllipse(Size.X/2,Size.Y/2,P);
{Restore to original position}
Result:=VectorAdd2D(Result,C);
end;




{----------------- Line clipping and wrapping routines ------------------------}


{Clipping codes specify the direction the lines are to be clipped}

const ccInside=0; ccBottom=1;ccTop=2; ccRight=4; ccLeft=8;

{Combinations of clipping code for diagonal exit points}

const ccTopLeft = ccTop or ccLeft;
const ccTopRight = ccTop or ccRight;
const ccBottomLeft = ccBottom or ccLeft;
const ccBottomRight = ccBottom or ccRight;


function ClipLine2D(ClipWin: T2DRect; var Line: T2DLine; var ExitCode1,ExitCode2: integer): boolean;
{ Clip Line to ClipWin based on Cohen & Sutherland algorithm }
{ Returns clipped line in "Line," returns True if line is visible}
{Uses upside down windows coordinates, where Y increased down screen}
{ "ExitCode1" and "ExitCode2" returns the Clip-Codes for the two ends of the line}
{  This is either the code at the point the line exits }
{ or if it is outside the direction it is outside }
var Code1, Code2: integer;
var Delta: T2DVector;
var TL: T2DLine;
var W,H: double;
var I: integer;



	function GetClipCode(P: T2DVector): integer;
	{Calculate the clip code for specified point}
	begin
	if (P.Y>ClipWin.Bottom) then Result:=ccBottom
	else if (P.Y<ClipWin.Top) then Result:=ccTop else Result:=ccInside;

	if P.X>ClipWin.Right then Result := Result  or  ccRight
	else if P.X<ClipWin.Left then Result := Result  or  ccLeft;
	end;




	procedure ClipLineAgainstBorder(var ClipLine: T2DLine; Code: integer; var ExitCode: integer);
	{Clip the first point in ClipLine to the window border}
	{If the line is actually clipped, returns Code for that Exit Point}
	begin
	if (Code and ccBottom) <> 0 then
		begin
		{Off Bottom}
		ClipLine.P1.X:= ClipLine.P1.X + Delta.X  * (ClipWin.Bottom-ClipLine.P1.Y)/Delta.Y;
		ClipLine.P1.Y := ClipWin.Bottom;
		ExitCode:=ccBottom;
		end
	else if (Code1 and ccTop) <> 0 then
		begin
		{Off Top}
		ClipLine.P1.X:=ClipLine.P1.X + Delta.X  * (ClipWin.Top-ClipLine.P1.Y)/Delta.Y;
		ClipLine.P1.Y := ClipWin.Top;
		ExitCode:=ccTop;
		end
	else if (Code1 and ccRight) <> 0 then
		begin
		{Off Right}
		ClipLine.P1.Y:=ClipLine.P1.Y + Delta.Y  * (ClipWin.Right-ClipLine.P1.X)/Delta.X;
		ClipLine.P1.X := ClipWin.Right;
		ExitCode:=ccRight;
		end
	else if (Code1 and ccLeft) <> 0 then
		begin
		{Off Left}
		ClipLine.P1.Y:=ClipLine.P1.Y + Delta.Y  * (ClipWin.Left-ClipLine.P1.X)/Delta.X;
		ClipLine.P1.X := ClipWin.Left;
		ExitCode:=ccLeft;
		end;
	end;



begin
{Normalize clip window coordinates to make X increase right to left and Y top to bottom}
SwapDoubleArgs(ClipWin.Left>ClipWin.Right,ClipWin.Left,ClipWin.Right);
SwapDoubleArgs(ClipWin.Top>ClipWin.Bottom,ClipWin.Top,ClipWin.Bottom);
{Find amount of change in X/Y directions}
Delta:=VectorSubtract2D(Line.P2,Line.P1);
{Get starting clip condition}
ExitCode1:=GetClipCode(Line.P1);
ExitCode2:=GetClipCode(Line.P2);
while True do
	begin
	Code1:=GetClipCode(Line.P1);
	Code2:=GetClipCode(Line.P2);
	Result:=True;
	{Exit if both points are inside}
	if (Code1  or Code2) = 0 then break;
	Result:=False;
	{Exit if both points are outside and in the same quadrant}
	if (Code1 and Code2) <> 0 then	break;
	{One must be inside and one outside}
	if Code1=ccInside then
		begin
		{Flip to put outside point in P1}
		ExchangeVectors2D(Line.P1,Line.P2);
		SwapIntegers(Code1,Code2);
		SwapIntegers(ExitCode1,ExitCode2);
		end;
	{Clip the P1 end of line against border}
	ClipLineAgainstBorder(Line,Code1,ExitCode1);
	end;
end;




{-------------------------------------------------------------------------------}


procedure WrapLine2D(ClipWin: T2DRect; Line: T2DLine; var WrappedLines: T2DLineArray; WrapHorizontal,WrapVertical: boolean);
{Clip and wrap "Line" to "ClipWin"  results returned "WrappedLines" dynamic array }
{ "WrapHorizontal" and "WrapVertical"  control the direction of the wrap. }
{ Diagonal wrapping only occurs if both are enabled}
var W,H: double;
var RecCnt: integer;

	procedure AddExtLine(Line: T2DLine);
	{Add one time to dynamic array of wrapped lines}
	begin
	SetLength(WrappedLines,Length(WrappedLines)+1);
	WrappedLines[High(WrappedLines)]:=Line;
	end;


	procedure RecurseWindows(Win: T2DRect; Line: T2DLine; FromCode: integer);
	{Process specified windowed recursively}
	var Code1,Code2: integer;
	var R: T2DRect;
	var Offset: T2DVector;
	var OffsetLine,ClipLine: T2DLine;


		function TestLineExit(ClipCode,TargetCode,FromCode,ExitCode: integer; Offset: T2DVector): boolean;
		{Test if line exits the window in a specific direction. Recurse in that direction if it does}
		{ "ClipCode" = exit direction. "TargetCode" = Desired direction to recurse to.}
		{ "FromCode" = where we came from. "ExitCode= where the next recursion comes from }
		{ "Offset" = amount to offset retangle for next recursion. }
		{ Do not recurse if the recursion takes us backe where we came from }
		begin
		if (ClipCode=TargetCode) and (FromCode<>TargetCode) then
			begin
			R:=GetOffsetRect2D(Win,Offset);
			RecurseWindows(R, Line, ExitCode);
			Result:=True;
			end
		else Result:=False;
		end;


		procedure TestEndPoint(Code: integer);
		{Test one end-point code to see which direction it may be out of the Window}
		{Continue recursing in various directions until there nothing in that direction}
		begin
		{Do diagonal directions first because they contain two bits set}
		if WrapHorizontal and WrapVertical then
			begin
			if TestLineExit(Code,ccTopLeft,FromCode,ccBottomRight, MakeVector2D(-W,-H)) then exit;
			if TestLineExit(Code,ccTopRight,FromCode,ccBottomLeft, MakeVector2D(W,-H)) then exit;
			if TestLineExit(Code,ccBottomRight,FromCode,ccTopLeft, MakeVector2D(W,H)) then exit;
			if TestLineExit(Code,ccBottomLeft,FromCode,ccTopRight, MakeVector2D(-W,H)) then exit;
			end;
		{Test horizontal and vertical directions}
		if WrapHorizontal  then
			begin
			if TestLineExit(Code,ccRight,FromCode,ccLeft, MakeVector2D(W,0)) then exit;
			if TestLineExit(Code,ccLeft,FromCode,ccRight, MakeVector2D(-W,0)) then exit;
			end;
		if WrapVertical then
			begin
			if TestLineExit(Code,ccBottom,FromCode,ccTop, MakeVector2D(0,H)) then exit;
			if TestLineExit(Code,ccTop,FromCode,ccBottom, MakeVector2D(0,-H)) then exit;
			end;
		end;




	begin	{Recurse window}
	Inc(RecCnt);
	if RecCnt>50 then exit;
	ClipLine:=Line;
	{Find the amount of offset from the base window}
	Offset:=VectorSubtract2D(ClipWin.TopLeft,Win.TopLeft);
	{Clip the line and if some of it is in the Window, insert in array}
	if ClipLine2D(Win, ClipLine, Code1,Code2) then
		begin
		OffsetLine:=OffsetLine2D(ClipLine,Offset.X,Offset.Y);
		AddExtLine(OffsetLine);
		end;
	{Test the end points to see if they are out of the window and recurse if they are.}
	TestEndPoint(Code1);
	TestEndPoint(Code2);
	end;


begin
RecCnt:=0;
{Get the size of the base window}
W:=Abs(ClipWin.Right-ClipWin.Left);
H:=Abs(ClipWin.Bottom-ClipWin.Top);
{Clear dynamic array of wrapped lines}
SetLength(WrappedLines,0);
{Begin recursion}
RecurseWindows(ClipWin, Line, ccInside);
end;



function WrapPoint2D(ClipWin: T2DRect; V: T2DVector; WrapHorizontal,WrapVertical: boolean): T2DVector;
{Wrap any point around ClipWin that is outside the window}
var W,H: double;
begin
{Normalize clip window coordinates to make X increase right to left and Y top to bottom}
SwapDoubleArgs(ClipWin.Left>ClipWin.Right,ClipWin.Left,ClipWin.Right);
SwapDoubleArgs(ClipWin.Top>ClipWin.Bottom,ClipWin.Top,ClipWin.Bottom);
Result:=V;
W:=ClipWin.Right - ClipWin.Left;
H:=ClipWin.Bottom - ClipWin.Top;
if WrapHorizontal then
	begin
	while Result.X<ClipWin.Left do Result.X:=Result.X + W;
	while Result.X>ClipWin.Right do Result.X:=Result.X - W;
	end;
if WrapVertical then
	begin
	while Result.Y<ClipWin.Top do Result.Y:=Result.Y + H;
	while Result.Y>ClipWin.Bottom do Result.Y:=Result.Y - H;
	end;
end;

{---------------- Routines for 2D rectangles ----------------------------------}


function Rect2DToRect(const R: T2DRect): TRect;
{Convert 2D rectangle to Windows integer rectangle}
begin
Result.TopLeft:=Vector2DToPoint(R.TopLeft);
Result.BottomRight:=Vector2DToPoint(R.BottomRight);
end;


function MakeRect2D(const X1,Y1,X2,Y2: double): T2DRect;  overload;
{Create rectangle from indvidual points}
begin
Result.Left:=X1;
Result.Top:=Y1;
Result.Right:=X2;
Result.Bottom:=Y2;
end;



function MakeRect2D(const TopLeft,BottomRight: T2DVector): T2DRect; overload;
{Make rectangle from 2D vertices}
begin
Result.TopLeft:=TopLeft;
Result.BottomRight:=BottomRight;
end;


function RectToRect2D(const Rect: TRect): T2DRect;
{Convert integer rectangle to real rectangle}
begin
Result.Left:=Rect.Left;
Result.Right:=Rect.Right;
Result.Top:=Rect.Top;
Result.Bottom:=Rect.Bottom;
end;


procedure OffsetRect2D(var Rect: T2DRect; const Offset: T2DVector);
{Move rectangle by specified offset}
begin
Rect.TopLeft:=VectorAdd2D(Rect.TopLeft,Offset);
Rect.BottomRight:=VectorAdd2D(Rect.BottomRight,Offset);
end;





function GetOffsetRect2D(Rect: T2DRect; const Offset: T2DVector): T2DRect;
begin
Result:=Rect;
OffsetRect2D(Result,Offset);
end;


procedure InflateRect2D(var Rect: T2DRect; const Delta: T2DVector);
{Expand retangle by absolute amount specified in Delta}
begin
Rect.TopLeft:=VectorSubtract2D(Rect.TopLeft,Delta);
Rect.BottomRight:=VectorAdd2D(Rect.BottomRight,Delta);
end;


procedure ExpandRect2D(var R: T2DRect; A: T2DVector);
{Expand rectangle by percentage specified in A}
var D,N,X: T2DVector;
begin
D:=VectorSubtract2D(R.BottomRight,R.TopLeft);
N:=VectorMultiply2D(D,A);
X:=ScalarDivide2D(VectorSubtract2D(N,D),2);
InflateRect2D(R,X);
end;



procedure RotateRectangleCenter2D(var Rect: T2DRect; Center: T2DVector; Angle: double);
{Rotate the center of specified rectangle around center}
{Does not rotate the individual corner points, only the center}
var C1,C2,Offset: T2DVector;
begin
C1:=ScalarDivide2D(VectorAdd2D(Rect.TopLeft,Rect.BottomRight),2);
C2:=C1;
RotatePoint2D(C2,Center,-Angle);
Offset:=VectorSubtract2D(C2,C1);
OffsetRect2D(Rect,Offset);
end;




function MaxRotatedRect2D(const Rect: T2DRect; const Center: T2DVector; const Angle: double): T2DRect;
{Returns the maximum enclosing rectangle after rotation}
var UL,UR,LL,LR: T2DVector;
var MinX,MinY,MaxX,MaxY: double;
begin
UL:=Rect.TopLeft; LR:=Rect.BottomRight;
LL:=MakeVector2D(Rect.Left,Rect.Bottom);
UR:=MakeVector2D(Rect.Right,Rect.Top);
RotatePoint2D(UL,Center,Angle);
RotatePoint2D(UR,Center,Angle);
RotatePoint2D(LL,Center,Angle);
RotatePoint2D(LR,Center,Angle);

MinX:=Min(Min(UL.X,UR.X), Min(LL.X,LR.X));
MaxX:=Max(Max(UL.X,UR.X), Max(LL.X,LR.X));
MinY:=Min(Min(UL.Y,UR.Y), Min(LL.Y,LR.Y));
MaxY:=Max(Max(UL.Y,UR.Y), Max(LL.Y,LR.Y));
Result:=MakeRect2D(MinX,MinY,MaxX,MaxY);
end;




function PointInRect2D(const P: T2DVector; R: T2DRect): boolean;
{Is 2D point in 2D Rectangle}
begin
Result:=False;
{Normalize to screen coordinate order}
SwapDoubleArgs(R.Left>R.Right,R.Left,R.Right);
SwapDoubleArgs(R.Top>R.Bottom,R.Top,R.Bottom);

if (P.X<R.Left) or (P.X>R.Right) then exit;
if (P.Y<R.Top) or (P.Y>R.Bottom) then exit;
Result:=True;
end;



function DoRectsOverlap2D(R1,R2: T2DRect): boolean;
{Do rectangles overlap?}
begin
Result:=False;
if R1.BottomRight.X < R2.TopLeft.X then exit; { R1 is left of R2}
if R1.TopLeft.X > R2.BottomRight.X then exit; { R1 is right of R2}
if R1.BottomRight.Y < R2.TopLeft.Y then exit; { R1 is above R2}
if R1.TopLeft.Y > R2.BottomRight.Y then exit; { R1 is below R2}
Result:=True;
end;



{------------ 3D Vector math routines --------------------}



function VectorAdd3D(const V1,V2: T3DVector): T3DVector;
{Do dot product of two vectors}
begin
Result.X:= V1.X + V2.X;
Result.Y:= V1.Y + V2.Y;
Result.Z:= V1.Z + V2.Z;
end;


function VectorSubtract3D(const V1,V2: T3DVector): T3DVector;
{Do vector substract V1-V2}
begin
Result.X:= V1.X - V2.X;
Result.Y:= V1.Y - V2.Y;
Result.Z:= V1.Z - V2.Z;
end;



function VectorMultiply3D(const V1,V2: T3DVector): T3DVector;
{Multiply vector components}
begin
Result.X:=V1.X * V2.X;
Result.Y:=V1.Y * V2.Y;
Result.Z:=V1.Z * V2.Z;
end;



function VectorDivide3D(const V1,V2: T3DVector): T3DVector;
{Multiply vector components}
begin
Result.X:=V1.X / V2.X;
Result.Y:=V1.Y / V2.Y;
Result.Z:=V1.Z / V2.Z;
end;


function VectorSquare3D(const V: T3DVector): T3DVector;
{Square the components of specified vector}
begin
Result:=VectorMultiply3D(V,V);
end;


function VectorSqrRoot3D(const V: T3DVector): T3DVector;
{Take square root of the vector's components}
begin
Result.X:=Sqrt(V.X);
Result.Y:=Sqrt(V.Y);
Result.Z:=Sqrt(V.Z);
end;



function VectorFraction3D(const V: T3DVector): T3DVector;
{Return fractional part of vector components}
begin
Result.X:=Frac(V.X);
Result.Y:=Frac(V.Y);
Result.Z:=Frac(V.Z);
end;



function InvertVector3d(const V: T3DVector): T3DVector;
begin
Result.X:=1/V.X;
Result.Y:=1/V.Y;
Result.Z:=1/V.Z;
end;



function DotProduct3D(const V1,V2: T3DVector): double;
{Do dot product of two vectors}
begin
Result:=(V1.X * V2.X) + (V1.Y * V2.Y) + (V1.Z * V2.Z);
end;



function CrossProduct3D(const V1,V2: T3DVector): T3DVector;
{Do crosss product of two vectors - assume left-hand coordinate system}
begin
Result.X:= ((V1.Y * V2.Z) - (V1.Z * V2.Y));
Result.Y:= -((V1.X * V2.Z) - (V1.Z * V2.X));
Result.Z:= ((V1.X * V2.Y) - (V1.Y * V2.X));
end;



function RightCrossProduct3D(const V1,V2: T3DVector): T3DVector;
{Do crosss product of two vectors - assumes right-hand coordinate system}
begin
Result.X:= V1.Y * V2.Z - V1.Z * V2.Y;
Result.Y:= V1.Z * V2.X - V1.X * V2.Z;
Result.Z:= V1.X * V2.Y - V1.Y * V2.X;
end;



function ScalarProduct3D(const V: T3DVector; const S: double): T3DVector;
{Multiply vector by scalar}
begin
Result.X:=V.X * S;
Result.Y:=V.Y * S;
Result.Z:=V.Z * S;
end;



function ScalarAdd3D(const V: T3DVector; const S: double): T3DVector;
{Add scalar to vector }
begin
Result.X:=V.X + S;
Result.Y:=V.Y + S;
Result.Z:=V.Z + S;
end;


function ScalarSubtract3D(const V: T3DVector; const S: double): T3DVector;
{Subtract scalar from vector }
begin
Result.X:=V.X - S;
Result.Y:=V.Y - S;
Result.Z:=V.Z - S;
end;


function ScalarDivide3D(const V: T3DVector; const S: double): T3DVector;
{Divide vector by scalar}
begin
Result.X:=V.X / S;
Result.Y:=V.Y / S;
Result.Z:=V.Z / S;
end;




function VectorABS3D(const V: T3DVector): double;
{Find ABS of vector}
begin
Result:=Sqrt(Sqr(V.X) + Sqr(V.Y) + Sqr(V.Z));
end;


function UnitVector3D(const V: T3DVector): T3DVector;
{Return unit vector}
var L: double;
begin
L:=VectorABS3D(V);
if L=0.0 then L:=1E-99;
Result.X:=V.X / L;
Result.Y:=V.Y / L;
Result.Z:=V.Z / L;
end;


function NegateVector3D(const V: T3DVector): T3DVector;
{Return negation of vector}
begin
Result.X:=-V.X;
Result.Y:=-V.Y;
Result.Z:=-V.Z;
end;



function LineLength3D(const A, B: T3DVector): double;
{Find distance between two points}
begin
Result:=VectorABS3D(VectorSubtract3D(B,A));
end;



function LineLength3D(const L: T3DLine): double;
{Find distance between two points}
begin
Result:=LineLength3D(L.P2,L.P1);
end;




function AreEqual3D(const V1,V2: T3DVector): boolean;
{Return True if vectors are equal}
begin
Result:=(V1.X=V2.X) and (V1.Y=V2.Y) and (V1.Z=V2.Z);
end;


function VectorAngle3D(const V1,V2: T3DVector): double;
{Return angle of two vectors}
var T: double;
begin
T:=VectorABS3D(V1) * VectorABS3D(V2);
if T=0 then Result:=0
else
	begin
	T:=DotProduct3D(V1,V2) / T;
	if T>1 then T:=1 else if T<-1 then T:=-1;
	Result:=ArcCos(T);
	end;
end;




function PolarToVector3D(const Point: T3DVector; const Angle1,Angle2,Length: double): T3DVector;
{Convert point length and angle to vector}
var SinA1,CosA1,SinA2,CosA2: double;
var T: T3DVector;
begin
SinA1:=Sin(Angle1 * Degree2Radian);
CosA1:=Cos(Angle1 * Degree2Radian);
SinA2:=Sin(Angle2 * Degree2Radian);
CosA2:=Cos(Angle2 * Degree2Radian);
Result:=MakeVector3D(0,Length,0);
T:=Result;

Result.Y := (SinA1  * -T.X) + (CosA1  * T.Y);
Result.X := (CosA1  * T.X) + (SinA1  * T.Y);
Result.Z := (SinA2  * -Result.Y) + (CosA2  * T.Z);
Result.Y := (CosA2  * Result.Y) + (SinA2  * T.Z);
Result:=VectorAdd3D(Result,Point);
end;



procedure VectorToPolar3D(P1,P2: T3DVector; var Azimuth,Inclination,Length: double);
{ Convert two points, from P1 to P2, to Azimuth and Inclination }
var D:T3DVector;
begin
D:=VectorSubtract3D(P2,P1);
Length:=VectorABS3D(D);
Azimuth:=(ArcTan2(D.X,D.Y) * Radian2Degree);
if Azimuth<0 then Azimuth:=360 + Azimuth;

if D.Z=0 then Inclination:=0
else
	begin
	Inclination:=D.Z / Length;
	if Inclination>1 then Inclination:=1;
	if Inclination<-1 then Inclination:=-1;
	Inclination:=ArcSin(Inclination)  * Radian2Degree;
	end;
end;


function MakeVector3D(const X,Y,Z: double): T3DVector;
{Create a vector from component values }
begin
Result.X:=X;
Result.Y:=Y;
Result.Z:=Z;
end;


function MakeVector3D(const V: T2DVector; const Z: double): T3DVector;
{Create a vector from from a 2D vector a  }
begin
Result:=MakeVector3D(V.X,V.Y,Z);
end;


function MakeVector3DY(const V: T2DVector): T3DVector;
begin
Result:=MakeVector3D(V,V.Y);
end;


function RandomVector3D(const Range: T3DVector): T3DVector;
{Return Random vector of specified range}
begin
Result:=MakeVector3D((Random * Range.X)-(Range.X/2),(Random * Range.Y)-(Range.Y/2),(Random * Range.Z)-(Range.Z/2));
end;




function GetMinimumVector(const V1,V2: T3DVector): T3DVector;
{Return minimum components of two vectors}
begin
Result.X:=Min(V1.X,V2.X);
Result.Y:=Min(V1.Y,V2.Y);
Result.Z:=Min(V1.Z,V2.Z);
end;


function GetMaximumVector(const V1,V2: T3DVector): T3DVector;
{Return minimum components of two vectors}
begin
Result.X:=Max(V1.X,V2.X);
Result.Y:=Max(V1.Y,V2.Y);
Result.Z:=Max(V1.Z,V2.Z);
end;



function RangeCheckVectorMin(const V,Min: T3DVector): T3DVector; overload;
{Ensure vector components are greater or equal to Min Vector}
begin
Result:=V;
if Result.X<Min.X then Result.X:=Min.X;
if Result.Y<Min.Y then Result.Y:=Min.Y;
if Result.Z<Min.Z then Result.Z:=Min.Z;
end;



function RangeCheckVectorMin(const V: T3DVector; const Min: double): T3DVector;  overload;
{Ensure vector components are greater or equal to Min value}
begin
Result:=RangeCheckVectorMin(V,MakeVector3D(Min,Min,Min));
end;



function RangeCheckVectorMax(const V,Max: T3DVector): T3DVector; overload;
{Ensure vector components are less than or equal to Max Vector}
begin
Result:=V;
if Result.X>Max.X then Result.X:=Max.X;
if Result.Y>Max.Y then Result.Y:=Max.Y;
if Result.Z>Max.Z then Result.Z:=Max.Z;
end;



function RangeCheckVectorMax(const V: T3DVector; const Max: double): T3DVector;  overload;
{Ensure vector components are less than or equal to max value}
begin
Result:=RangeCheckVectorMax(V,MakeVector3D(Max,Max,Max));
end;



procedure ExchangeVectors3D(var V1,V2: T3DVector);
{Swap Vectors V1 and V2}
var T: T3DVector;
begin
T:=V1; V1:=V2; V2:=T;
end;




function Vector3DToPoint(const V: T3DVector): TPoint;
{Convert 3D Vector to a point}
begin
Result.X:=round(V.X);
Result.Y:=round(V.Y);
end;



function PointToVector3D(const P: TPoint): T3DVector;
{Convertpoint to a 3D Vector }
begin
Result.X:=P.X;
Result.Y:=P.Y;
Result.Z:=0
end;


function Vector3DToString(const V: T3DVector; const Digits: integer): string; overload
{Return 3d vector as string. Digits past decimal in Digits}
begin
Result:='X: '+FloatToStrF(V.X,ffFixed,18,Digits)+
        ' Y: '+FloatToStrF(V.Y,ffFixed,18,Digits)+
        ' Z: '+FloatToStrF(V.Z,ffFixed,18,Digits);
end;



function Vector3DToString(const V: T3DVector; const Digits: integer; const Separator: char): string; overload;
{Return 3d vector as string. Digits past decimal in Digits}
begin
Result:=FloatToStrF(V.X,ffFixed,18,Digits)+Separator+
	FloatToStrF(V.Y,ffFixed,18,Digits)+Separator+
	FloatToStrF(V.Z,ffFixed,18,Digits);
end;



function Vector3DToString(const V: T3DVector): string; overload;
{Return 3d vector as string. Digits past decimal in Digits}
begin
Result:=FloatToStrF(V.X,ffGeneral,18,4)+','+
	FloatToStrF(V.Y,ffGeneral,18,4)+','+
	FloatToStrF(V.Z,ffGeneral,18,4);
end;


function VectorNearestMultiple3D(const V,N: T3DVector; const Low: boolean): T3DVector;
begin
Result.X:=NearestMultiple(V.X,N.X,Low);
Result.Y:=NearestMultiple(V.Y,N.Y,Low);
Result.Y:=NearestMultiple(V.Y,N.Z,Low);
end;



function VectorNearestMultiple3D(const V: T3DVector; const N: double; const Low: boolean): T3DVector;
begin
Result:=VectorNearestMultiple3D(V,MakeVector3D(N,N,N),Low);
end;


procedure RotatePoint3D(var P: T3DVector; const Center: T3DVector; const Yaw,Pitch: double);
{ Rotate 3D Coordinates around center - angles are in degrees }
var D: T3DVector;
var Y,YawSin,YawCos,PitchSin,PitchCos: extended;
begin
SinCos(Yaw * Degree2Radian,YawSin,YawCos);
SinCos(Pitch * Degree2Radian,PitchSin,PitchCos);
D:=VectorSubtract3D(P,Center);

Y   := (YawSin  * -D.X) + (YawCos  * D.Y);
P.X := (YawCos  *  D.X) + (YawSin  * D.Y) + Center.X;

P.Z := (PitchSin  * -Y) + (PitchCos  * D.Z) + Center.Z;
P.Y := (PitchCos  *  Y) + (PitchSin  * D.Z) + Center.Y;
end;


procedure RotatePoint3D(var P: T3DVector; const Center: T3DVector; const  Pitch,Yaw,Roll: double);
var M: TGLMatrix;
begin
M:=EulerToMatrix(Pitch,Yaw,Roll);
P:=VectorAdd3D(MatrixVectorMult(M,VectorSubtract3D(P,Center)),Center);
end;


procedure VectorToUpRight(Dir: T3DVector; var Up,Right: T3DVector);
begin
if Dir.Z=0 then
	begin
	Up:=MakeVector3D(0,0,1);
	Right:=UnitVector3D(CrossProduct3D(Dir,Up));
	end
else
	begin
	if Dir.Z>0 then Right:=MakeVector3D(-1,0,0)
	else Right:=MakeVector3D(1,0,0);
	Up:=UnitVector3D(CrossProduct3D(Right,Dir));
	Right:=UnitVector3D(CrossProduct3D(Dir,Up));
	end;

end;



function CardinalBasisVector(V: T3DVector): T3DVector;
{ Return Cardinal Bias Vector corresponding to "V" }
{ This is a unit vector pointing in the X, Y or Z direction}
{ according to which elements has the smallest absolute magnitue }
var absX,absY,absZ: double;
begin
absX:=abs(V.x);
absY:=abs(V.y);
absZ:=abs(V.z);
{Is abs(Y) smaller than everything else? Then use use Y-cardinal direction}
if (absY <= absX) and (absY <= absZ) then Result := MakeVector3D(0, 1, 0)
{Is abs(Z) smaller than everything else? Then use use Z-cardinal direction}
else if (absZ <= absX) and (absZ <= absY) then Result := MakeVector3D(0, 0, 1)
{Abs(X) must be smaller than everything else so use use X-cardinal direction}
else Result := MakeVector3D(1, 0, 0);
end;



procedure NormalToPlaneVectors(Normal: T3DVector; var V1,V2: T3DVector);
{Convert a normal to two vectors on the corresponding plane}
var CrossVector: T3DVector;
begin
{ Use Cardinal basis vector to get the first vector on the plane}
CrossVector := CardinalBasisVector(Normal);
V1 := UnitVector3D(CrossProduct3D(Normal, crossVector));
{Cross the first two vectors to get the second one in a plane.}
V2 := UnitVector3D(CrossProduct3D(Normal, V1));
end;



function InterpolateVector3D(V1,V2: T3DVector; D: double): T3DVector;
{Linear interpolation between two points}
begin
Result.X:=((V2.X-V1.X)*D)+V1.X;
Result.Y:=((V2.Y-V1.Y)*D)+V1.Y;
Result.Z:=((V2.Z-V1.Z)*D)+V1.Z;
end;



{----------- 3D Line Routines ------------------------}


function ExtendLine3D(const Line: T3DLine; const Len: double): T3DVector; overload;
{ Return a point that extends line L1 by Len  }
var U: T3DVector;
begin
U:=UnitVector3D(VectorSubtract3D(Line.P2,Line.P1));
Result:=VectorAdd3D(Line.P2,ScalarProduct3D(U,Len));
end;



function ExtendLine3D(const P1,P2: T3DVector; const Len: double): T3DVector; overload;
{ Return a point that extends line P1-P2 by Len  }
begin
Result:=ExtendLine3D(MakeLine3D(P1,P2),Len);
end;




function GetDirectedVector3D(const P1,P2: T3DVector; const Length: double): T3DVector;
{Returns a vector in direction defined by P1 to P2 of specified length }
begin
Result:=VectorSubtract3D(P1,P2);
Result:=UnitVector3D(Result);
Result:=ScalarProduct3D(Result,Length);
end;



function GetDirectedVector3D(const Line: T3DLine; const Len: double): T3DVector;
{Returns a vector in direction defined by P1 to P2 of specified length }
begin
Result:=GetDirectedVector3D(Line.P1,Line.P2,Len);
end;




procedure TrimStartLine3D(var L: T3DLine; const Amount: double);
{Trim start of line by specified amount}
var O: T3DVector;
begin
O:=GetDirectedVector3D(L.P2,L.P1,Amount);
L.P1:=VectorAdd3D(L.P1,O);
end;



procedure TrimEndLine3D(var L: T3DLine; const Amount: double);
{Trim end of line by specified amount}
var O: T3DVector;
begin
O:=GetDirectedVector3D(L.P1,L.P2,Amount);
L.P2:=VectorAdd3D(L.P2,O);
end;


function MakeLine3D(const P1,P2: T3DVector): T3DLine;
{Return line specified by two vectors}
begin
Result.P1:=P1;
Result.P2:=P2;
end;


function Line2DToLine3D(L: T2DLine): T3DLine;
{Make 3D line from 2D line}
begin
Result.P1.X:=L.P1.X; Result.P1.Y:=L.P1.Y; Result.P1.Z:=0;
Result.P2.X:=L.P2.X; Result.P2.Y:=L.P2.Y; Result.P2.Z:=0;
end;


{-----------------------------------------------------------------------------}


function PointLineExtDist3D(const Point,LineStart,LineEnd: T3DVector; var Intercept: T3DVector): double;
{Get distance between point and extended line}
var U,LineMag: double;
var LineLength,StartDist: T3DVector;
begin
LineLength:=VectorSubtract3D(LineEnd,LineStart);
StartDist:=VectorSubtract3D(Point,LineStart);
LineMag:=VectorABS3D(LineLength);

U:=((StartDist.X * LineLength.X) + (StartDist.Y * LineLength.Y) + (StartDist.Z * LineLength.Z) ) / ( LineMag * LineMag );
Intercept:=VectorAdd3D(LineStart,ScalarProduct3D(LineLength,U));
Result:= VectorABS3D(VectorSubtract3D(Point, Intercept));
end;



function PointLineExtDist3D(const Point: T3DVector; const Line: T3DLine; var Intercept: T3DVector): double;
{Find distance between a point and a line segment}
begin
Result:=PointLineExtDist3D(Point,Line.P1,Line.P2,Intercept);
end;




function PointLineSegDist3D(const Point,LineStart,LineEnd: T3DVector; var Distance: double; var Intercept: T3DVector): boolean;
{Get distance between point and line. Returns true if intersect within line segment}
var U,LineMag: double;
var LineLength,StartDist: T3DVector;
begin
Result:=False;
LineLength:=VectorSubtract3D(LineEnd,LineStart);
StartDist:=VectorSubtract3D(Point,LineStart);
LineMag:=VectorABS3D(LineLength);

U:=((StartDist.X * LineLength.X) + (StartDist.Y * LineLength.Y) + (StartDist.Z * LineLength.Z) ) / ( LineMag * LineMag );
{ Does closest point fall within the line segment }
if U<0 then Intercept:=LineStart
else if U>1 then Intercept:=LineEnd
else
	begin
	Result:=True;
	Intercept:=VectorAdd3D(LineStart,ScalarProduct3D(LineLength,U));
	end;
Distance:= VectorABS3D(VectorSubtract3D(Point, Intercept));
end;



function PointLineSegDist3D(const Point: T3DVector; const Line: T3DLine; var Distance: double; var Intercept: T3DVector): boolean;
{Find distance between a point and a line segment}
begin
Result:=PointLineSegDist3D(Point,Line.P1,Line.P2,Distance,Intercept);
end;




function LineSegLineSegDist3D(S1,S2: T3DLine): double;
{Find distance between two line segments}
{Does not extend lines in both directions}
var u,v,w,dP: T3DVector;
var a,b,c,d,e,DDD,sc,sN,sD,tc,tN,tD: double;
const SMALL_NUM=1E-16;
begin
u:=VectorSubtract3D(S1.P2,S1.P1);
v:=VectorSubtract3D(S2.P2,S2.P1);
w:=VectorSubtract3D(S1.P1,S2.P1);
a:=DotProduct3D(u,u);	{ always >:= 0}
b:=DotProduct3D(u,v);
c:=DotProduct3D(v,v);	{ always >:= 0}
d:=DotProduct3D(u,w);
e:=DotProduct3D(v,w);
DDD:=a*c - b*b;	{ always >:= 0}
sD:=DDD;	{ sc:=sN / sD, default sD:=DDD >:= 0}
tD:=DDD;	{ tc:=tN / tD, default tD:=DDD >:= 0}

{ compute the line parameters of the two closest points}
if (DDD < SMALL_NUM) then
	begin	{ the lines are almost parallel}
	sN:=0.0;	{ force using point P0 on segment S1}
	sD:=1.0;	{ to prevent possible division by 0.0 later}
	tN:=e;
	tD:=c;
	end
else
	begin	{ get the closest points on the infinite lines}
	sN:=(b*e - c*d);
	tN:=(a*e - b*d);
	if (sN < 0.0) then
		begin	{ sc < 0 :=> the s:=0 edge is visible}
		sN:=0.0;
		tN:=e;
		tD:=c;
		end
	else if (sN > sD) then
		begin	{ sc > 1  :=> the s:=1 edge is visible}
		sN:=sD;
		tN:=e + b;
		tD:=c;
		end;
	end;

if (tN < 0.0) then
	begin	{ tc < 0 :=> the t:=0 edge is visible}
	tN:=0.0;
	{ recompute sc for this edge}
	if (-d < 0.0) then sN:=0.0
	else if (-d > a) then sN:=sD
	else
		begin
		sN:=-d;
		sD:=a;
		end
	end
else if (tN > tD) then
	begin	{ tc > 1  => the t:=1 edge is visible}
	tN:=tD;
	{ recompute sc for this edge}
	if ((-d + b) < 0.0) then sN:=0
        else if ((-d + b) > a) then sN:=sD
        else
        	begin
        	sN:=(-d +  b);
        	sD:=a;
        	end;
        end;
{ finally do the division to get sc and tc}
if abs(sN) < SMALL_NUM then sc:=0.0 else  sc:=sN / sD;
if abs(tN) < SMALL_NUM then tc:=0.0 else tc:=tN / tD;

{ get the difference of the two closest points = S1(sc) - S2(tc)}
dP:=VectorSubtract3D(VectorAdd3D(w,ScalarProduct3D(u,sc)),ScalarProduct3D(v,tc));

Result:=VectorAbs3D(dP);	{ return the closest distance}
end;




{---------- Triangle Routines --------------------------}






function MakeTriangle(const P1,P2,P3: T3DVector): TTriangle;
{Convert points to a triangle}
begin
Result.P1:=P1; Result.P2:=P2; Result.P3:=P3;
end;



function GetTriangleArea(const T: TTriangle): double;
var A,B,C,S: double;
begin
A:=VectorABS3D(VectorSubtract3D(T.P2, T.P1));
B:=VectorABS3D(VectorSubtract3D(T.P3, T.P2));
C:=VectorABS3D(VectorSubtract3D(T.P1, T.P3));
S:=(A+B+C)/2;
Result:=Sqrt(abs(S*(S-A)*(S-B)*(S-C)));
end;



function GetTriangleNormal(const Tri: TTriangle): T3DVector;
{Calculate the normal of triangle face}
var V1,V2: T3DVector;
begin
V1:=VectorSubtract3D(Tri.P2,Tri.P1);
V2:=VectorSubtract3D(Tri.P3,Tri.P1);
Result:=CrossProduct3D(V1,V2);
Result:=UnitVector3D(Result);
end;



function GetTriangleNormal(const P1,P2,P3: T3DVector): T3DVector;
{Calculate the normal of triangle face}
begin
Result:=GetTriangleNormal(MakeTriangle(P1,P2,P3));
end;



function ReverseWinding(const T: TTriangle): TTriangle;
{Reverse a triangle's winding direction}
begin
Result.P1:=T.P3;
Result.P2:=T.P2;
Result.P3:=T.P1;
end;



function BaryCentricInterpolation(var T: TTriangle; const U,V: double): T3DVector;
{Convert Barycentric point U,V to point in 3-Space (on triangle T)}
var J,L: T3DVector;
begin
J:=VectorAdd3D(ScalarProduct3D(T.P3,V), ScalarProduct3D(T.P1,1-V));
L:=VectorAdd3D(ScalarProduct3D(T.P3,V), ScalarProduct3D(T.P2,1-V));
Result:=VectorAdd3D(ScalarProduct3D(J,1-U), ScalarProduct3D(L,U));
end;



function RayTriangleIntersect(const Orig, Dir,  Vert0, Vert1, Vert2: T3DVector; const CullFaces: boolean; var T, U, V: double): boolean;  overload;
{ From "Fast Minimum Storage Ray/Triangle Intersection" by Tomas Mller and Ben Trumbore}
{Orig = Origin of Ray, Dir = Diretion of Ray, Vert0, Vert1, Vert2 = triangle}
{If CullFaces is true, Triangles facing away from the origin are ignored}
{T = distance from the Ray Origin to intersection of the triangle}
{U,V = the barycentric coordinates of the triangle intersection}
var Edge1, Edge2, TVector, PVector, QVector: T3DVector;
var Determinant,InverseDeterminant: double;
const EPSILON = 0.000001;
begin
Result:=False;
{ find vectors for two Edges sharing Vert0 }
Edge1:=VectorSubtract3D(Vert1, Vert0);
Edge2:=VectorSubtract3D(Vert2, Vert0);

{ begin calculating Determinant - also used to calculate U parameter }
PVector:=RightCrossProduct3D(Dir, Edge2);

{ if Determinant is near zero, ray lies in plane of triangle }
Determinant:= DotProduct3D(Edge1, PVector);

{ define TEST_CULL if culling is desired }
if CullFaces then
	begin
	if Determinant < EPSILON then exit;

	{ calculate distance from Vert0 to ray Origin }
	TVector:=VectorSubtract3D(Orig, Vert0);

	{ calculate U parameter and test bounds }
	U:= DotProduct3D(TVector, PVector);
	if (U < 0.0) or (U > Determinant) then exit;

	{ prepare to test V parameter }
	QVector:=RightCrossProduct3D( TVector, Edge1);

	{ calculate V parameter and test bounds }
	V:= DotProduct3D(Dir, QVector);
	if (V < 0.0) or (U + V > Determinant) then exit;

	{ calculate T, scale parameters, ray intersects triangle }
	T:= DotProduct3D(Edge2, QVector);
	InverseDeterminant:= 1.0 / Determinant;
	T:=T * InverseDeterminant;
	U:=U * InverseDeterminant;
	V:=V * InverseDeterminant;
	end
else
	begin
	{ the non-culling branch }
	if (Determinant > -EPSILON) and (Determinant < EPSILON) then exit;
	InverseDeterminant:= 1.0 / Determinant;

	{ calculate distance from Vert0 to ray Origin }
	 TVector:=VectorSubtract3D( Orig, Vert0);

	{ calculate U parameter and test bounds }
	U:= DotProduct3D(TVector, PVector) * InverseDeterminant;
	if ( U < 0.0) or (U > 1.0) then exit;

	{ prepare to test V parameter }
	 QVector:=RightCrossProduct3D( TVector, Edge1);

	{ calculate V parameter and test bounds }
	V:= DotProduct3D(Dir, QVector) * InverseDeterminant;
	if (V < 0.0) or (U + V > 1.0) then exit;

	{ calculate T, ray intersects triangle }
	T:= DotProduct3D(Edge2, QVector) * InverseDeterminant;
	end;
Result:= True;
end;




function RayTriangleIntersect(const Ray: T3DLine; const Triangle: TTriangle; const CullFaces: boolean): boolean; overload;
var T, U, V: double;
var Dir: T3DVector;
begin
Dir:=VectorSubtract3D(Ray.P2,Ray.P1);
Result:=RayTriangleIntersect(Ray.P1, Dir,  Triangle.P1, Triangle.P2, Triangle.P3, CullFaces, T, U, V);
end;




function RayTriangleIntersect(const Ray: T3DLine; Triangle: TTriangle; const CullFaces: boolean; var Intercept: T3DVector): boolean; overload;
var T, U, V: double;
var Dir: T3DVector;
begin
Dir:=VectorSubtract3D(Ray.P2,Ray.P1);
Result:=RayTriangleIntersect(Ray.P1, Dir, Triangle.P1, Triangle.P2, Triangle.P3, CullFaces, T, U, V);
if Result then Intercept:=BaryCentricInterpolation(Triangle,U,V);
end;



function GetTriangleCenter(T: TTriangle): T3DVector;
begin
Result:=VectorAdd3D(VectorAdd3D(T.P1,T.P2),T.P3);
Result:=ScalarDivide3D(Result,3);
end;



function GetTrianglePerimeter(T: TTriangle): double;
begin
Result:=VectorABS3D(VectorSubtract3D(T.P2,T.P1));
Result:=Result+VectorABS3D(VectorSubtract3D(T.P3,T.P2));
Result:=Result+VectorABS3D(VectorSubtract3D(T.P1,T.P3));
end;


function GetLongestTriangleEdge(T: TTriangle): double;
var E1,E2,E3: double;
begin
E1:=VectorABS3D(VectorSubtract3D(T.P2,T.P1));
E2:=VectorABS3D(VectorSubtract3D(T.P3,T.P2));
E3:=VectorABS3D(VectorSubtract3D(T.P1,T.P3));
Result:=Max(Max(E1,E2),E3);
end;



function IsFacingTriangle(T: TTriangle): boolean;
{Does triangle face front using opengl coordinate system}
var A: double;
var V1,V2,N: T3DVector;
begin
V1:=VectorSubtract3D(T.P2,T.P1);
V2:=VectorSubtract3D(T.P3,T.P1);
N:=CrossProduct3D(V1,V2);
N:=UnitVector3D(N);
A:=DotProduct3D(MakeVector3D(0,0,1),N);
Result:=A>=0;
end;


{------------ Rectangle Routines ---------------------}


function MakeRectangle(const P1,P2,P3,P4: T3DVector): T3DRectangle;
begin
Result.P1:=P1; Result.P2:=P2;
Result.P3:=P3; Result.P4:=P4;
end;


{----------- Plane handling procedures ---------------}

procedure TPlane.SetPlane(Azm,Dip: double);
{Set plane based on azimuth and dip }
var PLen: double;
var V1,V2: T3DVector;
begin
Dip:=Dip * Degree2Radian;
Azm:=Azm * Degree2Radian;

PLen:=Cos(Dip);
V1.Z:=Sin(Dip);
V1.Y:=PLen * Cos(Azm);
V1.X:=PLen * Sin(Azm);

V2.Y:=Cos(Azm - 90);
V2.X:=Sin(Azm - 90);
V2.Z:=0;

Con:=CrossProduct3D(V1,V2);
end;


function TPlane.GetPlaneZ(X,Y:double): double;
{Get elevation based on East and North position}
begin
Result:=(-(Con.X*X)-Con.Y*Y)/Con.Z;
end;

constructor TPlane.Create;
begin
inherited Create;
SetPlane(0,0);
end;


procedure TPlane.GetAzmDip(var A,D: double);
{Get Azimuth and Dip angle of specified plane}
var V1,V2: T3DVector;
begin
if (Con.X=0) and (Con.Y=0) then begin A:=0; D:=0; exit; end;
V1.X:=0; V1.Y:=0; V1.Z:=1;
D:=-Radian2Degree * VectorAngle3D(V1,Con);

V2:=CrossProduct3D(Con,V1);
V1.X:=0; V1.Y:=1; V1.Z:=0;
A:=Radian2Degree * VectorAngle3D(V1,V2);
A:=Radian2Degree * ArcTan2(V2.X,V2.Y)-90;
if A<0 then A:=A+360;
end;


procedure TPlane.SetEquation(V: T3DVector);
{Set plane equation based on Azimuth/Dip vector}
begin
Con:=V;
if Con.Z<0 then Con:=ScalarProduct3D(Con,-1);
end;



function TPlane.GetEquation: T3DVector;
{Return the actual plan equations}
begin
Result:=Con;
end;


{------------------ Polygon Routines ------------------------------------}








end.