
unit VectorHash;

interface

uses SysUtils,HugeArry,Vector;

{------------------------------------------------------------}
{------ Declaration for Vector Hash Table object ------------}
{------------------------------------------------------------}

type TVectorHash = class(THuge3DVector)
 private
  HashLink: THugeLongint;		{ List Linkage Pointers }
  HashBox: THugeLongint;		{ Hash Boxes (symbol List Headers) }
  HashValue: Integer;
 protected
  procedure  ClearHashBox;
  function  GetHashValue(V: T3DVector): integer;
 public
  constructor Create(StartAlloc: LongInt);
  destructor Destroy; override;
    procedure GetDistribution(var Dist: THugeInteger);
  function Search(V: T3DVector): Longint;
  function AddUnique(V: T3DVector): integer;
  function Insert(V: T3DVector): integer;
  procedure Rehash;
 end;

{ Variables For Hash Table Lookup }

const
 BoxNumber=2048;	{ Number Of Boxes MUST be power of two}
 HasmMask=BoxNumber-1;	{ Hash Mask }
 HashEmpty=-1;		{ Empty Symbol Pointer }


 implementation

{-------------------------------------------------------------------------}
{                  Implementation for huge array object                   }
{-------------------------------------------------------------------------}


constructor TVectorHash.Create(StartAlloc: LongInt);
{Create a hash table item}
begin
inherited Create(StartAlloc);
HashLink:=THugeLongInt.Create(StartAlloc);
HashBox:=THugeLongInt.Create(BoxNumber);
ClearHashBox;
end;


destructor TVectorHash.Destroy;
{Destroy hash table item }
begin
HashLink.Free;
HashBox.Free;
inherited Destroy;
end;



procedure  TVectorHash.ClearHashBox;
{ Clear Hash Box }
var P : Integer;
begin
for P:=0 to BoxNumber-1 do  HashBox.Value[P]:=HashEmpty;
end;



procedure  TVectorHash.Rehash;
{ Re-do The Hash Links For The Table }
var P : Longint;
begin
ClearHashBox;
HashLink.ItemIndex:=ItemIndex;
for P:=0 to ItemIndex-1 do
	begin
	HashValue:=GetHashValue(Value[P]);
	HashLink.Value[P]:=HashBox.Value[HashValue];
	HashBox.Value[HashValue]:=P;
	end;
end;




function  TVectorHash.GetHashValue(V: T3DVector): integer;
{ Return Hash For String S }
var I,T : Integer;
var B: PByteArray;
begin
Result:=0;
B:=@V;
for I:=0 to Sizeof(T3DVector)-1 do
	begin
	Result:=(Result shl 2) + B[I];
        T:=Result and $C000;
	if T<>0 then Result:= Result xor (T shr 12) and $3FFF;
	end;
Result:= Result  and  HasmMask;
end;


function VectorDistance3D(V1,V2: T3DVector): double;
begin
Result:=VectorABS3D(VectorSubtract3D(V1,V2));
end;



function TVectorHash.Search(V: T3DVector): Longint;
{ Search For Vector V, return index of Vector }
{ Returns -1 if not found }
begin
HashValue:=GetHashValue(V);
Result:=HashBox.Value[HashValue];
while True do
	begin
	if Result=HashEmpty then break;
	if VectorDistance3D(V,Value[Result])=0 then break;
	Result:=HashLink.Value[Result];
	end;
end;




function TVectorHash.AddUnique(V: T3DVector): integer;
{ Insert Vertex Into Hash Table if item is unique}
begin
Result:=Search(V);
if Result=-1 then Result:=Insert(V);
end;




function TVectorHash.Insert(V: T3DVector): integer;
{ Insert Into Hash Table }
begin
HashValue:=GetHashValue(V);
Result:=Add(V);
HashLink.Add(HashBox.Value[HashValue]);
HashBox.Value[HashValue]:=HashLink.ItemIndex-1;
end;


procedure TVectorHash.GetDistribution(var Dist: THugeInteger);
{Get the distribution of hashbox values - for testing purposes}
var I,Inx,Cnt: integer;
begin
Dist.Clear;
for I:=0 to HashBox.ItemIndex-1 do
	begin
	Inx:=HashBox[I];
	Cnt:=0;
	while Inx<>HashEmpty do
		begin
		Inc(Cnt);
		Inx:=HashLink.Value[Inx];
		end;
	Dist.Add(Cnt)
	end;
end;



end.
