unit WinSupport;

interface

uses Windows,Messages,Types,Classes,Forms,Graphics,Controls,
     StdCtrls,MMSystem,PSAPI,TlHelp32,SysUtils,DSupport;

{Formatting information for Splash message}

type TSplashFormat = record
 FontStyle: TFontStyles;
 FontSize: integer;
 Alignment :TAlignment;
 end;


{Object to display spash screen for a specified number of milliseconds}


type TSplashMessageDlg = class(TForm)
 private
 public
  MsgLabel: TLabel;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure Show(Msg: string; Seconds: integer); overload;
 end;


{Windows version constants}


const
  SE_CREATE_TOKEN_NAME = 'SeCreateTokenPrivilege';
  SE_ASSIGNPRIMARYTOKEN_NAME = 'SeAssignPrimaryTokenPrivilege';
  SE_LOCK_MEMORY_NAME = 'SeLockMemoryPrivilege';
  SE_INCREASE_QUOTA_NAME = 'SeIncreaseQuotaPrivilege';
  SE_UNSOLICITED_INPUT_NAME = 'SeUnsolicitedInputPrivilege';
  SE_MACHINE_ACCOUNT_NAME = 'SeMachineAccountPrivilege';
  SE_TCB_NAME = 'SeTcbPrivilege';
  SE_SECURITY_NAME = 'SeSecurityPrivilege';
  SE_TAKE_OWNERSHIP_NAME = 'SeTakeOwnershipPrivilege';
  SE_LOAD_DRIVER_NAME = 'SeLoadDriverPrivilege';
  SE_SYSTEM_PROFILE_NAME = 'SeSystemProfilePrivilege';
  SE_SYSTEMTIME_NAME = 'SeSystemtimePrivilege';
  SE_PROF_SINGLE_PROCESS_NAME = 'SeProfileSingleProcessPrivilege';
  SE_INC_BASE_PRIORITY_NAME = 'SeIncreaseBasePriorityPrivilege';
  SE_CREATE_PAGEFILE_NAME = 'SeCreatePagefilePrivilege';
  SE_CREATE_PERMANENT_NAME = 'SeCreatePermanentPrivilege';
  SE_BACKUP_NAME = 'SeBackupPrivilege';
  SE_RESTORE_NAME = 'SeRestorePrivilege';
  SE_SHUTDOWN_NAME = 'SeShutdownPrivilege';
  SE_DEBUG_NAME = 'SeDebugPrivilege';
  SE_AUDIT_NAME = 'SeAuditPrivilege';
  SE_SYSTEM_ENVIRONMENT_NAME = 'SeSystemEnvironmentPrivilege';
  SE_CHANGE_NOTIFY_NAME = 'SeChangeNotifyPrivilege';
  SE_REMOTE_SHUTDOWN_NAME = 'SeRemoteShutdownPrivilege';
  SE_UNDOCK_NAME = 'SeUndockPrivilege';
  SE_SYNC_AGENT_NAME = 'SeSyncAgentPrivilege';
  SE_ENABLE_DELEGATION_NAME = 'SeEnableDelegationPrivilege';
  SE_MANAGE_VOLUME_NAME = 'SeManageVolumePrivilege';


type TFontExt = class(TFont)
 public
  TextMetric: TTextMetric;
  LogFont: TLogFont;
  FontType: integer;
 end;


function ProcessIDToFilename(ProcID: DWORD; var Error: string): string;
function GetProcessIdByName(const ProcName: string): DWORD;
function SetPrivilege(hToken: THandle; Privilege: LPCTSTR; bEnablePrivilege: Boolean): Boolean;
function IsProgramRunning(const FullPath: string; UseFullPath: boolean): boolean;
procedure KillProcessByName(filename: string);


type TWinVersion = (wvWinNT31=31,wvWinNT35=35, wvWin95=40,wvWin98=41,wvWinME=49,
		    wvWin2000=50,wvWinXP=51,wvWinVista=60,wvWin7=61,wvWin8=62,
		    wvWin81=63);

function GetVersionStr(WV: TWinVersion): string;
function GetWinVersion: TWinVersion;
function IsWindowsNT: boolean;

function GetFileVersion(const Filename: string; var V1,V2,V3,V4: word): string;
function GetDLLVersion(DLLName: string; var V1,V2,V3,V4: word): string;

function GetTextBox(Canvas: TCanvas; S: string): TRect;
procedure SplashMessageDlg(Msg: string; Seconds: integer);  overload;
procedure SplashMessageDlg(Msg: string; Seconds: integer; SplashFormat: TSplashFormat); overload;
procedure SetKeyState(Key: integer; Down: boolean);
function GetExpandedEnviroString(S: string): string;
procedure PlaySoundFile(Sound: string);
function ShortToLongFileName(FileName: string): string;

function IsButtonPushed(Btn: TButton): boolean;
function GetLocalComputerName: string;

procedure GetFontList(ListBox: TListBox);
procedure GetFontStrings(Strings: TStrings);
function GetFontTypeStr(FT: integer): string;

procedure OpenRegedit(Key: string);

function AscentToHeight(Canvas: TCanvas; Ascent: integer): integer;
function HeightToAscent(Canvas: TCanvas; Height: integer): double;

function IsTrueTypeFont(DC: HDC): Boolean; overload;
function IsTrueTypeFont(Canvas: TCanvas): Boolean; overload;
function IsTrueTypeFont(Font: TFont): Boolean; overload;

function GetNearestTrueTypeName(Canvas: TCanvas): string;

function GetKeyboardDynamics(var Delay, Speed: integer): boolean;
function SetKeyboardDynamics(var Delay, Speed: integer): boolean;

function FileToWinHandle(FileName: string): HWND;


implementation

uses ShellAPI;

var KernelHandle: THandle;	{Handle for DLL Kernel32 }


function GetTextBox(Canvas: TCanvas; S: string): TRect;
{Returns a retangle for specified text}
begin
Result:=Rect(0,0,0,0);
DrawTextEx(Canvas.Handle,PChar(S),-1,Result,DT_CALCRECT,0);
end;


function GetTextWidth(Canvas: TCanvas; S: string): integer;
var R: TRect;
begin
R:=GetTextBox(Canvas,S);
Result:=R.Right - R.Left;
end;


function GetTextHeight(Canvas: TCanvas; S: string): integer;
var R: TRect;
begin
R:=GetTextBox(Canvas,S);
Result:=R.Bottom - R.Top;
end;

{----------------- Splash screen object --------------------------------------}

constructor TSplashMessageDlg.Create(AOwner: TComponent);
begin
CreateNew(AOwner);
Caption:='';
BorderStyle:=bsToolWindow;
BorderIcons:=BorderIcons - [biSystemMenu];
Position := poScreenCenter;
FormStyle:=fsStayOnTop;
MsgLabel:=TLabel.Create(Self);
MsgLabel.Parent:=Self;
MsgLabel.Align:=alClient;
end;



destructor TSplashMessageDlg.Destroy;
begin
MsgLabel.Free;
inherited Destroy;
end;


procedure TSplashMessageDlg.Show(Msg: string; Seconds: integer);
begin
Width:=10+GetTextWidth(Canvas,Msg);
Height:=40+GetTextHeight(Canvas,Msg);
MsgLabel.Caption:=Msg;
Show;
DelayProcessMessages(Seconds * 1000);
Hide;
end;


procedure SplashMessageDlg(Msg: string; Seconds: integer); overload;
{Instantiate and display a splash screen with message}
{for the specified amount of time in seconds}
var SD: TSplashMessageDlg;
begin
SD:=TSplashMessageDlg.Create(Application);
SD.Font.Style:=[fsBold];
SD.Font.Size:=10;
SD.MsgLabel.Alignment:=taCenter;
SD.Show(Msg,Seconds);
SD.Free;
end;



procedure SplashMessageDlg(Msg: string; Seconds: integer; SplashFormat: TSplashFormat); overload;
{Instantiate and display a splash screen with message}
{for the specified amount of time in seconds}
var SD: TSplashMessageDlg;
begin
SD:=TSplashMessageDlg.Create(Application);
SD.Font.Style:=SplashFormat.FontStyle;
SD.Font.Size:=SplashFormat.FontSize;
SD.MsgLabel.Alignment:=SplashFormat.Alignment;
SD.Show(Msg,Seconds);
SD.Free;
end;


{------------------- Windows utility routines --------------------------------}




function GetVersionStr(WV: TWinVersion): string;
begin
case WV of
 wvWinNT31: Result:='Windows NT 3.1';
 wvWinNT35: Result:='Windows NT 3.5';
 wvWin95: Result:='Windows 95';
 wvWin98: Result:='Windows 98';
 wvWinME: Result:='Windows ME';
 wvWin2000: Result:='Windows 2000';
 wvWinXP: Result:='Windows XP';
 wvWinVista: Result:='Windows Vista';
 wvWin7: Result:='Windows 7';
 wvWin8: Result:='Windows 8';
 wvWin81: Result:='Windows 8.1';
 end;
end;



function GetWinVersion: TWinVersion;
var Version: TOSVersionInfo;
begin
Version.dwOSVersionInfoSize:=SizeOf(Version);
GetVersionEx(Version);
if Version.dwMinorVersion>9 then Version.dwMinorVersion:=Version.dwMinorVersion div 10;
Result:=TWinVersion(Version.dwMajorVersion * 10 + Version.dwMinorVersion);
end;



function IsWindowsNT: boolean;
{Returns true if Windows NT or greater}
var Version: TOSVersionInfo;
begin
Version.dwOSVersionInfoSize:=SizeOf(Version);
GetVersionEx(Version);
Result:=Version.dwPlatformId=VER_PLATFORM_WIN32_NT;
end;




procedure SetKeyState(Key: integer; Down: boolean);
{Sets Windows key state including Caps lock,}
var O: OSVERSIONINFO;
var KeyState: TKeyboardState;
var State: byte;
var ToggleKey: boolean;
begin
{Test windows version}
O.dwOSVersionInfoSize:=SizeOf(O);
GetVersionEx(O);
GetKeyboardState(KeyState);

if Down then State:=1 else State:=0;
ToggleKey:=(Key=VK_Capital) or (Key=VK_NumLock) or (Key=VK_Scroll);
if o.dwPlatformId = VER_PLATFORM_WIN32_WINDOWS then
	begin
	{Win95/98 Methoc}
	KeyState[Key]:=State;
	SetKeyboardState(KeyState);
	end
else If o.dwPlatformId = VER_PLATFORM_WIN32_NT then
	begin
	{Use Nt/2000/XP method}
	if Down = (KeyState[Key]<>0) then exit;
	if Down or ToggleKey then keybd_event(Key, $45, KEYEVENTF_EXTENDEDKEY or 0, 0);
	if not Down or ToggleKey then keybd_event(Key, $45, KEYEVENTF_EXTENDEDKEY or KEYEVENTF_KEYUP, 0);
	end;
end;




function GetExpandedEnviroString(S: string): string;
{Return string with expanded environmental string}
var L: integer;
begin
SetLength(Result,255);
L:=ExpandEnvironmentStrings(PChar(S),PChar(Result),255);
SetLength(Result,L-1);
end;




procedure PlaySoundFile(Sound: string);
{Play specified sound file}
begin
Sound:=GetExpandedEnviroString(Sound);
if FileExists(Sound) then
	begin
        PlaySound(PChar(Sound),0,SND_ASYNC or SND_FILENAME or SND_NOWAIT);
	end;
end;



type TFNGetLongPathName = function(lpszShortName: LPCTSTR; lpszLongName: LPTSTR; cchBuffer: DWORD): DWORD; stdcall;
var  GetLongPathName: TFNGetLongPathName = nil;

function ShortToLongFileName(FileName: string): string;
{Convert short path/filename from short to long format}
var FindData: TWin32FindData;
var Search: THandle;
begin
Result := '';
if Assigned(GetLongPathName) then
	begin
	{Use GetLongPathName where available (Win98 and later)}
	SetLength(Result, MAX_PATH + 1);
	SetLength(Result, GetLongPathName(PChar(FileName), @Result[1], MAX_PATH));
	exit;
	end;

{if earlier version of Windows, use FindFirstFile function to dissect path}
while True do
	begin
	{Search for file and path}
	Search := Windows.FindFirstFile(PChar(FileName), FindData);
	if Search = INVALID_HANDLE_VALUE then Break;
	{Add long name to result}
	Result := string('\') + FindData.cFileName + Result;
	{Strip off last name }
	FileName := ExtractFileDir(FileName);
	Windows.FindClose(Search);
	{ Last two letters are Drive and colon.}
	if Length(FileName) <= 2 then Break;	
	end;

Result:= ExtractFileDrive(FileName) + Result;
end;




function IsButtonPushed(Btn: TButton): boolean;
{Returns true if button is pushed}
begin
Result:=(SendMessage(Btn.Handle, BM_GETSTATE, 0, 0) and BST_PUSHED)=BST_PUSHED;
end;

{------------------------------------------------------------------------------}

function GetFileVersion(const Filename: string; var V1,V2,V3,V4: word): string;
{Get file version number}
var InfoSize: DWORD;
var VerBuf:   pointer;
var VerSize:  UINT;
var wnd:      UINT;
var FileInfo : PVSFixedFileInfo;
begin
InfoSize:= GetFileVersionInfoSize(PChar(Filename), Wnd);
V1:=0; V2:=0; V3:=0; V4:=0;
Result:='';
if infoSize <> 0 then
	begin
	GetMem(VerBuf, InfoSize);
	try
	if GetFileVersionInfo(PChar(Filename), wnd, infoSize, VerBuf) then
		begin
		VerQueryValue(VerBuf, '\', Pointer(FileInfo), VerSize);
		V1:=FileInfo.dwFileVersionMS div $10000;
		V2:=FileInfo.dwFileVersionMS and $0FFFF;
		V3:=FileInfo.dwFileVersionLS div $10000;
		V4:=FileInfo.dwFileVersionLS and $0FFFF;
		result:= IntToStr(V1)+'.'+ IntToStr(V2)+'.'+ IntToStr(V3)+'.'+  IntToStr(V4);
		end;
	finally FreeMem(VerBuf); end;
	end;
end;



function GetDLLVersion(DLLName: string; var V1,V2,V3,V4: word): string;
{Get specified DLL's version number}
var hInstDLL: THandle;
var Filename: string;
var L: integer;
begin
hInstDll:=LoadLibrary(PChar(DllName));
L:=MAX_PATH + 1;
SetLength(Filename,L);
L:=GetModuleFileName(hInstDll, Pointer(Filename),L);
SetLength(Filename,L);
Result:=GetFileVersion(Filename,V1,V2,V3,V4);
end;

{------------------------------------------------------------------------------}



function SetPrivilege(hToken: THandle; Privilege: LPCTSTR; bEnablePrivilege: Boolean): Boolean;
{Set privilege for specified token}
var tp: TOKEN_PRIVILEGES;
var luid: TLargeInteger;
var tpPrevious: TOKEN_PRIVILEGES;
var cbPrevious: DWORD;
var dwWritten: DWORD;
begin
Result:=False;
cbPrevious:=sizeof(TOKEN_PRIVILEGES);
if not LookupPrivilegeValue(nil, Privilege, luid) then exit;
tp.PrivilegeCount:=1;
tp.Privileges[0].Luid:=luid;
tp.Privileges[0].Attributes:=0;

AdjustTokenPrivileges(hToken, False, tp, sizeof(TOKEN_PRIVILEGES), tpPrevious, cbPrevious);

if GetLastError<>ERROR_SUCCESS then exit;
tpPrevious.PrivilegeCount:=1;
tpPrevious.Privileges[0].Luid:=luid;
if bEnablePrivilege then
	begin
	tpPrevious.Privileges[0].Attributes:=tpPrevious.Privileges[0].Attributes or (SE_PRIVILEGE_ENABLED);
	end
else
	begin
	tpPrevious.Privileges[0].Attributes:=tpPrevious.Privileges[0].Attributes xor (SE_PRIVILEGE_ENABLED and tpPrevious.Privileges[0].Attributes);
	end;

AdjustTokenPrivileges(hToken, False, tpPrevious, cbPrevious, nil, dwWritten);
if GetLastError<>ERROR_SUCCESS then exit;
Result:=True;
end;



function GetProcessIdByName(const ProcName: string): DWORD;
{Get process ID for first process that matches the name }
var Process32: TProcessEntry32;
var SHandle: THandle;
var Next: BOOL;
begin
Result:=0;

Process32.dwSize := SizeOf(TProcessEntry32);
SHandle := CreateToolHelp32Snapshot(TH32CS_SNAPPROCESS, 0);
if Process32First(SHandle, Process32) then
	begin
	repeat
		begin
		Next := Process32Next(SHandle, Process32);
		if AnsiCompareText(Process32.szExeFile, Trim(ProcName))=0 then
			begin
			Result:=Process32.th32ProcessID;
			break;
			end;
		end
	until not Next;
	end;
CloseHandle(SHandle);
end;



function ProcessIDToFilename(ProcID: DWORD; var Error: string): string;
{Return full path for specified process ID}
{If result is empty, Error string indicates cause}
var hProcess: THandle;
var hToken: THandle;
begin
Error:='';
if not OpenThreadToken(GetCurrentThread, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, False, hToken) then
	begin
	{OpenThreadToken failed - exit if not because of ERROR_NO_TOKEN}
	if GetLastError<>ERROR_NO_TOKEN then begin Error:='Error: OpenThreadToken: '+ SysErrorMessage(GetLastError); exit; end;
	{Thread doesn't have token, use impersonation to force one to be generated}
	if not ImpersonateSelf(SecurityImpersonation) then begin Error:='Error: ImpersonateSelf: '+ SysErrorMessage(GetLastError); exit; end;
	{Now try to get Thread Token again }
	if not OpenThreadToken(GetCurrentThread, TOKEN_ADJUST_PRIVILEGES or TOKEN_QUERY, False, hToken) then begin Error:='Error: OpenThreadToken: '+ SysErrorMessage(GetLastError); exit; end;
	end;
try
{Set debug privileges for this thread}
if not SetPrivilege(hToken, SE_DEBUG_NAME, True) then begin Error:='Error: SetPrivilege: '+ SysErrorMessage(GetLastError); exit; end;

{ OpenProcess }
hProcess:=OpenProcess(PROCESS_ALL_ACCESS, False, ProcID);
if hProcess=0 then begin Error:='Error: OpenProcess: '+ SysErrorMessage(GetLastError); exit; end;
try
{Get filename}
SetLength(Result, MAX_PATH);
if GetModuleFileNameEx(hProcess,0, PChar(Result), MAX_PATH) > 0 then
	begin
	SetLength(Result,StrLen(PChar(Result)));
	end
else begin Error:='Error: GetModuleFileNameEx: '+ SysErrorMessage(GetLastError); exit; end;
finally CloseHandle(hProcess); end;

{ Disable debug privilege }
finally
 SetPrivilege(hToken, SE_DEBUG_NAME, False);
 CloseHandle(hToken);
 end;
end;





function IsProgramRunning(const FullPath: string; UseFullPath: boolean): boolean;
{Return true if the specified program is running}
{If UseFullPath is true, the full path of the program must match}
var Process32: TProcessEntry32;
var SHandle: THandle;
var Next: BOOL;
var ProgName,FN,Error: string;
begin
Result:=False;
ProgName:=ExtractFilename(FullPath);
Process32.dwSize := SizeOf(TProcessEntry32);
SHandle := CreateToolHelp32Snapshot(TH32CS_SNAPPROCESS, 0);
try
if Process32First(SHandle, Process32) then
	begin
	repeat
		begin
		Next := Process32Next(SHandle, Process32);
		if AnsiCompareText(Process32.szExeFile, Trim(ProgName))=0 then
			begin
			if UseFullPath then
				begin
				FN:=ProcessIDToFilename(Process32.th32ProcessID,Error);
				if AnsiCompareText(FN, Trim(FullPath))=0 then begin Result:=True; break; end;
				end
			else begin Result:=True; break; end;
			end;
		end
	until not Next;
	end;
finally CloseHandle(SHandle); end;
end;


procedure KillProcessByName(filename: string);
{Kill all processes with specified name}
var hSnapShot,hProcess: THandle;
var pEntry: PROCESSENTRY32;
var hRes: boolean;
begin
hSnapShot:=CreateToolhelp32Snapshot(TH32CS_SNAPALL, 0);
pEntry.dwSize:=sizeof (pEntry);
hRes:=Process32First(hSnapShot, pEntry);
while (hRes) do
	begin
	if AnsiCompareText(pEntry.szExeFile,filename)=0 then
		begin
		hProcess:=OpenProcess(PROCESS_TERMINATE, false, pEntry.th32ProcessID);
		if hProcess <> 0 then
			begin
			TerminateProcess(hProcess, 9);
			CloseHandle(hProcess);
			end;
		end;
	hRes:=Process32Next(hSnapShot, pEntry);
	end;
CloseHandle(hSnapShot);
end;



{------------------------------------------------------------------------------}


function GetLocalComputerName: string;
{Return local computer name in string}
var Count: DWORD;
begin
Count := MAX_COMPUTERNAME_LENGTH + 1;
SetLength(Result, Count);
GetComputerName(PChar(Result), Count);
SetLength(Result, Count);
end;

{------------------------------------------------------------------------------}



function GetFontTypeStr(FT: integer): string;
begin
case FT of
 DEVICE_FONTTYPE: Result:='Device';
 RASTER_FONTTYPE: Result:='Raster';
 TRUETYPE_FONTTYPE: Result:='TrueType';
 else Result:='Unknown';
 end;
end;



function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
var F: TFontExt;
var H: HFont;
begin
if TStrings(Data^).IndexOf(LogFont.lfFaceName)=-1 then
	begin
	F:=TFontExt.Create;
	F.TextMetric:=TextMetric;
	F.LogFont:=LogFont;
	F.FontType:=FontType;
	F.Handle:=CreateFontIndirect(LogFont);
	TStrings(Data^).AddObject(LogFont.lfFaceName,F);
	end;
Result := 1;
end;



procedure GetFontStrings(Strings: TStrings);
var DC: HDC;
var LF: TLogFont;
begin
Strings.Clear;
ZeroMemory(@LF,sizeof(LF));
LF.lfCharSet:=DEFAULT_CHARSET;
LF.lfPitchAndFamily:=0;
DC:= GetDC(0);
EnumFontFamiliesEx(DC,LF,@EnumFontsProc,integer(@Strings),0);
ReleaseDC(0,DC);
end;


procedure GetFontList(ListBox: TListBox);
begin
GetFontStrings(ListBox.Items);
end;


{------------------------------------------------------------------------------}

procedure OpenRegedit(Key: string);
{Open Regedit on the specified Key}
var i, n: Integer;
var hWin: HWND;
var ExecInfo: ShellExecuteInfoA;
begin
hWin := FindWindowA(PAnsiChar('RegEdit_RegEdit'), nil);
{if Regedit isn't runing then  launch it}
if hWin = 0 then
	begin
	FillChar(ExecInfo, SizeOf(ExecInfo), #0);
	ExecInfo.cbSize:=60;
	ExecInfo.fMask:=SEE_MASK_NOCLOSEPROCESS;
	ExecInfo.lpVerb:=PAnsiChar('open');
	ExecInfo.lpFile:=PAnsiChar('regedit.exe');
	ExecInfo.nShow:=1;
	ShellExecuteExA(@ExecInfo);
	WaitForInputIdle(ExecInfo.hProcess, 200);
	hWin := FindWindowA(PAnsiChar('RegEdit_RegEdit'), nil);
	end;
ShowWindow(hWin, SW_SHOWNORMAL);
hWin := FindWindowExA(hWin, 0, PAnsiChar('SysTreeView32'), nil);
SetForegroundWindow(hWin);
i := 30;
for I:=1 to 30 do
	begin
	SendMessageA(hWin, WM_KEYDOWN, VK_LEFT, 0);
	end;
Sleep(500);
SendMessageA(hWin, WM_KEYDOWN, VK_RIGHT, 0);
Sleep(500);
I := 1;
for I:=1 to Length(Key) do
	begin
	if Key[i] = '\' then
		begin
		SendMessageA(hWin, WM_KEYDOWN, VK_RIGHT, 0);
		Sleep(500);
		end
	else SendMessageA(hWin, WM_CHAR, Integer(Key[i]), 0);
	end ;
end;

{------------------------------------------------------------------------------}


function AscentToHeight(Canvas: TCanvas; Ascent: integer): integer;
{Get the necessary font-height to produce an ascent of a certain value}
{Ascent is the height of the font, from baseline to baseline}
var LC: TCanvas;
var TM: TTextMetric;
begin
LC:=TCanvas.Create;
try
LC.Handle:=CreateCompatibleDC(Canvas.Handle);
LC.Font.Assign(Canvas.Font);
LC.Font.Height:=100;
GetTextMetrics(LC.Handle, TM);
Result:=MulDiv(Ascent,TM.tmHeight,TM.tmAscent);
finally LC.Free; end;
end;




function HeightToAscent(Canvas: TCanvas; Height: integer): double;
{Get the necessary ascent  to produce a font-height of a certain value}
{Ascent is the height of the font, from baseline to baseline}
var LC: TCanvas;
var TM: TTextMetric;
begin
LC:=TCanvas.Create;
try
LC.Handle:=CreateCompatibleDC(Canvas.Handle);
LC.Font.Assign(Canvas.Font);
LC.Font.Height:=200;
GetTextMetrics(LC.Handle, TM);
Result:=Height * (TM.tmAscent/TM.tmHeight);
finally LC.Free; end;
end;



function GetNearestTrueTypeName(Canvas: TCanvas): string;
var LC: TCanvas;
var LogFont: TLogFont;
var FaceName: array[0..255] of char;
begin
LC:=TCanvas.Create;
try
LC.Handle:=CreateCompatibleDC(Canvas.Handle);
LC.Font.Assign(Canvas.Font);
{Get logical font info from the Delphi canvas}
if GetObject(Canvas.Font.Handle, SizeOf(LogFont), @LogFont) <> 0 then
	begin
	{Force font to be TrueType}
	Logfont.lfOutPrecision:=OUT_TT_ONLY_PRECIS;
	LC.Font.Handle:=CreateFontIndirect(LogFont);
	{Creating the handle doesn't change the FaceName, so get the face name }
	GetTextFace(LC.Handle, 256, FaceName);
	Result:=FaceName;
	end;
finally LC.Free; end;
end;




{------------------------------------------------------------------------------}

const TrueTypeFlag=$04;
const TrueTypeMask=$06;

function IsTrueTypeFont(DC: HDC): Boolean;
var LMetric: TTextMetric;
begin
LMetric.tmCharSet:=0;
GetTextMetrics(DC, LMetric);
Result:=(LMetric.tmPitchAndFamily and TrueTypeFlag)<>0;
end;




function IsTrueTypeFont(Canvas: TCanvas): Boolean;
begin
Result:=IsTrueTypeFont(Canvas.Handle);
end;



function IsTrueTypeFont(Font: TFont): Boolean;
var Canvas: TCanvas;
begin
Canvas:=TCanvas.Create;
Canvas.Handle:=GetDC(0);
Canvas.Font:=Font;
Result:=IsTrueTypeFont(Canvas.Handle);
Canvas.Free;
end;

{------------------------------------------------------------------------------}



function GetKeyboardDynamics(var Delay, Speed: integer): boolean;
begin
Result:=SystemParametersInfo(SPI_GETKEYBOARDSPEED, 0, @Speed, 0);
Result:=SystemParametersInfo(SPI_GETKEYBOARDDELAY, 0, @Delay, 0);
end;




function SetKeyboardDynamics(var Delay, Speed: integer): boolean;
begin
Result:=SystemParametersInfo(SPI_SETKEYBOARDSPEED, Speed, Pointer(0), SPIF_UPDATEINIFILE);
Result:=SystemParametersInfo(SPI_SETKEYBOARDDELAY, Delay, Pointer(0), SPIF_UPDATEINIFILE);
end;


{------------------------------------------------------------------------------}


function FilenameToProcessID(Filename: string): integer;
{Get Process ID from Exe Filename}
var Next: Boolean;
var ToolHand: tHandle;
var PE: TProcessEntry32;
begin
ToolHand:=CreateToolHelp32SnapShot(TH32CS_SNAPALL, 0);
PE.dwSize:=SizeOf(PE);
Next:=Process32First(ToolHand, PE);
if Next and not SameText(PE.szExeFile,Filename) then
	begin
	repeat Next := Process32Next(ToolHand, PE);
	until (not Next) or SameText(PE.szExeFile,Filename);
	end;
if Next then Result := PE.th32ProcessID
else Result := 0;
CloseHandle(ToolHand);
end;

{Data passed back from call back routine}

type TCallBackData = record
 ProcessID: integer;
 Handle: HWnd;
 end;

function IsMainwindow(Handle: HWND): boolean;
{Test if specified Windows is the main Window}
begin
{Its owner will have a handle of zero and it is always visible}
Result:=(GetWindow(handle, GW_OWNER) = 0) and IsWindowVisible(Handle);
end;



function HandleTopWindow(Wnd: HWND; var Data: TCallBackData): Boolean; stdcall;
{Callback routine for EnumWindows call}
var ProcessID: integer;
begin
Result:=True;
ProcessID:=0;
GetWindowThreadProcessId(Wnd, @ProcessID);
{Returning True tells the iteration to continue }
if (Data.PROCESSID<>ProcessID) or not IsMainwindow(Wnd) then exit;
Data.Handle:=Wnd;
Result:=False;
end;


function ProcessIDToHandle(ProcessID: integer): HWnd;
{Convert a Process ID to and Windows Handle}
var Data: TCallBackData;
begin
Data.ProcessID:=ProcessID;
Data.Handle:=0;
EnumWindows(@HandleTopWindow, integer(@Data));
Result:=Data.Handle;
end;


function FileToWinHandle(FileName: string): HWND;
{Convert Exe Filename to Windows Handle}
{Filename should not include path}
var PID: integer;
begin
PID:=FilenameToProcessID(FileName);
Result:=ProcessIDToHandle(PID);
end;



{------------------------------------------------------------------------------}


initialization
 begin
 KernelHandle := GetModuleHandle('KERNEL32');
 if KernelHandle <> 0 then @GetLongPathName := GetProcAddress(KernelHandle, 'GetLongPathNameA');
 end;
end.
