unit XEDMacroLib;

interface

uses Forms,Windows,Classes,SysUtils,IniFiles,DSupport,CmdLine,
     XEDScoopReg, ScrollEditor,XEDExpressEval,XEDGlobal;

{ Command line array variables }

const CmdArraySize=32;



type TBookMark = record
 Loc,Offset: TPoint;
 end;


type TMacroDecoder = class(TObject)
 private
  FOnStatusUpdate: TStatusEvent;
  FOnChange: TNotifyEvent;
  FOnSave: TNotifyEvent;
  procedure RemoveLeadingEscape(var S: string);
  function GetUnsignedNumber(var S: string): integer;
  function GetHexNumber(var S: string): integer;
  procedure HandleSign(var S: string);
  function GetCount(var S: string): integer;
  function DoIterate(var Counter: integer): boolean;
  procedure DisplayStatus(S: string);
  function DoImmediateCmd(Counter: integer; var S: string): boolean;
  procedure InsertMacro(var Counter: integer; var S: string);
  procedure DeleteMacro(var Counter: integer; var S: string);
  procedure ForwardLineDeleteMacro(var Counter: integer; var S: string);
  procedure MacroInsertScoop(var Counter: integer; var S: string);
  procedure EraseScoop(var Counter: integer; var S: string);
  function BasicSearch(S: string): boolean;
  function ReplaceMacro(var Counter: integer; var S: string): boolean;
  function SearchMacro(var Counter: integer; var S: string): boolean;
  function InsertHex(var Counter: integer; var S: string): boolean;
  procedure UnknownCommand(var S: string);
  procedure CallCommandLine(var Counter: integer; var S: string);
  procedure Change;
  function ExecuteBlock(var Counter: integer; var S: string): boolean;
  function GetBlock(var S: string): string;
  procedure ExecuteExpression(var Counter: integer; var S: string);
  function GetExpression(var S: string): string;
  function HandleInternalStatus(var S: TXedError): boolean;
  function DoDecode(Macro: string): boolean;
  procedure SaveHandler(Sender: TObject);
  procedure EmptyX;
  procedure NextMark;
  procedure EditorToCmdLine(var S: string);
 protected
 public
  LoopNesting: integer;
  BreakFlag: boolean;
  FwdSearch: boolean;
  Negative: boolean;
  XRegister: TXRegister;
  ExpressEval: TXEDExpression;
  Editor,CmdEditor: TScrollEditor;
  CmdArray: array [0..CmdArraySize-1] of String;
  CmdArrayInx: integer;
  BookMarks: array [0..1] of TBookMark;
  BookMarkInx: integer;
  constructor Create(MainEdit,CmdEdit: TScrollEditor);
  destructor Destroy; override;
  procedure DecCmdArrayInx;
  procedure IncCmdArrayInx;
  procedure DropLocMarker;
  procedure GoToLocMarker;
  procedure ResetLocMarkers;
  function DecodeMacro(Macro: string): boolean;
  function ExtendedImmediate(C: char): boolean;
  property OnStatusUpdate: TStatusEvent read FOnStatusUpdate write FOnStatusUpdate;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property OnSave: TNotifyEvent read FOnSave write FOnSave;
 published
 end;


implementation



{ ---------------- Handle Marking Commands ------------------------------ }


procedure TMacroDecoder.NextMark;
{ Advance To Next Marker }
begin
if BookMarkInx<1 then BookMarkInx:=1 else BookMarkInx:=0;
end;


procedure TMacroDecoder.ResetLocMarkers;
{Reset marker data}
begin
BookMarkInx:=0;
BookMarks[0].Loc:=Point(0,0);
BookMarks[1].Loc:=Point(0,0);
end;


procedure TMacroDecoder.DropLocMarker;
{ Save A Marker Of The Current Location }
begin
BookMarks[BookMarkInx].Loc:=Editor.WorkingCursor.Position;
BookMarks[BookMarkInx].Offset:=Editor.ScrollOffset;
NextMark;
end;



procedure  TMacroDecoder.GoToLocMarker;
{ Move Cursor To Last Location }
begin
NextMark;
if not Editor.SetCursorPosition(BookMarks[BookMarkInx].Loc) then DisplayStatus('Range?');
if not Editor.SetScrollPosition(BookMarks[BookMarkInx].Offset) then DisplayStatus('Scroll?');
Editor.Invalidate;
end;


{------------------------------------------------------------------------------}


procedure TMacroDecoder.IncCmdArrayInx;
begin
Inc(CmdArrayInx);
if CmdArrayInx>=CmdArraySize then CmdArrayInx:=0;
end;


procedure TMacroDecoder.DecCmdArrayInx;
begin
Dec(CmdArrayInx);
if CmdArrayInx<0 then CmdArrayInx:=CmdArraySize-1;
end;


{------------------------------------------------------------------------------}


procedure TMacroDecoder.DisplayStatus(S: string);
begin
if Assigned(FOnStatusUpdate) then FOnStatusUpdate(S,Self);
end;



procedure TMacroDecoder.Change;
begin
if Assigned(FOnChange) then FOnChange(Self);
end;


{------------------------------------------------------------------------------}

procedure TMacroDecoder.RemoveLeadingEscape(var S: string);
begin
while (Length(S)>0) and (S[1]=ESC_CHAR) do Delete(S,1,1);
end;



function TMacroDecoder.GetUnsignedNumber(var S: string): integer;
var I: integer;
var S2: string;
begin
S2:='';
for I:=1 to Length(S) do
 if IsDigit(S[I]) then S2:=S2+S[I] else break;
Result:=StrToIntDef(S2,0);
Delete(S,1,Length(S2));
end;



function TMacroDecoder.GetHexNumber(var S: string): integer;
var I: integer;
var S2: string;
begin
S2:='';
for I:=1 to Length(S) do
 if IsHexDigit(S[I]) then S2:=S2+S[I] else break;
Result:=StrToIntDef('$'+S2,0);
Delete(S,1,Length(S2));
end;



procedure TMacroDecoder.HandleSign(var S: string);
begin
Negative:=S[1]='-';
Delete(S,1,1);
end;


function TMacroDecoder.GetCount(var S: string): integer;
begin
Result:=GetUnsignedNumber(S);
if Result<1 then Result:=1;
end;

{------------------------------------------------------------------------------}



function TMacroDecoder.DoIterate(var Counter: integer): boolean;
begin
Application.ProcessMessages;
Result:=(Counter>0) and not BreakFlag;
Dec(Counter);
end;


procedure TMacroDecoder.SaveHandler(Sender: TObject);
begin
if Assigned(FOnSave) then FOnSave(Sender);
end;



procedure TMacroDecoder.EmptyX;
{Empty X register into screen}
begin
XRegister.InsertScoop;
XRegister.Initialize;
Editor.ResetMarkers;
end;



function TMacroDecoder.ExtendedImmediate(C: char): boolean;
begin
Result:=True;
case C of
 CtrlF: XRegister.DoScoopOff(Editor);
 CtrlP: SaveHandler(Self);
 CtrlR: XRegister.DoScoopOn(Editor);
 CtrlY: EmptyX;
 CtrlRtBrkt: begin Editor.Searcher.IgnoreCase:=not Editor.Searcher.IgnoreCase; Change; end;
 CtrlUScore: begin FwdSearch:=not FwdSearch; Change; end;
 else Result:=False;
 end;
end;




function TMacroDecoder.DoImmediateCmd(Counter: integer; var S: string): boolean;
var Shift: boolean;
begin
Shift:=(GetAsyncKeyState(VK_SHIFT) and $8000)<>0;
Result:=True;
while Result and DoIterate(Counter) do
	begin
	Result:=ExtendedImmediate(S[1]);
	if not Result then Result:=Editor.DoXEDControlKeys(S[1],Shift);
	end;
if Result then Delete(S,1,1);
end;



procedure TMacroDecoder.InsertMacro(var Counter: integer; var S: string);
var S1: string;
var I: integer;
begin
I:=2;
S1:=ExtractToken(S,Esc_Char,I);
Delete(S,1,I-1);
while DoIterate(Counter) do
	begin
	Editor.InsertString(S1);
	end;
end;




procedure TMacroDecoder.DeleteMacro(var Counter: integer; var S: string);
begin
Delete(S,1,1);
while DoIterate(Counter) do Editor.BackDelete;
end;




procedure TMacroDecoder.ForwardLineDeleteMacro(var Counter: integer;  var S: string);
begin
Delete(S,1,1);
while DoIterate(Counter) do Editor.DelForwardLine;
end;



procedure TMacroDecoder.MacroInsertScoop(var Counter: integer;  var S: string);
begin
Delete(S,1,1);
while DoIterate(Counter) do Editor.InsertString(XRegister.GetString);
end;



procedure TMacroDecoder.EraseScoop(var Counter: integer;  var S: string);
begin
Delete(S,1,1);
XRegister.Initialize;
end;



function TMacroDecoder.BasicSearch(S: string): boolean;
begin
if Negative then FwdSearch:=False;
Editor.Searcher.WholeWordMode:=False;
if FwdSearch then Result:=Editor.ForwardSearch(S)
else Result:=Editor.BackwardSearch(S);
Change;
end;



function TMacroDecoder.ReplaceMacro(var Counter: integer; var S: string): boolean;
var S1,S2: string;
var I: integer;
begin
Result:=False;
I:=2;
S1:=ExtractToken(S,Esc_Char,I);
S2:=ExtractToken(S,Esc_Char,I);
Delete(S,1,I-1);
while DoIterate(Counter) do
	begin
	Result:=BasicSearch(S1);
	if Result then
		begin
		Editor.Highlight.NonDelete:=False;
		if S2='' then Editor.DeleteHighlighted
		else Editor.InsertString(S2);
		end
	else
		begin
	 	DisplayStatus('Not Found: "'+S1+'"');
		break;
		end;
	end;
end;


function TMacroDecoder.SearchMacro(var Counter: integer; var S: string): boolean;
var S1: string;
var I: integer;
begin
Result:=False;
I:=2;
S1:=ExtractToken(S,Esc_Char,I);
Delete(S,1,I-1);
while DoIterate(Counter) do
	begin
	Result:=BasicSearch(S1);
	if not Result then
		begin
	 	DisplayStatus('Not Found: "'+S1+'"');
		break;
		end;
	end;
end;



function TMacroDecoder.InsertHex( var Counter: integer; var S: string): boolean;
{ Insert Literal Hex Value }
var H: Integer;
begin
Result:=False;
Delete(S,1,1);
try
if not IsHexDigit(S[1]) then Abort;
H:=GetHexNumber(S);
while DoIterate(Counter) do Editor.InsertChar(char(H));
except DisplayStatus('Hex Number?'); Result:=False; end;
end;



procedure TMacroDecoder.UnknownCommand(var S: string);
begin
DisplayStatus('Unknown Command: '+S[1]);
Delete(S,1,1);
end;




procedure TMacroDecoder.CallCommandLine(var Counter: integer; var S: string);
{ Call A Command Line (recursive) }
var CmdInx: integer;
begin
Delete(S,1,1);
if IsDigit(S[1]) then CmdInx:=GetUnsignedNumber(S)
else begin DisplayStatus('Missing Number Argument'); exit; end;

if CmdInx=CmdArrayInx then DisplayStatus('Infinite Loop')
else if (CmdInx>=0) and (CmdInx<CmdArraySize) then
	begin
	while DoIterate(Counter) do DoDecode(CmdArray[CmdInx]);
	end
else DisplayStatus('Invalid Command Line Number');
end;


function TMacroDecoder.GetBlock(var S: string): string;
var I,BB: integer;
var C: Char;
var StrFlg: boolean;
const StrStart = ['f','F','s','S','i','I'];
begin
Result:='';
BB:=1;
StrFlg:=False;
for I:=1 to Length(S) do
	begin
	C:=S[I];
	if C in StrStart then StrFlg:=true
	else if C=ESC_Char then StrFlg:=false;
	if not StrFlg then
	 case C of
	  '{': Inc(BB);
	  '}': begin Dec(BB); if BB<1 then Break; end;
	  end;
	Result:=Result+C;
	end;
Delete(S,1,Length(Result));
if S[1]='}' then Delete(S,1,1);
end;



function TMacroDecoder.ExecuteBlock(var Counter: integer; var S: string): boolean;
{Execute command between braces (recursive) }
var Block: string;
begin
Delete(S,1,1);
Block:=GetBlock(S);
while DoIterate(Counter) do
	begin
	Inc(LoopNesting);
	Result:=DoDecode(Block);
	Dec(LoopNesting);
	if not Result then break;
	end;
Counter:=0;
end;




function TMacroDecoder.GetExpression(var S: string): string;
var I,BB: integer;
var C: Char;
begin
Result:='';
BB:=0;
for I:=1 to Length(S) do
	begin
	C:=S[I];
	case C of
	 '(': Inc(BB);
	 ')': Dec(BB);
	 end;
	Result:=Result+C;
	if BB<1 then Break;
	end;
Delete(S,1,Length(Result));
if (Length(S)>0) and (S[1]=')') then Delete(S,1,1);
end;


procedure TMacroDecoder.ExecuteExpression(var Counter: integer; var S: string);
{Execute command between braces (recursive) }
var Exp: string;
begin
Exp:=GetExpression(S);
while DoIterate(Counter) do ExpressEval.EvaluateExpression(Exp);
end;



function TMacroDecoder.HandleInternalStatus(var S: TXedError): boolean;
{ Show internal editor errors }
begin
Result:=False;
case S of
 xeNone: DisplayStatus('Ok');
 xeNotFound: begin DisplayStatus('Not Found'); Result:=True; end;
 xeNoUndo: DisplayStatus('Undo Empty');
 xeNoRedo: DisplayStatus('Redo Empty');
 xeCursor: begin DisplayStatus('Bad Position'); Result:=True; end;
 xeStart: begin DisplayStatus('Start'); Result:= true; end;
 xeEnd: begin DisplayStatus('End'); Result:= true; end;
 end;
S:=xeNone;
end;


procedure DeleteToEndOfLoop(var S: string);
{Delete to the last character of loop}
var I: integer;
begin
I:=Pos(S,'}');
if I>0 then Delete(S,1,I);
end;



procedure TMacroDecoder.EditorToCmdLine(var S: string);
{ Copy Text Into Command Line }
var L: integer;
var S2: string;
const DoubleEscape = ESC_Char + ESC_Char;
begin
Delete(S,1,1);
Editor.GetCharsAfterCursor(S2,255);
L:=Pos(DoubleEscape,S2);
if L>0 then SetLength(S2,L) else DisplayStatus('Esc Esc?');
CmdArray[CmdArrayInx]:=S2;
CmdEditor.Text:=S2;
end;


function TMacroDecoder.DoDecode(Macro: string): boolean;
var S: string;
var Counter: integer;
const WhiteSpace = [SP_CHAR,CR_CHAR,ESC_CHAR,'+',']'];
begin
Result:=False;
S:=Macro;
Negative:=False;
Counter:=1;
DisplayStatus('');
while true do
	begin
	RemoveLeadingEscape(S);
	if Length(S)<1 then break;
	if not DoImmediateCmd(Counter,S) then
	case UpperChar(S[1]) of
	 'B': if not ExpressEval.GlobalCommand(Negative,S) then exit;
	 'D': DeleteMacro(Counter,S);
	 'E': EraseScoop(Counter,S);
	 'F': if not ReplaceMacro(Counter,S) then exit;
	 'H': If not InsertHex(Counter,S) then exit;
	 'I': InsertMacro(Counter,S);
	 'J': S:=Macro;
	 'P': CallCommandLine(Counter,S);
	 'R': ForwardLineDeleteMacro(Counter,S);
	 'S': if not SearchMacro(Counter,S) then exit;
	 'X': MacroInsertScoop(Counter,S);
	 'V': EditorToCmdLine(S);
	 '{': ExecuteBlock(Counter,S);
	 '(': ExecuteExpression(Counter,S);
	 '[': if not ExpressEval.DoConditional(S) then break;
	 '_': begin Delete(S,1,1); Counter:= ExpressEval.Global; end;
	 ';': begin if LoopNesting<1 then DisplayStatus('No Loop'); exit; end;
	 '+','-': HandleSign(S);
	 else if IsDigit(S[1]) then Counter:=GetCount(S)
	 else if S[1]='#' then begin Delete(S,1,1); Counter:=High(Integer); end
	 else if S[1] in WhiteSpace then Delete(S,1,1)
	 else if BreakFlag then begin DisplayStatus('Break'); exit; end
	 else begin UnknownCommand(S); exit; end;
	 end;
	if Counter<1 then Counter:=1;
	if HandleInternalStatus(Editor.Error) then exit;
	end;
Result:=True;
end;


function TMacroDecoder.DecodeMacro(Macro: string): boolean;
begin
Editor.Error:=xeNone;
Result:=DoDecode(Macro);
end;



{ TMacroDecoder }


constructor TMacroDecoder.Create(MainEdit,CmdEdit: TScrollEditor);
begin
Editor:=MainEdit;
Self.CmdEditor:=CmdEdit;
XRegister:=TXRegister.Create(Editor);
ExpressEval:=TXEDExpression.Create(Editor,XRegister);
FwdSearch:=True;
end;



destructor TMacroDecoder.Destroy;
begin
ExpressEval.Free;
XRegister.Free;
inherited;
end;


end.

