unit geotiff;

interface

uses Windows,Classes,SysUtils,DSupport,TifGlobal,StreamLib;


{GeoTiff Extension Tags}

const ModelPixelScaleTag = 33550;
const IntergraphMatrixTag = 33920;
const ModelTiepointTag = 33922;
const ModelTransformationTag = 34264;

const GeoKeyDirectoryTag = 34735;
const GeoDoubleParamsTag = 34736;
const GeoAsciiParamsTag = 34737;

{GeoTiff Extension Keys}

const GTModelTypeKey = 1024;
const GTRasterTypeKey = 1025;
const GTCitationKey = 1026;
const GeographicTypeKey = 2048;
const GeogCitationKey = 2049;
const GeogLinearUnitsKey = 2052;

const GeogAngularUnitsKey = 2054;
const ProjectedCSTypeKey = 3072;
const PCSCitationKey = 3073;
const ProjectionKey = 3074;
const ProjCoordTransKey = 3075;
const ProjLinearUnitsKey = 3076;
const ProjStdParallel1Key = 3078;
const ProjStdParallel2Key = 3079;
const ProjNatOriginLongKey = 3080;
const ProjFalseEastingKey = 3082;
const ProjFalseNorthingKey = 3083;
const ProjFalseOriginLongKey = 3084;
const ProjFalseOriginLatKey = 3085;


const MaxItems =20;

type
  T3DVector=record
  X,Y,Z: double;
  end;

type
 TTiePoint = record
 I,J,K,X,Y,Z: double;
 end;

type
 TTiePoints = record
 Count: integer;
 Points: array [0..MaxItems-1] of TTiePoint;
 end;


type
 TKeyHeader = record
 Version, Revision, MinorRev, Count: Word;
 end;

type
TKeyEntry = record
 KeyID, TagLoc, Count, Offset: Word;
 end;

type
 TKeyDirectory = record
 Header: TKeyHeader;
 Count: integer;
 Entries: array [0..MaxItems-1] of TKeyEntry;
 end;


type
 TGeoReals = record
 Count: integer;
 Reals: array [0..MaxItems-1] of double;
 end;



type
 TGeoTiffData = record
 KeyDirectory: TKeyDirectory;
 GeoStrings: AnsiString;
 GeoReals: TGeoReals;
 PixelScale: T3DVector;
 TiePoints: TTiePoints;
 end;


type TTIFFZoneItem = record
 Name: AnsiString;
 Zone1,Zone2: integer;
 Hemisphere: AnsiChar;
 PCS1,PCS2: integer;
 end;

const ZoneTableMax = 59;

const TTIFFZoneTable: array [0..ZoneTableMax-1] of TTIFFZoneItem = (
(Name: 'Adindan UTM'; Zone1: 37; Zone2: 38; Hemisphere: 'N'; PCS1: 20137; PCS2: 20138),
(Name: 'Ain el Abd UTM'; Zone1: 37; Zone2: 39; Hemisphere: 'N'; PCS1: 20437; PCS2: 20439),
(Name: 'Afgooye UTM'; Zone1: 38; Zone2: 39; Hemisphere: 'N'; PCS1: 20538; PCS2: 20539),
(Name: 'Aratu UTM'; Zone1: 22; Zone2: 24; Hemisphere: 'S'; PCS1: 20822; PCS2: 20824),
(Name: 'Batavia UTM'; Zone1: 48; Zone2: 50; Hemisphere: 'S'; PCS1: 21148; PCS2: 21150),
(Name: 'Beijing Gauss'; Zone1: 13; Zone2: 23; Hemisphere: 'N'; PCS1: 21473; PCS2: 21483),
(Name: 'Bogota UTM'; Zone1: 17; Zone2: 18; Hemisphere: 'N'; PCS1: 21817; PCS2: 21818),
(Name: 'Camacupa UTM'; Zone1: 32; Zone2: 33; Hemisphere: 'S'; PCS1: 22032; PCS2: 22033),
(Name: 'Carthage UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'N'; PCS1: 22332; PCS2: 22332),
(Name: 'Corrego Alegre UTM'; Zone1: 23; Zone2: 24; Hemisphere: 'S'; PCS1: 22523; PCS2: 22524),
(Name: 'Douala UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'N'; PCS1: 22832; PCS2: 22832),
(Name: 'ED50 UTM'; Zone1: 28; Zone2: 38; Hemisphere: 'N'; PCS1: 23028; PCS2: 23038),
(Name: 'Fahud UTM'; Zone1: 39; Zone2: 40; Hemisphere: 'N'; PCS1: 23239; PCS2: 23240),
(Name: 'Garoua UTM'; Zone1: 33; Zone2: 33; Hemisphere: 'N'; PCS1: 23433; PCS2: 23433),
(Name: 'ID74 UTM'; Zone1: 46; Zone2: 53; Hemisphere: 'N'; PCS1: 23846; PCS2: 23853),
(Name: 'ID74 UTM'; Zone1: 46; Zone2: 54; Hemisphere: 'S'; PCS1: 23886; PCS2: 23894),
(Name: 'Indian 1954 UTM'; Zone1: 47; Zone2: 48; Hemisphere: 'N'; PCS1: 23947; PCS2: 23948),
(Name: 'Indian 1975 UTM'; Zone1: 47; Zone2: 48; Hemisphere: 'N'; PCS1: 24047; PCS2: 24048),
(Name: 'Kertau UTM'; Zone1: 47; Zone2: 48; Hemisphere: 'N'; PCS1: 24547; PCS2: 24548),
(Name: 'La Canoa UTM'; Zone1: 20; Zone2: 21; Hemisphere: 'N'; PCS1: 24720; PCS2: 24721),
(Name: 'PSAD56 UTM'; Zone1: 18; Zone2: 21; Hemisphere: 'N'; PCS1: 24818; PCS2: 24821),
(Name: 'PSAD56 UTM'; Zone1: 17; Zone2: 20; Hemisphere: 'S'; PCS1: 24877; PCS2: 24880),
(Name: 'Lome UTM'; Zone1: 31; Zone2: 31; Hemisphere: 'N'; PCS1: 25231; PCS2: 25231),
(Name: 'Malongo 1987 UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'S'; PCS1: 25932; PCS2: 25932),
(Name: 'Massawa UTM'; Zone1: 37; Zone2: 37; Hemisphere: 'N'; PCS1: 26237; PCS2: 26237),
(Name: 'Minna UTM'; Zone1: 31; Zone2: 32; Hemisphere: 'N'; PCS1: 26331; PCS2: 26332),
(Name: 'Mhast UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'S'; PCS1: 26432; PCS2: 26432),
(Name: 'M poraloko UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'N'; PCS1: 26632; PCS2: 26632),
(Name: 'M poraloko UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'S'; PCS1: 26692; PCS2: 26692),
(Name: 'NAD27 UTM'; Zone1: 3; Zone2: 22; Hemisphere: 'N'; PCS1: 26703; PCS2: 26722),
(Name: 'NAD27 Alaska'; Zone1: 1; Zone2: 10; Hemisphere: 'N'; PCS1: 26731; PCS2: 26740),
(Name: 'NAD27 Hawaii'; Zone1: 1; Zone2:  5; Hemisphere: 'N'; PCS1: 26761; PCS2: 26765),
(Name: 'NAD27 BLM'; Zone1: 14; Zone2: 17; Hemisphere: 'N'; PCS1: 26774; PCS2: 26777),
(Name: 'NAD83 UTM'; Zone1: 3; Zone2: 23; Hemisphere: 'N'; PCS1: 26903; PCS2: 26923),
(Name: 'NAD83 Alaska'; Zone1: 1; Zone2: 10; Hemisphere: 'N'; PCS1: 26931; PCS2: 26940),
(Name: 'NAD83 Hawaii'; Zone1: 1; Zone2:  5; Hemisphere: 'N'; PCS1: 26961; PCS2: 26965),
(Name: 'Nahrwan 1967 UTM'; Zone1: 38; Zone2: 40; Hemisphere: 'N'; PCS1: 27038; PCS2: 27040),
(Name: 'Naparima UTM'; Zone1: 20; Zone2: 20; Hemisphere: 'N'; PCS1: 27120; PCS2: 27120),
(Name: 'Datum 73 UTM'; Zone1: 29; Zone2: 29; Hemisphere: 'N'; PCS1: 27429; PCS2: 27429),
(Name: 'Point Noire UTM'; Zone1: 32; Zone2: 32; Hemisphere: 'S'; PCS1: 28232; PCS2: 28232),
(Name: 'GDA94 MGA'; Zone1: 48; Zone2: 58; Hemisphere: 'S'; PCS1: 28348; PCS2: 28358),
(Name: 'Pulkovo Gauss'; Zone1: 4; Zone2: 32; Hemisphere: 'N'; PCS1: 28464; PCS2: 28492),
(Name: 'SAD69 UTM'; Zone1: 18; Zone2: 22; Hemisphere: 'N'; PCS1: 29118; PCS2: 29122),
(Name: 'SAD69 UTM'; Zone1: 17; Zone2: 25; Hemisphere: 'S'; PCS1: 29177; PCS2: 29185),
(Name: 'Sapper Hill UTM'; Zone1: 20; Zone2: 21; Hemisphere: 'S'; PCS1: 29220; PCS2: 29221),
(Name: 'Schwarzeck UTM'; Zone1: 33; Zone2: 33; Hemisphere: 'S'; PCS1: 29333; PCS2: 29333),
(Name: 'Sudan UTM'; Zone1: 35; Zone2: 36; Hemisphere: 'N'; PCS1: 29635; PCS2: 29636),
(Name: 'Tananarive UTM'; Zone1: 38; Zone2: 39; Hemisphere: 'S'; PCS1: 29738; PCS2: 29739),
(Name: 'Timbalai 1948 UTM'; Zone1: 49; Zone2: 50; Hemisphere: 'N'; PCS1: 29849; PCS2: 29850),
(Name: 'TC 1948 UTM'; Zone1: 39; Zone2: 40; Hemisphere: 'N'; PCS1: 30339; PCS2: 30340),
(Name: 'Nord Sahara UTM'; Zone1: 29; Zone2: 32; Hemisphere: 'N'; PCS1: 30729; PCS2: 30732),
(Name: 'Yoff UTM'; Zone1: 28; Zone2: 28; Hemisphere: 'N'; PCS1: 31028; PCS2: 31028),
(Name: 'Zanderij UTM'; Zone1: 21; Zone2: 21; Hemisphere: 'N'; PCS1: 31121; PCS2: 31121),
(Name: 'WGS72 UTM'; Zone1: 1; Zone2: 60; Hemisphere: 'N'; PCS1: 32201; PCS2: 32260),
(Name: 'WGS72 UTM'; Zone1: 60; Zone2: 60; Hemisphere: 'S'; PCS1: 32301; PCS2: 32360),
(Name: 'WGS72BE UTM'; Zone1: 1; Zone2: 60; Hemisphere: 'N'; PCS1: 32401; PCS2: 32460),
(Name: 'WGS72BE UTM'; Zone1: 1; Zone2: 60; Hemisphere: 'S'; PCS1: 32501; PCS2: 32560),
(Name: 'WGS84 UTM'; Zone1: 1; Zone2: 60; Hemisphere: 'N'; PCS1: 32601; PCS2: 32660),
(Name: 'WGS84 UTM'; Zone1: 1; Zone2: 60; Hemisphere: 'S'; PCS1: 32701; PCS2: 32760));


var GeoData: TGeoTiffData;


procedure DecodeGeoTiffTag(Stream: TStream; Tag,Count,Offset: integer);
procedure DisplayGeoData(Strings: TStrings; GeoData: TGeoTiffData);
procedure WriteGeoData(Stream: TStream);
procedure WriteGeoTiffTags(Stream: TStream);

function GetWGS72Code(Zone: integer; North: boolean): integer;
function GetWGS84Code(Zone: integer; North: boolean): integer;
procedure DecodePCS(PCSCode: integer; var Name: AnsiString; var Zone: integer; var North: boolean);


implementation



function FindCodeInTable(Code: integer): integer;
{Find Projected Coordinate System code in Table}
begin
for Result:=0 to ZoneTableMax-1 do
 with TTIFFZoneTable[Result] do
	if (Code>=PCS1) and (Code<=PCS2) then exit;
Result:=-1;
end;



procedure DecodePCS(PCSCode: integer; var Name: AnsiString; var Zone: integer; var North: boolean);
{Decode  into UTM zone}
var Inx: integer;
begin
Inx:=FindCodeInTable(PCSCode);
if Inx>-1 then
	begin
	Name:=TTIFFZoneTable[Inx].Name;
	Zone:=TTIFFZoneTable[Inx].Zone1 + (PCSCode - TTIFFZoneTable[Inx].PCS1);
	North:=TTIFFZoneTable[Inx].Hemisphere ='N';
	end
else
	begin
	Zone:=LeftTruncateInteger(PCSCode,2);
	if Zone>60 then Zone:=Zone - 60;
	North:=True;
	Name:='Unknow Zone';
	end;
end;




function GetWGS72Code(Zone: integer; North: boolean): integer;
{Convert zone and north values WGS72 PCS Code}
const PCS_WGS72_UTM_zone_01N = 32201;
const BaseZone = PCS_WGS72_UTM_zone_01N - 1;
begin
Result:=-1;
if Zone<1 then exit;
Result:=BaseZone +Zone;
if not North then Result:=Result + 100;
end;



function GetWGS84Code(Zone: integer; North: boolean): integer;
{Convert zone and north values WGS84 PCS Code}
const PCS_WGS84_UTM_zone_01N = 32601;
const BaseZone = PCS_WGS84_UTM_zone_01N -1;
begin
Result:=-1;
if Zone<1 then exit;
Result:=BaseZone +Zone;
if not North then Result:=Result + 100;
end;



function GetSubString(Start,Size: integer): AnsiString;
var I: integer;
begin
Result:='';
for I:=Start+1 to Start+Size do
	Result:=Result+GeoData.GeoStrings[I];
end;


function GetCoordinateSysStr(PCSCode: integer): AnsiString;
var Name: AnsiString;
var Zone: integer;
var North: boolean;
begin
DecodePCS(PCSCode,Name,Zone,North);
Result:='Projected Coord. Sys: '+IntToStr(PCSCode);
Result:=Result+' '+Name+' '+IntToStr(Zone)+' '+PickString(North,'N','S');
end;


function DecodeKey(KeyEntry: TKeyEntry): AnsiString;
begin
Result:='';
case KeyEntry.KeyId of
 GTModelTypeKey: Result:='Model Coordinate System: '+IntToStr(KeyEntry.Offset);
 GTRasterTypeKey: Result:='Raster Coordinate System: '+IntToStr(KeyEntry.Offset);
 GTCitationKey: Result:='CT Citation: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 GeographicTypeKey: Result:='Ellipsoid: '+IntToStr(KeyEntry.Offset);
 GeogCitationKey: Result:='Geographic Citation: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 GeogLinearUnitsKey: Result:='Geographic Linear Units: '+IntToStr(KeyEntry.Offset);
 GeogAngularUnitsKey: Result:='Angular Units: '+IntToStr(KeyEntry.Offset);
 ProjectedCSTypeKey: Result:=GetCoordinateSysStr(KeyEntry.Offset);
 PCSCitationKey: Result:='PCS Citation: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjectionKey: Result:='Projection: '+IntToStr(KeyEntry.Offset);
 ProjCoordTransKey: Result:='Projection Transform: '+IntToStr(KeyEntry.Offset);
 ProjLinearUnitsKey: Result:='Projection Units: '+IntToStr(KeyEntry.Offset);
 ProjStdParallel1Key: Result:='Projection Parallel-1: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjStdParallel2Key: Result:='Projection Parallel-2: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjNatOriginLongKey: Result:='Origin Longitude: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjFalseEastingKey: Result:='Easting: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjFalseNorthingKey: Result:='Northing: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjFalseOriginLongKey: Result:='Origin Longitude: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 ProjFalseOriginLatKey: Result:='Origin Latitude: '+GetSubString(KeyEntry.Offset,KeyEntry.Count);
 end;
end;



procedure DisplayGeoData(Strings: TStrings; GeoData: TGeoTiffData);
var I: integer;
begin
with GeoData do
 begin
 Strings.Add('Version: '+IntToStr(KeyDirectory.Header.Version));
 Strings.Add('Revision: '+IntToStr(KeyDirectory.Header.Revision));
 Strings.Add('Minor Rev.: '+IntToStr(KeyDirectory.Header.MinorRev));
 Strings.Add('Count: '+IntToStr(KeyDirectory.Header.Count));
 Strings.Add('Keys ------------------------');
 for I:=0 to KeyDirectory.Header.Count-1 do
  with KeyDirectory.Entries[I] do
	begin
	Strings.Add(IntToStr(KeyID)+', '+IntToStr(TagLoc)+', '+IntToStr(Count)+', '+IntToStr(Offset));
	end;
 for I:=0 to KeyDirectory.Header.Count-1 do
  Strings.Add(DecodeKey(GeoData.KeyDirectory.Entries[I]));
 Strings.Add('GeoStrings ----------------------------');
 Strings.Add(GeoStrings);
 Strings.Add('GeoReals ----------------------------');
 for I:=0 to GeoReals.Count-1 do
	Strings.Add(IntToStr(I)+' '+FloatToStr(GeoReals.Reals[I]));
 Strings.Add('PixelScale ----------------------------');
 Strings.Add('X: '+FloatToStr(PixelScale.X)+' Y: '+FloatToStr(PixelScale.Y)+' Z: '+FloatToStr(PixelScale.Z));
 Strings.Add('TiePoints ----------------------------');
 for I:=0 to TiePoints.Count-1 do
	begin
	Strings.Add(IntToStr(I)+
	' I: '+FloatToStr(TiePoints.Points[I].I)+' J: '+FloatToStr(TiePoints.Points[I].J)+' K: '+FloatToStr(TiePoints.Points[I].K)+
	' X: '+FloatToStr(TiePoints.Points[I].X)+' Y: '+FloatToStr(TiePoints.Points[I].Y)+' Z: '+FloatToStr(TiePoints.Points[I].Z));
	end;
 end;
end;





procedure GetASCIIData(Stream: TStream; Count,Offset: integer);
var I,Pos: integer;
var S: AnsiString;
begin
Pos:=Stream.Position;
Stream.Seek(Offset,soFromBeginning);
GeoData.GeoStrings:='';
for I:=0 to Count-1 do S:=S+Char(GetStreamByte(Stream));
GeoData.GeoStrings:=S;
Stream.Seek(Pos,soFromBeginning);
end;




procedure GetRealData(Stream: TStream; Count,Offset: integer);
var I,Pos: integer;
begin
Pos:=Stream.Position;
Stream.Seek(Offset,soFromBeginning);
GeoData.GeoReals.Count:=0;
for I:=0 to Count-1 do
	begin
	GeoData.GeoReals.Reals[I]:=GetStreamDouble(Stream);
	Inc(GeoData.GeoReals.Count);
	end;
Stream.Seek(Pos,soFromBeginning);
end;



procedure GetKeyDirectory(Stream: TStream; Offset: integer);
var I,Pos: integer;
begin
Pos:=Stream.Position;
Stream.Seek(Offset,soFromBeginning);
GeoData.KeyDirectory.Count:=0;
with GeoData.KeyDirectory.Header do
 begin
 Version:=GetStreamWord(Stream);
 Revision:=GetStreamWord(Stream);
 MinorRev:=GetStreamWord(Stream);
 Count:=GetStreamWord(Stream);
 for I:=0 to Count-1 do
  with GeoData.KeyDirectory.Entries[I] do
	begin
	KeyID:=GetStreamWord(Stream);
	TagLoc:=GetStreamWord(Stream);
	Count:=GetStreamWord(Stream);
	Offset:=GetStreamWord(Stream);
	Inc(GeoData.KeyDirectory.Count);
	end;
 end;
Stream.Seek(Pos,soFromBeginning);
end;



procedure GetPixelScale(Stream: TStream; Count,Offset: integer);
var Pos: integer;
begin
Pos:=Stream.Position;
Stream.Seek(Offset,soFromBeginning);
GeoData.PixelScale.X:=GetStreamDouble(Stream);
GeoData.PixelScale.Y:=GetStreamDouble(Stream);
GeoData.PixelScale.Z:=GetStreamDouble(Stream);

Stream.Seek(Pos,soFromBeginning);
end;




procedure GetTiePoint(Stream: TStream; Count,Offset: integer);
var P,Pos: integer;
begin
Pos:=Stream.Position;
Stream.Seek(Offset,soFromBeginning);
GeoData.TiePoints.Count:=0;
for P:=0 to (Count div 6)-1 do
	begin
	GeoData.TiePoints.Points[P].I:=GetStreamDouble(Stream);
	GeoData.TiePoints.Points[P].J:=GetStreamDouble(Stream);
	GeoData.TiePoints.Points[P].K:=GetStreamDouble(Stream);
	GeoData.TiePoints.Points[P].X:=GetStreamDouble(Stream);
	GeoData.TiePoints.Points[P].Y:=GetStreamDouble(Stream);
	GeoData.TiePoints.Points[P].Z:=GetStreamDouble(Stream);
	Inc(GeoData.TiePoints.Count);
	end;

Stream.Seek(Pos,soFromBeginning);
end;



procedure DecodeGeoTiffTag(Stream: TStream; Tag,Count,Offset: integer);
begin
 case Tag of
 ModelPixelScaleTag: GetPixelScale(Stream,Count,Offset);
 ModelTiepointTag: GetTiePoint(Stream,Count,Offset);
 GeoKeyDirectoryTag: GetKeyDirectory(Stream,Offset);
 GeoDoubleParamsTag: GetRealData(Stream,Count,Offset);
 GeoAsciiParamsTag: GetASCIIData(Stream,Count,Offset);
 end;
end;


{------------------------- Code for Writing GeoTiff Tags ----------------------}

type TPosOffset = record
 Offset,Length: integer;
 end;


type TGeoPosOffset = record
 ModelPixelScale, ModelTiepoint,
 GeoKeyDirectory, GeoDoubleParams, GeoAsciiParams: TPosOffset;
 end;

var GeoPosOffset: TGeoPosOffset;



procedure WritePixelScale(Stream: TStream; PixelScale: T3DVector; var PosOffset: TPosOffset);
begin
PosOffset.Offset:=Stream.Position;
PosOffset.Length:=3;
WriteStreamDouble(Stream,PixelScale.X);
WriteStreamDouble(Stream,PixelScale.Y);
WriteStreamDouble(Stream,PixelScale.Z);
end;




procedure WriteTiePoint(Stream: TStream; TiePoints: TTiePoints; var PosOffset: TPosOffset);
var P: integer;
begin
PosOffset.Offset:=Stream.Position;
PosOffset.Length:=6 * TiePoints.Count;
for P:=0 to TiePoints.Count-1 do
	begin
	WriteStreamDouble(Stream,TiePoints.Points[P].I);
	WriteStreamDouble(Stream,TiePoints.Points[P].J);
	WriteStreamDouble(Stream,TiePoints.Points[P].K);
	WriteStreamDouble(Stream,TiePoints.Points[P].X);
	WriteStreamDouble(Stream,TiePoints.Points[P].Y);
	WriteStreamDouble(Stream,TiePoints.Points[P].Z);
	end;
end;



procedure WriteKeyDirectory(Stream: TStream; KeyDirectory: TKeyDirectory; var PosOffset: TPosOffset);
var I: integer;
begin
PosOffset.Offset:=Stream.Position;
with KeyDirectory.Header do
 begin
 PosOffset.Length:=4 * Count;
 WriteStreamWord(Stream,Version);
 WriteStreamWord(Stream,Revision);
 WriteStreamWord(Stream,MinorRev);
 WriteStreamWord(Stream,Count);
 for I:=0 to Count-1 do
  with KeyDirectory.Entries[I] do
	begin
	WriteStreamWord(Stream,KeyID);
	WriteStreamWord(Stream,TagLoc);
	WriteStreamWord(Stream,Count);
	WriteStreamWord(Stream,Offset);
	end;
 end;
end;



procedure WriteRealData(Stream: TStream; GeoReals: TGeoReals; var PosOffset: TPosOffset);
var I: integer;
begin
PosOffset.Offset:=Stream.Position;
PosOffset.Length:=GeoReals.Count;
for I:=0 to GeoReals.Count-1 do
	begin
	WriteStreamDouble(Stream,GeoReals.Reals[I]);
	end;
end;




procedure WriteASCIIData(Stream: TStream; GeoString: AnsiString; var PosOffset: TPosOffset);
var I: integer;
begin
PosOffset.Offset:=Stream.Position;
PosOffset.Length:=Length(GeoString)+1;
for I:=1 to Length(GeoString) do WriteStreamChar(Stream,GeoString[I]);
WriteStreamByte(Stream,0);
end;



procedure WriteGeoData(Stream: TStream);
begin
WritePixelScale(Stream,Geodata.PixelScale,GeoPosOffset.ModelPixelScale);
WriteTiePoint(Stream,Geodata.TiePoints,GeoPosOffset.ModelTiepoint);
WriteKeyDirectory(Stream,Geodata.KeyDirectory,GeoPosOffset.GeoKeyDirectory);
WriteRealData(Stream,Geodata.GeoReals,GeoPosOffset.GeoDoubleParams);
WriteASCIIData(Stream,Geodata.GeoStrings,GeoPosOffset.GeoAsciiParams);
end;




procedure WriteGeoTiffTags(Stream: TStream);
begin
with GeoPosOffset do
 begin
 WriteTIFTag(Stream, ModelPixelScaleTag, tifDouble, ModelPixelScale.Length, ModelPixelScale.Offset);
 WriteTIFTag(Stream, ModelTiepointTag, tifDouble, ModelTiepoint.Length, ModelTiepoint.Offset);
 WriteTIFTag(Stream, GeoKeyDirectoryTag, tifShort, GeoKeyDirectory.Length, GeoKeyDirectory.Offset);
 WriteTIFTag(Stream, GeoDoubleParamsTag, tifDouble, GeoDoubleParams.Length, GeoDoubleParams.Offset);
 WriteTIFTag(Stream, GeoAsciiParamsTag, tifAscii, GeoAsciiParams.Length, GeoAsciiParams.Offset);
 end;
end;



end.
