unit sdtslib;

interface

uses Classes,SysUtils,DSupport;



const LeaderLen = 24;
const MaxEntries = 20;


{----------- Data Descriptive Record (DDR) structures ---------}

{Structure for Data Description Leader}

type
 TDDRLeader = record
 RecLength: integer;
 IntLevel: integer;
 LeaderID: string;
 InlineCodeExt: string;
 ReserveSpace: string;
 AppIndict: string;
 FieldConIndict: integer;
 DDAAddress: integer;
 CodeCharSet: string;
 FieldLenSize: integer;
 FieldPosSize: integer;
 ReserveDigit: string;
 TagSize: integer;
 end;

{Data Description Directory Items}

type
 TDDRDirItem = record
 Tag: string;
 FieldLen: integer;
 FieldPos: integer;
 end;

{Array Data Description Directory Items}

type
 TDDRDirectory = record
 Count: integer;
 DataPos: integer;
 DirArray: array [0..MaxEntries-1] of TDDRDirItem;
 end;

{Data Description Items}

type
 TDataDescriptItem = record
 FieldControl: string;
 FieldName: string;
 Fields: string;
 Format: string;
 end;

{Data Description Area Array}

type
 TDataDescriptArea = record
 Count: integer;
 DescriptArray: array [0..MaxEntries-1] of TDataDescriptItem;
 end;

{Structure for Data Reacord Leader}

type
 TDRLeader = record
 RecLength: integer;
 Reserve1: string;
 LeaderID: string;
 Reserve2: string;
 BaseAddress: integer;
 Reserve3: string;
 FieldLenSize: integer;
 FieldPosSize: integer;
 ReserveDigit: string;
 TagSize: integer;
 end;

{Combined records of Data Record leaders and Directories}

type
 TDataRecordArray = record
 Count: integer;
 DRLeaders: array [0..MaxEntries-1] of TDRLeader;
 DRDirectorys: array [0..MaxEntries-1] of TDDRDirectory;
 end;


{Combined record that includes all description information on the file}

type
 TDataDescription = record
 DDRLeader: TDDRLeader;
 DDRDirectory: TDDRDirectory;
 DataDescriptArea: TDataDescriptArea;
 DataRecords: TDataRecordArray;
 end;

{Record to hold Subfield information }

type
 TFieldItem = record
 Field: string;
 Format: char;
 Size: integer;
 end;

{Array of subfield information}

type
 TFieldArray = record
 Count: integer;
 Items: array [0..MaxEntries-1] of TFieldItem;
 end;

{Array of strings}

type
 TStringArray = record
 Count: integer;
 Strings: array [0..MaxEntries-1] of string;
 end;


function GetDDRDirectoryCount(Leader: TDDRLeader): integer;
procedure ReadDDRLeader(Stream: TStream; var Leader: TDDRLeader);
procedure ReadDDRDirectory(Stream: TStream; var Dir: TDDRDirectory; Leader: TDDRLeader);
procedure ReadDataDirArea(Stream: TStream; var DDA: TDataDescriptArea;  Dir: TDDRDirectory);
procedure ReadDRDirectory(Stream: TStream; var Dir: TDDRDirectory; Leader: TDRLeader);
procedure ReadDRLeader(Stream: TStream; var Leader: TDRLeader);
function ExtractItem(S: String; Sep: char; var P: integer): String;
procedure ExtractFields(Field: string; var FieldArray: TFieldArray);
function FindTag(TagName: string; Dir: TDDRDirectory): integer;
function FindField(FieldName: string; FieldArray: TFieldArray): integer;
function GetFormatByTag(Tag: string; DataDescript: TDataDescription): string;
procedure GetTagFieldIndexes(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer; var TagInx,FieldInx: integer);
function GetStringSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): string;
function GetBinarySubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx,Size: integer): integer;
function GetArraySubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): TStringArray;
function GetIntegerSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): integer;
function GetDoubleSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): double;
procedure ReadDataDescription(Stream: TStream; var DataDescript: TDataDescription; MaxDRecords: integer);

procedure DisplayDDRLeader(Strings: TStrings; Leader: TDDRLeader);
procedure DisplayDirectory(Strings: TStrings; Dir: TDDRDirectory);
procedure DisplayDDRDirectory(Strings: TStrings; Dir: TDDRDirectory);
procedure DisplayDRDirectory(Strings: TStrings; Dir: TDDRDirectory; N: integer);
procedure DisplayDescriptArea(Strings: TStrings; DDA: TDataDescriptArea);
procedure DisplayDRLeader(Strings: TStrings; Leader: TDRLeader; N: Integer);
procedure DisplayDataRecords(Strings: TStrings; DataRecords: TDataRecordArray);


implementation



{--------------- Routines to display data in memo -----------------}


procedure DisplayDDRLeader(Strings: TStrings; Leader: TDDRLeader);
begin
with Leader do
 begin
 Strings.Add('-----------------------------------');
 Strings.Add('DDR Leader');
 Strings.Add('-----------------------------------');
 Strings.Add('Record Length: '+IntToStr(RecLength));
 Strings.Add('Interchange Level: '+IntToStr(IntLevel));
 Strings.Add('Leader ID: '+LeaderID);
 Strings.Add('Inline Code Extension: '+InlineCodeExt);
 Strings.Add('Reserved Space: '+ReserveSpace);
 Strings.Add('Application Indicator: '+AppIndict);
 Strings.Add('Field Control Indicator: '+IntToStr(FieldConIndict));
 Strings.Add('Data Descriptive Address: '+IntToStr(DDAAddress));
 Strings.Add('Code Charactor Set Indicator: '+CodeCharSet);
 Strings.Add('Size Of Field Length: '+IntToStr(FieldLenSize));
 Strings.Add('Size Of Field Position: '+IntToStr(FieldPosSize));
 Strings.Add('Reserved Digit: '+ReserveDigit);
 Strings.Add('Size Of Tag Field: '+IntToStr(TagSize));
 end;
end;



procedure DisplayDirectory(Strings: TStrings; Dir: TDDRDirectory);
var I: integer;
var S: string;
begin
for I:=0 to Dir.Count-1 do
with Dir.DirArray[I] do
 begin
 S:=IntToStr(I)+' Tag: '+Tag+' Field Pos: '+IntToStr(FieldPos)+' Field Length: '+IntToStr(FieldLen);
 Strings.Add(S);
 end;
end;



procedure DisplayDDRDirectory(Strings: TStrings; Dir: TDDRDirectory);
begin
Strings.Add('-----------------------------------');
Strings.Add(' DDR Directory');
Strings.Add('-----------------------------------');
DisplayDirectory(Strings,Dir);
end;



procedure DisplayDRDirectory(Strings: TStrings; Dir: TDDRDirectory; N: integer);
begin
Strings.Add('-----------------------------------');
Strings.Add(IntToStr(N)+' DR Directory');
Strings.Add('-----------------------------------');
DisplayDirectory(Strings,Dir);
end;


procedure DisplayDescriptArea(Strings: TStrings; DDA: TDataDescriptArea);
var I: integer;
var S: string;
begin
 Strings.Add('-----------------------------------');
 Strings.Add('DDR Data Description Area');
 Strings.Add('-----------------------------------');
for I:=0 to DDA.Count-1 do
with DDA.DescriptArray[I] do
 begin
 S:=IntToStr(I)+' Field Control: '+FieldControl+' Field Name: '+FieldName+' Fields: '+Fields+' Format: '+Format;
 Strings.Add(S);
 end;
end;




procedure DisplayDRLeader(Strings: TStrings; Leader: TDRLeader; N: Integer);
begin
with Leader do
 begin
 Strings.Add('-----------------------------------');
 Strings.Add(IntToStr(N)+' DR Leader');
 Strings.Add('-----------------------------------');
 Strings.Add('Record Length: '+IntToStr(RecLength));
 Strings.Add('Reserved 1: '+Reserve1);
 Strings.Add('Leader ID: '+LeaderID);
 Strings.Add('Reserved 2: '+Reserve2);
 Strings.Add('Base Address: '+IntToStr(BaseAddress));

 Strings.Add('Reserved 3: '+Reserve3);
 Strings.Add('Size Of Field Length: '+IntToStr(FieldLenSize));
 Strings.Add('Size Of Field Position: '+IntToStr(FieldPosSize));
 Strings.Add('Reserved Digit: '+ReserveDigit);
 Strings.Add('Size Of Field Tag: '+IntToStr(TagSize));
 end;
end;



procedure DisplayDataRecords(Strings: TStrings; DataRecords: TDataRecordArray);
var I: integer;
begin
for I:=0 to DataRecords.Count-1 do
	begin
 	DisplayDRLeader(Strings,DataRecords.DRLeaders[I],I);
 	DisplayDRDirectory(Strings,DataRecords.DRDirectorys[I],I);
 	Strings.Add('Data Address: '+IntToStr(DataRecords.DRDirectorys[I].DataPos));
	end;
end;



{----------------Low level Stream Handling Routines ---------------}



procedure ReverseBytes(var Value; Size: integer);
{Reverse byte order converting big-endian to little-endian}
var Bytes: PByteArray;
var T: byte;
var I: integer;
begin
Bytes:=@Value;
for I:=0 to (Size div 2)-1 do
	begin
	T:=Bytes^[I];
	Bytes^[I]:=Bytes^[(Size-1)-I];
	Bytes^[(Size-1)-I]:=T;
	end;
end;


function GetStreamString(Stream: TStream; Len: integer): string;
{Get string of specified length from stream}
begin
SetString(Result, nil, Len);
Stream.Read(Pointer(Result)^, Len);
end;



function GetStringInteger(Stream: TStream; Len: integer): integer;
{Get string integer of specified length from stream}
var S: string;
begin
S:=GetStreamString(Stream,Len);
Result:=StrToInt(S);
end;



function ReadToTerminator(Stream: TStream;var Term: Char): string;
{Get string of specified length from stream}
{Terminator is returned in Term}
var I: integer;
begin
Result:='';
for I:=Stream.Position to Stream.Size-1 do
	begin
	Stream.Read(Term, 1);
	if (Term=#$1E) or (Term=#$1F) then break;
	Result:=Result+Term;
	end;
end;



function ReadBinary(Stream: TStream; Bytes: integer): integer;
{Get string of specified length from stream}
begin
Result:=0;
Stream.Read(Result,Bytes);
ReverseBytes(Result,Bytes);
end;



{-------------------Routines to disect SDTS Files -------------------------}


function GetDDRDirectoryCount(Leader: TDDRLeader): integer;
{Calculate the number of entries in the directory}
{By dividing the size of the directory by the entry size}
var EntrySize: integer;
begin
EntrySize:=Leader.FieldLenSize+Leader.FieldPosSize+Leader.TagSize;
Result:=(Leader.DDAAddress-LeaderLen-1) div EntrySize;
end;



procedure ReadDDRLeader(Stream: TStream; var Leader: TDDRLeader);
{Read Data Description}
begin
with Leader do
 begin
 RecLength:=GetStringInteger(Stream,5);
 IntLevel:=GetStringInteger(Stream,1);
 LeaderID:=GetStreamString(Stream,1);
 InlineCodeExt:=GetStreamString(Stream,1);
 ReserveSpace:=GetStreamString(Stream,1);
 AppIndict:=GetStreamString(Stream,1);
 FieldConIndict:=GetStringInteger(Stream,2);
 DDAAddress:=GetStringInteger(Stream,5);
 CodeCharSet:=GetStreamString(Stream,3);
 FieldLenSize:=GetStringInteger(Stream,1);
 FieldPosSize:=GetStringInteger(Stream,1);
 ReserveDigit:=GetStreamString(Stream,1);
 TagSize:=GetStringInteger(Stream,1);
 end;
end;



procedure ReadDDRDirectory(Stream: TStream; var Dir: TDDRDirectory; Leader: TDDRLeader);
{Read Data Descriptive Record Directory}
var I: integer;
var T: byte;
var EntrySize: integer;
begin
EntrySize:=Leader.FieldLenSize+Leader.FieldPosSize+Leader.TagSize;
Dir.Count:=(Leader.DDAAddress-LeaderLen-1) div EntrySize;
for I:=0 to Dir.Count-1 do
 with Dir.DirArray[I] do
	begin
	Tag:=GetStreamString(Stream,Leader.TagSize);
	FieldLen:=GetStringInteger(Stream,Leader.FieldLenSize);
	FieldPos:=GetStringInteger(Stream,Leader.FieldPosSize);
	end;
Stream.Read(T,1);
Dir.DataPos:=Stream.Position;
end;



procedure ReadDataDirArea(Stream: TStream; var DDA: TDataDescriptArea;  Dir: TDDRDirectory);
{Read the Data Descriptive Area data}
var I: integer;
var Term: char;
begin
DDA.Count:=Dir.Count;
for I:=0 to Dir.Count-1 do
 with DDA.DescriptArray[I] do
	begin
	FieldControl:=GetStreamString(Stream,6);
	FieldName:=ReadToTerminator(Stream,Term);
	Fields:=ReadToTerminator(Stream,Term);
	Format:=ReadToTerminator(Stream,Term);
	end;
end;






procedure ReadDRDirectory(Stream: TStream; var Dir: TDDRDirectory; Leader: TDRLeader);
{Read Data Record Directory}
var I: integer;
var T: byte;
var EntrySize: integer;
begin
EntrySize:=Leader.FieldLenSize+Leader.FieldPosSize+Leader.TagSize;
Dir.Count:=(Leader.BaseAddress-LeaderLen-1) div EntrySize;
for I:=0 to Dir.Count-1 do
 with Dir.DirArray[I] do
	begin
	Tag:=GetStreamString(Stream,Leader.TagSize);
	FieldLen:=GetStringInteger(Stream,Leader.FieldLenSize);
	FieldPos:=GetStringInteger(Stream,Leader.FieldPosSize);
	end;
Stream.Read(T,1);
Dir.DataPos:=Stream.Position;
end;


procedure ReadDRLeader(Stream: TStream; var Leader: TDRLeader);
{Read Data Record Leader}
begin
with Leader do
 begin
 RecLength:=GetStringInteger(Stream,5);
 Reserve1:=GetStreamString(Stream,1);
 LeaderID:=GetStreamString(Stream,1);
 Reserve2:=GetStreamString(Stream,5);
 BaseAddress:=GetStringInteger(Stream,5);
 Reserve3:=GetStreamString(Stream,3);
 FieldLenSize:=GetStringInteger(Stream,1);
 FieldPosSize:=GetStringInteger(Stream,1);
 ReserveDigit:=GetStreamString(Stream,1);
 TagSize:=GetStringInteger(Stream,1);
 end;
end;

{---------------Routines to read subfield Items ------------------}


function ExtractItem(S: String; Sep: char; var P: integer): String;
{Extract Item from S, starting at P up to but not including Sep}
{Terminates with P pointing past Sep or past end of string}
var C: char;
begin
Result:='';
while P<=Length(S) do
	begin
	C:=S[P]; Inc(P);
	if C=Sep then break
	else Result:=Result+C;
	end;
end;


procedure ExtractFields(Field: string; var FieldArray: TFieldArray);
{Extract all fields from Field and copy into FieldArray}
var C,P: integer;
begin
P:=1; C:=0;
while true do
	begin
	FieldArray.Items[C].Field:=ExtractItem(Field,'!',P);
	if FieldArray.Items[C].Field='' then break;
	Inc(C);
	end;
FieldArray.Count:=C;
end;


function FindTag(TagName: string; Dir: TDDRDirectory): integer;
{Find specified Tag in directory data}
{Returns -1 if fails }
var I: integer;
begin
Result:=-1;
for I:=0 to Dir.Count-1 do
 if TagName=Dir.DirArray[I].Tag then
	begin Result:=I; break; end;
end;


function FindField(FieldName: string; FieldArray: TFieldArray): integer;
{Find specified Field in Field Array}
{Returns -1 if fails }
var I: integer;
begin
Result:=-1;
for I:=0 to FieldArray.Count-1 do
 if FieldName=FieldArray.Items[I].Field then
	begin Result:=I; break; end;
end;



function GetFormatByTag(Tag: string; DataDescript: TDataDescription): string;
{Find format string based on Tag}
var Inx: integer;
begin
Inx:=FindTag(Tag,DataDescript.DDRDirectory);
Result:=DataDescript.DataDescriptArea.DescriptArray[Inx].Format
end;


procedure GetTagFieldIndexes(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer; var TagInx,FieldInx: integer);
{Get the indexes for the specified TagName and Field}
var DDRTagInx: integer;
var FieldArray: TFieldArray;
begin
TagInx:=-1; FieldInx:=-1;
DDRTagInx:=FindTag(TagName,DataDescript.DDRDirectory);
if DDRTagInx<0 then exit;

ExtractFields(DataDescript.DataDescriptArea.DescriptArray[DDRTagInx].Fields,FieldArray);
FieldInx:=FindField(FieldName,FieldArray);
if FieldInx<0 then exit;

TagInx:=FindTag(TagName,DataDescript.DataRecords.DRDirectorys[Inx]);
end;



function GetStringSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): string;
{Extract field specified in Inx as a string based on TagName and FieldName}
var TagIndex,FieldIndex: integer;
var I: integer;
var Term: char;
begin
Result:='';
GetTagFieldIndexes(Stream, TagName, FieldName, DataDescript, Inx, TagIndex, FieldIndex);
if (TagIndex<0) or (FieldIndex<0) then exit;
Stream.Position:=DataDescript.DataRecords.DRDirectorys[Inx].DataPos+DataDescript.DataRecords.DRDirectorys[Inx].DirArray[TagIndex].FieldPos;
for I:=0 to FieldIndex do Result:=ReadToTerminator(Stream,Term);
end;



function GetBinarySubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx,Size: integer): integer;
{Extract field specified in Inx as a binary integer based on TagName and FieldName}
{Size specifies the number of bytes in the binary number}
var TagIndex,FieldIndex: integer;
var I: integer;
begin
Result:=0;
GetTagFieldIndexes(Stream, TagName, FieldName, DataDescript, Inx, TagIndex, FieldIndex);
if (TagIndex<0) or (FieldIndex<0) then exit;
Stream.Position:=DataDescript.DataRecords.DRDirectorys[Inx].DataPos+DataDescript.DataRecords.DRDirectorys[Inx].DirArray[TagIndex].FieldPos;
for I:=0 to FieldIndex do Result:=ReadBinary(Stream,Size);
end;



function GetArraySubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): TStringArray;
{Extract field specified in Inx as an array of strings based on TagName and FieldName}
{Size specifies the number of bytes in the binary number}
var TagIndex,FieldIndex: integer;
var I: integer;
var S: string;
var Term: char;
begin
Result.Count:=0;
GetTagFieldIndexes(Stream, TagName, FieldName, DataDescript, Inx, TagIndex, FieldIndex);
if (TagIndex<0) or (FieldIndex<0) then exit;
Stream.Position:=DataDescript.DataRecords.DRDirectorys[Inx].DataPos+DataDescript.DataRecords.DRDirectorys[Inx].DirArray[TagIndex].FieldPos;
for I:=0 to FieldIndex-1 do S:=ReadToTerminator(Stream,Term);
repeat
	begin
	Result.Strings[Result.Count]:=ReadToTerminator(Stream,Term);
	Inc(Result.Count);
	end
until Term=#$1E;
end;


function GetIntegerSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): integer;
{Extract field specified in Inx as AsCII integer based on TagName and FieldName}
var S: string;
begin
S:=GetStringSubField(Stream,TagName,FieldName,DataDescript,Inx);
Result:=StrToIntDef(S,0);
end;



function GetDoubleSubField(Stream: TStream; TagName, FieldName: string; DataDescript: TDataDescription; Inx: integer): double;
{Extract field specified in Inx as a ASCII double based on TagName and FieldName}
var S: string;
begin
S:=GetStringSubField(Stream,TagName,FieldName,DataDescript,Inx);
Result:=GetNumberValue(S);
end;



procedure ReadDataDescription(Stream: TStream; var DataDescript: TDataDescription; MaxDRecords: integer);
{Read all descriptive data up to actual data area}
var I,P,Max: integer;
begin
if MaxDRecords>MaxEntries then Max:=MaxEntries
else Max:=MaxDRecords;
with DataDescript do
 begin
 ReadDDRLeader(Stream,DDRLeader);
 ReadDDRDirectory(Stream, DDRDirectory, DDRLeader);
 ReadDataDirArea(Stream,DataDescriptArea,DDRDirectory);
 DataDescript.DataRecords.Count:=0;
 for I:=0 to Max-1 do
	begin
	if Stream.Position>=Stream.Size then break;
	P:=Stream.Position;
	ReadDRLeader(Stream,DataRecords.DRLeaders[I]);
	ReadDRDirectory(Stream, DataRecords.DRDirectorys[I], DataRecords.DRLeaders[I]);
	Stream.Position:=P+DataRecords.DRLeaders[I].RecLength;
	Inc(DataDescript.DataRecords.Count);
	if DataRecords.DRLeaders[I].LeaderID='R' then break;
	end;
 end;
end;



end.
