unit BitCursor;

interface

uses Windows,Classes,Graphics,Types,Controls,Forms;


type
TBitCursor=class(TComponent)
 private
    FImage,FBackImage: TBitmap;
    CursorSet: boolean;
    Left,Top: integer;
    procedure SetPicture(Value: TBitmap);
  public
    CursorX,CursorY: integer;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ShowCursor(Canvas: TCanvas);
    procedure SetCursor(Canvas: TCanvas; X,Y: integer);
    procedure EraseCursor(Canvas: TCanvas);
    procedure MoveCursor(Canvas: TCanvas; X,Y: integer);
    procedure MakeOpaque(Control: TControl);
 published
    property Image: TBitmap read FImage write SetPicture;
end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TBitCursor]);
end;


constructor TBitCursor.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FImage := TBitmap.Create;
FBackImage:=TBitmap.Create;
FImage.Transparent:=True;
FImage.TransparentMode:=tmAuto;
CursorSet:=False;
end;


destructor TBitCursor.Destroy;
begin
FImage.Free;
FBackImage.Free;
inherited Destroy;
end;


procedure TBitCursor.SetPicture(Value: TBitmap);
begin
FImage.Assign(Value);
end;


procedure TBitCursor.MakeOpaque(Control: TControl);
begin
Control.Parent.ControlStyle := Control.Parent.ControlStyle + [csOpaque];
end;



procedure TBitCursor.SetCursor(Canvas: TCanvas; X,Y: integer);
var R1,R2: TRect;
begin
CursorX:=X; CursorY:=Y;
FBackImage.Width:=FImage.Width;
FBackImage.Height:=FImage.Height;
Left:=X - FImage.Width div 2;
Top:=Y - FImage.Height div 2;
R1:=Rect(0,0,FBackImage.Width,FBackImage.Height);
R2:=R1;

OffsetRect(R2,Left,Top);
FBackImage.Canvas.CopyRect(R1,Canvas,R2);
Canvas.Draw(Left,Top,FImage);
CursorSet:=True;
end;


procedure TBitCursor.EraseCursor(Canvas: TCanvas);
begin
if not CursorSet then exit;
Canvas.Draw(Left,Top,FBackimage);
CursorSet:=False;
end;


procedure TBitCursor.MoveCursor(Canvas: TCanvas; X,Y: integer);
begin
EraseCursor(Canvas);
SetCursor(Canvas,X,Y);
end;

procedure TBitCursor.ShowCursor(Canvas: TCanvas);
begin
SetCursor(Canvas,CursorX,CursorY);
end;

end.
