unit CheckBoxExt;

interface

uses
  Windows,Forms,Graphics,StdCtrls, ComCtrls, ExtCtrls, Messages, SysUtils, Classes,
  Controls, Dialogs, CommCtrl;

type
  TCheckBoxExt = class(TCheckBox)
  private
    procedure SetNoEventChecked(const Value: boolean);
    function GetNoEventChecked: boolean;
  protected
  public
  published
   property Align;
   property NoEventChecked: boolean read GetNoEventChecked write SetNoEventChecked;
  end;


type TRadioGroupExt = class(TRadioGroup)
 private
   FEnableEvent: boolean;
   function GetNoEventItemIndex: integer;
   procedure SetNoEventItemIndex(const Value: integer);
 protected
   procedure Click; override;
 public
 published
  constructor Create(AOwner: TComponent); override;
  property NoEventItemIndex: integer read GetNoEventItemIndex write SetNoEventItemIndex;
 end;



type TTrackBarExt = class(TTrackBar)
 private
  FEnableEvent: boolean;
  FRealPos: double;
  function GetPosition: integer;
  procedure SetNoEventPosition(const Value: integer);
  procedure SetPosition(const Value: Integer);
  procedure SetRealPos(const Value: double);
 protected
  procedure Changed; override;
 public
 published
  constructor Create(AOwner: TComponent); override;
  property NoEventPosition: integer read GetPosition write SetNoEventPosition;
  property RealPos: double read FRealPos write SetRealPos;
  property Position: Integer read GetPosition write SetPosition;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
 end;


type TPageControlExt = class(TPageControl)
 private
 protected
  procedure DrawTab(TabIndex: Integer; const Rect: TRect; Active: Boolean); override;
 public
 published
  constructor Create(AOwner: TComponent); override;
  property Color;
 end;



type TLVColumnResizeEvent = procedure(Sender: TCustomListview; ColumnIndex,ColumnWidth: Integer) of object;


type TWMNotifyLV = record { TWMNotify }
 Msg: Cardinal;
 IDCtrl: WPARAM;
 case Integer of
  0: (NMHdr: PNMHdr);
  1: (HDNotify: PHDNotify);
  2: (LVDispInfo: PLVDispInfo);
  3: (NMCustomDraw: PNMCustomDraw);
  4: (NMListView: PNMListView);
  5: (NMLVCacheHint: PNMLVCacheHint);
  6: (NMLVCustomDraw: PNMLVCustomDraw);
  7: (NMLVFindItem: PNMLVFindItem);
  8: (NMLVODStateChange: PNMLVODStateChange;
  Result: LRESULT);
  end;

type TLVCheckedItemEvent = procedure(Sender: TObject; Item: TListItem) of object;


type TListViewExt = class(TListview)
 private
  FBeginColumnResizeEvent: TLVColumnResizeEvent;
  FEndColumnResizeEvent: TLVColumnResizeEvent;
  FColumnResizeEvent: TLVColumnResizeEvent;
  FOnItemChecked: TLVCheckedItemEvent;
  function GetOwnerData: Boolean;
  procedure SetOwnerData(const Value: Boolean);
  function CreateImageList: TImageList;
 protected
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure DoBeginColumnResize(columnindex, columnwidth: Integer); virtual;
  procedure DoEndColumnResize(columnindex, columnwidth: Integer); virtual;
  procedure DoColumnResize(columnindex, columnwidth: Integer); virtual;
  procedure WMNotify(var Msg: TWMNotify); message WM_NOTIFY;
  function FindColumnIndex(pHeader: pNMHdr): Integer;
  function FindColumnWidth(pHeader: pNMHdr): Integer;
  procedure CreateWnd; override;
  procedure CNNotify(var Msg: TWMNotifyLV); message CN_NOTIFY;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
 published
  property OnBeginColumnResize: TLVColumnResizeEvent read FBeginColumnResizeEvent write FBeginColumnResizeEvent;
  property OnEndColumnResize: TLVColumnResizeEvent read FEndColumnResizeEvent write FEndColumnResizeEvent;
  property OnColumnResize: TLVColumnResizeEvent read FColumnResizeEvent write FColumnResizeEvent;
  property OnItemChecked: TLVCheckedItemEvent read FOnItemChecked write FOnItemChecked;
  property OwnerData: Boolean read GetOwnerData write SetOwnerData default False;
  end;


procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Fountain', [TCheckBoxExt]);
RegisterComponents('Fountain', [TRadioGroupExt]);
RegisterComponents('Fountain', [TTrackBarExt]);
RegisterComponents('Fountain', [TPageControlExt]);
RegisterComponents('Fountain', [TListViewExt]);
end;



procedure TCheckBoxExt.SetNoEventChecked(const Value: boolean);
begin
ClicksDisabled := True;
try
Checked := Value;
finally ClicksDisabled:=False; end;
end;



function TCheckBoxExt.GetNoEventChecked: boolean;
begin
Result:=Checked;
end;



{ TRadioGroupExt }

procedure TRadioGroupExt.Click;
begin
if FEnableEvent then inherited Click;
end;



constructor TRadioGroupExt.Create(AOwner: TComponent);
begin
inherited;
FEnableEvent:=True;
end;



function TRadioGroupExt.GetNoEventItemIndex: integer;
begin
Result:=ItemIndex;
end;



procedure TRadioGroupExt.SetNoEventItemIndex(const Value: integer);
begin
FEnableEvent:=False;
try
ItemIndex:=Value;
finally FEnableEvent:=True; end;
end;



{ TTrackBarExt }

procedure TTrackBarExt.Changed;
begin
if FEnableEvent then inherited;
FRealPos:=Position;
end;

constructor TTrackBarExt.Create(AOwner: TComponent);
begin
inherited;
FEnableEvent:=True;
end;

function TTrackBarExt.GetPosition: integer;
begin
Result:=inherited Position;
end;

procedure TTrackBarExt.SetNoEventPosition(const Value: integer);
begin
FEnableEvent:=False;
try
if Position<>Value then Position:=Value;
finally FEnableEvent:=True; end;
end;



procedure TTrackBarExt.SetPosition(const Value: Integer);
begin
inherited Position:=Value;
end;


procedure TTrackBarExt.SetRealPos(const Value: double);
begin
if Value<Min then FRealPos:=Min
else if Value>Max then FRealPos:=Max
else FRealPos := Value;
inherited Position:=round(FRealPos);
end;


{ TPageControlExt }

constructor TPageControlExt.Create(AOwner: TComponent);
begin
inherited;
OwnerDraw:=True;
end;

procedure TPageControlExt.DrawTab(TabIndex: Integer; const Rect: TRect;
  Active: Boolean);
begin
inherited;
Self.Canvas.Brush.Color:=Self.Color;
Self.Canvas.Pen.Color:=Self.Color;
Self.Canvas.Rectangle(Rect);
//Control.Canvas.Brush.Style:=bsClear;
Self.Canvas.TextOut(Rect.left+5,Rect.top+3,Self.Pages[tabindex].Caption);
Pages[TabIndex].Brush.Color:=Self.Canvas.Brush.Color;
end;



{ TListViewExt }



procedure TListViewExt.DoBeginColumnResize(columnindex, columnwidth: Integer);
begin
if Assigned(FBeginColumnResizeEvent) then FBeginColumnResizeEvent(Self, columnindex, columnwidth);
end;



procedure TListViewExt.DoEndColumnResize(columnindex, columnwidth: Integer);
begin
if Assigned(FEndColumnResizeEvent) then FEndColumnResizeEvent(Self, columnindex, columnwidth);
end;


procedure TListViewExt.DoColumnResize(columnindex, columnwidth: Integer);
begin
if Assigned(FColumnResizeEvent) then FColumnResizeEvent(Self, columnindex, columnwidth);
end;


function TListViewExt.FindColumnIndex(pHeader: pNMHdr): Integer;
var hwndHeader: HWND;
var iteminfo: THdItem;
var ItemIndex: Integer;
var buf: array [0..128] of Char;
begin
Result := -1;
hwndHeader := pHeader^.hwndFrom;
ItemIndex := pHDNotify(pHeader)^.Item;
FillChar(iteminfo, SizeOf(iteminfo), 0);
iteminfo.Mask := HDI_TEXT;
iteminfo.pszText := buf;
iteminfo.cchTextMax := SizeOf(buf) - 1;
Header_GetItem(hwndHeader, ItemIndex, iteminfo);
if CompareStr(Columns[ItemIndex].Caption, iteminfo.pszText) = 0 then Result := ItemIndex
else
	begin
	for ItemIndex := 0 to Columns.Count - 1 do
	   if CompareStr(Columns[ItemIndex].Caption, iteminfo.pszText) = 0 then
		begin
		Result := ItemIndex;
		Break;
		end;
	end;
end;



procedure TListViewExt.WMNotify(var Msg: TWMNotify);
begin
inherited;
case Msg.NMHdr^.code of
 HDN_ENDTRACK: DoEndColumnResize(FindColumnIndex(Msg.NMHdr), FindColumnWidth(Msg.NMHdr));
 HDN_BEGINTRACK: DoBeginColumnResize(FindColumnIndex(Msg.NMHdr), FindColumnWidth(Msg.NMHdr));
 HDN_ITEMCHANGING: DoColumnResize(FindColumnIndex(Msg.NMHdr), FindColumnWidth(Msg.NMHdr));
 end;
end;



procedure TListViewExt.CreateWnd;
var Wnd: HWND;
begin
inherited;
Wnd := GetWindow(Handle, GW_CHILD);
SetWindowLong(wnd, GWL_STYLE, GetWindowLong(wnd, GWL_STYLE) and not HDS_FULLDRAG );
end;



function TListViewExt.FindColumnWidth(pHeader: pNMHdr): Integer;
begin
Result := -1;
if Assigned(PHDNotify(pHeader)^.pItem) and
   ((PHDNotify(pHeader)^.pItem^.mask and HDI_WIDTH) <> 0) then Result := PHDNotify(pHeader)^.pItem^.cxy;
end;




procedure TListViewExt.CNNotify(var Msg: TWMNotifyLV);
{Handle checkbox clicks when not in OwnerData mode}
{This doesn't work when "OwnerData" is enabled}
var Item: TListItem;
begin
inherited;
if Msg.NMHdr.code = LVN_ITEMCHANGED then
	begin
	if Assigned(FOnItemChecked) and
	   (Msg.NMListView.uChanged = LVIF_STATE) and
	   {This expression is used to prevent trigger when a new item's added}
	   ((Msg.NMListView.uOldState and LVIS_STATEIMAGEMASK)<>0) and
	   (((Msg.NMListView.uOldState and LVIS_STATEIMAGEMASK) shr 12) <>
	   ((Msg.NMListView.uNewState and LVIS_STATEIMAGEMASK) shr 12)) then
		begin
		Item := Items[Msg.NMListView.iItem];
		FOnItemChecked(Self, Item);
		end;
	end;
end;




procedure TListViewExt.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{Handle checkbox clicks in OwnerData mode}
{In OwnerData mode, the "Item" contain no valid data except the index}
var Item: TListItem;
begin
if Button <> mbLeft then exit;
if not OwnerData then exit;
if htOnStateIcon in Self.GetHitTestInfoAt(X, Y) then
	begin
	Item := Self.GetItemAt(X, Y);
	if Assigned(FOnItemChecked) then FOnItemChecked(Self, Item);
	end;

inherited;
end;



function TListViewExt.GetOwnerData: Boolean;
begin
Result:=inherited OwnerData;
end;




function TListViewExt.CreateImageList: TImageList;
{When the control is in OwnerData mode, we have to create our own check boxes}
var BMP: TBitmap;
const Size = 15;
begin
Result:= TImageList.Create(Self);
Bmp := TBitmap.Create;
try
Bmp.PixelFormat := pf32bit;
Bmp.Width:=16;
Bmp.Height:=16;
DrawFrameControl(Bmp.Canvas.Handle, Rect(2, 2, Size, Size), DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_FLAT);
Result.Add(Bmp, nil);
DrawFrameControl(Bmp.Canvas.Handle, Rect(2, 2, Size, Size), DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_CHECKED or DFCS_FLAT);
Result.Add(Bmp, nil);
finally Bmp.Free; end;
end;




procedure TListViewExt.SetOwnerData(const Value: Boolean);
begin
inherited OwnerData := Value;
if Value then StateImages:=CreateImageList
else StateImages.Free;
end;




constructor TListViewExt.Create(AOwner: TComponent);
begin
inherited ;
end;



destructor TListViewExt.Destroy;
begin
inherited;
end;




end.
