unit Compass;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls,DSupport,Vector;


type TArm=record
 E,N,V: double;
 C: AnsiChar;
 end;


type TCompass = class(TPaintBox)
 private
  FOnChanging: TNotifyEvent;
  FOnChanged: TNotifyEvent;
  CosA,SinA,PCosA,PSinA :double;
  Moving: boolean;
  MoveX,MoveY: integer;
  CenterX,CenterY: integer;
  FPitch,FYaw: double;
  Arms: array [0..5] of TArm;
  FScreenColor,FBackColor,FForeColor: TColor;
  ArmSize: double;
  FLockPitch: boolean;
  FLineWidth: integer;
  procedure SetAngle(A,P: double);
  procedure RotateCoordinate(var E,N,V: double);
  procedure NormalAngles;
  procedure DrawJack(Erase: boolean);
  procedure DrawCurrent;
  procedure SetLineWidth(Value: integer);
  procedure SetYaw(Value: double);
  procedure SetPitch(Value: double);
  procedure SetLockPitch(Value: boolean);
  procedure SetScreenColor(Value: TColor);
  procedure SetBackColor(Value: TColor);
  procedure SetForeColor(Value: TColor);
  procedure SetSize;
 protected
  procedure Paint; override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure Changing; dynamic;
  procedure Changed; dynamic;
  procedure VectorToAngle(Up,Dir: T3DVector; var Yaw,Pitch: double);
 public
  constructor Create(AOwner: TComponent); override;
  procedure GetOrientation(var Up,Dir: T3DVector); overload;
  procedure GetOrientation(var Up,Dir,Right: T3DVector); overload;
  procedure SetOrientation(Up,Dir: T3DVector); overload;
  procedure SetOrientation(Dir: T3DVector); overload;
 published
  property Yaw: double read FYaw write SetYaw;
  property Pitch: double Read FPitch Write SetPitch;
  property ScreenColor: TColor read FScreenColor write SetScreenColor default clBlack;
  property BackColor: TColor read FBackColor write SetBackColor default clBlue;
  property ForeColor: TColor read FForeColor write SetForeColor default clRed;
  property LockPitch: boolean read FLockPitch write SetLockPitch default False;
  property OnChanging: TNotifyEvent read FOnChanging write FOnChanging;
  property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  property LineWidth: integer read FLineWidth write SetLineWidth;
  end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TCompass]);
end;


function Sign(X:double) : double;
{ Return 1 or -1 depending on sign of X}
begin
if X < 0.0  then Result:= -1.0 else Result:= 1.0;
end;


function ArcTan2(Y,X:double) : double;
{ Return four quadrant ArcTangent }
begin
if X > 0.0 then Result:= ArcTan(Y/X)
else if X < 0.0  then  Result:= ArcTan(Y/X) + PI * Sign(Y)
else Result:= PI/2.0 * Sign(Y);
end;


function ArcCos(X: extended): extended;
{Calculate ArcCosine of X }
begin
Result:= ArcTan2(sqrt (1-sqr (X)),X);
end;


procedure TCompass.SetAngle(A,P: double);
{Set trig values for Pitch and Yaw}
begin
CosA:=cos((360-A)*Degree2Radian); SinA:=sin((360-A)*Degree2Radian);
PCosA:=cos(P*Degree2Radian); PSinA:=sin(P*Degree2Radian);
end;


procedure TCompass.RotateCoordinate(var E,N,V: double);
{  Rotate Coordinates  }
var TE,TN,TV: double;
begin
TE:=E; TN:=N; TV:=V;
N := (SinA  * -TE) + (CosA  * TN);
E := (CosA  * TE) + (SinA  * TN);
V := (PSinA  * -N) + (PCosA  * TV);
N := (PCosA  * N) + (PSinA  * TV);
end;



procedure TCompass.VectorToAngle(Up,Dir: T3DVector; var Yaw,Pitch: double);
var TV: double;
begin
Pitch:=ArcCos(Up.Z);
TV:=Dir.Z;
Dir.Z := (Sin(-Pitch)  * -Dir.Y) + (Cos(-Pitch)  * TV);
Dir.Y := (Cos(-Pitch)  * Dir.Y) + (Sin(-Pitch)  * TV);

if Dir.Y=0 then Yaw:=0 else Yaw:=(ArcTan2(-Dir.X,Dir.Y) * Radian2Degree);
if Yaw<0 then Yaw := 360 + Yaw;
Pitch:=Pitch*Radian2Degree;
end;




procedure TCompass.NormalAngles;
begin
if FPitch>180 then FPitch:=180
else if FPitch<0 then FPitch:=0;
while FYaw>360 do FYaw:=FYaw-360;
while FYaw<0 do FYaw:=FYaw+360;
end;



procedure TCompass.DrawJack(Erase: boolean);
var P: integer;
var C1,C2:TColor;
var TW,TH: integer;
begin
Canvas.Pen.Style:=psSolid;
if Erase then begin C1:=FScreenColor; C2:=FScreenColor; end
else begin C1:=FBackColor; C2:=FForeColor; end;
Arms[0].E:=-1;Arms[0].N:=0;Arms[0].V:=0;Arms[0].C:='W';
Arms[1].E:=1;Arms[1].N:=0;Arms[1].V:=0;Arms[1].C:='E';
Arms[2].E:=0;Arms[2].N:=-1;Arms[2].V:=0;Arms[2].C:='N';
Arms[3].E:=0;Arms[3].N:=1;Arms[3].V:=0;Arms[3].C:='S';
Arms[4].E:=0;Arms[4].N:=0;Arms[4].V:=1;Arms[4].C:='D';
Arms[5].E:=0;Arms[5].N:=0;Arms[5].V:=-1;Arms[5].C:='U';
for P:=0 to 5 do
 with Arms[P] do RotateCoordinate(E,N,V);
{Draw background Arms}
Canvas.Pen.Color:=C1;
Canvas.Pen.Width:=FLineWidth;
Canvas.Font.Color:=C1;
for P:=0 to 5 do
 with Arms[P] do
 begin
 if V>0 then
	begin
	byte(C):=byte(C)+$20;
	TW:=Canvas.TextWidth(C) div 2;
	TH:=Canvas.TextHeight(C) div 2;

	Canvas.MoveTo(CenterX,CenterY);
	Canvas.LineTo(CenterX+Trunc(E*ArmSize),CenterY+Trunc(N*ArmSize));
	Canvas.TextOut(CenterX+Trunc(E*ArmSize)-TW,CenterY+Trunc(N*ArmSize)-TH,C);
	end;
 end;
{Draw foreground Arms}
Canvas.Pen.Color:=C2;
Canvas.Font.Color:=C2;
for P:=0 to 5 do
 with Arms[P] do
 begin
 if V<=0 then
	begin
	TW:=Canvas.TextWidth(C) div 2;
	TH:=Canvas.TextHeight(C) div 2;
	Canvas.MoveTo(CenterX,CenterY);
	Canvas.LineTo(CenterX+Trunc(E*ArmSize),CenterY+Trunc(N*ArmSize));
	Canvas.TextOut(CenterX+Trunc(E*ArmSize)-TW,CenterY+Trunc(N*ArmSize)-TH,C);
	end;
 end;
end;



procedure TCompass.DrawCurrent;
begin
DrawJack(True);
SetAngle(FYaw,FPitch);
DrawJack(False);
end;



procedure TCompass.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseDown(Button, Shift, X, Y);
Moving:=True;
MoveX:=X;MoveY:=Y;
end;


procedure TCompass.MouseMove(Shift: TShiftState; X, Y: Integer);
var DX,DY: integer;
begin
inherited MouseMove(Shift, X, Y);
if not Moving then exit;
DX:=X-MoveX; DY:=Y-MoveY;
if FLockPitch then
	begin
	if Y>CenterY then FYaw:=FYaw-DX
	else FYaw:=FYaw+DX;
	if X>CenterX then FYaw:=FYaw+DY
	else FYaw:=FYaw-DY;
	end
else
	begin
	FYaw:=FYaw-DX;
	FPitch:=FPitch-DY;
	end;
NormalAngles;
DrawCurrent;
MoveX:=X; MoveY:=Y;
Changing;
end;

procedure TCompass.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseUp(Button, Shift, X, Y);
Moving:=False;
DrawCurrent;
Changed;
end;

procedure TCompass.SetSize;
begin
CenterX:=Width div 2;
CenterY:=Height div 2;
if Width>Height then ArmSize:=Height else ArmSize:=Width;
ArmSize:=(ArmSize/2) * 0.90;
end;


constructor TCompass.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FLockPitch:=False;
SetSize;
FPitch:=45;
FYaw:=45;
FScreenColor:=clBlack;
FBackColor:=clBlue;
FForeColor:=clRed;
FLineWidth:=1;
end;



procedure TCompass.Paint;
begin
inherited Paint;
SetSize;
SetAngle(FYaw,FPitch);
Canvas.Brush.Color:=FScreenColor;
Canvas.FillRect(Canvas.ClipRect);
DrawJack(False);
end;



procedure TCompass.GetOrientation(var Up,Dir: T3DVector);
begin
Up:=MakeVector3D(0,0,1);
RotateCoordinate(Up.X,Up.Y,Up.Z);
Dir:=MakeVector3D(0,1,0);
RotateCoordinate(Dir.X,Dir.Y,Dir.Z);
end;



procedure TCompass.GetOrientation(var Up,Dir,Right: T3DVector);
begin
Up:=MakeVector3D(0,0,1);
RotateCoordinate(Up.X,Up.Y,Up.Z);
Dir:=MakeVector3D(0,1,0);
RotateCoordinate(Dir.X,Dir.Y,Dir.Z);
Right:=MakeVector3D(1,0,0);
RotateCoordinate(Right.X,Right.Y,Right.Z);
end;



procedure TCompass.SetOrientation(Up,Dir: T3DVector);
begin
VectorToAngle(Up,Dir,FYaw,FPitch);
DrawCurrent;
end;




function GetUpVector(Dir: T3DVector): T3DVector;
var E: T3DVector;
begin
if Dir.Z>0 then E:=MakeVector3D(1,0,0) else E:=MakeVector3D(-1,0,0);
Result:=Vector.CrossProduct3D(Dir,E);
end;



procedure TCompass.SetOrientation(Dir: T3DVector);
var Up: T3DVector;
begin
Up:=GetUpVector(Dir);
VectorToAngle(Up,Dir,FYaw,FPitch);
DrawCurrent;
end;



procedure TCompass.SetYaw(Value: double);
begin
FYaw:=Value;
DrawCurrent;
end;




procedure TCompass.SetLineWidth(Value: integer);
begin
if Value<>FLineWidth then
	begin
	FLineWidth:=Value;
	Invalidate;
	end;
end;


procedure TCompass.SetPitch(Value: double);
begin
FPitch:=Value;
DrawCurrent;
end;


procedure TCompass.SetLockPitch(Value: boolean);
begin
FLockPitch:=Value;
if Value then SetPitch(0);
end;


procedure TCompass.SetScreenColor(Value: TColor);
begin
FScreenColor:=Value;
Invalidate;
end;


procedure TCompass.SetBackColor(Value: TColor);
begin
FBackColor:=Value;
Invalidate;
end;


procedure TCompass.SetForeColor(Value: TColor);
begin
FForeColor:=Value;
Invalidate;
end;


procedure TCompass.Changing;
begin
if Assigned(FOnChanging) then FOnChanging(Self);
end;


procedure TCompass.Changed;
begin
if Assigned(FOnChanged) then FOnChanged(Self);
end;


end.
