unit Degspin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Menus, StrUtils, DSupport, SpinButtons,
  Harmonizer;

const MaxDigits = 5;

type TDecimalDigits = 0..MaxDigits;	{Constrain range of digits}

type TLimitEvent = procedure(Sender: TObject; Upper: boolean) of object;
type TLimitState = (lsNone,lsUpper,lsLower);


type
 TDegreeSpin = class(TCustomPanel)
 private
  FAlignment: TAlignment;
  BaseCon,DegCon,MinCon,SecCon: Int64;
  FDigits: TDecimalDigits;
  FBlockUpdate: boolean;
  FDegEdit: TEditEx;
  FDegSpin: TUpDownButton;
  FMinEdit: TEditEx;
  FMinSpin: TUpDownButton;
  FSecEdit: TEditEx;
  FSecSpin: TUpDownButton;
  FUseLimit: boolean;
  FWrap: boolean;
  FMinValue,FMaxValue: double;

  FDegLabel: TLabel;
  FMinLabel: TLabel;
  FSecLabel: TLabel;
  FSpinFont: TFont;
  FLabelFont: TFont;
  FOnChange: TNotifyEvent;
  FBlockChangeEvent: boolean;
  FAutoCorrect: boolean;
  FSecInc: double;
  FLimitEvent: TLimitEvent;
  function ReadHours: integer;
  procedure SetHours(V: integer);
  function ReadMinutes: integer;
  procedure SetMinutes(V: integer);
  function ReadSeconds: double;
  procedure SetSeconds(V: double);
  procedure DegreeToDegMinSec(Deg: Int64; var DS,MS,SS: string; Digits: integer);
  function DegMinSecToDegrees(D,M: Int64; S: double; Sign: Int64): Int64;
  procedure SetUnitsString(V: string);
  function GetUnitsString: string;
  procedure IncDegrees(V: Int64);
  procedure DegSpinDownClick(Sender: TObject);
  procedure DegSpinUpClick(Sender: TObject);
  procedure MinSpinDownClick(Sender: TObject);
  procedure MinSpinUpClick(Sender: TObject);
  procedure SecSpinDownClick(Sender: TObject);
  procedure SecSpinUpClick(Sender: TObject);
  procedure DegEditChange(Sender: TObject);
  procedure ShowDegrees(Deg: Int64);
  procedure SetAlignment(Value: TAlignment);

  procedure Arrange;
  procedure DegChange(LimitState: TLimitState);
  procedure ValidateChars(Sender: TObject; var Key: Char);
  procedure SetSpinFont(Value: TFont);
  procedure SetLabelFont(Value: TFont);
  procedure SetDigits(Value: TDecimalDigits);
  procedure FontChanged(Sender: TObject);
  function GetDegrees: double;
  procedure SetDegrees(Deg: double);
  procedure SetNoEventValue(NewValue: double);
  function SetWithLimits(Value: Int64): TLimitState;
  procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure SetMaxValue(const Value: double);
    procedure SetMinValue(const Value: double);
 protected
  procedure SetEnabled(Value: Boolean); override;
  procedure Paint; override;
 public
  FDegrees: Int64;
  EntryError: boolean;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property NoEventValue: double read GetDegrees write SetNoEventValue;
 published
  property OnLimit: TLimitEvent read FLimitEvent write FLimitEvent;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property SpinFont: TFont read FSpinFont write SetSpinFont;
  property LabelFont: TFont read FLabelFont write SetLabelFont;
  property Degrees: double read GetDegrees write SetDegrees;
  property Hours: Integer read ReadHours write SetHours;
  property Minutes: Integer read ReadMinutes write SetMinutes;
  property Seconds: double read ReadSeconds write SetSeconds;
  property Digits: TDecimalDigits read FDigits write SetDigits default 1;
  property MaxValue: double read FMaxValue write SetMaxValue;
  property MinValue: double read FMinValue write SetMinValue;
  property UseLimit: boolean read FUselimit write FUselimit default False;
  property BlockChangeEvent: boolean read FBlockChangeEvent write FBlockChangeEvent default false;
  property Wrap: boolean read FWrap write FWrap default False;
  property AutoCorrect: boolean read FAutoCorrect write FAutoCorrect stored True default True;
  property UnitsString: string read GetUnitsString write SetUnitsString;
  property Alignment: TAlignment read FAlignment write SetAlignment;
  property SecInc: double read FSecInc write FSecInc;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color;
    property Ctl3D;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
 end;



procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Fountain', [TDegreeSpin]);
end;



procedure TDegreeSpin.SetAlignment(Value: TAlignment);
begin
if FAlignment<>Value then
	begin
	FAlignment:=Value;
	FDegEdit.Alignment:=Value;
	FMinEdit.Alignment:=Value;
	FSecEdit.Alignment:=Value;
	end;
end;



procedure TDegreeSpin.SetEnabled(Value: Boolean);
begin
inherited SetEnabled(Value);
FDegEdit.Enabled:=Value;
FDegSpin.Enabled:=Value;
FMinEdit.Enabled:=Value;
FMinSpin.Enabled:=Value;
FSecEdit.Enabled:=Value;
FSecSpin.Enabled:=Value;
FDegLabel.Enabled:=Value;
FMinLabel.Enabled:=Value;
FSecLabel.Enabled:=Value;
end;



procedure TDegreeSpin.SetUnitsString(V: string);
begin
FDegLabel.Caption:=V;
end;


function TDegreeSpin.GetUnitsString: string;
begin
Result:=FDegLabel.Caption;
end;


function TDegreeSpin.DegMinSecToDegrees(D,M: Int64; S: double; Sign: Int64): Int64;
begin
Result:=(D*DegCon)+(M*MinCon)+round(S*SecCon);
Result:=Result*Sign;
end;


procedure SetEdit(var E: TEditEx; S: string);
var Start: integer;
begin
Start:=E.SelStart;
E.Text:=S;
E.SelStart:=Start;
end;




procedure TDegreeSpin.DegreeToDegMinSec(Deg: Int64; var DS,MS,SS: string; Digits: integer);
{Convert Degrees to Deg, Min, Sec}
var D,M,S,SF: Int64;
var Sign: Int64;
var T: string;
begin
if Deg<0 then Sign:=-1 else Sign:=1;
Deg:=abs(Deg);
D:=Deg div DegCon;
Deg:=Deg-(D*DegCon);
M:=Deg div MinCon;
Deg:=Deg-(M * MinCon);
S:=DivMod(Deg, SecCon,SF);
if (D=0) and (Sign<0) then DS:='-0' else DS:=IntToStr(D*Sign);
MS:=IntToStr(M);

SS:=IntToStr(S);
T:=IntToStr(SF);
T:=PadLeftString(T,5,'0');
T:=PadRightString(T,20,'0');
T:=LeftStr(T,Digits);
if SF<>0 then SS:=SS + '.' + T;
end;




procedure TDegreeSpin.ShowDegrees(Deg: Int64);
var D,M,S: string;
begin
FBlockUpdate:=True;
try
EntryError:=False;
DegreeToDegMinSec(Deg,D,M,S,FDigits);

SetEdit(FDegEdit,D);
SetEdit(FMinEdit,M);
SetEdit(FSecEdit,S);

finally FBlockUpdate:=False; end;
end;



procedure TDegreeSpin.IncDegrees(V: Int64);
var LS: TLimitState;
begin
LS:=SetWithLimits(FDegrees+V);
ShowDegrees(FDegrees);
DegChange(LS);
end;


procedure TDegreeSpin.DegSpinDownClick(Sender: TObject);
begin
IncDegrees(-DegCon);
end;

procedure TDegreeSpin.DegSpinUpClick(Sender: TObject);
begin
IncDegrees(DegCon);
end;

procedure TDegreeSpin.MinSpinDownClick(Sender: TObject);
begin
IncDegrees(-MinCon);
end;

procedure TDegreeSpin.MinSpinUpClick(Sender: TObject);
begin
IncDegrees(MinCon);
end;


procedure TDegreeSpin.SecSpinDownClick(Sender: TObject);
begin
IncDegrees(round(-SecCon*SecInc));
end;


procedure TDegreeSpin.SecSpinUpClick(Sender: TObject);
begin
IncDegrees(round(SecCon*SecInc));
end;

{-----------------------------------------------------------------------------}


function ExtractInteger(E: TEditEx; var Value: integer): boolean;
var Code: Integer;
begin
Val(E.Text,Value,Code);
Result:=Code=0;
end;


function EditToInt(E: TEditEx): integer;
begin
Result:=StrToIntDef(E.Text,0);
end;


function ExtractFloat(E: TEditEx; var Value: double): boolean;
begin
Result:=TryStrToFloat(E.Text,Value);
end;



function EditToFloat(E: TEditEx): double;
begin
Result:=GetNumberValue(E.Text);
end;




procedure TDegreespin.ValidateChars(Sender: TObject; var Key: Char);
begin
if (Key in [IndSystemData.DecimalSeparator, '+', '-', '0'..'9']) or
   ((Key < #32) and (Key <> Chr(VK_RETURN))) then exit;
Key:=#0;
end;



function RangeCheckDegs(Minutes: integer; Seconds: double): boolean;
begin
Result:=(Minutes>=0) and (Minutes<60) and (Seconds>=0) and (Seconds<60);
end;



procedure TDegreeSpin.DegEditChange(Sender: TObject);
var D,M,Sign: integer;
var Tmp: Int64;
var S: double;
var LS: TLimitState;
begin
if FBlockUpdate then exit;
if csReadingState in Self.ControlState then exit;
if not ExtractInteger(FDegEdit,D) then exit;
if not ExtractInteger(FMinEdit,M) then exit;
if not ExtractFloat(FSecEdit,S) then exit;
if Pos('-',FDegEdit.Text)<>0 then Sign:=-1 else Sign:=1;
if (D<0) or (M<0) or (S<0) or (Sign<0) then Sign:=-1 else Sign:=1;
EntryError:=not RangeCheckDegs(M,S);
if not FAutoCorrect and EntryError then DegChange(lsNone)
else
	begin
	Tmp:=DegMinSecToDegrees(abs(D),abs(M),abs(S),Sign);
	if FDegrees<>Tmp then
		begin
		LS:=SetWithLimits(Tmp);
		ShowDegrees(FDegrees);
		DegChange(LS);
		end;
	end;
end;



procedure TDegreeSpin.DegChange(LimitState: TLimitState);
begin
if BlockChangeEvent then exit;
if Assigned(FOnChange) then FOnChange(Self);
if Assigned(FLimitEvent) then
 if LimitState=lsUpper then FLimitEvent(Self,True)
 else if LimitState=lsLower then FLimitEvent(Self,False);
end;



procedure TDegreeSpin.Arrange;
var W1,W2,W3,R1,R2,R3,T: integer;
const SpinWidth=15;
const DigitWidth=12;
begin
Caption:='';
FDegEdit.Font:=FSpinFont;
FMinEdit.Font:=FSpinFont;
FSecEdit.Font:=FSpinFont;
FDegLabel.Font:=FLabelFont;
FMinLabel.Font:=FLabelFont;
FSecLabel.Font:=FLabelFont;
case FDigits of
 0: begin R1:=36; R2:=24; R3:=24; end;
 1: begin R1:=32; R2:=24; R3:=28; end;
 2: begin R1:=30; R2:=24; R3:=30; end;
 3: begin R1:=28; R2:=24; R3:=32; end;
 4: begin R1:=26; R2:=24; R3:=34; end;
 5: begin R1:=24; R2:=24; R3:=36; end;
 else begin R1:=24; R2:=24; R3:=36; end;
 end;
W1:=MulDiv(Width,R1,100);
W2:=MulDiv(Width,R2,100);
W3:=MulDiv(Width,R3,100);
FDegEdit.Width:=W1-SpinWidth;
FMinEdit.Width:=W2-SpinWidth;
FSecEdit.Width:=W3-SpinWidth;
T:=(Width-(W1+W2+W3)) div 4;
FDegEdit.Left:=T+1;
FMinEdit.Left:=(2*T)+W1+1;
FSecEdit.Left:=(T*3)+(W1+W2)+1;
FDegLabel.Left:=FDegEdit.Left;
FMinLabel.Left:=FMinEdit.Left;
FSecLabel.Left:=FSecEdit.Left;

T:=Height-(FDegEdit.Height+FDegLabel.Height+2);
T:=MulDiv(T,75,100);

FDegEdit.Top:=T; FMinEdit.Top:=T; FSecEdit.Top:=T;

FDegSpin.Top:=T; FMinSpin.Top:=T; FSecSpin.Top:=T;

FDegSpin.Width:=SpinWidth; FMinSpin.Width:=SpinWidth; FSecSpin.Width:=SpinWidth;
FDegSpin.Height:=FDegEdit.Height;
FMinSpin.Height:=FMinEdit.Height;
FSecSpin.Height:=FSecEdit.Height;

FDegSpin.Left:=FDegEdit.Left+FDegEdit.Width-1;
FMinSpin.Left:=FMinEdit.Left+FMinEdit.Width-1;
FSecSpin.Left:=FSecEdit.Left+FSecEdit.Width-1;

FDegLabel.Top:=FDegEdit.Top+FDegEdit.Height;
FMinLabel.Top:=FMinEdit.Top+FMinEdit.Height;
FSecLabel.Top:=FSecEdit.Top+FSecEdit.Height;
end;



procedure TDegreeSpin.WMSize(var Message: TWMSize);
begin
inherited;
Arrange;
end;



procedure TDegreeSpin.Paint;
begin
inherited Paint;
ShowDegrees(FDegrees);
end;


function TDegreeSpin.ReadHours: integer;
begin
Result:=EditToInt(FDegEdit);
end;


procedure TDegreeSpin.SetHours(V: integer);
begin
FDegEdit.Text:=IntToStr(V);
DegEditChange(Self);
end;


function TDegreeSpin.ReadMinutes: integer;
begin
Result:=EditToInt(FMinEdit);
end;


procedure TDegreeSpin.SetMinutes(V: integer);
begin
FMinEdit.Text:=IntToStr(V);
DegEditChange(Self);
end;


function TDegreeSpin.ReadSeconds: double;
begin
Result:=EditToFloat(FSecEdit);
end;


procedure TDegreeSpin.SetSeconds(V: double);
begin
FSecEdit.Text:=FloatToStrF(V,ffFixed,18,FDigits);
DegEditChange(Self);
end;


procedure TDegreeSpin.FontChanged(Sender: TObject);
begin
Arrange;
end;


procedure TDegreeSpin.SetSpinFont(Value: TFont);
begin
FSpinFont.Assign(Value);
Arrange;
end;


procedure TDegreeSpin.SetLabelFont(Value: TFont);
begin
FLabelFont.Assign(Value);
Arrange;
end;


procedure TDegreeSpin.SetDigits(Value: TDecimalDigits);
begin
if Value<> FDigits then
	begin
	FDigits:=Value;
	Arrange;
	Update;
	end;
end;


function TDegreeSpin.GetDegrees: double;
begin
Result:=FDegrees/DegCon;
end;


procedure TDegreeSpin.SetDegrees(Deg: double);
var LS: TLimitState;
begin
LS:=SetWithLimits(round(Deg*DegCon));
ShowDegrees(FDegrees);
if not (csLoading in ComponentState) then DegChange(LS);
end;



procedure TDegreeSpin.SetNoEventValue (NewValue: double);
var OldBlock: boolean;
begin
OldBlock:=BlockChangeEvent;
try
BlockChangeEvent:=True;
SetDegrees(NewValue);
finally BlockChangeEvent:=OldBlock; end;
end;


function TDegreeSpin.SetWithLimits(Value: Int64): TLimitState;
var T: double;
begin
Result:=lsNone;
if FUseLimit then
	begin
	T:=Value/DegCon;
	if T>=FMaxValue then
		begin
		if FWrap then
			begin
			while T>=FMaxValue do T:=T - FMaxValue;
			Value:=round((T+FMinValue)*DegCon);
			end
		else Value:=trunc(FMaxValue*DegCon);
		Result:=lsUpper;
		end;
	if T<FMinValue then
		begin
		if FWrap then
			begin
			while T<FMinValue do T:=FMinValue-T;
			Value:=round((FMaxValue-T)*DegCon);
			end
		else Value:=round(FMinValue*DegCon);
		Result:=lsLower;
		end;
	end;
Value:=RightZeroInteger(Value,MaxDigits-FDigits);
FDegrees:=Value;
end;



constructor TDegreeSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
BaseCon:=Trunc(Power(10,MaxDigits));
DegCon:=3600*BaseCon;
MinCon:=60*BaseCon;
SecCon:=1*BaseCon;
SecInc:=1;
FLabelFont:=TFont.Create;
FLabelFont.Size:=8;
FLabelFont.Name:='Arial';
FLabelFont.Style:=[];
FLabelFont.OnChange:=FontChanged;

FSpinFont:=TFont.Create;
FSpinFont.OnChange:=FontChanged;

Height:=50;

FDegEdit:=TEditEx.Create(Self); FDegEdit.Parent:=Self;
FMinEdit:=TEditEx.Create(Self); FMinEdit.Parent:=Self;
FSecEdit:=TEditEx.Create(Self); FSecEdit.Parent:=Self;
FDegSpin:=TUpDownButton.Create(Self); FDegSpin.Parent:=Self;
FMinSpin:=TUpDownButton.Create(Self); FMinSpin.Parent:=Self;
FSecSpin:=TUpDownButton.Create(Self); FSecSpin.Parent:=Self;


FDegLabel:=TLabel.Create(Self);
FDegLabel.Parent:=Self;
FMinLabel:=TLabel.Create(Self);
FMinLabel.Parent:=Self;
FSecLabel:=TLabel.Create(Self);
FSecLabel.Parent:=Self;
FDegLabel.Caption:='Deg';
FMinLabel.Caption:='Min';
FSecLabel.Caption:='Sec';
FBlockUpdate:=False;

FDegSpin.OnDownClick:=DegSpinDownClick;
FDegSpin.OnUpClick:=DegSpinUpClick;

FMinSpin.OnDownClick:=MinSpinDownClick;
FMinSpin.OnUpClick:=MinSpinUpClick;

FSecSpin.OnDownClick:=SecSpinDownClick;
FSecSpin.OnUpClick:=SecSpinUpClick;

FDegEdit.OnChange:=DegEditChange;
FMinEdit.OnChange:=DegEditChange;
FSecEdit.OnChange:=DegEditChange;

FDegEdit.OnKeyPress:=ValidateChars;
FMinEdit.OnKeyPress:=ValidateChars;
FSecEdit.OnKeyPress:=ValidateChars;

FDigits:=1;
Arrange;
FDegrees:=0;
FMaxValue:=0;
FMinValue:=0;
FUseLimit:=False;
FBlockChangeEvent:=False;
FWrap:=False;
FAutoCorrect:=True;
end;



destructor TDegreeSpin.Destroy;
begin
FDegSpin.Free; FMinSpin.Free; FSecSpin.Free;
FDegEdit.Free; FMinEdit.Free; FSecEdit.Free;
FDegLabel.Free; FMinLabel.Free; FSecLabel.Free;
FLabelFont.Free;
FSpinFont.Free;

inherited Destroy;
end;



procedure TDegreeSpin.SetMaxValue(const Value: double);
begin
if FMaxValue<>Value then
	begin
	FMaxValue:=Value;
	if FUseLimit and (Degrees>FMaxValue) then Degrees:=Value;
	end;
end;

procedure TDegreeSpin.SetMinValue(const Value: double);
begin
if FMinValue<>Value then
	begin
	FMinValue := Value;
	if FUseLimit and (Degrees<FMinValue) then Degrees:=Value;
	end;
end;

end.
