unit FancyButton;

interface

uses Forms, Windows, Messages, SysUtils, Classes, Graphics, Controls,
     StdCtrls, Buttons, ExtCtrls, DSupport, BitmapLib, GraphicsLib,
     IntVector, Autobttn;

{TFancyButton - Simplest fancy button - supports Gradients and accepts keystrokes like the TButton
 New Features:
 1. Gradients
    A. Set "Gradient" to true enable gradient
    B. GradColor1, GradColor2 sets gradient start/end color
 2. Button Box Frame Colors = FrameDarkest, FrameDark, FrameLight, FrameLightest
 3. Supports Glyphs - only single glyphs at the moment.
}

type TFancyButton = class(TButton)
 private
  FDDown: Boolean;
  FGlyph: TBitmap;
  Hovering: boolean;
  FCanvas: TCanvas;
  IsDefault, IsFocused: Boolean;
  IsDown, IsDisabled: boolean;
  FGradient: boolean;
  FGradColor2: TColor;
  FGradColor1: TColor;
  FSpacing: Integer;
  FMargin: Integer;
  FFrameDarkest: TColor;
  FFrameLightest: TColor;
  FFrameDark: TColor;
  FFrameLight: TColor;
  FDownInvert: boolean;
  procedure ShowText(Disabled: boolean; Rect: TRect);
  procedure DrawFocusRect(Rect: TRect);
  procedure SetGradient(const Value: boolean);
  procedure DrawBaseButton(Rect: TRect);
  procedure DrawButtonFrame(var Rect: TRect);
  procedure DrawEmbossedText(Rect: TRect);
  procedure SetGradColor1(const Value: TColor);
  procedure SetGradColor2(const Value: TColor);
  function GetGlyph: TBitmap;
  procedure SetGlyph(const Value: TBitmap);
  procedure SetSpacing(const Value: Integer);
  procedure SetMargin(const Value: Integer);
  procedure GetLayout(BtnRect: TRect; var BitmapRect, TextRect: TRect);
  function ValidGlyph: boolean;
  procedure SetFrameLightest(const Value: TColor);
  procedure SetFrameDarkest(const Value: TColor);
  procedure SetFrameDark(const Value: TColor);
  procedure SetFrameLight(const Value: TColor);
  function GetDownState: boolean;
  procedure SetDownState(Value: Boolean);
    procedure SetDownInvert(const Value: boolean);
 protected
  procedure CreateParams(var Params: TCreateParams); override;
  procedure SetButtonStyle(Value: Boolean); override;
  procedure DrawButton(Rect: TRect);
  procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
  procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
  procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property FrameDarkest: TColor read FFrameDarkest write SetFrameDarkest;
  property FrameDark: TColor read FFrameDark write SetFrameDark;
  property FrameLightest: TColor read FFrameLightest write SetFrameLightest;
  property FrameLight: TColor read FFrameLight write SetFrameLight;
  property Margin: Integer read FMargin write SetMargin default 4;
  property Spacing: Integer read FSpacing write SetSpacing default 4;
  property Glyph: TBitmap read GetGlyph write SetGlyph;
  property GradColor1: TColor read FGradColor1 write SetGradColor1 default clWhite;
  property GradColor2: TColor read FGradColor2 write SetGradColor2 default clBlack;
  property Gradient: boolean read FGradient write SetGradient default False;
  property Down: Boolean read GetDownState write SetDownState default false;
  property DownInvert: boolean read FDownInvert write SetDownInvert default false;
  property Color;
end;

{ TFanceSpeedBtn - Fancy button with additional features
 Features:
 1. Gradients
    A. Set "Gradient" to true enable gradient
    B. GradColor1, GradColor2 sets gradient start/end color
 2. Button Box Frame Colors = FrameDarkest, FrameDark, FrameLight, FrameLightest
 3. Supports Glyphs including multiple glyphs the same as TSpeedButton
 4. Supports "AllowAllUp" and "GroupIndex" like TSpeedButton
 5. Supports "DoRepeat" like TAutoButton

This control does not accept focus and it can't respond to keystrokes.
This is because it is based on a TSpeedButton, which is based on the
TGraphicControl which does not have a Window. To make make it focus
and respond to keystrokes, you'd need to write a new TSpeedButton
control based on the TWinControl object. I already have the routines
for drawing a focus box at the point that it can he focused.

In the meantime, use the TFancyButton instead.
}



type TFancySpeedBtn = class(TAutoButton)
 private
  FDDown: Boolean;
  FGlyph: TBitmap;
  Hovering: boolean;
  IsDefault, IsFocused: boolean;
  IsDown, IsDisabled: boolean;
  FGradient: boolean;
  FGradColor2: TColor;
  FGradColor1: TColor;
  FSpacing: Integer;
  FMargin: Integer;
  FFrameDarkest: TColor;
  FFrameLightest: TColor;
  FFrameDark: TColor;
  FFrameLight: TColor;
  BMArray: array [0..3] of TBitmap;
  FDownInvert: boolean;
  GlyphIndex: integer;
  FStepGlyphs: boolean;
  FBlockClick: boolean;
  FStandardFrame: boolean;
  procedure ShowText(Disabled: boolean; Box: TRect);
  procedure DrawFocusRect(Rect: TRect);
  procedure SetGradient(const Value: boolean);
  procedure DrawBaseButton(Rect: TRect);
  procedure DrawButtonFrame(var Rect: TRect);
  procedure DrawEmbossedText(Rect: TRect);
  procedure SetGradColor1(const Value: TColor);
  procedure SetGradColor2(const Value: TColor);
  function GetGlyph: TBitmap;
  procedure SetGlyph(const Value: TBitmap);
  procedure SetSpacing(const Value: Integer);
  procedure SetMargin(const Value: Integer);
  procedure GetLayout(BtnRect: TRect; var BitmapRect, TextRect: TRect);
  function ValidGlyph: boolean;
  procedure SetFrameLightest(const Value: TColor);
  procedure SetFrameDarkest(const Value: TColor);
  procedure SetFrameDark(const Value: TColor);
  procedure SetFrameLight(const Value: TColor);
  procedure DrawGlyph(Inx, X, Y: integer);
  function GetImageIndex: integer;
  procedure GlyphChange(Sender: TObject);
  function GetNumGlyphs: TNumGlyphs;
  procedure SetNumGlyphs(const Value: TNumGlyphs);
    procedure SetDownInvert(const Value: boolean);
    procedure SetStepGlyphs(const Value: boolean);
    procedure SetBlockClick(const Value: boolean);
    procedure SetStandardFrame(const Value: boolean);
 protected
  procedure DrawButton(Rect: TRect);
  procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
  procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
  procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X,Y: Integer); override;
  procedure Paint; override;
 public
  property BlockClick: boolean read FBlockClick write SetBlockClick;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure Click; override;
 published
  property FrameDarkest: TColor read FFrameDarkest write SetFrameDarkest;
  property FrameDark: TColor read FFrameDark write SetFrameDark;
  property FrameLightest: TColor read FFrameLightest write SetFrameLightest;
  property FrameLight: TColor read FFrameLight write SetFrameLight;
  property Margin: Integer read FMargin write SetMargin default 4;
  property Spacing: Integer read FSpacing write SetSpacing default 4;
  property Glyph: TBitmap read GetGlyph write SetGlyph;
  property GradColor1: TColor read FGradColor1 write SetGradColor1 default clWhite;
  property GradColor2: TColor read FGradColor2 write SetGradColor2 default clBlack;
  property Gradient: boolean read FGradient write SetGradient default False;
  property DownInvert: boolean read FDownInvert write SetDownInvert default false;
  property Color;
  property NumGlyphs: TNumGlyphs read GetNumGlyphs write SetNumGlyphs default 1;
  property StepGlyphs: boolean read FStepGlyphs write SetStepGlyphs default False;
  property StandardFrame: boolean read FStandardFrame write SetStandardFrame default False;
end;




procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TFancyButton,TFancySpeedBtn]);
end;



function LightenColor(C: TColor; Percent: double): TColor;
{Increase the lightness by a specific amount}
{Increases the RGB values by a percentage of the amount left}
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
R:=round(((255-R)*Percent)+R);
G:=round(((255-G)*Percent)+G);
B:=round(((255-B)*Percent)+B);
Result:=RGB(R,G,B);
end;

{================== TFancyButton Code =========================================}


{----------------- Routines to Draw Button ------------------------------------}


function TFancyButton.ValidGlyph: boolean;
begin
Result:=(FGlyph <> nil) and (FGlyph.Height > 0);
end;



procedure TFancyButton.GetLayout(BtnRect: TRect; var BitmapRect, TextRect: TRect);
var Base: TRect;
var G,T: TPoint;
begin
T:=SizeToPoint(FCanvas.TextExtent(Caption));
if ValidGlyph then G:=Point(Glyph.Width+FSpacing,Glyph.Height)
else G:=Point(0,0);

Base:=Rect(0,0,G.X+T.X,T.Y);
Base:=CenterRectOnRect(Base,BtnRect);
if FMargin>=0 then OffsetRect(Base,FMargin-Base.Left,0);
TextRect:=Rect(Base.Left+G.X,Base.Top,Base.Right,Base.Bottom);

Base:=Rect(0,0,G.X+T.X,G.Y);
Base:=CenterRectOnRect(Base,BtnRect);
if FMargin>=0 then OffsetRect(Base,FMargin-Base.Left,0);
BitmapRect:=Rect(Base.Left,Base.Top,Base.Left+G.X,Base.Top+G.Y);
end;



procedure TFancyButton.DrawEmbossedText(Rect: TRect);
{Draw embossed style text}
begin
DrawStateA(FCanvas.Handle, FCanvas.Brush.Handle, nil, Integer(Caption), 0,
	  Rect.Left,Rect.Top, 0, 0, DST_TEXT or DSS_DISABLED)
end;


procedure TFancyButton.ShowText(Disabled: boolean; Rect: TRect);
{Display Text Caption on button}
var OldMode: Longint;
var BR,TR: TRect;
var X,Y: integer;
begin
if IsDown then OffsetRect(Rect,1,1);
OldMode:=SetBkMode(FCanvas.Handle, TRANSPARENT);
GetLayout(Rect,BR,TR);
if ValidGlyph then
	begin
   	Y:=(Rect.Bottom-Rect.Top-FGlyph.Height) div 2;
	FGlyph.Transparent:=True;
	FGlyph.TransParentColor:=FGlyph.Canvas.Pixels[0,FGlyph.Height-1];
      	FGlyph.TransparentMode:=tmAuto;
	FCanvas.Draw(BR.Left,BR.Top,FGlyph);
	end;
if Disabled then DrawEmbossedText(TR)
else DrawText(FCanvas.Handle, PChar(Caption), - 1, TR, DT_SINGLELINE or DT_Center or DT_VCENTER);
SetBkMode(FCanvas.Handle, OldMode);
end;



procedure TFancyButton.DrawFocusRect(Rect: TRect);
{Draw a dotted focus box on button}
begin
if Gradient then InflateRect(Rect, - 2, - 2)
else InflateRect(Rect, - 4, - 4);
FCanvas.Pen.Mode:=pmCopy;
FCanvas.Pen.Color:=clWindowFrame;
FCanvas.Brush.Color:=clBtnFace;
FCanvas.DrawFocusRect(Rect);
end;




procedure TFancyButton.DrawButtonFrame(var Rect: TRect);
{Draw the outer frame work of the button - ignoring the center}
begin
if IsDefault or IsFocused then
	begin
	{Draw the outer focus box}
	FCanvas.Pen.Color:=FrameDarkest;
	FCanvas.Rectangle(Rect);
	InflateRect(Rect,-1,-1);
	end;
if IsDown then
	begin
	{Add the light border used when button is down}
	FCanvas.Pen.Color:=FrameDark;
	FCanvas.Rectangle(Rect);
	InflateRect(Rect,-1,-1);
	end
else
	begin
	{Add the two-layter border when button is up}
	Draw3DRect(FCanvas,Rect,FrameLightest,FrameDarkest);
	InflateRect(Rect,-1,-1);
	Draw3DRect(FCanvas,Rect,FrameLight,FrameDark);
	InflateRect(Rect,-1,-1);
	end;
end;




procedure TFancyButton.DrawBaseButton(Rect: TRect);
{Draw basic button without Text or Focus highlight}
var C1,C2: TColor;

	procedure DoColorScheme;
	{Work out color scheme}
	var Bright: double;
	begin
	if IsDown then
		begin
		if not DownInvert then Bright:=0.5
		end
	else if Hovering then Bright:=0.25
	else Bright:=0;
	FCanvas.Brush.Color:=LightenColor(Color,Bright);
	C1:=LightenColor(GradColor1,Bright);
	C2:=LightenColor(GradColor2,Bright);
	if DownInvert and IsDown then SwapColors(C1,C2);
	end;

begin
DoColorScheme;
DrawButtonFrame(Rect);
if Gradient then VertGradientRect(FCanvas, Rect,C1,C2)
else FCanvas.FillRect(Rect);
end;




procedure TFancyButton.DrawButton(Rect: TRect);
{Draw full button}
begin
DrawBaseButton(Rect);
ShowText(IsDisabled,Rect);
if IsDefault and IsFocused then DrawFocusRect(Rect);
end;


{-----------------------------------------------------------------------------}


procedure TFancyButton.CreateParams(var Params: TCreateParams);
begin
inherited CreateParams(Params);
with Params do
Style:=Style or BS_OWNERDRAW;
end;



constructor TFancyButton.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FCanvas:=TCanvas.Create;
FGlyph:=TBitmap.Create;
Color:=clBtnFace;
GradColor1:=clWhite;
GradColor2:=clBlack;
FFrameDarkest:=cl3DDKShadow;
FFrameDark:=clBtnShadow;
FFrameLight:=cl3DLight;
FFrameLightest:=clBtnHighlight;
Gradient:=False;
FSpacing:=4;
FMargin:=-1;
end;



destructor TFancyButton.Destroy;
begin
FGlyph.Free;
FCanvas.Free;
inherited Destroy;
end;


procedure TFancyButton.SetButtonStyle(Value: Boolean);
{Intercepts call that defines whether button is a Default-Button or not}
{Default-Buttons always receive input even if they don't have the focus}
begin
if Value <> IsDefault then
	begin
	IsDefault:=Value;
	Invalidate;
	end;
end;

{-------------- Windows event handlers -------------------------------------}



procedure TFancyButton.CNDrawItem(var Message: TWMDrawItem);
var SaveIndex: Integer;
begin
with Message.DrawItemStruct^ do
 begin
 SaveIndex:=SaveDC(hDC);
 FCanvas.Lock;
 try
 FCanvas.Handle:=hDC;
 FCanvas.Font:=Font;
 FCanvas.Brush:=Brush;
 IsDown:=itemState and ODS_SELECTED <> 0;
 IsDisabled:=(itemState and ODS_DISABLED <> 0);
 IsFocused:=itemState and ODS_FOCUS <> 0;
 DrawButton(rcItem);
 finally
  FCanvas.Handle:=0;
  FCanvas.Unlock;
  RestoreDC(hDC, SaveIndex);
  end;
 end;
Message.Result:=1;
end;


procedure TFancyButton.CMMouseEnter(var Message: TMessage);
begin
Hovering:=True;
invalidate;
end;



procedure TFancyButton.CMMouseLeave(var Message: TMessage);
begin
Hovering:=False;
invalidate;
end;



procedure TFancyButton.CNMeasureItem(var Message: TWMMeasureItem);
begin
with Message.MeasureItemStruct^ do
	begin
	itemWidth :=Width;
	itemHeight:=Height;
	end;
end;



procedure TFancyButton.CMEnabledChanged(var Message: TMessage);
begin
inherited;
Invalidate;
end;



procedure TFancyButton.CMFontChanged(var Message: TMessage);
begin
inherited;
Invalidate;
end;


{-------------- Property handlers -----------------------------------------}


procedure TFancyButton.SetDownState(Value: Boolean);
begin
 FDDown := Value;
 SendMessage(self.Handle, BM_SETSTATE, Longint(Value), 0);
end;



function TFancyButton.GetDownState: boolean;
begin
Result:=(SendMessage(self.Handle, BM_GETSTATE, 0, 0) and $4)<>0;
end;


procedure TFancyButton.SetGradient(const Value: boolean);
begin
if FGradient<>Value then
	begin
	FGradient:=Value;
	Invalidate;
	end;
end;




procedure TFancyButton.SetGradColor1(const Value: TColor);
begin
if Value<>FGradColor1 then
	begin
	FGradColor1 := Value;
	Invalidate;
	end;
end;




procedure TFancyButton.SetGradColor2(const Value: TColor);
begin
if Value<>FGradColor2 then
	begin
  	FGradColor2 := Value;
	Invalidate;
	end;
end;



function TFancyButton.GetGlyph: TBitmap;
begin
Result:=FGlyph;
end;



procedure TFancyButton.SetGlyph(const Value: TBitmap);
begin
FGlyph.Assign(Value);
Invalidate;
end;


procedure TFancyButton.SetSpacing(const Value: Integer);
begin
if Value <> FSpacing then
	begin
	FSpacing := Value;
	Invalidate;
	end;
end;



procedure TFancyButton.SetMargin(const Value: Integer);
begin
if (Value <> FMargin) and (Value >= -1) then
	begin
	FMargin := Value;
	Invalidate;
	end;
end;


procedure TFancyButton.SetFrameLightest(const Value: TColor);
begin
if FFrameLightest<>Value then
	begin
	FFrameLightest := Value;
	Invalidate;
	end
end;

procedure TFancyButton.SetFrameDarkest(const Value: TColor);
begin
if FFrameDarkest<>Value then
	begin
	FFrameDarkest := Value;
	Invalidate;
	end;
end;


procedure TFancyButton.SetFrameDark(const Value: TColor);
begin
if FFrameDark<>Value then
	begin
	FFrameDark := Value;
	Invalidate;
	end;
end;

procedure TFancyButton.SetFrameLight(const Value: TColor);
begin
if FFrameLight<>Value then
	begin
	FFrameLight := Value;
	Invalidate;
	end
end;



procedure TFancyButton.SetDownInvert(const Value: boolean);
begin
FDownInvert := Value;
Invalidate;
end;

{================== TFancySpeedBtn Code =======================================}




{There as many as four bitmaps embedded in the Glyph. The number is specified
by NumGlyphs. There are used as follows 1) Up image, 2) Disabled Image,
3) Held-down image 4) Stays-down image. }



procedure TFancySpeedBtn.GlyphChange(Sender: TObject);
{Rebuild Bitmap array when main glyph changes}
var I: integer;
var BW,Off: integer;
var Src,Dest: TRect;
var BM: TBitmap;


	procedure CopyBitmap(Inx: integer; var BM: TBitmap);
	{Copy the sub-bitmaps from glyph}
	var Off: integer;
	begin
	BM.Width:=BW;
	BM.Height:=FGlyph.Height;
	{Use first sub-bitmap if we are beyond number available}
	if Inx>=NumGlyphs then Inx:=0;
	{Copy spefic portion of Glyph to BM}
	Off:=Inx * BW;
	Src:=Dest;
	OffsetRect(Src,Off,0);
	BM.Monochrome:=False;
	BM.Canvas.CopyRect(Dest,FGlyph.Canvas,Src);
	BM.Transparent:=True;
	BM.TransParentColor:=BM.Canvas.Pixels[0,FGlyph.Height-1];
	BM.TransparentMode:=tmAuto;
	end;


begin
BW:=FGlyph.Width div NumGlyphs;
Dest:=Rect(0,0,BW,FGlyph.Height);
for I:=0 to High(BMArray) do
	begin
	CopyBitmap(I,BMArray[I]);
	end;
{Gray out second bitmap }
if NumGlyphs<2 then
	begin
	{This is necessary to prevent memory leak}
	BM:=CreateDisabledBitmap(BMArray[1]);
	try
	BMArray[1].Assign(BM);
	finally BM.Free; end;
	end;
end;

{----------------- Routines to Draw Button ------------------------------------}


function TFancySpeedBtn.ValidGlyph: boolean;
begin
Result:=(FGlyph <> nil) and (FGlyph.Height > 0);
end;



procedure TFancySpeedBtn.GetLayout(BtnRect: TRect; var BitmapRect, TextRect: TRect);
var Base: TRect;
var G,T: TPoint;
begin
T:=SizeToPoint(Canvas.TextExtent(Caption));
if ValidGlyph then G:=Point(Glyph.Width div NumGlyphs+FSpacing,Glyph.Height)
else G:=Point(0,0);

Base:=Rect(0,0,G.X+T.X,T.Y);
Base:=CenterRectOnRect(Base,BtnRect);
if FMargin>=0 then OffsetRect(Base,FMargin-Base.Left,0);
TextRect:=Rect(Base.Left+G.X,Base.Top,Base.Right,Base.Bottom);

Base:=Rect(0,0,G.X+T.X,G.Y);
Base:=CenterRectOnRect(Base,BtnRect);
if FMargin>=0 then OffsetRect(Base,FMargin-Base.Left,0);
BitmapRect:=Rect(Base.Left,Base.Top,Base.Left+G.X,Base.Top+G.Y);
end;



procedure TFancySpeedBtn.DrawEmbossedText(Rect: TRect);
{Draw embossed style text}
begin
DrawStateA(Canvas.Handle, Canvas.Brush.Handle, nil, Integer(Caption), 0,
	  Rect.Left,Rect.Top, 0, 0, DST_TEXT or DSS_DISABLED)
end;




procedure TFancySpeedBtn.DrawGlyph(Inx: integer; X,Y: integer);
{Draw part of Glyph specified by Inx at loc X,Y}
var Src,Dest: TRect;
var BW,Off: integer;
var BM: TBitmap;
begin
Canvas.Draw(X,Y,BMArray[Inx]);
end;




function TFancySpeedBtn.GetImageIndex: integer;
{Get the image index based on the Button State}
begin
if StepGlyphs then Result:=GlyphIndex
else
	begin
	if BlockClick then Result:=1
	else case FState of
	 bsUp: Result:=0;
	 bsDisabled: Result:=1;
	 bsDown: Result:=2;
	 bsExclusive: Result:=3;
	 end;
	end;
end;


procedure TFancySpeedBtn.ShowText(Disabled: boolean; Box: TRect);
{Display Text Caption on button}
var OldMode: Longint;
var X,Y: integer;
var BR,TR: TRect;
var DR,SR: TRect;
var ImageInx: integer;
begin
if IsDown then OffsetRect(Box,1,1);
OldMode:=SetBkMode(Canvas.Handle, Windows.TRANSPARENT);
GetLayout(Box,BR,TR);
if ValidGlyph then DrawGlyph(GetImageIndex,BR.Left,BR.Top);
if Disabled then DrawEmbossedText(TR)
else DrawText(Canvas.Handle, PChar(Caption), - 1, TR, DT_SINGLELINE or DT_Center or DT_VCENTER);
SetBkMode(Canvas.Handle, OldMode);
end;



procedure TFancySpeedBtn.DrawFocusRect(Rect: TRect);
{Draw a dotted focus box on button}
begin
if Gradient then InflateRect(Rect, - 2, - 2)
else InflateRect(Rect, - 4, - 4);
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Color:=clWindowFrame;
Canvas.Brush.Color:=clBtnFace;
Canvas.DrawFocusRect(Rect);
end;





procedure TFancySpeedBtn.DrawButtonFrame(var Rect: TRect);
{Draw the outer frame work of the button - ignoring the center}
begin
if IsDefault or IsFocused then
	begin
	{Draw the outer focus box}
	Canvas.Pen.Color:=FrameDarkest;
	Canvas.Rectangle(Rect);
	InflateRect(Rect,-1,-1);
	end;
if IsDown and StandardFrame then
	begin
	DrawFrameControl(Canvas.Handle, Rect, DFC_BUTTON, DFCS_BUTTONPUSH or DFCS_ADJUSTRECT or DFCS_PUSHED);
	end
else if IsDown then
	begin
	{Add the light border used when button is down}
	Canvas.Pen.Color:=FrameDark;
	Canvas.Rectangle(Rect);
	InflateRect(Rect,-1,-1);
	end
else
	begin
	{Add the two-layter border when button is up}
	Draw3DRect(Canvas,Rect,FrameLightest,FrameDarkest);
	InflateRect(Rect,-1,-1);
	Draw3DRect(Canvas,Rect,FrameLight,FrameDark);
	InflateRect(Rect,-1,-1);
	end;
end;




procedure TFancySpeedBtn.DrawBaseButton(Rect: TRect);
{Draw basic button without Text or Focus highlight}
var C1,C2: TColor;

	procedure DoColorScheme;
	{Work out color scheme}
	var Bright: double;
	begin
	if IsDown then
		begin
		if not DownInvert then Bright:=0.5
		end
	else if Hovering then Bright:=0.25
	else Bright:=0;
	Canvas.Brush.Color:=LightenColor(Color,Bright);
	C1:=LightenColor(GradColor1,Bright);
	C2:=LightenColor(GradColor2,Bright);
	if DownInvert and IsDown then SwapColors(C1,C2);
	end;

begin
DoColorScheme;
DrawButtonFrame(Rect);
if Gradient then VertGradientRect(Canvas, Rect,C1,C2)
else Canvas.FillRect(Rect);
end;




procedure TFancySpeedBtn.DrawButton(Rect: TRect);
{Draw full button}
begin
DrawBaseButton(Rect);
ShowText(IsDisabled,Rect);
if IsDefault and IsFocused then DrawFocusRect(Rect);
end;



procedure TFancySpeedBtn.Paint;
begin
inherited;
IsDown:=FState in [bsDown, bsExclusive];
IsDisabled:=not Enabled;
DrawButton(Rect(0, 0, Width, Height));
end;


{-----------------------------------------------------------------------------}



constructor TFancySpeedBtn.Create(AOwner: TComponent);
var I: integer;
begin
inherited Create(AOwner);
ControlStyle:=ControlStyle+[csSetCaption];

for I:=0 to High(BMArray) do BMArray[I]:=TBitmap.Create;
FGlyph:=TBitmap.Create;
Width:=75;
Height:=25;
Caption:=Name;
FGlyph.OnChange:=GlyphChange;
Color:=clBtnFace;
GradColor1:=clWhite;
GradColor2:=clBlack;
FFrameDarkest:=cl3DDKShadow;
FFrameDark:=clBtnShadow;
FFrameLight:=cl3DLight;
FFrameLightest:=clBtnHighlight;
Gradient:=False;
FSpacing:=4;
FMargin:=-1;
StandardFrame:=False;

end;



destructor TFancySpeedBtn.Destroy;
var I: integer;
begin
for I:=0 to High(BMArray) do BMArray[I].Free;
FGlyph.Free;
inherited Destroy;
end;


{-------------- Windows event handlers -------------------------------------}



procedure TFancySpeedBtn.CMMouseEnter(var Message: TMessage);
begin
Hovering:=True;
invalidate;
end;



procedure TFancySpeedBtn.CMMouseLeave(var Message: TMessage);
begin
Hovering:=False;
invalidate;
end;



procedure TFancySpeedBtn.CNMeasureItem(var Message: TWMMeasureItem);
begin
with Message.MeasureItemStruct^ do
	begin
	itemWidth :=Width;
	itemHeight:=Height;
	end;
end;



procedure TFancySpeedBtn.CMEnabledChanged(var Message: TMessage);
begin
inherited;
Invalidate;
end;



procedure TFancySpeedBtn.CMFontChanged(var Message: TMessage);
begin
inherited;
Invalidate;
end;



procedure TFancySpeedBtn.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var Temp: TButtonState;
begin
inherited;
if StepGlyphs then
	begin
	Inc(GlyphIndex);
	if GlyphIndex>=NumGlyphs then GlyphIndex:=0;
	end;
end;

{-------------- Property handlers -----------------------------------------}




procedure TFancySpeedBtn.SetGradient(const Value: boolean);
begin
if FGradient<>Value then
	begin
	FGradient:=Value;
	Invalidate;
	end;
end;




procedure TFancySpeedBtn.SetGradColor1(const Value: TColor);
begin
if Value<>FGradColor1 then
	begin
	FGradColor1 := Value;
	Invalidate;
	end;
end;




procedure TFancySpeedBtn.SetGradColor2(const Value: TColor);
begin
if Value<>FGradColor2 then
	begin
  	FGradColor2 := Value;
	Invalidate;
	end;
end;



function TFancySpeedBtn.GetGlyph: TBitmap;
begin
Result:=FGlyph;
end;



procedure TFancySpeedBtn.SetGlyph(const Value: TBitmap);
begin
FGlyph.Assign(Value);
Invalidate;
end;


procedure TFancySpeedBtn.SetSpacing(const Value: Integer);
begin
if Value <> FSpacing then
	begin
	FSpacing := Value;
	Invalidate;
	end;
end;



procedure TFancySpeedBtn.SetMargin(const Value: Integer);
begin
if (Value <> FMargin) and (Value >= -1) then
	begin
	FMargin := Value;
	Invalidate;
	end;
end;


procedure TFancySpeedBtn.SetFrameLightest(const Value: TColor);
begin
if FFrameLightest<>Value then
	begin
	FFrameLightest := Value;
	Invalidate;
	end
end;

procedure TFancySpeedBtn.SetFrameDarkest(const Value: TColor);
begin
if FFrameDarkest<>Value then
	begin
	FFrameDarkest := Value;
	Invalidate;
	end;
end;


procedure TFancySpeedBtn.SetFrameDark(const Value: TColor);
begin
if FFrameDark<>Value then
	begin
	FFrameDark := Value;
	Invalidate;
	end;
end;

procedure TFancySpeedBtn.SetFrameLight(const Value: TColor);
begin
if FFrameLight<>Value then
	begin
	FFrameLight := Value;
	Invalidate;
	end
end;


function TFancySpeedBtn.GetNumGlyphs: TNumGlyphs;
begin
Result:=inherited NumGlyphs;
end;

procedure TFancySpeedBtn.SetNumGlyphs(const Value: TNumGlyphs);
begin
inherited NumGlyphs:=Value;
GlyphChange(Self);
end;


procedure TFancySpeedBtn.SetDownInvert(const Value: boolean);
begin
  FDownInvert := Value;
end;



procedure TFancySpeedBtn.SetStepGlyphs(const Value: boolean);
begin
if FStepGlyphs<>Value then
	begin
	FStepGlyphs := Value;
	GlyphIndex:=0;
	end;
end;



procedure TFancySpeedBtn.Click;
begin
if not BlockClick then inherited;
end;



procedure TFancySpeedBtn.SetBlockClick(const Value: boolean);
begin
if FBlockClick<>Value then
	begin
	FBlockClick := Value;
	Repaint;
	end;
end;

procedure TFancySpeedBtn.SetStandardFrame(const Value: boolean);
begin
if FStandardFrame<>Value then
	begin
	FStandardFrame := Value;
	Invalidate;
	end;
end;

end.
