unit FlexGrid;


interface

uses Messages, Windows, SysUtils, Classes, Grids, Variants, Graphics,
  Menus, Controls, Forms, StdCtrls, Mask,DSupport,ScrollBarWindow,IntVector;


type TCellSize = record
 Size,SplitPos,Offset: integer;
 end;

type TCellSizeArray = array of TCellSize;

type TCellPos = (cpInside,cpOutLeft,cpOutRight,cpOutTop,cpOutBottom);
type TCellStates = set of TCellPos;



type TFlexGrid = class(TDrawGrid)
 private
  FScrollOffset: TPoint;
  FColSizes: TCellSizeArray;
  FRowSIzes: TCellSizeArray;
  function GetOffsetRect(X, Y: integer): TRect;
    procedure Draw3DRect(Canvas: TCanvas; R: TRect);
    procedure DrawAllCells;
    procedure DrawCell(X, Y: integer);
    function GetCellState(X, Y: integer; var R: TRect): TCellStates;
    function IsFixed(Col, Row: integer): boolean;
    procedure Paint;
 protected
 public
 published
 end;



procedure Register;

implementation


uses Math, Consts, RTLConsts;


procedure Register;
begin
RegisterComponents('Fountain', [TFlexGrid]);
end;

{---------------- Cell drawing routines --------------------------------------}






procedure TFlexGrid.Draw3DRect(Canvas: TCanvas; R: TRect);
{Draw the 3D raise rectangle for fixed cells}
begin
Canvas.Pen.Color:=clWhite;
Canvas.Polyline([Point(R.Left,R.Bottom),Point(R.Left,R.Top),Point(R.Right,R.Top)]);
Canvas.Pen.Color:=clBtnShadow;
Canvas.Polyline([Point(R.Left+1,R.Bottom-1),Point(R.Right-1,R.Bottom-1),Point(R.Right-1,R.Top)]);
end;



function TFlexGrid.IsFixed(Col,Row: integer): boolean;
{Determines if a cell is in the fixed region}
begin
Result:=(Col<FixedCols) or (Row<FixedRows);
end;



function TFlexGrid.GetCellState(X,Y: integer; var R: TRect): TCellStates;
var X2,Y2: integer;
begin
Result:=[];
if (FixedCols>0) and (X>=FixedCols) then
	begin
	X2:=X-FScrollOffset.X;
	if X2<FixedCols then Result:=Result+[cpOutLeft];
	end;
if (FixedRows>0) and (Y>=FixedRows) then
	begin
	Y2:=Y-FScrollOffset.Y;
	if Y2<FixedRows then Result:=Result+[cpOutTop];
	end;
R:=GetOffsetRect(X,Y);
if R.Left>Width then Result:=Result+[cpOutRight];
if R.Top>Height then Result:=Result+[cpOutBottom];
if Result=[] then Result:=[cpInside];
end;



procedure TFlexGrid.DrawCell(X,Y: integer);
{Draw the specified cell}
var R: TRect;
var DrawState: TGridDrawState;


	procedure ClearCell(Fixed: boolean);
	var C: TColor;
	begin
	if Fixed then C:=FixedColor else C:=clWhite;
	Canvas.Brush.Color:=C;
	Canvas.Pen.Color:=C;
	Canvas.Rectangle(R);
	end;


	procedure DrawFixedCell(X,Y: integer);
	{Draw the specified cell as a fixed cell}
	begin
	Draw3DRect(Canvas,R);
	Canvas.Pen.Color:=clBlack;
	Canvas.Polyline([Point(R.Left,R.Bottom),Point(R.Right,R.Bottom),Point(R.Right,R.Top-1)]);
	end;


	procedure DrawNormalCell(X,Y: integer);
	{Draw the specified cell as a normal cell}
	begin
	Canvas.Pen.Color:=clSilver;
	Canvas.Polyline([Point(R.Left,R.Bottom),Point(R.Right,R.Bottom),Point(R.Right,R.Top-1)]);
        if (Y>=FixedRows) and (FColSizes[X].Offset<>0) then Canvas.Polyline([Point(R.Left,R.Top-1),Point(R.Right,R.Top-1)]);
        if (X>FixedCols) then Canvas.Polyline([Point(R.Left-1,R.Top-1),Point(R.Left-1,R.Bottom)]);

	if gdSelected in DrawState  then
		begin
		DrawFocusRect(Canvas.Handle,R);
		//PlaceEditor(R,X,Y);
		end;
	end;


begin
R:=GetOffsetRect(X,Y);
DrawState:=[];
if IsFixed(X,Y) then Include(DrawState,gdFixed);
if (FSelected.X=X) and (FSelected.Y=Y) then Include(DrawState, gdSelected);
if Self.Focused then Include(DrawState, gdFocused);
ClearCell(gdFixed in DrawState);
if Assigned(FOnDrawCell) then FOnDrawCell(Self, X, Y, R, DrawState);
Canvas.Font:=Self.Font;
if gdFixed in DrawState then DrawFixedCell(X,Y)
else DrawNormalCell(X,Y);
end;




procedure TFlexGrid.DrawAllCells;
var X,Y,Start: integer;
var CS: TCellStates;
var R: TRect;
begin
{Draw }
for X:=FixedCols+FScrollOffset.X to ColCount-1 do
	begin
	CS:=GetCellState(X,0,R);
	if cpOutRight in CS then break;
	Start:=FixedRows+FScrollOffset.Y;
	if FColSizes[X].Offset<>0 then
		Start:=Start-(1+(FColSizes[X].Offset div (R.Bottom-R.Top)));
	for Y:=Start to RowCount-1 do
		begin
		CS:=GetCellState(X,Y,R);
		if cpOutBottom in CS then break
		else
			begin
			DrawCell(X,Y);
			end;
		end;
	end;

{Draw up left corner}
DrawCell(0,0);
{Draw top fixed rows}
for Y:=0 to FixedRows-1 do
 for X:=FixedCols+FScrollOffset.X to ColCount-1 do
	if cpInside in GetCellState(X,Y,R) then DrawCell(X,Y)
	else break;

{Draw left fixed columns}
for X:=0 to FixedCols-1 do
 for Y:=FixedRows+FScrollOffset.Y to RowCount-1 do
	if cpInside in GetCellState(X,Y,R) then DrawCell(X,Y)
	else break;
end;



procedure TFlexGrid.Paint;
begin
inherited;
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Color;
Canvas.Rectangle(Self.ClientRect);
DrawAllCells;
end;
end.
