unit ImageEx;

interface

uses  Windows,Classes,Types,Messages,Graphics,Controls,StdCtrls,ExtCtrls;

type TImageEx = class(TImage)
 private
  FBackBitmap,FOverlayBitmap: TBitmap;
  FOnMouseLeave: TNotifyEvent;
  FOnMouseEnter: TNotifyEvent;
    FOnBackChanged: TNotifyEvent;
  procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
  procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
 protected
  procedure BackChanged;
 public
  procedure SaveImage;
  procedure SaveOverlay;
  procedure RestoreImage;
  procedure RestoreRectangle(R: TRect);
  procedure RestoreOverlay; overload;
  procedure RestoreOverlay(R: TRect); overload;
  property SavedImage: TBitmap read FBackBitmap;
  property OverlayImage: TBitmap read FOverlayBitmap;
 published
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
  property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  property OnBackChanged: TNotifyEvent read FOnBackChanged write FOnBackChanged;
 end;


procedure Register;


implementation

uses BitmapLib;


procedure Register;
begin
RegisterComponents('Fountain', [TImageEx]);
end;


{ TImageEx }


procedure TImageEx.RestoreImage;
begin
CopyBitmap(Picture.Bitmap,FBackBitmap);
end;



procedure TImageEx.RestoreOverlay;
begin
CopyBitmap(Picture.Bitmap,FOverlayBitmap);
end;



procedure TImageEx.RestoreRectangle(R: TRect);
begin
Picture.Bitmap.Canvas.CopyRect(R,FBackBitmap.Canvas,R);
end;


procedure TImageEx.RestoreOverlay(R: TRect);
begin
Picture.Bitmap.Canvas.CopyRect(R,FOverlayBitmap.Canvas,R);
end;



procedure TImageEx.SaveImage;
begin
CopyBitmap(FBackBitmap,Picture.Bitmap);
BackChanged;
end;


procedure TImageEx.SaveOverlay;
begin
CopyBitmap(FOverlayBitmap,Picture.Bitmap);
end;




constructor TImageEx.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FBackBitmap:=TBitmap.Create;
FOverlayBitmap:=TBitmap.Create;
end;




destructor TImageEx.Destroy;
begin
FBackBitmap.Free;
FOverlayBitmap.Free;
inherited;
end;




procedure TImageEx.CMMouseEnter(var Message: TMessage);
begin
inherited;
if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;



procedure TImageEx.CMMouseLeave(var Message: TMessage);
begin
inherited;
if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;


procedure TImageEx.BackChanged;
begin
if Assigned(FOnBackChanged) then FOnBackChanged(Self);
end;


end.
