unit ImageList;

interface

uses Forms, Windows, Messages, Controls, SysUtils, Graphics, Grids, StdCtrls,
     Classes, ClipBrd, BitmapLib, GraphicsLib, Filmstp, HugeArry, TreeLib;

{Used to access protected method - protected methods
{ are only accessible inside the same module}


type TXBitmap = class(TBitmap)
 end;

type TImageListBox = class(TCustomListBox)
 private
  FImageHeight: integer;
  FTextHeight: integer;
  FSizeArray: THugePoint;
  FAuxInfo: THugeInteger;
  FZeroBased,FFixedSize: boolean;
  FShowLabel,FShowIndex,FShowSize: boolean;
  FAutoScroller: TAutoScroller;
  FOnChange: TNotifyEvent;
  FHighlightColor: TColor;
  function GetBitmap(Inx: integer): TBitmap;
  procedure PutBitmap(Inx: integer; Item: TBitmap);
  procedure SetFixedSize(V: boolean);
  procedure SetShowIndex(V: boolean);
  procedure SetShowSize(V: boolean);
  procedure SetZeroBased(V: boolean);
  procedure SetHighlightColor(V: TColor);
  procedure DoClear(TriggerChange: boolean);
  function GetLabel(Index: integer): string;
  procedure PutLabel(Inx: integer; const Value: string);
  function GetAuxInfo(Index: integer): integer;
  procedure PutAuxInfo(Index: integer; const Value: integer);
  function CreateBitmap: TBitmap;
  procedure HandleBitmapChange(Sender: TObject);
    procedure SetShowLabel(const Value: boolean);
 protected
  function GetImageHeight(Index: integer; Fixed: boolean): integer;
  procedure MeasureItem(Index: Integer; var Height: Integer); override;
  procedure HighlightImage(Canvas: TCanvas; var Rect: TRect);
  procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  procedure Resize; override;
  procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean); override;
  procedure DragDrop(Source: TObject; X, Y: Integer); override;
  procedure Changed; dynamic;
  function GetSelectCount: integer; virtual;
 public
  Modified: boolean;
  procedure ForceRemeasure;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure Assign(Source: TPersistent); override;
  function AddBitmap(BM: TBitmap; S: string): integer; overload; virtual;
  function AddBitmap(BM: TBitmap): integer; overload; virtual;
  procedure InsertBitmap(Index: integer; BM: TBitmap); overload; virtual;
  procedure InsertBitmap(Index: integer; BM: TBitmap; S: string); overload; virtual;
  procedure Clear; override;
  procedure Delete(Index: Integer); virtual;
   procedure DeleteSelected;
    function GetFileSize: int64;
    function GetSelectedSize: int64;
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    procedure SaveSelectedToStream(Stream: TStream);
  procedure ReAnalyzeSize;
  procedure Load(Filename: string; Progress: TBitmapProgress);
  procedure Save(Filename: string; Progress: TBitmapProgress);
  procedure PasteFromClipboard;
  procedure CopyToClipboard(Inx: integer);
  procedure Exchange(Index1,Index2: integer); virtual;
  procedure Move(CurIndex, NewIndex: Integer); virtual;
  property Bitmaps[Inx: integer]: TBitmap read GetBitmap write PutBitmap;
  property Labels[Inx: integer]: string read GetLabel write PutLabel;
  property AuxInfo[Inx: integer]: integer read GetAuxInfo write PutAuxInfo;
  procedure BeginUpdate;
  procedure EndUpdate;
 published
  property ImageHeight: integer read FImageHeight write FImageHeight;
  property FixedSize: boolean read FFixedSize write SetFixedSize;
  property ShowIndex: boolean read FShowIndex write SetShowIndex;
  property ShowLabel: boolean read FShowLabel write SetShowLabel;
  property ShowSize: boolean read FShowSize write SetShowSize;
  property ZeroBased: boolean read FZeroBased write SetZeroBased;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
  property SelectCount: integer read GetSelectCount;
  property Align;
  property Anchors;
  property BevelEdges;
  property BevelInner;
  property BevelKind default bkNone;
  property BevelOuter;
  property BiDiMode;
  property BorderStyle;
  property Caption;
  property Color;
  property Constraints;
  property Ctl3D;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property ExtendedSelect;
  property Font;
  property ImeMode;
  property ImeName;
  property IntegralHeight;
  property MultiSelect;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ScrollWidth;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnClick;
  property OnContextPopup;
  property OnData;
  property OnDataFind;
  property OnDataObject;        
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnDrawItem;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnKeyDown;
  property OnKeyPress;
  property OnKeyUp;
  property OnMeasureItem;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnStartDock;
  property OnStartDrag;
end;



type TThumbnailList = class(TBitmapList)
 private
  FHeight: integer;
  FWidth: integer;
  FKeepAspectRatio: boolean;
  procedure HandleAspectRatio(BM: TBitmap; var NewX, NewY: integer);
 protected
 public
  property Width: integer read FWidth write FWidth;
  property Height: integer read FHeight write FHeight;
  property KeepAspectRatio: boolean read FKeepAspectRatio write FKeepAspectRatio;
  function AddBitmap(BM: TBitmap): integer; override;
  constructor Create;
 end;


type TDrawImageEvent = procedure (Sender: TObject; Index, ACol, ARow: Longint; CellRect,ImageRect: TRect; State: TGridDrawState) of object;
type TGetImageEvent = procedure (Sender: TObject; Index, ACol, ARow: Longint; var DoDraw: boolean; var BM: TBitmap; var Caption: string) of object;


type TImageGrid = class(TDrawGrid)
 private
  FBorderWidth: integer;
  FImageWidth: integer;
  FImageHeight: integer;
  FOnDrawImage: TDrawImageEvent;
  FOnGetImage: TGetImageEvent;
  FImageCount: integer;
  FAutoColumn: boolean;
  FKeepAspectRatio: boolean;
  FShowCaption: boolean;
  FCaptionAlignment: TAlignment;
  procedure SetImageWidth(const Value: integer);
  procedure SetImageHeight(const Value: integer);
  procedure SetBorderWidth(const Value: integer);
  procedure SetupGrid;
  procedure SetImageCount(const Value: integer);
  procedure CalculateRowCount;
  procedure SetColCount(const Value: Longint);
  function GetColumnCount: Longint;
  procedure HandleResize;
  procedure SetAutoColumn(const Value: boolean);
  procedure HandleAspectRatio(BM: TBitmap; Origin: TPoint; var NewRect: TRect);
  procedure DrawImage(AState: TGridDrawState; CellRect,ImageRect: TRect; BM: TBitmap; Caption: string);
  procedure SetShowCaption(const Value: boolean);
    procedure SetCaptionAlignment(const Value: TAlignment);
    procedure HighlightImage(Rect: TRect);
 protected
  procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
  procedure Resize; override;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property AutoColumn: boolean read FAutoColumn write SetAutoColumn default False;
  property BorderWidth: integer read FBorderWidth write SetBorderWidth default 5;
  property CaptionAlignment: TAlignment read FCaptionAlignment write SetCaptionAlignment default taLeftJustify;
  property ColCount: Longint read GetColumnCount write SetColCount default 5;
  property KeepAspectRatio: boolean read FKeepAspectRatio write FKeepAspectRatio default False;
  property ImageCount: integer read FImageCount write SetImageCount;
  property ImageHeight: integer read FImageHeight write SetImageHeight default 90;
  property ImageWidth: integer read FImageWidth write SetImageWidth default 120;
  property OnDrawImage: TDrawImageEvent read FOnDrawImage write FOnDrawImage;
  property OnGetImage: TGetImageEvent read FOnGetImage write FOnGetImage;
  property ShowCaption: boolean read FShowCaption write SetShowCaption default False;
end;



procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TImageListBox]);
RegisterComponents('Fountain', [TImageGrid]);
end;



constructor TImageListBox.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FSizeArray:=THugePoint.Create(5);
FAuxInfo:=THugeInteger.Create(5);
FAutoScroller:=TAutoScroller.Create;
Style:=lbOwnerDrawVariable;
FImageHeight:=100;
FFixedSize:=True;
FShowIndex:=True;
FShowSize:=True;
FShowLabel:=False;

FHighlightColor:=clNavy;
Modified:=False;
end;



destructor TImageListBox.Destroy;
begin
{don't do any operation here that require access}
{to the list. Its window handle is gone}
{Do everything in WMDestroy}
DoClear(False);
FAutoScroller.Free;
FAuxInfo.Free;
FSizeArray.Free;
inherited Destroy;
end;


{-----------------------------------------------------------------------------}


procedure TImageListBox.SetFixedSize(V: boolean);
begin
if FFixedSize<>V then
	begin
	FFixedSize:=V;
	Self.Invalidate;
	ForceRemeasure;
	end;
end;




procedure TImageListBox.SetZeroBased(V: boolean);
begin
if FZeroBased<>V then
	begin
	FZeroBased:=V;
	ForceRemeasure;
	end;
end;


procedure TImageListBox.SetShowIndex(V: boolean);
begin
if FShowIndex<>V then
	begin
	FShowIndex:=V;
	ForceRemeasure;
	end;
end;





procedure TImageListBox.SetShowLabel(const Value: boolean);
begin
if FShowLabel<>Value then
	begin
	FShowLabel := Value;
	ForceRemeasure;
	end;
end;




procedure TImageListBox.SetShowSize(V: boolean);
begin
if FShowSize<>V then
	begin
	FShowSize:=V;
	ForceRemeasure;
	end;
end;


{-----------------------------------------------------------------------------}



procedure TImageListBox.HandleBitmapChange(Sender: TObject);
begin
Changed;
end;



function TImageListBox.CreateBitmap: TBitmap;
begin
Result:=TBitmap.Create;
Result.OnChange:=HandleBitmapChange;
end;



function TImageListBox.AddBitmap(BM: TBitmap; S: string): integer;
var TBM: TBitmap;
begin
FSizeArray.Add(Point(BM.Width,BM.Height));
FAuxInfo.Add(0);
TBM:=CreateBitmap;
TBM.Assign(BM);
ForceBitmapHandle(TBM);
Result:=Items.AddObject(S,TBM);
ItemIndex:=Result;
TopIndex:=Result;
Changed;
end;


function TImageListBox.AddBitmap(BM: TBitmap): integer;
begin
Result:=AddBitmap(BM,'');
end;



function TImageListBox.GetBitmap(Inx: integer): TBitmap;
begin
Result:=nil;
if Inx>=Count then exit;
Result:=TBitmap(Items.Objects[Inx]);
end;



procedure TImageListBox.PutBitmap(Inx: integer; Item: TBitmap);
begin
if Inx>=Count then exit;
TBitmap(Items.Objects[Inx]).Assign(Item);
ForceBitmapHandle(TBitmap(Items.Objects[Inx]));
FSizeArray[Inx]:=Point(TBitmap(Item).Width,TBitmap(Item).Height);
Changed;
end;


procedure TImageListBox.ReAnalyzeSize;
var I: Integer;
begin
for I:=0 to Items.Count-1 do
	begin
	FSizeArray[I]:=Point(TBitmap(Items.Objects[I]).Width,TBitmap(Items.Objects[I]).Height);
	end;
end;


procedure TImageListBox.DoClear(TriggerChange: boolean);
var I: integer;
begin
if Parent=nil then exit;
for I:=0 to Count-1 do
	begin
	TBitmap(Items.Objects[I]).Free;
	end;
FSizeArray.Clear;
FAuxInfo.Clear;
inherited Clear;
Modified:=False;
if TriggerChange then Changed;
end;



procedure TImageListBox.Clear;
begin
DoClear(True);
end;


procedure TImageListBox.Delete(Index: Integer);
begin
TBitmap(Items.Objects[Index]).Free;
FSizeArray.Delete(Index);
FAuxInfo.Delete(Index);
inherited Items.Delete(Index);
ForceRemeasure;
Changed;
end;




procedure TImageListBox.DeleteSelected;
var I: integer;
begin
for I:=Self.Count-1 downto 0 do
 if Selected[I] then
	begin
	Delete(I);
	end;
end;



procedure TImageListBox.Exchange(Index1, Index2: integer);
begin
Items.Exchange(Index1,Index2);
FSizeArray.Exchange(Index1,Index2);
FAuxInfo.Exchange(Index1,Index2);
ForceRemeasure;
Changed;
end;



procedure TImageListBox.InsertBitmap(Index: integer; BM: TBitmap; S: string);
var TBM: TBitmap;
begin
TBM:=TBitmap.Create;
TBM.Assign(BM);
ForceBitmapHandle(TBM);
Items.InsertObject(Index,S,TBM);
FSizeArray.Insert(Point(BM.Width,BM.Height),Index);
FAuxInfo.Insert(0,Index);
ForceRemeasure;
ItemIndex:=Index;
Changed;
end;



procedure TImageListBox.InsertBitmap(Index: integer; BM: TBitmap);
begin
InsertBitmap(Index,BM,'');
end;



procedure TImageListBox.Move(CurIndex, NewIndex: Integer);
begin
Items.Move(CurIndex,NewIndex);
ItemIndex:=NewIndex;
FSizeArray.Move(CurIndex,NewIndex);
FAuxInfo.Move(CurIndex,NewIndex);
ForceRemeasure;
Changed;
end;



procedure TImageListBox.Load(Filename: string; Progress: TBitmapProgress);
var InFilm: TInFilm;
var I: integer;
var BM: TBitmap;
begin
InFilm:=TInFilm.Create;
try
BM:=TBitmap.Create;
try
InFilm.OpenFile(FileName);
InFilm.Rewind;
Self.Clear;
for I:=0 to InFilm.Count-1 do
	begin
	InFilm.ReadBitmap(BM);
	Self.AddBitmap(BM);
	if Assigned(Progress) then Progress(MulDiv(100,I,InFilm.Count));
	Application.ProcessMessages;
	end;
ItemIndex:=0;
Self.TopIndex:=0;
Modified:=False;
finally BM.Free; end;
finally InFilm.Free; end;
end;


function TImageListBox.GetFileSize: int64;
{Get files size of all bitmaps in list}
var I: integer;
begin
Result:=SizeOf(integer);
for I:=0 to Self.Count-1 do
	Result:=Result+GetBitmapSize(TXBitmap(Self.Bitmaps[I]));
end;




function TImageListBox.GetSelectedSize: int64;
{Get files size of all selected bitmaps in list}
var I: integer;
begin
Result:=SizeOf(integer);
for I:=0 to Self.Count-1 do
 if Selected[I] then
	Result:=Result+GetBitmapSize(TXBitmap(Self.Bitmaps[I]));
end;



procedure TImageListBox.SaveToStream(Stream: TStream);
var I,Cnt: integer;
begin
Stream.Position:=0;
Cnt:=Self.Count;
Stream.WriteBuffer(Cnt,Sizeof(Cnt));
for I:=0 to Self.Count-1 do
	begin
	TXBitmap(Self.Bitmaps[I]).WriteData(Stream);
	end;
end;


function TImageListBox.GetSelectCount: integer;
var I: integer;
begin
Result:=0;
for I:=0 to Self.Count-1 do
 if Selected[I] then Inc(Result);
end;


procedure TImageListBox.SaveSelectedToStream(Stream: TStream);
var I,Cnt: integer;
begin
Stream.Position:=0;
Cnt:=GetSelectCount;
Stream.WriteBuffer(Cnt,Sizeof(Cnt));
for I:=0 to Self.Count-1 do
 if Selected[I] then
	begin
	TXBitmap(Self.Bitmaps[I]).WriteData(Stream);
	end;
end;




procedure TImageListBox.LoadFromStream(Stream: TStream);
var I, Cnt: integer;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
Stream.Position:=0;
Stream.Read(Cnt,Sizeof(Cnt));
Self.BeginUpdate;
try
for I:=0 to Cnt-1 do
	begin
	TXBitmap(BM).ReadData(Stream);
	Self.AddBitmap(BM);
	end;
finally Self.EndUpdate; end;
finally BM.Free; end;
end;



procedure TImageListBox.Save(Filename: string; Progress: TBitmapProgress);
var OutFilm: TOutFilm;
var I: integer;
begin
OutFilm:=TOutFilm.Create;
try
OutFilm.OpenFile(Filename);
for I:=0 to Self.Count-1 do
	begin
	OutFilm.WriteBitmap(Self.Bitmaps[I]);
	if Assigned(Progress) then Progress(MulDiv(100,I,Self.Count));
	Application.ProcessMessages;
	end;
 finally
 Modified:=False;
 OutFilm.CloseFile;
 OutFilm.Free;
 end;
end;



procedure TImageListBox.PasteFromClipboard;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.LoadFromClipboardFormat(cf_BitMap,ClipBoard.GetAsHandle(cf_Bitmap),0);
AddBitmap(BM);
finally BM.Free; end;
end;



procedure TImageListBox.CopyToClipboard(Inx: integer);
var MyFormat: Word;
var AData: THandle;
var APalette: HPALETTE;
begin
if Inx>=Count then raise Exception.Create('Index out of range');
Self.Bitmaps[Inx].SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
end;



function TImageListBox.GetImageHeight(Index: integer; Fixed: boolean): integer;
var H,W: integer;
begin
FTextHeight:=abs(Font.Height)+4;
if Fixed then Result:=FImageHeight + FTextHeight
else
	begin
	W:=FSizeArray[Index].X;
	H:=FSizeArray[Index].Y;
	Result:=MulDiv(Self.ClientWidth,H,W)+FTextHeight;
	end;
end;


procedure TImageListBox.ForceRemeasure;
var I: integer;
var H: integer;
begin
for I:=0 to Self.Count-1 do
	begin
	H:=GetImageHeight(I,FFixedSize);
	SendMessage(Self.Handle, LB_SETITEMHEIGHT, I , H);
	end;
Self.Invalidate;
end;



procedure TImageListBox.MeasureItem(Index: Integer; var Height: Integer);
begin
Height:=GetImageHeight(Index,FFixedSize);
inherited;
end;




procedure TImageListBox.HighlightImage(Canvas: TCanvas; var Rect: TRect);
begin
Canvas.Pen.Width:=1;
Canvas.Brush.Color:=FHighlightColor;
InflateRect(Rect,-1,-1);
Canvas.Pen.Color:=FHighlightColor;
Canvas.Rectangle(Rect);
Canvas.Pen.Color:=FHighlightColor;
InflateRect(Rect,-1,-1);
Canvas.Rectangle(Rect);
Canvas.Pen.Color:=clBlack;
InflateRect(Rect,-1,-1);
Canvas.Rectangle(Rect);
end;



procedure TImageListBox.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var BM: TBitmap;
var ImageRect: TRect;
var S: string;
begin
inherited;
Canvas.Brush.Style:=bsClear;
Canvas.Pen.Mode:=pmCopy;
if odSelected in State then HighlightImage(Canvas,Rect);
ImageRect:=Rect;
if FShowIndex or FShowSize or FShowLabel then ImageRect.Bottom:=ImageRect.Bottom - FTextHeight;

BM:=Self.GetBitmap(Index);
Canvas.StretchDraw(ImageRect,BM);
Canvas.Pen.Color:=clBlack;
Canvas.Pen.Mode:=pmCopy;
Canvas.Brush.Style:=bsClear;
Canvas.Rectangle(ImageRect);
S:='';
if FShowIndex then
	begin
	if FZeroBased then S:=S+IntToStr(Index)+' - '
	else S:=S+IntToStr(Index+1)+' - ';
	end;
if FShowSize then S:=S+IntToStr(FSizeArray[Index].X)+'  X '+IntToStr(FSizeArray[Index].Y)+' - ';
if FShowLabel then S:=S+Items[Index];

Canvas.TextOut(ImageRect.Left,ImageRect.Bottom,S);
end;




procedure TImageListBox.Resize;
begin
inherited;
ForceRemeasure;
end;



{------------------ Drag and drop routines ------------------------------------}

procedure TImageListBox.DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
var I: integer;
begin
inherited;
I:=Self.ItemAtPos(Point(X,Y),True);
Accept:=(I>=0) and (ItemIndex<>I);
FAutoScroller.HandleScroll(Self);
end;



procedure TImageListBox.DragDrop(Source: TObject; X, Y: Integer);
var I,T: integer;
begin
inherited;
T:=Self.TopIndex;
I:=Self.ItemAtPos(Point(X,Y),True);
if I<0 then exit;
Self.Move(Self.ItemIndex,I);
Self.TopIndex:=T;
Changed;
end;



procedure TImageListBox.Changed;
begin
if Assigned(FOnChange) then FOnChange(Self);
Modified:=True;
end;



procedure TImageListBox.Assign(Source: TPersistent);
var I: Integer;
begin
if Source is TImageListBox then
	begin
	Clear;
	for I := 0 to TImageListBox(Source).Count - 1 do
		AddBitmap(TImageListBox(Source).Bitmaps[I]);
	exit;
	end;
inherited Assign(Source);
end;




procedure TImageListBox.SetHighlightColor(V: TColor);
begin
if V<>FHighlightColor then
	begin
	FHighlightColor:=V;
	Invalidate;
	end;
end;



function TImageListBox.GetLabel(Index: integer): string;
begin
Result:=Items[Index];
end;



procedure TImageListBox.PutLabel(Inx: integer; const Value: string);
begin
Items[Inx]:=Value;
end;



function TImageListBox.GetAuxInfo(Index: integer): integer;
begin
Result:=FAuxInfo[Index];
end;



procedure TImageListBox.PutAuxInfo(Index: integer; const Value: integer);
begin
FAuxInfo[Index]:=Value;
end;



procedure TImageListBox.BeginUpdate;
begin
Self.Items.BeginUpdate;
end;



procedure TImageListBox.EndUpdate;
begin
Self.Items.EndUpdate;
end;


{ TThumbnailList }


procedure TThumbnailList.HandleAspectRatio(BM: TBitmap; var NewX,NewY: integer);
var BMAspect,ThumbAspect: double;
begin
BMAspect:=BM.Width/BM.Height;
ThumbAspect:=Width/Height;
if BMAspect<=ThumbAspect then
	begin
	NewY:=Height;
	NewX:=Round(NewY*BMAspect);
	end
else
	begin
	NewX:=Width;
	NewY:=Round(NewX/BMAspect);
	end;
end;



function TThumbnailList.AddBitmap(BM: TBitmap): integer;
var TBM: TBitmap;
var X,Y: integer;
var H: HBITMAP;
begin
TBM:=TBitmap.Create;
TBM.Assign(BM);
ForceBitmapHandle(TBM);
if Self.KeepAspectRatio then
	begin
	HandleAspectRatio(BM,X,Y);
	TBM.Width:=X;
	TBM.Height:=Y;
	end
else
	begin
	TBM.Width:=Width;
	TBM.Height:=Height;
	end;
DoWinResize(TBM.Canvas,Rect(0,0,Width,Height),BM,HALFTONE);

Result:=Add(TBM);
FAuxInfo.Add(0);
end;



constructor TThumbnailList.Create;
begin
inherited Create;
KeepAspectRatio:=True;
Width:=150;
Height:=150;
end;


{ TImageGrid }



constructor TImageGrid.Create(AOwner: TComponent);
begin
inherited;
FixedCols:=0;
FixedRows:=0;

FBorderWidth:=5;
ImageWidth:=120;
ImageHeight:=90;
FKeepAspectRatio:=False;
FShowCaption:=False;
FCaptionAlignment:=taLeftJustify;
end;



destructor TImageGrid.Destroy;
begin
inherited;
end;



procedure TImageGrid.SetupGrid;
var CaptionHeight: integer;
begin
DefaultColWidth:=BorderWidth * 2 + ImageWidth;
if ShowCaption then CaptionHeight:=Canvas.TextHeight('X')+2 else CaptionHeight:=0;
DefaultRowHeight:=BorderWidth * 2 + ImageHeight + CaptionHeight;
end;




procedure TImageGrid.SetBorderWidth(const Value: integer);
begin
if FBorderWidth<Value then
	begin
	FBorderWidth := Value;
	SetupGrid;
	end;
end;



procedure TImageGrid.SetImageHeight(const Value: integer);
begin
if FImageHeight<>Value then
	begin
	FImageHeight := Value;
	SetupGrid;
	end;
end;



procedure TImageGrid.SetImageWidth(const Value: integer);
begin
if FImageWidth<>Value then
	begin
	FImageWidth := Value;
	SetupGrid;
	end;
end;


{=============================================================}



procedure TImageGrid.HandleAspectRatio(BM: TBitmap; Origin: TPoint; var NewRect: TRect);
var BMAspect,ThumbAspect: double;
var NewX,NewY,OffX,OffY: integer;
begin
BMAspect:=BM.Width/BM.Height;
ThumbAspect:=ImageWidth/ImageHeight;
if BMAspect<=ThumbAspect then
	begin
	NewY:=ImageHeight;
	NewX:=Round(NewY*BMAspect);
	OffY:=0;
	OffX:=(ImageWidth-NewX) div 2;
	end
else
	begin
	NewX:=ImageWidth;
	NewY:=Round(NewX/BMAspect);
	OffX:=0;
	OffY:=(ImageHeight-NewY) div 2;
	end;
NewRect.TopLeft:=Origin;
NewRect.Right:=Origin.X + NewX;
NewRect.Bottom:=Origin.Y + NewY;
OffsetRect(NewRect,OffX,OffY);
end;






procedure TImageGrid.HighlightImage(Rect: TRect);
begin
Canvas.Pen.Width:=1;
Canvas.Brush.Color:=clHighlight;
InflateRect(Rect,-1,-1);
Canvas.Pen.Color:=clHighlight;
Canvas.Rectangle(Rect);
end;




procedure TImageGrid.DrawImage(AState: TGridDrawState; CellRect,ImageRect: TRect; BM: TBitmap; Caption: string);
var R: TRect;
var Left: integer;


	function GetTextOffset: integer;
	var BoxWidth,Gap: integer;
	const Offset = 2;
	begin
	BoxWidth:=R.Right-R.Left;
	Gap:=BoxWidth - Canvas.TextWidth(Caption);
	case CaptionAlignment of
	 taLeftJustify: Result:=Offset+R.Left;
	 taRightJustify: Result:=(Offset+R.Left+Gap)-Offset;
	 taCenter: Result:=Offset+R.Left+(Gap div 2);
	 end;
	end;




function GetState: string;
{Debug tool}
begin
Result:='';
if AState=[] then Result:=Result+'None';
if gdSelected in AState then Result:=Result+'Selected,';
if gdFocused in AState then Result:=Result+'Focused,';
if gdFixed in AState then Result:=Result+'Fixed,';
end;




begin
if Self.KeepAspectRatio then HandleAspectRatio(BM,ImageRect.TopLeft,ImageRect);
if gdSelected in AState then HighlightImage(CellRect);
OversampleBitmap(Canvas,ImageRect,BM);
Canvas.Brush.Style:=bsClear;
Canvas.Pen.Color:=clBlack;
Canvas.Rectangle(ImageRect);
Canvas.Brush.Style:=bsSolid;
if ShowCaption then
	begin
	R.Left:=CellRect.Left+2;
	R.Right:=CellRect.Right-2;
	R.Top:=ImageRect.Bottom+1;
	R.Bottom:=CellRect.Bottom-1;
	Left:=GetTextOffset;
	Canvas.Pen.Color:=clBlack;
	Canvas.Pen.Mode:=pmCopy;
	Canvas.Brush.Style:=bsClear;
	if gdSelected in AState then Canvas.Font.Color:=clHighlightText else Canvas.Font.Color:=Font.Color;
	Canvas.TextRect(R,Left,R.Top,Caption);
	end;
end;




procedure TImageGrid.DrawCell(ACol, ARow: Integer; ARect: TRect; AState: TGridDrawState);
var Index: integer;
var ImageRect: TRect;
var BM: TBitmap;
var DoDraw: boolean;
var Caption: string;
begin
inherited ;
BM:=TBitmap.Create;
try
Index:=ARow * ColCount + ACol;
ImageRect:=ARect;
InflateRect(ImageRect,-BorderWidth,-BorderWidth);
if ShowCaption then ImageRect.Bottom:=ImageRect.Bottom - (Canvas.TextHeight('X')+2);
if Assigned(FOnDrawImage) then
	begin
	FOnDrawImage(Self,Index, ACol, ARow, ARect,ImageRect,AState);
	end;


if Assigned(FOnGetImage) then
	begin
	FOnGetImage(Self,Index, ACol, ARow, DoDraw, BM, Caption);
	if DoDraw and (BM<>nil) then DrawImage(AState,ARect,ImageRect,BM,Caption);
	end;
finally BM.Free; end;
end;





procedure TImageGrid.CalculateRowCount;
begin
RowCount:=(ImageCount div ColCount)+1;
end;



procedure TImageGrid.SetImageCount(const Value: integer);
begin
if FImageCount<>Value then
	begin
	FImageCount := Value;
	CalculateRowCount;
	end;
end;


procedure TImageGrid.SetColCount(const Value: Longint);
begin
Inherited ColCount:=Value;
CalculateRowCount;
end;




function TImageGrid.GetColumnCount: Longint;
begin
Result:=Inherited ColCount;
end;



procedure TImageGrid.HandleResize;
var T: integer;
begin
T:=(Width div DefaultColWidth);
if T<1 then T:=1;
ColCount:=T;
end;



procedure TImageGrid.Resize;
begin
inherited;
if AutoColumn then HandleResize;
end;



procedure TImageGrid.SetAutoColumn(const Value: boolean);
begin
if FAutoColumn<>Value then
	begin
	FAutoColumn := Value;
	if FAutoColumn then HandleResize
	end;
end;


procedure TImageGrid.SetShowCaption(const Value: boolean);
begin
if FShowCaption<>Value then
	begin
	FShowCaption := Value;
	SetupGrid;
	end;
end;



procedure TImageGrid.SetCaptionAlignment(const Value: TAlignment);
begin
if FCaptionAlignment<>Value then
	begin
	FCaptionAlignment := Value;
	Invalidate;
	end;
end;

end.
