unit Posgrid;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TPositionGrid = class(TPaintBox)
private
  FXPos,FYPos: integer;
  FOnChangeSelect: TNotifyEvent;
  procedure SetXPos(New: integer);
  procedure SetYPos(New: integer);
protected
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure Paint; override;
  procedure ChangeSelect; dynamic;
public
  constructor Create(AOwner: TComponent); override;
published
  property OnChangeSelect: TNotifyEvent read FOnChangeSelect write FOnChangeSelect;
  property XPos: integer read FXPos write SetXPos default 1;
  property YPos: integer read FYPos write SetYPos default 1;
  end;

procedure Register;

implementation


procedure TPositionGrid.Paint;
var DX,DY,X,Y,P,iXP,iYP : Integer;
const GridSize=7;
begin
inherited Paint;
iXP:=FXPos; iYP:=FYPos;
with Canvas do
 begin
 Pen.Color:=clBlack;
 Pen.Style:=psSolid;
 Brush.Color:=clLtGray;
 Rectangle(ClientRect.left,ClientRect.top,ClientRect.right-ClientRect.left,ClientRect.bottom-ClientRect.top);

 DX := ClientRect.right - ClientRect.left;
 DY := ClientRect.bottom - ClientRect.top;
 Pen.Color:=clWhite;
 for P:=0 to GridSize-1 do
	begin 
	X:=MulDiv(P,DX,GridSize);
	Y:=MulDiv(P,DY,GridSize);
	MoveTo(X,ClientRect.top);
	LineTo(X,ClientRect.bottom);
	MoveTo(ClientRect.left,Y);
	LineTo(ClientRect.right,Y);
	end;

 Pen.Color:=clBlack;
 for P:=1 to GridSize-1 do
	begin
	X:=MulDiv(P,DX,GridSize)-1;
	Y:=MulDiv(P,DY,GridSize)-1;
	MoveTo(X,ClientRect.top);
	LineTo(X,ClientRect.bottom);
	MoveTo(ClientRect.left,Y);
	LineTo(ClientRect.right,Y);
	end;
 Pen.Color:=clBlack;
 Brush.Style:=bsClear;
 Rectangle(MulDiv(GridSize div 2,DX,GridSize),MulDiv(GridSize div 2,DY,GridSize),
 MulDiv(GridSize div 2+1,DX,GridSize)-1,MulDiv(GridSize div 2+1,DY,GridSize)-1);
 iXP:=iXP + (GridSize div 2); iYP:=iYP + (GridSize div 2);
 Pen.Color:=clRed;
 Brush.Color:=clRed;
 Rectangle(MulDiv(iXP,DX,GridSize)+1,MulDiv(iYP,DY,GridSize)+1,
 MulDiv(iXP+1,DX,GridSize)-2,MulDiv(iYP+1,DY,GridSize)-2);
 end;
end;

procedure TPositionGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var DX,DY,TX,TY: integer;
const GridSize=7;
begin
inherited MouseDown(Button, Shift, X, Y);
if Button=mbLeft then
 with Canvas do
	begin
	DX := ClientRect.right - ClientRect.left;
	DY := ClientRect.bottom - ClientRect.top;
	TX:=((GridSize*X) div DX)-(GridSize Div 2);
	TY:=((GridSize*Y) div DY)-(GridSize Div 2);
	if (TX<>FXPos) or (TY<>FYPos) then
		begin
		FXPos:=TX; FYPos:=TY;
		Invalidate;
		ChangeSelect;
		end;
	end;
end;

procedure TPositionGrid.ChangeSelect;
begin
if Assigned(FOnChangeSelect) then FOnChangeSelect(Self);
end;

constructor TPositionGrid.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FXPos:=0;
FYPos:=0;
end;


procedure TPositionGrid.SetXPos(New: integer);
begin
FXPos:=New;
Invalidate;
end;

procedure TPositionGrid.SetYPos(New: integer);
begin
FYPos:=New;
Invalidate;
end;

procedure Register;
begin
 RegisterComponents('Fountain', [TPositionGrid]);
end;

end.
