unit PrecisionClock;

{This object is a precision system clock that gives
millisecond level resolution by synchtronizing a
high performance timer with the system clock}

interface

uses Windows,Messages,Classes,SysUtils,mmSystem,DSupport;


type TPrecisionSystemClock = class(TComponent)
 private
  FBase: integer;
  FWinHandle: HWND;
 protected
  procedure WndProc(var msg: TMessage);
 public
  Status: string;
  procedure Calibrate;
  function GetTime: TSystemTime;
  function GetDateTime: TDateTime;
  function Milliseconds: integer;
  function Seconds: double;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property TimeBase: integer read FBase;
 end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain',[TPrecisionSystemClock]);
end;



function GetPrecisionMS: cardinal;
begin
Result:=GetHiResTick64 div 1000;
end;



procedure TPrecisionSystemClock.Calibrate;
{Synchronize the Multimedia timer to the System Clock}
var ST1,ST2: TSystemTime;
var I: integer;
var MS: DWord;
begin
GetSystemTime(ST1);
for I:=1 to 100000 do
	begin
	GetSystemTime(ST2);
	MS:=GetPrecisionMS;
	if ST2.wMilliseconds<>ST1.wMilliseconds then
		begin
		FBase:=(MS mod 1000) - ST2.wMilliseconds;
		Status:='Calibrated';
		exit;
		end;
	Sleep(1);
	end;
Status:='Unable To Calibrate';
FBase:=0;
end;



procedure TPrecisionSystemClock.WndProc(var msg: TMessage);
{Capture time-change events and recalibrate}
begin
if Msg.Msg = WM_TIMECHANGE then Calibrate
else Msg.Result:= DefWindowProc(FWinHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;



constructor TPrecisionSystemClock.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FWinHandle:=AllocateHWND(WndProc);
timeBeginPeriod(1);
Status:='Uncalibrated';
Calibrate;
end;



destructor TPrecisionSystemClock.Destroy;
begin
timeEndPeriod(1);
DeallocateHWnd(FWinHandle);
inherited;
end;




function TPrecisionSystemClock.GetTime: TSystemTime;
{Get adjusted system with ms that increment properly}
var MS,Delta: integer;
var DT: TDateTime;
var C: cardinal;
const Sec = 1/(24 * 60 * 60);
begin
GetSystemTime(Result);
{Get current adjusted millisecond value}
C:=GetPrecisionMS;
MS:=(C-FBase) mod 1000;
{Handle situation where the two clocks don't cross at the same time}
Delta:=Result.wMilliseconds - MS;
if Delta<-500 then
	begin
        {System clock crossed 0 first so adjust by subtracting a second}
	DT:=SystemTimeToDateTime(Result)-Sec;
	DateTimeToSystemTime(DT,Result);
	end
else if Delta>500 then
	begin
        {Precision clock crossed 0 first so adjust by adding a second}
	DT:=SystemTimeToDateTime(Result)+Sec;
	DateTimeToSystemTime(DT,Result);
	end;
Result.wMilliseconds:=MS;
end;



function TPrecisionSystemClock.GetDateTime: TDateTime;
{Get precision time as Delphi TDateTime format}
begin
Result:=SystemTimeToDateTime(GetTime);
end;



function TPrecisionSystemClock.Seconds: double;
{Returns number of seconds since midnight}
var ST: TSystemTime;
begin
ST:=GetTime;
Result:=(ST.wHour * 3600) + (ST.wMinute * 60) + ST.wSecond + (ST.wMilliseconds/1000);
end;



function TPrecisionSystemClock.Milliseconds: integer;
{Returns number of Milliseconds since midnight}
begin
Result:=round(Seconds * 1000);
end;



end.
