unit Progmon;

interface

uses WinTypes,WinProcs,ExtCtrls,Classes,SysUtils,Forms;

type TMonitorType =(mtWait,mtMonitor);

type TMonitorEvent = procedure(Sender: TObject; MonitorType: TMonitorType) of object;

type
 TProgramMonitor=class(TComponent)
 protected
  FTimer: TTimer;
  ProcessInfo: TProcessInformation;
  FOnFinish: TMonitorEvent;
  FMonitorType: TMonitorType;
  procedure HandleFinish;
  procedure MonitorProgram(Sender: TObject);
 private
 public
  Running: boolean;
  ReturnCode: DWord;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function RunProgram(Path: String): boolean;
  function RunProgramAndWait(Path: string): boolean;
 published
  property OnFinish: TMonitorEvent read FOnFinish write FOnFinish;
 end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Fountain', [TProgramMonitor]);
end;



constructor TProgramMonitor.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FTimer:=TTimer.Create(Self);
FTimer.OnTimer:=MonitorProgram;
FTimer.Interval:=250;
FTimer.Enabled:=False;
Running:=False;
FMonitorType:=mtWait;
end;


destructor TProgramMonitor.Destroy;
begin
FTimer.Free;
inherited Destroy;
end;



procedure TProgramMonitor.HandleFinish;
begin
GetExitCodeProcess(ProcessInfo.hProcess,ReturnCode);
CloseHandle(ProcessInfo.hThread);
CloseHandle(ProcessInfo.hProcess);
if Assigned(FOnFinish) then FOnFinish(Self,FMonitorType);
end;




procedure TProgramMonitor.MonitorProgram(Sender: TObject);
{Monitor running program so the main program can do things when the }
{external program finishes. For example, bring itself to the front}
var V: DWord;
begin
V:=WaitForSingleObject(ProcessInfo.hProcess, 0);
if WAIT_TIMEOUT<>V then
	begin
	FTimer.Enabled:=False;
	Running:=False;
	HandleFinish;
	end;
end;



function TProgramMonitor.RunProgram(Path: String): boolean;
{Execute the specified program}
var Start: TStartupInfo;
begin
FillChar(Start,Sizeof(Start),0);
start.cb:=Sizeof(start);
FMonitorType:=mtMonitor;
Result:=CreateProcess(Nil,PChar(Path),Nil,Nil,True,NORMAL_PRIORITY_CLASS,Nil,Nil, Start, ProcessInfo);
FTimer.Enabled:=Result;
Running:=Result;
end;






function TProgramMonitor.RunProgramAndWait(Path: string): boolean;
{Execute the specified program and wait}
var Start: TStartupInfo;
begin
FillChar(Start,Sizeof(Start),0);
start.cb:=Sizeof(start);
FMonitorType:=mtWait;
Result:=CreateProcess(Nil,PChar(Path),Nil,Nil,True,NORMAL_PRIORITY_CLASS,Nil,Nil, Start, ProcessInfo);
Running:=Result;
if Result then
	begin
	try
	repeat Application.ProcessMessages
	until WaitForSingleObject(ProcessInfo.hProcess, 100) <> WAIT_TIMEOUT;
	finally
	 HandleFinish;
	 end;
	end;
end;



end.
