unit Quadspin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, SpinButtons;
type
  TSpinChangeEvent = procedure(Sender: TObject; OldCol,OldRow: integer) of object;

type
  TQuadSpin = class(TPanel)
private
  FBlockChangeEvent: boolean;
  FLeftSpin,FRightSpin: TUpDownButton;
  FColFirst,FColLetter,FRowLetter: boolean;
  FColValue,FRowValue: integer;
  FOnChanged: TSpinChangeEvent;
  procedure MakeSpinButton(var B: TUpDownButton;A:TAlign);
  procedure ShowQuad;

  procedure SetColFirst(New: boolean);
  procedure SetColLetter(New: boolean);
  procedure SetRowLetter(New: boolean);
  procedure SetColValue(New: integer);
  procedure SetRowValue(New: integer);
protected
  procedure Changed(OCol,ORow: integer); dynamic;
  procedure UpClick (Sender: TObject); virtual;
  procedure DownClick (Sender: TObject); virtual;
public
  function GetGridLabel(Col,Row: integer): string;
  function NumberToLetter(N: integer): string;
  constructor Create(AOwner: TComponent); override;
published
  property BlockChangeEvent: boolean read FBlockChangeEvent write FBlockChangeEvent default False;
  property OnChanged: TSpinChangeEvent read FOnChanged write FOnChanged;
  property ColFirst: boolean read FColFirst write SetColFirst default True;
  property ColLetter: boolean read FColLetter write SetColLetter default True;
  property RowLetter: boolean read FRowLetter write SetRowLetter default False;
  property ColValue: integer read FColValue write SetColValue default 0;
  property RowValue: integer read FRowValue write SetRowValue default 0;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Fountain', [TQuadSpin]);
end;


function TQuadSpin.NumberToLetter(N: integer): string;
var R: integer;
var Neg: boolean;
begin
Neg:=N<0;
N:=abs(N);
Result:='';
repeat
	begin
	R:=N mod 26;
	N:=(N div 26)-1;
	if Neg then Result:=char(R+byte('a'))+Result
	else Result:=char(R+byte('A'))+Result;
        end
until N<0;
end;



function TQuadSpin.GetGridLabel(Col,Row:integer): string;
var RowStr,ColStr,Sep: string;
begin
Sep:='';
if FColLetter=FRowLetter then Sep:=',';
if FColLetter then ColStr:=NumberToLetter(Col) else ColStr:=IntToStr(Col);
if FRowLetter then RowStr:=NumberToLetter(Row) else RowStr:=IntToStr(Row);
if FColFirst then Result:=ColStr+Sep+RowStr else Result:=RowStr+Sep+ColStr;
end;


procedure TQuadSpin.ShowQuad;
begin
Caption:=GetGridLabel(FColValue,FRowValue);
end;


procedure TQuadSpin.MakeSpinButton(var B: TUpDownButton;A:TAlign);
begin
B:=TUpDownButton.Create(Self);
B.Visible := True;
B.Parent := Self;
B.Width := 15;
B.Align:=A;
B.OnUpClick:=UpClick;
B.OnDownClick:=DownClick;
end;

constructor TQuadSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
Alignment:=taCenter;
BevelOuter:=bvLowered;
Width := 131;
 Height := 26;
MakeSpinButton(FLeftSpin,alLeft);
MakeSpinButton(FRightSpin,alRight);
FColLetter:=True;
FRowLetter:=False;
FColFirst:=True;
FColValue:=0;
FRowValue:=0;
FBlockChangeEvent:=False;
ShowQuad;
end;


procedure TQuadSpin.Changed(OCol,ORow: integer);
begin
if FBlockChangeEvent then exit;
if Assigned(FOnChanged) then FOnChanged(Self,OCol,ORow);
end;


procedure TQuadSpin.UpClick(Sender: TObject);
var OCol,ORow: integer;
begin
OCol:=FColValue;
ORow:=FRowValue;
if Sender=FLeftSpin then
	begin
	if FColFirst then FColValue:=FColValue+1
	else FRowValue:=FRowValue+1;
	end
else
	begin
	if not FColFirst then FColValue:=FColValue+1
	else FRowValue:=FRowValue+1;
	end;
ShowQuad;
Changed(OCol,ORow);
end;

procedure TQuadSpin.DownClick(Sender: TObject);
var OCol,ORow: integer;
begin
OCol:=FColValue;
ORow:=FRowValue;
if Sender=FLeftSpin then
	begin
	if FColFirst then FColValue:=FColValue-1
	else FRowValue:=FRowValue-1;
	end
else
	begin
	if not FColFirst then FColValue:=FColValue-1
	else FRowValue:=FRowValue-1;
	end;
ShowQuad;
Changed(OCol,ORow);
end;


procedure TQuadSpin.SetColFirst(New: boolean);
begin
FColFirst:=New;
ShowQuad;
end;

procedure TQuadSpin.SetColLetter(New: boolean);
begin
FColLetter:=New;
ShowQuad;
end;

procedure TQuadSpin.SetRowLetter(New: boolean);
begin
FRowLetter:=New;
ShowQuad;
end;

procedure TQuadSpin.SetColValue(New: integer);
var OCol,ORow: integer;
begin
OCol:=FColValue;
ORow:=FRowValue;
FColValue:=New;
ShowQuad;
Changed(OCol,ORow);
end;

procedure TQuadSpin.SetRowValue(New: integer);
var OCol,ORow: integer;
begin
OCol:=FColValue;
ORow:=FRowValue;
FRowValue:=New;
ShowQuad;
Changed(OCol,ORow);
end;


end.
