unit ScrollBarWindow;

interface

uses Windows,Forms,Classes,Controls,StdCtrls,ExtCtrls,Messages,Consts,Grids,Math,FLatSB,Graphics;

type
  TInternalScrollBar = class(TPersistent)
  private
    FCreated: boolean;
    FWindow: TWinControl;
    FKind: TScrollBarKind;
    FPosition: Integer;
    FMin: Integer;
    FMax: Integer;
    FPageSize: Integer;
    FSmallChange: TScrollBarInc;
    FLargeChange: TScrollBarInc;
    FOnChange: TNotifyEvent;
    FOnScroll: TScrollEvent;
    procedure DoScroll(var Msg: TWMScroll);
    procedure SetMax(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetPageSize(Value: Integer);
  protected
    procedure Scroll(ScrollCode: TScrollCode; var ScrollPos: Integer); dynamic;
    procedure Change; dynamic;
  public
    DoTrack: boolean;
    constructor Create(AOwner: TWinControl; Kind: TScrollBarKind);
    procedure SetParams(APosition, AMin, AMax: Integer);
  published
    property Created: boolean read FCreated write FCreated;
    property LargeChange: TScrollBarInc read FLargeChange write FLargeChange default 1;
    property Max: Integer read FMax write SetMax default 100;
    property Min: Integer read FMin write SetMin default 0;
    property PageSize: Integer read FPageSize write SetPageSize;
    property Position: Integer read FPosition write SetPosition default 0;
    property SmallChange: TScrollBarInc read FSmallChange write FSmallChange default 1;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnScroll: TScrollEvent read FOnScroll write FOnScroll;
  end;



type TCustomScrollBarWin = class(TCustomControl)
 private
  FHorzScrollBar: TInternalScrollBar;
  FVertScrollBar: TInternalScrollBar;
  FScrollBars: TScrollStyle;
  FOnVertScroll: TScrollEvent;
  FOnHorzScroll: TScrollEvent;
  FOnHorzChange: TNotifyEvent;
  FOnVertChange: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
  procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
  procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
 protected
  procedure KeyPress(var Key: Char); override;
  procedure SetScrollBars(Value: TScrollStyle);
  procedure CreateParams(var Params: TCreateParams); override;
  procedure WMVScroll(var Msg: TWMVScroll); message WM_VSCROLL;
  procedure WMHScroll(var Msg: TWMHScroll); message WM_HSCROLL;
  procedure CreateScrollbar(var Bar: TInternalScrollBar; Kind: TScrollBarKind);
  procedure DoHorzScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer); dynamic;
  procedure DoVertScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer); dynamic;
  procedure DoHorzChange(Sender: TObject); dynamic;
  procedure DoVertChange(Sender: TObject); dynamic;
  property ScrollBars: TScrollStyle read FScrollBars write SetScrollBars default ssBoth;
  property HorzScrollBar: TInternalScrollBar read FHorzScrollBar write FHorzScrollBar;
  property VertScrollBar: TInternalScrollBar read FVertScrollBar write FHorzScrollBar;
  property OnVertScroll: TScrollEvent read FOnVertScroll write FOnVertScroll;
  property OnHorzScroll: TScrollEvent read FOnHorzScroll write FOnHorzScroll;
  property OnHorzChange: TNotifyEvent read FOnHorzChange write FOnHorzChange;
  property OnVertChange: TNotifyEvent read FOnVertChange write FOnVertChange;
  procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
  property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
 end;



type TScrollBarWindow = class(TCustomScrollBarWin)
 published
  property ScrollBars;
  property HorzScrollBar;
  property VertScrollBar;
  property OnVertScroll;
  property OnHorzScroll;
  property OnHorzChange;
  property OnVertChange;
  property Align;
  property Anchors;
  property BevelEdges;
  property BevelInner;
  property BevelOuter;
  property BevelKind;
  property BevelWidth;
  property BiDiMode;
  property Color;
  property Constraints;
  property DockSite;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property Ctl3D;
  property Font;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnCanResize;
  property OnClick;
  property OnConstrainedResize;
  property OnContextPopup;
  property OnDblClick;
  property OnDockDrop;
  property OnDockOver;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnGetSiteInfo;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnMouseWheel;
  property OnMouseWheelDown;
  property OnMouseWheelUp;
  property OnResize;
  property OnStartDock;
  property OnStartDrag;
  property OnUnDock;
 end;



procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TScrollBarWindow]);
end;


function GetScrollCommand(Kind: TScrollBarKind): integer;
begin
case Kind of
 sbHorizontal: Result:=SB_HORZ;
 sbVertical: Result:=SB_VERT;
 else  Result:=SB_HORZ;
 end;
end;


{------------------ Handle Internal Scrollbar ---------------------------------}


procedure TInternalScrollBar.DoScroll(var Msg: TWMScroll);
var ScrollPos: Integer;
var NewPos: Longint;
var ScrollInfo: TScrollInfo;
begin
if not FWindow.HandleAllocated then exit;
NewPos:= FPosition;
case TScrollCode(Msg.ScrollCode) of
 scLineUp: Dec(NewPos, FSmallChange);
 scLineDown: Inc(NewPos, FSmallChange);
 scPageUp: Dec(NewPos, FLargeChange);
 scPageDown: Inc(NewPos, FLargeChange);
 scPosition:
	begin
	ScrollInfo.cbSize:= SizeOf(ScrollInfo);
	ScrollInfo.fMask:= SIF_ALL;
	GetScrollInfo(FWindow.Handle, GetScrollCommand(FKind), ScrollInfo);
	NewPos:= ScrollInfo.nTrackPos;
	end;
 scTrack: if DoTrack then
	begin
	ScrollInfo.cbSize:= SizeOf(ScrollInfo);
	ScrollInfo.fMask:= SIF_ALL;
	GetScrollInfo(FWindow.Handle, GetScrollCommand(FKind), ScrollInfo);
	NewPos:= ScrollInfo.nTrackPos;
	end;
 scTop: NewPos:= FMin;
 scBottom: NewPos:= FMax;
 end;
if NewPos < FMin then NewPos:= FMin;
if NewPos > FMax then NewPos:= FMax;
ScrollPos:= NewPos;
Scroll(TScrollCode(Msg.ScrollCode), ScrollPos);
SetPosition(ScrollPos);
{Change should only occur when user changes }
{scrollbars not when another program sets it}
Change;
end;



procedure TInternalScrollBar.SetParams(APosition, AMin, AMax: Integer);
begin
{We must use the created flag here - HandleAllocated doesn't work}
if not Created then exit;

if (AMax < AMin) or (AMax < FPageSize) then raise EInvalidOperation.Create(SScrollBarRange);
if APosition < AMin then APosition:= AMin;
if APosition > AMax then APosition:= AMax;
if (FMin <> AMin) or (FMax <> AMax) then
	begin
	FMin:= AMin;
	FMax:= AMax;
	SetScrollRange(FWindow.Handle, GetScrollCommand(FKind), AMin, AMax, FPosition = APosition);
	end;
if FPosition <> APosition then
	begin
	FPosition:= APosition;
	SetScrollPos(FWindow.Handle, GetScrollCommand(FKind), FPosition, True);
	end;
end;



procedure TInternalScrollBar.SetPageSize(Value: Integer);
var ScrollInfo: TScrollInfo;
begin
//if not Created then exit;
if not FWindow.HandleAllocated then exit;
if (FPageSize = Value) or (Value > FMax) then exit;
FPageSize:= Value;
ScrollInfo.cbSize:= SizeOf(ScrollInfo);
ScrollInfo.nPage:= Value;
ScrollInfo.fMask:= SIF_PAGE;
SetScrollInfo(FWindow.Handle, GetScrollCommand(FKind), ScrollInfo, True);
end;

{--------------------------------------------------------------------------------}


procedure TInternalScrollBar.SetMin(Value: Integer);
begin
SetParams(FPosition, Value, FMax);
end;


procedure TInternalScrollBar.SetMax(Value: Integer);
begin
SetParams(FPosition, FMin, Value);
end;


procedure TInternalScrollBar.Change;
begin
if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TInternalScrollBar.SetPosition(Value: Integer);
begin
SetParams(Value, FMin, FMax);
end;


procedure TInternalScrollBar.Scroll(ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
if Assigned(FOnScroll) then FOnScroll(Self, ScrollCode, ScrollPos);
end;


constructor TInternalScrollBar.Create(AOwner: TWinControl; Kind: TScrollBarKind);
begin
inherited Create;
DoTrack:=True;
FWindow:=AOwner;
FKind:=Kind;
FPosition:= 0;
FMin:= 0;
FMax:= 100;
FSmallChange:= 1;
FLargeChange:= 1;
end;





{----------------------------------------------------------------------------}



procedure TCustomScrollBarWin.CreateParams(var Params: TCreateParams);
const ScrollBar: array[TScrollStyle] of DWORD = (0, WS_HSCROLL, WS_VSCROLL,  WS_HSCROLL or WS_VSCROLL);
begin
inherited CreateParams(Params);
Params.Style:=Params.Style or ScrollBar[FScrollBars];
HorzScrollBar.Created:=ScrollBars in [ssHorizontal, ssBoth];
VertScrollBar.Created:=ScrollBars in [ssVertical, ssBoth];
end;


procedure TCustomScrollBarWin.SetScrollBars(Value: TScrollStyle);
begin
if FScrollBars <> Value then
	begin
	FScrollBars:= Value;
	RecreateWnd;
	end;
end;


procedure TCustomScrollBarWin.CreateScrollbar(var Bar: TInternalScrollBar; Kind: TScrollBarKind);
begin
Bar:=TInternalScrollBar.Create(Self,Kind);
Bar.Position:= 0;
Bar.Min:= 0;
Bar.Max:= 100;
Bar.SmallChange:= 1;
Bar.LargeChange:= 1;
end;



constructor TCustomScrollBarWin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ScrollBars:=ssBoth;
CreateScrollBar(FHorzScrollBar,sbHorizontal);
CreateScrollBar(FVertScrollBar,sbVertical);
FHorzScrollBar.OnScroll:=DoHorzScroll;
FVertScrollBar.OnScroll:=DoVertScroll;
FHorzScrollBar.OnChange:=DoHorzChange;
FVertScrollBar.OnChange:=DoVertChange;
end;



destructor TCustomScrollBarWin.Destroy;
begin
FHorzScrollBar.Free;
FVertScrollBar.Free;
inherited Destroy;
end;

procedure TCustomScrollBarWin.WMVScroll(var Msg: TWMVScroll);
begin
FVertScrollBar.DoScroll(Msg);
end;


procedure TCustomScrollBarWin.WMHScroll(var Msg: TWMHScroll);
begin
FHorzScrollBar.DoScroll(Msg);
end;


procedure TCustomScrollBarWin.DoHorzScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
if Assigned(FOnHorzScroll) then FOnHorzScroll(Sender,ScrollCode,ScrollPos);
end;


procedure TCustomScrollBarWin.DoVertScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
if Assigned(FOnVertScroll) then FOnVertScroll(Sender,ScrollCode,ScrollPos);
end;


procedure TCustomScrollBarWin.DoHorzChange(Sender: TObject);
begin
if Assigned(FOnHorzChange) then FOnHorzChange(Sender);
end;



procedure TCustomScrollBarWin.DoVertChange(Sender: TObject);
begin
if Assigned(FOnVertChange) then FOnVertChange(Sender);
end;


procedure TCustomScrollBarWin.WMNCPaint(var Message: TMessage);
const
  InnerStyles: array[TBevelCut] of Integer = (0, BDR_SUNKENINNER, BDR_RAISEDINNER, 0);
  OuterStyles: array[TBevelCut] of Integer = (0, BDR_SUNKENOUTER, BDR_RAISEDOUTER, 0);
  EdgeStyles: array[TBevelKind] of Integer = (0, 0, BF_SOFT, BF_FLAT);
  Ctl3DStyles: array[Boolean] of Integer = (BF_MONO, 0);
var
  DC: HDC;
  RC, RW, SaveRW: TRect;
  EdgeSize: Integer;
  WinStyle: Longint;
var C: TColor;
var X1,Y1,X2,Y2: integer;
begin
{This code is patched to fix a problem when BevelKind<>bkNone}
{ Get window DC that is clipped to the non-client area }
{Changes marked with a Star *}
if (BevelKind <> bkNone) or (BorderWidth > 0) then
	begin
	DC := GetWindowDC(Handle);
	try
	Windows.GetClientRect(Handle, RC);
	GetWindowRect(Handle, RW);
	MapWindowPoints(0, Handle, RW, 2);
	OffsetRect(RC, -RW.Left, -RW.Top);
	ExcludeClipRect(DC, RC.Left, RC.Top, RC.Right, RC.Bottom);
	{ Draw borders in non-client area }
	{** The order of these actions changed from the Delphi original}
	{** Make sure that SaveRW retains full client rectanle}
	InflateRect(RC, BorderWidth, BorderWidth);
	RW := RC;
	SaveRW := RW;
	if BevelKind <> bkNone then
		begin
		EdgeSize := 0;
		if BevelInner <> bvNone then Inc(EdgeSize, BevelWidth);
		if BevelOuter <> bvNone then Inc(EdgeSize, BevelWidth);
		WinStyle := GetWindowLong(Handle, GWL_STYLE);
		if beLeft in BevelEdges then Dec(RW.Left, EdgeSize);
		if beTop in BevelEdges then Dec(RW.Top, EdgeSize);
		if beRight in BevelEdges then Inc(RW.Right, EdgeSize);
		if (WinStyle and WS_VSCROLL) <> 0 then Inc(RW.Right, GetSystemMetrics(SM_CYVSCROLL));
		if beBottom in BevelEdges then Inc(RW.Bottom, EdgeSize);
		if (WinStyle and WS_HSCROLL) <> 0 then Inc(RW.Bottom, GetSystemMetrics(SM_CXHSCROLL));

		DrawEdge(DC, RW, InnerStyles[BevelInner] or OuterStyles[BevelOuter], Byte(BevelEdges) or EdgeStyles[BevelKind] or Ctl3DStyles[Ctl3D] or BF_ADJUST);
		end;
	RC:=RW;
	IntersectClipRect(DC, RW.Left, RW.Top, RW.Right, RW.Bottom);
	RW := SaveRW;
	{ Erase parts not drawn }
	OffsetRect(RW, -RW.Left, -RW.Top);
	Windows.FillRect(DC, RW, Brush.Handle);
	{** Now clean up corner area at the ends of the scrollbars}
	C:=Brush.Color;
	RW:=RC;
	X1:=RC.Right-GetSystemMetrics(SM_CYHSCROLL); X2:=RC.Right;
	Y1:=RC.Bottom-GetSystemMetrics(SM_CXHSCROLL); Y2:=RC.Bottom;
	RW:=Rect(X1,Y1,X2,Y1+BevelWidth*2);
	Brush.Color:=clBtnFace;
	Windows.FillRect(DC, RW, Brush.Handle);
	RW:=Rect(X1,Y1,X1+BevelWidth*2,Y2);
	Windows.FillRect(DC, RW, Brush.Handle);
	Brush.Color:=C;
	finally ReleaseDC(Handle, DC); end;
	end;
inherited DefaultHandler(Message);
end;



procedure TCustomScrollBarWin.KeyPress(var Key: Char);
begin
inherited;
end;



procedure TCustomScrollBarWin.CMMouseEnter(var Message: TMessage);
begin
inherited;
if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TCustomScrollBarWin.CMMouseLeave(var Message: TMessage);
begin
inherited;
if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

end.
