unit Sonagram;

interface

uses Windows,Types,Classes,ExtCtrls,Graphics,HugeArry,ImageEx,DepthColors,BitmapLib;

type TSonagramArray = array of double;

type TSonagram = class(TImageEx)
 private
  FMinValue,FMaxValue: double;
  FColorCount: TColorDepth;
  DepthArray: TDepthArray;
  procedure SetColorCount(const Value: TColorDepth);
 public
  procedure DrawScale;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure DisplayLine(var A: THugeDouble); overload;
  procedure DisplayLine(var A: array of double); overload;
  procedure ScrollImage(X,Y: integer);
  procedure SetSize(Columns, Rows: integer);
  property MinValue: double read FMinValue write FMinValue;
  property MaxValue: double read FMaxValue write FMaxValue;
 published
  property ColorCount: TColorDepth read FColorCount write SetColorCount default dc24;
 end;
                       

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TSonagram]);
end;



function ColorTempColor(I: byte): TColor;
var R,G,B: byte;
begin
B:=0;
if I>127 then
	begin
	G:=(I-128) * 2;
	R:=255;
	end
else
	begin
	R:=I * 2;
	G:=0;
	end;
Result:=RGB(R,G,B);
end;





constructor TSonagram.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FMinValue:=0;
FMaxValue:=100;
ColorCount:=DC24;
end;


destructor TSonagram.Destroy;
begin
inherited Destroy;
end;

procedure TSonagram.DrawScale;
var I,X,Y1,Y2: integer;
const Count = 5;
begin
Picture.Bitmap.Canvas.Pen.Color:=clWhite;
Picture.Bitmap.Canvas.Pen.Width:=1;
for I:=1 to Count do
	begin
	X:=MulDiv(I,Picture.Bitmap.Width,Count);
	Y1:=Picture.Bitmap.Height;
	Y2:=Y1 - 50;
	Picture.Bitmap.Canvas.MoveTo(X,Y1);
	Picture.Bitmap.Canvas.LineTo(X,Y2);
	end;
Invalidate;
end;


procedure TSonagram.ScrollImage(X,Y: integer);
var R1,R2: TRect;
begin
R1:=Rect(0,0,Picture.Bitmap.Width,Picture.Bitmap.Height);
R2:=R1;
OffsetRect(R2,0,-1);
Picture.Bitmap.Canvas.CopyRect(R2,Picture.Bitmap.Canvas,R1);
end;



procedure TSonagram.DisplayLine(var A: THugeDouble);
var I: integer;
var LineDelta,ColorDelta,V: double;
var C: TRGBTriple;
var Line: pRGBTripleArray;
begin
RestoreImage;
ScrollImage(0,-1);
LineDelta:=A.Count /Picture.Bitmap.Width;
ColorDelta:=High(DepthArray)/(FMaxValue - FMinValue);
Line:=Picture.Bitmap.ScanLine[Picture.Bitmap.Height-1];
for I:=0 to Picture.Bitmap.Width-1 do
	begin
	V:=A[trunc(I * LineDelta)]-FMinValue;
	C:=ColorToTriple(DepthArray[trunc(V * ColorDelta)+1]);
	Line[I]:=C;
	end;
SaveImage;
DrawScale;
end;





procedure TSonagram.DisplayLine(var A: array of double);
var I: integer;
var LineDelta,ColorDelta,V: double;
var C: TRGBTriple;
var Line: pRGBTripleArray;
begin
RestoreImage;
ScrollImage(0,-1);
LineDelta:=Length(A) /Picture.Bitmap.Width;
ColorDelta:=255/(FMaxValue - FMinValue);
Line:=Picture.Bitmap.ScanLine[Picture.Bitmap.Height-1];
for I:=0 to Picture.Bitmap.Width-1 do
	begin
	V:=A[trunc(I * LineDelta)]-FMinValue;
	C:=ColorToTriple(ColorTempColor(trunc(V * ColorDelta)+1));
	Line[I]:=C;
	end;
SaveImage;
DrawScale;
end;



procedure TSonagram.SetColorCount(const Value: TColorDepth);
{Set default colors }
begin
if FColorCount<>Value then
	begin
	FColorCount:=Value;
	SetDepthColors(FColorCount,DepthArray);
	end;
end;



procedure TSonagram.SetSize(Columns, Rows: integer);
begin
Picture.Bitmap.Width:=Columns;
Picture.Bitmap.Height:=Rows;
SaveImage;
ClearBitmap(Picture.Bitmap,clBlack);
end;

end.
