unit Spectrogram;

interface

uses Windows,ColorBar,SysUtils,Controls,Classes,Graphics,ExtCtrls,HugeArry,
     DepthColors,BitmapLib,IntVector,GraphBox,Clipbrd,Printers,Vector;

type TRange = record
 case integer of
 0: (Min,Max: double);
 1: (V: T2DVector);
 end;

function Range(Min,Max: double): TRange;

type TSpectroGram = class(TCustomPanel)
 private
  FBitmap: TBitmap;
  FColorCount: TColorDepth;
  FBorder: TRect;
  FZRange: TRange;
  FYRange: TRange;
  FXRange: TRange;
  FColorBar: TColorBar;
  FOnBarChanging: TBarEvent;
  FOnBarSpread: TNotifyEvent;
  FOnBarSelect: TNotifyEvent;
    FBarDragDrop: TDragDropEvent;
    FOnBarDragOver: TDragOverEvent;
 procedure SetColorCount(const Value: TColorDepth);
  procedure SetBorder(const Value: TRect);
  procedure PositionColorBar;
  procedure Resize; override;
  procedure GetImageSize(var ImageWidth, ImageHeight: integer; var ImageRect: TRect);
  procedure DrawSpectrogram(Canvas: TCanvas);
  procedure SetZRange(const Value: TRange);
    procedure HandleBarDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure HandleBarDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure HandleBarMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure HandleBarMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
 protected
  procedure Paint; override;
  procedure HandleBarSelect(Sender: TObject);
  procedure HandleChangeSpread(Sender: TObject);
  procedure HandleChangingBar(Sender: TObject; Index: integer; Bottom:LongInt;Depth: double);
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure XYToScaled(X,Y: integer; var X2,Y2: double);
  procedure DrawYScale(Canvas: TCanvas);
  procedure DrawXScale(Canvas: TCanvas);
  procedure DisplayLine(var A: THugeDouble); overload;
  procedure DisplayLine(var A: array of double); overload;
  procedure CopyToClipboard;
  procedure Print;
  procedure ScrollImage(X,Y: integer);
  procedure SetSize(Columns, Rows: integer);
  property XRange: TRange read FXRange write FXRange;
  property YRange: TRange read FYRange write FYRange;
  property ZRange: TRange read FZRange write SetZRange;
 published
  property ColorCount: TColorDepth read FColorCount write SetColorCount default dc24;
  property Border: TRect read FBorder write SetBorder;
  property DepthBar: TColorBar read FColorBar;
  property OnBarSelect: TNotifyEvent read FOnBarSelect write FOnBarSelect;
  property OnBarSpread: TNotifyEvent read FOnBarSpread write FOnBarSpread;
  property OnBarChanging: TBarEvent read FOnBarChanging write FOnBarChanging;
  property OnBarDragOver: TDragOverEvent read FOnBarDragOver write FOnBarDragOver;
  property BarDragDrop: TDragDropEvent read FBarDragDrop write FBarDragDrop;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
 end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TSpectroGram]);
end;


function Range(Min,Max: double): TRange;
begin
Result.Min:=Min;
Result.Max:=Max;
end;

{ TSpectroGram }


procedure TSpectroGram.PositionColorBar;
begin
FColorBar.Width:=FBorder.Right-4;
FColorBar.Top:=FBorder.Top;
FColorBar.Height:=Height-(FBorder.Top+FBorder.Bottom);
FColorBar.Left:=(Width-FBorder.Right)+2;
end;




procedure TSpectroGram.HandleBarSelect(Sender: TObject);
begin
if Assigned(FOnBarSelect) then FOnBarSelect(Sender);
end;


procedure TSpectroGram.HandleChangeSpread(Sender: TObject);
begin
if Assigned(FOnBarSpread) then FOnBarSpread(Sender);
end;


procedure TSpectroGram.HandleChangingBar(Sender: TObject; Index: integer; Bottom:LongInt;Depth: double);
begin
if Assigned(FOnBarChanging) then FOnBarChanging(Sender,Index,Bottom,Depth);
end;


procedure TSpectroGram.HandleBarDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
if Assigned(FOnBarDragOver) then FOnBarDragOver(Sender, Source, X, Y, State, Accept);
end;


procedure TSpectroGram.HandleBarDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
if Assigned(FBarDragDrop) then FBarDragDrop(Sender, Source, X, Y);
end;


procedure TSpectroGram.HandleBarMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
if not TColorBar(Sender).BarMoving then
	TColorBar(Sender).BeginDrag(False);
end;


procedure TSpectroGram.HandleBarMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
if TColorBar(Sender).BarMoving then
	TColorBar(Sender).EndDrag(False);

end;



constructor TSpectroGram.Create(AOwner: TComponent);
begin
inherited;
FBitmap:=TBitmap.Create;
FBitmap.PixelFormat:=pf24bit;
ControlStyle := ControlStyle + [csOpaque];
FBorder.Left:=30;
FBorder.Top:=10;
FBorder.Right:=20;
FBorder.Bottom:=20;
FColorBar:=TColorBar.Create(Self);
FColorBar.Visible := True;
FColorBar.Parent := Self;
FColorBar.ColorCount:=ColorBar.dc32;
FColorBar.Adjustable:=True;
FColorBar.OnChangeSelect:=HandleBarSelect;
FColorBar.OnChangeSpread:=HandleChangeSpread;
FColorBar.OnChangingBar:=HandleChangingBar;
FColorBar.OnDragDrop:=HandleBarDragDrop;
FColorBar.OnDragOver:=HandleBarDragOver;
FColorBar.OnMouseDown:=HandleBarMouseDown;
FColorBar.OnMouseUp:=HandleBarMouseUp;
FColorBar.DragMode:=dmManual;
Self.DoubleBuffered:=True;
Self.ColorCount:=dc24;
PositionColorBar;
end;




destructor TSpectroGram.Destroy;
begin
FColorBar.Free;
FBitmap.Free;
inherited;
end;




procedure TSpectroGram.DrawXScale(Canvas: TCanvas);
var I,XStep: integer;
var SStep: double;
const Count = 5;

	procedure DrawLabel(I: integer);
	var X,Y1,Y2,W,H: integer;
	var S: string;
	begin
	S:=FloatToStrF(I*SStep+FXRange.Min,ffFixed,18,0);
	W:=Canvas.TextWidth(S);
	H:=Canvas.TextHeight(S);
	X:=MulDiv(I,XStep,Count)+Border.Left;
	Y1:=Height - H;
	Y2:=Height - Border.Bottom;
	Canvas.MoveTo(X,Y1);
	Canvas.LineTo(X,Y2+2);
	Canvas.TextOut(X-(W div 2),Y1,S);
	end;

begin
SStep:=(FXRange.Max - FXRange.Min)/Count;
XStep:=Width-(Border.Left+Border.Right);
Canvas.Font.Color:=clWhite;
Canvas.Pen.Color:=clWhite;
Canvas.Pen.Width:=1;
for I:=0 to Count do DrawLabel(I);
end;


procedure TSpectroGram.DrawYScale(Canvas: TCanvas);
var I,YStep: integer;
var SStep: double;
const Gutter = 1;
const Count = 5;

	procedure DrawLabel(I: integer);
	var Y,X1,X2,W,H: integer;
	var S: string;
	begin
	S:=FloatToStrF(I*SStep+FYRange.Min,ffFixed,18,1);
	W:=Canvas.TextWidth(S);
	H:=Canvas.TextHeight(S);
	Y:=MulDiv(I,YStep,Count)+Border.Top;
	Canvas.TextOut(Gutter,Y-(H div 2),S);
	X1:=Gutter+W+3;
	X2:=Border.Left-3;
	Canvas.MoveTo(X1,Y);
	Canvas.LineTo(X2,Y);
	end;

begin
SStep:=(FYRange.Max - FYRange.Min)/Count;
YStep:=Height-(Border.Top+Border.Bottom);
Canvas.Font.Color:=clWhite;
Canvas.Pen.Color:=clWhite;
Canvas.Pen.Width:=1;
for I:=0 to Count do DrawLabel(I);
end;



procedure TSpectroGram.DrawSpectrogram(Canvas: TCanvas);
var R: TRect;
begin
PositionColorBar;
Canvas.Brush.Color:=clBlack;
Canvas.Rectangle(ClientRect);
R:=RectDeflate(ClientRect,Border);
OversampleBitmap(Canvas,R,FBitmap);
Canvas.Pen.Color:=clWhite;
Canvas.Brush.Style:=bsClear;
Canvas.Pen.Style:=psSolid;
InflateRect(R,1,1);
Canvas.Rectangle(R);
DrawXScale(Canvas);
DrawYScale(Canvas);
end;





procedure TSpectroGram.Paint;
begin
inherited;
DrawSpectrogram(Canvas);
end;




function ColorTempColor(I: byte): TColor;
var R,G,B: byte;
begin
B:=0;
if I>127 then
	begin
	G:=(I-128) * 2;
	R:=255;
	end
else
	begin
	R:=I * 2;
	G:=0;
	end;
Result:=RGB(R,G,B);
end;


procedure TSpectroGram.ScrollImage(X,Y: integer);
var R1,R2: TRect;
begin
R1:=Rect(0,0,FBitmap.Width,FBitmap.Height);
R2:=R1;
OffsetRect(R2,0,-1);
FBitmap.Canvas.CopyRect(R2,FBitmap.Canvas,R1);
end;



procedure TSpectroGram.DisplayLine(var A: THugeDouble);
var I: integer;
var LineDelta,V: double;
var Line: pRGBTripleArray;
begin
ScrollImage(0,-1);
LineDelta:=(A.Count-1) / FBitmap.Width;
FColorBar.SetRange(FZRange.Max,FZRange.Min);
Line:=FBitmap.ScanLine[FBitmap.Height-1];
for I:=0 to FBitmap.Width-1 do
	begin
	V:=A[trunc(I * LineDelta)];
	Line[I]:=ColorToTriple(FColorBar.DepthToColor(V));
	end;
Invalidate;
end;





procedure TSpectroGram.DisplayLine(var A: array of double);
var I: integer;
var LineDelta,V: double;
var Line: pRGBTripleArray;
begin
ScrollImage(0,-1);
LineDelta:=High(A) / FBitmap.Width;
FColorBar.SetRange(FZRange.Max,FZRange.Min);
Line:=FBitmap.ScanLine[FBitmap.Height-1];
for I:=0 to FBitmap.Width-1 do
	begin
	V:=A[trunc(I * LineDelta)];
	Line[I]:=ColorToTriple(FColorBar.DepthToColor(V));
	end;
Invalidate;
end;



procedure TSpectroGram.SetColorCount(const Value: TColorDepth);
{Set default colors }
begin
if FColorCount<>Value then
	begin
	FColorCount:=Value;
	FColorBar.ColorCount:=TColorCount(Value);
	end;
end;



procedure TSpectroGram.SetSize(Columns, Rows: integer);
begin
FBitmap.Width:=Columns;
FBitmap.Height:=Rows;
ClearBitmap(FBitmap,clBlack);
end;



procedure TSpectroGram.SetBorder(const Value: TRect);
begin
  FBorder := Value;
end;


procedure TSpectroGram.Resize;
begin
inherited;
PositionColorBar;
end;


procedure TSpectroGram.GetImageSize(var ImageWidth,ImageHeight: integer;var ImageRect: TRect);
var BevelSize: Integer;
begin
ImageRect:=ClientRect;
BevelSize := BorderWidth;
if BevelOuter <> bvNone then Inc(BevelSize, BevelWidth);
if BevelInner <> bvNone then Inc(BevelSize, BevelWidth);
InflateRect(ImageRect, -BevelSize, -BevelSize);
ImageWidth:=ImageRect.Right-ImageRect.Left;
ImageHeight:=ImageRect.Bottom-ImageRect.Top;
end;



procedure TSpectroGram.CopyToClipboard;
var B1,B2: TBitmap;
var R: TRect;
var W,H: integer;
begin
GetImageSize(W,H,R);
B1:=TBitmap.Create;
B2:=TBitmap.Create;
try
B1.Width:=W; B1.Height:=H;
B2.Width:=FColorBar.Width;
B2.Height:=FColorBar.Height;
DrawSpectrogram(B1.Canvas);
FColorBar.DrawColorbar(B2.Canvas);
B1.Canvas.Draw(FColorBar.Left,FColorBar.Top,B2);
ClipBoard.Assign(B1);
finally
 B1.Free;
 B2.Free;
 end;
end;



procedure TSpectroGram.Print;
var B1,B2: TBitmap;
var R: TRect;
var W,H: integer;
begin
GetImageSize(W,H,R);
B1:=TBitmap.Create;
B2:=TBitmap.Create;
try
B1.Width:=W; B1.Height:=H;
B2.Width:=FColorBar.Width;
B2.Height:=FColorBar.Height;
DrawSpectrogram(B1.Canvas);
FColorBar.DrawColorbar(B2.Canvas);
B1.Canvas.Draw(FColorBar.Left,FColorBar.Top,B2);
if W>H then R:=Rect(0,0,Printer.PageWidth,MulDiv(Printer.PageWidth,H,W))
else R:=Rect(0,0,MulDiv(Printer.PageHeight,W,H),Printer.PageHeight);
Printer.BeginDoc;
Printer.Canvas.StretchDraw(R,B1);
Printer.EndDoc;
finally
 B1.Free;
 B2.Free;
 end;
end;


procedure TSpectroGram.SetZRange(const Value: TRange);
begin
if (FZRange.Min<>Value.Min) or (FZRange.Max<>Value.Max) then
	begin
	FZRange:=Value;
	FColorBar.SetRange(FZRange.Max,FZRange.Min);
	end;
end;



procedure TSpectroGram.XYToScaled(X, Y: integer; var X2, Y2: double);
var W,H: integer;
var XScale,YScale: double;
begin
W:=Width - (Border.Left + Border.Right);
XScale:=W/(XRange.Max - XRange.Min);
X2:=(X-Self.Border.Left)/XScale+XRange.Min;
H:=Height - (Border.Top + Border.Bottom);
YScale:=H/(YRange.Max - YRange.Min);
Y2:=(Y-Border.Top)/YScale+YRange.Min;
end;



end.
