unit TreeViewEx;

interface

uses
  Windows, Messages, Graphics, ExtCtrls, SysUtils, Classes, Controls, ComCtrls, CommCtrl, TreeLib, IntVector ;

Type TCheckNotify = procedure(Sender: TObject; Node: TTreeNode; Button: TMouseButton; Shift: TShiftState) of object;


type
  TTreeViewEx = class(TTreeView)
  private
    DragBMBuilt: boolean;
    FDragScroller: TAutoScroller;
    FDragExpander: TAutoExpander;
    FCheckBoxes: boolean;
    FOnCheckChange: TCheckNotify;
    FDragScroll,FDragExpand: boolean;
    FOnCreateNodeClass: TTVCreateNodeClassEvent;
    procedure SetCheckboxes(Value: Boolean);
    procedure SetChecked(Index: Integer; AChecked: Boolean);
    function GetChecked(Index: Integer): Boolean;
    procedure CheckChange(Node: TTreeNode; Button: TMouseButton; Shift: TShiftState);
    function BuildDragImage(var BM: TBitmap): TRect;
    function GetExpandDelay: integer;
    procedure SetExpandDelay(const Value: integer);
  protected
    function CustomDrawItem(Node: TTreeNode; State: TCustomDrawState; Stage: TCustomDrawStage; var PaintImages: Boolean): Boolean; override;
    function IsCustomDrawn(Target: TCustomDrawTarget; Stage: TCustomDrawStage): Boolean; override;
    function CreateNode: TTreeNode; override;
    function GetDragImages: TDragImageList; override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function IsShowing(Node: TTreeNode): boolean;
    property Checked[Index: Integer]: Boolean read GetChecked write SetChecked;
    function GetCheckState(TN: TTreeNode): boolean;
    function SetCheckState(TN: TTreeNode; Check: boolean): boolean;
    procedure SetAllChildren(Node: TTreeNode; Checked: boolean);
    procedure SetAllParents(Node: TTreeNode; Checked: boolean);
    function GetSortedSelections(AList: TList; Compare: TListSortCompare): TTreeNode;
  published
    property ExpandDelay: integer read GetExpandDelay write SetExpandDelay;
    property CheckBoxes: boolean read FCheckBoxes write SetCheckboxes default False;
    property OnCheckChange: TCheckNotify read FOnCheckChange write FOnCheckChange;
    property DragExpand: boolean read FDragExpand write FDragExpand default False;
    property DragScroll: boolean read FDragScroll write FDragScroll default False;
    property OnCreateNodeClass: TTVCreateNodeClassEvent read FOnCreateNodeClass write FOnCreateNodeClass;
  end;

type TTreeNodeEx = class(TTreeNode)
  private
   FChecked: boolean;
  protected
  public
   constructor Create(AOwner: TTreeNodes);
   property Checked: boolean read FChecked write FChecked;
  end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TTreeViewEx]);
end;


{ TTreeNodeEx }

constructor TTreeNodeEx.Create(AOwner: TTreeNodes);
begin
FChecked:=False;
end;

{--------------- Support Routines -----------------------------------}



function TTreeViewEx.CreateNode: TTreeNode;
var LClass: TTreeNodeClass;
begin
LClass:=TTreeNodeEx;
if Assigned(FOnCreateNodeClass) then FOnCreateNodeClass(Self, LClass);
Result:=LClass.Create(Items);
end;



function TTreeViewEx.GetCheckState(TN: TTreeNode): boolean;
{This is slow because it runs through the Windows message loop}
{We only use it when we have to, otherwise we use the Checked property of the TTreeNodeEx}
var tvItem: TTVItem;
begin
if (TN.Handle=0) or (Self.Handle=0) then exit;
if (GetWindowLong(Self.Handle, GWL_STYLE) and TVS_CHECKBOXES) <> TVS_CHECKBOXES then exit;

tvItem.mask := TVIF_HANDLE or  TVIF_STATE;
tvItem.hItem := TN.ItemId;
tvItem.stateMask := TVIS_STATEIMAGEMASK;

TreeView_GetItem(Self.Handle, tvItem);
Result:=bool((tvItem.state shr 12) -1);
end;



function TTreeViewEx.SetCheckState(TN: TTreeNode; Check: boolean): boolean;
var tvItem: TTVItem;
var Inx: integer;
begin
tvItem.mask := TVIF_HANDLE or  TVIF_STATE;
tvItem.hItem := TN.ItemId;
tvItem.stateMask := TVIS_STATEIMAGEMASK;
if Check then Inx:=2 else Inx:=1;
tvItem.state:= INDEXTOSTATEIMAGEMASK(Inx);
Result:=TreeView_SetItem(Self.Handle, tvItem);
TTreeNodeEx(TN).Checked:=Check;
end;




procedure SetComCtlStyle(Ctl: TWinControl; Value: Integer; UseStyle: Boolean);
var Style: Integer;
begin
if Ctl.HandleAllocated then
	begin
	Style:= GetWindowLong(Ctl.Handle, GWL_STYLE);
	if not UseStyle then Style := Style and not Value
	else Style:= Style or Value;
	SetWindowLong(Ctl.Handle, GWL_STYLE, Style);
	end;
end;



function TVRemoveStateImagelist(TVHandle: HWND): Boolean;
{Remove image list so check boxes go away}
var ImageListHandle: HWND;
begin
ImageListHandle:= SendMessage(TVHandle,TVM_SETIMAGELIST, TVSIL_STATE, 0);
if ImageListHandle<>0 then Result:=ImageList_Destroy(ImageListHandle);
end;


{---------------- Object Methods --------------------------------------------}


procedure TTreeViewEx.CreateParams(var Params: TCreateParams);
begin
inherited CreateParams(Params);
//Params.Style:=Params.Style or TVS_CHECKBOXES;
end;




procedure TTreeViewEx.SetCheckboxes(Value: Boolean);
begin
if FCheckBoxes <> Value then
	begin
	FCheckBoxes:= Value;
	SetComCtlStyle(Self, TVS_CHECKBOXES, Value);
	if not Value then TVRemoveStateImagelist(Handle);
	end;
end;



procedure TTreeViewEx.SetChecked(Index: Integer; AChecked: Boolean);
begin
SetCheckState(Items[Index],AChecked);
end;

    
function TTreeViewEx.GetChecked(Index: Integer): Boolean;
{This will always be slow because accessing by index requires search}
{It is faster to put nodes in an array and index into the array}
begin
Result:=TTreeNodeEx(Items[Index]).Checked;
end;


procedure TTreeViewEx.CheckChange(Node: TTreeNode; Button: TMouseButton; Shift: TShiftState);
begin
if Assigned(FOnCheckChange) then FOnCheckChange(Self,Node,Button,Shift);
end;



procedure TTreeViewEx.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var HitTests: THitTests;
var TN: TTreeNode;
begin
inherited MouseDown(Button,Shift,X,Y);
if FCheckBoxes then
	begin
	HitTests:= GetHitTestInfoAt(X, Y);
	if htOnStateIcon in HitTests then
		begin
		TN:=GetNodeAt(X, Y);
		if TN is TTreeNodeEx then TTreeNodeEx(TN).Checked:=GetCheckState(TN);
		CheckChange(TN, Button, Shift);
		end;
        end;
end;



procedure TTreeViewEx.SetAllChildren(Node: TTreeNode; Checked: boolean);
begin
Node:=Node.getFirstChild;
while Node<>nil do
	begin
	Self.SetCheckState(Node,Checked);
	SetAllChildren(Node,Checked);
	Node:=Node.GetNextSibling;
	end;
end;


procedure TTreeViewEx.SetAllParents(Node: TTreeNode; Checked: boolean);
begin
Node:=Node.Parent;
while Node<>nil do
	begin
	Self.SetCheckState(Node,Checked);
	Node:=Node.Parent;
	end;
end;

{------------------------------------------------------------------------------}


constructor TTreeViewEx.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FDragScroller:=TAutoScroller.Create;
FDragExpander:=TAutoExpander.Create;
FDragScroll:=False;
FDragExpand:=False;
end;



destructor TTreeViewEx.Destroy;
begin
FDragScroller.Free;
FDragExpander.Free;
inherited Destroy;
end;


function TTreeViewEx.IsShowing(Node: TTreeNode): boolean;
var R: TRect;
begin
Result:=IntersectRect(R,Node.DisplayRect(False),ClientRect);
end;



{------------------------------------------------------------------------------}



procedure TTreeViewEx.DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
inherited DragOver(Source,X, Y,State,Accept);
if FDragScroll and not FDragExpander.Expanding then FDragScroller.HandleScroll(Self);
if FDragExpand and not FDragScroller.Scrolling then FDragExpander.HandleExpand(Self);
end;


{----------------- Routines to handle MultiSelect Drag and Drop ---------------}


function TTreeViewEx.IsCustomDrawn(Target: TCustomDrawTarget;
  Stage: TCustomDrawStage): Boolean;
{Tell the Tree that we are going to custom draw individual items}
{Normally TreeView doesn't custom draw unless the OnCustomDrawItem}
{is assigned. Overriding this function allows custum draw}
begin
if (Stage = cdPrePaint) and (Target = dtItem) then Result:=True
else Result:=inherited IsCustomDrawn(Target,Stage);
end;




function TTreeViewEx.CustomDrawItem(Node: TTreeNode; State: TCustomDrawState;
  Stage: TCustomDrawStage; var PaintImages: Boolean): Boolean;
{Handle setting the highlight for selected items}
{This prevents the highlight from changing while dragging over an item}
begin
{Note: you cannot use clHighlight as the color }
{Because it doesn't prevent the hightlight from changing}
if cdsSelected in State then Self.Canvas.Brush.Color:=clMenuHighlight
else Self.Canvas.Brush.Color:=clWhite;
Result:=True;
inherited CustomDrawItem(Node,State,Stage,PaintImages);
end;



procedure TTreeViewEx.DoStartDrag(var DragObject: TDragObject);
begin
inherited;
{Flag that we need a new drag Bitmap}
DragBMBuilt:=False;
end;



procedure TTreeViewEx.DoEndDrag(Target: TObject; X, Y: Integer);
begin
inherited;
Invalidate;
end;


function TTreeViewEx.BuildDragImage(var BM: TBitmap): TRect;
{Construct the drag bitmap by copying the image from each of the selected nodes}
var Offset,Size: TPoint;
var R: TRect;
var I: integer;
var Source,Dest: TRect;
begin
{Construct the bounding Rect for the elected items}
Result:=Rect(0,0,0,0);
for I:=0 to SelectionCount-1 do
 if IsShowing(Selections[I]) then
	begin
        R:=Selections[I].DisplayRect(True);
        R.Left:=0;
	UnionRect(Result,Result,R);
	end;
{Make sure we don't capture anything beyond the Tree View}
IntersectRect(Result,Result,ClientRect);
Offset:=Result.TopLeft;
Size:=IntVectorSubtract(Result.BottomRight,Result.TopLeft);
Dest.TopLeft:=Point(0,0);
Dest.BottomRight:=Size;
BM.Width:=Size.X;
BM.Height:=Size.Y;
{Copy the selected items from the TreeView to the bitmap}
for I:=0 to SelectionCount-1 do
 if IsShowing(Selections[I]) then
	begin
        Source:=Selections[I].DisplayRect(True);
        Source.Left:=0;
	Dest:=Source;
	OffsetRect(Dest,-Offset.X,-Offset.Y);
	BM.Canvas.CopyRect(Dest,Canvas,Source);
	end;
end;




function TTreeViewEx.GetDragImages: TDragImageList;
{Provide the drag image for the TreeView object}
var XImage: TBitMap;
var P,P2: TPoint;
var Base: TRect;
begin
Result:=inherited GetDragImages;
if Assigned(Result) and Assigned(Images) and not DragBMBuilt then
	begin
	XImage := TBitMap.Create;
	try
	GetCursorPos(P);
	P2:=ScreenToClient(P);
	{Tell the Drag Object to stop using the current image so we can rebuild it}
	Result.EndDrag;
	{Create our own drage image}
	Result.Clear;
	Base:=BuildDragImage(XImage);
	Result.Width:=XImage.Width;
	Result.Height:=XImage.Height;
	{Tell the Drag Object about it}
	Result.AddMasked(XImage, clWhite);
	Result.SetDragImage(0, P2.X, P2.Y-Base.Top);
	{Tell the drag object to start using our new image}
	Result.BeginDrag(GetDesktopWindow, P.X, P.Y);
	finally XImage.Free; end;
	DragBMBuilt:=True;
	end;
end;




function TTreeViewEx.GetSortedSelections(AList: TList; Compare: TListSortCompare): TTreeNode;
begin
Result:=GetSelections(AList);
AList.Sort(Compare);
end;



function TTreeViewEx.GetExpandDelay: integer;
begin
Result:=FDragExpander.Delay;
end;



procedure TTreeViewEx.SetExpandDelay(const Value: integer);
begin
FDragExpander.Delay:=Value;
end;


end.
