unit Trkruler;

interface

uses Forms,SysUtils,Classes,Controls,ExtCtrls,WinTypes,WinProcs,Graphics,Menus;


const StartLimit=500;

type
TTextRuler=class(TCustomPanel)
 private
  FStart: longint;
  FSpacing,FGutter: integer;
  FMarker1,FMarker2,FMarker3: integer;
  FMarkMoving: integer;
  OldMark1,OldMark2,OldMark3: integer;
  FOnMarkerChange: TNotifyEvent;
  FLineColor: TColor;
  FShowNumbers: boolean;
    FScale: double;
  procedure SetShowNumbers(Value: boolean);
  function HitTest(X,Y: integer): integer;
  function GetBorderWidth: integer;
  function GetLastTick: integer;
  procedure GetTopMarkerOutline(Loc: integer;var Rect: TRect;var XC,YC: integer);
  procedure DrawTopMarker(Loc: integer);
  procedure EraseTopMarker(Loc: integer);
  procedure GetLowMarkerOutline(Loc: integer;var Rect: TRect;var XC,YC: integer);
  procedure DrawLowMarker(Loc: integer);
  procedure EraseLowMarker(Loc: integer);
  procedure DrawRuler;
    procedure SetScale(const Value: double);
 protected
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure Paint; override;
  procedure SetMarker1(Value: Integer);
  procedure SetMarker2(Value: Integer);
  procedure SetMarker3(Value: Integer);
  procedure MoveMarker1(Value: Integer);
  procedure MoveMarker2(Value: Integer);
  procedure MoveMarker3(Value: Integer);
  procedure SetSpacing(Value: Integer);
  procedure SetStart(Value: longint);
  procedure SetGutter(Value: Integer);
  procedure SetLineColor(Value: TColor);
  procedure MarkerChange(Sender: TObject);
 public
  CursorCol: integer;
  constructor Create(AOwner: TComponent); override;
  property Scale: double read FScale write SetScale;
 published
  property Spacing: integer read FSpacing write SetSpacing default 8;
  property Start: longint read FStart write SetStart default 0;
  property Gutter: integer read FGutter write SetGutter default 0;
  property Marker1: integer read FMarker1 write SetMarker1 default 1;
  property Marker2: integer read FMarker2 write SetMarker2 default 2;
  property Marker3: integer read FMarker3 write SetMarker3 default 1;
  property OnMarkerChange: TNotifyEvent read FOnMarkerChange write FOnMarkerChange;
  property LineColor: TColor read FLineColor write SetLineColor default clBlack;
  property ShowNumbers: boolean read FShowNumbers write SetShowNumbers default true;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color;
    property Ctl3D;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
 end;



procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Fountain', [TTextRuler]);
end;

{------------------Property Setting Routines ---------------------------}

procedure TTextRuler.SetSpacing(Value: integer);
begin
if (Value>0) and (FSpacing<>Value) then
	begin
	FSpacing:=Value;
	Invalidate;
	end;
end;

procedure TTextRuler.SetStart(Value: longint);
begin
if FStart<>Value then
	begin
	FStart:=Value;
	Invalidate;
	end;

end;

procedure TTextRuler.SetGutter(Value: integer);
begin
if FGutter<>Value then
	begin
	FGutter:=Value;
	Invalidate;
	end;
end;


procedure TTextRuler.SetShowNumbers(Value: boolean);
begin
if FShowNumbers<>Value then
	begin
	FShowNumbers:=Value;
	Invalidate;
	end;
end;

procedure TTextRuler.SetLineColor(Value: TColor);
begin
if FLineColor<>Value then
	begin
	FLineColor:=Value;
	Invalidate;
	end;
end;



{----------------- Draw Rulers and Markers -----------------------------}

function TTextRuler.GetBorderWidth: integer;
begin
Result:=0;
if BevelInner<>bvNone then Inc(Result,BevelWidth);
if BevelOuter<>bvNone then Inc(Result,BevelWidth);
if BorderStyle<>bsNone then Inc(Result);
Inc(Result,BorderWidth);
end;


function TTextRuler.GetLastTick: integer;
begin
Result:=Round(Start+((Width - (GetBorderWidth*2) - FGutter) / FSpacing * FScale));
end;


procedure TTextRuler.GetLowMarkerOutline(Loc: integer;var Rect: TRect;var XC,YC: integer);
var Brd: integer;
begin
Brd:=GetBorderWidth;
XC:=Round((Loc*FSpacing*FScale+Brd)+FGutter);
Rect.Bottom:=Height-Brd-1;
Rect.Top:=Rect.Bottom-8;
YC:=Rect.Bottom-4;
Rect.Left:=XC-4;
Rect.Right:=XC+4;
end;


procedure TTextRuler.EraseLowMarker(Loc: integer);
var YCenter,XCenter: integer;
var Rect: TRect;
begin
GetLowMarkerOutLine(Loc,Rect,XCenter,YCenter);
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Color;
Canvas.Polygon([Point(XCenter,Rect.Top),Point(Rect.Left,YCenter),
Point(Rect.Left,Rect.Bottom),Point(Rect.Right,Rect.Bottom),
Point(Rect.Right,YCenter),Point(XCenter,Rect.Top)]);
end;


procedure TTextRuler.DrawLowMarker(Loc: integer);
var YCenter,XCenter: integer;
var Rect: TRect;
begin
GetLowMarkerOutLine(Loc,Rect,XCenter,YCenter);
Canvas.Pen.Color:=clBlack;
Canvas.Brush.Color:=clLtGray;
Canvas.Polygon([Point(XCenter,Rect.Top),Point(Rect.Left,YCenter),
Point(Rect.Left,Rect.Bottom),Point(Rect.Right,Rect.Bottom),
Point(Rect.Right,YCenter),Point(XCenter,Rect.Top)]);
Canvas.Pen.Color:=clWhite;
Canvas.PolyLine([Point(XCenter,Rect.Top+1),Point(Rect.Left+1,YCenter),Point(Rect.Left+1,Rect.Bottom-1)]);
Canvas.Pen.Color:=clDkGray;
Canvas.PolyLine([Point(XCenter,Rect.Top+1),Point(Rect.Right-1,YCenter),
Point(Rect.Right-1,Rect.Bottom-1),Point(Rect.Left,Rect.Bottom-1)]);
end;


procedure TTextRuler.GetTopMarkerOutline(Loc: integer;var Rect: TRect;var XC,YC: integer);
var Brd: integer;
begin
Brd:=GetBorderWidth;
XC:=Round((Loc*FSpacing*FScale+Brd)+FGutter);
Rect.Top:=Brd;
Rect.Bottom:=Rect.Top+7;
YC:=Rect.Top+3;
Rect.Left:=XC-4;
Rect.Right:=XC+4;
end;


procedure TTextRuler.EraseTopMarker(Loc: integer);
var YCenter,XCenter: integer;
var Rect: TRect;
begin
GetTopMarkerOutLine(Loc,Rect,XCenter,YCenter);
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Color;
Canvas.Polygon([Point(XCenter,Rect.Bottom),Point(Rect.Left,YCenter),
Point(Rect.Left,Rect.Top),Point(Rect.Right,Rect.Top),
Point(Rect.Right,YCenter),Point(XCenter,Rect.Bottom)]);
end;


procedure TTextRuler.DrawTopMarker(Loc: integer);
var YCenter,XCenter: integer;
var Rect: TRect;
begin
GetTopMarkerOutLine(Loc,Rect,XCenter,YCenter);
Canvas.Pen.Color:=clBlack;
Canvas.Brush.Color:=clLtGray;

Canvas.Polygon([Point(XCenter,Rect.Bottom),Point(Rect.Left,YCenter),
		Point(Rect.Left,Rect.Top),Point(Rect.Right,Rect.Top),
		Point(Rect.Right,YCenter),Point(XCenter,Rect.Bottom)]);

Canvas.Pen.Color:=clWhite;
Canvas.PolyLine([Point(Rect.Left+1,YCenter),Point(Rect.Left+1,Rect.Top+1),
		Point(Rect.Right-1,Rect.Top+1)]);

Canvas.Pen.Color:=clDkGray;
Canvas.PolyLine([Point(Rect.Left+1,YCenter),Point(XCenter,Rect.Bottom-1),
Point(Rect.Right-1,YCenter),Point(Rect.Right-1,Rect.Top+1)]);
end;



procedure TTextRuler.DrawRuler;
var I,X,Brd,CW,CH,CO,W: integer;
var H2,H3,H4: integer;
var Offset: longint;
var Rct: TRect;
var tm: TTextMetric;
var S: string;
begin
GetTextMetrics(Canvas.Handle,tm);
CH:=tm.tmHeight;
Brd:=GetBorderWidth;
H2:=(Height-Brd*2) div 2;
H3:=(Height-Brd*2) div 3;
H4:=(Height-Brd*2) div 4;

Canvas.Brush.Color:=Color;
Canvas.Pen.Color:=Color;

Rct:=Canvas.ClipRect;
InflateRect(Rct,-Brd*2,-Brd*2);
Canvas.Rectangle(Rct);

Canvas.Pen.Color:=FLineColor;
SetBKMode(Canvas.Handle,TRANSPARENT);
I:=0;
while true do
	begin
	Offset:=I+FStart;
	X:=Round(FGutter+Brd+(I*FSpacing*FScale));
	if X>=(Width-Brd) then break;
	if (Offset Mod 10)=0 then
		begin
		Canvas.Moveto(X,Brd);
		Canvas.LineTo(X,Height-Brd);
		if FShowNumbers then
			begin
			S:=IntToStr(Offset);
			W:=Canvas.TextWidth(S);
			Canvas.TextOut(X-W div 2,(Height-Brd-CH) div 2,IntToStr(Offset));
			end;
		end
	else if (Offset mod 5)=0 then
		begin
		Canvas.Moveto(X,H4+Brd);
		Canvas.LineTo(X,H4+H2+Brd);
		end
	else if (I=1) or (((Offset-1) mod 10)<>0) and (((Offset+1) mod 10)<>0) then
		begin
		Canvas.Moveto(X,H3+Brd);
		Canvas.LineTo(X,H3+H3+Brd);
		end;
	if CursorCol=Offset then
		begin
                Canvas.Brush.Color:=clRed;
		Canvas.Rectangle(X,H3+Brd,Round(X+FSpacing*FScale),H3+H3+Brd);
		Canvas.Brush.Color:=Color;
		end;
	Inc(I);
        end;
if FStart<StartLimit then
	begin
	DrawLowMarker(FMarker1-FStart);
	DrawLowMarker(FMarker2-FStart);
	DrawTopMarker(FMarker3-FStart);
	end;
end;

{---------------- Routines to set markers ------------------------------}


procedure TTextRuler.MarkerChange(Sender: TObject);
begin
if Assigned(FOnMarkerChange) then FOnMarkerChange(Self);
end;


procedure TTextRuler.MoveMarker1(Value: Integer);
var LastTick: integer;
begin
LastTick:=GetLastTick;
if Value<>FMarker1 then
	begin
	OldMark1:=FMarker1;
	OldMark2:=FMarker2;
	OldMark3:=FMarker3;
	FMarker1:=Value;
	if FMarker1<Start then FMarker1:=Start;
	if FMarker1>LastTick then FMarker1:=LastTick;
	if FMarker1>=FMarker2 then
		begin
		FMarker2:=FMarker1+1;
		if FMarker2>LastTick then
			begin
		        FMarker2:=LastTick;
			FMarker1:=LastTick-1;
			end;
		end;
	if FMarker1>FMarker3 then
		begin
		FMarker3:=FMarker1;
		if FMarker3>LastTick then FMarker3:=LastTick;
 		end;
	Invalidate;
	end;
end;



procedure TTextRuler.MoveMarker2(Value: Integer);
var LastTick: integer;
begin
LastTick:=GetLastTick;
if Value<>FMarker2 then
	begin
	OldMark1:=FMarker1;
	OldMark2:=FMarker2;
	FMarker2:=Value;
	if FMarker2<Start then FMarker2:=Start;
	if FMarker2>LastTick then FMarker2:=LastTick;
	if FMarker2<=FMarker1 then
		begin
		FMarker1:=FMarker2-1;
		if FMarker1<Start then
        		begin
        		FMarker1:=Start;
        		FMarker2:=Start+1;
        		end;
		end;
	Invalidate;
	end;
end;


procedure TTextRuler.MoveMarker3(Value: Integer);
var LastTick: integer;
begin
LastTick:=GetLastTick;
if Value<>FMarker3 then
	begin
	OldMark1:=FMarker1;
	OldMark3:=FMarker3;
	FMarker3:=Value;
	if FMarker3<Start then FMarker3:=Start;
	if FMarker3>LastTick then FMarker3:=LastTick;
	if FMarker3<FMarker1 then
		begin
		FMarker1:=FMarker3;
		if FMarker1<Start then FMarker1:=Start;
		end;
	Invalidate;
	end;
end;


procedure TTextRuler.SetMarker1(Value: Integer);
begin
if Value<>FMarker1 then
	begin
	OldMark1:=FMarker1;
	OldMark2:=FMarker2;
	OldMark3:=FMarker3;
	FMarker1:=Value;
	Invalidate;
	end;
end;


procedure TTextRuler.SetMarker2(Value: Integer);
begin
if Value<>FMarker2 then
	begin
	OldMark1:=FMarker1;
	OldMark2:=FMarker2;
	FMarker2:=Value;
	Invalidate;
	end;
end;


procedure TTextRuler.SetMarker3(Value: Integer);
begin
if Value<>FMarker3 then
	begin
	OldMark1:=FMarker1;
	OldMark2:=FMarker2;
	OldMark3:=FMarker3;
	FMarker3:=Value;
	Invalidate;
	end;
end;



{----------------------------------------------------------------------}


function TTextRuler.HitTest(X,Y: integer): integer;
var Rect: TRect;
var XC,YC: integer;
begin
GetLowMarkerOutline(FMarker1-Start,Rect,XC,YC);
if PtInRect(Rect,Point(X,Y)) then begin Result:=1; exit; end;
GetLowMarkerOutline(FMarker2-Start,Rect,XC,YC);
if PtInRect(Rect,Point(X,Y)) then begin Result:=2; exit; end;
GetTopMarkerOutline(FMarker3-Start,Rect,XC,YC);
if PtInRect(Rect,Point(X,Y)) then begin Result:=3; exit; end;
Result:=0;
end;


procedure TTextRuler.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseDown(Button, Shift, X, Y);
FMarkMoving:=HitTest(X,Y);
end;


procedure TTextRuler.MouseMove(Shift: TShiftState; X, Y: Integer);
var NewX: integer;
begin
inherited MouseMove(Shift, X, Y);
if FMarkMoving=0 then exit;
NewX:=Round((((X-FGutter) / FSpacing)+Start)/FScale);
case FMarkMoving of
 1: MoveMarker1(NewX);
 2: MoveMarker2(NewX);
 3: MoveMarker3(NewX);
 end;
end;



procedure TTextRuler.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseUp(Button, Shift, X, Y);
if FMarkMoving>0 then MarkerChange(Self);
FMarkMoving:=0;
end;


constructor TTextRuler.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ControlStyle :=ControlStyle-[csSetCaption];
FShowNumbers:=True;
Height:=17;
FStart:=0;
FGutter:=0;
FSpacing:=8;
FScale:=1;
FMarker1:=1;
FMarker2:=2;
FMarker3:=1;
end;


procedure TTextRuler.Paint;
begin
inherited Paint;
OldMark1:=FMarker1;
OldMark2:=FMarker2;
OldMark3:=FMarker3;
DrawRuler;
end;

procedure TTextRuler.SetScale(const Value: double);
begin
if FScale<>Value then
	begin
	FScale := Value;
	Invalidate;
	end;
end;

end.
