unit Version;


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TypInfo, DSupport;

{ $M+ tells delphi to publish RTTI info for enumerated
types. This allows your enumerated types to act as strings
with GetEnumName }

{$M+}
type
  TVersionType=(vtCompanyName, vtFileDescription, vtFileVersion, vtInternalName,
                vtLegalCopyright,vtLegalTradeMark, vtOriginalFileName,
                vtProductName, vtProductVersion, vtComments);
{$M-}

TVersionInfo = class(TComponent)
  private
    FVersionInfo : Array [0 .. ord(high(TVersionType))] of string;
  protected
    function GetCompanyName: string;
    function GetFileDescription: string;
    function GetFileVersion: string;
    function GetInternalName: string;
    function GetLegalCopyright: string;
    function GetLegalTradeMark: string;
    function GetOriginalFileName: string;
    function GetProductName: string;
    function GetProductVersion: string;
    function GetComments: string;
    function GetVersionInfo(VersionType: TVersionType): string; virtual;
    procedure SetVersionInfo; overload; virtual;
    procedure SetVersionInfo(Filename: string); overload; virtual;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AOwner: TComponent; Filename: string); overload;
  published
    function GetVersionNumbers(var Major, Minor, Release, Build: integer): boolean;
    property CompanyName: string read GetCompanyName;
    property FileDescription: string read GetFileDescription;
    property FileVersion: string read GetFileVersion;
    property InternalName: string read GetInternalName;
    property LegalCopyright: string read GetLegalCopyright;
    property LegalTradeMark: string read GetLegalTradeMark;
    property OriginalFileName: string read GetOriginalFileName;
    property ProductName: string read GetProductName;
    property ProductVersion: string read GetProductVersion;
    property Comments: string read GetComments;
  end;



procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TVersionInfo]);
end;



constructor TVersionInfo.Create(AOwner: TComponent);
{Create to get version info from the application}
begin
inherited Create(AOwner);
SetVersionInfo;
end;


constructor TVersionInfo.Create(AOwner: TComponent; Filename: string);
{Create to get version info from specified EXE file}
begin
inherited Create(AOwner);
SetVersionInfo(Filename);
end;



function TVersionInfo.GetCompanyName: string;
begin
result := GeTVersionInfo(vtCompanyName);
end;



function TVersionInfo.GetFileDescription: string;
begin
result := GeTVersionInfo(vtFileDescription);
end;



function TVersionInfo.GetFileVersion: string;
begin
result := GeTVersionInfo(vtFileVersion);
end;



function TVersionInfo.GetInternalName: string;
begin
result := GeTVersionInfo(vtInternalName);
end;



function TVersionInfo.GetLegalCopyright: string;
begin
result := GeTVersionInfo(vtLegalCopyright);
end;



function TVersionInfo.GetLegalTradeMark: string;
begin
result := GeTVersionInfo(vtLegalTradeMark);
end;



function TVersionInfo.GetOriginalFileName: string;
begin
result := GeTVersionInfo(vtOriginalFileName);
end;



function TVersionInfo.GetProductName: string;
begin
result := GeTVersionInfo(vtProductName);
end;



function TVersionInfo.GetProductVersion: string;
begin
result := GeTVersionInfo(vtProductVersion);
end;



function TVersionInfo.GetVersionNumbers(var Major,Minor,Release,Build: integer): boolean;
var I: integer;
var S,S1: string;
begin
Result:=True;
S:=FileVersion;
I:=1;
S1:=ExtractToken(S,'.',I); Major:=StrToIntDeF(S1,0);
S1:=ExtractToken(S,'.',I); Minor:=StrToIntDeF(S1,0);
S1:=ExtractToken(S,'.',I); Release:=StrToIntDeF(S1,0);
S1:=ExtractToken(S,'.',I); Build:=StrToIntDeF(S1,0);
end;



function TVersionInfo.GetComments: string;
begin
result := GeTVersionInfo(vtComments);
end;



function TVersionInfo.GeTVersionInfo(VersionType: TVersionType): string;
begin
result := FVersionInfo[ord(VersionType)];
end;



procedure TVersionInfo.SetVersionInfo(Filename: string);
var sAppName,sVersionType : string;
var iAppSize,iLenOfValue: cardinal;
var I: integer;
var pcBuf: Pointer;
var pcValue: PChar;
begin
sAppName := Filename;
iAppSize:= GetFileVersionInfoSize(PChar(sAppName),iAppSize);
if iAppSize > 0 then
	begin
	pcBuf := AllocMem(iAppSize);
	GetFileVersionInfo(PChar(sAppName),0,iAppSize,pcBuf);
	for I := 0 to Ord(High(TVersionType)) do
		begin
		sVersionType := GetEnumName(TypeInfo(TVersionType),I);
		sVersionType := Copy(sVersionType,3,length(sVersionType));
		if VerQueryValue(pcBuf,PChar('StringFileInfo\040904E4\'+ sVersionType), Pointer(pcValue),iLenOfValue) then FVersionInfo[I] := pcValue;
		end;
	FreeMem(pcBuf,iAppSize);
	end;
end;




procedure TVersionInfo.SetVersionInfo;
begin
SetVersionInfo(Application.ExeName);
end;



end.

