unit Waveread;

interface

uses Wintypes,WinProcs,Forms,Messages,Classes,MMSystem,StdCtrls,Dialogs,Math,
     DSupport;


{Enumerated wave data format types}

type TWaveTypes =	(wfNone,wf11Khz_Mono_8Bit,wf11Khz_Stereo_8Bit,wf11Khz_Mono_16Bit,wf11Khz_Stereo_16Bit,
		wf22Khz_Mono_8Bit,wf22Khz_Stereo_8Bit,wf22Khz_Mono_16Bit,wf22Khz_Stereo_16Bit,
		wf44Khz_Mono_8Bit,wf44Khz_Stereo_8Bit,wf44Khz_Mono_16Bit,wf44Khz_Stereo_16Bit);

{Structure that holds the details of the of the data channel }

type TWaveDataFormat = record
 Frequency,
 BytesPerSample,
 Channels: integer;
 end;

{Define a procedure prototype for the "Data-Ready" call back}

type TWaveInEvent = procedure(Sender: TObject; Buffer: Pointer; SampleCount: integer; Format: TWaveDataFormat) of object;

{Declaration for Wave Reader - Object reads data from Windows audio device}

type TWaveReader = class(TComponent)
  private
    FHWnd: THandle;
    WaveInHandle: hWaveIn;
    BufferHandle1,BufferHandle2,BufferHandle3,BufferHandle4: THandle;
    BufferHeader1,BufferHeader2,BufferHeader3,BufferHeader4: TWaveHdr;
    FSampleCount: dword;
    IsSampling: boolean;
    IsStopping: boolean;
    FWaveType: TWaveTypes;
    FOnWaveIn: TWaveInEvent;
    FormatList: TStringList;
    FComboBox: TComboBox;
    procedure SetOnWaveIn(Value: TWaveInEvent);
    procedure InitializeWaveDevice;
    procedure SetFormatList(Format: TStringList; X: longint);
    procedure SetComboBox(Value: TComboBox);

    procedure SetupBuffer(var BufferHandle: THandle; var BufferHeader: TWaveHdr; Size: dword);
    procedure MessageHandler(var Msg:TMessage);
    function ReadWaveType: TWaveTypes;
    procedure WriteWaveType(V: TWaveTypes);
    function ReadDataFormat: TWaveDataFormat;
    procedure FinalizeBuffer(var BufferHandle: THandle; var BufferHeader: TWaveHdr);
    function GetIsRunning: boolean;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    WorkingFormat: TWaveDataFormat;
    PendingBuffers: integer;
    procedure SetupWaveHeader(var WH: TWaveFormatEx; DataFormat: TWaveDataFormat);
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure StopSampling;
    procedure StartSampling;
    property IsRunning: boolean read GetIsRunning;
  published
    property OnWaveIn: TWaveInEvent read FOnWaveIn write SetOnWaveIn;
    property ComboBox: TComboBox read FComboBox write SetComboBox;
    property WaveType: TWaveTypes read ReadWaveType write WriteWaveType;
    property SampleCount: DWord read FSampleCount write FSampleCount default 256;
    property DataFormat: TWaveDataFormat read ReadDataFormat;
  end;

procedure Register;

function DecodeWaveType(W: TWaveTypes): integer;
procedure FillFormatList(List: TStringList; X: integer);
procedure WaveTypeToDataFormat(WaveType: TWaveTypes; var DataFormat: TWaveDataFormat);

implementation


{------------ Wave Support Routines ------------------------------------------}

 

function DecodeWaveType(W: TWaveTypes): integer;
{Convert Our Wave type to the Windows Enumberated data type}
begin
case W of
 wfNone:               Result:=WAVE_INVALIDfORMAT;
 wf11Khz_Mono_8Bit:    Result:=WAVE_FORMAT_1M08;
 wf11Khz_Stereo_8Bit:  Result:=WAVE_FORMAT_1S08;
 wf11Khz_Mono_16Bit:   Result:=WAVE_FORMAT_1M16;
 wf11Khz_Stereo_16Bit: Result:=WAVE_FORMAT_1S16;
 wf22Khz_Mono_8Bit:    Result:=WAVE_FORMAT_2M08;
 wf22Khz_Stereo_8Bit:  Result:=WAVE_FORMAT_2S08;
 wf22Khz_Mono_16Bit:   Result:=WAVE_FORMAT_2M16;
 wf22Khz_Stereo_16Bit: Result:=WAVE_FORMAT_2S16;
 wf44Khz_Mono_8Bit:    Result:=WAVE_FORMAT_4M08;
 wf44Khz_Stereo_8Bit:  Result:=WAVE_FORMAT_4S08;
 wf44Khz_Mono_16Bit:   Result:=WAVE_FORMAT_4M16;
 wf44Khz_Stereo_16Bit: Result:=WAVE_FORMAT_4S16
 else Result:=WAVE_INVALIDfORMAT;
 end;
end;


procedure FillFormatList(List: TStringList; X: integer);
{Fill a string list with valid wave types specified by bit pattern in X}
begin
List.Clear;
if X=0 then List.AddObject('No Wave Mapper',TObject(wave_InvalidFormat));
if IsBitSet(X,WAVE_FORMAT_1M08) then List.AddObject('11.025 kHz, Mono, 8-bit',TObject(wf11Khz_Mono_8Bit));
if IsBitSet(X,WAVE_FORMAT_1S08) then List.AddObject('11.025 kHz, Stereo, 8-bit',TObject(wf11Khz_Stereo_8Bit));
if IsBitSet(X,WAVE_FORMAT_1M16) then List.AddObject('11.025 kHz, Mono, 16-bit',TObject(wf11Khz_Mono_16Bit));
if IsBitSet(X,WAVE_FORMAT_1S16) then List.AddObject('11.025 kHz, Stereo, 16-bit',TObject(wf11Khz_Stereo_16Bit));
if IsBitSet(X,WAVE_FORMAT_2M08) then List.AddObject('22.05 kHz, Mono, 8-bit',TObject(wf22Khz_Mono_8Bit));
if IsBitSet(X,WAVE_FORMAT_2S08) then List.AddObject('22.05 kHz, Stereo, 8-bit',TObject(wf22Khz_Stereo_8Bit));
if IsBitSet(X,WAVE_FORMAT_2M16) then List.AddObject('22.05 kHz, Mono, 16-bit',TObject(wf22Khz_Mono_16Bit));
if IsBitSet(X,WAVE_FORMAT_2S16) then List.AddObject('22.05 kHz, Stereo, 16-bit',TObject(wf22Khz_Stereo_16Bit));
if IsBitSet(X,WAVE_FORMAT_4M08) then List.AddObject('44.1 kHz, Mono, 8-bit',TObject(wf44Khz_Mono_8Bit));
if IsBitSet(X,WAVE_FORMAT_4S08) then List.AddObject('44.1 kHz, Stereo, 8-bit',TObject(wf44Khz_Stereo_8Bit));
if IsBitSet(X,WAVE_FORMAT_4M16) then List.AddObject('44.1 kHz, Mono, 16-bit',TObject(wf44Khz_Mono_16Bit));
if IsBitSet(X,WAVE_FORMAT_4S16) then List.AddObject('44.1 kHz, Stereo, 16-bit',TObject(wf44Khz_Stereo_16Bit));
end;



procedure WaveTypeToDataFormat(WaveType: TWaveTypes; var DataFormat: TWaveDataFormat);
{Fill data structure with specific details for each wave data type }
begin
case DecodeWaveType(WaveType) of
 WAVE_FORMAT_1M08: begin DataFormat.Frequency:=11025; DataFormat.Channels:=1; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_1S08: begin DataFormat.Frequency:=11025; DataFormat.Channels:=2; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_1M16: begin DataFormat.Frequency:=11025; DataFormat.Channels:=1; DataFormat.BytesPerSample:=2; end;
 WAVE_FORMAT_1S16: begin DataFormat.Frequency:=11025; DataFormat.Channels:=2; DataFormat.BytesPerSample:=2; end;
 WAVE_FORMAT_2M08: begin DataFormat.Frequency:=22050; DataFormat.Channels:=1; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_2S08: begin DataFormat.Frequency:=22050; DataFormat.Channels:=2; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_2M16: begin DataFormat.Frequency:=22050; DataFormat.Channels:=1; DataFormat.BytesPerSample:=2; end;
 WAVE_FORMAT_2S16: begin DataFormat.Frequency:=22050; DataFormat.Channels:=2; DataFormat.BytesPerSample:=2; end;
 WAVE_FORMAT_4M08: begin DataFormat.Frequency:=44100; DataFormat.Channels:=1; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_4S08: begin DataFormat.Frequency:=44100; DataFormat.Channels:=2; DataFormat.BytesPerSample:=1; end;
 WAVE_FORMAT_4M16: begin DataFormat.Frequency:=44100; DataFormat.Channels:=1; DataFormat.BytesPerSample:=2; end;
 WAVE_FORMAT_4S16: begin DataFormat.Frequency:=44100; DataFormat.Channels:=2; DataFormat.BytesPerSample:=2; end;
 end;
end;


{ TWaveReader }


procedure Register;
{Register component so Delph knows about it}
begin
RegisterComponents('Fountain', [TWaveReader]);
end;



procedure TWaveReader.Notification(AComponent: TComponent; Operation: TOperation);
{Handle the live deletion of a combo box controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) and (AComponent = FComboBox) then FComboBox := nil;
end;



function GetWaveTypeFromCombo(ComboBox: TComboBox; Default: TWaveTypes): TWaveTypes;
{Read the selected wave-type from the specified combo box}
begin
Result:=Default;
if (ComboBox<>nil) and (ComboBox.Items.Count>-1) then
 Result:=TWavetypes(ComboBox.Items.Objects[ComboBox.ItemIndex]);
end;



function TWaveReader.ReadWaveType: TWaveTypes;
{Read the selected wave-type from the combobox assigned to this object}
begin
Result:=GetWaveTypeFromCombo(FComboBox,FWaveType);
end;



function FindWaveTypeInCombo(ComboBox: TComboBox; WaveType: TWaveTypes): integer;
{Test to see if a specific wave type is found in the combobox}
var WT: TWaveTypes;
begin
if ComboBox=nil then exit;
for Result:=0 to ComboBox.Items.Count-1 do
  if WaveType=TWavetypes(ComboBox.Items.Objects[Result]) then exit;
Result:=-1;
end;



procedure TWaveReader.WriteWaveType(V: TWaveTypes);
{Set the objects to use the specified wave type}
{If there is a combobox associated with the Onject}
{ select the matching item in the combobox}
begin
if V<>WaveType then
	begin
	FWaveType:=V;
	if (FComboBox<>nil) then
		begin
		FComboBox.ItemIndex:=FindWaveTypeInCombo(FComboBox,FWaveType);
		end;
	end;
end;


procedure TWaveReader.SetComboBox(Value: TComboBox);
{If we have a comobox associated with the object}
{fill it with a list of valid wave types}
begin
FComboBox:=Value;
if FComboBox <> nil then
	begin
	FComboBox.FreeNotification(Self);
	FComboBox.Clear;
	FComboBox.Items.AddStrings(FormatList);
	if FComboBox.Items.Count>0 then FComboBox.ItemIndex:=0;
	end;
end;



function TWaveReader.ReadDataFormat: TWaveDataFormat;
{Get the currently selected data format}
begin
WaveTypeToDataFormat(WaveType,Result);
end;



procedure TWaveReader.SetFormatList(Format: TStringList; X: longint);
{Get a string list of all valid formats for the current sound hardware}
begin
FillFormatList(Format,X);
end;



procedure TWaveReader.InitializeWaveDevice;
{Initialize The first device that can read audio}
var DevCnt: integer;
var Caps: TWaveInCaps;
begin
DevCnt:=waveInGetNumDevs;
if DevCnt=0 then
	begin
	MessageDlg('No Wave Device Available', mtInformation,[mbOk], 0);
	SetFormatList(FormatList,0);
	exit;
        end;

waveInGetDevCaps(0,@Caps,Sizeof(Caps));
SetFormatList(FormatList,Caps.dwFormats);
end;



constructor TWaveReader.Create(AOwner: TComponent);
{Handle operations associated with the object creation}
begin
inherited Create(AOwner);
FHWnd := Classes.AllocateHWnd(MessageHandler);
FormatList:=TStringList.Create;
FSampleCount:=256;
InitializeWaveDevice;
IsSampling:=False;
IsStopping:=False;
end;



destructor TWaveReader.Destroy;
{Handle object desstruction}
begin
FormatList.Free;
Classes.DeallocateHWnd(FHWnd);
inherited Destroy;
end;


procedure TWaveReader.SetOnWaveIn(Value: TWaveInEvent);
{Set user specified routine to handle when a data sample arrives}
begin
FOnWaveIn := Value;
end;


function TWaveReader.GetIsRunning: boolean;
{Returns true if the Reader is reading and streaming data}
begin
Result:=IsSampling or IsStopping;
end;



{----------------- Routines to Start sampling --------------------------------}


procedure TWaveReader.SetupWaveHeader(var WH: TWaveFormatEx; DataFormat: TWaveDataFormat);
{Build the header used to open the samplier}
begin
WH.wFormatTag := WAVE_FORMAT_PCM;
WH.nChannels := DataFormat.Channels;
WH.nSamplesPerSec := DataFormat.Frequency;
WH.wBitsPerSample := DataFormat.BytesPerSample * 8;
WH.nBlockAlign := WH.nChannels * (WH.wBitsPerSample div 8);
WH.nAvgBytesPerSec := WH.nSamplesPerSec * WH.nBlockAlign;
WH.cbSize := 0;
end;



procedure TWaveReader.SetupBuffer(var BufferHandle: THandle; var BufferHeader: TWaveHdr; Size: dword);
{Build a buffer for the data}
var Buffer: pointer;
begin
BufferHandle := GlobalAlloc(GMEM_MOVEABLE or GMEM_SHARE or GMEM_ZEROINIT, Size);
Buffer := GlobalLock(BufferHandle);

BufferHeader.lpData := Buffer;
BufferHeader.dwBufferLength := Size;
BufferHeader.dwFlags := WHDR_BEGINLOOP;
WaveInPrepareHeader(WaveInHandle, @BufferHeader, sizeof(BufferHeader));
WaveInAddBuffer(WaveInHandle, @BufferHeader, sizeof(BufferHeader));
end;



procedure TWaveReader.StartSampling;
{Start the sampling process }
var WaveHeader: TWaveFormatEx;
begin
if IsSampling or IsStopping then exit;
WorkingFormat:=DataFormat;
SetupWaveHeader(WaveHeader,WorkingFormat);
WaveInOpen(@WaveInHandle, WAVE_MAPPER, @WaveHeader, FHWnd, 0, CALLBACK_WINDOW);

SetupBuffer(BufferHandle1,BufferHeader1,WaveHeader.nBlockAlign * FSampleCount);
SetupBuffer(BufferHandle2,BufferHeader2,WaveHeader.nBlockAlign * FSampleCount);
SetupBuffer(BufferHandle3,BufferHeader3,WaveHeader.nBlockAlign * FSampleCount);
SetupBuffer(BufferHandle4,BufferHeader4,WaveHeader.nBlockAlign * FSampleCount);
PendingBuffers:=4;
WaveInStart(WaveInHandle);
IsSampling:=true;
IsStopping:=False;
end;



{----------------- Routines to Stop sampling --------------------------------}




procedure TWaveReader.FinalizeBuffer(var BufferHandle: THandle; var BufferHeader: TWaveHdr);
{Handle deallocation of a data buffer}
begin
WaveInUnPrepareHeader(WaveInHandle, @BufferHeader, sizeof(BufferHeader));
WaveInClose(WaveInHandle);
GlobalUnlock(BufferHandle);
GlobalFree(BufferHandle);
end;



procedure TWaveReader.StopSampling;
{Handle stopping the sampling process }
begin
if not IsSampling or IsStopping then exit;
IsSampling := false;

IsStopping:=True;
WaveInReset(WaveInHandle);
while PendingBuffers>0 do Application.ProcessMessages;
IsStopping:=False;

FinalizeBuffer(BufferHandle1,BufferHeader1);
FinalizeBuffer(BufferHandle2,BufferHeader2);
FinalizeBuffer(BufferHandle3,BufferHeader3);
FinalizeBuffer(BufferHandle4,BufferHeader4);
end;



procedure TWaveReader.MessageHandler(var Msg: TMessage);
{Message handler }
begin
if Msg.Msg <> MM_WIM_DATA then exit;

Dec(PendingBuffers);
if Assigned(FOnWaveIn) then FOnWaveIn(Self,PWaveHdr(Msg.lParam)^.lpData,FSampleCount,WorkingFormat);
if IsSampling then
	begin
	WaveInAddBuffer(WaveInHandle, PWaveHdr(Msg.lParam), SizeOf(TWaveHdr));
	Inc(PendingBuffers);
	end;
end;



end.
