unit WinHTTPClient;

{Object to provide HTTP access to the internet using windows WinHTTP API}
{Object duplicated the TNMHttp interface }

interface

uses Windows,Classes,SysUtils,StrUtils, DSupport, WinHTTP;




type TStatusState = (ssOpen, ssConnect, ssOpenRequest, ssRequest, ssResponse,
		    ssQueryData, ssSetTimeouts, ssGetIEProxy, ssReadData, ssCloseconnection,
		    ssSetOptions,ssWriteData, ssSetCredentials, ssGetAuthSchemes, ssCloseRequest,
		    ssCloseSession);


type TOnStatus = procedure(Sender: TComponent; Status: string) of object;
type TOnStatusEx = procedure(Sender: TComponent; State: TStatusState; Status: string) of object;


type TWinHTTPClient = class(TComponent)
 private
  FHeader: string;
  FBody: string;
  FCookieIn: string;
  FInputFileMode: boolean;
  FOnError: TOnStatusEx;
  FUserAgent: string;
  FStatus: string;
  FOnStatus: TOnStatus;
  FConnectTimeout: integer;
  FReceiveTimeout: integer;
  FResolveTimeout: integer;
  FSendTimeout: integer;
  FConnectRetries: integer;
  FServerUserName: string;
  FServerPassword: string;
  FProxyUserName: string;
  FProxyPassword: string;
  FUseIEProxy: boolean;
  FContentType: string;
  FPacketRecvd: TNotifyEvent;
  FBytesRecvd: cardinal;
  FOnStatusEx: TOnStatusEx;
  CancelFlag: boolean;
  function DoHTTPCommand(URL, Com, SendData: string; var Data,Header, Cookies: string): boolean;
  function HandleError(Error: boolean; State: TSTatusState; ErrorMsg: string): boolean;
  procedure StatusMessage(State: TSTatusState; Value: string);
  function AuthorizeTransation(hRequest: HINTERNET; Com,SendData: string): boolean;
  function ChooseAuthScheme(dwSupportedSchemes: DWORD): DWORD;
 protected
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function Get(URL: string): boolean;
  function Head(URL: string): boolean;
  function Post(URL, PostData: string): boolean;
  function Put(URL, PutData: string): boolean;
  function Patch(URL, PatchData: string): boolean;
  function Delete(URL: string; DeleteData: string): boolean;
  function Trace(URL, TraceData: string): boolean;
  procedure Cancel;
  property CookieIn: string read FCookieIn;
  property Status: string read FStatus;
  property BytesRecvd: cardinal read FBytesRecvd;
 published


  property Body: string read FBody write FBody;
  property Header: string read FHeader write FHeader;
  property InputFileMode: boolean read FInputFileMode write FInputFileMode;
  property OnError: TOnStatusEx read FOnError write FOnError;
  property UserAgent: string read FUserAgent write FUserAgent;
  property OnStatus: TOnStatus read FOnStatus write FOnStatus;
  property OnStatusEx: TOnStatusEx read FOnStatusEx write FOnStatusEx;

  property ResolveTimeout: integer read FResolveTimeout write FResolveTimeout default 0;
  property ConnectTimeout: integer read FConnectTimeout write FConnectTimeout default 60000;
  property SendTimeout: integer read FSendTimeout write FSendTimeout default 30000;
  property ReceiveTimeout: integer read FReceiveTimeout write FReceiveTimeout default 30000;
  property Timeout: integer read FConnectTimeout write FConnectTimeout default 60000;
  property ConnectRetries: integer read FConnectRetries write FConnectRetries default 5;
  property ServerUserName: string read FServerUserName write FServerUserName;
  property ServerPassword: string read FServerPassword write FServerPassword;

  property ProxyUserName: string read FProxyUserName write FProxyUserName;
  property ProxyPassword: string read FProxyPassword write FProxyPassword;
  property UseIEProxy: boolean read FUseIEProxy write FUseIEProxy default false;
  property ContentType: string read FContentType write FContentType;
  property OnPacketRecvd: TNotifyEvent read FPacketRecvd write FPacketRecvd;

//  property OutputFileMode: boolean read FOutputFileMode write FOutputFileMode;
//  property OnPacketRecvd;
//  property OnPacketSent;

{  property Proxy;
  property ProxyPort;

  property OnAboutToSend: TNotifyEvent read FOnAboutToSend write FOnAboutToSend;
  property OnSuccess: TResultEvent read FOnSuccess write FOnSuccess;
  property OnRedirect: THandlerEvent read FOnRedirect write FOnRedirect;
  property OnAuthenticationNeeded: TNotifyEvent read FOnAuthenticationNeeded write FOnAuthenticationNeeded;
}
 end;


procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TWinHTTPClient]);
end;


{ TWinHTTPClient }






{------------------------------------------------------------------------------}



function SchemeToStr(Scheme: INTERNET_SCHEME): string;
begin
case Scheme of
 INTERNET_SCHEME_FILLER: Result:='Filler';
 INTERNET_SCHEME_HTTP: Result:='HTTP';
 INTERNET_SCHEME_HTTPS: Result:='HTTPS';
 else Result:='Unknown';
 end;
end;



//type TCrackString = array [0..Max_Path] of WideChar;
type TCrackString = string;

type TCrackedURL = record
 SchemeName: TCrackString;
 Scheme: INTERNET_SCHEME;
 HostName: TCrackString;
 Port: word;
 UserName: TCrackString;
 Password: TCrackString;
 UrlPath: TCrackString;
 ExtraInfo: TCrackString;
 end;


function GetCrackedStr(CU: TCrackedURL): string;
begin
Result:='';
Result:=Result+'SchemeName: '+CU.SchemeName+CRLF_Char;
Result:=Result+'Scheme: '+ SchemeToStr(CU.Scheme)+CRLF_Char;
Result:=Result+'HostName: '+CU.HostName+CRLF_Char;
Result:=Result+'Port: '+IntToStr(CU.Port)+CRLF_Char;
Result:=Result+'UserName: '+CU.UserName+CRLF_Char;
Result:=Result+'Password: '+CU.Password+CRLF_Char;
Result:=Result+'UrlPath: '+CU.UrlPath+CRLF_Char;
Result:=Result+'ExtraInfo: '+CU.ExtraInfo+CRLF_Char;
end;


{------------------------------------------------------------------------------}


function GetCrackedURL(URL: string; var CU: TCrackedURL): boolean;
var UC: URL_COMPONENTS;
begin
FillChar(UC, Sizeof(UC), 0);
UC.dwStructSize := Sizeof(UC);

UC.dwSchemeLength := MAX_PATH;
GetMem(UC.lpszScheme,MAX_Path);

UC.dwHostNameLength := MAX_PATH;
GetMem(UC.lpszHostName,MAX_Path);

UC.dwUserNameLength := MAX_PATH;
GetMem(UC.lpszUserName,MAX_Path);

UC.dwPasswordLength := MAX_PATH;
GetMem(UC.lpszPassword,MAX_Path);

UC.dwUrlPathLength := MAX_PATH;
GetMem(UC.lpszUrlPath,MAX_Path);

UC.dwExtraInfoLength := MAX_PATH;
GetMem(UC.lpszExtraInfo,MAX_Path);

Result:=WinHttpCrackUrl(PWideChar(WideString(URL)), 0, ICU_DECODE, UC);
CU.Port:=UC.nPort;
CU.Scheme:=UC.nScheme;
CU.SchemeName:=UC.lpszScheme;
CU.HostName:=UC.lpszHostName;
CU.UserName:=UC.lpszUserName;
CU.Password:=UC.lpszPassword;
CU.URLPath:=UC.lpszUrlPath;
CU.ExtraInfo:=UC.lpszExtraInfo;

FreeMem(UC.lpszScheme);
FreeMem(UC.lpszHostName);
FreeMem(UC.lpszUserName);
FreeMem(UC.lpszPassword);
FreeMem(UC.lpszUrlPath);
FreeMem(UC.lpszExtraInfo);
end;




procedure TWinHTTPClient.StatusMessage(State: TSTatusState; Value: string);
begin
try
FStatus := Value;
if not (csDestroying in ComponentState) then
	begin
	if Assigned(FOnStatus) then FOnStatus(Self, FStatus);
	if Assigned(FOnStatusEx) then FOnStatusEx(Self, State, FStatus);
	end;

{Dispose of any errors in user's code}
except end;
end;




function TWinHTTPClient.HandleError(Error: boolean; State: TSTatusState; ErrorMsg: string): boolean;
var ErrorNumber: cardinal;
begin
Result:=Error;
if Error then
	begin
	ErrorNumber:=GetLastError;
	if Assigned(FOnError) then FOnError(Self,State,ErrorMsg+': '+WinHttpSysErrorMessage(ErrorNumber));
	end;
StatusMessage(State, ErrorMsg);
end;


{It is the server's responsibility only to accept authentication schemes that
provide a sufficient level of security to protect the servers resources.

The client is also obligated only to use an authentication scheme that
adequately protects its username and password.

Thus, this code does not use Basic authentication because Basic authentication
exposes the client's username and password to anyone monitoring the connection.}



function  TWinHTTPClient.ChooseAuthScheme(dwSupportedSchemes: DWORD): DWORD;
{Choose best authentication scheme available}
begin
if (dwSupportedSchemes and WINHTTP_AUTH_SCHEME_NEGOTIATE)<>0 then Result:= WINHTTP_AUTH_SCHEME_NEGOTIATE
else if (dwSupportedSchemes and WINHTTP_AUTH_SCHEME_NTLM)<>0 then Result:= WINHTTP_AUTH_SCHEME_NTLM
else if (dwSupportedSchemes and WINHTTP_AUTH_SCHEME_PASSPORT)<>0 then Result:= WINHTTP_AUTH_SCHEME_PASSPORT
else if (dwSupportedSchemes and WINHTTP_AUTH_SCHEME_DIGEST)<>0 then Result:= WINHTTP_AUTH_SCHEME_DIGEST
else if (dwSupportedSchemes and WINHTTP_AUTH_SCHEME_BASIC)<>0 then Result:= WINHTTP_AUTH_SCHEME_BASIC
else Result:= 0;
end;



function TWinHTTPClient.AuthorizeTransation(hRequest: HINTERNET; Com,SendData: string): boolean;
{Subroutine to handle authorization transactions with server}
var dwProxyAuthScheme: DWORD;
var bResults: bool;
var dwContext,dwStatusCode: DWord;
var dwSize,DataWritten: DWord;
var dwSupportedSchemes: DWORD;
var dwFirstScheme: DWORD;
var dwTarget: DWORD;
var dwSelectedScheme: DWORD;
var dwLastStatus: DWORD;
begin
Result:=False;
dwProxyAuthScheme:=0;
dwLastStatus:=0;
dwContext:=0;
dwStatusCode := 0;
while true do
	begin
	{ If a proxy authentication challenge was responded to, reset those credentials before each SendRequest, because the proxy }
	{ may require re-authentication after responding to a 401 or to a redirect. If you don't, you can get into a  407-401-407-401- loop.}
	if dwProxyAuthScheme <> 0 then
		begin
		bResults:=WinHttpSetCredentials(hRequest, WINHTTP_AUTH_TARGET_PROXY, dwProxyAuthScheme, PWidechar(WideString(ProxyUsername)), PWidechar(WideString(ProxyPassword)), nil );
		if HandleError(not bResults,ssSetCredentials,'Set Credentials-1') then exit;
		end;

	{ Send a request.}
	bResults:=WinHttpSendRequest(hRequest, WINHTTP_NO_ADDITIONAL_HEADERS, 0, WINHTTP_NO_REQUEST_DATA , 0, 0, cardinal(nil^));
	if HandleError(not bResults,ssRequest,'Send Request') then exit;

	{Add body-data}
	if SendData<>'' then
		begin
		dwSize:=Length(SendData);
		if HandleError(not WinHttpWriteData(hRequest,@SendData[1],dwSize, DataWritten),ssWriteData,'Write '+Com+' Data') then exit;
		end;

	{ End the request.}
	bResults:=WinHttpReceiveResponse(hRequest, nil);
	{ Resend the request in case of ERROR_WINHTTP_RESEND_REQUEST error.}
	if not bResults then
		begin
		if GetLastError= ERROR_WINHTTP_RESEND_REQUEST then continue
		else if HandleError(true,ssResponse,'Get Response') then exit;
		end;

	{ Check the status code.}
	dwSize := sizeof(DWORD);
	bResults:=WinHttpQueryHeaders(hRequest, WINHTTP_QUERY_STATUS_CODE or WINHTTP_QUERY_FLAG_NUMBER, nil, @dwStatusCode, dwSize, nil );

	if bResults then
		begin
		case dwStatusCode of
		 200:
			begin
			{ The resource was successfully retrieved.}
			{ You can use WinHttpReadData to read the}
			{ contents of the server's response.}
			Result:=True;
			exit;
			end;
		 401:
			begin
			{ The server requires authentication. Obtain the supported and preferred schemes.}
			bResults:=WinHttpQueryAuthSchemes( hRequest, dwSupportedSchemes, dwFirstScheme, dwTarget );
			if HandleError(not bResults,ssGetAuthSchemes,'Get Authorization Schemes') then exit;

			{ Set the credentials before resending the request.}
			if bResults then
				begin
				dwSelectedScheme:=ChooseAuthScheme(dwSupportedSchemes);
				if dwSelectedScheme = 0 then exit;
				bResults:=WinHttpSetCredentials( hRequest, dwTarget, dwSelectedScheme, PWidechar(WideString(ServerUsername)), PWidechar(WideString(ServerPassword)), nil );
				if HandleError(not bResults,ssSetCredentials,'Set Credentials-2') then exit;
				end;

			{ If the same credentials are requested twice, abort the}
			{ request.  For simplicity, this sample does not check}
			{ for a repeated sequence of status codes.}
			if dwLastStatus = 401 then exit;
			end;

		 407:
		 	begin
		 	{ The proxy requires authentication.}
			{ Obtain the supported and preferred schemes.}
			bResults:=WinHttpQueryAuthSchemes(hRequest, dwSupportedSchemes, dwFirstScheme, dwTarget );

			{ Set the credentials before resending the request.}
			if bResults then dwProxyAuthScheme:=ChooseAuthScheme(dwSupportedSchemes);

			{ If the same credentials are requested twice, abort the}
			{ request.  For simplicity, this sample does not check}
			{ for a repeated sequence of status codes.}
			if dwLastStatus = 407 then exit;
			end;
		 else 
			{ The status code is error.}
			exit;
			end;
		 end;


		{ Keep track of the last status code.}
		dwLastStatus:=dwStatusCode;

		{ If there are any errors, break out of the loop.}
		if  not bResults then exit;
		end;
end;




function TWinHTTPClient.DoHTTPCommand(URL, Com, SendData: string; var Data,Header, Cookies: string): boolean;
var dwSize: DWord;
var dwDownloaded: DWord;
var OutBuffer: string;
var bResults: bool;
var hSession,hConnect, hRequest: HINTERNET;
var RequestFlag: DWord;
var CU: TCrackedURL;
var proxyConfig: WINHTTP_CURRENT_USER_IE_PROXY_CONFIG;
var CT: WideString;


	function GetHeader: string;
	{Get header}
	var HeaderBuffer: WideString;
	begin
	{Find out how big the buffer needs to be}
	WinHttpQueryHeaders(hRequest, WINHTTP_QUERY_RAW_HEADERS_CRLF, WINHTTP_HEADER_NAME_BY_INDEX, nil, dwSize, WINHTTP_NO_HEADER_INDEX);
	{ Allocate memory for the buffer.}
	if GetLastError = ERROR_INSUFFICIENT_BUFFER then SetLength(HeaderBuffer,(dwSize div 2)+2);
	{ Now, use WinHttpQueryHeaders to retrieve the header.}
	bResults:= WinHttpQueryHeaders(hRequest, WINHTTP_QUERY_RAW_HEADERS_CRLF, WINHTTP_HEADER_NAME_BY_INDEX, @HeaderBuffer[1], dwSize, WINHTTP_NO_HEADER_INDEX);
	Result:=HeaderBuffer;
	end;


	function GetCookies(Header: string): string;
	{Extract cookie information from the header}
	var SL: TStringList;
	var I: integer;
	const CookieStr = 'Set-Cookie: ';
	begin
	SL:=TStringList.Create;
	try
	Result:='';
	SL.Text:=Header;
	for I:=0 to SL.Count-1 do
		begin
		if PosNoCase(CookieStr,SL[I])>0 then
			Result:=Result+RightStr(SL[I],Length(SL[I])-Length(CookieStr))+CRLF_Char;
		end;
	finally SL.Free; end;
	end;


	function ValidWideCharStr(WS: PWideChar): boolean;
	begin
	Result:=(WS<>nil) and (WS[0]>#0);
	end;


begin
CancelFlag:=False;
Result:=False;
dwSize:=0;
dwDownloaded:=0;
bResults:=False;
GetCrackedURL(URL, CU);
FBytesRecvd:=0;

{Figure out which flag to use based on the HTTP vs HTTPS}
if CU.Scheme = INTERNET_SCHEME_HTTPS then RequestFlag:=WINHTTP_FLAG_SECURE else RequestFlag:=0;

{Get Internet Explorer proxy settings. Some thing require it like "Fiddler."}
if USeIEProxy then
 if HandleError(not WinHttpGetIEProxyConfigForCurrentUser(proxyConfig),ssGetIEProxy,'Get IE Proxy') then exit;

{ Use WinHttpOpen to obtain a session handle. Use IE proxy depending of settings}
if UseIEProxy and ValidWideCharStr(proxyConfig.lpszProxy) and
   ValidWideCharStr(proxyConfig.lpszProxyBypass) then hSession:=WinHttpOpen(PWChar(WideString(FUserAgent)), WINHTTP_ACCESS_TYPE_NAMED_PROXY, proxyConfig.lpszProxy, proxyConfig.lpszProxyBypass, 0)
else hSession:=WinHttpOpen(PWChar(WideString(FUserAgent)), WINHTTP_ACCESS_TYPE_DEFAULT_PROXY, WINHTTP_NO_PROXY_NAME, WINHTTP_NO_PROXY_BYPASS, 0);

if HandleError(hSession=nil,ssOpen,'Open')  then exit;

{ Specify an HTTP server.}
try
hConnect:=WinHttpConnect(hSession, PWChar(WideString(CU.HostName)), CU.Port, 0);
if HandleError(hConnect=nil,ssConnect,'Connect') then exit;

try
{ Create an HTTP request handle.}
hRequest:=WinHttpOpenRequest(hConnect, PWChar(WideString(Com)), PWChar(WideString(CU.UrlPath+CU.ExtraInfo)), nil, WINHTTP_NO_REFERER, WINHTTP_DEFAULT_ACCEPT_TYPES, RequestFlag);
if HandleError(hRequest=nil,ssOpenRequest,'Open '+Com+' Request') then exit;

try
{Set Timeouts }
if HandleError(not WinHttpSetTimeouts(hSession, ResolveTimeout, ConnectTimeout, SendTimeout, ReceiveTimeout),ssSetTimeouts,'Set Timeouts') then exit;

{Set retries}
if HandleError(not WinHttpSetOption(hRequest, WINHTTP_OPTION_CONNECT_RETRIES , Pointer(@ConnectRetries), SizeOf(ConnectRetries)),ssSetOptions,'Set Retries') then exit;


{Set content-type}
CT :='Content-Type: '+ContentType;
WinHttpAddRequestHeaders(hRequest, PWideChar(CT), Length(CT), WINHTTP_ADDREQ_FLAG_REPLACE or WINHTTP_ADDREQ_FLAG_ADD);


if not AuthorizeTransation(hRequest, Com,SendData) then exit;

{Get headers and cookies}
Header:=GetHeader;
Cookies:=GetCookies(Header);

{ Keep checking for data until there is nothing left.}
Data:='';
repeat
	begin
	{ Check for available data.}
	dwSize:=0;
	if HandleError(not WinHttpQueryDataAvailable(hRequest, dwSize),ssQueryData,'Query Data') then exit;
	{ Allocate space for the buffer.}
	SetLength(OutBuffer,dwSize);

	{ Read the data.}
	if HandleError(not WinHttpReadData(hRequest, @OutBuffer[1], dwSize, dwDownloaded),ssReadData,'Read Data: '+IntToStr(dwSize)) then exit;
	Data:=Data+OutBuffer;
	FBytesRecvd:=FBytesRecvd+dwSize;
	if Assigned(FPacketRecvd) then FPacketRecvd(self);
	end;
until CancelFlag or (dwSize<= 0);

{ Close any open handles.}
finally WinHttpCloseHandle(hRequest); end;
HandleError(False,ssCloseRequest,'Close Request');

finally WinHttpCloseHandle(hConnect); end;
HandleError(False,ssCloseconnection,'Close Connection');

finally WinHttpCloseHandle(hSession); end;
HandleError(False,ssCloseSession,'Close Session');
Result:=True;
end;



constructor TWinHTTPClient.Create(AOwner: TComponent);
begin
inherited;
FUserAgent:='Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36';
ResolveTimeout:=0;
ConnectTimeout:=60000;
SendTimeout:=30000;
ReceiveTimeout:=30000;
ConnectRetries:=5;
ProxyUserName:='';
ProxyPassword:='';
UseIEProxy:=False;
ContentType:='application/x-www-form-urlencoded';
end;



destructor TWinHTTPClient.Destroy;
begin

inherited;
end;




function TWinHTTPClient.Get(URL: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'GET','',FBody,FHeader, FCookieIn);
end;



function TWinHTTPClient.Head(URL: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'HEAD','',FBody,FHeader, FCookieIn);
end;


function TWinHTTPClient.Post(URL, PostData: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'POST',PostData,FBody,FHeader, FCookieIn);
end;




function TWinHTTPClient.Put(URL, PutData: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'PUT',PutData,FBody,FHeader, FCookieIn);
end;



function TWinHTTPClient.Delete(URL: string; DeleteData: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'DELETE',DeleteData,FBody,FHeader, FCookieIn);
end;



function TWinHTTPClient.Patch(URL, PatchData: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'PATCH',PatchData,FBody,FHeader, FCookieIn);
end;


function TWinHTTPClient.Trace(URL, TraceData: string): boolean;
begin
FBody:=''; FHeader:=''; FCookieIn:='';
Result:=DoHTTPCommand(URL,'TRACE',TraceData,FBody,FHeader, FCookieIn);
end;


procedure TWinHTTPClient.Cancel;
begin
CancelFlag:=True;
end;



end.
