unit Xedcntrl;

interface
uses
  WinTypes, WinProcs, Messages, Classes, Controls, Menus,
  Forms, StdCtrls, Spin, XedScrn;

type
 TXEDControl = class(TWinControl)
 private
  HScrollBar,VScrollBar: TScrollBar;
  ScrollButton: TSpinButton;
  FOnChange: TXedChange;
  FOnKeyPress: TKeyPressEvent;
  FOnKeyDown: TKeyEvent;
  FScrollBars: TScrollStyle;
  FirstTime: boolean;
  procedure AdjustComponents(Width,Height: integer);
  procedure SetScrollBars(V: TScrollStyle);
  procedure HandleKeyPress(Sender: TObject; var Key: Char);
  procedure HandleKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  procedure Change(ScreenChange: boolean);
  procedure HandleUpdate(Sender: TObject;ScreenChange: boolean);

 protected
  procedure WMSize(var Message: TWMSize); message WM_SIZE;
  procedure DoEnter; override;
  procedure Erase(var Msg: TWMEraseBKGnd); message WM_ERASEBKGND;
 public
  Screen: TXEDScreen;
  procedure ScrollHorz(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
  procedure ScrollVert(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
  procedure ScrollScrnUp(Sender: TObject);
  procedure ScrollScrnDown(Sender: TObject);
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property Align;
  property PopupMenu;
  property OnKeyDown:TKeyEvent read FOnKeyDown write FOnKeyDown;
  property OnKeyPress:TKeyPressEvent read FOnKeyPress write FOnKeyPress;
  property OnChange: TXedChange read FOnChange write FOnChange;
  property ScrollBars: TScrollStyle read FScrollBars write SetScrollBars default ssBoth;
end;


procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Fountain', [TXEDControl]);
end;

procedure TXEDControl.Erase(var Msg: TWMEraseBKGnd);
begin
if FirstTime then inherited
else Msg.Result:=1;
FirstTime:=False;
end;


procedure TXEDControl.HandleKeyPress(Sender: TObject; var Key: Char);
begin
if Assigned(FOnKeyPress) then FOnKeyPress(Self, Key);
end;


procedure TXEDControl.HandleKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
end;


procedure TXEDControl.AdjustComponents(Width,Height: integer);
var W,H: integer;
begin
if VScrollBar.Visible then W:=Width-VScrollBar.Width else W:=Width;
if HScrollBar.Visible then H:=Height-HScrollBar.Height else H:=Height;

ScrollButton.SetBounds(W,Height-ScrollButton.Height,VScrollBar.Width,ScrollButton.Height);
VScrollBar.SetBounds(W,0,VScrollBar.Width,Height-ScrollButton.Height);
HScrollBar.SetBounds(0,H,W,HScrollBar.Height);
Screen.SetBounds(0,0,W,H);
end;


procedure ShowComponent(C: TWinControl; Show: boolean);
var SW: word;
begin
C.Visible:=Show;
if (csDesigning in C.ComponentState) then
	begin
	if Show then SW:=SW_SHOW
	else SW:=SW_HIDE;
	TScrollBar(C).Ctl3D:=Show;
	ShowWindow(C.Handle,SW);
	end;
end;


procedure TXEDControl.SetScrollBars(V: TScrollStyle);
begin
if FScrollBars<>V then
	begin
	FScrollBars:=V;
	ShowComponent(HScrollBar,(V=ssBoth) or (V=ssHorizontal));
	ShowComponent(VScrollBar,(V=ssBoth) or (V=ssVertical));
	ShowComponent(ScrollButton,(V=ssBoth) or (V=ssVertical));
	AdjustComponents(Width,Height);
	end;
end;



constructor TXEDControl.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FirstTime:=True;
Width:=185;
Height:=89;
Enabled:=True;
Visible:=True;
FScrollBars:=ssBoth;

Screen:=TXEDScreen.Create(Self);
Screen.Parent:=Self;
Screen.TabStop:=True;
Screen.OnUpdate:=HandleUpdate;
Screen.OnKeyPress:=HandleKeyPress;
Screen.OnKeyDown:=HandleKeyDown;

ControlStyle :=ControlStyle-[csSetCaption]+[csOpaque];
HScrollBar:=TScrollBar.Create(Self);
HScrollBar.Parent:=Self;
HScrollBar.Kind:=sbHorizontal;
HScrollBar.SetParams(0,0,255);
HScrollBar.OnScroll:=ScrollHorz;
HScrollBar.TabStop:=False;

VScrollBar:=TScrollBar.Create(Self);
VScrollBar.Parent:=Self;
VScrollBar.Kind:=sbVertical;
VScrollBar.SetParams(0,0,32767);
VScrollBar.OnScroll:=ScrollVert;
VScrollBar.TabStop:=False;

ScrollButton:=TSpinButton.Create(Self);
ScrollButton.Parent:=Self;
ScrollButton.OnDownClick:=ScrollScrnDown;
ScrollButton.OnUpClick:=ScrollScrnUp;
ScrollButton.TabStop:=False;
end;


procedure TXEDControl.Change(ScreenChange: boolean);
begin
if Assigned(FOnChange) then FOnChange(Self,ScreenChange);
end;

procedure TXEDControl.HandleUpdate(Sender: TObject;ScreenChange: boolean);
begin
VScrollBar.Position:=trunc(32767*Screen.GetRelPosition);
if Screen.ScrollOffset>32767 then HScrollBar.Position:=32767
else HScrollBar.Position:=Screen.ScrollOffset;
Change(ScreenChange);
end;

procedure TXEDControl.ScrollScrnUp(Sender: TObject);
begin
Screen.DoScrollUp;
Screen.UpdateDisplay(False);
end;

procedure TXEDControl.ScrollScrnDown(Sender: TObject);
begin
Screen.DoScrollDown;
Screen.UpdateDisplay(False);
end;

procedure TXEDControl.ScrollHorz(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
with Screen do
case ScrollCode of
 scLineUp: DoScrollLeft;
 scLineDown: DoScrollRight;
 scPageUp:
	begin
	Dec(ScrollOffset,CharWidth-GuardBand);
	if ScrollOffset<0 then ScrollOffset:=0;
	end;
 scPageDown:
 	begin
	Inc(ScrollOffset,CharWidth-GuardBand);
	if ScrollOffset>255 then ScrollOffset:=255;
	end;

 scTrack: ScrollOffset:=ScrollPos;
 end;
ScrollPos:=Screen.ScrollOffset;
Screen.UpdateDisplay(False);
end;

procedure TXEDControl.ScrollVert(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);

	procedure Track;
	var C: integer;
	var T: extended;
	begin
	C:=Screen.CurColumn;
	T:=Screen.GetUsedSpace*(ScrollPos/32767);
	Screen.MoveToLocation(trunc(T));
	Screen.StartOfLine;
	Screen.DoColumnMove(C);
	end;
begin
case ScrollCode of
 scLineUp: Screen.DoScrollUpLine;
 scLineDown: Screen.DoScrollDownLine;
 scPageUp:  Screen.DoPageUp;
 scPageDown: Screen.DoPageDown;
 scTrack: Track;
 end;
Screen.CursorMoveMarking;
Screen.UpdateDisplay(False);
ScrollPos:=trunc(32767*Screen.GetRelPosition);
end;


destructor TXEDControl.Destroy;
begin
inherited Destroy;
end;


procedure TXEDControl.WMSize(var Message: TWMSize);
begin
inherited;
AdjustComponents(Message.Width,Message.Height);
end;


procedure TXEDControl.DoEnter;
begin
inherited DoEnter;
Screen.SetFocus;
end;



end.
