unit Xedscrn;

interface

uses
   WinTypes, WinProcs, Messages, Classes, Graphics, Controls, Menus,
   SysUtils,Clipbrd,HugeMem,HugeArry,DSupport;

const MaxBufferSize = 16000000;	{ Maximum buffer allowed size }
const BufferIncrement = 10000;	{ Size increase buffer on insert}
const MinimumFreeSize = 1000;	{ Minimum size upon insert}
const
 CtrlA= #$01;	CtrlB= #$02;	CtrlC= #$03;	CtrlD= #$04;
 CtrlE= #$05;	CtrlF= #$06;	CtrlG= #$07;	CtrlH= #$08;
 CtrlI= #$09;	CtrlJ= #$0A;	CtrlK= #$0B;	CtrlL= #$0C;
 CtrlM= #$0D;	CtrlN= #$0E;	CtrlO= #$0F;	CtrlP= #$10;
 CtrlQ= #$11;	CtrlR= #$12;	CtrlS= #$13;	CtrlT= #$14;
 CtrlU= #$15;	CtrlV= #$16;	CtrlW= #$17;	CtrlX= #$18;
 CtrlY= #$19;	CtrlZ= #$1A;


const
 CtrlRtBrace= #$1C;  CtrlRtBrkt= #$1D;
 CtrlCaret= #$1E;    CtrlUScore= #$1F;


Const
 SP_CHAR=#$20;  ESC_CHAR=#$1B;  EOF_CHAR=#$1A;  FF_CHAR=#$0C;
 CR_CHAR=#$0D;  LF_CHAR=#$0A;   TAB_CHAR=#$09;

type
 TXEDError=(xeNone,xeFull,xeStart,xeEnd);


TXedChange = procedure(Sender: TObject; ScreenUpdate: boolean) of object;


type
 TXEDScreen = class(TCustomControl)
 private
  FCursorWidth: integer;
  ScreenStart: longint;
  RefBitmap: TBitmap;
  LeftMouse: boolean;
  CursorMarking: boolean;
  MarkStart: longint;
  DefFixedLine: integer;
  OldBlinkRate: integer;
  FBlitRefresh: boolean;
  FFixedScroll,FOldScrlMode: boolean;
  FLinearHScroll: boolean;
  FOnUpdate: TXedChange;
  FBufSize: longint;
  FLineBreak: char;
  FSingleLine: boolean;
  FShowAll: boolean;
  FGutterSize: integer;
  ForcedModified: boolean;
  WordMarked: boolean;
  FWorkColumn: longint;
  FIsModified: boolean;
  FControlColor,FFontColor,FScreenColor,FHiLiteColor: TColor;
  procedure DoOnUpdate(ScreenChange: boolean);
  procedure WMKillFocus(var Message:TWMKillFocus); message WM_KILLFOCUS;
  procedure WMSetFocus(var Message:TWMSetFocus); message WM_SETFOCUS;
  procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  procedure WMSize(var Message: TWMSize); message WM_SIZE;
  procedure SetCursorWidth(V: integer);
  procedure SetControlColor(V: TColor);
  procedure SetFontColor(V: TColor);
  procedure SetScreenColor(V: TColor);
  procedure SetHiLiteColor(V: TColor);
  procedure SetSingleLine(V: boolean);
  procedure SetShowAll(V: boolean);
  procedure SetBufSize(V: longint);
  procedure SetScrollLine(Offset: integer);
  procedure SetScrollMode(V: boolean);
  procedure SetLinearScroll(V: boolean);
  procedure SetGutter(V: integer);
  procedure NormalizeMarkers(var P1,P2: longint);
  procedure CanonizeMarkers(var P1,P2: longint);
  procedure MarkDifference;
  procedure MarkCursorDifference;
  procedure ScreenToBuffer(X,Y: integer);
  function CountColumns(P1,P2: longint;ExpandTabs: boolean): longint;
  procedure OptimiseHorizontal;
  procedure SetWorkLine(V: integer);
  procedure SetSizeParams;
  procedure SetFont(Value: TFont);
  function GetFont: TFont;
  procedure SetModified(V: boolean);
 protected
  OldScrollOffset: longint;
  OldWorkLine: integer;
  OldGap,OldInx1,OldInx2,OldScreenStart: Longint;
  OldMark1,OldMark2: longint;
  LastLine: integer;
  procedure Paint; override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure CreateTextCursor;
 public
  CharWidth, CharHeight: integer;
  GuardBand: integer;
  Mark1,Mark2: longint;
  WorkLine: integer;
  CurColumn: longint;
  CurPos: TPoint;
  Buffer: THugeChar;
  Inx1,Inx2: longint;
  ScrollOffSet: longint;
  AutoUpdate: boolean;
  OverWrite: Boolean;
  MoveMarking: boolean;
  MarkHyphenated: boolean;
  Reallocating: boolean;
  Error: TXEDError;

  property Canvas;

  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;

  procedure MarkWord;
  procedure ResetMarkers;
  procedure CursorMoveMarking;
  procedure DeleteMarked;
  procedure GetMarkerIndexes(var I1,I2,I3,I4: longint);
  function MoveToMarkerEnd: longint;
  function MoveToMarkerBegin: longint;

  procedure MarkedToClipBoard;
  procedure InsertClipBoard;

  function IsWordBreak(C: Char): boolean;
  function IsWordBreakEx(C,Adj: Char): boolean;
  procedure UpdateCurColumn;
  function GetCurrentColumn: longint;
  procedure UpdateDisplay(ForceUpdate: boolean);
  procedure OptimiseDisplay;
  function DoScrollUp: boolean;
  function DoScrollDown: boolean;
  procedure DoScrollRight;
  procedure DoScrollLeft;
  function GetUsedSpace: longint;
  function GetRelPosition: double;
  function MoveToLocation(Loc: longint): boolean;
  function DoInsertChar(C: char): boolean;
  function DoInsertString(S: ShortString): boolean;
  function GetHighlightString: ShortString;
  function DoBackDelete: boolean;
  function DoBackChar: boolean;
  function DoBackLine: boolean;
  function DoForwardChar: boolean;
  function DoForwardLine: boolean;
  function DoColumnMove(WC: longint) : longint;
  function DoUpLine: boolean;
  procedure DoScrollUpLine;
  function DoDownLine: integer;
  procedure DoScrollDownLine;
  procedure DoBackAll;
  procedure DoForwardAll;
  procedure DoPageUp;
  procedure DoPageDown;
  function DoForwardDelete: boolean;
  procedure DoDelBackLine;
  procedure DoDelForwardLine;
  function DoOverwriteChar(C: char): boolean;
  procedure DoGotoLine(Line: integer);

  procedure ClearBuffer;
  procedure InsertChar(C: char);
  procedure DeleteChar;
  procedure ForwardDelete;
  procedure DelForwardLine;
  procedure DelBackLine;
  procedure BackChar;
  procedure BackLine;
  procedure ForwardChar;
  procedure ForwardLine;
  procedure StartOfLine;
  procedure DownLine;
  procedure ScrollDownLine;
  Procedure UpLine;
  procedure ScrollUpLine;

  procedure BackAll;
  procedure ForwardAll;
  procedure PageDown;
  procedure PageUp;
 published
  property IsModified: Boolean read FIsModified write SetModified default False;
  property ControlColor: TColor read FControlColor write SetControlColor default clRed;
  property FontColor: TColor read FFontColor write SetFontColor default clBlack;
  property ScreenColor: TColor read FScreenColor  write SetScreenColor default clWhite;
  property HiLiteColor: TColor read FHiLiteColor write SetHiLiteColor default clNavy;
  property ShowAll: boolean read FShowAll write SetShowall default false;
  property Font: TFont read GetFont write SetFont;
  property GutterSize: integer read FGutterSize write SetGutter default 5;
  property BufSize: longint read FBufSize write SetBufSize default 1024;
  property SingleLine: boolean read FSingleLine write SetSingleLine default False;
  property Align;
  property OnUpdate: TXedChange read FOnUpdate write FOnUpdate;
  property FixedScroll: boolean read FFixedScroll write SetScrollMode default False;
  property BlitRefresh: boolean read FBlitRefresh write FBlitRefresh default True;
  property LinearHScroll: boolean read FLinearHScroll write SetLinearScroll default False;
  property CursorWidth: integer read FCursorWidth write SetCursorWidth default 3;
  property OnKeyPress;
  property OnKeyDown;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
 end;

procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TXEDScreen]);
end;

{------------------- Overridden Setup Methods  ---------------------}

constructor TXEDScreen.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
DefFixedLine:=4;
ControlStyle :=ControlStyle-[csSetCaption]+[csOpaque];
Color:=clWhite;
Width:=185;
Height:=89;
FGutterSize:=5;
Canvas.Font.Pitch:=fpFixed;
Canvas.Font.Name:='FixedSys';
RefBitmap := TBitmap.Create;
RefBitmap.Width:=GetSystemMetrics(SM_CXFULLSCREEN);
RefBitmap.Height:=GetSystemMetrics(SM_CYFULLSCREEN);
FBufSize:=1024;
Buffer:=THugeChar.Create(FBufSize);
GuardBand:=4;
ClearBuffer;
Cursor:=crIBeam;
FFixedScroll:=false;
FLinearHScroll:=False;
FBlitRefresh:=True;
FControlColor:=clRed;
FFontColor:=clBlack;
FScreenColor:=clWhite;
FHiLiteColor:=clNavy;
CursorMarking:=False;
FLineBreak:=CR_CHAR;
FSingleLine:=False;
FShowAll:=False;
AutoUpdate:=True;
OverWrite:=False;
ForcedModified:=False;
MoveMarking:=False;
MarkHyphenated:=False;
LeftMouse:=False;
WordMarked:=False;
Error:=xeNone;
FWorkColumn:=-1;
FCursorWidth:=3;
Reallocating:=False;
end;


destructor TXEDScreen.Destroy;
begin
RefBitmap.Free;
Buffer.Free;
inherited Destroy;
end;

{------------------- Routines to read/write object properties ----------------}


procedure TXEDScreen.SetGutter(V: integer);
begin
if V<>FGutterSize then
	begin
	FGutterSize:=V;
	UpdateDisplay(True);
	end;
end;



procedure TXEDScreen.SetBufSize(V: longint);
var OldStart,OldSize: longint;
begin
if V<1 then V:=1;
if V<>FBufSize then
	begin
	Reallocating:=True;
	OldSize:=FBufSize;
	OldStart:=Inx1;
	DoForwardAll;
	try
	Buffer.ReAllocMem(V);
	finally
	 FBufSize:=Buffer.MaxItems;
	 if V<FBufSize then FBufSize:=V;
	 Inx2:=FBufSize;
	 if Inx1>FBufSize then Inx1:=FBufSize;
	 if OldStart<FBufSize then MoveToLocation(OldStart);
	 if (Inx1=0) and (Inx2=FBufSize) then OldGap:=Inx2-Inx1;
	 if ((Inx1>0) or (Inx2<FBufSize)) and (FBufSize<OldSize) then UpdateDisplay(True);
	 Reallocating:=False;
	 end;
	end;
end;



procedure TXEDScreen.SetScrollMode(V: boolean);
{Used to set scrolling property }
var O: boolean;
begin
O:=FFixedScroll;
FFixedScroll:=V;
if not O and V then
	begin
	SetWorkLine(DefFixedLine);
	UpdateDisplay(False)
	end;
end;



procedure TXEDScreen.SetLinearScroll(V: boolean);
{Used to set linear horizonal scrolling }
var O: boolean;
begin
O:=FLinearHScroll;
FLinearHScroll:=V;
if O<>V then UpdateDisplay(False);
end;


procedure TXEDScreen.SetShowAll(V: boolean);
begin
if V<>FShowAll then
	begin
	FShowAll:=V;
	OptimiseDisplay;
	UpdateDisplay(True);
	end;
end;


procedure TXEDScreen.SetSingleLine(V: boolean);
begin
if V<>FSingleLine then
	begin
	FSingleLine:=V;
	if FSingleLine then FLineBreak:=#$7F
	else FLineBreak:=CR_CHAR;
	OptimiseDisplay;
	end;
end;


function TXEDScreen.GetFont: TFont;
{ Set current Font }
begin
Result:=Canvas.Font;
end;


procedure TXEDScreen.SetFont(Value: TFont);
{ Set current Font }
begin
Canvas.Font.Assign(Value);
SetSizeParams;
DestroyCaret;
CreateTextCursor;
UpdateDisplay(True);
end;



procedure TXEDScreen.SetControlColor(V: TColor);
begin
if V<>FControlColor then
	begin
	FControlColor:=V;
	UpdateDisplay(True);
	end;
end;



procedure TXEDScreen.SetFontColor(V: TColor);
begin
if V<>FFontColor then
	begin
	FFontColor:=V;
	UpdateDisplay(True);
	end;
end;



procedure TXEDScreen.SetScreenColor(V: TColor);
begin
if V<>FScreenColor then
	begin
	FScreenColor:=V;
	UpdateDisplay(True);
	end;
end;



procedure TXEDScreen.SetHiLiteColor(V: TColor);
begin
if V<>FHiLiteColor then
	begin
	FHiLiteColor:=V;
	UpdateDisplay(True);
	end;
end;



procedure TXEDScreen.SetModified(V: Boolean);
begin
OldGap:=Inx2-Inx1;
FIsModified:=V;
end;

{---------- Routines to handle Windows messages -------------}

procedure TXEDScreen.WMGetDlgCode(var Message: TWMGetDlgCode);
{This block of code is required so Edit Screen recieves arrows and tab keys}
begin
inherited;
Message.Result:=Message.Result or
	DLGC_WANTALLKEYS or DLGC_WANTTAB or DLGC_WANTARROWS;
end;



procedure TXEDScreen.WMSetFocus(var Message:TWMSetFocus);
begin
inherited;
OldBlinkRate:=GetCaretBlinkTime;
CreateTextCursor;
end;


procedure TXEDScreen.WMKillFocus(var Message:TWMKillFocus);
begin
inherited;
SetCaretBlinkTime(OldBlinkRate);
DestroyCaret;
end;


procedure TXEDScreen.SetSizeParams;
var tm: TTextMetric;
begin
GetTextMetrics(Canvas.Handle,tm);
CharHeight:=ClientHeight div tm.tmHeight;
CharWidth:=Round(ClientWidth / tm.tmAveCharWidth);
if CharHeight<4 then DefFixedLine:=CharHeight div 2
else DefFixedLine:=CharHeight-4;
SetWorkLine(DefFixedLine);
OptimiseHorizontal;
end;



procedure TXEDScreen.WMSize(var Message: TWMSize);
begin
inherited;
SetSizeParams;
end;


{--------------------- Published Event handlers --------------------------}

procedure TXEDScreen.DoOnUpdate(ScreenChange: boolean);
begin
if Assigned(FOnUpdate) then FOnUpdate(Self,ScreenCHange);
end;


{-------------------- Support for text marking ---------------------------}


procedure TXEDScreen.ResetMarkers;
begin
Mark1:=0;Mark2:=0;
end;


procedure TXEDScreen.MarkDifference;
{Mark the text between MarkStart and Inx1 or Inx2}
var DX: longint;
begin
if Inx1>MarkStart then begin Mark1:=MarkStart; Mark2:=Inx1; end
else
	begin
	DX:=MarkStart-Inx1;
	Mark1:=Inx2;
	Mark2:=Inx2+DX;
	end;
NormalizeMarkers(Mark1,Mark2)
end;


procedure TXEDScreen.DeleteMarked;
{Delete characters between markers}
var M1,M2: longint;
begin
if Mark1=Mark2 then exit;
M1:=Mark1; M2:=Mark2;
CanonizeMarkers(M1,M2);
if M2<Inx1 then
	begin
	Buffer.BlockMove(M2,M1,Inx1-M2);
	Inx1:=Inx1-(M2-M1);
	end
else if M1>Inx2 then
	begin
	Buffer.BlockMove(Inx2,Inx2+(M2-M1),M1-Inx2);
	Inx2:=Inx2+(M2-M1);
	end
else
	begin
	if M1<Inx1 then Inx1:=M1;
	if M2>Inx2 then Inx2:=M2;
	end;

ResetMarkers;
MarkStart:=Inx1;
DoOnUpdate(False);
end;


procedure TXEDScreen.MarkCursorDifference;
{Mark screen based on change in cursor position}
begin
if not CursorMarking then MarkStart:=OldInx1;
CursorMarking:=True;
MarkDifference;
end;


procedure TXEDScreen.CursorMoveMarking;
{Handling marking for simple cursor movements}
begin
If ((GetAsyncKeyState(VK_SHIFT) and $8000)=0) and not MoveMarking then
	begin
	ResetMarkers; CursorMarking:=False;
	end
else MarkCursorDifference;
end;


procedure TXEDScreen.ScreenToBuffer(X,Y: integer);
{Convert Screen X,Y to new Inx1 and Inx2 positions}
var DX,DY,VP,P: integer;
var tm: TTextMetric;
begin
GetTextMetrics(Canvas.Handle,tm);
DX:=round(((X)/tm.tmAveCharWidth));

VP:=round(((Y)/tm.tmHeight)-0.5);
DY:=VP-round(CurPos.Y/tm.tmHeight);

StartOfLine;
if DY<0 then for P:=1 to abs(DY) do DoBackLine
else if DY>0 then for P:=1 to DY do DoForwardLine;
DoColumnMove(DX+ScrollOffset);
if not FFixedScroll then
  if VP>LastLine then SetWorkLine(LastLine) else
	begin
	if VP>=CharHeight then SetWorkLine(CharHeight-1)
	else SetWorkLine(VP);
	end;
end;


{------------------- Overridden Mouse Movement Methods -------------}


function IsExtendedAlpha(C: char): boolean;
begin
Result:=((C>=#$C0) and (C<=#$C6)) or ((C>=#$C8) and (C<=#$CF)) or
	((C>=#$D0) and (C<=#$D6)) or ((C>=#$D8) and (C<=#$DF)) or
	((C>=#$E0) and (C<=#$E6)) or ((C>=#$E8) and (C<=#$EF)) or
	((C>=#$F0) and (C<=#$F6)) or ((C>=#$F8) and (C<=#$FF));
end;



function TXEDScreen.IsWordBreak(C: Char): boolean;
begin
Result:=(C<>'_') and (C<>'''') and
	((C<'0') or ((C>'9') and (C<'A')) or
	((C>'Z') and (C<'a')) or (C>'z'));

Result:=Result and not IsExtendedAlpha(C);
if MarkHyphenated then Result:=Result and (C<>'-');
end;



function TXEDScreen.IsWordBreakEx(C,Adj: Char): boolean;
{Test for normal word break, plus differentiate between}
{appostrophy and single quote by looking at the adjacent char}
begin
Result:=True;
if (C='''') and not IsAlphaNumeric(Adj) then exit;
Result:=IsWordBreak(C);
end;




procedure TXEDScreen.MarkWord;
var P: integer;
var C1,C2,C3: char;
begin
{Move backward to find beginning of word}
While True do
	begin
	if Inx1<1 then break;
	C1:=Buffer.Value[Inx1-1];
	if Inx1-2>=0 then C2:=Buffer.Value[Inx1-2] else C2:=' ';
	if Inx2<FBufSize then C3:=Buffer.Value[Inx2] else C3:=' ';
	{Handle single quote vs apostrophe}
	if IsWordBreakEx(C1,C2) then break;
	if IsWordBreakEx(C1,C3) then break;
	DoBackChar;
	end;
P:=Inx1;

{Move forward to find end of word}
While True do
	begin
	if Inx2>=FBufSize then break;
	C1:=Buffer.Value[Inx2];
	if Inx1-1>=0 then C3:=Buffer.Value[Inx1-1] else C3:=' ';
	if Inx2+1<FBufSize then C2:=Buffer.Value[Inx2+1] else C2:=' ';
	{Handle single quote vs apostrophe}
	if IsWordBreakEx(C1,C2) then break;
	if IsWordBreakEx(C1,C3) then break;
	DoForwardChar;
	end;

MarkStart:=P;
MarkDifference;
WordMarked:=True;
end;



procedure TXEDScreen.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
inherited MouseDown (Button, Shift, X, Y);
if (ssRight in Shift) then exit;
if (ssLeft in Shift) then LeftMouse:=True;
SetFocus;
X:=X-FGutterSize;
ScreenToBuffer(X,Y);
if ssDouble in Shift then MarkWord
else if ssShift in Shift then MarkCursorDifference
else begin ResetMarkers; MarkStart:=Inx1 end;
FWorkColumn:=-1;
FOldScrlMode:=FFixedScroll;
FFixedScroll:=False;
UpdateDisplay(False);
end;


function IsLButtonDown: boolean;
begin
if boolean(GetSystemMetrics(SM_SWAPBUTTON)) then Result:=(GetAsyncKeyState(VK_RBUTTON) and $8000)<>0
else Result:=(GetAsyncKeyState(VK_LBUTTON) and $8000)<>0;
end;


procedure TXEDScreen.MouseMove(Shift: TShiftState; X, Y: Integer);
var PT: TPoint;

	procedure MMove(X,Y: integer);
	begin
	X:=X-FGutterSize;
	ScreenToBuffer(X,Y);
	MarkCursorDifference;
	if WorkLine<0 then SetWorkLine(0)
	else if WorkLine>CharHeight then SetWorkLine(CharHeight-1);
	UpdateDisplay(False);
	end;


	procedure Pause(T: dword);
	begin
	T:=T+GetTickCount;
	repeat until GetTickCount>=T;
	end;


	procedure ScrollDelay;
	var T,DY: integer;
	begin
	if PT.Y<0 then DY:=abs(PT.Y)
	else DY:=PT.Y-ClientHeight;
	if DY>25 then T:=0 else T:=75;
	Pause(T);
	end;

begin
inherited MouseMove(Shift,X,Y);
if not LeftMouse or WordMarked then exit;
FWorkColumn:=-1;
while True do
	begin
	GetCursorPos(PT);
	PT:=ScreenToClient(PT);
	MMove(PT.X,PT.Y);
	if not IsLButtonDown then break;
	if ((PT.Y>0) and (PT.Y<ClientHeight)) then break;
	ScrollDelay;
	end;
end;


procedure TXEDScreen.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseUp(Button,Shift,X,Y);
FWorkColumn:=-1;
WordMarked:=False;
LeftMouse:=False;
FixedScroll:=FOldScrlMode;
end;

{-------------- Text Marker Support Routines ----------------------------}

procedure TXEDScreen.NormalizeMarkers(var P1,P2: longint);
{Convert markers to cursor independent form}
var Gap: longint;
begin
Gap:=Inx2-Inx1;
if P2<=Inx1 then exit;
P2:=P2-Gap;
if P1>=Inx1 then P1:=P1-Gap;
end;


procedure TXEDScreen.CanonizeMarkers(var P1,P2: longint);
{Convert markers to cursor relative form}
var Gap: longint;
begin
Gap:=Inx2-Inx1;
if P2<Inx1 then exit;
P2:=P2+Gap;
if P1>=Inx1 then P1:=P1+Gap;
end;



procedure TXEDScreen.GetMarkerIndexes(var I1,I2,I3,I4: longint);
{Get canonized marker index set}
var M1,M2: longint;
begin
I1:=0; I2:=0; I3:=0; I4:=0;
if Mark1=Mark2 then exit;
M1:=Mark1; M2:=Mark2;
CanonizeMarkers(M1,M2);
if (M2<Inx1) then begin I1:=M1; I2:=M2; end
else if (M1>Inx2) then begin I3:=M1; I4:=M2; end
else	begin
	if M1<Inx1 then begin I1:=M1; I2:=Inx1; end;
	if M2>Inx2 then begin I3:=Inx2; I4:=M2; end;
	end;
end;


function TXEDScreen.MoveToMarkerEnd: longint;
{Move cursor to end of marked block, returns size of marked}
begin
Result:=Mark2-Mark1;
if Result=0 then exit;
MoveToLocation(Mark2);
end;


function TXEDScreen.MoveToMarkerBegin: longint;
{Move cursor to begining of marked block, returns size of marked}
begin
Result:=Mark2-Mark1;
if Result=0 then exit;
MoveToLocation(Mark1);
end;

{-------------- Handle Cursor Creation -----------------------------------}


procedure TXEDScreen.CreateTextCursor;
{Create the cursor we are using}
var tm: TTextMetric;
begin
GetTextMetrics(Canvas.Handle,tm);
CreateCaret(Handle,0,FCursorWidth,tm.tmHeight);
SetCaretPos(CurPos.X,CurPos.Y);
SetCaretBlinkTime(500);
ShowCaret(Handle);
end;


procedure TXEDScreen.SetCursorWidth(V: integer);
{Change the cursor width }
begin
if V<>FCursorWidth then
	begin
	DestroyCaret;
	FCursorWidth:=V;
	CreateTextCursor;
	end;
end;



{-------------- Support Routines For Display Udate------------------------}

procedure TXEDScreen.SetWorkLine(V: integer);
begin
if FSingleLine then WorkLine:=0
else WorkLine:=V;
end;


procedure TXEDScreen.SetScrollLine(Offset: integer);
{Set the current scrolling line }
begin
if FFixedScroll or FSingleLine then exit;
WorkLine:=WorkLine+Offset;
if WorkLine<0 then WorkLine:=0
else if WorkLine>=CharHeight then WorkLine:=CharHeight-1;
end;


procedure TXEDScreen.ClearBuffer;
begin
SetWorkLine(DefFixedLine);
Inx1:=0;
Inx2:=FBufSize;
CurPos.X:=0;CurPos.Y:=0;
OldScrollOffset:=0;
OldWorkLine:=0;
OldScreenStart:=0;
OldGap:=FBufSize;
OldInx1:=-1;
OldInx2:=FBufSize+1;
OldMark1:=0;OldMark2:=0;
end;


function TXEDScreen.GetUsedSpace: longint;
{Returns amount of buffers space used}
begin
Result:=(FBufSize -(Inx2 -Inx1));
end;


function TXEDScreen.GetRelPosition: double;
{Get position relative to text in buffer}
var U: longint;
begin
U:=GetUsedSpace;
if U=0 then Result:=0 else Result:=Inx1/U;
end;


function TXEDScreen.CountColumns(P1,P2: longint; ExpandTabs: boolean): longint;
{Count columns between P1 and P2}
var P: longint;
begin
Result:=0;
for P:=P1 to P2-1 do
 if (Buffer.Value[P]=TAB_CHAR) and ExpandTabs then  Result:=(Result or 7)+1
 else Inc(Result);
end;


procedure TXEDScreen.UpdateCurColumn;
{ Update column calculation }
{in case display hasn't been updated}
var P: longint;
begin
for P:=Inx1 downto 0 do if (P<1) or (Buffer.Value[P-1]=FLineBreak) then break;
CurColumn:=CountColumns(P,Inx1,True);
end;


function TXEDScreen.GetCurrentColumn: longint;
{Get current editor column}
begin
UpdateCurColumn;
Result:=CurColumn;
end;



procedure TXEDScreen.UpdateDisplay(ForceUpdate: boolean);
var WP: longint;		{Working index into buffer}
var LinesBefore: longint;	{Lines before cursor}
var StartOfLine: longint;	{Index to of the current line}
var CanMark1,CanMark2: longint;	{Canonized markers }
var MarkCol1,MarkCol2: longint;	{Marker columns for individual lines}
var BackCan: TCanvas;		{Background canvas for flicker reduction}
var UpdateLine,UpdateScreen: boolean;
var S: ShortString;
var tm: TTextMetric;
var SpaceMarker: char;

	procedure WriteLine(X,Y: integer; Col: longint;S: ShortString;Clear: boolean);
	{Write string in S  to window @ X,Y}
	{Clear = clear to edge of window}
	var P,MP: integer;
	var PT: TPoint;
	var C: Char;
	var OutStr: ShortString;
	var CharMode,OldCharMode: integer;
	const cmNormal=0;cmControl=1;cmInvert=8;
	const cmNotNormal=cmNormal or cmInvert;
	const cmNotControl=cmControl or cmInvert;
	begin
	OutStr:='';
	OldCharMode:=$7FFF;
	PT.X:=X; PT.Y:=Y;
	BackCan.MoveTo(X,Y);
	for P:=1 to Length(S) do
		begin
		C:=S[P];
		if C<#$20 then begin C:=char(byte(C)+$40); CharMode:=cmControl end
		else CharMode:=cmNormal;
		MP:=(P-1)+Col;
		if (MP>=MarkCol1) and (MP<MarkCol2) then CharMode:=CharMode or cmInvert;
                if CharMode<>OldCharMode then
			begin
			if OutStr<>'' then
				begin
				BackCan.TextOut(PT.X,PT.Y,OutStr);
				GetCurrentPositionEx(BackCan.Handle,@PT);
				PT.X:=PT.X-tm.tmOverhang;
				OutStr:='';
				end;
		 	case CharMode of
		  	 cmNormal:
				begin
				BackCan.Font.Color:=FFontColor;
				BackCan.Brush.Color:=FScreenColor;
				end;
		  	 cmControl:
				begin
				BackCan.Font.Color:=clWhite;
				BackCan.Brush.Color:=FControlColor;
				end;
		  	 cmNotNormal:
				begin
				BackCan.Font.Color:=clWhite;
				BackCan.Brush.Color:=FHiLiteColor;
				end;
		  	 cmNotControl:
				begin
				BackCan.Font.Color:=clWhite xor clWhite;
				BackCan.Brush.Color:=FControlColor xor clWhite;
				end;
			 end;
			end;
		OutStr:=OutStr+C;
		OldCharMode:=CharMode;
		end;
	if OutStr<>'' then
		begin
		BackCan.TextOut(PT.X,PT.Y,OutStr);
		GetCurrentPositionEx(BackCan.Handle,@PT);
		PT.X:=PT.X-tm.tmOverhang;
		OutStr:='';
		end;

	BackCan.Brush.Color:=FScreenColor;
        if Clear then
		BackCan.FillRect(Rect(PT.X, PT.Y,PT.X+ClientWidth,PT.Y+tm.tmHeight));
	end;


 	function ClipLeading(var P1,P2: longint): longint;
	{Clip off leading character scrolled off screen}
	var CharCnt: longint;
	var C: char;
	begin
	CharCnt:=0;
	while (P1<P2) and (CharCnt<ScrollOffset) do
		begin
		C:=Buffer.Value[P1];
		if C=FLineBreak then break;
		if P1=CanMark1 then MarkCol1:=CharCnt-ScrollOffset;
		if P1=CanMark2 then MarkCol2:=CharCnt-ScrollOffset;
		if C=TAB_CHAR then
			begin
			S:='';
			repeat
				begin
				if CharCnt>=ScrollOffset then S:=S+' ';
				Inc(CharCnt);
				end
			until (CharCnt and 7)=0;
			end
		else Inc(CharCnt);
		Inc(P1);
		end;
	Result:=CharCnt-Length(S);
	end;



	procedure GetLine(var P1:longint; P2,Col:longint; Clip: boolean);
	{Extract text from buffer between P1 & P2}
	{Column offset for calculating tabs is in Col}
	{Clip off beginning characters for horizontal scroll}
	var C: char;
	var PO: longint;
	begin
	PO:=P1;
	MarkCol1:=0; MarkCol2:=CharWidth+ScrollOffset;
	S:='';
        if Clip and (ScrollOffSet>0) then Col:=ClipLeading(P1,P2);
	while (P1<P2) do
		begin
		C:=Buffer.Value[P1];
		if C=FLineBreak then
			begin
			if FShowAll then S:=S+C;
			break;
			end;
		if P1=CanMark1 then
			begin
			MarkCol1:=Col+Length(S);
			if Clip then MarkCol1:=MarkCol1-ScrollOffset;
			end;
		if P1=CanMark2 then
			begin
			MarkCol2:=Col+Length(S);
			if Clip then MarkCol2:=MarkCol2-ScrollOffset;
			end;
		if (C=TAB_CHAR) and (not FShowAll) and (Length(S)<247) then
			begin
			repeat S:=S+' '
			until ((Length(S)+Col) and 7)=0;
			end
		else if (C=SP_CHAR) and FShowAll then S:=S+SpaceMarker
		else S:=S+C;
		Inc(P1);
		end;

	if (CanMark1=CanMark2) or (CanMark1>=P1) or (CanMark2<PO) then begin MarkCol1:=-1; MarkCol2:=-1; end;
 	Inc(P1);
	end;



	procedure ShowLinesBefore;
	{Show lines before current line}
	var LP: longint;
	begin
	if LinesBefore>0 then
		begin
		for LP:=0 to LinesBefore-1 do
			begin
			GetLine(WP,Inx1,0,True);
			WriteLine(FGutterSize,LP*tm.tmHeight,0,S,True);
			end;

		end;
	end;


	procedure ShowFirstHalf;
	{Show current line before cursor}
	begin
	GetLine(StartOfLine,Inx1,0,True);
	WriteLine(FGutterSize,LinesBefore*tm.tmHeight,0,S,False);
	GetCurrentPositionEx(BackCan.Handle,@CurPos);
	if S='' then CurPos.X:=CurPos.X+((CurColumn-ScrollOffset)*tm.tmAveCharWidth)
	else CurPos.X:=CurPos.X-tm.tmOverhang;
	end;


	procedure DisplayCursor;
	begin
	SetCaretPos(CurPos.X,CurPos.Y);
	ShowCaret(Handle);
	end;


	procedure ShowSecondHalf;
	begin
	{Show current line after cursor}
	WP:=Inx2;
	GetLine(WP,FBufSize,CurColumn,False);
	WriteLine(CurPos.X,CurPos.Y,CurColumn,S,True);
	end;


	procedure ShowLinesAfter;
	{Show lines after current;}
	var LP: longint;
	begin
	LP:=LinesBefore;
	while (LP<-1) and (WP<FBufSize) do
		begin
		if Buffer.Value[WP]=FLineBreak then Inc(LP);
		Inc(WP);
		end;

	while true do
		begin
		if WP>=FBufSize then break;
		Inc(LP);
		if (LP*tm.tmHeight)>ClientHeight then break;
		GetLine(WP,FBufSize,0,True);
		WriteLine(FGutterSize,LP*tm.tmHeight,0,S,True);
		end;
	BackCan.FillRect(Rect(FGutterSize, (LP+1)*tm.tmHeight,ClientWidth,ClientHeight));
	BackCan.FillRect(Rect(0,0,FGutterSize,ClientHeight));
	if (Inx2<FBufSize) and (Buffer.Value[FBufSize-1]=FLineBreak) then Inc(LP);
	LastLine:=LP;
	end;


	procedure GetScrollOffset;
	{Calculate horizontal scroll offset}
	var M: longint;
	begin
	if (OldInx1=Inx1) and not ForceUpdate then exit;
	CurColumn:=CountColumns(StartOfLine,Inx1,not FShowAll);
        If FLinearHScroll then
		begin
		if CurColumn>(CharWidth-GuardBand) then ScrollOffSet:=CurColumn-(CharWidth-GuardBand)
		else ScrollOffSet:=0;
                end
	else
		begin
		M:=CurColumn-ScrollOffset;
		if (M<GuardBand) or (M>(CharWidth-GuardBand)) then
			begin
			if CurColumn>(CharWidth-GuardBand) then ScrollOffSet:=CurColumn-(CharWidth-GuardBand);
			if M<GuardBand then ScrollOffSet:=CurColumn-GuardBand;
			end;
		if ScrollOffset<0 then ScrollOffset:=0;
		end;
	end;



	procedure CursorOffTop;
	var P: longint;
	var C: integer;
	begin
	LinesBefore:=WorkLine;
	C:=Workline;
	for P:=Inx2 to FBufSize-1 do
	 if Buffer.Value[P]=FLineBreak then
        	begin Inc(C); if C>=0 then break; end;
	ScreenStart:=P;
	end;



	procedure CursorOffBottom;
	begin
	ScreenStart:=Inx1-1;
	while true do
		begin
		if Buffer.Value[ScreenStart]=FLineBreak then
			begin
			if LinesBefore=0 then StartOfLine:=ScreenStart+1;
			if LinesBefore>=WorkLine then
				begin
				Inc(ScreenStart);
				break;
				end;
			Inc(LinesBefore);
			end;
		if ScreenStart=0 then break;
		Dec(ScreenStart);
		end;
	end;

	procedure CalcLinesBefore;
	{Calculate pointers to lines before cursor}
	begin
	LinesBefore:=0;
	StartOfLine:=0;
        if Inx1=0 then begin ScreenStart:=0; exit; end;
	if WorkLine<0 then begin CursorOffTop; exit; end;
	if WorkLine>CharHeight then begin CursorOffBottom; exit; end;

	ScreenStart:=Inx1-1;
	while true do
		begin
		if Buffer.Value[ScreenStart]=FLineBreak then
			begin
			if LinesBefore=0 then StartOfLine:=ScreenStart+1;
			if LinesBefore>=WorkLine then
				begin
				Inc(ScreenStart);
				break;
				end;
			Inc(LinesBefore);
			end;
		if ScreenStart=0 then break;
		Dec(ScreenStart);
		end;
	end;

	procedure AnalyzeChange;
	var IsHScroll,IsVScroll,IsInsert,IsDelete:boolean;
	var IsBigChange,IsCRLF1,IsCRLF2,IsCRLF3: boolean;
	var IsMarkChange: boolean;
	var Gap: longint;
	begin
	Gap:=Inx2-Inx1;
	IsInsert:=Gap<OldGap;
	IsDelete:=Gap>OldGap;
	if IsInsert or IsDelete or ForcedModified then FIsModified:=true;
	IsMarkChange:=(Mark1<>OldMark1) or (Mark2<>OldMark2);
	IsCRLF1:=(IsInsert or ForcedModified) and (Inx1>0) and (Buffer.Value[Inx1-1]=FLineBreak);
	IsCRLF2:=IsDelete and ((Inx2>OldInx2) and (Buffer.Value[OldInx2]=FLineBreak));
	IsCRLF3:=IsDelete and ((Inx1<OldInx1) and (Buffer.Value[Inx1]=FLineBreak));
	IsBigChange:=abs(Gap-OldGap)>1;
	IsHScroll:=OldScrollOffset<>ScrollOffset;
	IsVScroll:=(OldScreenStart<>ScreenStart) or
			IsCRLF1 or IsCRLF2 or IsCRLF3;
	UpdateScreen:=ForceUpdate or IsHScroll or IsVScroll or
			(WorkLine<0) or IsBigChange or IsMarkChange;
	UpdateLine:=UpdateScreen or IsInsert or IsDelete or ForcedModified;

	OldGap:=Gap;
	OldScrollOffset:=ScrollOffset;
	OldWorkLine:=WorkLine;
	OldScreenStart:=ScreenStart;
	OldInx1:=Inx1;
	OldInx2:=Inx2;
	OldMark1:=Mark1;
	OldMark2:=Mark2;
	end;



	procedure DisplayNormal;
	begin
	WP:=ScreenStart;
	if UpdateScreen then ShowLinesBefore;
	ShowFirstHalf;
	if UpdateLine then ShowSecondHalf;
	if UpdateScreen then ShowLinesAfter;
	if (not FixedScroll) and (LinesBefore<Workline) then SetWorkLine(LinesBefore);
	end;



begin
GetTextMetrics(Canvas.Handle,tm);
case Canvas.Font.Charset of
 ANSI_CHARSET,DEFAULT_CHARSET : SpaceMarker:=#$B7;
 OEM_CHARSET: SpaceMarker:=#$F9;
 else SpaceMarker:='_';
 end;

CanMark1:=Mark1; CanMark2:=Mark2;
CanonizeMarkers(CanMark1,CanMark2);

CalcLinesBefore;
GetScrollOffSet;
AnalyzeChange;

RefBitmap.Canvas.Font.Assign(Canvas.Font);
if FBlitRefresh and UpdateScreen then BackCan:=RefBitmap.Canvas
else BackCan:=Canvas;

if GetFocus=Handle then HideCaret(Handle);
DisplayNormal;

if FBlitRefresh and UpdateScreen then Canvas.Draw(0,0,RefBitmap);
if GetFocus=Handle then DisplayCursor;
ForcedModified:=False;
DoOnUpdate(True);
end;



procedure TXEDScreen.OptimiseHorizontal;
var Start,Cur,P: longint;
begin
Start:=0;
for P:=Inx1-1 downto 0 do
 if Buffer.Value[P]=FLineBreak then
	begin
	Start:=P+1;
	break;
	end;

Cur:=CountColumns(Start,Inx1,not FShowAll);
if Cur>(CharWidth-GuardBand) then ScrollOffSet:=Cur-(CharWidth-GuardBand)
else ScrollOffSet:=0;
end;


procedure TXedScreen.OptimiseDisplay;
begin
OptimiseHorizontal;
if (WorkLine<1) or (WorkLine>CharHeight-1) then SetWorkLine(DefFixedLine);
UpdateDisplay(True);
end;


procedure TXedScreen.Paint;
begin
UpdateDisplay(True);
end;


{-------------- Scrolling Operations (without cursor movements ------------}


function TXEDScreen.DoScrollUp: boolean;
begin
Result:=False;
if FixedScroll then exit;
if (WorkLine<CharHeight-1) and (ScreenStart>0) then
	begin
	SetWorkLine(WorkLine+1);
	Result:=True;
	end;
end;


function TXEDScreen.DoScrollDown: boolean;
begin
Result:=False;
if FixedScroll then exit;
if WorkLine>1 then
	begin
	SetWorkLine(WorkLine-1);
	Result:=True;
	end;
end;

procedure TXEDScreen.DoScrollRight;
begin
Inc(ScrollOffset);
end;

procedure TXEDScreen.DoScrollLeft;
begin
if ScrollOffset<=0 then exit;
Dec(ScrollOffset);
end;

{---------------- Clipboard Routines ----------------------------------}



procedure TXEDScreen.MarkedToClipBoard;
var hbuf: THandle;
var MemBlock: TMemoryBlock;
var Start,S1,S2: longint;
var P1,P2,P3,P4: longint;

	procedure MoveBytes(P1,P2: longint;var Start: longint);
	var P: longint;
	var B: byte;
	begin
	for P:=P1 to P2-1 do
		begin
		B:=byte(Buffer.Value[P]);
		MemBlock.Value[Start]:=B; Inc(Start);
		if Char(B)=CR_CHAR then
			begin
			MemBlock.Value[Start]:=byte(LF_CHAR);
			Inc(Start);
			end;
		MemBlock.TestReAlloc(Start+10);
		end;
	end;

begin
if Mark1=Mark2 then exit;
GetMarkerIndexes(P1,P2,P3,P4);
S1:=P2-P1; S2:=P4-P3;

hbuf := GlobalAlloc(GMEM_MOVEABLE,S1+S2+1);
MemBlock:=TMemoryBlock.Create(hbuf);
Start:=0;

if S1>0 then MoveBytes(P1,P2,Start);
if S2>0 then MoveBytes(P3,P4,Start);
MemBlock.Value[Start]:=0;
Clipboard.SetAsHandle(CF_TEXT, hbuf);
MemBlock.Free;
end;



procedure TXEDScreen.InsertClipBoard;
var hbuf: THandle;
var MemBlock: TMemoryBlock;
var Size,I: longint;
var C: char;
begin
hbuf := Clipboard.GetAsHandle(CF_TEXT);
if hbuf= 0 then exit;
MemBlock:=TMemoryBlock.Create(hbuf);

Size:=MemBlock.MemSize;
for I:=0 to Size-1 do
	begin
	byte(C):=MemBlock.Value[I];
	case C of
	 #0: break;
	 LF_CHAR:;
	 else if not DoInsertChar(C) then break;
	 end;
	end;
MemBlock.Free;
end;


{--------------- Basic routines to handle text in the Buffer ----------}

function TXEDScreen.MoveToLocation(Loc: longint): boolean;
{Move To Location Loc Within Text Buffer}
var	MSize : longint;
begin
Result:=False;
if Loc>GetUsedSpace then exit;
if Loc=Inx1 then  begin Result:=True; exit; end;
if Loc<Inx1 then
	begin
	MSize:= Inx1 - Loc;
	Buffer.BlockMove(Loc, Inx2-MSize, MSize);
	Inx2:= Inx2 - MSize;
	Inx1:= Loc;
	end
 else	begin
	MSize:= Loc -Inx1;
	Buffer.BlockMove(Inx2, Inx1, MSize);
	Inx1:= Inx1 +MSize;
	Inx2:= Inx2 +MSize;
	end;
Result:=True;
end;



function TXEDScreen.DoInsertChar(C: char): boolean;
var Free: integer;
begin
Result:=False;
Free:=Inx2 - Inx1;
if Free<MinimumFreeSize then
	begin
	if BufSize>MaxBufferSize then exit;
	BufSize:=BufSize+BufferIncrement;
	end;
Buffer.Value[Inx1]:=C;
Inc(Inx1);
if (C=CR_CHAR) and (WorkLine<DefFixedLine) then SetWorkLine(WorkLine+1);
Result:=True;
end;




function TXEDScreen.DoInsertString(S: ShortString): boolean;
var I: integer;
begin
Result:=False;
for I:=1 to Length(S) do if not DoInsertChar(S[I]) then exit;
Result:=True;
end;


function TXEDScreen.GetHighlightString: ShortString;
{Copy highlighted text to string}
var P,P1,P2,P3,P4: longint;
begin
Result:='';
if Mark1=Mark2 then exit;
GetMarkerIndexes(P1,P2,P3,P4);
for P:=P1 to P2-1 do Result:=Result+Buffer.Value[P];
for P:=P3 to P4-1 do Result:=Result+Buffer.Value[P];
end;


function TXEDScreen.DoBackDelete;
begin
if Inx1>0 then
	begin
	Dec(Inx1);
	Result:=True;
	end
else Result:=False;
end;


function TXEDScreen.DoForwardDelete;
begin
if Inx2<FBufSize then
	begin
	Inc(Inx2);
	Result:=True;
	end
else Result:=False;
end;



procedure TXEDScreen.DoDelBackLine;
begin
while DoBackDelete and (Inx1>0) and (Buffer.Value[Inx1-1]<>CR_CHAR) do;
end;


procedure TXEDScreen.DoDelForwardLine;
begin
while DoForwardDelete and (Inx2<BufSize) and  (Buffer.Value[Inx2-1]<>CR_CHAR) do;
end;

{---------------- Handle Moving cursor backwards -----------------------------}




function TXEDScreen.DoBackChar: boolean;
begin
if Inx1>0 then
	begin
	Dec(Inx1);Dec(Inx2);
	Buffer.Value[Inx2]:=Buffer.Value[Inx1];
	if Buffer.Value[Inx1]=CR_CHAR then SetScrollLine(-1);
	Result:=True;
	end
else Result:=False;
end;



function TXEDScreen.DoBackLine: boolean;
begin
if Inx1>0 then
	begin
	while DoBackChar do
		begin
		if Inx1=0 then break;
		if Buffer.Value[Inx1-1]=CR_CHAR then break;
		end;
	Result:=True;
	end
else Result:=False;
end;


procedure TXEDScreen.StartOfLine;
begin
while (Inx1>0) and (Buffer.Value[Inx1-1]<>FLineBreak) do
	DoBackChar;
end;


function TXEDScreen.DoUpLine;
begin
if Inx1>0 then
	begin
	StartOfLine;
	if FWorkColumn<0 then FWorkColumn:=CurColumn;
	if DoBackLine then DoColumnMove(FWorkColumn);
	Result:=True;
	end
else Result:=False;
end;




procedure TXEDScreen.DoScrollUpLine;
{Move up a line and scroll screen}
var O: boolean;
begin
O:=FFixedScroll;
FFixedScroll:=True;
DoUpLine;
FFixedScroll:=O;
end;


procedure TXEDScreen.DoPageUp;
var I: Integer;
begin
for I:=0 to CharHeight-1 do DoBackLine;
end;


procedure TXEDScreen.DoBackAll;
begin
Buffer.BlockMove(0, Inx2-Inx1, Inx1);
Inx2:=Inx2-Inx1;
Inx1:=0;
end;


{---------------- Handle Moving cursor forward -----------------------------}

function TXEDScreen.DoForwardChar: boolean;
begin
if Inx2<FBufSize then
	begin
	Buffer.Value[Inx1]:=Buffer.Value[Inx2];
	Inc(Inx1); Inc(Inx2);
	if Buffer.Value[Inx1-1]=CR_CHAR then SetScrollLine(1);
	Result:=True;
	end
else Result:=False;
end;


function TXEDScreen.DoForwardLine: boolean;
begin
if Inx2<FBufSize then
	begin
	while DoForwardChar do if Buffer.Value[Inx1-1]=CR_CHAR then break;
	Result:=True;
	end
else Result:=False;
end;


function TXEDScreen.DoColumnMove(WC: longint): longint;
{ Move Cursor Forward To Column Wc }
var C: longint;
begin
C:=0;
while C<WC do
	begin
	if Inx2>=FBufSize then break;
	if Buffer.Value[INX2]=FLineBreak then break;
	case Buffer.Value[INX2] of
	  TAB_CHAR: C:= (C  or  7);
	  end;
	DoForwardChar;
	Inc(C);
	end;
Result:=C+1;
end;


function TXEDScreen.DoDownLine: integer;
begin
if Inx2<FBufSize then
	begin
	DoForwardLine;
	if FWorkColumn<0 then FWorkColumn:=CurColumn;
	Result:=DoColumnMove(FWorkColumn);
	end
else Result:=-1;
end;


procedure TXEDScreen.DoScrollDownLine;
{Move down a line and scroll screen}
var O: boolean;
begin
O:=FFixedScroll;
FFixedScroll:=True;
DoDownLine;
FFixedScroll:=O;
end;



procedure TXEDScreen.DoPageDown;
var I: Integer;
begin
for I:=0 to CharHeight-1 do DoForwardLine;
end;


procedure TXEDScreen.DoForwardAll;
begin
Buffer.BlockMove(Inx2,Inx1, BufSize-Inx2);
Inx1:= Inx1 + BufSize - Inx2;
Inx2:= BufSize;
end;


function TXEDScreen.DoOverwriteChar(C: char):boolean;
begin
DoForwardDelete;
DoOnUpdate(False);
Result:=DoInsertChar(C);
ForcedModified:=True;
end;


procedure TXEDScreen.DoGotoLine(Line: integer);
{ Goto the specified line}
var I: integer;
begin
DoBackAll;
for I:=0 to Line-1 do
 if not DoForwardLine then break;
end;


{-------- High level Text manipulation, with display update, marking etc. ------}

procedure TXEDScreen.DeleteChar;
begin
if Mark1<>Mark2 then DeleteMarked
else if DoBackDelete then
	begin
	if OverWrite and (Buffer.Value[Inx1]<>CR_CHAR) then
		begin
		DoOnUpdate(False);
		Dec(Inx2);
		Buffer.Value[Inx2]:=' ';
		end;
	Error:=xeNone;
	end
else Error:=xeStart;
ForcedModified:=True;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ForwardDelete;
begin
if Mark1<>Mark2 then DeleteMarked
else if DoForwardDelete then Error:=xeNone
else Error:=xeEnd;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.DelBackLine;
begin
if Inx1>0 then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	ResetMarkers;
	DoDelBackLine;
	end
else Error:=xeStart;
if AutoUpdate then UpdateDisplay(False);
end;



procedure TXEDScreen.DelForwardLine;
begin
if Inx2<BufSize then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	ResetMarkers;
	DoDelForwardLine;
	end
else Error:=xeStart;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.InsertChar(C: char);
begin
Error:=xeNone;
if Mark1<>Mark2 then DeleteMarked;
if OverWrite and
   (C<>CR_Char) and
   (Inx2<FBufSize) and
   (Buffer.Value[Inx2]<>CR_CHAR) then DoOverwriteChar(C)
else if not DoInsertChar(C) then Error:=xeFull;
ForcedModified:=True;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.BackLine;
begin
if DoBackLine then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	CursorMoveMarking;
	end
else Error:=xeStart;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ForwardLine;
begin
if DoForwardLine then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	CursorMoveMarking;
	end
else Error:=xeEnd;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.BackChar;
begin
if DoBackChar then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	CursorMoveMarking;
	end
else Error:=xeStart;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ForwardChar;
begin
if DoForwardChar then
	begin
	FWorkColumn:=-1;
	Error:=xeNone;
	CursorMoveMarking;
	end
else Error:=xeEnd;

if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.Upline;
begin
if DoUpline then
	begin
	Error:=xeNone;
	CursorMoveMarking;
	end
else Error:=xeEnd;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ScrollUpLine;
{Move up a line and scroll screen}
var O: boolean;
begin
O:=FFixedScroll;
FFixedScroll:=True;
Upline;
FFixedScroll:=O;
end;



procedure TXEDScreen.DownLine;
begin
DoDownLine;
CursorMoveMarking;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ScrollDownLine;
{Move down a line and scroll screen}
var O: boolean;
begin
O:=FFixedScroll;
FFixedScroll:=True;
Downline;
FFixedScroll:=O;
end;


procedure TXEDScreen.PageDown;
begin
DoPageDown;
CursorMoveMarking;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.PageUp;
begin
DoPageUp;
CursorMoveMarking;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;



procedure TXEDScreen.BackAll;
begin
DoBackAll;
CursorMoveMarking;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;


procedure TXEDScreen.ForwardAll;
begin
DoForwardAll;
CursorMoveMarking;
OptimiseDisplay;
FWorkColumn:=-1;
if AutoUpdate then UpdateDisplay(False);
end;


end.

