unit CheckedGroup;

interface

uses Classes,Controls,StdCtrls,Graphics,Types,SysUtils,CheckBoxExt,VCLLibrary;


type TCheckedGroup = class(TGroupBox)
 private
  FCheckBox: TCheckBoxExt;
  FOnCheckClick: TNotifyEvent;
  FEnableChildren: boolean;
    FCaption: string;
  procedure CheckClick(Sender: TObject);
  procedure CMControlChange(var Message: TCMControlChange); message CM_CONTROLCHANGE;
  procedure EnableChildControls(Control: TWinControl; Enable: boolean);
  procedure SetNoEventChecked(const Value: boolean);
    procedure SetCaption(const Value: string);
 protected
  procedure Paint; override;
  procedure SetEnableChildren(V: boolean);
  procedure SetChecked(V: boolean);
  function ReadChecked: boolean;
  procedure Loaded; override;
  procedure SetEnabled(Value: Boolean); override;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  {Note: don't make the internal checkbox a property: it causes problems if you attempt copy/paste the control}
  property OnCheckClick: TNotifyEvent read FOnCheckClick write FOnCheckClick;
  property EnableChildren: boolean read FEnableChildren write FEnableChildren;
  property Checked: boolean read ReadChecked write SetChecked;
  property NoEventChecked: boolean read ReadChecked write SetNoEventChecked;
  property Caption: string read FCaption write SetCaption;
 end;

const CheckBoxName = 'InternalCheckBox';




procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TCheckedGroup]);
end;




procedure TCheckedGroup.EnableChildControls(Control: TWinControl; Enable: boolean);
{Enable/Disable child control of specified Control}
var I: Integer;
var ChildControl: TControl;
var SubEnable: boolean;
begin
for I:= 0 to Control.ControlCount -1 do
	begin
	ChildControl := Control.Controls[I];
	if ChildControl.Name=CheckBoxName then continue;
	{Handle subcontrols of other WinControls by recursing}
	if ChildControl is TWinControl then
		begin
		{Enable to the bottom only if it is CheckGroup and it is checked}
		{However, always disable all the way to the bottom of object tree}
		if (ChildControl is TCheckedGroup) and Enable then SubEnable:=TCheckedGroup(ChildControl).Checked
		else SubEnable:=Enable;
		EnableChildControls(TWinControl(ChildControl),SubEnable);
		end;
	ChildControl.Enabled:=Enable;
	end;
end;



procedure TCheckedGroup.SetEnableChildren(V: boolean);
begin
if V<>FEnableChildren then
	begin
        FEnableChildren:=V;
	if FEnableChildren then EnableChildControls(Self,FCheckBox.Checked);
	end;
end;


procedure TCheckedGroup.CheckClick(Sender: TObject);
begin
if FEnableChildren then EnableChildControls(Self,FCheckBox.Checked);
if Assigned(FOnCheckClick) then FOnCheckClick(Self);
end;



procedure TCheckedGroup.SetCaption(const Value: string);
begin
if FCaption<>Value then
	begin
	FCaption := Value;
	Invalidate;
	end;
end;

procedure TCheckedGroup.SetChecked(V: boolean);
begin
if csLoading in ComponentState then FCheckBox.NoEventChecked:=V
else FCheckBox.Checked:=V;
end;


function TCheckedGroup.ReadChecked: boolean;
begin
Result:=FCheckBox.Checked;
end;


constructor TCheckedGroup.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FCheckBox:=TCheckBoxExt.Create(Self);
FCheckBox.Name:=CheckBoxName;
FCheckBox.Parent := Self;
FCheckBox.Top:=-2;
FCheckBox.Left:=8;
FCheckBox.Visible := True;
FCheckBox.Enabled:=True;
FCheckBox.OnClick:=CheckClick;
FEnableChildren:=False;
inherited Caption:='';
end;



destructor TCheckedGroup.Destroy;
begin
FCheckBox.Free;
inherited Destroy;
end;


procedure TCheckedGroup.Paint;
{I think this works because the check box is always bigger }
{ than the real caption so it always remains hidden underneath}
var Size: TSize;
begin
FCheckBox.Caption:=Caption;
Size:=Canvas.TextExtent(Caption);
FCheckBox.Width:=22+Size.cx;
inherited Paint;
end;



procedure TCheckedGroup.Loaded;
begin
inherited Loaded;
if FEnableChildren then EnableChildControls(Self,FCheckBox.Checked);
end;


procedure TCheckedGroup.CMControlChange(var Message: TCMControlChange);
begin
inherited;
if Message.Inserting and (Message.Control<>FCheckBox) then
	begin
	if EnableChildren then Message.Control.Enabled:=FCheckBox.Checked;
	end;
end;



procedure TCheckedGroup.SetNoEventChecked(const Value: boolean);
begin
if FCheckBox.Checked<>Value then FCheckBox.NoEventChecked:=Value;
if FEnableChildren then EnableChildControls(Self,FCheckBox.Checked);
end;



procedure TCheckedGroup.SetEnabled(Value: Boolean);
begin
inherited SetEnabled(Value);
FCheckBox.Enabled:=Value;
end;



end.
