unit CollapsePanel;

interface

uses Windows,Forms,SysUtils,Messages,Graphics,StdCtrls,Types,Classes,Controls,
     ExtCtrls,SpinButtons,DSupport,Dialogs;


type TArrowButtonEx = class(TArrowButton)
 private
 protected
  procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
 public
 published
 end;



type TCaptionPanel = class(TCustomPanel)
 private
 protected
  procedure CreateParams(var params: TCreateParams); override;
 public
  FButton: TArrowButtonEx;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
 end;


type TCollapsePanel = class(TCustomPanel)
 private
  FSubPanel: TCaptionPanel;
  FAlignment: TAlignment;
   FExpanded: boolean;
  FExpandHeight: integer;
  FExpandWidth: integer;
  procedure HandleBtnClick(Sender: TObject);
  function GetText: TCaption;
  procedure SetBandColor(const Value: TColor);
  procedure SetText(const Value: TCaption);
  function GetBandColor: TColor;
  procedure HandleFontChange(Sender: TObject);
  procedure SetTopBandSize;
  procedure SetAlignment(const Value: TAlignment);
  procedure HideChildren(Hide: boolean);
  procedure DoCollapse;
  procedure DoExpand;
  procedure ExpandPanel(Expand: boolean);
  procedure AlignControls(AControl: TControl; var Rect: TRect); override;
  procedure SetExpanded(const Value: boolean);
  procedure Resize; override;
 protected
  procedure Loaded; override;
  procedure Change;
 public
  Expanding,Collapsing: boolean;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property ExpandHeight: integer read FExpandHeight write FExpandHeight;
  property ExpandWidth: integer read FExpandWidth write FExpandWidth;
  property Align;
  property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
  property Anchors;
  property AutoSize;
  property BandColor: TColor read GetBandColor write SetBandColor;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BiDiMode;
  property BorderWidth;
  property BorderStyle;
  property Caption: TCaption read GetText write SetText stored True;
  property Color;
  property Constraints;
  property Ctl3D;
  property UseDockManager default True;
  property DockSite;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property Expanded: boolean read FExpanded write SetExpanded stored True;
  property FullRepaint;
  property Font;
  property Locked;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnCanResize;
  property OnClick;
  property OnConstrainedResize;
  property OnContextPopup;
  property OnDockDrop;
  property OnDockOver;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnGetSiteInfo;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
  property OnStartDock;
  property OnStartDrag;
  property OnUnDock;
 end;


procedure Register;



implementation




procedure TArrowButtonEx.CMDesignHitTest(var Message: TCMDesignHitTest);
var P: TPoint;
begin
Message.Result := 1;
end;




{ TCaptionPanel }

constructor TCaptionPanel.Create(AOwner: TComponent);
begin
inherited;
{$IF CompilerVersion >= 17}
ParentBackground:=False;
{$IFEND}
FButton:=TArrowButtonEx.Create(Self);
FButton.Parent:=Self;
FButton.Align:=alRight;
FButton.Visible:=True;
FButton.UpArrow:=True;
end;



procedure TCaptionPanel.CreateParams(var params: TCreateParams);
begin
params.ExStyle := params.ExStyle or WS_EX_TOPMOST;
inherited CreateParams(params);
end;



destructor TCaptionPanel.Destroy;
begin
FButton.Free;
inherited;
end;


{ TCollapsePanel }

procedure Register;
begin
RegisterComponents('Fountain', [TCollapsePanel]);
end;



procedure TCollapsePanel.SetTopBandSize;
var H: integer;
begin
H:=abs(Font.Height)+10;
FSubPanel.Height:=H;
FSubPanel.FButton.Width:=H;
end;



procedure TCollapsePanel.HandleFontChange(Sender: TObject);
begin
FSubPanel.Font:=Font;
SetTopBandSize;
end;



constructor TCollapsePanel.Create(AOwner: TComponent);
begin
inherited;
ControlStyle:=ControlStyle - [csSetCaption];
Height:=100;

FSubPanel:=TCaptionPanel.Create(Self);
FSubPanel.Parent:= Self;
FSubPanel.Name:='SubPanel';
FSubPanel.Align:=alTop;
FSubPanel.Top:=0;
FSubPanel.ControlStyle:=FSubPanel.ControlStyle - [csAcceptsControls,csDesignInteractive];
FSubPanel.Visible:=True;
FSubPanel.BevelInner:=bvNone;
FSubPanel.BevelOuter:=bvLowered;
FSubPanel.BorderStyle:=bsNone;
FSubPanel.ParentCtl3D:=True;
FSubPanel.ParentFont:=True;
FSubPanel.Color:=clSkyBlue;
FSubPanel.FButton.OnClick:=HandleBtnClick;
Font.OnChange:=HandleFontChange;
SetTopBandSize;
Caption:=Name;
FExpanded:=True;

end;



destructor TCollapsePanel.Destroy;
begin
FSubPanel.Free;
inherited;
end;



procedure TCollapsePanel.HideChildren(Hide: boolean);
var I: integer;
begin
for I:=0 to Self.ControlCount-1 do
 if FSubPanel<>Self.Controls[I] then
	Self.Controls[I].Visible:=not Hide;
end;




procedure TCollapsePanel.Change;
{Tell form designer about any changes made}
{So it can update the Object Inspector properties}
{And insure that the property values get streamed }
var Form: TCustomForm;
begin
if csDesigning in ComponentState then
	begin
	Form := GetParentForm(Self);
	if (Form <> nil) and (Form.Designer <> nil) then Form.Designer.Modified;
	end;
end;


procedure TCollapsePanel.DoExpand;
begin
Expanding:=True;
try
Self.Width:=ExpandWidth;
Self.Height:=ExpandHeight;
HideChildren(False);
FSubPanel.FButton.UpArrow:=True;
finally Expanding:=False; end;
Change;
end;



procedure TCollapsePanel.DoCollapse;
begin
Collapsing:=True;
try
ExpandWidth:=Self.Width;
ExpandHeight:=Self.Height;
Self.Width:=FSubPanel.Width;
Self.Height:=FSubPanel.Height+BorderWidth * 2 + BevelWidth * 2;
HideChildren(True);
FSubPanel.FButton.UpArrow:=False;
finally Collapsing:=False; end;
Change;
end;




procedure TCollapsePanel.ExpandPanel(Expand: boolean);
begin
if Expand then DoExpand
else DoCollapse;
end;




procedure TCollapsePanel.HandleBtnClick(Sender: TObject);
begin
Expanded:=not Expanded;
end;


function TCollapsePanel.GetBandColor: TColor;
begin
Result:=FSubPanel.Color;
end;


procedure TCollapsePanel.SetBandColor(const Value: TColor);
begin
FSubPanel.Color:=Value;
end;


procedure TCollapsePanel.SetText(const Value: TCaption);
begin
FSubPanel.Caption:=Value;
end;


function TCollapsePanel.GetText: TCaption;
begin
Result:=FSubPanel.Caption;
end;


procedure TCollapsePanel.Loaded;
begin
inherited Loaded;
FSubPanel.FButton.UpArrow:=Expanded;
end;



procedure TCollapsePanel.SetAlignment(const Value: TAlignment);
begin
FAlignment:=Value;
FSubPanel.Alignment:=Value;
Invalidate;
end;



procedure TCollapsePanel.AlignControls(AControl: TControl; var Rect: TRect);
{This routine was necessary to keep the Caption Panel }
{ at the top of the window during design time operations}
begin
{Don't allow any control to be dragged higher than the top of FSubPanel}
if (AControl<>nil) and (AControl.Top<1) then AControl.Top:=1;
inherited AlignControls(AControl,Rect);
{Force SubPanel to top of collapse panel}
FSubPanel.Top:=0;
{Make sure SubPanel always clips contained controls}
FSubPanel.BringToFront;
end;



procedure TCollapsePanel.SetExpanded(const Value: boolean);
begin
if FExpanded<>Value then
	begin
	FExpanded := Value;
	if not (csLoading in ComponentState) then ExpandPanel(Value);
	end;
{Unable to make "Expanded" or "Height" property change in the Object Inspector
when clicking on the button in design mode. The underlying property changes
but the Object Inspector isn't updated. It has something to do with trapping
the "CM_DESIGNHITTEST" message at a low level in the customized button.}
//Caption:=PickString(Expanded,'Expanded','Collapsed');
end;

var ResizeBusy: boolean;

procedure TCollapsePanel.Resize;
{Capture Expanded size only if designing}
begin
inherited Resize;
if ResizeBusy then exit;
ResizeBusy:=True;
try
{Only during design time }
if not (csDesigning in ComponentState) then exit;
{And not if Expanding or Collapsing}
if Collapsing or Expanding then exit;
{Don't change size if already collapsed}
if not Expanded then
	begin
	//ShowMessage('Expand Control Before Adjusting');
	Self.Height:=FSubPanel.Height+BorderWidth * 2 + BevelWidth * 2;
	exit;
	end;

ExpandWidth:=Self.Width;
ExpandHeight:=Self.Height;
finally ResizeBusy:=False; end;
end;


end.
