unit xopendlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, extctrls, StdActns;


{ TExtOpenDialog }

type
  TExtOpenDialog = class(TOpenDialog)
  private
    FOptionPanel: TPanel;
  protected
    procedure DoClose; override;
    procedure DoShow; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean; override;
    procedure AttachControl(var C: TControl);
    procedure AttachCheckBox(var C: TCheckBox;Caption: string; Checked: boolean);
    procedure ClearControls;
  end;

{ TSavePictureDialog }

 TExtSaveDialog = class(TExtOpenDialog)
    function Execute: Boolean; override;
  end;

{ TExtPrintDlg }

type
  TExtPrintDialog = class(TPrintDialog)
  private
    FOptionPanel: TPanel;
  protected
    procedure DoClose; override;
    procedure DoShow; override;
    function GetStaticRect: TRect;
 public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AttachControl(var C: TControl);
    procedure AttachCheckBox(var C: TCheckBox;Caption: string; Checked: boolean);
  end;

procedure Register;

implementation

{$R XOpenDlg.res}

uses Consts,  CommDlg, Dlgs;

procedure Register;
begin
RegisterComponents('Fountain', [TExtOpenDialog]);
RegisterComponents('Fountain', [TExtSaveDialog]);
RegisterComponents('Fountain', [TExtPrintDialog]);
end;



procedure TExtOpenDialog.ClearControls;
var I: Integer;
var ChildControl: TControl;
begin
for I:= 0 to FOptionPanel.ControlCount -1 do
	begin
	ChildControl:=FOptionPanel.Controls[I];
	ChildControl.Parent:=nil;
	ChildControl.Free;
	end
end;


procedure TExtOpenDialog.AttachControl(var C: TControl);
begin
C.Parent:=FOptionPanel;
C.Align := alTop;
end;

procedure TExtOpenDialog.AttachCheckBox(var C: TCheckBox;Caption: string; Checked: boolean);
begin
C:=TCheckBox.Create(Self);
C.Caption:=Caption;
C.Checked:=Checked;
AttachControl(TControl(C));
end;


constructor TExtOpenDialog.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FOptionPanel := TPanel.Create(Self);
with FOptionPanel do
 begin
  Name := 'OptionPanel';
  Caption := '';
  BevelOuter := bvNone;
  BorderWidth := 6;
  TabOrder := 1;
 end;
end;

destructor TExtOpenDialog.Destroy;
begin
FOptionPanel.Free;
inherited Destroy;
end;



procedure TExtOpenDialog.DoClose;
begin
inherited DoClose;
Application.HideHint;
FOptionPanel.ParentWindow := 0;
end;



procedure TExtOpenDialog.DoShow;
var PreviewRect, StaticRect: TRect;
begin
{ Set preview area to entire dialog }
GetClientRect(Handle, PreviewRect);
StaticRect := GetStaticRect;
{ Move preview area to right of static area }
PreviewRect.Left :=StaticRect.Right;
Inc(PreviewRect.Top, 35);
FOptionPanel.BoundsRect := PreviewRect;
FOptionPanel.ParentWindow := Handle;
inherited DoShow;
end;



function TExtOpenDialog.Execute: boolean;
begin
if NewStyleControls and not (ofOldStyleDialog in Options) then Template := 'EXTDLGTEMPLATE'
else Template := nil;
Result := inherited Execute;
end;


{ TExtSaveDialog }

function TExtSaveDialog.Execute: Boolean;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'EXTDLGTEMPLATE' else
    Template := nil;
  Result := DoExecute(@GetSaveFileName);
end;


{TExtPrintDialog}

procedure TExtPrintDialog.AttachControl(var C: TControl);
begin
C.Parent:=FOptionPanel;
C.Align := alTop;
end;

procedure TExtPrintDialog.AttachCheckBox(var C: TCheckBox;Caption: string; Checked: boolean);
begin
C:=TCheckBox.Create(Self);
C.Caption:=Caption;
C.Checked:=Checked;
AttachControl(TControl(C));
end;


constructor TExtPrintDialog.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FOptionPanel := TPanel.Create(Self);
with FOptionPanel do
 begin
 Name := 'OptionPanel';
 Caption := '';
 BevelOuter := bvNone;
 BorderWidth := 4;
 TabOrder := 1;
 end;
end;



destructor TExtPrintDialog.Destroy;
begin
FOptionPanel.Free;
inherited Destroy;
end;



procedure TExtPrintDialog.DoClose;
begin
inherited DoClose;
Application.HideHint;
FOptionPanel.ParentWindow := 0;
end;



function TExtPrintDialog.GetStaticRect: TRect;
begin
if Handle <> 0 then
	begin
	{ print range group box }
	GetWindowRect(GetDlgItem(Handle, grp1), Result);
	MapWindowPoints(0, Handle, Result, 2);
	end
else Result:= Rect(0, 0, 0, 0)
end;



procedure TExtPrintDialog.DoShow;
{Because TPrintDialog does not handle the Template}
{We are stuck the original size of the dialog box}
var ExtensionRect, StaticRect: TRect;
begin
{ Set preview area to entire dialog }
GetClientRect(Handle, ExtensionRect);
StaticRect := GetStaticRect;
{ Move preview area to right of static area }
ExtensionRect.Top :=StaticRect.Bottom;
Inc(ExtensionRect.Left,13);
FOptionPanel.BoundsRect := ExtensionRect;
FOptionPanel.ParentWindow := Handle;
inherited DoShow;
end;



end.
 